/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/* $Source: src/import/chips/p9/common/include/p9_mc_scom_addresses.H $   */
/*                                                                        */
/* OpenPOWER sbe Project                                                  */
/*                                                                        */
/* Contributors Listed Below - COPYRIGHT 2015,2016                        */
/* [+] International Business Machines Corp.                              */
/*                                                                        */
/*                                                                        */
/* Licensed under the Apache License, Version 2.0 (the "License");        */
/* you may not use this file except in compliance with the License.       */
/* You may obtain a copy of the License at                                */
/*                                                                        */
/*     http://www.apache.org/licenses/LICENSE-2.0                         */
/*                                                                        */
/* Unless required by applicable law or agreed to in writing, software    */
/* distributed under the License is distributed on an "AS IS" BASIS,      */
/* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or        */
/* implied. See the License for the specific language governing           */
/* permissions and limitations under the License.                         */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
///
/// @file p9_mc_scom_addresses.H
/// @brief  Defines constants for scom addresses
///
// *HWP HWP Owner: Ben Gass <bgass@us.ibm.com>
// *HWP FW Owner: Thi Tran <thi@us.ibm.com>
// *HWP Team: SOA
// *HWP Level: 1
// *HWP Consumed by: FSP:HB:HS:OCC:SBE:CME:SGPE:PGPE:FPPE:IPPE

/*---------------------------------------------------------------
 *
 *---------------------------------------------------------------
 *
 *  Issues:
 *
 * Closed
 *    TOD reg same address.             HW323439
 *       - Issue was closed with the explaination "same as p8"
 *    IO0 registers need fixed.         HW320437
 *    PHB registers need fixed.         HW320416  ( all regs commented out now )
 *    OSC/perv regs same address.       HW323437
 *    MC regs with same address.        HW323435 (matteo)
 *         Duplicate IOM registers.          HW320456 (designers)
 *    PEC Sat_id issue                  HW329652
 *    PB.PB_PPE registers need fixed.   HW320435
 *    EX05 registers need fixed.        HW320427  (9020) - L2 ring id's are incorrect
 *    IOFPPE registers need fixed.      HW320424  (9020) - Investigate NULL scope
 *    PEC addresses are wrong.          HW322598  (9020)
 *    MC registers need fixed.          HW320433
 *    VA.VA_NORTH registers need fixed. HW320436
 *
 * Format:
 *
 *  <UNIT>_<SUBUNIT>_<INSTANCE>_<REGISTER>_<ACCESS/TYPE>
 *
 *  Notes: Subunits are only added to make names unique when
 *  there are name collisions.
 *  Only units with more than one instance has instance numbers.
 *  If there is only one, the instance number is omitted.
 *
 *  Instance numbers are chiplet id's for the PERV unit.  The
 *  chiplet id's are mapped to their name and used instead of
 *  instance numbers.  See bellow.
 *
 *  For registers with a single access type the type and access
 *  methods are omitted.
 *
 *  For access types where all bits have the same access methods, the
 *  access method is appended to the name.  If the access methods
 *  are different for some bits, the access type is appended to the
 *  name _SCOM instead of _RO.  The _RW(X) access method is omitted
 *  and assumed to be default.
 *
 *  Valid units / subunits
 *    PU                    : No unit chip level
 *        MCD0[0..1]        : mcd subunit
 *        PIB2OPB[0..1]     : PIB2OPB subunit
 *        OTPROM[0..1]      : otprom subunit
 *        NPU               : common npu subunit
 *        NPU[0..2]         : Npu stacks 0 to 2
 *          CTL             : Npu CTL subunit
 *          DAT             : Npu DAT subunit
 *          SM[0..3]        : Npu SM subunits
 *          NTL[0..1]       : Npu NTL subunit
 *    PERV                  : Pervasive
 *        FSI2PIB           : subunit
 *        FSISHIFT          : subunit
 *        FSII2C            : subunit
 *        FSB               : subunit
 *    EX                    : Ex unit (1/2 quad, 2 cores)
 *        L2                : L2 subunit
 *        L3                : L3 subunit
 *    PEC                   : PCI Pec unit
 *        STACK0            : subunit
 *        STACK1            : subunit
 *        STACK2            : subunit
 *    C                     : core
 *    EQ                    : quad
 *    OBUS                  : obus
 *    CAPP                  : capp
 *    MCBIST                : mcbist
 *    MCA                   : mca
 *    NVBUS                 : (not implemented yet)
 *    PHB                   : (not implemented yet)
 *    MI                    : (not implemented yet)
 *    DMI                   : (not implemented yet)
 *    MCS                   : (not implemented yet)
 *    OCC                   : (not implemented yet)
 *    PPE                   : (not implemented yet)
 *    SBE                   : (not implemented yet)
 *    XBUS                  : (not implemented yet)
 *
 * Pervasive instance names follow chiplet id.
 *
 * Instance/  | Chiplet
 * Chiplet    | name
 * -----------+-----------
 * 0x00       | PIB
 * 0x01       | TP
 * 0x02       | N0
 * 0x03       | N1
 * 0x04       | N2
 * 0x05       | N3
 * 0x06       | XB
 * 0x07       | MC01
 * 0x08       | MC23
 * 0x09       | OB0
 * 0x0A       | OB1
 * 0x0B       | OB2
 * 0x0C       | OB3
 * 0x0D       | PCI0
 * 0x0E       | PCI1
 * 0x0F       | PCI2
 * 0x10       | EP00
 * 0x11       | EP01
 * 0x12       | EP02
 * 0x13       | EP03
 * 0x14       | EP04
 * 0x15       | EP05
 * 0x20       | EC00
 * 0x21       | EC01
 * 0x22       | EC02
 * 0x23       | EC03
 * 0x24       | EC04
 * 0x25       | EC05
 * 0x26       | EC06
 * 0x27       | EC07
 * 0x28       | EC08
 * 0x29       | EC09
 * 0x2A       | EC10
 * 0x2B       | EC11
 * 0x2C       | EC12
 * 0x2D       | EC13
 * 0x2E       | EC14
 * 0x2F       | EC15
 * 0x30       | EC16
 * 0x31       | EC17
 * 0x32       | EC18
 * 0x33       | EC19
 * 0x34       | EC20
 * 0x35       | EC21
 * 0x36       | EC22
 * 0x37       | EC23
 *
 *
 *---------------------------------------------------------------
 *
 * NOTES:
 *
 * there is a SPR ring that goes around the chip with an
 * address(0:9)/tid(0:1) (thread id)/mfspr_data(0:63) and return mfspr_data_v/mfspr_data(0:63)
 *
 *    Add PU_<SUBUNITS>  (only if there are conflicts on these registers)
 *       0x0001XXXX OTPROM
 *       0x0002XXXX FSIM0
 *       0x0003XXXX FSIM1
 *       0x0004XXXX TOD
 *       0x0005XXXX FSI_MBOX
 *       0x0006XXXX OCI_BRIDGE
 *       0x0007XXXX SPI_ADC
 *       0x0008XXXX PIBMEM
 *       0x0009XXXX ADU
 *       0x000AXXXX I2CM
 *       0x000BXXXX SBE_FIFO
 *       0x000DXXXX PSU
 *       0x000EXXXX SBE
 *
 * 0x0000100A for FSI2PIB  => PERV_FSI2PIB
 * 0x00000Cxx for FSISHIFT =>  PERV_FSISHIFT
 * 0x000018xx for FSI I2C =>  PERV_FSII2C
 * 0x000024xx for FSI SBEFIFO => PERV_FSB
 *
 * 0x00000400 PEEK_TABLE
 * 0x00000800 FSI_SLAVE
 * 0x00000C00 FSI_SHIFT
 * 0x00001000 FSI2PIB
 * 0x00001400 FSI_SCRATCHPAD
 * 0x00001800 FSI_I2CM
 * 0x00002400 FSI_SBE_FIFO
 *
 * address fields
 * 0xCCRPxxxx
 *
 * CC=chiplet
 * R=always 0?
 * P=port
 *   0=gpregs
 *   1=normal unit scom ring (exclude)
 *   3=clock controller
 *   4=firs
 *   5=cpm
 *
 * =============================================================================
 * Compiling
 *
 * Precompile the header to save time on subsquent compiles:
 * g++ -I. -c scom_addresses.H
 *
 * Use these options to help reduce the binary size
 * g++ -I. -Os -fdata-sections -ffunction-sections <file>.C -o <output> -Wl,--gc-sections
 *
 *
 *---------------------------------------------------------------
 */

#include <p9_const_common.H>


#ifndef __P9_MC_SCOM_ADDRESSES_H
#define __P9_MC_SCOM_ADDRESSES_H


#include <p9_scom_template_consts.H>
#include <p9_mc_scom_addresses_fixes.H>


REG64( MCA_0_WREITE_AACR                                       , RULL(0x07010A29), SH_UNT_MCA_0_WREITE,
       SH_ACS_SCOM_RW   );
REG64( MCA_1_WREITE_AACR                                       , RULL(0x07010A69), SH_UNT_MCA_1_WREITE,
       SH_ACS_SCOM_RW   );
REG64( MCA_2_WREITE_AACR                                       , RULL(0x07010AA9), SH_UNT_MCA_2_WREITE,
       SH_ACS_SCOM_RW   );
REG64( MCA_3_WREITE_AACR                                       , RULL(0x07010AE9), SH_UNT_MCA_3_WREITE,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_WREITE_AACR                                       , RULL(0x08010A29), SH_UNT_MCA_4_WREITE,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_WREITE_AACR                                       , RULL(0x08010A69), SH_UNT_MCA_5_WREITE,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_WREITE_AACR                                       , RULL(0x08010AA9), SH_UNT_MCA_6_WREITE,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_WREITE_AACR                                       , RULL(0x08010AE9), SH_UNT_MCA_7_WREITE,
       SH_ACS_SCOM_RW   );
REG64( MCA_WREITE_AACR                                         , RULL(0x07010A29), SH_UNT_MCA_WREITE,
       SH_ACS_SCOM_RW   );
REG64( MCS_0_PORT02_AACR                                       , RULL(0x0501082C), SH_UNT_MCS_0_PORT02,
       SH_ACS_SCOM_RW   );
REG64( MCS_1_PORT02_AACR                                       , RULL(0x050108AC), SH_UNT_MCS_1_PORT02,
       SH_ACS_SCOM_RW   );
REG64( MCS_2_PORT02_AACR                                       , RULL(0x0301082C), SH_UNT_MCS_2_PORT02,
       SH_ACS_SCOM_RW   );
REG64( MCS_3_PORT02_AACR                                       , RULL(0x030108AC), SH_UNT_MCS_3_PORT02,
       SH_ACS_SCOM_RW   );
REG64( MCS_PORT02_AACR                                         , RULL(0x0501082C), SH_UNT_MCS_PORT02,
       SH_ACS_SCOM_RW   );

REG64( MCA_AADR                                                , RULL(0x07010A2A), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_AADR                                              , RULL(0x07010A2A), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_AADR                                              , RULL(0x07010A6A), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_AADR                                              , RULL(0x07010AAA), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_AADR                                              , RULL(0x07010AEA), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_AADR                                              , RULL(0x08010A2A), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_AADR                                              , RULL(0x08010A6A), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_AADR                                              , RULL(0x08010AAA), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_AADR                                              , RULL(0x08010AEA), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );
REG64( MCS_0_PORT02_AADR                                       , RULL(0x0501082D), SH_UNT_MCS_0_PORT02,
       SH_ACS_SCOM_RO   );
REG64( MCS_1_PORT02_AADR                                       , RULL(0x050108AD), SH_UNT_MCS_1_PORT02,
       SH_ACS_SCOM_RO   );
REG64( MCS_2_PORT02_AADR                                       , RULL(0x0301082D), SH_UNT_MCS_2_PORT02,
       SH_ACS_SCOM_RO   );
REG64( MCS_3_PORT02_AADR                                       , RULL(0x030108AD), SH_UNT_MCS_3_PORT02,
       SH_ACS_SCOM_RO   );
REG64( MCS_PORT02_AADR                                         , RULL(0x0501082D), SH_UNT_MCS_PORT02,
       SH_ACS_SCOM_RO   );

REG64( MCA_AAER                                                , RULL(0x07010A2B), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_AAER                                              , RULL(0x07010A2B), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_AAER                                              , RULL(0x07010A6B), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_AAER                                              , RULL(0x07010AAB), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_AAER                                              , RULL(0x07010AEB), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_AAER                                              , RULL(0x08010A2B), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_AAER                                              , RULL(0x08010A6B), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_AAER                                              , RULL(0x08010AAB), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_AAER                                              , RULL(0x08010AEB), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );
REG64( MCS_0_PORT02_AAER                                       , RULL(0x0501082E), SH_UNT_MCS_0_PORT02,
       SH_ACS_SCOM_RO   );
REG64( MCS_1_PORT02_AAER                                       , RULL(0x050108AE), SH_UNT_MCS_1_PORT02,
       SH_ACS_SCOM_RO   );
REG64( MCS_2_PORT02_AAER                                       , RULL(0x0301082E), SH_UNT_MCS_2_PORT02,
       SH_ACS_SCOM_RO   );
REG64( MCS_3_PORT02_AAER                                       , RULL(0x030108AE), SH_UNT_MCS_3_PORT02,
       SH_ACS_SCOM_RO   );
REG64( MCS_PORT02_AAER                                         , RULL(0x0501082E), SH_UNT_MCS_PORT02,
       SH_ACS_SCOM_RO   );

REG64( MCA_ACTION0                                             , RULL(0x07010A06), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_ACTION0                                           , RULL(0x07010A06), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_ACTION0                                           , RULL(0x07010A46), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_ACTION0                                           , RULL(0x07010A86), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_ACTION0                                           , RULL(0x07010AC6), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_ACTION0                                           , RULL(0x08010A06), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_ACTION0                                           , RULL(0x08010A46), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_ACTION0                                           , RULL(0x08010A86), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_ACTION0                                           , RULL(0x08010AC6), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_ACTION1                                             , RULL(0x07010A07), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_ACTION1                                           , RULL(0x07010A07), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_ACTION1                                           , RULL(0x07010A47), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_ACTION1                                           , RULL(0x07010A87), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_ACTION1                                           , RULL(0x07010AC7), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_ACTION1                                           , RULL(0x08010A07), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_ACTION1                                           , RULL(0x08010A47), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_ACTION1                                           , RULL(0x08010A87), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_ACTION1                                           , RULL(0x08010AC7), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_ADDR_TRAP_REG                                       , RULL(0x07010003), SH_UNT_MCA      , SH_ACS_SCOM      );
REG64( MCA_0_ADDR_TRAP_REG                                     , RULL(0x07010003), SH_UNT_MCA_0    , SH_ACS_SCOM      );
REG64( MCA_4_ADDR_TRAP_REG                                     , RULL(0x08010003), SH_UNT_MCA_4    , SH_ACS_SCOM      );

REG64( MCA_ATOMIC_LOCK_MASK_LATCH_REG                          , RULL(0x07010007), SH_UNT_MCA      , SH_ACS_SCOM      );
REG64( MCA_0_ATOMIC_LOCK_MASK_LATCH_REG                        , RULL(0x07010007), SH_UNT_MCA_0    , SH_ACS_SCOM      );
REG64( MCA_4_ATOMIC_LOCK_MASK_LATCH_REG                        , RULL(0x08010007), SH_UNT_MCA_4    , SH_ACS_SCOM      );

REG64( MCBIST_CCSARRERRINJQ                                    , RULL(0x070123DE), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCSARRERRINJQ                                  , RULL(0x070123DE), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCSARRERRINJQ                                  , RULL(0x080123DE), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_CNTLQ                                        , RULL(0x070123A5), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_CNTLQ                                      , RULL(0x070123A5), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_CNTLQ                                      , RULL(0x080123A5), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_FIXED_DATA0Q                                 , RULL(0x070123E5), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_FIXED_DATA0Q                               , RULL(0x070123E5), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_FIXED_DATA0Q                               , RULL(0x080123E5), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_FIXED_DATA1Q                                 , RULL(0x070123E6), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_FIXED_DATA1Q                               , RULL(0x070123E6), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_FIXED_DATA1Q                               , RULL(0x080123E6), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_00                                 , RULL(0x07012315), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_00                               , RULL(0x07012315), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_00                               , RULL(0x08012315), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_01                                 , RULL(0x07012316), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_01                               , RULL(0x07012316), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_01                               , RULL(0x08012316), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_02                                 , RULL(0x07012317), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_02                               , RULL(0x07012317), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_02                               , RULL(0x08012317), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_03                                 , RULL(0x07012318), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_03                               , RULL(0x07012318), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_03                               , RULL(0x08012318), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_04                                 , RULL(0x07012319), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_04                               , RULL(0x07012319), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_04                               , RULL(0x08012319), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_05                                 , RULL(0x0701231A), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_05                               , RULL(0x0701231A), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_05                               , RULL(0x0801231A), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_06                                 , RULL(0x0701231B), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_06                               , RULL(0x0701231B), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_06                               , RULL(0x0801231B), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_07                                 , RULL(0x0701231C), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_07                               , RULL(0x0701231C), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_07                               , RULL(0x0801231C), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_08                                 , RULL(0x0701231D), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_08                               , RULL(0x0701231D), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_08                               , RULL(0x0801231D), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_09                                 , RULL(0x0701231E), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_09                               , RULL(0x0701231E), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_09                               , RULL(0x0801231E), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_10                                 , RULL(0x0701231F), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_10                               , RULL(0x0701231F), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_10                               , RULL(0x0801231F), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_11                                 , RULL(0x07012320), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_11                               , RULL(0x07012320), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_11                               , RULL(0x08012320), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_12                                 , RULL(0x07012321), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_12                               , RULL(0x07012321), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_12                               , RULL(0x08012321), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_13                                 , RULL(0x07012322), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_13                               , RULL(0x07012322), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_13                               , RULL(0x08012322), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_14                                 , RULL(0x07012323), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_14                               , RULL(0x07012323), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_14                               , RULL(0x08012323), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_15                                 , RULL(0x07012324), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_15                               , RULL(0x07012324), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_15                               , RULL(0x08012324), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_16                                 , RULL(0x07012325), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_16                               , RULL(0x07012325), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_16                               , RULL(0x08012325), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_17                                 , RULL(0x07012326), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_17                               , RULL(0x07012326), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_17                               , RULL(0x08012326), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_18                                 , RULL(0x07012327), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_18                               , RULL(0x07012327), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_18                               , RULL(0x08012327), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_19                                 , RULL(0x07012328), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_19                               , RULL(0x07012328), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_19                               , RULL(0x08012328), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_20                                 , RULL(0x07012329), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_20                               , RULL(0x07012329), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_20                               , RULL(0x08012329), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_21                                 , RULL(0x0701232A), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_21                               , RULL(0x0701232A), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_21                               , RULL(0x0801232A), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_22                                 , RULL(0x0701232B), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_22                               , RULL(0x0701232B), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_22                               , RULL(0x0801232B), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_23                                 , RULL(0x0701232C), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_23                               , RULL(0x0701232C), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_23                               , RULL(0x0801232C), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_24                                 , RULL(0x0701232D), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_24                               , RULL(0x0701232D), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_24                               , RULL(0x0801232D), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_25                                 , RULL(0x0701232E), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_25                               , RULL(0x0701232E), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_25                               , RULL(0x0801232E), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_26                                 , RULL(0x0701232F), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_26                               , RULL(0x0701232F), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_26                               , RULL(0x0801232F), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_27                                 , RULL(0x07012330), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_27                               , RULL(0x07012330), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_27                               , RULL(0x08012330), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_28                                 , RULL(0x07012331), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_28                               , RULL(0x07012331), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_28                               , RULL(0x08012331), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_29                                 , RULL(0x07012332), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_29                               , RULL(0x07012332), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_29                               , RULL(0x08012332), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_30                                 , RULL(0x07012333), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_30                               , RULL(0x07012333), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_30                               , RULL(0x08012333), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR0_31                                 , RULL(0x07012334), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR0_31                               , RULL(0x07012334), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR0_31                               , RULL(0x08012334), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_00                                 , RULL(0x07012335), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_00                               , RULL(0x07012335), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_00                               , RULL(0x08012335), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_01                                 , RULL(0x07012336), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_01                               , RULL(0x07012336), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_01                               , RULL(0x08012336), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_02                                 , RULL(0x07012337), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_02                               , RULL(0x07012337), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_02                               , RULL(0x08012337), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_03                                 , RULL(0x07012338), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_03                               , RULL(0x07012338), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_03                               , RULL(0x08012338), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_04                                 , RULL(0x07012339), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_04                               , RULL(0x07012339), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_04                               , RULL(0x08012339), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_05                                 , RULL(0x0701233A), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_05                               , RULL(0x0701233A), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_05                               , RULL(0x0801233A), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_06                                 , RULL(0x0701233B), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_06                               , RULL(0x0701233B), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_06                               , RULL(0x0801233B), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_07                                 , RULL(0x0701233C), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_07                               , RULL(0x0701233C), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_07                               , RULL(0x0801233C), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_08                                 , RULL(0x0701233D), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_08                               , RULL(0x0701233D), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_08                               , RULL(0x0801233D), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_09                                 , RULL(0x0701233E), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_09                               , RULL(0x0701233E), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_09                               , RULL(0x0801233E), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_10                                 , RULL(0x0701233F), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_10                               , RULL(0x0701233F), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_10                               , RULL(0x0801233F), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_11                                 , RULL(0x07012340), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_11                               , RULL(0x07012340), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_11                               , RULL(0x08012340), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_12                                 , RULL(0x07012341), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_12                               , RULL(0x07012341), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_12                               , RULL(0x08012341), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_13                                 , RULL(0x07012342), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_13                               , RULL(0x07012342), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_13                               , RULL(0x08012342), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_14                                 , RULL(0x07012343), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_14                               , RULL(0x07012343), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_14                               , RULL(0x08012343), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_15                                 , RULL(0x07012344), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_15                               , RULL(0x07012344), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_15                               , RULL(0x08012344), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_16                                 , RULL(0x07012345), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_16                               , RULL(0x07012345), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_16                               , RULL(0x08012345), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_17                                 , RULL(0x07012346), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_17                               , RULL(0x07012346), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_17                               , RULL(0x08012346), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_18                                 , RULL(0x07012347), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_18                               , RULL(0x07012347), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_18                               , RULL(0x08012347), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_19                                 , RULL(0x07012348), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_19                               , RULL(0x07012348), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_19                               , RULL(0x08012348), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_20                                 , RULL(0x07012349), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_20                               , RULL(0x07012349), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_20                               , RULL(0x08012349), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_21                                 , RULL(0x0701234A), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_21                               , RULL(0x0701234A), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_21                               , RULL(0x0801234A), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_22                                 , RULL(0x0701234B), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_22                               , RULL(0x0701234B), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_22                               , RULL(0x0801234B), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_23                                 , RULL(0x0701234C), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_23                               , RULL(0x0701234C), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_23                               , RULL(0x0801234C), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_24                                 , RULL(0x0701234D), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_24                               , RULL(0x0701234D), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_24                               , RULL(0x0801234D), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_25                                 , RULL(0x0701234E), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_25                               , RULL(0x0701234E), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_25                               , RULL(0x0801234E), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_26                                 , RULL(0x0701234F), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_26                               , RULL(0x0701234F), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_26                               , RULL(0x0801234F), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_27                                 , RULL(0x07012350), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_27                               , RULL(0x07012350), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_27                               , RULL(0x08012350), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_28                                 , RULL(0x07012351), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_28                               , RULL(0x07012351), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_28                               , RULL(0x08012351), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_29                                 , RULL(0x07012352), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_29                               , RULL(0x07012352), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_29                               , RULL(0x08012352), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_30                                 , RULL(0x07012353), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_30                               , RULL(0x07012353), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_30                               , RULL(0x08012353), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_INST_ARR1_31                                 , RULL(0x07012354), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_INST_ARR1_31                               , RULL(0x07012354), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_INST_ARR1_31                               , RULL(0x08012354), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_MODEQ                                        , RULL(0x070123A7), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_MODEQ                                      , RULL(0x070123A7), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_MODEQ                                      , RULL(0x080123A7), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_CCS_STATQ                                        , RULL(0x070123A6), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_CCS_STATQ                                      , RULL(0x070123A6), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_CCS_STATQ                                      , RULL(0x080123A6), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCA_CERR0                                               , RULL(0x07010A0E), SH_UNT_MCA      ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_0_CERR0                                             , RULL(0x07010A0E), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_1_CERR0                                             , RULL(0x07010A4E), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_2_CERR0                                             , RULL(0x07010A8E), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_3_CERR0                                             , RULL(0x07010ACE), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_4_CERR0                                             , RULL(0x08010A0E), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_5_CERR0                                             , RULL(0x08010A4E), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_6_CERR0                                             , RULL(0x08010A8E), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_7_CERR0                                             , RULL(0x08010ACE), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_WCLRREG );

REG64( MCA_CERR1                                               , RULL(0x07010A0F), SH_UNT_MCA      , SH_ACS_SCOM_RO   );
REG64( MCA_0_CERR1                                             , RULL(0x07010A0F), SH_UNT_MCA_0    , SH_ACS_SCOM_RO   );
REG64( MCA_1_CERR1                                             , RULL(0x07010A4F), SH_UNT_MCA_1    , SH_ACS_SCOM_RO   );
REG64( MCA_2_CERR1                                             , RULL(0x07010A8F), SH_UNT_MCA_2    , SH_ACS_SCOM_RO   );
REG64( MCA_3_CERR1                                             , RULL(0x07010ACF), SH_UNT_MCA_3    , SH_ACS_SCOM_RO   );
REG64( MCA_4_CERR1                                             , RULL(0x08010A0F), SH_UNT_MCA_4    , SH_ACS_SCOM_RO   );
REG64( MCA_5_CERR1                                             , RULL(0x08010A4F), SH_UNT_MCA_5    , SH_ACS_SCOM_RO   );
REG64( MCA_6_CERR1                                             , RULL(0x08010A8F), SH_UNT_MCA_6    , SH_ACS_SCOM_RO   );
REG64( MCA_7_CERR1                                             , RULL(0x08010ACF), SH_UNT_MCA_7    , SH_ACS_SCOM_RO   );

REG64( MCBIST_CLKRATIO                                         , RULL(0x070123F0), SH_UNT_MCBIST   , SH_ACS_SCOM_RO   );
REG64( MCBIST_0_CLKRATIO                                       , RULL(0x070123F0), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RO   );
REG64( MCBIST_1_CLKRATIO                                       , RULL(0x080123F0), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RO   );

REG64( MCBIST_DBGCFG0Q                                         , RULL(0x070123E8), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_DBGCFG0Q                                       , RULL(0x070123E8), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_DBGCFG0Q                                       , RULL(0x080123E8), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_DBGCFG1Q                                         , RULL(0x070123E9), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_DBGCFG1Q                                       , RULL(0x070123E9), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_DBGCFG1Q                                       , RULL(0x080123E9), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_DBGCFG2Q                                         , RULL(0x070123EA), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_DBGCFG2Q                                       , RULL(0x070123EA), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_DBGCFG2Q                                       , RULL(0x080123EA), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_DBGCFG3Q                                         , RULL(0x070123EB), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_DBGCFG3Q                                       , RULL(0x070123EB), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_DBGCFG3Q                                       , RULL(0x080123EB), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCA_DBGR                                                , RULL(0x07010A0B), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_DBGR                                              , RULL(0x07010A0B), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_DBGR                                              , RULL(0x07010A4B), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_DBGR                                              , RULL(0x07010A8B), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_DBGR                                              , RULL(0x07010ACB), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_DBGR                                              , RULL(0x08010A0B), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_DBGR                                              , RULL(0x08010A4B), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_DBGR                                              , RULL(0x08010A8B), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_DBGR                                              , RULL(0x08010ACB), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_3_DBG_INST1_COND_REG_1                              , RULL(0x070107C1), SH_UNT_MCA_3    , SH_ACS_SCOM      );
REG64( MCA_7_DBG_INST1_COND_REG_1                              , RULL(0x080107C1), SH_UNT_MCA_7    , SH_ACS_SCOM      );

REG64( MCA_3_DBG_INST1_COND_REG_2                              , RULL(0x070107C2), SH_UNT_MCA_3    , SH_ACS_SCOM      );
REG64( MCA_7_DBG_INST1_COND_REG_2                              , RULL(0x080107C2), SH_UNT_MCA_7    , SH_ACS_SCOM      );

REG64( MCA_3_DBG_INST1_COND_REG_3                              , RULL(0x070107C3), SH_UNT_MCA_3    , SH_ACS_SCOM      );
REG64( MCA_7_DBG_INST1_COND_REG_3                              , RULL(0x080107C3), SH_UNT_MCA_7    , SH_ACS_SCOM      );

REG64( MCA_3_DBG_INST2_COND_REG_1                              , RULL(0x070107C4), SH_UNT_MCA_3    , SH_ACS_SCOM      );
REG64( MCA_7_DBG_INST2_COND_REG_1                              , RULL(0x080107C4), SH_UNT_MCA_7    , SH_ACS_SCOM      );

REG64( MCA_3_DBG_INST2_COND_REG_2                              , RULL(0x070107C5), SH_UNT_MCA_3    , SH_ACS_SCOM      );
REG64( MCA_7_DBG_INST2_COND_REG_2                              , RULL(0x080107C5), SH_UNT_MCA_7    , SH_ACS_SCOM      );

REG64( MCA_3_DBG_INST2_COND_REG_3                              , RULL(0x070107C6), SH_UNT_MCA_3    , SH_ACS_SCOM      );
REG64( MCA_7_DBG_INST2_COND_REG_3                              , RULL(0x080107C6), SH_UNT_MCA_7    , SH_ACS_SCOM      );

REG64( MCA_3_DBG_MODE_REG                                      , RULL(0x070107C0), SH_UNT_MCA_3    , SH_ACS_SCOM      );
REG64( MCA_7_DBG_MODE_REG                                      , RULL(0x080107C0), SH_UNT_MCA_7    , SH_ACS_SCOM      );

REG64( MCA_3_DBG_TRACE_MODE_REG_2                              , RULL(0x070107CF), SH_UNT_MCA_3    , SH_ACS_SCOM      );
REG64( MCA_7_DBG_TRACE_MODE_REG_2                              , RULL(0x080107CF), SH_UNT_MCA_7    , SH_ACS_SCOM      );

REG64( MCA_3_DBG_TRACE_REG_0                                   , RULL(0x070107CD), SH_UNT_MCA_3    , SH_ACS_SCOM      );
REG64( MCA_7_DBG_TRACE_REG_0                                   , RULL(0x080107CD), SH_UNT_MCA_7    , SH_ACS_SCOM      );

REG64( MCA_3_DBG_TRACE_REG_1                                   , RULL(0x070107CE), SH_UNT_MCA_3    , SH_ACS_SCOM      );
REG64( MCA_7_DBG_TRACE_REG_1                                   , RULL(0x080107CE), SH_UNT_MCA_7    , SH_ACS_SCOM      );

REG64( MCA_DDRPHY_ADR_BIT_ENABLE_P0_ADR0                       , RULL(0x800040000701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_BIT_ENABLE_P0_ADR0                     , RULL(0x800040000701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_BIT_ENABLE_P0_ADR0                     , RULL(0x800040000801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_BIT_ENABLE_P0_ADR1                       , RULL(0x800044000701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_BIT_ENABLE_P0_ADR1                     , RULL(0x800044000701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_BIT_ENABLE_P0_ADR1                     , RULL(0x800044000801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_BIT_ENABLE_P0_ADR2                       , RULL(0x800048000701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_BIT_ENABLE_P0_ADR2                     , RULL(0x800048000701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_BIT_ENABLE_P0_ADR2                     , RULL(0x800048000801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_BIT_ENABLE_P0_ADR3                       , RULL(0x80004C000701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_BIT_ENABLE_P0_ADR3                     , RULL(0x80004C000701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_BIT_ENABLE_P0_ADR3                     , RULL(0x80004C000801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_BIT_ENABLE_P1_ADR0                     , RULL(0x800040000701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_BIT_ENABLE_P1_ADR0                     , RULL(0x800040000801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_BIT_ENABLE_P1_ADR1                     , RULL(0x800044000701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_BIT_ENABLE_P1_ADR1                     , RULL(0x800044000801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_BIT_ENABLE_P1_ADR2                     , RULL(0x800048000701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_BIT_ENABLE_P1_ADR2                     , RULL(0x800048000801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_BIT_ENABLE_P1_ADR3                     , RULL(0x80004C000701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_BIT_ENABLE_P1_ADR3                     , RULL(0x80004C000801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_BIT_ENABLE_P2_ADR0                     , RULL(0x800040000701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_BIT_ENABLE_P2_ADR0                     , RULL(0x800040000801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_BIT_ENABLE_P2_ADR1                     , RULL(0x800044000701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_BIT_ENABLE_P2_ADR1                     , RULL(0x800044000801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_BIT_ENABLE_P2_ADR2                     , RULL(0x800048000701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_BIT_ENABLE_P2_ADR2                     , RULL(0x800048000801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_BIT_ENABLE_P2_ADR3                     , RULL(0x80004C000701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_BIT_ENABLE_P2_ADR3                     , RULL(0x80004C000801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_BIT_ENABLE_P3_ADR0                     , RULL(0x8000400007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_BIT_ENABLE_P3_ADR0                     , RULL(0x8000400008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_BIT_ENABLE_P3_ADR1                     , RULL(0x8000440007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_BIT_ENABLE_P3_ADR1                     , RULL(0x8000440008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_BIT_ENABLE_P3_ADR2                     , RULL(0x8000480007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_BIT_ENABLE_P3_ADR2                     , RULL(0x8000480008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_BIT_ENABLE_P3_ADR3                     , RULL(0x80004C0007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_BIT_ENABLE_P3_ADR3                     , RULL(0x80004C0008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DCD_CONTROL_P0_ADR32S0                   , RULL(0x800080380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_ADR_DCD_CONTROL_P0_ADR32S0                 , RULL(0x800080380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_ADR_DCD_CONTROL_P0_ADR32S0                 , RULL(0x800080380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_ADR_DCD_CONTROL_P0_ADR32S1                   , RULL(0x800084380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_ADR_DCD_CONTROL_P0_ADR32S1                 , RULL(0x800084380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_ADR_DCD_CONTROL_P0_ADR32S1                 , RULL(0x800084380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_ADR_DCD_CONTROL_P1_ADR32S0                 , RULL(0x800080380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_ADR_DCD_CONTROL_P1_ADR32S0                 , RULL(0x800080380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_ADR_DCD_CONTROL_P1_ADR32S1                 , RULL(0x800084380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_ADR_DCD_CONTROL_P1_ADR32S1                 , RULL(0x800084380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_ADR_DCD_CONTROL_P2_ADR32S0                 , RULL(0x800080380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_ADR_DCD_CONTROL_P2_ADR32S0                 , RULL(0x800080380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_ADR_DCD_CONTROL_P2_ADR32S1                 , RULL(0x800084380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_ADR_DCD_CONTROL_P2_ADR32S1                 , RULL(0x800084380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_ADR_DCD_CONTROL_P3_ADR32S0                 , RULL(0x8000803807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_ADR_DCD_CONTROL_P3_ADR32S0                 , RULL(0x8000803808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_ADR_DCD_CONTROL_P3_ADR32S1                 , RULL(0x8000843807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_ADR_DCD_CONTROL_P3_ADR32S1                 , RULL(0x8000843808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_ADR_DELAY0_P0_ADR0                           , RULL(0x800040040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY0_P0_ADR0                         , RULL(0x800040040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY0_P0_ADR0                         , RULL(0x800040040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY0_P0_ADR1                           , RULL(0x800044040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY0_P0_ADR1                         , RULL(0x800044040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY0_P0_ADR1                         , RULL(0x800044040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY0_P0_ADR2                           , RULL(0x800048040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY0_P0_ADR2                         , RULL(0x800048040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY0_P0_ADR2                         , RULL(0x800048040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY0_P0_ADR3                           , RULL(0x80004C040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY0_P0_ADR3                         , RULL(0x80004C040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY0_P0_ADR3                         , RULL(0x80004C040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY0_P1_ADR0                         , RULL(0x800040040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY0_P1_ADR0                         , RULL(0x800040040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY0_P1_ADR1                         , RULL(0x800044040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY0_P1_ADR1                         , RULL(0x800044040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY0_P1_ADR2                         , RULL(0x800048040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY0_P1_ADR2                         , RULL(0x800048040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY0_P1_ADR3                         , RULL(0x80004C040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY0_P1_ADR3                         , RULL(0x80004C040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY0_P2_ADR0                         , RULL(0x800040040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY0_P2_ADR0                         , RULL(0x800040040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY0_P2_ADR1                         , RULL(0x800044040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY0_P2_ADR1                         , RULL(0x800044040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY0_P2_ADR2                         , RULL(0x800048040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY0_P2_ADR2                         , RULL(0x800048040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY0_P2_ADR3                         , RULL(0x80004C040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY0_P2_ADR3                         , RULL(0x80004C040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY0_P3_ADR0                         , RULL(0x8000400407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY0_P3_ADR0                         , RULL(0x8000400408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY0_P3_ADR1                         , RULL(0x8000440407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY0_P3_ADR1                         , RULL(0x8000440408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY0_P3_ADR2                         , RULL(0x8000480407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY0_P3_ADR2                         , RULL(0x8000480408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY0_P3_ADR3                         , RULL(0x80004C0407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY0_P3_ADR3                         , RULL(0x80004C0408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY1_P0_ADR0                           , RULL(0x800040050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY1_P0_ADR0                         , RULL(0x800040050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY1_P0_ADR0                         , RULL(0x800040050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY1_P0_ADR1                           , RULL(0x800044050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY1_P0_ADR1                         , RULL(0x800044050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY1_P0_ADR1                         , RULL(0x800044050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY1_P0_ADR2                           , RULL(0x800048050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY1_P0_ADR2                         , RULL(0x800048050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY1_P0_ADR2                         , RULL(0x800048050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY1_P0_ADR3                           , RULL(0x80004C050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY1_P0_ADR3                         , RULL(0x80004C050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY1_P0_ADR3                         , RULL(0x80004C050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY1_P1_ADR0                         , RULL(0x800040050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY1_P1_ADR0                         , RULL(0x800040050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY1_P1_ADR1                         , RULL(0x800044050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY1_P1_ADR1                         , RULL(0x800044050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY1_P1_ADR2                         , RULL(0x800048050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY1_P1_ADR2                         , RULL(0x800048050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY1_P1_ADR3                         , RULL(0x80004C050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY1_P1_ADR3                         , RULL(0x80004C050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY1_P2_ADR0                         , RULL(0x800040050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY1_P2_ADR0                         , RULL(0x800040050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY1_P2_ADR1                         , RULL(0x800044050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY1_P2_ADR1                         , RULL(0x800044050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY1_P2_ADR2                         , RULL(0x800048050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY1_P2_ADR2                         , RULL(0x800048050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY1_P2_ADR3                         , RULL(0x80004C050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY1_P2_ADR3                         , RULL(0x80004C050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY1_P3_ADR0                         , RULL(0x8000400507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY1_P3_ADR0                         , RULL(0x8000400508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY1_P3_ADR1                         , RULL(0x8000440507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY1_P3_ADR1                         , RULL(0x8000440508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY1_P3_ADR2                         , RULL(0x8000480507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY1_P3_ADR2                         , RULL(0x8000480508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY1_P3_ADR3                         , RULL(0x80004C0507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY1_P3_ADR3                         , RULL(0x80004C0508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY2_P0_ADR0                           , RULL(0x800040060701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY2_P0_ADR0                         , RULL(0x800040060701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY2_P0_ADR0                         , RULL(0x800040060801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY2_P0_ADR1                           , RULL(0x800044060701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY2_P0_ADR1                         , RULL(0x800044060701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY2_P0_ADR1                         , RULL(0x800044060801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY2_P0_ADR2                           , RULL(0x800048060701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY2_P0_ADR2                         , RULL(0x800048060701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY2_P0_ADR2                         , RULL(0x800048060801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY2_P0_ADR3                           , RULL(0x80004C060701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY2_P0_ADR3                         , RULL(0x80004C060701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY2_P0_ADR3                         , RULL(0x80004C060801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY2_P1_ADR0                         , RULL(0x800040060701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY2_P1_ADR0                         , RULL(0x800040060801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY2_P1_ADR1                         , RULL(0x800044060701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY2_P1_ADR1                         , RULL(0x800044060801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY2_P1_ADR2                         , RULL(0x800048060701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY2_P1_ADR2                         , RULL(0x800048060801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY2_P1_ADR3                         , RULL(0x80004C060701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY2_P1_ADR3                         , RULL(0x80004C060801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY2_P2_ADR0                         , RULL(0x800040060701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY2_P2_ADR0                         , RULL(0x800040060801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY2_P2_ADR1                         , RULL(0x800044060701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY2_P2_ADR1                         , RULL(0x800044060801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY2_P2_ADR2                         , RULL(0x800048060701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY2_P2_ADR2                         , RULL(0x800048060801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY2_P2_ADR3                         , RULL(0x80004C060701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY2_P2_ADR3                         , RULL(0x80004C060801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY2_P3_ADR0                         , RULL(0x8000400607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY2_P3_ADR0                         , RULL(0x8000400608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY2_P3_ADR1                         , RULL(0x8000440607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY2_P3_ADR1                         , RULL(0x8000440608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY2_P3_ADR2                         , RULL(0x8000480607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY2_P3_ADR2                         , RULL(0x8000480608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY2_P3_ADR3                         , RULL(0x80004C0607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY2_P3_ADR3                         , RULL(0x80004C0608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY3_P0_ADR0                           , RULL(0x800040070701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY3_P0_ADR0                         , RULL(0x800040070701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY3_P0_ADR0                         , RULL(0x800040070801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY3_P0_ADR1                           , RULL(0x800044070701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY3_P0_ADR1                         , RULL(0x800044070701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY3_P0_ADR1                         , RULL(0x800044070801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY3_P0_ADR2                           , RULL(0x800048070701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY3_P0_ADR2                         , RULL(0x800048070701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY3_P0_ADR2                         , RULL(0x800048070801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY3_P0_ADR3                           , RULL(0x80004C070701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY3_P0_ADR3                         , RULL(0x80004C070701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY3_P0_ADR3                         , RULL(0x80004C070801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY3_P1_ADR0                         , RULL(0x800040070701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY3_P1_ADR0                         , RULL(0x800040070801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY3_P1_ADR1                         , RULL(0x800044070701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY3_P1_ADR1                         , RULL(0x800044070801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY3_P1_ADR2                         , RULL(0x800048070701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY3_P1_ADR2                         , RULL(0x800048070801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY3_P1_ADR3                         , RULL(0x80004C070701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY3_P1_ADR3                         , RULL(0x80004C070801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY3_P2_ADR0                         , RULL(0x800040070701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY3_P2_ADR0                         , RULL(0x800040070801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY3_P2_ADR1                         , RULL(0x800044070701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY3_P2_ADR1                         , RULL(0x800044070801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY3_P2_ADR2                         , RULL(0x800048070701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY3_P2_ADR2                         , RULL(0x800048070801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY3_P2_ADR3                         , RULL(0x80004C070701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY3_P2_ADR3                         , RULL(0x80004C070801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY3_P3_ADR0                         , RULL(0x8000400707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY3_P3_ADR0                         , RULL(0x8000400708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY3_P3_ADR1                         , RULL(0x8000440707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY3_P3_ADR1                         , RULL(0x8000440708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY3_P3_ADR2                         , RULL(0x8000480707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY3_P3_ADR2                         , RULL(0x8000480708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY3_P3_ADR3                         , RULL(0x80004C0707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY3_P3_ADR3                         , RULL(0x80004C0708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY4_P0_ADR0                           , RULL(0x800040080701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY4_P0_ADR0                         , RULL(0x800040080701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY4_P0_ADR0                         , RULL(0x800040080801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY4_P0_ADR1                           , RULL(0x800044080701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY4_P0_ADR1                         , RULL(0x800044080701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY4_P0_ADR1                         , RULL(0x800044080801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY4_P0_ADR2                           , RULL(0x800048080701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY4_P0_ADR2                         , RULL(0x800048080701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY4_P0_ADR2                         , RULL(0x800048080801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY4_P0_ADR3                           , RULL(0x80004C080701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY4_P0_ADR3                         , RULL(0x80004C080701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY4_P0_ADR3                         , RULL(0x80004C080801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY4_P1_ADR0                         , RULL(0x800040080701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY4_P1_ADR0                         , RULL(0x800040080801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY4_P1_ADR1                         , RULL(0x800044080701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY4_P1_ADR1                         , RULL(0x800044080801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY4_P1_ADR2                         , RULL(0x800048080701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY4_P1_ADR2                         , RULL(0x800048080801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY4_P1_ADR3                         , RULL(0x80004C080701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY4_P1_ADR3                         , RULL(0x80004C080801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY4_P2_ADR0                         , RULL(0x800040080701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY4_P2_ADR0                         , RULL(0x800040080801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY4_P2_ADR1                         , RULL(0x800044080701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY4_P2_ADR1                         , RULL(0x800044080801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY4_P2_ADR2                         , RULL(0x800048080701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY4_P2_ADR2                         , RULL(0x800048080801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY4_P2_ADR3                         , RULL(0x80004C080701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY4_P2_ADR3                         , RULL(0x80004C080801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY4_P3_ADR0                         , RULL(0x8000400807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY4_P3_ADR0                         , RULL(0x8000400808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY4_P3_ADR1                         , RULL(0x8000440807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY4_P3_ADR1                         , RULL(0x8000440808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY4_P3_ADR2                         , RULL(0x8000480807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY4_P3_ADR2                         , RULL(0x8000480808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY4_P3_ADR3                         , RULL(0x80004C0807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY4_P3_ADR3                         , RULL(0x80004C0808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY5_P0_ADR0                           , RULL(0x800040090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY5_P0_ADR0                         , RULL(0x800040090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY5_P0_ADR0                         , RULL(0x800040090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY5_P0_ADR1                           , RULL(0x800044090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY5_P0_ADR1                         , RULL(0x800044090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY5_P0_ADR1                         , RULL(0x800044090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY5_P0_ADR2                           , RULL(0x800048090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY5_P0_ADR2                         , RULL(0x800048090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY5_P0_ADR2                         , RULL(0x800048090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY5_P0_ADR3                           , RULL(0x80004C090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY5_P0_ADR3                         , RULL(0x80004C090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY5_P0_ADR3                         , RULL(0x80004C090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY5_P1_ADR0                         , RULL(0x800040090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY5_P1_ADR0                         , RULL(0x800040090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY5_P1_ADR1                         , RULL(0x800044090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY5_P1_ADR1                         , RULL(0x800044090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY5_P1_ADR2                         , RULL(0x800048090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY5_P1_ADR2                         , RULL(0x800048090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY5_P1_ADR3                         , RULL(0x80004C090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY5_P1_ADR3                         , RULL(0x80004C090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY5_P2_ADR0                         , RULL(0x800040090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY5_P2_ADR0                         , RULL(0x800040090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY5_P2_ADR1                         , RULL(0x800044090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY5_P2_ADR1                         , RULL(0x800044090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY5_P2_ADR2                         , RULL(0x800048090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY5_P2_ADR2                         , RULL(0x800048090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY5_P2_ADR3                         , RULL(0x80004C090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY5_P2_ADR3                         , RULL(0x80004C090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY5_P3_ADR0                         , RULL(0x8000400907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY5_P3_ADR0                         , RULL(0x8000400908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY5_P3_ADR1                         , RULL(0x8000440907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY5_P3_ADR1                         , RULL(0x8000440908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY5_P3_ADR2                         , RULL(0x8000480907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY5_P3_ADR2                         , RULL(0x8000480908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY5_P3_ADR3                         , RULL(0x80004C0907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY5_P3_ADR3                         , RULL(0x80004C0908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY6_P0_ADR0                           , RULL(0x8000400A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY6_P0_ADR0                         , RULL(0x8000400A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY6_P0_ADR0                         , RULL(0x8000400A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY6_P0_ADR1                           , RULL(0x8000440A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY6_P0_ADR1                         , RULL(0x8000440A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY6_P0_ADR1                         , RULL(0x8000440A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY6_P0_ADR2                           , RULL(0x8000480A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY6_P0_ADR2                         , RULL(0x8000480A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY6_P0_ADR2                         , RULL(0x8000480A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY6_P0_ADR3                           , RULL(0x80004C0A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY6_P0_ADR3                         , RULL(0x80004C0A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY6_P0_ADR3                         , RULL(0x80004C0A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY6_P1_ADR0                         , RULL(0x8000400A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY6_P1_ADR0                         , RULL(0x8000400A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY6_P1_ADR1                         , RULL(0x8000440A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY6_P1_ADR1                         , RULL(0x8000440A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY6_P1_ADR2                         , RULL(0x8000480A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY6_P1_ADR2                         , RULL(0x8000480A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY6_P1_ADR3                         , RULL(0x80004C0A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY6_P1_ADR3                         , RULL(0x80004C0A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY6_P2_ADR0                         , RULL(0x8000400A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY6_P2_ADR0                         , RULL(0x8000400A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY6_P2_ADR1                         , RULL(0x8000440A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY6_P2_ADR1                         , RULL(0x8000440A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY6_P2_ADR2                         , RULL(0x8000480A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY6_P2_ADR2                         , RULL(0x8000480A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY6_P2_ADR3                         , RULL(0x80004C0A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY6_P2_ADR3                         , RULL(0x80004C0A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY6_P3_ADR0                         , RULL(0x8000400A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY6_P3_ADR0                         , RULL(0x8000400A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY6_P3_ADR1                         , RULL(0x8000440A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY6_P3_ADR1                         , RULL(0x8000440A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY6_P3_ADR2                         , RULL(0x8000480A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY6_P3_ADR2                         , RULL(0x8000480A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY6_P3_ADR3                         , RULL(0x80004C0A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY6_P3_ADR3                         , RULL(0x80004C0A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY7_P0_ADR0                           , RULL(0x8000400B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY7_P0_ADR0                         , RULL(0x8000400B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY7_P0_ADR0                         , RULL(0x8000400B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY7_P0_ADR1                           , RULL(0x8000440B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY7_P0_ADR1                         , RULL(0x8000440B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY7_P0_ADR1                         , RULL(0x8000440B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY7_P0_ADR2                           , RULL(0x8000480B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY7_P0_ADR2                         , RULL(0x8000480B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY7_P0_ADR2                         , RULL(0x8000480B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DELAY7_P0_ADR3                           , RULL(0x80004C0B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DELAY7_P0_ADR3                         , RULL(0x80004C0B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DELAY7_P0_ADR3                         , RULL(0x80004C0B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY7_P1_ADR0                         , RULL(0x8000400B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY7_P1_ADR0                         , RULL(0x8000400B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY7_P1_ADR1                         , RULL(0x8000440B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY7_P1_ADR1                         , RULL(0x8000440B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY7_P1_ADR2                         , RULL(0x8000480B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY7_P1_ADR2                         , RULL(0x8000480B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DELAY7_P1_ADR3                         , RULL(0x80004C0B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DELAY7_P1_ADR3                         , RULL(0x80004C0B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY7_P2_ADR0                         , RULL(0x8000400B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY7_P2_ADR0                         , RULL(0x8000400B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY7_P2_ADR1                         , RULL(0x8000440B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY7_P2_ADR1                         , RULL(0x8000440B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY7_P2_ADR2                         , RULL(0x8000480B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY7_P2_ADR2                         , RULL(0x8000480B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DELAY7_P2_ADR3                         , RULL(0x80004C0B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DELAY7_P2_ADR3                         , RULL(0x80004C0B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY7_P3_ADR0                         , RULL(0x8000400B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY7_P3_ADR0                         , RULL(0x8000400B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY7_P3_ADR1                         , RULL(0x8000440B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY7_P3_ADR1                         , RULL(0x8000440B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY7_P3_ADR2                         , RULL(0x8000480B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY7_P3_ADR2                         , RULL(0x8000480B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DELAY7_P3_ADR3                         , RULL(0x80004C0B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DELAY7_P3_ADR3                         , RULL(0x80004C0B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P0_ADR0          , RULL(0x8000400C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P0_ADR0        , RULL(0x8000400C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P0_ADR0        , RULL(0x8000400C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P0_ADR1          , RULL(0x8000440C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P0_ADR1        , RULL(0x8000440C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P0_ADR1        , RULL(0x8000440C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P0_ADR2          , RULL(0x8000480C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P0_ADR2        , RULL(0x8000480C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P0_ADR2        , RULL(0x8000480C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P0_ADR3          , RULL(0x80004C0C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P0_ADR3        , RULL(0x80004C0C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P0_ADR3        , RULL(0x80004C0C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P1_ADR0        , RULL(0x8000400C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P1_ADR0        , RULL(0x8000400C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P1_ADR1        , RULL(0x8000440C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P1_ADR1        , RULL(0x8000440C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P1_ADR2        , RULL(0x8000480C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P1_ADR2        , RULL(0x8000480C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P1_ADR3        , RULL(0x80004C0C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P1_ADR3        , RULL(0x80004C0C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P2_ADR0        , RULL(0x8000400C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P2_ADR0        , RULL(0x8000400C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P2_ADR1        , RULL(0x8000440C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P2_ADR1        , RULL(0x8000440C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P2_ADR2        , RULL(0x8000480C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P2_ADR2        , RULL(0x8000480C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P2_ADR3        , RULL(0x80004C0C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P2_ADR3        , RULL(0x80004C0C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P3_ADR0        , RULL(0x8000400C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P3_ADR0        , RULL(0x8000400C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P3_ADR1        , RULL(0x8000440C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P3_ADR1        , RULL(0x8000440C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P3_ADR2        , RULL(0x8000480C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P3_ADR2        , RULL(0x8000480C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P3_ADR3        , RULL(0x80004C0C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_ADR_DFT_WRAP_STATUS_CONTROL_P3_ADR3        , RULL(0x80004C0C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_ADR_DIFFPAIR_ENABLE_P0_ADR0                  , RULL(0x800040010701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DIFFPAIR_ENABLE_P0_ADR0                , RULL(0x800040010701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DIFFPAIR_ENABLE_P0_ADR0                , RULL(0x800040010801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DIFFPAIR_ENABLE_P0_ADR1                  , RULL(0x800044010701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DIFFPAIR_ENABLE_P0_ADR1                , RULL(0x800044010701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DIFFPAIR_ENABLE_P0_ADR1                , RULL(0x800044010801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DIFFPAIR_ENABLE_P0_ADR2                  , RULL(0x800048010701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DIFFPAIR_ENABLE_P0_ADR2                , RULL(0x800048010701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DIFFPAIR_ENABLE_P0_ADR2                , RULL(0x800048010801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DIFFPAIR_ENABLE_P0_ADR3                  , RULL(0x80004C010701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DIFFPAIR_ENABLE_P0_ADR3                , RULL(0x80004C010701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DIFFPAIR_ENABLE_P0_ADR3                , RULL(0x80004C010801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DIFFPAIR_ENABLE_P1_ADR0                , RULL(0x800040010701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DIFFPAIR_ENABLE_P1_ADR0                , RULL(0x800040010801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DIFFPAIR_ENABLE_P1_ADR1                , RULL(0x800044010701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DIFFPAIR_ENABLE_P1_ADR1                , RULL(0x800044010801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DIFFPAIR_ENABLE_P1_ADR2                , RULL(0x800048010701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DIFFPAIR_ENABLE_P1_ADR2                , RULL(0x800048010801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DIFFPAIR_ENABLE_P1_ADR3                , RULL(0x80004C010701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DIFFPAIR_ENABLE_P1_ADR3                , RULL(0x80004C010801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DIFFPAIR_ENABLE_P2_ADR0                , RULL(0x800040010701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DIFFPAIR_ENABLE_P2_ADR0                , RULL(0x800040010801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DIFFPAIR_ENABLE_P2_ADR1                , RULL(0x800044010701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DIFFPAIR_ENABLE_P2_ADR1                , RULL(0x800044010801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DIFFPAIR_ENABLE_P2_ADR2                , RULL(0x800048010701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DIFFPAIR_ENABLE_P2_ADR2                , RULL(0x800048010801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DIFFPAIR_ENABLE_P2_ADR3                , RULL(0x80004C010701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DIFFPAIR_ENABLE_P2_ADR3                , RULL(0x80004C010801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DIFFPAIR_ENABLE_P3_ADR0                , RULL(0x8000400107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DIFFPAIR_ENABLE_P3_ADR0                , RULL(0x8000400108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DIFFPAIR_ENABLE_P3_ADR1                , RULL(0x8000440107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DIFFPAIR_ENABLE_P3_ADR1                , RULL(0x8000440108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DIFFPAIR_ENABLE_P3_ADR2                , RULL(0x8000480107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DIFFPAIR_ENABLE_P3_ADR2                , RULL(0x8000480108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DIFFPAIR_ENABLE_P3_ADR3                , RULL(0x80004C0107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DIFFPAIR_ENABLE_P3_ADR3                , RULL(0x80004C0108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DLL_CNTL_P0_ADR32S0                      , RULL(0x8000803A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_ADR_DLL_CNTL_P0_ADR32S0                    , RULL(0x8000803A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_ADR_DLL_CNTL_P0_ADR32S0                    , RULL(0x8000803A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_ADR_DLL_CNTL_P0_ADR32S1                      , RULL(0x8000843A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_ADR_DLL_CNTL_P0_ADR32S1                    , RULL(0x8000843A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_ADR_DLL_CNTL_P0_ADR32S1                    , RULL(0x8000843A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_ADR_DLL_CNTL_P1_ADR32S0                    , RULL(0x8000803A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_ADR_DLL_CNTL_P1_ADR32S0                    , RULL(0x8000803A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_ADR_DLL_CNTL_P1_ADR32S1                    , RULL(0x8000843A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_ADR_DLL_CNTL_P1_ADR32S1                    , RULL(0x8000843A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_ADR_DLL_CNTL_P2_ADR32S0                    , RULL(0x8000803A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_ADR_DLL_CNTL_P2_ADR32S0                    , RULL(0x8000803A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_ADR_DLL_CNTL_P2_ADR32S1                    , RULL(0x8000843A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_ADR_DLL_CNTL_P2_ADR32S1                    , RULL(0x8000843A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_ADR_DLL_CNTL_P3_ADR32S0                    , RULL(0x8000803A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_ADR_DLL_CNTL_P3_ADR32S0                    , RULL(0x8000803A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_ADR_DLL_CNTL_P3_ADR32S1                    , RULL(0x8000843A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_ADR_DLL_CNTL_P3_ADR32S1                    , RULL(0x8000843A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_ADR_DLL_DAC_LOWER_P0_ADR32S0                 , RULL(0x8000803B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DLL_DAC_LOWER_P0_ADR32S0               , RULL(0x8000803B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DLL_DAC_LOWER_P0_ADR32S0               , RULL(0x8000803B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DLL_DAC_LOWER_P0_ADR32S1                 , RULL(0x8000843B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DLL_DAC_LOWER_P0_ADR32S1               , RULL(0x8000843B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DLL_DAC_LOWER_P0_ADR32S1               , RULL(0x8000843B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DLL_DAC_LOWER_P1_ADR32S0               , RULL(0x8000803B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DLL_DAC_LOWER_P1_ADR32S0               , RULL(0x8000803B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DLL_DAC_LOWER_P1_ADR32S1               , RULL(0x8000843B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DLL_DAC_LOWER_P1_ADR32S1               , RULL(0x8000843B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DLL_DAC_LOWER_P2_ADR32S0               , RULL(0x8000803B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DLL_DAC_LOWER_P2_ADR32S0               , RULL(0x8000803B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DLL_DAC_LOWER_P2_ADR32S1               , RULL(0x8000843B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DLL_DAC_LOWER_P2_ADR32S1               , RULL(0x8000843B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DLL_DAC_LOWER_P3_ADR32S0               , RULL(0x8000803B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DLL_DAC_LOWER_P3_ADR32S0               , RULL(0x8000803B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DLL_DAC_LOWER_P3_ADR32S1               , RULL(0x8000843B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DLL_DAC_LOWER_P3_ADR32S1               , RULL(0x8000843B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DLL_DAC_UPPER_P0_ADR32S0                 , RULL(0x8000803C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DLL_DAC_UPPER_P0_ADR32S0               , RULL(0x8000803C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DLL_DAC_UPPER_P0_ADR32S0               , RULL(0x8000803C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DLL_DAC_UPPER_P0_ADR32S1                 , RULL(0x8000843C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DLL_DAC_UPPER_P0_ADR32S1               , RULL(0x8000843C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DLL_DAC_UPPER_P0_ADR32S1               , RULL(0x8000843C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DLL_DAC_UPPER_P1_ADR32S0               , RULL(0x8000803C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DLL_DAC_UPPER_P1_ADR32S0               , RULL(0x8000803C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DLL_DAC_UPPER_P1_ADR32S1               , RULL(0x8000843C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DLL_DAC_UPPER_P1_ADR32S1               , RULL(0x8000843C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DLL_DAC_UPPER_P2_ADR32S0               , RULL(0x8000803C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DLL_DAC_UPPER_P2_ADR32S0               , RULL(0x8000803C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DLL_DAC_UPPER_P2_ADR32S1               , RULL(0x8000843C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DLL_DAC_UPPER_P2_ADR32S1               , RULL(0x8000843C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DLL_DAC_UPPER_P3_ADR32S0               , RULL(0x8000803C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DLL_DAC_UPPER_P3_ADR32S0               , RULL(0x8000803C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DLL_DAC_UPPER_P3_ADR32S1               , RULL(0x8000843C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DLL_DAC_UPPER_P3_ADR32S1               , RULL(0x8000843C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DLL_EXTRA_P0_ADR32S0                     , RULL(0x8000802D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DLL_EXTRA_P0_ADR32S0                   , RULL(0x8000802D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DLL_EXTRA_P0_ADR32S0                   , RULL(0x8000802D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DLL_EXTRA_P0_ADR32S1                     , RULL(0x8000842D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DLL_EXTRA_P0_ADR32S1                   , RULL(0x8000842D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DLL_EXTRA_P0_ADR32S1                   , RULL(0x8000842D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DLL_EXTRA_P1_ADR32S0                   , RULL(0x8000802D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DLL_EXTRA_P1_ADR32S0                   , RULL(0x8000802D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DLL_EXTRA_P1_ADR32S1                   , RULL(0x8000842D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DLL_EXTRA_P1_ADR32S1                   , RULL(0x8000842D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DLL_EXTRA_P2_ADR32S0                   , RULL(0x8000802D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DLL_EXTRA_P2_ADR32S0                   , RULL(0x8000802D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DLL_EXTRA_P2_ADR32S1                   , RULL(0x8000842D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DLL_EXTRA_P2_ADR32S1                   , RULL(0x8000842D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DLL_EXTRA_P3_ADR32S0                   , RULL(0x8000802D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DLL_EXTRA_P3_ADR32S0                   , RULL(0x8000802D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DLL_EXTRA_P3_ADR32S1                   , RULL(0x8000842D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DLL_EXTRA_P3_ADR32S1                   , RULL(0x8000842D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DLL_SLAVE_VREG_LOWER_P0_ADR32S0          , RULL(0x8000802E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DLL_SLAVE_VREG_LOWER_P0_ADR32S0        , RULL(0x8000802E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DLL_SLAVE_VREG_LOWER_P0_ADR32S0        , RULL(0x8000802E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DLL_SLAVE_VREG_LOWER_P0_ADR32S1          , RULL(0x8000842E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DLL_SLAVE_VREG_LOWER_P0_ADR32S1        , RULL(0x8000842E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DLL_SLAVE_VREG_LOWER_P0_ADR32S1        , RULL(0x8000842E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DLL_SLAVE_VREG_LOWER_P1_ADR32S0        , RULL(0x8000802E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DLL_SLAVE_VREG_LOWER_P1_ADR32S0        , RULL(0x8000802E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DLL_SLAVE_VREG_LOWER_P1_ADR32S1        , RULL(0x8000842E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DLL_SLAVE_VREG_LOWER_P1_ADR32S1        , RULL(0x8000842E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DLL_SLAVE_VREG_LOWER_P2_ADR32S0        , RULL(0x8000802E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DLL_SLAVE_VREG_LOWER_P2_ADR32S0        , RULL(0x8000802E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DLL_SLAVE_VREG_LOWER_P2_ADR32S1        , RULL(0x8000842E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DLL_SLAVE_VREG_LOWER_P2_ADR32S1        , RULL(0x8000842E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DLL_SLAVE_VREG_LOWER_P3_ADR32S0        , RULL(0x8000802E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DLL_SLAVE_VREG_LOWER_P3_ADR32S0        , RULL(0x8000802E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DLL_SLAVE_VREG_LOWER_P3_ADR32S1        , RULL(0x8000842E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DLL_SLAVE_VREG_LOWER_P3_ADR32S1        , RULL(0x8000842E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DLL_SLAVE_VREG_UPPER_P0_ADR32S0          , RULL(0x8000802F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DLL_SLAVE_VREG_UPPER_P0_ADR32S0        , RULL(0x8000802F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DLL_SLAVE_VREG_UPPER_P0_ADR32S0        , RULL(0x8000802F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DLL_SLAVE_VREG_UPPER_P0_ADR32S1          , RULL(0x8000842F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DLL_SLAVE_VREG_UPPER_P0_ADR32S1        , RULL(0x8000842F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DLL_SLAVE_VREG_UPPER_P0_ADR32S1        , RULL(0x8000842F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DLL_SLAVE_VREG_UPPER_P1_ADR32S0        , RULL(0x8000802F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DLL_SLAVE_VREG_UPPER_P1_ADR32S0        , RULL(0x8000802F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DLL_SLAVE_VREG_UPPER_P1_ADR32S1        , RULL(0x8000842F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DLL_SLAVE_VREG_UPPER_P1_ADR32S1        , RULL(0x8000842F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DLL_SLAVE_VREG_UPPER_P2_ADR32S0        , RULL(0x8000802F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DLL_SLAVE_VREG_UPPER_P2_ADR32S0        , RULL(0x8000802F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DLL_SLAVE_VREG_UPPER_P2_ADR32S1        , RULL(0x8000842F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DLL_SLAVE_VREG_UPPER_P2_ADR32S1        , RULL(0x8000842F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DLL_SLAVE_VREG_UPPER_P3_ADR32S0        , RULL(0x8000802F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DLL_SLAVE_VREG_UPPER_P3_ADR32S0        , RULL(0x8000802F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DLL_SLAVE_VREG_UPPER_P3_ADR32S1        , RULL(0x8000842F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DLL_SLAVE_VREG_UPPER_P3_ADR32S1        , RULL(0x8000842F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DLL_SW_CONTROL_P0_ADR32S0                , RULL(0x800080300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_ADR_DLL_SW_CONTROL_P0_ADR32S0              , RULL(0x800080300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_ADR_DLL_SW_CONTROL_P0_ADR32S0              , RULL(0x800080300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_ADR_DLL_SW_CONTROL_P0_ADR32S1                , RULL(0x800084300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_ADR_DLL_SW_CONTROL_P0_ADR32S1              , RULL(0x800084300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_ADR_DLL_SW_CONTROL_P0_ADR32S1              , RULL(0x800084300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_ADR_DLL_SW_CONTROL_P1_ADR32S0              , RULL(0x800080300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_ADR_DLL_SW_CONTROL_P1_ADR32S0              , RULL(0x800080300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_ADR_DLL_SW_CONTROL_P1_ADR32S1              , RULL(0x800084300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_ADR_DLL_SW_CONTROL_P1_ADR32S1              , RULL(0x800084300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_ADR_DLL_SW_CONTROL_P2_ADR32S0              , RULL(0x800080300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_ADR_DLL_SW_CONTROL_P2_ADR32S0              , RULL(0x800080300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_ADR_DLL_SW_CONTROL_P2_ADR32S1              , RULL(0x800084300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_ADR_DLL_SW_CONTROL_P2_ADR32S1              , RULL(0x800084300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_ADR_DLL_SW_CONTROL_P3_ADR32S0              , RULL(0x8000803007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_ADR_DLL_SW_CONTROL_P3_ADR32S0              , RULL(0x8000803008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_ADR_DLL_SW_CONTROL_P3_ADR32S1              , RULL(0x8000843007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_ADR_DLL_SW_CONTROL_P3_ADR32S1              , RULL(0x8000843008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_ADR_DLL_VREG_COARSE_P0_ADR32S0               , RULL(0x8000803E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DLL_VREG_COARSE_P0_ADR32S0             , RULL(0x8000803E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DLL_VREG_COARSE_P0_ADR32S0             , RULL(0x8000803E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DLL_VREG_COARSE_P0_ADR32S1               , RULL(0x8000843E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DLL_VREG_COARSE_P0_ADR32S1             , RULL(0x8000843E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DLL_VREG_COARSE_P0_ADR32S1             , RULL(0x8000843E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DLL_VREG_COARSE_P1_ADR32S0             , RULL(0x8000803E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DLL_VREG_COARSE_P1_ADR32S0             , RULL(0x8000803E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DLL_VREG_COARSE_P1_ADR32S1             , RULL(0x8000843E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DLL_VREG_COARSE_P1_ADR32S1             , RULL(0x8000843E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DLL_VREG_COARSE_P2_ADR32S0             , RULL(0x8000803E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DLL_VREG_COARSE_P2_ADR32S0             , RULL(0x8000803E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DLL_VREG_COARSE_P2_ADR32S1             , RULL(0x8000843E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DLL_VREG_COARSE_P2_ADR32S1             , RULL(0x8000843E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DLL_VREG_COARSE_P3_ADR32S0             , RULL(0x8000803E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DLL_VREG_COARSE_P3_ADR32S0             , RULL(0x8000803E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DLL_VREG_COARSE_P3_ADR32S1             , RULL(0x8000843E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DLL_VREG_COARSE_P3_ADR32S1             , RULL(0x8000843E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DLL_VREG_CONFIG_1_P0_ADR32S0             , RULL(0x800080310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DLL_VREG_CONFIG_1_P0_ADR32S0           , RULL(0x800080310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DLL_VREG_CONFIG_1_P0_ADR32S0           , RULL(0x800080310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DLL_VREG_CONFIG_1_P0_ADR32S1             , RULL(0x800084310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DLL_VREG_CONFIG_1_P0_ADR32S1           , RULL(0x800084310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DLL_VREG_CONFIG_1_P0_ADR32S1           , RULL(0x800084310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DLL_VREG_CONFIG_1_P1_ADR32S0           , RULL(0x800080310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DLL_VREG_CONFIG_1_P1_ADR32S0           , RULL(0x800080310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DLL_VREG_CONFIG_1_P1_ADR32S1           , RULL(0x800084310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DLL_VREG_CONFIG_1_P1_ADR32S1           , RULL(0x800084310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DLL_VREG_CONFIG_1_P2_ADR32S0           , RULL(0x800080310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DLL_VREG_CONFIG_1_P2_ADR32S0           , RULL(0x800080310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DLL_VREG_CONFIG_1_P2_ADR32S1           , RULL(0x800084310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DLL_VREG_CONFIG_1_P2_ADR32S1           , RULL(0x800084310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DLL_VREG_CONFIG_1_P3_ADR32S0           , RULL(0x8000803107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DLL_VREG_CONFIG_1_P3_ADR32S0           , RULL(0x8000803108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DLL_VREG_CONFIG_1_P3_ADR32S1           , RULL(0x8000843107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DLL_VREG_CONFIG_1_P3_ADR32S1           , RULL(0x8000843108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DLL_VREG_CONTROL_P0_ADR32S0              , RULL(0x8000803D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DLL_VREG_CONTROL_P0_ADR32S0            , RULL(0x8000803D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DLL_VREG_CONTROL_P0_ADR32S0            , RULL(0x8000803D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_DLL_VREG_CONTROL_P0_ADR32S1              , RULL(0x8000843D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_DLL_VREG_CONTROL_P0_ADR32S1            , RULL(0x8000843D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_DLL_VREG_CONTROL_P0_ADR32S1            , RULL(0x8000843D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DLL_VREG_CONTROL_P1_ADR32S0            , RULL(0x8000803D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DLL_VREG_CONTROL_P1_ADR32S0            , RULL(0x8000803D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_DLL_VREG_CONTROL_P1_ADR32S1            , RULL(0x8000843D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_DLL_VREG_CONTROL_P1_ADR32S1            , RULL(0x8000843D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DLL_VREG_CONTROL_P2_ADR32S0            , RULL(0x8000803D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DLL_VREG_CONTROL_P2_ADR32S0            , RULL(0x8000803D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_DLL_VREG_CONTROL_P2_ADR32S1            , RULL(0x8000843D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_DLL_VREG_CONTROL_P2_ADR32S1            , RULL(0x8000843D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DLL_VREG_CONTROL_P3_ADR32S0            , RULL(0x8000803D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DLL_VREG_CONTROL_P3_ADR32S0            , RULL(0x8000803D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_DLL_VREG_CONTROL_P3_ADR32S1            , RULL(0x8000843D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_DLL_VREG_CONTROL_P3_ADR32S1            , RULL(0x8000843D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P0_ADR0             , RULL(0x800040200701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P0_ADR0           , RULL(0x800040200701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P0_ADR0           , RULL(0x800040200801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P0_ADR1             , RULL(0x800044200701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P0_ADR1           , RULL(0x800044200701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P0_ADR1           , RULL(0x800044200801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P0_ADR2             , RULL(0x800048200701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P0_ADR2           , RULL(0x800048200701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P0_ADR2           , RULL(0x800048200801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P0_ADR3             , RULL(0x80004C200701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P0_ADR3           , RULL(0x80004C200701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P0_ADR3           , RULL(0x80004C200801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P1_ADR0           , RULL(0x800040200701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P1_ADR0           , RULL(0x800040200801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P1_ADR1           , RULL(0x800044200701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P1_ADR1           , RULL(0x800044200801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P1_ADR2           , RULL(0x800048200701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P1_ADR2           , RULL(0x800048200801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P1_ADR3           , RULL(0x80004C200701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P1_ADR3           , RULL(0x80004C200801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P2_ADR0           , RULL(0x800040200701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P2_ADR0           , RULL(0x800040200801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P2_ADR1           , RULL(0x800044200701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P2_ADR1           , RULL(0x800044200801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P2_ADR2           , RULL(0x800048200701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P2_ADR2           , RULL(0x800048200801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P2_ADR3           , RULL(0x80004C200701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P2_ADR3           , RULL(0x80004C200801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P3_ADR0           , RULL(0x8000402007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P3_ADR0           , RULL(0x8000402008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P3_ADR1           , RULL(0x8000442007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P3_ADR1           , RULL(0x8000442008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P3_ADR2           , RULL(0x8000482007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P3_ADR2           , RULL(0x8000482008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P3_ADR3           , RULL(0x80004C2007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_IO_FET_SLICE_EN_MAP0_P3_ADR3           , RULL(0x80004C2008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P0_ADR0             , RULL(0x800040210701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P0_ADR0           , RULL(0x800040210701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P0_ADR0           , RULL(0x800040210801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P0_ADR1             , RULL(0x800044210701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P0_ADR1           , RULL(0x800044210701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P0_ADR1           , RULL(0x800044210801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P0_ADR2             , RULL(0x800048210701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P0_ADR2           , RULL(0x800048210701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P0_ADR2           , RULL(0x800048210801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P0_ADR3             , RULL(0x80004C210701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P0_ADR3           , RULL(0x80004C210701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P0_ADR3           , RULL(0x80004C210801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P1_ADR0           , RULL(0x800040210701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P1_ADR0           , RULL(0x800040210801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P1_ADR1           , RULL(0x800044210701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P1_ADR1           , RULL(0x800044210801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P1_ADR2           , RULL(0x800048210701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P1_ADR2           , RULL(0x800048210801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P1_ADR3           , RULL(0x80004C210701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P1_ADR3           , RULL(0x80004C210801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P2_ADR0           , RULL(0x800040210701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P2_ADR0           , RULL(0x800040210801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P2_ADR1           , RULL(0x800044210701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P2_ADR1           , RULL(0x800044210801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P2_ADR2           , RULL(0x800048210701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P2_ADR2           , RULL(0x800048210801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P2_ADR3           , RULL(0x80004C210701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P2_ADR3           , RULL(0x80004C210801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P3_ADR0           , RULL(0x8000402107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P3_ADR0           , RULL(0x8000402108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P3_ADR1           , RULL(0x8000442107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P3_ADR1           , RULL(0x8000442108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P3_ADR2           , RULL(0x8000482107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P3_ADR2           , RULL(0x8000482108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P3_ADR3           , RULL(0x80004C2107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_IO_FET_SLICE_EN_MAP1_P3_ADR3           , RULL(0x80004C2108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P0_ADR0           , RULL(0x8000402A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P0_ADR0         , RULL(0x8000402A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P0_ADR0         , RULL(0x8000402A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P0_ADR1           , RULL(0x8000442A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P0_ADR1         , RULL(0x8000442A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P0_ADR1         , RULL(0x8000442A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P0_ADR2           , RULL(0x8000482A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P0_ADR2         , RULL(0x8000482A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P0_ADR2         , RULL(0x8000482A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P0_ADR3           , RULL(0x80004C2A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P0_ADR3         , RULL(0x80004C2A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P0_ADR3         , RULL(0x80004C2A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P1_ADR0         , RULL(0x8000402A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P1_ADR0         , RULL(0x8000402A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P1_ADR1         , RULL(0x8000442A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P1_ADR1         , RULL(0x8000442A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P1_ADR2         , RULL(0x8000482A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P1_ADR2         , RULL(0x8000482A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P1_ADR3         , RULL(0x80004C2A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P1_ADR3         , RULL(0x80004C2A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P2_ADR0         , RULL(0x8000402A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P2_ADR0         , RULL(0x8000402A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P2_ADR1         , RULL(0x8000442A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P2_ADR1         , RULL(0x8000442A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P2_ADR2         , RULL(0x8000482A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P2_ADR2         , RULL(0x8000482A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P2_ADR3         , RULL(0x80004C2A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P2_ADR3         , RULL(0x80004C2A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P3_ADR0         , RULL(0x8000402A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P3_ADR0         , RULL(0x8000402A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P3_ADR1         , RULL(0x8000442A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P3_ADR1         , RULL(0x8000442A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P3_ADR2         , RULL(0x8000482A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P3_ADR2         , RULL(0x8000482A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P3_ADR3         , RULL(0x80004C2A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP0_P3_ADR3         , RULL(0x80004C2A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P0_ADR0           , RULL(0x8000402B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P0_ADR0         , RULL(0x8000402B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P0_ADR0         , RULL(0x8000402B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P0_ADR1           , RULL(0x8000442B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P0_ADR1         , RULL(0x8000442B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P0_ADR1         , RULL(0x8000442B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P0_ADR2           , RULL(0x8000482B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P0_ADR2         , RULL(0x8000482B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P0_ADR2         , RULL(0x8000482B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P0_ADR3           , RULL(0x80004C2B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P0_ADR3         , RULL(0x80004C2B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P0_ADR3         , RULL(0x80004C2B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P1_ADR0         , RULL(0x8000402B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P1_ADR0         , RULL(0x8000402B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P1_ADR1         , RULL(0x8000442B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P1_ADR1         , RULL(0x8000442B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P1_ADR2         , RULL(0x8000482B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P1_ADR2         , RULL(0x8000482B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P1_ADR3         , RULL(0x80004C2B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P1_ADR3         , RULL(0x80004C2B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P2_ADR0         , RULL(0x8000402B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P2_ADR0         , RULL(0x8000402B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P2_ADR1         , RULL(0x8000442B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P2_ADR1         , RULL(0x8000442B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P2_ADR2         , RULL(0x8000482B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P2_ADR2         , RULL(0x8000482B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P2_ADR3         , RULL(0x80004C2B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P2_ADR3         , RULL(0x80004C2B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P3_ADR0         , RULL(0x8000402B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P3_ADR0         , RULL(0x8000402B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P3_ADR1         , RULL(0x8000442B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P3_ADR1         , RULL(0x8000442B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P3_ADR2         , RULL(0x8000482B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P3_ADR2         , RULL(0x8000482B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P3_ADR3         , RULL(0x80004C2B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_IO_SLEW_CTL_VALUE_MAP1_P3_ADR3         , RULL(0x80004C2B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_MCCLK_WRCLK_PR_STATIC_OFFSET_P0_ADR32S0  , RULL(0x800080330701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_MCCLK_WRCLK_PR_STATIC_OFFSET_P0_ADR32S0 , RULL(0x800080330701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_MCCLK_WRCLK_PR_STATIC_OFFSET_P0_ADR32S0 , RULL(0x800080330801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_MCCLK_WRCLK_PR_STATIC_OFFSET_P0_ADR32S1  , RULL(0x800084330701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_MCCLK_WRCLK_PR_STATIC_OFFSET_P0_ADR32S1 , RULL(0x800084330701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_MCCLK_WRCLK_PR_STATIC_OFFSET_P0_ADR32S1 , RULL(0x800084330801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_MCCLK_WRCLK_PR_STATIC_OFFSET_P1_ADR32S0 , RULL(0x800080330701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_MCCLK_WRCLK_PR_STATIC_OFFSET_P1_ADR32S0 , RULL(0x800080330801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_MCCLK_WRCLK_PR_STATIC_OFFSET_P1_ADR32S1 , RULL(0x800084330701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_MCCLK_WRCLK_PR_STATIC_OFFSET_P1_ADR32S1 , RULL(0x800084330801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_MCCLK_WRCLK_PR_STATIC_OFFSET_P2_ADR32S0 , RULL(0x800080330701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_MCCLK_WRCLK_PR_STATIC_OFFSET_P2_ADR32S0 , RULL(0x800080330801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_MCCLK_WRCLK_PR_STATIC_OFFSET_P2_ADR32S1 , RULL(0x800084330701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_MCCLK_WRCLK_PR_STATIC_OFFSET_P2_ADR32S1 , RULL(0x800084330801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_MCCLK_WRCLK_PR_STATIC_OFFSET_P3_ADR32S0 , RULL(0x8000803307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_MCCLK_WRCLK_PR_STATIC_OFFSET_P3_ADR32S0 , RULL(0x8000803308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_MCCLK_WRCLK_PR_STATIC_OFFSET_P3_ADR32S1 , RULL(0x8000843307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_MCCLK_WRCLK_PR_STATIC_OFFSET_P3_ADR32S1 , RULL(0x8000843308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE0_P0_ADR32S0    , RULL(0x800080360701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE0_P0_ADR32S0  , RULL(0x800080360701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE0_P0_ADR32S0  , RULL(0x800080360801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE0_P0_ADR32S1    , RULL(0x800084360701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE0_P0_ADR32S1  , RULL(0x800084360701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE0_P0_ADR32S1  , RULL(0x800084360801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE0_P1_ADR32S0  , RULL(0x800080360701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE0_P1_ADR32S0  , RULL(0x800080360801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE0_P1_ADR32S1  , RULL(0x800084360701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE0_P1_ADR32S1  , RULL(0x800084360801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE0_P2_ADR32S0  , RULL(0x800080360701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE0_P2_ADR32S0  , RULL(0x800080360801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE0_P2_ADR32S1  , RULL(0x800084360701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE0_P2_ADR32S1  , RULL(0x800084360801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE0_P3_ADR32S0  , RULL(0x8000803607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE0_P3_ADR32S0  , RULL(0x8000803608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE0_P3_ADR32S1  , RULL(0x8000843607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE0_P3_ADR32S1  , RULL(0x8000843608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE1_P0_ADR32S0    , RULL(0x800080370701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE1_P0_ADR32S0  , RULL(0x800080370701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE1_P0_ADR32S0  , RULL(0x800080370801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE1_P0_ADR32S1    , RULL(0x800084370701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE1_P0_ADR32S1  , RULL(0x800084370701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE1_P0_ADR32S1  , RULL(0x800084370801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE1_P1_ADR32S0  , RULL(0x800080370701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE1_P1_ADR32S0  , RULL(0x800080370801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE1_P1_ADR32S1  , RULL(0x800084370701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE1_P1_ADR32S1  , RULL(0x800084370801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE1_P2_ADR32S0  , RULL(0x800080370701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE1_P2_ADR32S0  , RULL(0x800080370801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE1_P2_ADR32S1  , RULL(0x800084370701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE1_P2_ADR32S1  , RULL(0x800084370801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE1_P3_ADR32S0  , RULL(0x8000803707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE1_P3_ADR32S0  , RULL(0x8000803708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE1_P3_ADR32S1  , RULL(0x8000843707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_OUTPUT_DRIVER_FORCE_VALUE1_P3_ADR32S1  , RULL(0x8000843708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_OUTPUT_FORCE_ATEST_CNTL_P0_ADR32S0       , RULL(0x800080350701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_OUTPUT_FORCE_ATEST_CNTL_P0_ADR32S0     , RULL(0x800080350701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_OUTPUT_FORCE_ATEST_CNTL_P0_ADR32S0     , RULL(0x800080350801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_OUTPUT_FORCE_ATEST_CNTL_P0_ADR32S1       , RULL(0x800084350701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_OUTPUT_FORCE_ATEST_CNTL_P0_ADR32S1     , RULL(0x800084350701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_OUTPUT_FORCE_ATEST_CNTL_P0_ADR32S1     , RULL(0x800084350801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_OUTPUT_FORCE_ATEST_CNTL_P1_ADR32S0     , RULL(0x800080350701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_OUTPUT_FORCE_ATEST_CNTL_P1_ADR32S0     , RULL(0x800080350801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_OUTPUT_FORCE_ATEST_CNTL_P1_ADR32S1     , RULL(0x800084350701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_OUTPUT_FORCE_ATEST_CNTL_P1_ADR32S1     , RULL(0x800084350801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_OUTPUT_FORCE_ATEST_CNTL_P2_ADR32S0     , RULL(0x800080350701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_OUTPUT_FORCE_ATEST_CNTL_P2_ADR32S0     , RULL(0x800080350801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_OUTPUT_FORCE_ATEST_CNTL_P2_ADR32S1     , RULL(0x800084350701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_OUTPUT_FORCE_ATEST_CNTL_P2_ADR32S1     , RULL(0x800084350801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_OUTPUT_FORCE_ATEST_CNTL_P3_ADR32S0     , RULL(0x8000803507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_OUTPUT_FORCE_ATEST_CNTL_P3_ADR32S0     , RULL(0x8000803508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_OUTPUT_FORCE_ATEST_CNTL_P3_ADR32S1     , RULL(0x8000843507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_OUTPUT_FORCE_ATEST_CNTL_P3_ADR32S1     , RULL(0x8000843508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_POWERDOWN_2_P0_ADR0                      , RULL(0x8000402C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_POWERDOWN_2_P0_ADR0                    , RULL(0x8000402C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_POWERDOWN_2_P0_ADR0                    , RULL(0x8000402C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_POWERDOWN_2_P0_ADR1                      , RULL(0x8000442C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_POWERDOWN_2_P0_ADR1                    , RULL(0x8000442C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_POWERDOWN_2_P0_ADR1                    , RULL(0x8000442C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_POWERDOWN_2_P0_ADR2                      , RULL(0x8000482C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_POWERDOWN_2_P0_ADR2                    , RULL(0x8000482C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_POWERDOWN_2_P0_ADR2                    , RULL(0x8000482C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_POWERDOWN_2_P0_ADR3                      , RULL(0x80004C2C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_POWERDOWN_2_P0_ADR3                    , RULL(0x80004C2C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_POWERDOWN_2_P0_ADR3                    , RULL(0x80004C2C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_POWERDOWN_2_P1_ADR0                    , RULL(0x8000402C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_POWERDOWN_2_P1_ADR0                    , RULL(0x8000402C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_POWERDOWN_2_P1_ADR1                    , RULL(0x8000442C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_POWERDOWN_2_P1_ADR1                    , RULL(0x8000442C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_POWERDOWN_2_P1_ADR2                    , RULL(0x8000482C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_POWERDOWN_2_P1_ADR2                    , RULL(0x8000482C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_POWERDOWN_2_P1_ADR3                    , RULL(0x80004C2C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_POWERDOWN_2_P1_ADR3                    , RULL(0x80004C2C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_POWERDOWN_2_P2_ADR0                    , RULL(0x8000402C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_POWERDOWN_2_P2_ADR0                    , RULL(0x8000402C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_POWERDOWN_2_P2_ADR1                    , RULL(0x8000442C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_POWERDOWN_2_P2_ADR1                    , RULL(0x8000442C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_POWERDOWN_2_P2_ADR2                    , RULL(0x8000482C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_POWERDOWN_2_P2_ADR2                    , RULL(0x8000482C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_POWERDOWN_2_P2_ADR3                    , RULL(0x80004C2C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_POWERDOWN_2_P2_ADR3                    , RULL(0x80004C2C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_POWERDOWN_2_P3_ADR0                    , RULL(0x8000402C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_POWERDOWN_2_P3_ADR0                    , RULL(0x8000402C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_POWERDOWN_2_P3_ADR1                    , RULL(0x8000442C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_POWERDOWN_2_P3_ADR1                    , RULL(0x8000442C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_POWERDOWN_2_P3_ADR2                    , RULL(0x8000482C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_POWERDOWN_2_P3_ADR2                    , RULL(0x8000482C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_POWERDOWN_2_P3_ADR3                    , RULL(0x80004C2C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_POWERDOWN_2_P3_ADR3                    , RULL(0x80004C2C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_SLEW_CAL_CNTL_P0_ADR32S0                 , RULL(0x800080390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_SLEW_CAL_CNTL_P0_ADR32S0               , RULL(0x800080390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_SLEW_CAL_CNTL_P0_ADR32S0               , RULL(0x800080390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_SLEW_CAL_CNTL_P0_ADR32S1                 , RULL(0x800084390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_SLEW_CAL_CNTL_P0_ADR32S1               , RULL(0x800084390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_SLEW_CAL_CNTL_P0_ADR32S1               , RULL(0x800084390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_SLEW_CAL_CNTL_P1_ADR32S0               , RULL(0x800080390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_SLEW_CAL_CNTL_P1_ADR32S0               , RULL(0x800080390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_SLEW_CAL_CNTL_P1_ADR32S1               , RULL(0x800084390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_SLEW_CAL_CNTL_P1_ADR32S1               , RULL(0x800084390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_SLEW_CAL_CNTL_P2_ADR32S0               , RULL(0x800080390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_SLEW_CAL_CNTL_P2_ADR32S0               , RULL(0x800080390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_SLEW_CAL_CNTL_P2_ADR32S1               , RULL(0x800084390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_SLEW_CAL_CNTL_P2_ADR32S1               , RULL(0x800084390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_SLEW_CAL_CNTL_P3_ADR32S0               , RULL(0x8000803907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_SLEW_CAL_CNTL_P3_ADR32S0               , RULL(0x8000803908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_SLEW_CAL_CNTL_P3_ADR32S1               , RULL(0x8000843907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_SLEW_CAL_CNTL_P3_ADR32S1               , RULL(0x8000843908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_SYSCLK_CNTL_PR_P0_ADR32S0                , RULL(0x800080320701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_SYSCLK_CNTL_PR_P0_ADR32S0              , RULL(0x800080320701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_SYSCLK_CNTL_PR_P0_ADR32S0              , RULL(0x800080320801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_SYSCLK_CNTL_PR_P0_ADR32S1                , RULL(0x800084320701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_ADR_SYSCLK_CNTL_PR_P0_ADR32S1              , RULL(0x800084320701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_ADR_SYSCLK_CNTL_PR_P0_ADR32S1              , RULL(0x800084320801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_SYSCLK_CNTL_PR_P1_ADR32S0              , RULL(0x800080320701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_SYSCLK_CNTL_PR_P1_ADR32S0              , RULL(0x800080320801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_ADR_SYSCLK_CNTL_PR_P1_ADR32S1              , RULL(0x800084320701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_ADR_SYSCLK_CNTL_PR_P1_ADR32S1              , RULL(0x800084320801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_SYSCLK_CNTL_PR_P2_ADR32S0              , RULL(0x800080320701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_SYSCLK_CNTL_PR_P2_ADR32S0              , RULL(0x800080320801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_ADR_SYSCLK_CNTL_PR_P2_ADR32S1              , RULL(0x800084320701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_ADR_SYSCLK_CNTL_PR_P2_ADR32S1              , RULL(0x800084320801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_SYSCLK_CNTL_PR_P3_ADR32S0              , RULL(0x8000803207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_SYSCLK_CNTL_PR_P3_ADR32S0              , RULL(0x8000803208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_ADR_SYSCLK_CNTL_PR_P3_ADR32S1              , RULL(0x8000843207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_ADR_SYSCLK_CNTL_PR_P3_ADR32S1              , RULL(0x8000843208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_ADR_SYSCLK_PR_VALUE_RO_P0_ADR32S0            , RULL(0x800080340701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_ADR_SYSCLK_PR_VALUE_RO_P0_ADR32S0          , RULL(0x800080340701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_ADR_SYSCLK_PR_VALUE_RO_P0_ADR32S0          , RULL(0x800080340801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_ADR_SYSCLK_PR_VALUE_RO_P0_ADR32S1            , RULL(0x800084340701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_ADR_SYSCLK_PR_VALUE_RO_P0_ADR32S1          , RULL(0x800084340701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_ADR_SYSCLK_PR_VALUE_RO_P0_ADR32S1          , RULL(0x800084340801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_ADR_SYSCLK_PR_VALUE_RO_P1_ADR32S0          , RULL(0x800080340701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_ADR_SYSCLK_PR_VALUE_RO_P1_ADR32S0          , RULL(0x800080340801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_ADR_SYSCLK_PR_VALUE_RO_P1_ADR32S1          , RULL(0x800084340701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_ADR_SYSCLK_PR_VALUE_RO_P1_ADR32S1          , RULL(0x800084340801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_ADR_SYSCLK_PR_VALUE_RO_P2_ADR32S0          , RULL(0x800080340701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_ADR_SYSCLK_PR_VALUE_RO_P2_ADR32S0          , RULL(0x800080340801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_ADR_SYSCLK_PR_VALUE_RO_P2_ADR32S1          , RULL(0x800084340701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_ADR_SYSCLK_PR_VALUE_RO_P2_ADR32S1          , RULL(0x800084340801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_ADR_SYSCLK_PR_VALUE_RO_P3_ADR32S0          , RULL(0x8000803407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_ADR_SYSCLK_PR_VALUE_RO_P3_ADR32S0          , RULL(0x8000803408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_ADR_SYSCLK_PR_VALUE_RO_P3_ADR32S1          , RULL(0x8000843407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_ADR_SYSCLK_PR_VALUE_RO_P3_ADR32S1          , RULL(0x8000843408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_APB_ATEST_MUX_SEL_P0                         , RULL(0x8000D0050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_APB_ATEST_MUX_SEL_P0                       , RULL(0x8000D0050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_APB_ATEST_MUX_SEL_P0                       , RULL(0x8000D0050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_APB_ATEST_MUX_SEL_P1                       , RULL(0x8000D0050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_APB_ATEST_MUX_SEL_P1                       , RULL(0x8000D0050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_APB_ATEST_MUX_SEL_P2                       , RULL(0x8000D0050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_APB_ATEST_MUX_SEL_P2                       , RULL(0x8000D0050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_APB_ATEST_MUX_SEL_P3                       , RULL(0x8000D00507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_APB_ATEST_MUX_SEL_P3                       , RULL(0x8000D00508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_APB_CONFIG0_P0                               , RULL(0x8000D0000701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_APB_CONFIG0_P0                             , RULL(0x8000D0000701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_APB_CONFIG0_P0                             , RULL(0x8000D0000801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_APB_CONFIG0_P1                             , RULL(0x8000D0000701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_APB_CONFIG0_P1                             , RULL(0x8000D0000801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_APB_CONFIG0_P2                             , RULL(0x8000D0000701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_APB_CONFIG0_P2                             , RULL(0x8000D0000801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_APB_CONFIG0_P3                             , RULL(0x8000D00007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_APB_CONFIG0_P3                             , RULL(0x8000D00008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_APB_ERROR_MASK0_P0                           , RULL(0x8000D0020701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_APB_ERROR_MASK0_P0                         , RULL(0x8000D0020701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_APB_ERROR_MASK0_P0                         , RULL(0x8000D0020801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_APB_ERROR_MASK0_P1                         , RULL(0x8000D0020701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_APB_ERROR_MASK0_P1                         , RULL(0x8000D0020801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_APB_ERROR_MASK0_P2                         , RULL(0x8000D0020701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_APB_ERROR_MASK0_P2                         , RULL(0x8000D0020801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_APB_ERROR_MASK0_P3                         , RULL(0x8000D00207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_APB_ERROR_MASK0_P3                         , RULL(0x8000D00208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_APB_ERROR_STATUS0_P0                         , RULL(0x8000D0010701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_APB_ERROR_STATUS0_P0                       , RULL(0x8000D0010701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_APB_ERROR_STATUS0_P0                       , RULL(0x8000D0010801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_APB_ERROR_STATUS0_P1                       , RULL(0x8000D0010701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_APB_ERROR_STATUS0_P1                       , RULL(0x8000D0010801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_APB_ERROR_STATUS0_P2                       , RULL(0x8000D0010701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_APB_ERROR_STATUS0_P2                       , RULL(0x8000D0010801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_APB_ERROR_STATUS0_P3                       , RULL(0x8000D00107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_APB_ERROR_STATUS0_P3                       , RULL(0x8000D00108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_APB_FIR_ERR0_P0                              , RULL(0x8000D0060701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_APB_FIR_ERR0_P0                            , RULL(0x8000D0060701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_APB_FIR_ERR0_P0                            , RULL(0x8000D0060801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_APB_FIR_ERR0_P1                            , RULL(0x8000D0060701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_APB_FIR_ERR0_P1                            , RULL(0x8000D0060801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_APB_FIR_ERR0_P2                            , RULL(0x8000D0060701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_APB_FIR_ERR0_P2                            , RULL(0x8000D0060801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_APB_FIR_ERR0_P3                            , RULL(0x8000D00607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_APB_FIR_ERR0_P3                            , RULL(0x8000D00608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_APB_FIR_ERR1_P0                              , RULL(0x8000D0070701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_APB_FIR_ERR1_P0                            , RULL(0x8000D0070701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_APB_FIR_ERR1_P0                            , RULL(0x8000D0070801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_APB_FIR_ERR1_P1                            , RULL(0x8000D0070701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_APB_FIR_ERR1_P1                            , RULL(0x8000D0070801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_APB_FIR_ERR1_P2                            , RULL(0x8000D0070701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_APB_FIR_ERR1_P2                            , RULL(0x8000D0070801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_APB_FIR_ERR1_P3                            , RULL(0x8000D00707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_APB_FIR_ERR1_P3                            , RULL(0x8000D00708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_APB_FIR_ERR2_P0                              , RULL(0x8000D0030701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_APB_FIR_ERR2_P0                            , RULL(0x8000D0030701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_APB_FIR_ERR2_P0                            , RULL(0x8000D0030801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_APB_FIR_ERR2_P1                            , RULL(0x8000D0030701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_APB_FIR_ERR2_P1                            , RULL(0x8000D0030801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_APB_FIR_ERR2_P2                            , RULL(0x8000D0030701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_APB_FIR_ERR2_P2                            , RULL(0x8000D0030801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_APB_FIR_ERR2_P3                            , RULL(0x8000D00307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_APB_FIR_ERR2_P3                            , RULL(0x8000D00308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_APB_FIR_ERR3_P0                              , RULL(0x8000D0040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_APB_FIR_ERR3_P0                            , RULL(0x8000D0040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_APB_FIR_ERR3_P0                            , RULL(0x8000D0040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_APB_FIR_ERR3_P1                            , RULL(0x8000D0040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_APB_FIR_ERR3_P1                            , RULL(0x8000D0040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_APB_FIR_ERR3_P2                            , RULL(0x8000D0040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_APB_FIR_ERR3_P2                            , RULL(0x8000D0040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_APB_FIR_ERR3_P3                            , RULL(0x8000D00407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_APB_FIR_ERR3_P3                            , RULL(0x8000D00408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_APB_LO_PROBE_SEL_P0                          , RULL(0x8000D0080701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_APB_LO_PROBE_SEL_P0                        , RULL(0x8000D0080701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_APB_LO_PROBE_SEL_P0                        , RULL(0x8000D0080801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_APB_LO_PROBE_SEL_P1                        , RULL(0x8000D0080701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_APB_LO_PROBE_SEL_P1                        , RULL(0x8000D0080801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_APB_LO_PROBE_SEL_P2                        , RULL(0x8000D0080701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_APB_LO_PROBE_SEL_P2                        , RULL(0x8000D0080801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_APB_LO_PROBE_SEL_P3                        , RULL(0x8000D00807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_APB_LO_PROBE_SEL_P3                        , RULL(0x8000D00808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P0_0                  , RULL(0x800000220701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P0_0                , RULL(0x800000220701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P0_0                , RULL(0x800000220801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P0_1                  , RULL(0x800004220701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P0_1                , RULL(0x800004220701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P0_1                , RULL(0x800004220801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P0_2                  , RULL(0x800008220701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P0_2                , RULL(0x800008220701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P0_2                , RULL(0x800008220801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P0_3                  , RULL(0x80000C220701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P0_3                , RULL(0x80000C220701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P0_3                , RULL(0x80000C220801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P0_4                  , RULL(0x800010220701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P0_4                , RULL(0x800010220701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P0_4                , RULL(0x800010220801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P1_0                , RULL(0x800000220701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P1_0                , RULL(0x800000220801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P1_1                , RULL(0x800004220701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P1_1                , RULL(0x800004220801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P1_2                , RULL(0x800008220701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P1_2                , RULL(0x800008220801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P1_3                , RULL(0x80000C220701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P1_3                , RULL(0x80000C220801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P1_4                , RULL(0x800010220701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P1_4                , RULL(0x800010220801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P2_0                , RULL(0x800000220701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P2_0                , RULL(0x800000220801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P2_1                , RULL(0x800004220701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P2_1                , RULL(0x800004220801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P2_2                , RULL(0x800008220701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P2_2                , RULL(0x800008220801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P2_3                , RULL(0x80000C220701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P2_3                , RULL(0x80000C220801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P2_4                , RULL(0x800010220701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P2_4                , RULL(0x800010220801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P3_0                , RULL(0x8000002207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P3_0                , RULL(0x8000002208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P3_1                , RULL(0x8000042207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P3_1                , RULL(0x8000042208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P3_2                , RULL(0x8000082207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P3_2                , RULL(0x8000082208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P3_3                , RULL(0x80000C2207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P3_3                , RULL(0x80000C2208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P3_4                , RULL(0x8000102207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_ACBOOST_CTL_BYTE0_P3_4                , RULL(0x8000102208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P0_0                  , RULL(0x800000230701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P0_0                , RULL(0x800000230701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P0_0                , RULL(0x800000230801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P0_1                  , RULL(0x800004230701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P0_1                , RULL(0x800004230701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P0_1                , RULL(0x800004230801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P0_2                  , RULL(0x800008230701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P0_2                , RULL(0x800008230701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P0_2                , RULL(0x800008230801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P0_3                  , RULL(0x80000C230701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P0_3                , RULL(0x80000C230701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P0_3                , RULL(0x80000C230801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P0_4                  , RULL(0x800010230701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P0_4                , RULL(0x800010230701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P0_4                , RULL(0x800010230801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P1_0                , RULL(0x800000230701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P1_0                , RULL(0x800000230801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P1_1                , RULL(0x800004230701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P1_1                , RULL(0x800004230801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P1_2                , RULL(0x800008230701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P1_2                , RULL(0x800008230801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P1_3                , RULL(0x80000C230701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P1_3                , RULL(0x80000C230801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P1_4                , RULL(0x800010230701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P1_4                , RULL(0x800010230801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P2_0                , RULL(0x800000230701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P2_0                , RULL(0x800000230801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P2_1                , RULL(0x800004230701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P2_1                , RULL(0x800004230801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P2_2                , RULL(0x800008230701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P2_2                , RULL(0x800008230801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P2_3                , RULL(0x80000C230701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P2_3                , RULL(0x80000C230801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P2_4                , RULL(0x800010230701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P2_4                , RULL(0x800010230801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P3_0                , RULL(0x8000002307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P3_0                , RULL(0x8000002308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P3_1                , RULL(0x8000042307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P3_1                , RULL(0x8000042308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P3_2                , RULL(0x8000082307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P3_2                , RULL(0x8000082308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P3_3                , RULL(0x80000C2307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P3_3                , RULL(0x80000C2308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P3_4                , RULL(0x8000102307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_ACBOOST_CTL_BYTE1_P3_4                , RULL(0x8000102308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_CTLE_CTL_BYTE0_P0_0                     , RULL(0x800000200701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_CTLE_CTL_BYTE0_P0_0                   , RULL(0x800000200701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_CTLE_CTL_BYTE0_P0_0                   , RULL(0x800000200801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_CTLE_CTL_BYTE0_P0_1                     , RULL(0x800004200701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_CTLE_CTL_BYTE0_P0_1                   , RULL(0x800004200701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_CTLE_CTL_BYTE0_P0_1                   , RULL(0x800004200801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_CTLE_CTL_BYTE0_P0_2                     , RULL(0x800008200701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_CTLE_CTL_BYTE0_P0_2                   , RULL(0x800008200701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_CTLE_CTL_BYTE0_P0_2                   , RULL(0x800008200801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_CTLE_CTL_BYTE0_P0_3                     , RULL(0x80000C200701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_CTLE_CTL_BYTE0_P0_3                   , RULL(0x80000C200701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_CTLE_CTL_BYTE0_P0_3                   , RULL(0x80000C200801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_CTLE_CTL_BYTE0_P0_4                     , RULL(0x800010200701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_CTLE_CTL_BYTE0_P0_4                   , RULL(0x800010200701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_CTLE_CTL_BYTE0_P0_4                   , RULL(0x800010200801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_CTLE_CTL_BYTE0_P1_0                   , RULL(0x800000200701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_CTLE_CTL_BYTE0_P1_0                   , RULL(0x800000200801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_CTLE_CTL_BYTE0_P1_1                   , RULL(0x800004200701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_CTLE_CTL_BYTE0_P1_1                   , RULL(0x800004200801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_CTLE_CTL_BYTE0_P1_2                   , RULL(0x800008200701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_CTLE_CTL_BYTE0_P1_2                   , RULL(0x800008200801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_CTLE_CTL_BYTE0_P1_3                   , RULL(0x80000C200701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_CTLE_CTL_BYTE0_P1_3                   , RULL(0x80000C200801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_CTLE_CTL_BYTE0_P1_4                   , RULL(0x800010200701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_CTLE_CTL_BYTE0_P1_4                   , RULL(0x800010200801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_CTLE_CTL_BYTE0_P2_0                   , RULL(0x800000200701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_CTLE_CTL_BYTE0_P2_0                   , RULL(0x800000200801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_CTLE_CTL_BYTE0_P2_1                   , RULL(0x800004200701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_CTLE_CTL_BYTE0_P2_1                   , RULL(0x800004200801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_CTLE_CTL_BYTE0_P2_2                   , RULL(0x800008200701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_CTLE_CTL_BYTE0_P2_2                   , RULL(0x800008200801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_CTLE_CTL_BYTE0_P2_3                   , RULL(0x80000C200701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_CTLE_CTL_BYTE0_P2_3                   , RULL(0x80000C200801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_CTLE_CTL_BYTE0_P2_4                   , RULL(0x800010200701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_CTLE_CTL_BYTE0_P2_4                   , RULL(0x800010200801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_CTLE_CTL_BYTE0_P3_0                   , RULL(0x8000002007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_CTLE_CTL_BYTE0_P3_0                   , RULL(0x8000002008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_CTLE_CTL_BYTE0_P3_1                   , RULL(0x8000042007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_CTLE_CTL_BYTE0_P3_1                   , RULL(0x8000042008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_CTLE_CTL_BYTE0_P3_2                   , RULL(0x8000082007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_CTLE_CTL_BYTE0_P3_2                   , RULL(0x8000082008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_CTLE_CTL_BYTE0_P3_3                   , RULL(0x80000C2007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_CTLE_CTL_BYTE0_P3_3                   , RULL(0x80000C2008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_CTLE_CTL_BYTE0_P3_4                   , RULL(0x8000102007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_CTLE_CTL_BYTE0_P3_4                   , RULL(0x8000102008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_CTLE_CTL_BYTE1_P0_0                     , RULL(0x800000210701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_CTLE_CTL_BYTE1_P0_0                   , RULL(0x800000210701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_CTLE_CTL_BYTE1_P0_0                   , RULL(0x800000210801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_CTLE_CTL_BYTE1_P0_1                     , RULL(0x800004210701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_CTLE_CTL_BYTE1_P0_1                   , RULL(0x800004210701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_CTLE_CTL_BYTE1_P0_1                   , RULL(0x800004210801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_CTLE_CTL_BYTE1_P0_2                     , RULL(0x800008210701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_CTLE_CTL_BYTE1_P0_2                   , RULL(0x800008210701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_CTLE_CTL_BYTE1_P0_2                   , RULL(0x800008210801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_CTLE_CTL_BYTE1_P0_3                     , RULL(0x80000C210701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_CTLE_CTL_BYTE1_P0_3                   , RULL(0x80000C210701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_CTLE_CTL_BYTE1_P0_3                   , RULL(0x80000C210801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_CTLE_CTL_BYTE1_P0_4                     , RULL(0x800010210701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_CTLE_CTL_BYTE1_P0_4                   , RULL(0x800010210701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_CTLE_CTL_BYTE1_P0_4                   , RULL(0x800010210801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_CTLE_CTL_BYTE1_P1_0                   , RULL(0x800000210701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_CTLE_CTL_BYTE1_P1_0                   , RULL(0x800000210801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_CTLE_CTL_BYTE1_P1_1                   , RULL(0x800004210701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_CTLE_CTL_BYTE1_P1_1                   , RULL(0x800004210801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_CTLE_CTL_BYTE1_P1_2                   , RULL(0x800008210701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_CTLE_CTL_BYTE1_P1_2                   , RULL(0x800008210801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_CTLE_CTL_BYTE1_P1_3                   , RULL(0x80000C210701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_CTLE_CTL_BYTE1_P1_3                   , RULL(0x80000C210801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_CTLE_CTL_BYTE1_P1_4                   , RULL(0x800010210701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_CTLE_CTL_BYTE1_P1_4                   , RULL(0x800010210801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_CTLE_CTL_BYTE1_P2_0                   , RULL(0x800000210701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_CTLE_CTL_BYTE1_P2_0                   , RULL(0x800000210801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_CTLE_CTL_BYTE1_P2_1                   , RULL(0x800004210701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_CTLE_CTL_BYTE1_P2_1                   , RULL(0x800004210801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_CTLE_CTL_BYTE1_P2_2                   , RULL(0x800008210701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_CTLE_CTL_BYTE1_P2_2                   , RULL(0x800008210801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_CTLE_CTL_BYTE1_P2_3                   , RULL(0x80000C210701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_CTLE_CTL_BYTE1_P2_3                   , RULL(0x80000C210801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_CTLE_CTL_BYTE1_P2_4                   , RULL(0x800010210701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_CTLE_CTL_BYTE1_P2_4                   , RULL(0x800010210801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_CTLE_CTL_BYTE1_P3_0                   , RULL(0x8000002107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_CTLE_CTL_BYTE1_P3_0                   , RULL(0x8000002108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_CTLE_CTL_BYTE1_P3_1                   , RULL(0x8000042107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_CTLE_CTL_BYTE1_P3_1                   , RULL(0x8000042108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_CTLE_CTL_BYTE1_P3_2                   , RULL(0x8000082107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_CTLE_CTL_BYTE1_P3_2                   , RULL(0x8000082108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_CTLE_CTL_BYTE1_P3_3                   , RULL(0x80000C2107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_CTLE_CTL_BYTE1_P3_3                   , RULL(0x80000C2108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_CTLE_CTL_BYTE1_P3_4                   , RULL(0x8000102107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_CTLE_CTL_BYTE1_P3_4                   , RULL(0x8000102108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DIR0_P0_0                      , RULL(0x800000020701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DIR0_P0_0                    , RULL(0x800000020701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DIR0_P0_0                    , RULL(0x800000020801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DIR0_P0_1                      , RULL(0x800004020701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DIR0_P0_1                    , RULL(0x800004020701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DIR0_P0_1                    , RULL(0x800004020801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DIR0_P0_2                      , RULL(0x800008020701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DIR0_P0_2                    , RULL(0x800008020701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DIR0_P0_2                    , RULL(0x800008020801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DIR0_P0_3                      , RULL(0x80000C020701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DIR0_P0_3                    , RULL(0x80000C020701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DIR0_P0_3                    , RULL(0x80000C020801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DIR0_P0_4                      , RULL(0x800010020701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DIR0_P0_4                    , RULL(0x800010020701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DIR0_P0_4                    , RULL(0x800010020801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DIR0_P1_0                    , RULL(0x800000020701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DIR0_P1_0                    , RULL(0x800000020801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DIR0_P1_1                    , RULL(0x800004020701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DIR0_P1_1                    , RULL(0x800004020801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DIR0_P1_2                    , RULL(0x800008020701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DIR0_P1_2                    , RULL(0x800008020801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DIR0_P1_3                    , RULL(0x80000C020701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DIR0_P1_3                    , RULL(0x80000C020801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DIR0_P1_4                    , RULL(0x800010020701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DIR0_P1_4                    , RULL(0x800010020801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DIR0_P2_0                    , RULL(0x800000020701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DIR0_P2_0                    , RULL(0x800000020801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DIR0_P2_1                    , RULL(0x800004020701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DIR0_P2_1                    , RULL(0x800004020801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DIR0_P2_2                    , RULL(0x800008020701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DIR0_P2_2                    , RULL(0x800008020801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DIR0_P2_3                    , RULL(0x80000C020701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DIR0_P2_3                    , RULL(0x80000C020801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DIR0_P2_4                    , RULL(0x800010020701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DIR0_P2_4                    , RULL(0x800010020801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DIR0_P3_0                    , RULL(0x8000000207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DIR0_P3_0                    , RULL(0x8000000208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DIR0_P3_1                    , RULL(0x8000040207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DIR0_P3_1                    , RULL(0x8000040208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DIR0_P3_2                    , RULL(0x8000080207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DIR0_P3_2                    , RULL(0x8000080208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DIR0_P3_3                    , RULL(0x80000C0207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DIR0_P3_3                    , RULL(0x80000C0208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DIR0_P3_4                    , RULL(0x8000100207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DIR0_P3_4                    , RULL(0x8000100208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DIR1_P0_0                      , RULL(0x800000030701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DIR1_P0_0                    , RULL(0x800000030701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DIR1_P0_0                    , RULL(0x800000030801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DIR1_P0_1                      , RULL(0x800004030701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DIR1_P0_1                    , RULL(0x800004030701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DIR1_P0_1                    , RULL(0x800004030801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DIR1_P0_2                      , RULL(0x800008030701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DIR1_P0_2                    , RULL(0x800008030701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DIR1_P0_2                    , RULL(0x800008030801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DIR1_P0_3                      , RULL(0x80000C030701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DIR1_P0_3                    , RULL(0x80000C030701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DIR1_P0_3                    , RULL(0x80000C030801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DIR1_P0_4                      , RULL(0x800010030701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DIR1_P0_4                    , RULL(0x800010030701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DIR1_P0_4                    , RULL(0x800010030801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DIR1_P1_0                    , RULL(0x800000030701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DIR1_P1_0                    , RULL(0x800000030801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DIR1_P1_1                    , RULL(0x800004030701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DIR1_P1_1                    , RULL(0x800004030801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DIR1_P1_2                    , RULL(0x800008030701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DIR1_P1_2                    , RULL(0x800008030801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DIR1_P1_3                    , RULL(0x80000C030701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DIR1_P1_3                    , RULL(0x80000C030801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DIR1_P1_4                    , RULL(0x800010030701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DIR1_P1_4                    , RULL(0x800010030801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DIR1_P2_0                    , RULL(0x800000030701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DIR1_P2_0                    , RULL(0x800000030801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DIR1_P2_1                    , RULL(0x800004030701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DIR1_P2_1                    , RULL(0x800004030801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DIR1_P2_2                    , RULL(0x800008030701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DIR1_P2_2                    , RULL(0x800008030801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DIR1_P2_3                    , RULL(0x80000C030701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DIR1_P2_3                    , RULL(0x80000C030801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DIR1_P2_4                    , RULL(0x800010030701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DIR1_P2_4                    , RULL(0x800010030801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DIR1_P3_0                    , RULL(0x8000000307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DIR1_P3_0                    , RULL(0x8000000308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DIR1_P3_1                    , RULL(0x8000040307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DIR1_P3_1                    , RULL(0x8000040308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DIR1_P3_2                    , RULL(0x8000080307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DIR1_P3_2                    , RULL(0x8000080308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DIR1_P3_3                    , RULL(0x80000C0307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DIR1_P3_3                    , RULL(0x80000C0308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DIR1_P3_4                    , RULL(0x8000100307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DIR1_P3_4                    , RULL(0x8000100308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P0_0              , RULL(0x8000007C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P0_0            , RULL(0x8000007C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P0_0            , RULL(0x8000007C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P0_1              , RULL(0x8000047C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P0_1            , RULL(0x8000047C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P0_1            , RULL(0x8000047C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P0_2              , RULL(0x8000087C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P0_2            , RULL(0x8000087C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P0_2            , RULL(0x8000087C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P0_3              , RULL(0x80000C7C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P0_3            , RULL(0x80000C7C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P0_3            , RULL(0x80000C7C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P0_4              , RULL(0x8000107C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P0_4            , RULL(0x8000107C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P0_4            , RULL(0x8000107C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P1_0            , RULL(0x8000007C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P1_0            , RULL(0x8000007C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P1_1            , RULL(0x8000047C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P1_1            , RULL(0x8000047C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P1_2            , RULL(0x8000087C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P1_2            , RULL(0x8000087C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P1_3            , RULL(0x80000C7C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P1_3            , RULL(0x80000C7C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P1_4            , RULL(0x8000107C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P1_4            , RULL(0x8000107C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P2_0            , RULL(0x8000007C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P2_0            , RULL(0x8000007C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P2_1            , RULL(0x8000047C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P2_1            , RULL(0x8000047C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P2_2            , RULL(0x8000087C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P2_2            , RULL(0x8000087C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P2_3            , RULL(0x80000C7C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P2_3            , RULL(0x80000C7C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P2_4            , RULL(0x8000107C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P2_4            , RULL(0x8000107C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P3_0            , RULL(0x8000007C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P3_0            , RULL(0x8000007C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P3_1            , RULL(0x8000047C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P3_1            , RULL(0x8000047C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P3_2            , RULL(0x8000087C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P3_2            , RULL(0x8000087C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P3_3            , RULL(0x80000C7C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P3_3            , RULL(0x80000C7C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P3_4            , RULL(0x8000107C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP0_P3_4            , RULL(0x8000107C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P0_0              , RULL(0x8000017C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P0_0            , RULL(0x8000017C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P0_0            , RULL(0x8000017C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P0_1              , RULL(0x8000057C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P0_1            , RULL(0x8000057C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P0_1            , RULL(0x8000057C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P0_2              , RULL(0x8000097C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P0_2            , RULL(0x8000097C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P0_2            , RULL(0x8000097C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P0_3              , RULL(0x80000D7C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P0_3            , RULL(0x80000D7C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P0_3            , RULL(0x80000D7C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P0_4              , RULL(0x8000117C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P0_4            , RULL(0x8000117C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P0_4            , RULL(0x8000117C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P1_0            , RULL(0x8000017C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P1_0            , RULL(0x8000017C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P1_1            , RULL(0x8000057C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P1_1            , RULL(0x8000057C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P1_2            , RULL(0x8000097C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P1_2            , RULL(0x8000097C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P1_3            , RULL(0x80000D7C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P1_3            , RULL(0x80000D7C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P1_4            , RULL(0x8000117C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P1_4            , RULL(0x8000117C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P2_0            , RULL(0x8000017C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P2_0            , RULL(0x8000017C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P2_1            , RULL(0x8000057C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P2_1            , RULL(0x8000057C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P2_2            , RULL(0x8000097C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P2_2            , RULL(0x8000097C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P2_3            , RULL(0x80000D7C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P2_3            , RULL(0x80000D7C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P2_4            , RULL(0x8000117C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P2_4            , RULL(0x8000117C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P3_0            , RULL(0x8000017C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P3_0            , RULL(0x8000017C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P3_1            , RULL(0x8000057C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P3_1            , RULL(0x8000057C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P3_2            , RULL(0x8000097C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P3_2            , RULL(0x8000097C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P3_3            , RULL(0x80000D7C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P3_3            , RULL(0x80000D7C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P3_4            , RULL(0x8000117C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP1_P3_4            , RULL(0x8000117C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P0_0              , RULL(0x8000027C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P0_0            , RULL(0x8000027C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P0_0            , RULL(0x8000027C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P0_1              , RULL(0x8000067C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P0_1            , RULL(0x8000067C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P0_1            , RULL(0x8000067C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P0_2              , RULL(0x80000A7C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P0_2            , RULL(0x80000A7C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P0_2            , RULL(0x80000A7C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P0_3              , RULL(0x80000E7C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P0_3            , RULL(0x80000E7C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P0_3            , RULL(0x80000E7C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P0_4              , RULL(0x8000127C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P0_4            , RULL(0x8000127C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P0_4            , RULL(0x8000127C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P1_0            , RULL(0x8000027C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P1_0            , RULL(0x8000027C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P1_1            , RULL(0x8000067C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P1_1            , RULL(0x8000067C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P1_2            , RULL(0x80000A7C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P1_2            , RULL(0x80000A7C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P1_3            , RULL(0x80000E7C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P1_3            , RULL(0x80000E7C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P1_4            , RULL(0x8000127C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P1_4            , RULL(0x8000127C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P2_0            , RULL(0x8000027C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P2_0            , RULL(0x8000027C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P2_1            , RULL(0x8000067C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P2_1            , RULL(0x8000067C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P2_2            , RULL(0x80000A7C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P2_2            , RULL(0x80000A7C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P2_3            , RULL(0x80000E7C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P2_3            , RULL(0x80000E7C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P2_4            , RULL(0x8000127C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P2_4            , RULL(0x8000127C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P3_0            , RULL(0x8000027C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P3_0            , RULL(0x8000027C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P3_1            , RULL(0x8000067C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P3_1            , RULL(0x8000067C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P3_2            , RULL(0x80000A7C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P3_2            , RULL(0x80000A7C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P3_3            , RULL(0x80000E7C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P3_3            , RULL(0x80000E7C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P3_4            , RULL(0x8000127C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP2_P3_4            , RULL(0x8000127C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P0_0              , RULL(0x8000037C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P0_0            , RULL(0x8000037C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P0_0            , RULL(0x8000037C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P0_1              , RULL(0x8000077C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P0_1            , RULL(0x8000077C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P0_1            , RULL(0x8000077C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P0_2              , RULL(0x80000B7C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P0_2            , RULL(0x80000B7C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P0_2            , RULL(0x80000B7C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P0_3              , RULL(0x80000F7C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P0_3            , RULL(0x80000F7C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P0_3            , RULL(0x80000F7C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P0_4              , RULL(0x8000137C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P0_4            , RULL(0x8000137C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P0_4            , RULL(0x8000137C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P1_0            , RULL(0x8000037C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P1_0            , RULL(0x8000037C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P1_1            , RULL(0x8000077C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P1_1            , RULL(0x8000077C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P1_2            , RULL(0x80000B7C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P1_2            , RULL(0x80000B7C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P1_3            , RULL(0x80000F7C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P1_3            , RULL(0x80000F7C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P1_4            , RULL(0x8000137C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P1_4            , RULL(0x8000137C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P2_0            , RULL(0x8000037C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P2_0            , RULL(0x8000037C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P2_1            , RULL(0x8000077C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P2_1            , RULL(0x8000077C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P2_2            , RULL(0x80000B7C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P2_2            , RULL(0x80000B7C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P2_3            , RULL(0x80000F7C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P2_3            , RULL(0x80000F7C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P2_4            , RULL(0x8000137C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P2_4            , RULL(0x8000137C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P3_0            , RULL(0x8000037C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P3_0            , RULL(0x8000037C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P3_1            , RULL(0x8000077C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P3_1            , RULL(0x8000077C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P3_2            , RULL(0x80000B7C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P3_2            , RULL(0x80000B7C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P3_3            , RULL(0x80000F7C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P3_3            , RULL(0x80000F7C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P3_4            , RULL(0x8000137C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE0_RP3_P3_4            , RULL(0x8000137C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P0_0              , RULL(0x8000007D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P0_0            , RULL(0x8000007D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P0_0            , RULL(0x8000007D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P0_1              , RULL(0x8000047D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P0_1            , RULL(0x8000047D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P0_1            , RULL(0x8000047D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P0_2              , RULL(0x8000087D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P0_2            , RULL(0x8000087D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P0_2            , RULL(0x8000087D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P0_3              , RULL(0x80000C7D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P0_3            , RULL(0x80000C7D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P0_3            , RULL(0x80000C7D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P0_4              , RULL(0x8000107D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P0_4            , RULL(0x8000107D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P0_4            , RULL(0x8000107D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P1_0            , RULL(0x8000007D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P1_0            , RULL(0x8000007D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P1_1            , RULL(0x8000047D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P1_1            , RULL(0x8000047D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P1_2            , RULL(0x8000087D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P1_2            , RULL(0x8000087D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P1_3            , RULL(0x80000C7D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P1_3            , RULL(0x80000C7D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P1_4            , RULL(0x8000107D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P1_4            , RULL(0x8000107D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P2_0            , RULL(0x8000007D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P2_0            , RULL(0x8000007D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P2_1            , RULL(0x8000047D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P2_1            , RULL(0x8000047D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P2_2            , RULL(0x8000087D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P2_2            , RULL(0x8000087D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P2_3            , RULL(0x80000C7D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P2_3            , RULL(0x80000C7D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P2_4            , RULL(0x8000107D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P2_4            , RULL(0x8000107D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P3_0            , RULL(0x8000007D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P3_0            , RULL(0x8000007D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P3_1            , RULL(0x8000047D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P3_1            , RULL(0x8000047D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P3_2            , RULL(0x8000087D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P3_2            , RULL(0x8000087D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P3_3            , RULL(0x80000C7D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P3_3            , RULL(0x80000C7D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P3_4            , RULL(0x8000107D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP0_P3_4            , RULL(0x8000107D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P0_0              , RULL(0x8000017D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P0_0            , RULL(0x8000017D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P0_0            , RULL(0x8000017D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P0_1              , RULL(0x8000057D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P0_1            , RULL(0x8000057D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P0_1            , RULL(0x8000057D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P0_2              , RULL(0x8000097D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P0_2            , RULL(0x8000097D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P0_2            , RULL(0x8000097D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P0_3              , RULL(0x80000D7D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P0_3            , RULL(0x80000D7D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P0_3            , RULL(0x80000D7D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P0_4              , RULL(0x8000117D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P0_4            , RULL(0x8000117D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P0_4            , RULL(0x8000117D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P1_0            , RULL(0x8000017D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P1_0            , RULL(0x8000017D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P1_1            , RULL(0x8000057D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P1_1            , RULL(0x8000057D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P1_2            , RULL(0x8000097D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P1_2            , RULL(0x8000097D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P1_3            , RULL(0x80000D7D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P1_3            , RULL(0x80000D7D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P1_4            , RULL(0x8000117D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P1_4            , RULL(0x8000117D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P2_0            , RULL(0x8000017D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P2_0            , RULL(0x8000017D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P2_1            , RULL(0x8000057D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P2_1            , RULL(0x8000057D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P2_2            , RULL(0x8000097D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P2_2            , RULL(0x8000097D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P2_3            , RULL(0x80000D7D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P2_3            , RULL(0x80000D7D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P2_4            , RULL(0x8000117D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P2_4            , RULL(0x8000117D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P3_0            , RULL(0x8000017D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P3_0            , RULL(0x8000017D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P3_1            , RULL(0x8000057D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P3_1            , RULL(0x8000057D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P3_2            , RULL(0x8000097D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P3_2            , RULL(0x8000097D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P3_3            , RULL(0x80000D7D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P3_3            , RULL(0x80000D7D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P3_4            , RULL(0x8000117D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP1_P3_4            , RULL(0x8000117D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P0_0              , RULL(0x8000027D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P0_0            , RULL(0x8000027D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P0_0            , RULL(0x8000027D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P0_1              , RULL(0x8000067D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P0_1            , RULL(0x8000067D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P0_1            , RULL(0x8000067D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P0_2              , RULL(0x80000A7D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P0_2            , RULL(0x80000A7D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P0_2            , RULL(0x80000A7D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P0_3              , RULL(0x80000E7D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P0_3            , RULL(0x80000E7D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P0_3            , RULL(0x80000E7D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P0_4              , RULL(0x8000127D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P0_4            , RULL(0x8000127D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P0_4            , RULL(0x8000127D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P1_0            , RULL(0x8000027D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P1_0            , RULL(0x8000027D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P1_1            , RULL(0x8000067D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P1_1            , RULL(0x8000067D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P1_2            , RULL(0x80000A7D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P1_2            , RULL(0x80000A7D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P1_3            , RULL(0x80000E7D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P1_3            , RULL(0x80000E7D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P1_4            , RULL(0x8000127D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P1_4            , RULL(0x8000127D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P2_0            , RULL(0x8000027D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P2_0            , RULL(0x8000027D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P2_1            , RULL(0x8000067D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P2_1            , RULL(0x8000067D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P2_2            , RULL(0x80000A7D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P2_2            , RULL(0x80000A7D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P2_3            , RULL(0x80000E7D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P2_3            , RULL(0x80000E7D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P2_4            , RULL(0x8000127D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P2_4            , RULL(0x8000127D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P3_0            , RULL(0x8000027D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P3_0            , RULL(0x8000027D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P3_1            , RULL(0x8000067D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P3_1            , RULL(0x8000067D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P3_2            , RULL(0x80000A7D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P3_2            , RULL(0x80000A7D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P3_3            , RULL(0x80000E7D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P3_3            , RULL(0x80000E7D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P3_4            , RULL(0x8000127D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP2_P3_4            , RULL(0x8000127D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P0_0              , RULL(0x8000037D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P0_0            , RULL(0x8000037D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P0_0            , RULL(0x8000037D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P0_1              , RULL(0x8000077D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P0_1            , RULL(0x8000077D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P0_1            , RULL(0x8000077D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P0_2              , RULL(0x80000B7D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P0_2            , RULL(0x80000B7D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P0_2            , RULL(0x80000B7D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P0_3              , RULL(0x80000F7D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P0_3            , RULL(0x80000F7D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P0_3            , RULL(0x80000F7D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P0_4              , RULL(0x8000137D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P0_4            , RULL(0x8000137D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P0_4            , RULL(0x8000137D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P1_0            , RULL(0x8000037D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P1_0            , RULL(0x8000037D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P1_1            , RULL(0x8000077D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P1_1            , RULL(0x8000077D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P1_2            , RULL(0x80000B7D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P1_2            , RULL(0x80000B7D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P1_3            , RULL(0x80000F7D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P1_3            , RULL(0x80000F7D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P1_4            , RULL(0x8000137D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P1_4            , RULL(0x8000137D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P2_0            , RULL(0x8000037D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P2_0            , RULL(0x8000037D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P2_1            , RULL(0x8000077D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P2_1            , RULL(0x8000077D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P2_2            , RULL(0x80000B7D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P2_2            , RULL(0x80000B7D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P2_3            , RULL(0x80000F7D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P2_3            , RULL(0x80000F7D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P2_4            , RULL(0x8000137D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P2_4            , RULL(0x8000137D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P3_0            , RULL(0x8000037D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P3_0            , RULL(0x8000037D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P3_1            , RULL(0x8000077D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P3_1            , RULL(0x8000077D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P3_2            , RULL(0x80000B7D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P3_2            , RULL(0x80000B7D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P3_3            , RULL(0x80000F7D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P3_3            , RULL(0x80000F7D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P3_4            , RULL(0x8000137D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_DISABLE1_RP3_P3_4            , RULL(0x8000137D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_ENABLE0_P0_0                   , RULL(0x800000000701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_ENABLE0_P0_0                 , RULL(0x800000000701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_ENABLE0_P0_0                 , RULL(0x800000000801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_ENABLE0_P0_1                   , RULL(0x800004000701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_ENABLE0_P0_1                 , RULL(0x800004000701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_ENABLE0_P0_1                 , RULL(0x800004000801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_ENABLE0_P0_2                   , RULL(0x800008000701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_ENABLE0_P0_2                 , RULL(0x800008000701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_ENABLE0_P0_2                 , RULL(0x800008000801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_ENABLE0_P0_3                   , RULL(0x80000C000701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_ENABLE0_P0_3                 , RULL(0x80000C000701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_ENABLE0_P0_3                 , RULL(0x80000C000801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_ENABLE0_P0_4                   , RULL(0x800010000701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_ENABLE0_P0_4                 , RULL(0x800010000701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_ENABLE0_P0_4                 , RULL(0x800010000801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_ENABLE0_P1_0                 , RULL(0x800000000701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_ENABLE0_P1_0                 , RULL(0x800000000801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_ENABLE0_P1_1                 , RULL(0x800004000701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_ENABLE0_P1_1                 , RULL(0x800004000801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_ENABLE0_P1_2                 , RULL(0x800008000701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_ENABLE0_P1_2                 , RULL(0x800008000801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_ENABLE0_P1_3                 , RULL(0x80000C000701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_ENABLE0_P1_3                 , RULL(0x80000C000801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_ENABLE0_P1_4                 , RULL(0x800010000701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_ENABLE0_P1_4                 , RULL(0x800010000801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_ENABLE0_P2_0                 , RULL(0x800000000701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_ENABLE0_P2_0                 , RULL(0x800000000801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_ENABLE0_P2_1                 , RULL(0x800004000701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_ENABLE0_P2_1                 , RULL(0x800004000801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_ENABLE0_P2_2                 , RULL(0x800008000701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_ENABLE0_P2_2                 , RULL(0x800008000801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_ENABLE0_P2_3                 , RULL(0x80000C000701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_ENABLE0_P2_3                 , RULL(0x80000C000801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_ENABLE0_P2_4                 , RULL(0x800010000701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_ENABLE0_P2_4                 , RULL(0x800010000801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_ENABLE0_P3_0                 , RULL(0x8000000007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_ENABLE0_P3_0                 , RULL(0x8000000008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_ENABLE0_P3_1                 , RULL(0x8000040007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_ENABLE0_P3_1                 , RULL(0x8000040008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_ENABLE0_P3_2                 , RULL(0x8000080007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_ENABLE0_P3_2                 , RULL(0x8000080008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_ENABLE0_P3_3                 , RULL(0x80000C0007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_ENABLE0_P3_3                 , RULL(0x80000C0008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_ENABLE0_P3_4                 , RULL(0x8000100007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_ENABLE0_P3_4                 , RULL(0x8000100008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_ENABLE1_P0_0                   , RULL(0x800000010701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_ENABLE1_P0_0                 , RULL(0x800000010701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_ENABLE1_P0_0                 , RULL(0x800000010801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_ENABLE1_P0_1                   , RULL(0x800004010701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_ENABLE1_P0_1                 , RULL(0x800004010701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_ENABLE1_P0_1                 , RULL(0x800004010801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_ENABLE1_P0_2                   , RULL(0x800008010701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_ENABLE1_P0_2                 , RULL(0x800008010701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_ENABLE1_P0_2                 , RULL(0x800008010801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_ENABLE1_P0_3                   , RULL(0x80000C010701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_ENABLE1_P0_3                 , RULL(0x80000C010701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_ENABLE1_P0_3                 , RULL(0x80000C010801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DATA_BIT_ENABLE1_P0_4                   , RULL(0x800010010701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DATA_BIT_ENABLE1_P0_4                 , RULL(0x800010010701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DATA_BIT_ENABLE1_P0_4                 , RULL(0x800010010801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_ENABLE1_P1_0                 , RULL(0x800000010701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_ENABLE1_P1_0                 , RULL(0x800000010801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_ENABLE1_P1_1                 , RULL(0x800004010701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_ENABLE1_P1_1                 , RULL(0x800004010801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_ENABLE1_P1_2                 , RULL(0x800008010701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_ENABLE1_P1_2                 , RULL(0x800008010801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_ENABLE1_P1_3                 , RULL(0x80000C010701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_ENABLE1_P1_3                 , RULL(0x80000C010801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DATA_BIT_ENABLE1_P1_4                 , RULL(0x800010010701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DATA_BIT_ENABLE1_P1_4                 , RULL(0x800010010801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_ENABLE1_P2_0                 , RULL(0x800000010701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_ENABLE1_P2_0                 , RULL(0x800000010801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_ENABLE1_P2_1                 , RULL(0x800004010701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_ENABLE1_P2_1                 , RULL(0x800004010801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_ENABLE1_P2_2                 , RULL(0x800008010701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_ENABLE1_P2_2                 , RULL(0x800008010801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_ENABLE1_P2_3                 , RULL(0x80000C010701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_ENABLE1_P2_3                 , RULL(0x80000C010801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DATA_BIT_ENABLE1_P2_4                 , RULL(0x800010010701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DATA_BIT_ENABLE1_P2_4                 , RULL(0x800010010801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_ENABLE1_P3_0                 , RULL(0x8000000107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_ENABLE1_P3_0                 , RULL(0x8000000108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_ENABLE1_P3_1                 , RULL(0x8000040107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_ENABLE1_P3_1                 , RULL(0x8000040108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_ENABLE1_P3_2                 , RULL(0x8000080107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_ENABLE1_P3_2                 , RULL(0x8000080108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_ENABLE1_P3_3                 , RULL(0x80000C0107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_ENABLE1_P3_3                 , RULL(0x80000C0108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DATA_BIT_ENABLE1_P3_4                 , RULL(0x8000100107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DATA_BIT_ENABLE1_P3_4                 , RULL(0x8000100108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DCD_CONTROL0_P0_0                       , RULL(0x800000A40701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DCD_CONTROL0_P0_0                     , RULL(0x800000A40701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DCD_CONTROL0_P0_0                     , RULL(0x800000A40801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DCD_CONTROL0_P0_1                       , RULL(0x800004A40701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DCD_CONTROL0_P0_1                     , RULL(0x800004A40701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DCD_CONTROL0_P0_1                     , RULL(0x800004A40801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DCD_CONTROL0_P0_2                       , RULL(0x800008A40701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DCD_CONTROL0_P0_2                     , RULL(0x800008A40701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DCD_CONTROL0_P0_2                     , RULL(0x800008A40801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DCD_CONTROL0_P0_3                       , RULL(0x80000CA40701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DCD_CONTROL0_P0_3                     , RULL(0x80000CA40701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DCD_CONTROL0_P0_3                     , RULL(0x80000CA40801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DCD_CONTROL0_P0_4                       , RULL(0x800010A40701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DCD_CONTROL0_P0_4                     , RULL(0x800010A40701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DCD_CONTROL0_P0_4                     , RULL(0x800010A40801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DCD_CONTROL0_P1_0                     , RULL(0x800000A40701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DCD_CONTROL0_P1_0                     , RULL(0x800000A40801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DCD_CONTROL0_P1_1                     , RULL(0x800004A40701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DCD_CONTROL0_P1_1                     , RULL(0x800004A40801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DCD_CONTROL0_P1_2                     , RULL(0x800008A40701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DCD_CONTROL0_P1_2                     , RULL(0x800008A40801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DCD_CONTROL0_P1_3                     , RULL(0x80000CA40701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DCD_CONTROL0_P1_3                     , RULL(0x80000CA40801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DCD_CONTROL0_P1_4                     , RULL(0x800010A40701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DCD_CONTROL0_P1_4                     , RULL(0x800010A40801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DCD_CONTROL0_P2_0                     , RULL(0x800000A40701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DCD_CONTROL0_P2_0                     , RULL(0x800000A40801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DCD_CONTROL0_P2_1                     , RULL(0x800004A40701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DCD_CONTROL0_P2_1                     , RULL(0x800004A40801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DCD_CONTROL0_P2_2                     , RULL(0x800008A40701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DCD_CONTROL0_P2_2                     , RULL(0x800008A40801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DCD_CONTROL0_P2_3                     , RULL(0x80000CA40701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DCD_CONTROL0_P2_3                     , RULL(0x80000CA40801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DCD_CONTROL0_P2_4                     , RULL(0x800010A40701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DCD_CONTROL0_P2_4                     , RULL(0x800010A40801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DCD_CONTROL0_P3_0                     , RULL(0x800000A407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DCD_CONTROL0_P3_0                     , RULL(0x800000A408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DCD_CONTROL0_P3_1                     , RULL(0x800004A407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DCD_CONTROL0_P3_1                     , RULL(0x800004A408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DCD_CONTROL0_P3_2                     , RULL(0x800008A407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DCD_CONTROL0_P3_2                     , RULL(0x800008A408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DCD_CONTROL0_P3_3                     , RULL(0x80000CA407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DCD_CONTROL0_P3_3                     , RULL(0x80000CA408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DCD_CONTROL0_P3_4                     , RULL(0x800010A407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DCD_CONTROL0_P3_4                     , RULL(0x800010A408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DCD_CONTROL1_P0_0                       , RULL(0x800000A50701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DCD_CONTROL1_P0_0                     , RULL(0x800000A50701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DCD_CONTROL1_P0_0                     , RULL(0x800000A50801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DCD_CONTROL1_P0_1                       , RULL(0x800004A50701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DCD_CONTROL1_P0_1                     , RULL(0x800004A50701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DCD_CONTROL1_P0_1                     , RULL(0x800004A50801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DCD_CONTROL1_P0_2                       , RULL(0x800008A50701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DCD_CONTROL1_P0_2                     , RULL(0x800008A50701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DCD_CONTROL1_P0_2                     , RULL(0x800008A50801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DCD_CONTROL1_P0_3                       , RULL(0x80000CA50701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DCD_CONTROL1_P0_3                     , RULL(0x80000CA50701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DCD_CONTROL1_P0_3                     , RULL(0x80000CA50801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DCD_CONTROL1_P0_4                       , RULL(0x800010A50701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DCD_CONTROL1_P0_4                     , RULL(0x800010A50701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DCD_CONTROL1_P0_4                     , RULL(0x800010A50801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DCD_CONTROL1_P1_0                     , RULL(0x800000A50701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DCD_CONTROL1_P1_0                     , RULL(0x800000A50801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DCD_CONTROL1_P1_1                     , RULL(0x800004A50701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DCD_CONTROL1_P1_1                     , RULL(0x800004A50801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DCD_CONTROL1_P1_2                     , RULL(0x800008A50701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DCD_CONTROL1_P1_2                     , RULL(0x800008A50801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DCD_CONTROL1_P1_3                     , RULL(0x80000CA50701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DCD_CONTROL1_P1_3                     , RULL(0x80000CA50801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DCD_CONTROL1_P1_4                     , RULL(0x800010A50701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DCD_CONTROL1_P1_4                     , RULL(0x800010A50801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DCD_CONTROL1_P2_0                     , RULL(0x800000A50701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DCD_CONTROL1_P2_0                     , RULL(0x800000A50801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DCD_CONTROL1_P2_1                     , RULL(0x800004A50701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DCD_CONTROL1_P2_1                     , RULL(0x800004A50801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DCD_CONTROL1_P2_2                     , RULL(0x800008A50701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DCD_CONTROL1_P2_2                     , RULL(0x800008A50801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DCD_CONTROL1_P2_3                     , RULL(0x80000CA50701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DCD_CONTROL1_P2_3                     , RULL(0x80000CA50801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DCD_CONTROL1_P2_4                     , RULL(0x800010A50701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DCD_CONTROL1_P2_4                     , RULL(0x800010A50801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DCD_CONTROL1_P3_0                     , RULL(0x800000A507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DCD_CONTROL1_P3_0                     , RULL(0x800000A508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DCD_CONTROL1_P3_1                     , RULL(0x800004A507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DCD_CONTROL1_P3_1                     , RULL(0x800004A508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DCD_CONTROL1_P3_2                     , RULL(0x800008A507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DCD_CONTROL1_P3_2                     , RULL(0x800008A508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DCD_CONTROL1_P3_3                     , RULL(0x80000CA507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DCD_CONTROL1_P3_3                     , RULL(0x80000CA508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DCD_CONTROL1_P3_4                     , RULL(0x800010A507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DCD_CONTROL1_P3_4                     , RULL(0x800010A508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DEBUG_SEL_P0_0                          , RULL(0x8000000B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DEBUG_SEL_P0_0                        , RULL(0x8000000B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DEBUG_SEL_P0_0                        , RULL(0x8000000B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DEBUG_SEL_P0_1                          , RULL(0x8000040B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DEBUG_SEL_P0_1                        , RULL(0x8000040B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DEBUG_SEL_P0_1                        , RULL(0x8000040B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DEBUG_SEL_P0_2                          , RULL(0x8000080B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DEBUG_SEL_P0_2                        , RULL(0x8000080B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DEBUG_SEL_P0_2                        , RULL(0x8000080B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DEBUG_SEL_P0_3                          , RULL(0x80000C0B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DEBUG_SEL_P0_3                        , RULL(0x80000C0B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DEBUG_SEL_P0_3                        , RULL(0x80000C0B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DEBUG_SEL_P0_4                          , RULL(0x8000100B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DEBUG_SEL_P0_4                        , RULL(0x8000100B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DEBUG_SEL_P0_4                        , RULL(0x8000100B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DEBUG_SEL_P1_0                        , RULL(0x8000000B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DEBUG_SEL_P1_0                        , RULL(0x8000000B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DEBUG_SEL_P1_1                        , RULL(0x8000040B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DEBUG_SEL_P1_1                        , RULL(0x8000040B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DEBUG_SEL_P1_2                        , RULL(0x8000080B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DEBUG_SEL_P1_2                        , RULL(0x8000080B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DEBUG_SEL_P1_3                        , RULL(0x80000C0B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DEBUG_SEL_P1_3                        , RULL(0x80000C0B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DEBUG_SEL_P1_4                        , RULL(0x8000100B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DEBUG_SEL_P1_4                        , RULL(0x8000100B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DEBUG_SEL_P2_0                        , RULL(0x8000000B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DEBUG_SEL_P2_0                        , RULL(0x8000000B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DEBUG_SEL_P2_1                        , RULL(0x8000040B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DEBUG_SEL_P2_1                        , RULL(0x8000040B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DEBUG_SEL_P2_2                        , RULL(0x8000080B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DEBUG_SEL_P2_2                        , RULL(0x8000080B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DEBUG_SEL_P2_3                        , RULL(0x80000C0B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DEBUG_SEL_P2_3                        , RULL(0x80000C0B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DEBUG_SEL_P2_4                        , RULL(0x8000100B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DEBUG_SEL_P2_4                        , RULL(0x8000100B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DEBUG_SEL_P3_0                        , RULL(0x8000000B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DEBUG_SEL_P3_0                        , RULL(0x8000000B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DEBUG_SEL_P3_1                        , RULL(0x8000040B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DEBUG_SEL_P3_1                        , RULL(0x8000040B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DEBUG_SEL_P3_2                        , RULL(0x8000080B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DEBUG_SEL_P3_2                        , RULL(0x8000080B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DEBUG_SEL_P3_3                        , RULL(0x80000C0B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DEBUG_SEL_P3_3                        , RULL(0x80000C0B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DEBUG_SEL_P3_4                        , RULL(0x8000100B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DEBUG_SEL_P3_4                        , RULL(0x8000100B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P0_0                 , RULL(0x8000006F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P0_0               , RULL(0x8000006F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P0_0               , RULL(0x8000006F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P0_1                 , RULL(0x8000046F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P0_1               , RULL(0x8000046F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P0_1               , RULL(0x8000046F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P0_2                 , RULL(0x8000086F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P0_2               , RULL(0x8000086F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P0_2               , RULL(0x8000086F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P0_3                 , RULL(0x80000C6F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P0_3               , RULL(0x80000C6F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P0_3               , RULL(0x80000C6F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P0_4                 , RULL(0x8000106F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P0_4               , RULL(0x8000106F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P0_4               , RULL(0x8000106F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P1_0               , RULL(0x8000006F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P1_0               , RULL(0x8000006F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P1_1               , RULL(0x8000046F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P1_1               , RULL(0x8000046F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P1_2               , RULL(0x8000086F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P1_2               , RULL(0x8000086F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P1_3               , RULL(0x80000C6F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P1_3               , RULL(0x80000C6F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P1_4               , RULL(0x8000106F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P1_4               , RULL(0x8000106F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P2_0               , RULL(0x8000006F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P2_0               , RULL(0x8000006F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P2_1               , RULL(0x8000046F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P2_1               , RULL(0x8000046F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P2_2               , RULL(0x8000086F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P2_2               , RULL(0x8000086F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P2_3               , RULL(0x80000C6F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P2_3               , RULL(0x80000C6F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P2_4               , RULL(0x8000106F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P2_4               , RULL(0x8000106F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P3_0               , RULL(0x8000006F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P3_0               , RULL(0x8000006F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P3_1               , RULL(0x8000046F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P3_1               , RULL(0x8000046F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P3_2               , RULL(0x8000086F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P3_2               , RULL(0x8000086F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P3_3               , RULL(0x80000C6F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P3_3               , RULL(0x80000C6F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P3_4               , RULL(0x8000106F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DELAY_LINE_PWR_CTL_P3_4               , RULL(0x8000106F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DFT_DIG_EYE_P0_0                        , RULL(0x800000080701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DFT_DIG_EYE_P0_0                      , RULL(0x800000080701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DFT_DIG_EYE_P0_0                      , RULL(0x800000080801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DFT_DIG_EYE_P0_1                        , RULL(0x800004080701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DFT_DIG_EYE_P0_1                      , RULL(0x800004080701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DFT_DIG_EYE_P0_1                      , RULL(0x800004080801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DFT_DIG_EYE_P0_2                        , RULL(0x800008080701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DFT_DIG_EYE_P0_2                      , RULL(0x800008080701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DFT_DIG_EYE_P0_2                      , RULL(0x800008080801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DFT_DIG_EYE_P0_3                        , RULL(0x80000C080701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DFT_DIG_EYE_P0_3                      , RULL(0x80000C080701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DFT_DIG_EYE_P0_3                      , RULL(0x80000C080801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DFT_DIG_EYE_P0_4                        , RULL(0x800010080701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DFT_DIG_EYE_P0_4                      , RULL(0x800010080701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DFT_DIG_EYE_P0_4                      , RULL(0x800010080801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DFT_DIG_EYE_P1_0                      , RULL(0x800000080701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DFT_DIG_EYE_P1_0                      , RULL(0x800000080801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DFT_DIG_EYE_P1_1                      , RULL(0x800004080701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DFT_DIG_EYE_P1_1                      , RULL(0x800004080801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DFT_DIG_EYE_P1_2                      , RULL(0x800008080701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DFT_DIG_EYE_P1_2                      , RULL(0x800008080801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DFT_DIG_EYE_P1_3                      , RULL(0x80000C080701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DFT_DIG_EYE_P1_3                      , RULL(0x80000C080801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DFT_DIG_EYE_P1_4                      , RULL(0x800010080701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DFT_DIG_EYE_P1_4                      , RULL(0x800010080801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DFT_DIG_EYE_P2_0                      , RULL(0x800000080701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DFT_DIG_EYE_P2_0                      , RULL(0x800000080801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DFT_DIG_EYE_P2_1                      , RULL(0x800004080701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DFT_DIG_EYE_P2_1                      , RULL(0x800004080801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DFT_DIG_EYE_P2_2                      , RULL(0x800008080701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DFT_DIG_EYE_P2_2                      , RULL(0x800008080801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DFT_DIG_EYE_P2_3                      , RULL(0x80000C080701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DFT_DIG_EYE_P2_3                      , RULL(0x80000C080801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DFT_DIG_EYE_P2_4                      , RULL(0x800010080701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DFT_DIG_EYE_P2_4                      , RULL(0x800010080801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DFT_DIG_EYE_P3_0                      , RULL(0x8000000807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DFT_DIG_EYE_P3_0                      , RULL(0x8000000808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DFT_DIG_EYE_P3_1                      , RULL(0x8000040807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DFT_DIG_EYE_P3_1                      , RULL(0x8000040808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DFT_DIG_EYE_P3_2                      , RULL(0x8000080807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DFT_DIG_EYE_P3_2                      , RULL(0x8000080808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DFT_DIG_EYE_P3_3                      , RULL(0x80000C0807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DFT_DIG_EYE_P3_3                      , RULL(0x80000C0808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DFT_DIG_EYE_P3_4                      , RULL(0x8000100807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DFT_DIG_EYE_P3_4                      , RULL(0x8000100808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DFT_WRAP_STATUS_P0_0                    , RULL(0x8000001D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DFT_WRAP_STATUS_P0_0                  , RULL(0x8000001D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DFT_WRAP_STATUS_P0_0                  , RULL(0x8000001D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DFT_WRAP_STATUS_P0_1                    , RULL(0x8000041D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DFT_WRAP_STATUS_P0_1                  , RULL(0x8000041D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DFT_WRAP_STATUS_P0_1                  , RULL(0x8000041D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DFT_WRAP_STATUS_P0_2                    , RULL(0x8000081D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DFT_WRAP_STATUS_P0_2                  , RULL(0x8000081D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DFT_WRAP_STATUS_P0_2                  , RULL(0x8000081D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DFT_WRAP_STATUS_P0_3                    , RULL(0x80000C1D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DFT_WRAP_STATUS_P0_3                  , RULL(0x80000C1D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DFT_WRAP_STATUS_P0_3                  , RULL(0x80000C1D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DFT_WRAP_STATUS_P0_4                    , RULL(0x8000101D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DFT_WRAP_STATUS_P0_4                  , RULL(0x8000101D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DFT_WRAP_STATUS_P0_4                  , RULL(0x8000101D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DFT_WRAP_STATUS_P1_0                  , RULL(0x8000001D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DFT_WRAP_STATUS_P1_0                  , RULL(0x8000001D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DFT_WRAP_STATUS_P1_1                  , RULL(0x8000041D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DFT_WRAP_STATUS_P1_1                  , RULL(0x8000041D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DFT_WRAP_STATUS_P1_2                  , RULL(0x8000081D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DFT_WRAP_STATUS_P1_2                  , RULL(0x8000081D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DFT_WRAP_STATUS_P1_3                  , RULL(0x80000C1D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DFT_WRAP_STATUS_P1_3                  , RULL(0x80000C1D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DFT_WRAP_STATUS_P1_4                  , RULL(0x8000101D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DFT_WRAP_STATUS_P1_4                  , RULL(0x8000101D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DFT_WRAP_STATUS_P2_0                  , RULL(0x8000001D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DFT_WRAP_STATUS_P2_0                  , RULL(0x8000001D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DFT_WRAP_STATUS_P2_1                  , RULL(0x8000041D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DFT_WRAP_STATUS_P2_1                  , RULL(0x8000041D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DFT_WRAP_STATUS_P2_2                  , RULL(0x8000081D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DFT_WRAP_STATUS_P2_2                  , RULL(0x8000081D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DFT_WRAP_STATUS_P2_3                  , RULL(0x80000C1D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DFT_WRAP_STATUS_P2_3                  , RULL(0x80000C1D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DFT_WRAP_STATUS_P2_4                  , RULL(0x8000101D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DFT_WRAP_STATUS_P2_4                  , RULL(0x8000101D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DFT_WRAP_STATUS_P3_0                  , RULL(0x8000001D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DFT_WRAP_STATUS_P3_0                  , RULL(0x8000001D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DFT_WRAP_STATUS_P3_1                  , RULL(0x8000041D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DFT_WRAP_STATUS_P3_1                  , RULL(0x8000041D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DFT_WRAP_STATUS_P3_2                  , RULL(0x8000081D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DFT_WRAP_STATUS_P3_2                  , RULL(0x8000081D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DFT_WRAP_STATUS_P3_3                  , RULL(0x80000C1D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DFT_WRAP_STATUS_P3_3                  , RULL(0x80000C1D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DFT_WRAP_STATUS_P3_4                  , RULL(0x8000101D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DFT_WRAP_STATUS_P3_4                  , RULL(0x8000101D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_CNTL0_P0_0                          , RULL(0x800000240701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_CNTL0_P0_0                        , RULL(0x800000240701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_CNTL0_P0_0                        , RULL(0x800000240801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_CNTL0_P0_1                          , RULL(0x800004240701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_CNTL0_P0_1                        , RULL(0x800004240701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_CNTL0_P0_1                        , RULL(0x800004240801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_CNTL0_P0_2                          , RULL(0x800008240701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_CNTL0_P0_2                        , RULL(0x800008240701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_CNTL0_P0_2                        , RULL(0x800008240801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_CNTL0_P0_3                          , RULL(0x80000C240701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_CNTL0_P0_3                        , RULL(0x80000C240701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_CNTL0_P0_3                        , RULL(0x80000C240801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_CNTL0_P0_4                          , RULL(0x800010240701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_CNTL0_P0_4                        , RULL(0x800010240701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_CNTL0_P0_4                        , RULL(0x800010240801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_CNTL0_P1_0                        , RULL(0x800000240701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_CNTL0_P1_0                        , RULL(0x800000240801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_CNTL0_P1_1                        , RULL(0x800004240701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_CNTL0_P1_1                        , RULL(0x800004240801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_CNTL0_P1_2                        , RULL(0x800008240701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_CNTL0_P1_2                        , RULL(0x800008240801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_CNTL0_P1_3                        , RULL(0x80000C240701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_CNTL0_P1_3                        , RULL(0x80000C240801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_CNTL0_P1_4                        , RULL(0x800010240701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_CNTL0_P1_4                        , RULL(0x800010240801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_CNTL0_P2_0                        , RULL(0x800000240701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_CNTL0_P2_0                        , RULL(0x800000240801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_CNTL0_P2_1                        , RULL(0x800004240701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_CNTL0_P2_1                        , RULL(0x800004240801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_CNTL0_P2_2                        , RULL(0x800008240701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_CNTL0_P2_2                        , RULL(0x800008240801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_CNTL0_P2_3                        , RULL(0x80000C240701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_CNTL0_P2_3                        , RULL(0x80000C240801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_CNTL0_P2_4                        , RULL(0x800010240701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_CNTL0_P2_4                        , RULL(0x800010240801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_CNTL0_P3_0                        , RULL(0x8000002407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_CNTL0_P3_0                        , RULL(0x8000002408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_CNTL0_P3_1                        , RULL(0x8000042407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_CNTL0_P3_1                        , RULL(0x8000042408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_CNTL0_P3_2                        , RULL(0x8000082407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_CNTL0_P3_2                        , RULL(0x8000082408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_CNTL0_P3_3                        , RULL(0x80000C2407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_CNTL0_P3_3                        , RULL(0x80000C2408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_CNTL0_P3_4                        , RULL(0x8000102407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_CNTL0_P3_4                        , RULL(0x8000102408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_CNTL1_P0_0                          , RULL(0x800000250701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_CNTL1_P0_0                        , RULL(0x800000250701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_CNTL1_P0_0                        , RULL(0x800000250801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_CNTL1_P0_1                          , RULL(0x800004250701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_CNTL1_P0_1                        , RULL(0x800004250701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_CNTL1_P0_1                        , RULL(0x800004250801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_CNTL1_P0_2                          , RULL(0x800008250701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_CNTL1_P0_2                        , RULL(0x800008250701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_CNTL1_P0_2                        , RULL(0x800008250801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_CNTL1_P0_3                          , RULL(0x80000C250701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_CNTL1_P0_3                        , RULL(0x80000C250701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_CNTL1_P0_3                        , RULL(0x80000C250801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_CNTL1_P0_4                          , RULL(0x800010250701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_CNTL1_P0_4                        , RULL(0x800010250701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_CNTL1_P0_4                        , RULL(0x800010250801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_CNTL1_P1_0                        , RULL(0x800000250701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_CNTL1_P1_0                        , RULL(0x800000250801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_CNTL1_P1_1                        , RULL(0x800004250701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_CNTL1_P1_1                        , RULL(0x800004250801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_CNTL1_P1_2                        , RULL(0x800008250701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_CNTL1_P1_2                        , RULL(0x800008250801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_CNTL1_P1_3                        , RULL(0x80000C250701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_CNTL1_P1_3                        , RULL(0x80000C250801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_CNTL1_P1_4                        , RULL(0x800010250701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_CNTL1_P1_4                        , RULL(0x800010250801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_CNTL1_P2_0                        , RULL(0x800000250701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_CNTL1_P2_0                        , RULL(0x800000250801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_CNTL1_P2_1                        , RULL(0x800004250701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_CNTL1_P2_1                        , RULL(0x800004250801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_CNTL1_P2_2                        , RULL(0x800008250701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_CNTL1_P2_2                        , RULL(0x800008250801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_CNTL1_P2_3                        , RULL(0x80000C250701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_CNTL1_P2_3                        , RULL(0x80000C250801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_CNTL1_P2_4                        , RULL(0x800010250701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_CNTL1_P2_4                        , RULL(0x800010250801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_CNTL1_P3_0                        , RULL(0x8000002507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_CNTL1_P3_0                        , RULL(0x8000002508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_CNTL1_P3_1                        , RULL(0x8000042507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_CNTL1_P3_1                        , RULL(0x8000042508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_CNTL1_P3_2                        , RULL(0x8000082507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_CNTL1_P3_2                        , RULL(0x8000082508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_CNTL1_P3_3                        , RULL(0x80000C2507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_CNTL1_P3_3                        , RULL(0x80000C2508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_CNTL1_P3_4                        , RULL(0x8000102507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_CNTL1_P3_4                        , RULL(0x8000102508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_CONFIG1_P0_0                        , RULL(0x800000770701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_CONFIG1_P0_0                      , RULL(0x800000770701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_CONFIG1_P0_0                      , RULL(0x800000770801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_CONFIG1_P0_1                        , RULL(0x800004770701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_CONFIG1_P0_1                      , RULL(0x800004770701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_CONFIG1_P0_1                      , RULL(0x800004770801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_CONFIG1_P0_2                        , RULL(0x800008770701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_CONFIG1_P0_2                      , RULL(0x800008770701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_CONFIG1_P0_2                      , RULL(0x800008770801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_CONFIG1_P0_3                        , RULL(0x80000C770701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_CONFIG1_P0_3                      , RULL(0x80000C770701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_CONFIG1_P0_3                      , RULL(0x80000C770801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_CONFIG1_P0_4                        , RULL(0x800010770701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_CONFIG1_P0_4                      , RULL(0x800010770701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_CONFIG1_P0_4                      , RULL(0x800010770801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_CONFIG1_P1_0                      , RULL(0x800000770701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_CONFIG1_P1_0                      , RULL(0x800000770801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_CONFIG1_P1_1                      , RULL(0x800004770701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_CONFIG1_P1_1                      , RULL(0x800004770801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_CONFIG1_P1_2                      , RULL(0x800008770701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_CONFIG1_P1_2                      , RULL(0x800008770801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_CONFIG1_P1_3                      , RULL(0x80000C770701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_CONFIG1_P1_3                      , RULL(0x80000C770801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_CONFIG1_P1_4                      , RULL(0x800010770701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_CONFIG1_P1_4                      , RULL(0x800010770801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_CONFIG1_P2_0                      , RULL(0x800000770701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_CONFIG1_P2_0                      , RULL(0x800000770801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_CONFIG1_P2_1                      , RULL(0x800004770701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_CONFIG1_P2_1                      , RULL(0x800004770801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_CONFIG1_P2_2                      , RULL(0x800008770701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_CONFIG1_P2_2                      , RULL(0x800008770801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_CONFIG1_P2_3                      , RULL(0x80000C770701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_CONFIG1_P2_3                      , RULL(0x80000C770801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_CONFIG1_P2_4                      , RULL(0x800010770701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_CONFIG1_P2_4                      , RULL(0x800010770801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_CONFIG1_P3_0                      , RULL(0x8000007707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_CONFIG1_P3_0                      , RULL(0x8000007708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_CONFIG1_P3_1                      , RULL(0x8000047707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_CONFIG1_P3_1                      , RULL(0x8000047708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_CONFIG1_P3_2                      , RULL(0x8000087707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_CONFIG1_P3_2                      , RULL(0x8000087708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_CONFIG1_P3_3                      , RULL(0x80000C7707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_CONFIG1_P3_3                      , RULL(0x80000C7708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_CONFIG1_P3_4                      , RULL(0x8000107707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_CONFIG1_P3_4                      , RULL(0x8000107708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_LOWER0_P0_0                     , RULL(0x800000260701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_LOWER0_P0_0                   , RULL(0x800000260701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_LOWER0_P0_0                   , RULL(0x800000260801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_LOWER0_P0_1                     , RULL(0x800004260701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_LOWER0_P0_1                   , RULL(0x800004260701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_LOWER0_P0_1                   , RULL(0x800004260801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_LOWER0_P0_2                     , RULL(0x800008260701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_LOWER0_P0_2                   , RULL(0x800008260701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_LOWER0_P0_2                   , RULL(0x800008260801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_LOWER0_P0_3                     , RULL(0x80000C260701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_LOWER0_P0_3                   , RULL(0x80000C260701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_LOWER0_P0_3                   , RULL(0x80000C260801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_LOWER0_P0_4                     , RULL(0x800010260701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_LOWER0_P0_4                   , RULL(0x800010260701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_LOWER0_P0_4                   , RULL(0x800010260801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_LOWER0_P1_0                   , RULL(0x800000260701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_LOWER0_P1_0                   , RULL(0x800000260801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_LOWER0_P1_1                   , RULL(0x800004260701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_LOWER0_P1_1                   , RULL(0x800004260801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_LOWER0_P1_2                   , RULL(0x800008260701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_LOWER0_P1_2                   , RULL(0x800008260801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_LOWER0_P1_3                   , RULL(0x80000C260701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_LOWER0_P1_3                   , RULL(0x80000C260801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_LOWER0_P1_4                   , RULL(0x800010260701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_LOWER0_P1_4                   , RULL(0x800010260801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_LOWER0_P2_0                   , RULL(0x800000260701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_LOWER0_P2_0                   , RULL(0x800000260801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_LOWER0_P2_1                   , RULL(0x800004260701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_LOWER0_P2_1                   , RULL(0x800004260801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_LOWER0_P2_2                   , RULL(0x800008260701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_LOWER0_P2_2                   , RULL(0x800008260801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_LOWER0_P2_3                   , RULL(0x80000C260701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_LOWER0_P2_3                   , RULL(0x80000C260801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_LOWER0_P2_4                   , RULL(0x800010260701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_LOWER0_P2_4                   , RULL(0x800010260801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_LOWER0_P3_0                   , RULL(0x8000002607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_LOWER0_P3_0                   , RULL(0x8000002608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_LOWER0_P3_1                   , RULL(0x8000042607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_LOWER0_P3_1                   , RULL(0x8000042608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_LOWER0_P3_2                   , RULL(0x8000082607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_LOWER0_P3_2                   , RULL(0x8000082608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_LOWER0_P3_3                   , RULL(0x80000C2607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_LOWER0_P3_3                   , RULL(0x80000C2608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_LOWER0_P3_4                   , RULL(0x8000102607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_LOWER0_P3_4                   , RULL(0x8000102608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_LOWER1_P0_0                     , RULL(0x800000270701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_LOWER1_P0_0                   , RULL(0x800000270701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_LOWER1_P0_0                   , RULL(0x800000270801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_LOWER1_P0_1                     , RULL(0x800004270701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_LOWER1_P0_1                   , RULL(0x800004270701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_LOWER1_P0_1                   , RULL(0x800004270801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_LOWER1_P0_2                     , RULL(0x800008270701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_LOWER1_P0_2                   , RULL(0x800008270701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_LOWER1_P0_2                   , RULL(0x800008270801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_LOWER1_P0_3                     , RULL(0x80000C270701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_LOWER1_P0_3                   , RULL(0x80000C270701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_LOWER1_P0_3                   , RULL(0x80000C270801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_LOWER1_P0_4                     , RULL(0x800010270701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_LOWER1_P0_4                   , RULL(0x800010270701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_LOWER1_P0_4                   , RULL(0x800010270801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_LOWER1_P1_0                   , RULL(0x800000270701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_LOWER1_P1_0                   , RULL(0x800000270801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_LOWER1_P1_1                   , RULL(0x800004270701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_LOWER1_P1_1                   , RULL(0x800004270801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_LOWER1_P1_2                   , RULL(0x800008270701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_LOWER1_P1_2                   , RULL(0x800008270801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_LOWER1_P1_3                   , RULL(0x80000C270701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_LOWER1_P1_3                   , RULL(0x80000C270801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_LOWER1_P1_4                   , RULL(0x800010270701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_LOWER1_P1_4                   , RULL(0x800010270801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_LOWER1_P2_0                   , RULL(0x800000270701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_LOWER1_P2_0                   , RULL(0x800000270801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_LOWER1_P2_1                   , RULL(0x800004270701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_LOWER1_P2_1                   , RULL(0x800004270801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_LOWER1_P2_2                   , RULL(0x800008270701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_LOWER1_P2_2                   , RULL(0x800008270801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_LOWER1_P2_3                   , RULL(0x80000C270701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_LOWER1_P2_3                   , RULL(0x80000C270801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_LOWER1_P2_4                   , RULL(0x800010270701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_LOWER1_P2_4                   , RULL(0x800010270801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_LOWER1_P3_0                   , RULL(0x8000002707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_LOWER1_P3_0                   , RULL(0x8000002708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_LOWER1_P3_1                   , RULL(0x8000042707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_LOWER1_P3_1                   , RULL(0x8000042708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_LOWER1_P3_2                   , RULL(0x8000082707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_LOWER1_P3_2                   , RULL(0x8000082708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_LOWER1_P3_3                   , RULL(0x80000C2707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_LOWER1_P3_3                   , RULL(0x80000C2708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_LOWER1_P3_4                   , RULL(0x8000102707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_LOWER1_P3_4                   , RULL(0x8000102708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_UPPER0_P0_0                     , RULL(0x800000280701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_UPPER0_P0_0                   , RULL(0x800000280701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_UPPER0_P0_0                   , RULL(0x800000280801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_UPPER0_P0_1                     , RULL(0x800004280701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_UPPER0_P0_1                   , RULL(0x800004280701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_UPPER0_P0_1                   , RULL(0x800004280801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_UPPER0_P0_2                     , RULL(0x800008280701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_UPPER0_P0_2                   , RULL(0x800008280701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_UPPER0_P0_2                   , RULL(0x800008280801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_UPPER0_P0_3                     , RULL(0x80000C280701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_UPPER0_P0_3                   , RULL(0x80000C280701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_UPPER0_P0_3                   , RULL(0x80000C280801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_UPPER0_P0_4                     , RULL(0x800010280701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_UPPER0_P0_4                   , RULL(0x800010280701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_UPPER0_P0_4                   , RULL(0x800010280801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_UPPER0_P1_0                   , RULL(0x800000280701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_UPPER0_P1_0                   , RULL(0x800000280801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_UPPER0_P1_1                   , RULL(0x800004280701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_UPPER0_P1_1                   , RULL(0x800004280801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_UPPER0_P1_2                   , RULL(0x800008280701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_UPPER0_P1_2                   , RULL(0x800008280801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_UPPER0_P1_3                   , RULL(0x80000C280701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_UPPER0_P1_3                   , RULL(0x80000C280801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_UPPER0_P1_4                   , RULL(0x800010280701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_UPPER0_P1_4                   , RULL(0x800010280801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_UPPER0_P2_0                   , RULL(0x800000280701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_UPPER0_P2_0                   , RULL(0x800000280801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_UPPER0_P2_1                   , RULL(0x800004280701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_UPPER0_P2_1                   , RULL(0x800004280801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_UPPER0_P2_2                   , RULL(0x800008280701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_UPPER0_P2_2                   , RULL(0x800008280801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_UPPER0_P2_3                   , RULL(0x80000C280701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_UPPER0_P2_3                   , RULL(0x80000C280801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_UPPER0_P2_4                   , RULL(0x800010280701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_UPPER0_P2_4                   , RULL(0x800010280801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_UPPER0_P3_0                   , RULL(0x8000002807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_UPPER0_P3_0                   , RULL(0x8000002808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_UPPER0_P3_1                   , RULL(0x8000042807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_UPPER0_P3_1                   , RULL(0x8000042808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_UPPER0_P3_2                   , RULL(0x8000082807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_UPPER0_P3_2                   , RULL(0x8000082808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_UPPER0_P3_3                   , RULL(0x80000C2807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_UPPER0_P3_3                   , RULL(0x80000C2808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_UPPER0_P3_4                   , RULL(0x8000102807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_UPPER0_P3_4                   , RULL(0x8000102808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_UPPER1_P0_0                     , RULL(0x800000290701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_UPPER1_P0_0                   , RULL(0x800000290701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_UPPER1_P0_0                   , RULL(0x800000290801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_UPPER1_P0_1                     , RULL(0x800004290701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_UPPER1_P0_1                   , RULL(0x800004290701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_UPPER1_P0_1                   , RULL(0x800004290801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_UPPER1_P0_2                     , RULL(0x800008290701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_UPPER1_P0_2                   , RULL(0x800008290701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_UPPER1_P0_2                   , RULL(0x800008290801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_UPPER1_P0_3                     , RULL(0x80000C290701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_UPPER1_P0_3                   , RULL(0x80000C290701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_UPPER1_P0_3                   , RULL(0x80000C290801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_DAC_UPPER1_P0_4                     , RULL(0x800010290701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_DAC_UPPER1_P0_4                   , RULL(0x800010290701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_DAC_UPPER1_P0_4                   , RULL(0x800010290801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_UPPER1_P1_0                   , RULL(0x800000290701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_UPPER1_P1_0                   , RULL(0x800000290801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_UPPER1_P1_1                   , RULL(0x800004290701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_UPPER1_P1_1                   , RULL(0x800004290801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_UPPER1_P1_2                   , RULL(0x800008290701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_UPPER1_P1_2                   , RULL(0x800008290801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_UPPER1_P1_3                   , RULL(0x80000C290701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_UPPER1_P1_3                   , RULL(0x80000C290801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_DAC_UPPER1_P1_4                   , RULL(0x800010290701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_DAC_UPPER1_P1_4                   , RULL(0x800010290801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_UPPER1_P2_0                   , RULL(0x800000290701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_UPPER1_P2_0                   , RULL(0x800000290801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_UPPER1_P2_1                   , RULL(0x800004290701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_UPPER1_P2_1                   , RULL(0x800004290801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_UPPER1_P2_2                   , RULL(0x800008290701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_UPPER1_P2_2                   , RULL(0x800008290801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_UPPER1_P2_3                   , RULL(0x80000C290701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_UPPER1_P2_3                   , RULL(0x80000C290801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_DAC_UPPER1_P2_4                   , RULL(0x800010290701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_DAC_UPPER1_P2_4                   , RULL(0x800010290801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_UPPER1_P3_0                   , RULL(0x8000002907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_UPPER1_P3_0                   , RULL(0x8000002908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_UPPER1_P3_1                   , RULL(0x8000042907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_UPPER1_P3_1                   , RULL(0x8000042908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_UPPER1_P3_2                   , RULL(0x8000082907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_UPPER1_P3_2                   , RULL(0x8000082908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_UPPER1_P3_3                   , RULL(0x80000C2907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_UPPER1_P3_3                   , RULL(0x80000C2908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_DAC_UPPER1_P3_4                   , RULL(0x8000102907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_DAC_UPPER1_P3_4                   , RULL(0x8000102908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_EXTRA0_P0_0                         , RULL(0x800000AC0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_EXTRA0_P0_0                       , RULL(0x800000AC0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_EXTRA0_P0_0                       , RULL(0x800000AC0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_EXTRA0_P0_1                         , RULL(0x800004AC0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_EXTRA0_P0_1                       , RULL(0x800004AC0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_EXTRA0_P0_1                       , RULL(0x800004AC0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_EXTRA0_P0_2                         , RULL(0x800008AC0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_EXTRA0_P0_2                       , RULL(0x800008AC0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_EXTRA0_P0_2                       , RULL(0x800008AC0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_EXTRA0_P0_3                         , RULL(0x80000CAC0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_EXTRA0_P0_3                       , RULL(0x80000CAC0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_EXTRA0_P0_3                       , RULL(0x80000CAC0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_EXTRA0_P0_4                         , RULL(0x800010AC0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_EXTRA0_P0_4                       , RULL(0x800010AC0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_EXTRA0_P0_4                       , RULL(0x800010AC0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_EXTRA0_P1_0                       , RULL(0x800000AC0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_EXTRA0_P1_0                       , RULL(0x800000AC0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_EXTRA0_P1_1                       , RULL(0x800004AC0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_EXTRA0_P1_1                       , RULL(0x800004AC0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_EXTRA0_P1_2                       , RULL(0x800008AC0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_EXTRA0_P1_2                       , RULL(0x800008AC0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_EXTRA0_P1_3                       , RULL(0x80000CAC0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_EXTRA0_P1_3                       , RULL(0x80000CAC0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_EXTRA0_P1_4                       , RULL(0x800010AC0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_EXTRA0_P1_4                       , RULL(0x800010AC0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_EXTRA0_P2_0                       , RULL(0x800000AC0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_EXTRA0_P2_0                       , RULL(0x800000AC0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_EXTRA0_P2_1                       , RULL(0x800004AC0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_EXTRA0_P2_1                       , RULL(0x800004AC0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_EXTRA0_P2_2                       , RULL(0x800008AC0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_EXTRA0_P2_2                       , RULL(0x800008AC0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_EXTRA0_P2_3                       , RULL(0x80000CAC0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_EXTRA0_P2_3                       , RULL(0x80000CAC0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_EXTRA0_P2_4                       , RULL(0x800010AC0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_EXTRA0_P2_4                       , RULL(0x800010AC0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_EXTRA0_P3_0                       , RULL(0x800000AC07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_EXTRA0_P3_0                       , RULL(0x800000AC08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_EXTRA0_P3_1                       , RULL(0x800004AC07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_EXTRA0_P3_1                       , RULL(0x800004AC08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_EXTRA0_P3_2                       , RULL(0x800008AC07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_EXTRA0_P3_2                       , RULL(0x800008AC08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_EXTRA0_P3_3                       , RULL(0x80000CAC07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_EXTRA0_P3_3                       , RULL(0x80000CAC08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_EXTRA0_P3_4                       , RULL(0x800010AC07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_EXTRA0_P3_4                       , RULL(0x800010AC08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_EXTRA1_P0_0                         , RULL(0x800000AD0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_EXTRA1_P0_0                       , RULL(0x800000AD0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_EXTRA1_P0_0                       , RULL(0x800000AD0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_EXTRA1_P0_1                         , RULL(0x800004AD0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_EXTRA1_P0_1                       , RULL(0x800004AD0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_EXTRA1_P0_1                       , RULL(0x800004AD0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_EXTRA1_P0_2                         , RULL(0x800008AD0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_EXTRA1_P0_2                       , RULL(0x800008AD0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_EXTRA1_P0_2                       , RULL(0x800008AD0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_EXTRA1_P0_3                         , RULL(0x80000CAD0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_EXTRA1_P0_3                       , RULL(0x80000CAD0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_EXTRA1_P0_3                       , RULL(0x80000CAD0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_EXTRA1_P0_4                         , RULL(0x800010AD0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_EXTRA1_P0_4                       , RULL(0x800010AD0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_EXTRA1_P0_4                       , RULL(0x800010AD0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_EXTRA1_P1_0                       , RULL(0x800000AD0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_EXTRA1_P1_0                       , RULL(0x800000AD0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_EXTRA1_P1_1                       , RULL(0x800004AD0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_EXTRA1_P1_1                       , RULL(0x800004AD0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_EXTRA1_P1_2                       , RULL(0x800008AD0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_EXTRA1_P1_2                       , RULL(0x800008AD0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_EXTRA1_P1_3                       , RULL(0x80000CAD0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_EXTRA1_P1_3                       , RULL(0x80000CAD0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_EXTRA1_P1_4                       , RULL(0x800010AD0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_EXTRA1_P1_4                       , RULL(0x800010AD0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_EXTRA1_P2_0                       , RULL(0x800000AD0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_EXTRA1_P2_0                       , RULL(0x800000AD0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_EXTRA1_P2_1                       , RULL(0x800004AD0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_EXTRA1_P2_1                       , RULL(0x800004AD0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_EXTRA1_P2_2                       , RULL(0x800008AD0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_EXTRA1_P2_2                       , RULL(0x800008AD0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_EXTRA1_P2_3                       , RULL(0x80000CAD0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_EXTRA1_P2_3                       , RULL(0x80000CAD0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_EXTRA1_P2_4                       , RULL(0x800010AD0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_EXTRA1_P2_4                       , RULL(0x800010AD0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_EXTRA1_P3_0                       , RULL(0x800000AD07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_EXTRA1_P3_0                       , RULL(0x800000AD08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_EXTRA1_P3_1                       , RULL(0x800004AD07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_EXTRA1_P3_1                       , RULL(0x800004AD08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_EXTRA1_P3_2                       , RULL(0x800008AD07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_EXTRA1_P3_2                       , RULL(0x800008AD08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_EXTRA1_P3_3                       , RULL(0x80000CAD07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_EXTRA1_P3_3                       , RULL(0x80000CAD08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_EXTRA1_P3_4                       , RULL(0x800010AD07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_EXTRA1_P3_4                       , RULL(0x800010AD08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P0_0              , RULL(0x800000A80701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P0_0            , RULL(0x800000A80701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P0_0            , RULL(0x800000A80801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P0_1              , RULL(0x800004A80701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P0_1            , RULL(0x800004A80701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P0_1            , RULL(0x800004A80801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P0_2              , RULL(0x800008A80701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P0_2            , RULL(0x800008A80701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P0_2            , RULL(0x800008A80801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P0_3              , RULL(0x80000CA80701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P0_3            , RULL(0x80000CA80701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P0_3            , RULL(0x80000CA80801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P0_4              , RULL(0x800010A80701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P0_4            , RULL(0x800010A80701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P0_4            , RULL(0x800010A80801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P1_0            , RULL(0x800000A80701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P1_0            , RULL(0x800000A80801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P1_1            , RULL(0x800004A80701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P1_1            , RULL(0x800004A80801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P1_2            , RULL(0x800008A80701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P1_2            , RULL(0x800008A80801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P1_3            , RULL(0x80000CA80701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P1_3            , RULL(0x80000CA80801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P1_4            , RULL(0x800010A80701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P1_4            , RULL(0x800010A80801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P2_0            , RULL(0x800000A80701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P2_0            , RULL(0x800000A80801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P2_1            , RULL(0x800004A80701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P2_1            , RULL(0x800004A80801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P2_2            , RULL(0x800008A80701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P2_2            , RULL(0x800008A80801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P2_3            , RULL(0x80000CA80701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P2_3            , RULL(0x80000CA80801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P2_4            , RULL(0x800010A80701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P2_4            , RULL(0x800010A80801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P3_0            , RULL(0x800000A807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P3_0            , RULL(0x800000A808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P3_1            , RULL(0x800004A807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P3_1            , RULL(0x800004A808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P3_2            , RULL(0x800008A807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P3_2            , RULL(0x800008A808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P3_3            , RULL(0x80000CA807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P3_3            , RULL(0x80000CA808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P3_4            , RULL(0x800010A807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER0_P3_4            , RULL(0x800010A808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P0_0              , RULL(0x800000A90701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P0_0            , RULL(0x800000A90701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P0_0            , RULL(0x800000A90801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P0_1              , RULL(0x800004A90701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P0_1            , RULL(0x800004A90701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P0_1            , RULL(0x800004A90801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P0_2              , RULL(0x800008A90701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P0_2            , RULL(0x800008A90701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P0_2            , RULL(0x800008A90801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P0_3              , RULL(0x80000CA90701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P0_3            , RULL(0x80000CA90701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P0_3            , RULL(0x80000CA90801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P0_4              , RULL(0x800010A90701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P0_4            , RULL(0x800010A90701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P0_4            , RULL(0x800010A90801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P1_0            , RULL(0x800000A90701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P1_0            , RULL(0x800000A90801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P1_1            , RULL(0x800004A90701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P1_1            , RULL(0x800004A90801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P1_2            , RULL(0x800008A90701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P1_2            , RULL(0x800008A90801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P1_3            , RULL(0x80000CA90701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P1_3            , RULL(0x80000CA90801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P1_4            , RULL(0x800010A90701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P1_4            , RULL(0x800010A90801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P2_0            , RULL(0x800000A90701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P2_0            , RULL(0x800000A90801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P2_1            , RULL(0x800004A90701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P2_1            , RULL(0x800004A90801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P2_2            , RULL(0x800008A90701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P2_2            , RULL(0x800008A90801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P2_3            , RULL(0x80000CA90701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P2_3            , RULL(0x80000CA90801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P2_4            , RULL(0x800010A90701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P2_4            , RULL(0x800010A90801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P3_0            , RULL(0x800000A907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P3_0            , RULL(0x800000A908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P3_1            , RULL(0x800004A907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P3_1            , RULL(0x800004A908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P3_2            , RULL(0x800008A907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P3_2            , RULL(0x800008A908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P3_3            , RULL(0x80000CA907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P3_3            , RULL(0x80000CA908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P3_4            , RULL(0x800010A907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_LOWER1_P3_4            , RULL(0x800010A908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P0_0              , RULL(0x800000AA0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P0_0            , RULL(0x800000AA0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P0_0            , RULL(0x800000AA0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P0_1              , RULL(0x800004AA0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P0_1            , RULL(0x800004AA0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P0_1            , RULL(0x800004AA0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P0_2              , RULL(0x800008AA0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P0_2            , RULL(0x800008AA0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P0_2            , RULL(0x800008AA0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P0_3              , RULL(0x80000CAA0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P0_3            , RULL(0x80000CAA0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P0_3            , RULL(0x80000CAA0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P0_4              , RULL(0x800010AA0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P0_4            , RULL(0x800010AA0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P0_4            , RULL(0x800010AA0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P1_0            , RULL(0x800000AA0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P1_0            , RULL(0x800000AA0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P1_1            , RULL(0x800004AA0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P1_1            , RULL(0x800004AA0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P1_2            , RULL(0x800008AA0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P1_2            , RULL(0x800008AA0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P1_3            , RULL(0x80000CAA0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P1_3            , RULL(0x80000CAA0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P1_4            , RULL(0x800010AA0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P1_4            , RULL(0x800010AA0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P2_0            , RULL(0x800000AA0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P2_0            , RULL(0x800000AA0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P2_1            , RULL(0x800004AA0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P2_1            , RULL(0x800004AA0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P2_2            , RULL(0x800008AA0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P2_2            , RULL(0x800008AA0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P2_3            , RULL(0x80000CAA0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P2_3            , RULL(0x80000CAA0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P2_4            , RULL(0x800010AA0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P2_4            , RULL(0x800010AA0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P3_0            , RULL(0x800000AA07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P3_0            , RULL(0x800000AA08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P3_1            , RULL(0x800004AA07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P3_1            , RULL(0x800004AA08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P3_2            , RULL(0x800008AA07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P3_2            , RULL(0x800008AA08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P3_3            , RULL(0x80000CAA07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P3_3            , RULL(0x80000CAA08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P3_4            , RULL(0x800010AA07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER0_P3_4            , RULL(0x800010AA08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P0_0              , RULL(0x800000AB0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P0_0            , RULL(0x800000AB0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P0_0            , RULL(0x800000AB0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P0_1              , RULL(0x800004AB0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P0_1            , RULL(0x800004AB0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P0_1            , RULL(0x800004AB0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P0_2              , RULL(0x800008AB0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P0_2            , RULL(0x800008AB0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P0_2            , RULL(0x800008AB0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P0_3              , RULL(0x80000CAB0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P0_3            , RULL(0x80000CAB0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P0_3            , RULL(0x80000CAB0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P0_4              , RULL(0x800010AB0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P0_4            , RULL(0x800010AB0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P0_4            , RULL(0x800010AB0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P1_0            , RULL(0x800000AB0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P1_0            , RULL(0x800000AB0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P1_1            , RULL(0x800004AB0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P1_1            , RULL(0x800004AB0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P1_2            , RULL(0x800008AB0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P1_2            , RULL(0x800008AB0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P1_3            , RULL(0x80000CAB0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P1_3            , RULL(0x80000CAB0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P1_4            , RULL(0x800010AB0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P1_4            , RULL(0x800010AB0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P2_0            , RULL(0x800000AB0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P2_0            , RULL(0x800000AB0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P2_1            , RULL(0x800004AB0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P2_1            , RULL(0x800004AB0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P2_2            , RULL(0x800008AB0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P2_2            , RULL(0x800008AB0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P2_3            , RULL(0x80000CAB0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P2_3            , RULL(0x80000CAB0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P2_4            , RULL(0x800010AB0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P2_4            , RULL(0x800010AB0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P3_0            , RULL(0x800000AB07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P3_0            , RULL(0x800000AB08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P3_1            , RULL(0x800004AB07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P3_1            , RULL(0x800004AB08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P3_2            , RULL(0x800008AB07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P3_2            , RULL(0x800008AB08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P3_3            , RULL(0x80000CAB07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P3_3            , RULL(0x80000CAB08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P3_4            , RULL(0x800010AB07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_SLAVE_VREG_UPPER1_P3_4            , RULL(0x800010AB08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_SW_CONTROL0_P0_0                    , RULL(0x800000A60701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_SW_CONTROL0_P0_0                  , RULL(0x800000A60701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_SW_CONTROL0_P0_0                  , RULL(0x800000A60801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_SW_CONTROL0_P0_1                    , RULL(0x800004A60701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_SW_CONTROL0_P0_1                  , RULL(0x800004A60701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_SW_CONTROL0_P0_1                  , RULL(0x800004A60801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_SW_CONTROL0_P0_2                    , RULL(0x800008A60701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_SW_CONTROL0_P0_2                  , RULL(0x800008A60701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_SW_CONTROL0_P0_2                  , RULL(0x800008A60801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_SW_CONTROL0_P0_3                    , RULL(0x80000CA60701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_SW_CONTROL0_P0_3                  , RULL(0x80000CA60701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_SW_CONTROL0_P0_3                  , RULL(0x80000CA60801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_SW_CONTROL0_P0_4                    , RULL(0x800010A60701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_SW_CONTROL0_P0_4                  , RULL(0x800010A60701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_SW_CONTROL0_P0_4                  , RULL(0x800010A60801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_SW_CONTROL0_P1_0                  , RULL(0x800000A60701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_SW_CONTROL0_P1_0                  , RULL(0x800000A60801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_SW_CONTROL0_P1_1                  , RULL(0x800004A60701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_SW_CONTROL0_P1_1                  , RULL(0x800004A60801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_SW_CONTROL0_P1_2                  , RULL(0x800008A60701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_SW_CONTROL0_P1_2                  , RULL(0x800008A60801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_SW_CONTROL0_P1_3                  , RULL(0x80000CA60701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_SW_CONTROL0_P1_3                  , RULL(0x80000CA60801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_SW_CONTROL0_P1_4                  , RULL(0x800010A60701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_SW_CONTROL0_P1_4                  , RULL(0x800010A60801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_SW_CONTROL0_P2_0                  , RULL(0x800000A60701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_SW_CONTROL0_P2_0                  , RULL(0x800000A60801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_SW_CONTROL0_P2_1                  , RULL(0x800004A60701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_SW_CONTROL0_P2_1                  , RULL(0x800004A60801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_SW_CONTROL0_P2_2                  , RULL(0x800008A60701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_SW_CONTROL0_P2_2                  , RULL(0x800008A60801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_SW_CONTROL0_P2_3                  , RULL(0x80000CA60701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_SW_CONTROL0_P2_3                  , RULL(0x80000CA60801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_SW_CONTROL0_P2_4                  , RULL(0x800010A60701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_SW_CONTROL0_P2_4                  , RULL(0x800010A60801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_SW_CONTROL0_P3_0                  , RULL(0x800000A607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_SW_CONTROL0_P3_0                  , RULL(0x800000A608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_SW_CONTROL0_P3_1                  , RULL(0x800004A607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_SW_CONTROL0_P3_1                  , RULL(0x800004A608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_SW_CONTROL0_P3_2                  , RULL(0x800008A607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_SW_CONTROL0_P3_2                  , RULL(0x800008A608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_SW_CONTROL0_P3_3                  , RULL(0x80000CA607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_SW_CONTROL0_P3_3                  , RULL(0x80000CA608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_SW_CONTROL0_P3_4                  , RULL(0x800010A607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_SW_CONTROL0_P3_4                  , RULL(0x800010A608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_SW_CONTROL1_P0_0                    , RULL(0x800000A70701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_SW_CONTROL1_P0_0                  , RULL(0x800000A70701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_SW_CONTROL1_P0_0                  , RULL(0x800000A70801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_SW_CONTROL1_P0_1                    , RULL(0x800004A70701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_SW_CONTROL1_P0_1                  , RULL(0x800004A70701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_SW_CONTROL1_P0_1                  , RULL(0x800004A70801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_SW_CONTROL1_P0_2                    , RULL(0x800008A70701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_SW_CONTROL1_P0_2                  , RULL(0x800008A70701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_SW_CONTROL1_P0_2                  , RULL(0x800008A70801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_SW_CONTROL1_P0_3                    , RULL(0x80000CA70701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_SW_CONTROL1_P0_3                  , RULL(0x80000CA70701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_SW_CONTROL1_P0_3                  , RULL(0x80000CA70801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_SW_CONTROL1_P0_4                    , RULL(0x800010A70701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_DLL_SW_CONTROL1_P0_4                  , RULL(0x800010A70701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_DLL_SW_CONTROL1_P0_4                  , RULL(0x800010A70801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_SW_CONTROL1_P1_0                  , RULL(0x800000A70701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_SW_CONTROL1_P1_0                  , RULL(0x800000A70801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_SW_CONTROL1_P1_1                  , RULL(0x800004A70701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_SW_CONTROL1_P1_1                  , RULL(0x800004A70801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_SW_CONTROL1_P1_2                  , RULL(0x800008A70701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_SW_CONTROL1_P1_2                  , RULL(0x800008A70801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_SW_CONTROL1_P1_3                  , RULL(0x80000CA70701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_SW_CONTROL1_P1_3                  , RULL(0x80000CA70801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_DLL_SW_CONTROL1_P1_4                  , RULL(0x800010A70701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_DLL_SW_CONTROL1_P1_4                  , RULL(0x800010A70801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_SW_CONTROL1_P2_0                  , RULL(0x800000A70701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_SW_CONTROL1_P2_0                  , RULL(0x800000A70801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_SW_CONTROL1_P2_1                  , RULL(0x800004A70701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_SW_CONTROL1_P2_1                  , RULL(0x800004A70801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_SW_CONTROL1_P2_2                  , RULL(0x800008A70701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_SW_CONTROL1_P2_2                  , RULL(0x800008A70801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_SW_CONTROL1_P2_3                  , RULL(0x80000CA70701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_SW_CONTROL1_P2_3                  , RULL(0x80000CA70801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_DLL_SW_CONTROL1_P2_4                  , RULL(0x800010A70701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_DLL_SW_CONTROL1_P2_4                  , RULL(0x800010A70801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_SW_CONTROL1_P3_0                  , RULL(0x800000A707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_SW_CONTROL1_P3_0                  , RULL(0x800000A708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_SW_CONTROL1_P3_1                  , RULL(0x800004A707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_SW_CONTROL1_P3_1                  , RULL(0x800004A708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_SW_CONTROL1_P3_2                  , RULL(0x800008A707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_SW_CONTROL1_P3_2                  , RULL(0x800008A708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_SW_CONTROL1_P3_3                  , RULL(0x80000CA707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_SW_CONTROL1_P3_3                  , RULL(0x80000CA708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_DLL_SW_CONTROL1_P3_4                  , RULL(0x800010A707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_DLL_SW_CONTROL1_P3_4                  , RULL(0x800010A708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_DLL_VREG_COARSE0_P0_0                   , RULL(0x8000002C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_COARSE0_P0_0                 , RULL(0x8000002C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_COARSE0_P0_0                 , RULL(0x8000002C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_COARSE0_P0_1                   , RULL(0x8000042C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_COARSE0_P0_1                 , RULL(0x8000042C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_COARSE0_P0_1                 , RULL(0x8000042C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_COARSE0_P0_2                   , RULL(0x8000082C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_COARSE0_P0_2                 , RULL(0x8000082C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_COARSE0_P0_2                 , RULL(0x8000082C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_COARSE0_P0_3                   , RULL(0x80000C2C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_COARSE0_P0_3                 , RULL(0x80000C2C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_COARSE0_P0_3                 , RULL(0x80000C2C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_COARSE0_P0_4                   , RULL(0x8000102C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_COARSE0_P0_4                 , RULL(0x8000102C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_COARSE0_P0_4                 , RULL(0x8000102C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_COARSE0_P1_0                 , RULL(0x8000002C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_COARSE0_P1_0                 , RULL(0x8000002C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_COARSE0_P1_1                 , RULL(0x8000042C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_COARSE0_P1_1                 , RULL(0x8000042C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_COARSE0_P1_2                 , RULL(0x8000082C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_COARSE0_P1_2                 , RULL(0x8000082C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_COARSE0_P1_3                 , RULL(0x80000C2C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_COARSE0_P1_3                 , RULL(0x80000C2C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_COARSE0_P1_4                 , RULL(0x8000102C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_COARSE0_P1_4                 , RULL(0x8000102C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_COARSE0_P2_0                 , RULL(0x8000002C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_COARSE0_P2_0                 , RULL(0x8000002C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_COARSE0_P2_1                 , RULL(0x8000042C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_COARSE0_P2_1                 , RULL(0x8000042C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_COARSE0_P2_2                 , RULL(0x8000082C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_COARSE0_P2_2                 , RULL(0x8000082C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_COARSE0_P2_3                 , RULL(0x80000C2C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_COARSE0_P2_3                 , RULL(0x80000C2C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_COARSE0_P2_4                 , RULL(0x8000102C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_COARSE0_P2_4                 , RULL(0x8000102C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_COARSE0_P3_0                 , RULL(0x8000002C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_COARSE0_P3_0                 , RULL(0x8000002C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_COARSE0_P3_1                 , RULL(0x8000042C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_COARSE0_P3_1                 , RULL(0x8000042C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_COARSE0_P3_2                 , RULL(0x8000082C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_COARSE0_P3_2                 , RULL(0x8000082C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_COARSE0_P3_3                 , RULL(0x80000C2C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_COARSE0_P3_3                 , RULL(0x80000C2C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_COARSE0_P3_4                 , RULL(0x8000102C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_COARSE0_P3_4                 , RULL(0x8000102C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_COARSE1_P0_0                   , RULL(0x8000002D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_COARSE1_P0_0                 , RULL(0x8000002D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_COARSE1_P0_0                 , RULL(0x8000002D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_COARSE1_P0_1                   , RULL(0x8000042D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_COARSE1_P0_1                 , RULL(0x8000042D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_COARSE1_P0_1                 , RULL(0x8000042D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_COARSE1_P0_2                   , RULL(0x8000082D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_COARSE1_P0_2                 , RULL(0x8000082D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_COARSE1_P0_2                 , RULL(0x8000082D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_COARSE1_P0_3                   , RULL(0x80000C2D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_COARSE1_P0_3                 , RULL(0x80000C2D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_COARSE1_P0_3                 , RULL(0x80000C2D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_COARSE1_P0_4                   , RULL(0x8000102D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_COARSE1_P0_4                 , RULL(0x8000102D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_COARSE1_P0_4                 , RULL(0x8000102D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_COARSE1_P1_0                 , RULL(0x8000002D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_COARSE1_P1_0                 , RULL(0x8000002D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_COARSE1_P1_1                 , RULL(0x8000042D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_COARSE1_P1_1                 , RULL(0x8000042D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_COARSE1_P1_2                 , RULL(0x8000082D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_COARSE1_P1_2                 , RULL(0x8000082D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_COARSE1_P1_3                 , RULL(0x80000C2D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_COARSE1_P1_3                 , RULL(0x80000C2D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_COARSE1_P1_4                 , RULL(0x8000102D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_COARSE1_P1_4                 , RULL(0x8000102D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_COARSE1_P2_0                 , RULL(0x8000002D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_COARSE1_P2_0                 , RULL(0x8000002D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_COARSE1_P2_1                 , RULL(0x8000042D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_COARSE1_P2_1                 , RULL(0x8000042D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_COARSE1_P2_2                 , RULL(0x8000082D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_COARSE1_P2_2                 , RULL(0x8000082D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_COARSE1_P2_3                 , RULL(0x80000C2D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_COARSE1_P2_3                 , RULL(0x80000C2D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_COARSE1_P2_4                 , RULL(0x8000102D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_COARSE1_P2_4                 , RULL(0x8000102D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_COARSE1_P3_0                 , RULL(0x8000002D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_COARSE1_P3_0                 , RULL(0x8000002D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_COARSE1_P3_1                 , RULL(0x8000042D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_COARSE1_P3_1                 , RULL(0x8000042D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_COARSE1_P3_2                 , RULL(0x8000082D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_COARSE1_P3_2                 , RULL(0x8000082D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_COARSE1_P3_3                 , RULL(0x80000C2D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_COARSE1_P3_3                 , RULL(0x80000C2D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_COARSE1_P3_4                 , RULL(0x8000102D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_COARSE1_P3_4                 , RULL(0x8000102D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_CONTROL0_P0_0                  , RULL(0x8000002A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_CONTROL0_P0_0                , RULL(0x8000002A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_CONTROL0_P0_0                , RULL(0x8000002A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_CONTROL0_P0_1                  , RULL(0x8000042A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_CONTROL0_P0_1                , RULL(0x8000042A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_CONTROL0_P0_1                , RULL(0x8000042A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_CONTROL0_P0_2                  , RULL(0x8000082A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_CONTROL0_P0_2                , RULL(0x8000082A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_CONTROL0_P0_2                , RULL(0x8000082A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_CONTROL0_P0_3                  , RULL(0x80000C2A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_CONTROL0_P0_3                , RULL(0x80000C2A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_CONTROL0_P0_3                , RULL(0x80000C2A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_CONTROL0_P0_4                  , RULL(0x8000102A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_CONTROL0_P0_4                , RULL(0x8000102A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_CONTROL0_P0_4                , RULL(0x8000102A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_CONTROL0_P1_0                , RULL(0x8000002A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_CONTROL0_P1_0                , RULL(0x8000002A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_CONTROL0_P1_1                , RULL(0x8000042A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_CONTROL0_P1_1                , RULL(0x8000042A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_CONTROL0_P1_2                , RULL(0x8000082A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_CONTROL0_P1_2                , RULL(0x8000082A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_CONTROL0_P1_3                , RULL(0x80000C2A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_CONTROL0_P1_3                , RULL(0x80000C2A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_CONTROL0_P1_4                , RULL(0x8000102A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_CONTROL0_P1_4                , RULL(0x8000102A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_CONTROL0_P2_0                , RULL(0x8000002A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_CONTROL0_P2_0                , RULL(0x8000002A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_CONTROL0_P2_1                , RULL(0x8000042A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_CONTROL0_P2_1                , RULL(0x8000042A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_CONTROL0_P2_2                , RULL(0x8000082A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_CONTROL0_P2_2                , RULL(0x8000082A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_CONTROL0_P2_3                , RULL(0x80000C2A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_CONTROL0_P2_3                , RULL(0x80000C2A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_CONTROL0_P2_4                , RULL(0x8000102A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_CONTROL0_P2_4                , RULL(0x8000102A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_CONTROL0_P3_0                , RULL(0x8000002A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_CONTROL0_P3_0                , RULL(0x8000002A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_CONTROL0_P3_1                , RULL(0x8000042A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_CONTROL0_P3_1                , RULL(0x8000042A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_CONTROL0_P3_2                , RULL(0x8000082A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_CONTROL0_P3_2                , RULL(0x8000082A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_CONTROL0_P3_3                , RULL(0x80000C2A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_CONTROL0_P3_3                , RULL(0x80000C2A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_CONTROL0_P3_4                , RULL(0x8000102A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_CONTROL0_P3_4                , RULL(0x8000102A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_CONTROL1_P0_0                  , RULL(0x8000002B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_CONTROL1_P0_0                , RULL(0x8000002B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_CONTROL1_P0_0                , RULL(0x8000002B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_CONTROL1_P0_1                  , RULL(0x8000042B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_CONTROL1_P0_1                , RULL(0x8000042B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_CONTROL1_P0_1                , RULL(0x8000042B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_CONTROL1_P0_2                  , RULL(0x8000082B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_CONTROL1_P0_2                , RULL(0x8000082B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_CONTROL1_P0_2                , RULL(0x8000082B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_CONTROL1_P0_3                  , RULL(0x80000C2B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_CONTROL1_P0_3                , RULL(0x80000C2B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_CONTROL1_P0_3                , RULL(0x80000C2B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DLL_VREG_CONTROL1_P0_4                  , RULL(0x8000102B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DLL_VREG_CONTROL1_P0_4                , RULL(0x8000102B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DLL_VREG_CONTROL1_P0_4                , RULL(0x8000102B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_CONTROL1_P1_0                , RULL(0x8000002B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_CONTROL1_P1_0                , RULL(0x8000002B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_CONTROL1_P1_1                , RULL(0x8000042B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_CONTROL1_P1_1                , RULL(0x8000042B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_CONTROL1_P1_2                , RULL(0x8000082B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_CONTROL1_P1_2                , RULL(0x8000082B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_CONTROL1_P1_3                , RULL(0x80000C2B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_CONTROL1_P1_3                , RULL(0x80000C2B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DLL_VREG_CONTROL1_P1_4                , RULL(0x8000102B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DLL_VREG_CONTROL1_P1_4                , RULL(0x8000102B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_CONTROL1_P2_0                , RULL(0x8000002B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_CONTROL1_P2_0                , RULL(0x8000002B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_CONTROL1_P2_1                , RULL(0x8000042B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_CONTROL1_P2_1                , RULL(0x8000042B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_CONTROL1_P2_2                , RULL(0x8000082B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_CONTROL1_P2_2                , RULL(0x8000082B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_CONTROL1_P2_3                , RULL(0x80000C2B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_CONTROL1_P2_3                , RULL(0x80000C2B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DLL_VREG_CONTROL1_P2_4                , RULL(0x8000102B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DLL_VREG_CONTROL1_P2_4                , RULL(0x8000102B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_CONTROL1_P3_0                , RULL(0x8000002B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_CONTROL1_P3_0                , RULL(0x8000002B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_CONTROL1_P3_1                , RULL(0x8000042B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_CONTROL1_P3_1                , RULL(0x8000042B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_CONTROL1_P3_2                , RULL(0x8000082B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_CONTROL1_P3_2                , RULL(0x8000082B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_CONTROL1_P3_3                , RULL(0x80000C2B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_CONTROL1_P3_3                , RULL(0x80000C2B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DLL_VREG_CONTROL1_P3_4                , RULL(0x8000102B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DLL_VREG_CONTROL1_P3_4                , RULL(0x8000102B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_OFFSET_P0_0                      , RULL(0x800000370701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_OFFSET_P0_0                    , RULL(0x800000370701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_OFFSET_P0_0                    , RULL(0x800000370801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_OFFSET_P0_1                      , RULL(0x800004370701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_OFFSET_P0_1                    , RULL(0x800004370701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_OFFSET_P0_1                    , RULL(0x800004370801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_OFFSET_P0_2                      , RULL(0x800008370701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_OFFSET_P0_2                    , RULL(0x800008370701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_OFFSET_P0_2                    , RULL(0x800008370801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_OFFSET_P0_3                      , RULL(0x80000C370701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_OFFSET_P0_3                    , RULL(0x80000C370701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_OFFSET_P0_3                    , RULL(0x80000C370801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_OFFSET_P0_4                      , RULL(0x800010370701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_OFFSET_P0_4                    , RULL(0x800010370701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_OFFSET_P0_4                    , RULL(0x800010370801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_OFFSET_P1_0                    , RULL(0x800000370701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_OFFSET_P1_0                    , RULL(0x800000370801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_OFFSET_P1_1                    , RULL(0x800004370701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_OFFSET_P1_1                    , RULL(0x800004370801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_OFFSET_P1_2                    , RULL(0x800008370701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_OFFSET_P1_2                    , RULL(0x800008370801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_OFFSET_P1_3                    , RULL(0x80000C370701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_OFFSET_P1_3                    , RULL(0x80000C370801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_OFFSET_P1_4                    , RULL(0x800010370701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_OFFSET_P1_4                    , RULL(0x800010370801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_OFFSET_P2_0                    , RULL(0x800000370701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_OFFSET_P2_0                    , RULL(0x800000370801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_OFFSET_P2_1                    , RULL(0x800004370701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_OFFSET_P2_1                    , RULL(0x800004370801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_OFFSET_P2_2                    , RULL(0x800008370701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_OFFSET_P2_2                    , RULL(0x800008370801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_OFFSET_P2_3                    , RULL(0x80000C370701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_OFFSET_P2_3                    , RULL(0x80000C370801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_OFFSET_P2_4                    , RULL(0x800010370701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_OFFSET_P2_4                    , RULL(0x800010370801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_OFFSET_P3_0                    , RULL(0x8000003707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_OFFSET_P3_0                    , RULL(0x8000003708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_OFFSET_P3_1                    , RULL(0x8000043707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_OFFSET_P3_1                    , RULL(0x8000043708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_OFFSET_P3_2                    , RULL(0x8000083707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_OFFSET_P3_2                    , RULL(0x8000083708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_OFFSET_P3_3                    , RULL(0x80000C3707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_OFFSET_P3_3                    , RULL(0x80000C3708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_OFFSET_P3_4                    , RULL(0x8000103707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_OFFSET_P3_4                    , RULL(0x8000103708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P0_0              , RULL(0x800000300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P0_0            , RULL(0x800000300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P0_0            , RULL(0x800000300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P0_1              , RULL(0x800004300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P0_1            , RULL(0x800004300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P0_1            , RULL(0x800004300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P0_2              , RULL(0x800008300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P0_2            , RULL(0x800008300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P0_2            , RULL(0x800008300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P0_3              , RULL(0x80000C300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P0_3            , RULL(0x80000C300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P0_3            , RULL(0x80000C300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P0_4              , RULL(0x800010300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P0_4            , RULL(0x800010300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P0_4            , RULL(0x800010300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P1_0            , RULL(0x800000300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P1_0            , RULL(0x800000300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P1_1            , RULL(0x800004300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P1_1            , RULL(0x800004300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P1_2            , RULL(0x800008300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P1_2            , RULL(0x800008300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P1_3            , RULL(0x80000C300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P1_3            , RULL(0x80000C300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P1_4            , RULL(0x800010300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P1_4            , RULL(0x800010300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P2_0            , RULL(0x800000300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P2_0            , RULL(0x800000300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P2_1            , RULL(0x800004300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P2_1            , RULL(0x800004300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P2_2            , RULL(0x800008300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P2_2            , RULL(0x800008300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P2_3            , RULL(0x80000C300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P2_3            , RULL(0x80000C300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P2_4            , RULL(0x800010300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P2_4            , RULL(0x800010300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P3_0            , RULL(0x8000003007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P3_0            , RULL(0x8000003008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P3_1            , RULL(0x8000043007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P3_1            , RULL(0x8000043008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P3_2            , RULL(0x8000083007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P3_2            , RULL(0x8000083008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P3_3            , RULL(0x80000C3007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P3_3            , RULL(0x80000C3008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P3_4            , RULL(0x8000103007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR0_P3_4            , RULL(0x8000103008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P0_0              , RULL(0x800001300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P0_0            , RULL(0x800001300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P0_0            , RULL(0x800001300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P0_1              , RULL(0x800005300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P0_1            , RULL(0x800005300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P0_1            , RULL(0x800005300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P0_2              , RULL(0x800009300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P0_2            , RULL(0x800009300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P0_2            , RULL(0x800009300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P0_3              , RULL(0x80000D300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P0_3            , RULL(0x80000D300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P0_3            , RULL(0x80000D300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P0_4              , RULL(0x800011300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P0_4            , RULL(0x800011300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P0_4            , RULL(0x800011300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P1_0            , RULL(0x800001300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P1_0            , RULL(0x800001300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P1_1            , RULL(0x800005300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P1_1            , RULL(0x800005300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P1_2            , RULL(0x800009300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P1_2            , RULL(0x800009300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P1_3            , RULL(0x80000D300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P1_3            , RULL(0x80000D300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P1_4            , RULL(0x800011300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P1_4            , RULL(0x800011300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P2_0            , RULL(0x800001300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P2_0            , RULL(0x800001300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P2_1            , RULL(0x800005300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P2_1            , RULL(0x800005300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P2_2            , RULL(0x800009300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P2_2            , RULL(0x800009300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P2_3            , RULL(0x80000D300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P2_3            , RULL(0x80000D300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P2_4            , RULL(0x800011300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P2_4            , RULL(0x800011300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P3_0            , RULL(0x8000013007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P3_0            , RULL(0x8000013008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P3_1            , RULL(0x8000053007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P3_1            , RULL(0x8000053008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P3_2            , RULL(0x8000093007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P3_2            , RULL(0x8000093008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P3_3            , RULL(0x80000D3007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P3_3            , RULL(0x80000D3008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P3_4            , RULL(0x8000113007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR1_P3_4            , RULL(0x8000113008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P0_0              , RULL(0x800002300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P0_0            , RULL(0x800002300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P0_0            , RULL(0x800002300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P0_1              , RULL(0x800006300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P0_1            , RULL(0x800006300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P0_1            , RULL(0x800006300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P0_2              , RULL(0x80000A300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P0_2            , RULL(0x80000A300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P0_2            , RULL(0x80000A300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P0_3              , RULL(0x80000E300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P0_3            , RULL(0x80000E300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P0_3            , RULL(0x80000E300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P0_4              , RULL(0x800012300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P0_4            , RULL(0x800012300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P0_4            , RULL(0x800012300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P1_0            , RULL(0x800002300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P1_0            , RULL(0x800002300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P1_1            , RULL(0x800006300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P1_1            , RULL(0x800006300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P1_2            , RULL(0x80000A300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P1_2            , RULL(0x80000A300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P1_3            , RULL(0x80000E300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P1_3            , RULL(0x80000E300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P1_4            , RULL(0x800012300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P1_4            , RULL(0x800012300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P2_0            , RULL(0x800002300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P2_0            , RULL(0x800002300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P2_1            , RULL(0x800006300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P2_1            , RULL(0x800006300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P2_2            , RULL(0x80000A300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P2_2            , RULL(0x80000A300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P2_3            , RULL(0x80000E300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P2_3            , RULL(0x80000E300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P2_4            , RULL(0x800012300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P2_4            , RULL(0x800012300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P3_0            , RULL(0x8000023007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P3_0            , RULL(0x8000023008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P3_1            , RULL(0x8000063007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P3_1            , RULL(0x8000063008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P3_2            , RULL(0x80000A3007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P3_2            , RULL(0x80000A3008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P3_3            , RULL(0x80000E3007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P3_3            , RULL(0x80000E3008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P3_4            , RULL(0x8000123007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR2_P3_4            , RULL(0x8000123008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P0_0              , RULL(0x800003300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P0_0            , RULL(0x800003300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P0_0            , RULL(0x800003300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P0_1              , RULL(0x800007300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P0_1            , RULL(0x800007300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P0_1            , RULL(0x800007300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P0_2              , RULL(0x80000B300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P0_2            , RULL(0x80000B300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P0_2            , RULL(0x80000B300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P0_3              , RULL(0x80000F300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P0_3            , RULL(0x80000F300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P0_3            , RULL(0x80000F300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P0_4              , RULL(0x800013300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P0_4            , RULL(0x800013300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P0_4            , RULL(0x800013300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P1_0            , RULL(0x800003300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P1_0            , RULL(0x800003300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P1_1            , RULL(0x800007300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P1_1            , RULL(0x800007300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P1_2            , RULL(0x80000B300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P1_2            , RULL(0x80000B300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P1_3            , RULL(0x80000F300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P1_3            , RULL(0x80000F300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P1_4            , RULL(0x800013300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P1_4            , RULL(0x800013300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P2_0            , RULL(0x800003300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P2_0            , RULL(0x800003300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P2_1            , RULL(0x800007300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P2_1            , RULL(0x800007300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P2_2            , RULL(0x80000B300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P2_2            , RULL(0x80000B300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P2_3            , RULL(0x80000F300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P2_3            , RULL(0x80000F300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P2_4            , RULL(0x800013300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P2_4            , RULL(0x800013300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P3_0            , RULL(0x8000033007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P3_0            , RULL(0x8000033008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P3_1            , RULL(0x8000073007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P3_1            , RULL(0x8000073008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P3_2            , RULL(0x80000B3007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P3_2            , RULL(0x80000B3008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P3_3            , RULL(0x80000F3007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P3_3            , RULL(0x80000F3008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P3_4            , RULL(0x8000133007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR0_RANK_PAIR3_P3_4            , RULL(0x8000133008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P0_0              , RULL(0x800000310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P0_0            , RULL(0x800000310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P0_0            , RULL(0x800000310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P0_1              , RULL(0x800004310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P0_1            , RULL(0x800004310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P0_1            , RULL(0x800004310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P0_2              , RULL(0x800008310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P0_2            , RULL(0x800008310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P0_2            , RULL(0x800008310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P0_3              , RULL(0x80000C310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P0_3            , RULL(0x80000C310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P0_3            , RULL(0x80000C310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P0_4              , RULL(0x800010310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P0_4            , RULL(0x800010310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P0_4            , RULL(0x800010310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P1_0            , RULL(0x800000310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P1_0            , RULL(0x800000310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P1_1            , RULL(0x800004310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P1_1            , RULL(0x800004310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P1_2            , RULL(0x800008310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P1_2            , RULL(0x800008310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P1_3            , RULL(0x80000C310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P1_3            , RULL(0x80000C310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P1_4            , RULL(0x800010310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P1_4            , RULL(0x800010310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P2_0            , RULL(0x800000310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P2_0            , RULL(0x800000310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P2_1            , RULL(0x800004310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P2_1            , RULL(0x800004310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P2_2            , RULL(0x800008310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P2_2            , RULL(0x800008310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P2_3            , RULL(0x80000C310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P2_3            , RULL(0x80000C310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P2_4            , RULL(0x800010310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P2_4            , RULL(0x800010310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P3_0            , RULL(0x8000003107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P3_0            , RULL(0x8000003108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P3_1            , RULL(0x8000043107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P3_1            , RULL(0x8000043108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P3_2            , RULL(0x8000083107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P3_2            , RULL(0x8000083108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P3_3            , RULL(0x80000C3107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P3_3            , RULL(0x80000C3108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P3_4            , RULL(0x8000103107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR0_P3_4            , RULL(0x8000103108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P0_0              , RULL(0x800001310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P0_0            , RULL(0x800001310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P0_0            , RULL(0x800001310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P0_1              , RULL(0x800005310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P0_1            , RULL(0x800005310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P0_1            , RULL(0x800005310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P0_2              , RULL(0x800009310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P0_2            , RULL(0x800009310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P0_2            , RULL(0x800009310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P0_3              , RULL(0x80000D310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P0_3            , RULL(0x80000D310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P0_3            , RULL(0x80000D310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P0_4              , RULL(0x800011310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P0_4            , RULL(0x800011310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P0_4            , RULL(0x800011310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P1_0            , RULL(0x800001310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P1_0            , RULL(0x800001310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P1_1            , RULL(0x800005310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P1_1            , RULL(0x800005310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P1_2            , RULL(0x800009310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P1_2            , RULL(0x800009310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P1_3            , RULL(0x80000D310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P1_3            , RULL(0x80000D310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P1_4            , RULL(0x800011310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P1_4            , RULL(0x800011310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P2_0            , RULL(0x800001310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P2_0            , RULL(0x800001310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P2_1            , RULL(0x800005310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P2_1            , RULL(0x800005310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P2_2            , RULL(0x800009310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P2_2            , RULL(0x800009310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P2_3            , RULL(0x80000D310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P2_3            , RULL(0x80000D310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P2_4            , RULL(0x800011310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P2_4            , RULL(0x800011310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P3_0            , RULL(0x8000013107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P3_0            , RULL(0x8000013108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P3_1            , RULL(0x8000053107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P3_1            , RULL(0x8000053108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P3_2            , RULL(0x8000093107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P3_2            , RULL(0x8000093108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P3_3            , RULL(0x80000D3107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P3_3            , RULL(0x80000D3108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P3_4            , RULL(0x8000113107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR1_P3_4            , RULL(0x8000113108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P0_0              , RULL(0x800002310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P0_0            , RULL(0x800002310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P0_0            , RULL(0x800002310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P0_1              , RULL(0x800006310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P0_1            , RULL(0x800006310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P0_1            , RULL(0x800006310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P0_2              , RULL(0x80000A310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P0_2            , RULL(0x80000A310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P0_2            , RULL(0x80000A310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P0_3              , RULL(0x80000E310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P0_3            , RULL(0x80000E310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P0_3            , RULL(0x80000E310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P0_4              , RULL(0x800012310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P0_4            , RULL(0x800012310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P0_4            , RULL(0x800012310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P1_0            , RULL(0x800002310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P1_0            , RULL(0x800002310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P1_1            , RULL(0x800006310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P1_1            , RULL(0x800006310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P1_2            , RULL(0x80000A310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P1_2            , RULL(0x80000A310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P1_3            , RULL(0x80000E310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P1_3            , RULL(0x80000E310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P1_4            , RULL(0x800012310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P1_4            , RULL(0x800012310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P2_0            , RULL(0x800002310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P2_0            , RULL(0x800002310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P2_1            , RULL(0x800006310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P2_1            , RULL(0x800006310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P2_2            , RULL(0x80000A310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P2_2            , RULL(0x80000A310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P2_3            , RULL(0x80000E310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P2_3            , RULL(0x80000E310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P2_4            , RULL(0x800012310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P2_4            , RULL(0x800012310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P3_0            , RULL(0x8000023107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P3_0            , RULL(0x8000023108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P3_1            , RULL(0x8000063107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P3_1            , RULL(0x8000063108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P3_2            , RULL(0x80000A3107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P3_2            , RULL(0x80000A3108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P3_3            , RULL(0x80000E3107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P3_3            , RULL(0x80000E3108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P3_4            , RULL(0x8000123107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR2_P3_4            , RULL(0x8000123108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P0_0              , RULL(0x800003310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P0_0            , RULL(0x800003310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P0_0            , RULL(0x800003310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P0_1              , RULL(0x800007310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P0_1            , RULL(0x800007310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P0_1            , RULL(0x800007310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P0_2              , RULL(0x80000B310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P0_2            , RULL(0x80000B310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P0_2            , RULL(0x80000B310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P0_3              , RULL(0x80000F310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P0_3            , RULL(0x80000F310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P0_3            , RULL(0x80000F310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P0_4              , RULL(0x800013310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P0_4            , RULL(0x800013310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P0_4            , RULL(0x800013310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P1_0            , RULL(0x800003310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P1_0            , RULL(0x800003310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P1_1            , RULL(0x800007310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P1_1            , RULL(0x800007310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P1_2            , RULL(0x80000B310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P1_2            , RULL(0x80000B310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P1_3            , RULL(0x80000F310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P1_3            , RULL(0x80000F310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P1_4            , RULL(0x800013310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P1_4            , RULL(0x800013310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P2_0            , RULL(0x800003310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P2_0            , RULL(0x800003310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P2_1            , RULL(0x800007310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P2_1            , RULL(0x800007310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P2_2            , RULL(0x80000B310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P2_2            , RULL(0x80000B310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P2_3            , RULL(0x80000F310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P2_3            , RULL(0x80000F310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P2_4            , RULL(0x800013310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P2_4            , RULL(0x800013310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P3_0            , RULL(0x8000033107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P3_0            , RULL(0x8000033108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P3_1            , RULL(0x8000073107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P3_1            , RULL(0x8000073108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P3_2            , RULL(0x80000B3107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P3_2            , RULL(0x80000B3108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P3_3            , RULL(0x80000F3107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P3_3            , RULL(0x80000F3108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P3_4            , RULL(0x8000133107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQSCLK_PR1_RANK_PAIR3_P3_4            , RULL(0x8000133108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P0_0                 , RULL(0x800000130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P0_0               , RULL(0x800000130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P0_0               , RULL(0x800000130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P0_1                 , RULL(0x800004130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P0_1               , RULL(0x800004130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P0_1               , RULL(0x800004130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P0_2                 , RULL(0x800008130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P0_2               , RULL(0x800008130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P0_2               , RULL(0x800008130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P0_3                 , RULL(0x80000C130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P0_3               , RULL(0x80000C130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P0_3               , RULL(0x80000C130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P0_4                 , RULL(0x800010130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P0_4               , RULL(0x800010130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P0_4               , RULL(0x800010130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P1_0               , RULL(0x800000130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P1_0               , RULL(0x800000130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P1_1               , RULL(0x800004130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P1_1               , RULL(0x800004130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P1_2               , RULL(0x800008130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P1_2               , RULL(0x800008130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P1_3               , RULL(0x80000C130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P1_3               , RULL(0x80000C130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P1_4               , RULL(0x800010130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P1_4               , RULL(0x800010130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P2_0               , RULL(0x800000130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P2_0               , RULL(0x800000130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P2_1               , RULL(0x800004130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P2_1               , RULL(0x800004130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P2_2               , RULL(0x800008130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P2_2               , RULL(0x800008130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P2_3               , RULL(0x80000C130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P2_3               , RULL(0x80000C130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P2_4               , RULL(0x800010130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P2_4               , RULL(0x800010130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P3_0               , RULL(0x8000001307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P3_0               , RULL(0x8000001308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P3_1               , RULL(0x8000041307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P3_1               , RULL(0x8000041308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P3_2               , RULL(0x8000081307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P3_2               , RULL(0x8000081308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P3_3               , RULL(0x80000C1307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P3_3               , RULL(0x80000C1308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P3_4               , RULL(0x8000101307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP0_P3_4               , RULL(0x8000101308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P0_0                 , RULL(0x800001130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P0_0               , RULL(0x800001130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P0_0               , RULL(0x800001130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P0_1                 , RULL(0x800005130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P0_1               , RULL(0x800005130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P0_1               , RULL(0x800005130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P0_2                 , RULL(0x800009130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P0_2               , RULL(0x800009130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P0_2               , RULL(0x800009130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P0_3                 , RULL(0x80000D130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P0_3               , RULL(0x80000D130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P0_3               , RULL(0x80000D130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P0_4                 , RULL(0x800011130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P0_4               , RULL(0x800011130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P0_4               , RULL(0x800011130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P1_0               , RULL(0x800001130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P1_0               , RULL(0x800001130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P1_1               , RULL(0x800005130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P1_1               , RULL(0x800005130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P1_2               , RULL(0x800009130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P1_2               , RULL(0x800009130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P1_3               , RULL(0x80000D130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P1_3               , RULL(0x80000D130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P1_4               , RULL(0x800011130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P1_4               , RULL(0x800011130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P2_0               , RULL(0x800001130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P2_0               , RULL(0x800001130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P2_1               , RULL(0x800005130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P2_1               , RULL(0x800005130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P2_2               , RULL(0x800009130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P2_2               , RULL(0x800009130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P2_3               , RULL(0x80000D130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P2_3               , RULL(0x80000D130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P2_4               , RULL(0x800011130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P2_4               , RULL(0x800011130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P3_0               , RULL(0x8000011307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P3_0               , RULL(0x8000011308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P3_1               , RULL(0x8000051307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P3_1               , RULL(0x8000051308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P3_2               , RULL(0x8000091307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P3_2               , RULL(0x8000091308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P3_3               , RULL(0x80000D1307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P3_3               , RULL(0x80000D1308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P3_4               , RULL(0x8000111307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP1_P3_4               , RULL(0x8000111308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P0_0                 , RULL(0x800002130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P0_0               , RULL(0x800002130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P0_0               , RULL(0x800002130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P0_1                 , RULL(0x800006130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P0_1               , RULL(0x800006130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P0_1               , RULL(0x800006130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P0_2                 , RULL(0x80000A130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P0_2               , RULL(0x80000A130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P0_2               , RULL(0x80000A130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P0_3                 , RULL(0x80000E130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P0_3               , RULL(0x80000E130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P0_3               , RULL(0x80000E130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P0_4                 , RULL(0x800012130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P0_4               , RULL(0x800012130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P0_4               , RULL(0x800012130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P1_0               , RULL(0x800002130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P1_0               , RULL(0x800002130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P1_1               , RULL(0x800006130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P1_1               , RULL(0x800006130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P1_2               , RULL(0x80000A130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P1_2               , RULL(0x80000A130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P1_3               , RULL(0x80000E130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P1_3               , RULL(0x80000E130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P1_4               , RULL(0x800012130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P1_4               , RULL(0x800012130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P2_0               , RULL(0x800002130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P2_0               , RULL(0x800002130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P2_1               , RULL(0x800006130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P2_1               , RULL(0x800006130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P2_2               , RULL(0x80000A130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P2_2               , RULL(0x80000A130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P2_3               , RULL(0x80000E130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P2_3               , RULL(0x80000E130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P2_4               , RULL(0x800012130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P2_4               , RULL(0x800012130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P3_0               , RULL(0x8000021307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P3_0               , RULL(0x8000021308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P3_1               , RULL(0x8000061307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P3_1               , RULL(0x8000061308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P3_2               , RULL(0x80000A1307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P3_2               , RULL(0x80000A1308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P3_3               , RULL(0x80000E1307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P3_3               , RULL(0x80000E1308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P3_4               , RULL(0x8000121307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP2_P3_4               , RULL(0x8000121308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P0_0                 , RULL(0x800003130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P0_0               , RULL(0x800003130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P0_0               , RULL(0x800003130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P0_1                 , RULL(0x800007130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P0_1               , RULL(0x800007130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P0_1               , RULL(0x800007130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P0_2                 , RULL(0x80000B130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P0_2               , RULL(0x80000B130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P0_2               , RULL(0x80000B130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P0_3                 , RULL(0x80000F130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P0_3               , RULL(0x80000F130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P0_3               , RULL(0x80000F130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P0_4                 , RULL(0x800013130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P0_4               , RULL(0x800013130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P0_4               , RULL(0x800013130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P1_0               , RULL(0x800003130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P1_0               , RULL(0x800003130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P1_1               , RULL(0x800007130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P1_1               , RULL(0x800007130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P1_2               , RULL(0x80000B130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P1_2               , RULL(0x80000B130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P1_3               , RULL(0x80000F130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P1_3               , RULL(0x80000F130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P1_4               , RULL(0x800013130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P1_4               , RULL(0x800013130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P2_0               , RULL(0x800003130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P2_0               , RULL(0x800003130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P2_1               , RULL(0x800007130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P2_1               , RULL(0x800007130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P2_2               , RULL(0x80000B130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P2_2               , RULL(0x80000B130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P2_3               , RULL(0x80000F130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P2_3               , RULL(0x80000F130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P2_4               , RULL(0x800013130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P2_4               , RULL(0x800013130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P3_0               , RULL(0x8000031307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P3_0               , RULL(0x8000031308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P3_1               , RULL(0x8000071307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P3_1               , RULL(0x8000071308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P3_2               , RULL(0x80000B1307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P3_2               , RULL(0x80000B1308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P3_3               , RULL(0x80000F1307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P3_3               , RULL(0x80000F1308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P3_4               , RULL(0x8000131307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_GATE_DELAY_RP3_P3_4               , RULL(0x8000131308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P0_0     , RULL(0x800000090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P0_0   , RULL(0x800000090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P0_0   , RULL(0x800000090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P0_1     , RULL(0x800004090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P0_1   , RULL(0x800004090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P0_1   , RULL(0x800004090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P0_2     , RULL(0x800008090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P0_2   , RULL(0x800008090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P0_2   , RULL(0x800008090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P0_3     , RULL(0x80000C090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P0_3   , RULL(0x80000C090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P0_3   , RULL(0x80000C090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P0_4     , RULL(0x800010090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P0_4   , RULL(0x800010090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P0_4   , RULL(0x800010090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P1_0   , RULL(0x800000090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P1_0   , RULL(0x800000090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P1_1   , RULL(0x800004090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P1_1   , RULL(0x800004090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P1_2   , RULL(0x800008090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P1_2   , RULL(0x800008090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P1_3   , RULL(0x80000C090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P1_3   , RULL(0x80000C090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P1_4   , RULL(0x800010090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P1_4   , RULL(0x800010090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P2_0   , RULL(0x800000090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P2_0   , RULL(0x800000090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P2_1   , RULL(0x800004090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P2_1   , RULL(0x800004090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P2_2   , RULL(0x800008090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P2_2   , RULL(0x800008090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P2_3   , RULL(0x80000C090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P2_3   , RULL(0x80000C090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P2_4   , RULL(0x800010090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P2_4   , RULL(0x800010090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P3_0   , RULL(0x8000000907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P3_0   , RULL(0x8000000908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P3_1   , RULL(0x8000040907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P3_1   , RULL(0x8000040908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P3_2   , RULL(0x8000080907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P3_2   , RULL(0x8000080908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P3_3   , RULL(0x80000C0907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P3_3   , RULL(0x80000C0908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P3_4   , RULL(0x8000100907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR0_P3_4   , RULL(0x8000100908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P0_0     , RULL(0x800001090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P0_0   , RULL(0x800001090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P0_0   , RULL(0x800001090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P0_1     , RULL(0x800005090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P0_1   , RULL(0x800005090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P0_1   , RULL(0x800005090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P0_2     , RULL(0x800009090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P0_2   , RULL(0x800009090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P0_2   , RULL(0x800009090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P0_3     , RULL(0x80000D090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P0_3   , RULL(0x80000D090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P0_3   , RULL(0x80000D090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P0_4     , RULL(0x800011090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P0_4   , RULL(0x800011090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P0_4   , RULL(0x800011090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P1_0   , RULL(0x800001090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P1_0   , RULL(0x800001090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P1_1   , RULL(0x800005090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P1_1   , RULL(0x800005090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P1_2   , RULL(0x800009090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P1_2   , RULL(0x800009090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P1_3   , RULL(0x80000D090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P1_3   , RULL(0x80000D090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P1_4   , RULL(0x800011090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P1_4   , RULL(0x800011090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P2_0   , RULL(0x800001090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P2_0   , RULL(0x800001090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P2_1   , RULL(0x800005090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P2_1   , RULL(0x800005090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P2_2   , RULL(0x800009090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P2_2   , RULL(0x800009090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P2_3   , RULL(0x80000D090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P2_3   , RULL(0x80000D090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P2_4   , RULL(0x800011090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P2_4   , RULL(0x800011090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P3_0   , RULL(0x8000010907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P3_0   , RULL(0x8000010908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P3_1   , RULL(0x8000050907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P3_1   , RULL(0x8000050908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P3_2   , RULL(0x8000090907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P3_2   , RULL(0x8000090908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P3_3   , RULL(0x80000D0907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P3_3   , RULL(0x80000D0908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P3_4   , RULL(0x8000110907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR1_P3_4   , RULL(0x8000110908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P0_0     , RULL(0x800002090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P0_0   , RULL(0x800002090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P0_0   , RULL(0x800002090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P0_1     , RULL(0x800006090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P0_1   , RULL(0x800006090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P0_1   , RULL(0x800006090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P0_2     , RULL(0x80000A090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P0_2   , RULL(0x80000A090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P0_2   , RULL(0x80000A090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P0_3     , RULL(0x80000E090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P0_3   , RULL(0x80000E090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P0_3   , RULL(0x80000E090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P0_4     , RULL(0x800012090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P0_4   , RULL(0x800012090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P0_4   , RULL(0x800012090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P1_0   , RULL(0x800002090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P1_0   , RULL(0x800002090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P1_1   , RULL(0x800006090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P1_1   , RULL(0x800006090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P1_2   , RULL(0x80000A090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P1_2   , RULL(0x80000A090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P1_3   , RULL(0x80000E090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P1_3   , RULL(0x80000E090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P1_4   , RULL(0x800012090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P1_4   , RULL(0x800012090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P2_0   , RULL(0x800002090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P2_0   , RULL(0x800002090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P2_1   , RULL(0x800006090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P2_1   , RULL(0x800006090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P2_2   , RULL(0x80000A090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P2_2   , RULL(0x80000A090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P2_3   , RULL(0x80000E090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P2_3   , RULL(0x80000E090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P2_4   , RULL(0x800012090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P2_4   , RULL(0x800012090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P3_0   , RULL(0x8000020907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P3_0   , RULL(0x8000020908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P3_1   , RULL(0x8000060907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P3_1   , RULL(0x8000060908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P3_2   , RULL(0x80000A0907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P3_2   , RULL(0x80000A0908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P3_3   , RULL(0x80000E0907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P3_3   , RULL(0x80000E0908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P3_4   , RULL(0x8000120907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR2_P3_4   , RULL(0x8000120908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P0_0     , RULL(0x800003090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P0_0   , RULL(0x800003090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P0_0   , RULL(0x800003090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P0_1     , RULL(0x800007090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P0_1   , RULL(0x800007090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P0_1   , RULL(0x800007090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P0_2     , RULL(0x80000B090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P0_2   , RULL(0x80000B090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P0_2   , RULL(0x80000B090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P0_3     , RULL(0x80000F090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P0_3   , RULL(0x80000F090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P0_3   , RULL(0x80000F090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P0_4     , RULL(0x800013090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P0_4   , RULL(0x800013090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P0_4   , RULL(0x800013090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P1_0   , RULL(0x800003090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P1_0   , RULL(0x800003090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P1_1   , RULL(0x800007090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P1_1   , RULL(0x800007090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P1_2   , RULL(0x80000B090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P1_2   , RULL(0x80000B090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P1_3   , RULL(0x80000F090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P1_3   , RULL(0x80000F090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P1_4   , RULL(0x800013090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P1_4   , RULL(0x800013090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P2_0   , RULL(0x800003090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P2_0   , RULL(0x800003090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P2_1   , RULL(0x800007090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P2_1   , RULL(0x800007090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P2_2   , RULL(0x80000B090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P2_2   , RULL(0x80000B090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P2_3   , RULL(0x80000F090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P2_3   , RULL(0x80000F090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P2_4   , RULL(0x800013090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P2_4   , RULL(0x800013090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P3_0   , RULL(0x8000030907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P3_0   , RULL(0x8000030908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P3_1   , RULL(0x8000070907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P3_1   , RULL(0x8000070908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P3_2   , RULL(0x80000B0907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P3_2   , RULL(0x80000B0908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P3_3   , RULL(0x80000F0907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P3_3   , RULL(0x80000F0908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P3_4   , RULL(0x8000130907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQS_RD_PHASE_SELECT_RANK_PAIR3_P3_4   , RULL(0x8000130908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P0_0                   , RULL(0x8000007E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P0_0                 , RULL(0x8000007E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P0_0                 , RULL(0x8000007E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P0_1                   , RULL(0x8000047E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P0_1                 , RULL(0x8000047E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P0_1                 , RULL(0x8000047E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P0_2                   , RULL(0x8000087E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P0_2                 , RULL(0x8000087E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P0_2                 , RULL(0x8000087E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P0_3                   , RULL(0x80000C7E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P0_3                 , RULL(0x80000C7E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P0_3                 , RULL(0x80000C7E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P0_4                   , RULL(0x8000107E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P0_4                 , RULL(0x8000107E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P0_4                 , RULL(0x8000107E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P1_0                 , RULL(0x8000007E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P1_0                 , RULL(0x8000007E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P1_1                 , RULL(0x8000047E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P1_1                 , RULL(0x8000047E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P1_2                 , RULL(0x8000087E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P1_2                 , RULL(0x8000087E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P1_3                 , RULL(0x80000C7E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P1_3                 , RULL(0x80000C7E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P1_4                 , RULL(0x8000107E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P1_4                 , RULL(0x8000107E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P2_0                 , RULL(0x8000007E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P2_0                 , RULL(0x8000007E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P2_1                 , RULL(0x8000047E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P2_1                 , RULL(0x8000047E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P2_2                 , RULL(0x8000087E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P2_2                 , RULL(0x8000087E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P2_3                 , RULL(0x80000C7E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P2_3                 , RULL(0x80000C7E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P2_4                 , RULL(0x8000107E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P2_4                 , RULL(0x8000107E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P3_0                 , RULL(0x8000007E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P3_0                 , RULL(0x8000007E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P3_1                 , RULL(0x8000047E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P3_1                 , RULL(0x8000047E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P3_2                 , RULL(0x8000087E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P3_2                 , RULL(0x8000087E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P3_3                 , RULL(0x80000C7E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P3_3                 , RULL(0x80000C7E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P3_4                 , RULL(0x8000107E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP0_P3_4                 , RULL(0x8000107E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P0_0                   , RULL(0x8000017E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P0_0                 , RULL(0x8000017E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P0_0                 , RULL(0x8000017E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P0_1                   , RULL(0x8000057E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P0_1                 , RULL(0x8000057E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P0_1                 , RULL(0x8000057E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P0_2                   , RULL(0x8000097E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P0_2                 , RULL(0x8000097E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P0_2                 , RULL(0x8000097E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P0_3                   , RULL(0x80000D7E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P0_3                 , RULL(0x80000D7E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P0_3                 , RULL(0x80000D7E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P0_4                   , RULL(0x8000117E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P0_4                 , RULL(0x8000117E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P0_4                 , RULL(0x8000117E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P1_0                 , RULL(0x8000017E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P1_0                 , RULL(0x8000017E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P1_1                 , RULL(0x8000057E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P1_1                 , RULL(0x8000057E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P1_2                 , RULL(0x8000097E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P1_2                 , RULL(0x8000097E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P1_3                 , RULL(0x80000D7E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P1_3                 , RULL(0x80000D7E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P1_4                 , RULL(0x8000117E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P1_4                 , RULL(0x8000117E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P2_0                 , RULL(0x8000017E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P2_0                 , RULL(0x8000017E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P2_1                 , RULL(0x8000057E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P2_1                 , RULL(0x8000057E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P2_2                 , RULL(0x8000097E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P2_2                 , RULL(0x8000097E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P2_3                 , RULL(0x80000D7E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P2_3                 , RULL(0x80000D7E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P2_4                 , RULL(0x8000117E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P2_4                 , RULL(0x8000117E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P3_0                 , RULL(0x8000017E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P3_0                 , RULL(0x8000017E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P3_1                 , RULL(0x8000057E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P3_1                 , RULL(0x8000057E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P3_2                 , RULL(0x8000097E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P3_2                 , RULL(0x8000097E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P3_3                 , RULL(0x80000D7E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P3_3                 , RULL(0x80000D7E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P3_4                 , RULL(0x8000117E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP1_P3_4                 , RULL(0x8000117E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P0_0                   , RULL(0x8000027E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P0_0                 , RULL(0x8000027E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P0_0                 , RULL(0x8000027E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P0_1                   , RULL(0x8000067E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P0_1                 , RULL(0x8000067E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P0_1                 , RULL(0x8000067E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P0_2                   , RULL(0x80000A7E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P0_2                 , RULL(0x80000A7E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P0_2                 , RULL(0x80000A7E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P0_3                   , RULL(0x80000E7E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P0_3                 , RULL(0x80000E7E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P0_3                 , RULL(0x80000E7E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P0_4                   , RULL(0x8000127E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P0_4                 , RULL(0x8000127E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P0_4                 , RULL(0x8000127E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P1_0                 , RULL(0x8000027E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P1_0                 , RULL(0x8000027E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P1_1                 , RULL(0x8000067E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P1_1                 , RULL(0x8000067E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P1_2                 , RULL(0x80000A7E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P1_2                 , RULL(0x80000A7E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P1_3                 , RULL(0x80000E7E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P1_3                 , RULL(0x80000E7E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P1_4                 , RULL(0x8000127E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P1_4                 , RULL(0x8000127E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P2_0                 , RULL(0x8000027E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P2_0                 , RULL(0x8000027E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P2_1                 , RULL(0x8000067E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P2_1                 , RULL(0x8000067E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P2_2                 , RULL(0x80000A7E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P2_2                 , RULL(0x80000A7E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P2_3                 , RULL(0x80000E7E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P2_3                 , RULL(0x80000E7E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P2_4                 , RULL(0x8000127E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P2_4                 , RULL(0x8000127E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P3_0                 , RULL(0x8000027E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P3_0                 , RULL(0x8000027E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P3_1                 , RULL(0x8000067E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P3_1                 , RULL(0x8000067E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P3_2                 , RULL(0x80000A7E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P3_2                 , RULL(0x80000A7E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P3_3                 , RULL(0x80000E7E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P3_3                 , RULL(0x80000E7E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P3_4                 , RULL(0x8000127E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP2_P3_4                 , RULL(0x8000127E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P0_0                   , RULL(0x8000037E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P0_0                 , RULL(0x8000037E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P0_0                 , RULL(0x8000037E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P0_1                   , RULL(0x8000077E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P0_1                 , RULL(0x8000077E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P0_1                 , RULL(0x8000077E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P0_2                   , RULL(0x80000B7E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P0_2                 , RULL(0x80000B7E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P0_2                 , RULL(0x80000B7E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P0_3                   , RULL(0x80000F7E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P0_3                 , RULL(0x80000F7E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P0_3                 , RULL(0x80000F7E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P0_4                   , RULL(0x8000137E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P0_4                 , RULL(0x8000137E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P0_4                 , RULL(0x8000137E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P1_0                 , RULL(0x8000037E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P1_0                 , RULL(0x8000037E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P1_1                 , RULL(0x8000077E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P1_1                 , RULL(0x8000077E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P1_2                 , RULL(0x80000B7E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P1_2                 , RULL(0x80000B7E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P1_3                 , RULL(0x80000F7E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P1_3                 , RULL(0x80000F7E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P1_4                 , RULL(0x8000137E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P1_4                 , RULL(0x8000137E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P2_0                 , RULL(0x8000037E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P2_0                 , RULL(0x8000037E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P2_1                 , RULL(0x8000077E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P2_1                 , RULL(0x8000077E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P2_2                 , RULL(0x80000B7E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P2_2                 , RULL(0x80000B7E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P2_3                 , RULL(0x80000F7E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P2_3                 , RULL(0x80000F7E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P2_4                 , RULL(0x8000137E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P2_4                 , RULL(0x8000137E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P3_0                 , RULL(0x8000037E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P3_0                 , RULL(0x8000037E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P3_1                 , RULL(0x8000077E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P3_1                 , RULL(0x8000077E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P3_2                 , RULL(0x80000B7E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P3_2                 , RULL(0x80000B7E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P3_3                 , RULL(0x80000F7E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P3_3                 , RULL(0x80000F7E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P3_4                 , RULL(0x8000137E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DQ_WR_OFFSET_RP3_P3_4                 , RULL(0x8000137E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DRIFT_LIMITS_P0_0                       , RULL(0x8000000A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DRIFT_LIMITS_P0_0                     , RULL(0x8000000A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DRIFT_LIMITS_P0_0                     , RULL(0x8000000A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DRIFT_LIMITS_P0_1                       , RULL(0x8000040A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DRIFT_LIMITS_P0_1                     , RULL(0x8000040A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DRIFT_LIMITS_P0_1                     , RULL(0x8000040A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DRIFT_LIMITS_P0_2                       , RULL(0x8000080A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DRIFT_LIMITS_P0_2                     , RULL(0x8000080A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DRIFT_LIMITS_P0_2                     , RULL(0x8000080A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DRIFT_LIMITS_P0_3                       , RULL(0x80000C0A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DRIFT_LIMITS_P0_3                     , RULL(0x80000C0A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DRIFT_LIMITS_P0_3                     , RULL(0x80000C0A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_DRIFT_LIMITS_P0_4                       , RULL(0x8000100A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_DRIFT_LIMITS_P0_4                     , RULL(0x8000100A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_DRIFT_LIMITS_P0_4                     , RULL(0x8000100A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DRIFT_LIMITS_P1_0                     , RULL(0x8000000A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DRIFT_LIMITS_P1_0                     , RULL(0x8000000A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DRIFT_LIMITS_P1_1                     , RULL(0x8000040A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DRIFT_LIMITS_P1_1                     , RULL(0x8000040A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DRIFT_LIMITS_P1_2                     , RULL(0x8000080A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DRIFT_LIMITS_P1_2                     , RULL(0x8000080A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DRIFT_LIMITS_P1_3                     , RULL(0x80000C0A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DRIFT_LIMITS_P1_3                     , RULL(0x80000C0A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_DRIFT_LIMITS_P1_4                     , RULL(0x8000100A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_DRIFT_LIMITS_P1_4                     , RULL(0x8000100A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DRIFT_LIMITS_P2_0                     , RULL(0x8000000A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DRIFT_LIMITS_P2_0                     , RULL(0x8000000A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DRIFT_LIMITS_P2_1                     , RULL(0x8000040A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DRIFT_LIMITS_P2_1                     , RULL(0x8000040A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DRIFT_LIMITS_P2_2                     , RULL(0x8000080A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DRIFT_LIMITS_P2_2                     , RULL(0x8000080A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DRIFT_LIMITS_P2_3                     , RULL(0x80000C0A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DRIFT_LIMITS_P2_3                     , RULL(0x80000C0A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_DRIFT_LIMITS_P2_4                     , RULL(0x8000100A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_DRIFT_LIMITS_P2_4                     , RULL(0x8000100A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DRIFT_LIMITS_P3_0                     , RULL(0x8000000A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DRIFT_LIMITS_P3_0                     , RULL(0x8000000A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DRIFT_LIMITS_P3_1                     , RULL(0x8000040A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DRIFT_LIMITS_P3_1                     , RULL(0x8000040A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DRIFT_LIMITS_P3_2                     , RULL(0x8000080A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DRIFT_LIMITS_P3_2                     , RULL(0x8000080A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DRIFT_LIMITS_P3_3                     , RULL(0x80000C0A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DRIFT_LIMITS_P3_3                     , RULL(0x80000C0A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_DRIFT_LIMITS_P3_4                     , RULL(0x8000100A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_DRIFT_LIMITS_P3_4                     , RULL(0x8000100A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P0_0      , RULL(0x8000005C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P0_0    , RULL(0x8000005C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P0_0    , RULL(0x8000005C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P0_1      , RULL(0x8000045C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P0_1    , RULL(0x8000045C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P0_1    , RULL(0x8000045C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P0_2      , RULL(0x8000085C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P0_2    , RULL(0x8000085C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P0_2    , RULL(0x8000085C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P0_3      , RULL(0x80000C5C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P0_3    , RULL(0x80000C5C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P0_3    , RULL(0x80000C5C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P0_4      , RULL(0x8000105C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P0_4    , RULL(0x8000105C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P0_4    , RULL(0x8000105C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P1_0    , RULL(0x8000005C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P1_0    , RULL(0x8000005C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P1_1    , RULL(0x8000045C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P1_1    , RULL(0x8000045C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P1_2    , RULL(0x8000085C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P1_2    , RULL(0x8000085C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P1_3    , RULL(0x80000C5C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P1_3    , RULL(0x80000C5C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P1_4    , RULL(0x8000105C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P1_4    , RULL(0x8000105C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P2_0    , RULL(0x8000005C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P2_0    , RULL(0x8000005C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P2_1    , RULL(0x8000045C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P2_1    , RULL(0x8000045C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P2_2    , RULL(0x8000085C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P2_2    , RULL(0x8000085C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P2_3    , RULL(0x80000C5C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P2_3    , RULL(0x80000C5C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P2_4    , RULL(0x8000105C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P2_4    , RULL(0x8000105C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P3_0    , RULL(0x8000005C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P3_0    , RULL(0x8000005C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P3_1    , RULL(0x8000045C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P3_1    , RULL(0x8000045C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P3_2    , RULL(0x8000085C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P3_2    , RULL(0x8000085C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P3_3    , RULL(0x80000C5C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P3_3    , RULL(0x80000C5C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P3_4    , RULL(0x8000105C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR0_P3_4    , RULL(0x8000105C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P0_0      , RULL(0x8000015C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P0_0    , RULL(0x8000015C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P0_0    , RULL(0x8000015C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P0_1      , RULL(0x8000055C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P0_1    , RULL(0x8000055C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P0_1    , RULL(0x8000055C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P0_2      , RULL(0x8000095C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P0_2    , RULL(0x8000095C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P0_2    , RULL(0x8000095C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P0_3      , RULL(0x80000D5C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P0_3    , RULL(0x80000D5C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P0_3    , RULL(0x80000D5C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P0_4      , RULL(0x8000115C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P0_4    , RULL(0x8000115C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P0_4    , RULL(0x8000115C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P1_0    , RULL(0x8000015C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P1_0    , RULL(0x8000015C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P1_1    , RULL(0x8000055C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P1_1    , RULL(0x8000055C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P1_2    , RULL(0x8000095C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P1_2    , RULL(0x8000095C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P1_3    , RULL(0x80000D5C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P1_3    , RULL(0x80000D5C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P1_4    , RULL(0x8000115C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P1_4    , RULL(0x8000115C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P2_0    , RULL(0x8000015C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P2_0    , RULL(0x8000015C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P2_1    , RULL(0x8000055C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P2_1    , RULL(0x8000055C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P2_2    , RULL(0x8000095C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P2_2    , RULL(0x8000095C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P2_3    , RULL(0x80000D5C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P2_3    , RULL(0x80000D5C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P2_4    , RULL(0x8000115C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P2_4    , RULL(0x8000115C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P3_0    , RULL(0x8000015C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P3_0    , RULL(0x8000015C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P3_1    , RULL(0x8000055C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P3_1    , RULL(0x8000055C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P3_2    , RULL(0x8000095C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P3_2    , RULL(0x8000095C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P3_3    , RULL(0x80000D5C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P3_3    , RULL(0x80000D5C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P3_4    , RULL(0x8000115C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR1_P3_4    , RULL(0x8000115C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P0_0      , RULL(0x8000025C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P0_0    , RULL(0x8000025C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P0_0    , RULL(0x8000025C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P0_1      , RULL(0x8000065C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P0_1    , RULL(0x8000065C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P0_1    , RULL(0x8000065C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P0_2      , RULL(0x80000A5C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P0_2    , RULL(0x80000A5C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P0_2    , RULL(0x80000A5C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P0_3      , RULL(0x80000E5C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P0_3    , RULL(0x80000E5C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P0_3    , RULL(0x80000E5C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P0_4      , RULL(0x8000125C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P0_4    , RULL(0x8000125C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P0_4    , RULL(0x8000125C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P1_0    , RULL(0x8000025C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P1_0    , RULL(0x8000025C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P1_1    , RULL(0x8000065C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P1_1    , RULL(0x8000065C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P1_2    , RULL(0x80000A5C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P1_2    , RULL(0x80000A5C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P1_3    , RULL(0x80000E5C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P1_3    , RULL(0x80000E5C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P1_4    , RULL(0x8000125C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P1_4    , RULL(0x8000125C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P2_0    , RULL(0x8000025C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P2_0    , RULL(0x8000025C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P2_1    , RULL(0x8000065C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P2_1    , RULL(0x8000065C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P2_2    , RULL(0x80000A5C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P2_2    , RULL(0x80000A5C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P2_3    , RULL(0x80000E5C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P2_3    , RULL(0x80000E5C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P2_4    , RULL(0x8000125C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P2_4    , RULL(0x8000125C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P3_0    , RULL(0x8000025C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P3_0    , RULL(0x8000025C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P3_1    , RULL(0x8000065C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P3_1    , RULL(0x8000065C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P3_2    , RULL(0x80000A5C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P3_2    , RULL(0x80000A5C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P3_3    , RULL(0x80000E5C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P3_3    , RULL(0x80000E5C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P3_4    , RULL(0x8000125C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR2_P3_4    , RULL(0x8000125C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P0_0      , RULL(0x8000035C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P0_0    , RULL(0x8000035C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P0_0    , RULL(0x8000035C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P0_1      , RULL(0x8000075C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P0_1    , RULL(0x8000075C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P0_1    , RULL(0x8000075C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P0_2      , RULL(0x80000B5C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P0_2    , RULL(0x80000B5C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P0_2    , RULL(0x80000B5C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P0_3      , RULL(0x80000F5C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P0_3    , RULL(0x80000F5C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P0_3    , RULL(0x80000F5C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P0_4      , RULL(0x8000135C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P0_4    , RULL(0x8000135C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P0_4    , RULL(0x8000135C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P1_0    , RULL(0x8000035C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P1_0    , RULL(0x8000035C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P1_1    , RULL(0x8000075C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P1_1    , RULL(0x8000075C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P1_2    , RULL(0x80000B5C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P1_2    , RULL(0x80000B5C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P1_3    , RULL(0x80000F5C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P1_3    , RULL(0x80000F5C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P1_4    , RULL(0x8000135C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P1_4    , RULL(0x8000135C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P2_0    , RULL(0x8000035C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P2_0    , RULL(0x8000035C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P2_1    , RULL(0x8000075C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P2_1    , RULL(0x8000075C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P2_2    , RULL(0x80000B5C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P2_2    , RULL(0x80000B5C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P2_3    , RULL(0x80000F5C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P2_3    , RULL(0x80000F5C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P2_4    , RULL(0x8000135C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P2_4    , RULL(0x8000135C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P3_0    , RULL(0x8000035C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P3_0    , RULL(0x8000035C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P3_1    , RULL(0x8000075C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P3_1    , RULL(0x8000075C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P3_2    , RULL(0x80000B5C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P3_2    , RULL(0x80000B5C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P3_3    , RULL(0x80000F5C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P3_3    , RULL(0x80000F5C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P3_4    , RULL(0x8000135C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN0_RANK_PAIR3_P3_4    , RULL(0x8000135C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P0_0      , RULL(0x8000005D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P0_0    , RULL(0x8000005D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P0_0    , RULL(0x8000005D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P0_1      , RULL(0x8000045D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P0_1    , RULL(0x8000045D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P0_1    , RULL(0x8000045D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P0_2      , RULL(0x8000085D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P0_2    , RULL(0x8000085D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P0_2    , RULL(0x8000085D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P0_3      , RULL(0x80000C5D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P0_3    , RULL(0x80000C5D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P0_3    , RULL(0x80000C5D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P0_4      , RULL(0x8000105D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P0_4    , RULL(0x8000105D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P0_4    , RULL(0x8000105D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P1_0    , RULL(0x8000005D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P1_0    , RULL(0x8000005D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P1_1    , RULL(0x8000045D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P1_1    , RULL(0x8000045D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P1_2    , RULL(0x8000085D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P1_2    , RULL(0x8000085D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P1_3    , RULL(0x80000C5D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P1_3    , RULL(0x80000C5D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P1_4    , RULL(0x8000105D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P1_4    , RULL(0x8000105D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P2_0    , RULL(0x8000005D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P2_0    , RULL(0x8000005D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P2_1    , RULL(0x8000045D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P2_1    , RULL(0x8000045D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P2_2    , RULL(0x8000085D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P2_2    , RULL(0x8000085D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P2_3    , RULL(0x80000C5D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P2_3    , RULL(0x80000C5D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P2_4    , RULL(0x8000105D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P2_4    , RULL(0x8000105D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P3_0    , RULL(0x8000005D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P3_0    , RULL(0x8000005D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P3_1    , RULL(0x8000045D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P3_1    , RULL(0x8000045D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P3_2    , RULL(0x8000085D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P3_2    , RULL(0x8000085D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P3_3    , RULL(0x80000C5D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P3_3    , RULL(0x80000C5D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P3_4    , RULL(0x8000105D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR0_P3_4    , RULL(0x8000105D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P0_0      , RULL(0x8000015D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P0_0    , RULL(0x8000015D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P0_0    , RULL(0x8000015D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P0_1      , RULL(0x8000055D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P0_1    , RULL(0x8000055D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P0_1    , RULL(0x8000055D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P0_2      , RULL(0x8000095D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P0_2    , RULL(0x8000095D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P0_2    , RULL(0x8000095D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P0_3      , RULL(0x80000D5D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P0_3    , RULL(0x80000D5D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P0_3    , RULL(0x80000D5D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P0_4      , RULL(0x8000115D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P0_4    , RULL(0x8000115D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P0_4    , RULL(0x8000115D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P1_0    , RULL(0x8000015D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P1_0    , RULL(0x8000015D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P1_1    , RULL(0x8000055D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P1_1    , RULL(0x8000055D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P1_2    , RULL(0x8000095D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P1_2    , RULL(0x8000095D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P1_3    , RULL(0x80000D5D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P1_3    , RULL(0x80000D5D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P1_4    , RULL(0x8000115D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P1_4    , RULL(0x8000115D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P2_0    , RULL(0x8000015D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P2_0    , RULL(0x8000015D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P2_1    , RULL(0x8000055D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P2_1    , RULL(0x8000055D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P2_2    , RULL(0x8000095D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P2_2    , RULL(0x8000095D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P2_3    , RULL(0x80000D5D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P2_3    , RULL(0x80000D5D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P2_4    , RULL(0x8000115D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P2_4    , RULL(0x8000115D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P3_0    , RULL(0x8000015D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P3_0    , RULL(0x8000015D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P3_1    , RULL(0x8000055D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P3_1    , RULL(0x8000055D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P3_2    , RULL(0x8000095D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P3_2    , RULL(0x8000095D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P3_3    , RULL(0x80000D5D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P3_3    , RULL(0x80000D5D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P3_4    , RULL(0x8000115D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR1_P3_4    , RULL(0x8000115D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P0_0      , RULL(0x8000025D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P0_0    , RULL(0x8000025D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P0_0    , RULL(0x8000025D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P0_1      , RULL(0x8000065D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P0_1    , RULL(0x8000065D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P0_1    , RULL(0x8000065D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P0_2      , RULL(0x80000A5D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P0_2    , RULL(0x80000A5D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P0_2    , RULL(0x80000A5D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P0_3      , RULL(0x80000E5D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P0_3    , RULL(0x80000E5D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P0_3    , RULL(0x80000E5D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P0_4      , RULL(0x8000125D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P0_4    , RULL(0x8000125D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P0_4    , RULL(0x8000125D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P1_0    , RULL(0x8000025D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P1_0    , RULL(0x8000025D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P1_1    , RULL(0x8000065D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P1_1    , RULL(0x8000065D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P1_2    , RULL(0x80000A5D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P1_2    , RULL(0x80000A5D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P1_3    , RULL(0x80000E5D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P1_3    , RULL(0x80000E5D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P1_4    , RULL(0x8000125D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P1_4    , RULL(0x8000125D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P2_0    , RULL(0x8000025D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P2_0    , RULL(0x8000025D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P2_1    , RULL(0x8000065D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P2_1    , RULL(0x8000065D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P2_2    , RULL(0x80000A5D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P2_2    , RULL(0x80000A5D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P2_3    , RULL(0x80000E5D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P2_3    , RULL(0x80000E5D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P2_4    , RULL(0x8000125D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P2_4    , RULL(0x8000125D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P3_0    , RULL(0x8000025D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P3_0    , RULL(0x8000025D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P3_1    , RULL(0x8000065D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P3_1    , RULL(0x8000065D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P3_2    , RULL(0x80000A5D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P3_2    , RULL(0x80000A5D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P3_3    , RULL(0x80000E5D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P3_3    , RULL(0x80000E5D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P3_4    , RULL(0x8000125D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR2_P3_4    , RULL(0x8000125D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P0_0      , RULL(0x8000035D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P0_0    , RULL(0x8000035D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P0_0    , RULL(0x8000035D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P0_1      , RULL(0x8000075D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P0_1    , RULL(0x8000075D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P0_1    , RULL(0x8000075D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P0_2      , RULL(0x80000B5D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P0_2    , RULL(0x80000B5D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P0_2    , RULL(0x80000B5D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P0_3      , RULL(0x80000F5D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P0_3    , RULL(0x80000F5D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P0_3    , RULL(0x80000F5D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P0_4      , RULL(0x8000135D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P0_4    , RULL(0x8000135D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P0_4    , RULL(0x8000135D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P1_0    , RULL(0x8000035D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P1_0    , RULL(0x8000035D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P1_1    , RULL(0x8000075D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P1_1    , RULL(0x8000075D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P1_2    , RULL(0x80000B5D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P1_2    , RULL(0x80000B5D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P1_3    , RULL(0x80000F5D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P1_3    , RULL(0x80000F5D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P1_4    , RULL(0x8000135D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P1_4    , RULL(0x8000135D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P2_0    , RULL(0x8000035D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P2_0    , RULL(0x8000035D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P2_1    , RULL(0x8000075D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P2_1    , RULL(0x8000075D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P2_2    , RULL(0x80000B5D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P2_2    , RULL(0x80000B5D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P2_3    , RULL(0x80000F5D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P2_3    , RULL(0x80000F5D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P2_4    , RULL(0x8000135D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P2_4    , RULL(0x8000135D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P3_0    , RULL(0x8000035D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P3_0    , RULL(0x8000035D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P3_1    , RULL(0x8000075D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P3_1    , RULL(0x8000075D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P3_2    , RULL(0x80000B5D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P3_2    , RULL(0x80000B5D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P3_3    , RULL(0x80000F5D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P3_3    , RULL(0x80000F5D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P3_4    , RULL(0x8000135D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_INITIAL_DQS_ALIGN1_RANK_PAIR3_P3_4    , RULL(0x8000135D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_IO_TX_CONFIG0_P0_0                      , RULL(0x800000750701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_IO_TX_CONFIG0_P0_0                    , RULL(0x800000750701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_IO_TX_CONFIG0_P0_0                    , RULL(0x800000750801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_IO_TX_CONFIG0_P0_1                      , RULL(0x800004750701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_IO_TX_CONFIG0_P0_1                    , RULL(0x800004750701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_IO_TX_CONFIG0_P0_1                    , RULL(0x800004750801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_IO_TX_CONFIG0_P0_2                      , RULL(0x800008750701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_IO_TX_CONFIG0_P0_2                    , RULL(0x800008750701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_IO_TX_CONFIG0_P0_2                    , RULL(0x800008750801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_IO_TX_CONFIG0_P0_3                      , RULL(0x80000C750701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_IO_TX_CONFIG0_P0_3                    , RULL(0x80000C750701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_IO_TX_CONFIG0_P0_3                    , RULL(0x80000C750801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_IO_TX_CONFIG0_P0_4                      , RULL(0x800010750701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_IO_TX_CONFIG0_P0_4                    , RULL(0x800010750701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_IO_TX_CONFIG0_P0_4                    , RULL(0x800010750801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_IO_TX_CONFIG0_P1_0                    , RULL(0x800000750701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_IO_TX_CONFIG0_P1_0                    , RULL(0x800000750801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_IO_TX_CONFIG0_P1_1                    , RULL(0x800004750701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_IO_TX_CONFIG0_P1_1                    , RULL(0x800004750801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_IO_TX_CONFIG0_P1_2                    , RULL(0x800008750701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_IO_TX_CONFIG0_P1_2                    , RULL(0x800008750801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_IO_TX_CONFIG0_P1_3                    , RULL(0x80000C750701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_IO_TX_CONFIG0_P1_3                    , RULL(0x80000C750801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_IO_TX_CONFIG0_P1_4                    , RULL(0x800010750701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_IO_TX_CONFIG0_P1_4                    , RULL(0x800010750801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_IO_TX_CONFIG0_P2_0                    , RULL(0x800000750701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_IO_TX_CONFIG0_P2_0                    , RULL(0x800000750801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_IO_TX_CONFIG0_P2_1                    , RULL(0x800004750701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_IO_TX_CONFIG0_P2_1                    , RULL(0x800004750801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_IO_TX_CONFIG0_P2_2                    , RULL(0x800008750701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_IO_TX_CONFIG0_P2_2                    , RULL(0x800008750801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_IO_TX_CONFIG0_P2_3                    , RULL(0x80000C750701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_IO_TX_CONFIG0_P2_3                    , RULL(0x80000C750801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_IO_TX_CONFIG0_P2_4                    , RULL(0x800010750701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_IO_TX_CONFIG0_P2_4                    , RULL(0x800010750801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_IO_TX_CONFIG0_P3_0                    , RULL(0x8000007507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_IO_TX_CONFIG0_P3_0                    , RULL(0x8000007508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_IO_TX_CONFIG0_P3_1                    , RULL(0x8000047507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_IO_TX_CONFIG0_P3_1                    , RULL(0x8000047508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_IO_TX_CONFIG0_P3_2                    , RULL(0x8000087507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_IO_TX_CONFIG0_P3_2                    , RULL(0x8000087508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_IO_TX_CONFIG0_P3_3                    , RULL(0x80000C7507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_IO_TX_CONFIG0_P3_3                    , RULL(0x80000C7508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_IO_TX_CONFIG0_P3_4                    , RULL(0x8000107507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_IO_TX_CONFIG0_P3_4                    , RULL(0x8000107508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_IO_TX_FET_SLICE_P0_0                    , RULL(0x800000780701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_IO_TX_FET_SLICE_P0_0                  , RULL(0x800000780701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_IO_TX_FET_SLICE_P0_0                  , RULL(0x800000780801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_IO_TX_FET_SLICE_P0_1                    , RULL(0x800004780701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_IO_TX_FET_SLICE_P0_1                  , RULL(0x800004780701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_IO_TX_FET_SLICE_P0_1                  , RULL(0x800004780801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_IO_TX_FET_SLICE_P0_2                    , RULL(0x800008780701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_IO_TX_FET_SLICE_P0_2                  , RULL(0x800008780701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_IO_TX_FET_SLICE_P0_2                  , RULL(0x800008780801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_IO_TX_FET_SLICE_P0_3                    , RULL(0x80000C780701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_IO_TX_FET_SLICE_P0_3                  , RULL(0x80000C780701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_IO_TX_FET_SLICE_P0_3                  , RULL(0x80000C780801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_IO_TX_FET_SLICE_P0_4                    , RULL(0x800010780701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_IO_TX_FET_SLICE_P0_4                  , RULL(0x800010780701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_IO_TX_FET_SLICE_P0_4                  , RULL(0x800010780801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_IO_TX_FET_SLICE_P1_0                  , RULL(0x800000780701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_IO_TX_FET_SLICE_P1_0                  , RULL(0x800000780801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_IO_TX_FET_SLICE_P1_1                  , RULL(0x800004780701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_IO_TX_FET_SLICE_P1_1                  , RULL(0x800004780801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_IO_TX_FET_SLICE_P1_2                  , RULL(0x800008780701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_IO_TX_FET_SLICE_P1_2                  , RULL(0x800008780801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_IO_TX_FET_SLICE_P1_3                  , RULL(0x80000C780701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_IO_TX_FET_SLICE_P1_3                  , RULL(0x80000C780801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_IO_TX_FET_SLICE_P1_4                  , RULL(0x800010780701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_IO_TX_FET_SLICE_P1_4                  , RULL(0x800010780801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_IO_TX_FET_SLICE_P2_0                  , RULL(0x800000780701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_IO_TX_FET_SLICE_P2_0                  , RULL(0x800000780801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_IO_TX_FET_SLICE_P2_1                  , RULL(0x800004780701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_IO_TX_FET_SLICE_P2_1                  , RULL(0x800004780801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_IO_TX_FET_SLICE_P2_2                  , RULL(0x800008780701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_IO_TX_FET_SLICE_P2_2                  , RULL(0x800008780801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_IO_TX_FET_SLICE_P2_3                  , RULL(0x80000C780701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_IO_TX_FET_SLICE_P2_3                  , RULL(0x80000C780801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_IO_TX_FET_SLICE_P2_4                  , RULL(0x800010780701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_IO_TX_FET_SLICE_P2_4                  , RULL(0x800010780801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_IO_TX_FET_SLICE_P3_0                  , RULL(0x8000007807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_IO_TX_FET_SLICE_P3_0                  , RULL(0x8000007808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_IO_TX_FET_SLICE_P3_1                  , RULL(0x8000047807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_IO_TX_FET_SLICE_P3_1                  , RULL(0x8000047808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_IO_TX_FET_SLICE_P3_2                  , RULL(0x8000087807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_IO_TX_FET_SLICE_P3_2                  , RULL(0x8000087808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_IO_TX_FET_SLICE_P3_3                  , RULL(0x80000C7807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_IO_TX_FET_SLICE_P3_3                  , RULL(0x80000C7808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_IO_TX_FET_SLICE_P3_4                  , RULL(0x8000107807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_IO_TX_FET_SLICE_P3_4                  , RULL(0x8000107808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_IO_TX_PFET_TERM_P0_0                    , RULL(0x8000007B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_IO_TX_PFET_TERM_P0_0                  , RULL(0x8000007B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_IO_TX_PFET_TERM_P0_0                  , RULL(0x8000007B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_IO_TX_PFET_TERM_P0_1                    , RULL(0x8000047B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_IO_TX_PFET_TERM_P0_1                  , RULL(0x8000047B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_IO_TX_PFET_TERM_P0_1                  , RULL(0x8000047B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_IO_TX_PFET_TERM_P0_2                    , RULL(0x8000087B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_IO_TX_PFET_TERM_P0_2                  , RULL(0x8000087B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_IO_TX_PFET_TERM_P0_2                  , RULL(0x8000087B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_IO_TX_PFET_TERM_P0_3                    , RULL(0x80000C7B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_IO_TX_PFET_TERM_P0_3                  , RULL(0x80000C7B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_IO_TX_PFET_TERM_P0_3                  , RULL(0x80000C7B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_IO_TX_PFET_TERM_P0_4                    , RULL(0x8000107B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_IO_TX_PFET_TERM_P0_4                  , RULL(0x8000107B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_IO_TX_PFET_TERM_P0_4                  , RULL(0x8000107B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_IO_TX_PFET_TERM_P1_0                  , RULL(0x8000007B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_IO_TX_PFET_TERM_P1_0                  , RULL(0x8000007B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_IO_TX_PFET_TERM_P1_1                  , RULL(0x8000047B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_IO_TX_PFET_TERM_P1_1                  , RULL(0x8000047B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_IO_TX_PFET_TERM_P1_2                  , RULL(0x8000087B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_IO_TX_PFET_TERM_P1_2                  , RULL(0x8000087B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_IO_TX_PFET_TERM_P1_3                  , RULL(0x80000C7B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_IO_TX_PFET_TERM_P1_3                  , RULL(0x80000C7B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_IO_TX_PFET_TERM_P1_4                  , RULL(0x8000107B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_IO_TX_PFET_TERM_P1_4                  , RULL(0x8000107B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_IO_TX_PFET_TERM_P2_0                  , RULL(0x8000007B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_IO_TX_PFET_TERM_P2_0                  , RULL(0x8000007B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_IO_TX_PFET_TERM_P2_1                  , RULL(0x8000047B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_IO_TX_PFET_TERM_P2_1                  , RULL(0x8000047B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_IO_TX_PFET_TERM_P2_2                  , RULL(0x8000087B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_IO_TX_PFET_TERM_P2_2                  , RULL(0x8000087B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_IO_TX_PFET_TERM_P2_3                  , RULL(0x80000C7B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_IO_TX_PFET_TERM_P2_3                  , RULL(0x80000C7B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_IO_TX_PFET_TERM_P2_4                  , RULL(0x8000107B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_IO_TX_PFET_TERM_P2_4                  , RULL(0x8000107B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_IO_TX_PFET_TERM_P3_0                  , RULL(0x8000007B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_IO_TX_PFET_TERM_P3_0                  , RULL(0x8000007B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_IO_TX_PFET_TERM_P3_1                  , RULL(0x8000047B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_IO_TX_PFET_TERM_P3_1                  , RULL(0x8000047B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_IO_TX_PFET_TERM_P3_2                  , RULL(0x8000087B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_IO_TX_PFET_TERM_P3_2                  , RULL(0x8000087B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_IO_TX_PFET_TERM_P3_3                  , RULL(0x80000C7B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_IO_TX_PFET_TERM_P3_3                  , RULL(0x80000C7B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_IO_TX_PFET_TERM_P3_4                  , RULL(0x8000107B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_IO_TX_PFET_TERM_P3_4                  , RULL(0x8000107B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_LO_PROBE_SELECT_P0_0                    , RULL(0x800000790701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_LO_PROBE_SELECT_P0_0                  , RULL(0x800000790701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_LO_PROBE_SELECT_P0_0                  , RULL(0x800000790801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_LO_PROBE_SELECT_P0_1                    , RULL(0x800004790701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_LO_PROBE_SELECT_P0_1                  , RULL(0x800004790701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_LO_PROBE_SELECT_P0_1                  , RULL(0x800004790801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_LO_PROBE_SELECT_P0_2                    , RULL(0x800008790701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_LO_PROBE_SELECT_P0_2                  , RULL(0x800008790701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_LO_PROBE_SELECT_P0_2                  , RULL(0x800008790801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_LO_PROBE_SELECT_P0_3                    , RULL(0x80000C790701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_LO_PROBE_SELECT_P0_3                  , RULL(0x80000C790701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_LO_PROBE_SELECT_P0_3                  , RULL(0x80000C790801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_LO_PROBE_SELECT_P0_4                    , RULL(0x800010790701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_LO_PROBE_SELECT_P0_4                  , RULL(0x800010790701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_LO_PROBE_SELECT_P0_4                  , RULL(0x800010790801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_LO_PROBE_SELECT_P1_0                  , RULL(0x800000790701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_LO_PROBE_SELECT_P1_0                  , RULL(0x800000790801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_LO_PROBE_SELECT_P1_1                  , RULL(0x800004790701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_LO_PROBE_SELECT_P1_1                  , RULL(0x800004790801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_LO_PROBE_SELECT_P1_2                  , RULL(0x800008790701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_LO_PROBE_SELECT_P1_2                  , RULL(0x800008790801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_LO_PROBE_SELECT_P1_3                  , RULL(0x80000C790701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_LO_PROBE_SELECT_P1_3                  , RULL(0x80000C790801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_LO_PROBE_SELECT_P1_4                  , RULL(0x800010790701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_LO_PROBE_SELECT_P1_4                  , RULL(0x800010790801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_LO_PROBE_SELECT_P2_0                  , RULL(0x800000790701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_LO_PROBE_SELECT_P2_0                  , RULL(0x800000790801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_LO_PROBE_SELECT_P2_1                  , RULL(0x800004790701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_LO_PROBE_SELECT_P2_1                  , RULL(0x800004790801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_LO_PROBE_SELECT_P2_2                  , RULL(0x800008790701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_LO_PROBE_SELECT_P2_2                  , RULL(0x800008790801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_LO_PROBE_SELECT_P2_3                  , RULL(0x80000C790701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_LO_PROBE_SELECT_P2_3                  , RULL(0x80000C790801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_LO_PROBE_SELECT_P2_4                  , RULL(0x800010790701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_LO_PROBE_SELECT_P2_4                  , RULL(0x800010790801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_LO_PROBE_SELECT_P3_0                  , RULL(0x8000007907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_LO_PROBE_SELECT_P3_0                  , RULL(0x8000007908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_LO_PROBE_SELECT_P3_1                  , RULL(0x8000047907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_LO_PROBE_SELECT_P3_1                  , RULL(0x8000047908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_LO_PROBE_SELECT_P3_2                  , RULL(0x8000087907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_LO_PROBE_SELECT_P3_2                  , RULL(0x8000087908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_LO_PROBE_SELECT_P3_3                  , RULL(0x80000C7907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_LO_PROBE_SELECT_P3_3                  , RULL(0x80000C7908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_LO_PROBE_SELECT_P3_4                  , RULL(0x8000107907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_LO_PROBE_SELECT_P3_4                  , RULL(0x8000107908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_PATTERN_POS_0_P0_0                      , RULL(0x800000320701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_PATTERN_POS_0_P0_0                    , RULL(0x800000320701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_PATTERN_POS_0_P0_0                    , RULL(0x800000320801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_PATTERN_POS_0_P0_1                      , RULL(0x800004320701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_PATTERN_POS_0_P0_1                    , RULL(0x800004320701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_PATTERN_POS_0_P0_1                    , RULL(0x800004320801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_PATTERN_POS_0_P0_2                      , RULL(0x800008320701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_PATTERN_POS_0_P0_2                    , RULL(0x800008320701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_PATTERN_POS_0_P0_2                    , RULL(0x800008320801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_PATTERN_POS_0_P0_3                      , RULL(0x80000C320701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_PATTERN_POS_0_P0_3                    , RULL(0x80000C320701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_PATTERN_POS_0_P0_3                    , RULL(0x80000C320801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_PATTERN_POS_0_P0_4                      , RULL(0x800010320701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_PATTERN_POS_0_P0_4                    , RULL(0x800010320701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_PATTERN_POS_0_P0_4                    , RULL(0x800010320801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_PATTERN_POS_0_P1_0                    , RULL(0x800000320701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_PATTERN_POS_0_P1_0                    , RULL(0x800000320801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_PATTERN_POS_0_P1_1                    , RULL(0x800004320701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_PATTERN_POS_0_P1_1                    , RULL(0x800004320801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_PATTERN_POS_0_P1_2                    , RULL(0x800008320701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_PATTERN_POS_0_P1_2                    , RULL(0x800008320801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_PATTERN_POS_0_P1_3                    , RULL(0x80000C320701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_PATTERN_POS_0_P1_3                    , RULL(0x80000C320801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_PATTERN_POS_0_P1_4                    , RULL(0x800010320701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_PATTERN_POS_0_P1_4                    , RULL(0x800010320801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_PATTERN_POS_0_P2_0                    , RULL(0x800000320701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_PATTERN_POS_0_P2_0                    , RULL(0x800000320801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_PATTERN_POS_0_P2_1                    , RULL(0x800004320701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_PATTERN_POS_0_P2_1                    , RULL(0x800004320801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_PATTERN_POS_0_P2_2                    , RULL(0x800008320701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_PATTERN_POS_0_P2_2                    , RULL(0x800008320801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_PATTERN_POS_0_P2_3                    , RULL(0x80000C320701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_PATTERN_POS_0_P2_3                    , RULL(0x80000C320801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_PATTERN_POS_0_P2_4                    , RULL(0x800010320701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_PATTERN_POS_0_P2_4                    , RULL(0x800010320801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_PATTERN_POS_0_P3_0                    , RULL(0x8000003207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_PATTERN_POS_0_P3_0                    , RULL(0x8000003208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_PATTERN_POS_0_P3_1                    , RULL(0x8000043207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_PATTERN_POS_0_P3_1                    , RULL(0x8000043208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_PATTERN_POS_0_P3_2                    , RULL(0x8000083207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_PATTERN_POS_0_P3_2                    , RULL(0x8000083208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_PATTERN_POS_0_P3_3                    , RULL(0x80000C3207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_PATTERN_POS_0_P3_3                    , RULL(0x80000C3208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_PATTERN_POS_0_P3_4                    , RULL(0x8000103207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_PATTERN_POS_0_P3_4                    , RULL(0x8000103208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_PATTERN_POS_1_P0_0                      , RULL(0x800000330701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_PATTERN_POS_1_P0_0                    , RULL(0x800000330701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_PATTERN_POS_1_P0_0                    , RULL(0x800000330801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_PATTERN_POS_1_P0_1                      , RULL(0x800004330701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_PATTERN_POS_1_P0_1                    , RULL(0x800004330701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_PATTERN_POS_1_P0_1                    , RULL(0x800004330801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_PATTERN_POS_1_P0_2                      , RULL(0x800008330701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_PATTERN_POS_1_P0_2                    , RULL(0x800008330701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_PATTERN_POS_1_P0_2                    , RULL(0x800008330801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_PATTERN_POS_1_P0_3                      , RULL(0x80000C330701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_PATTERN_POS_1_P0_3                    , RULL(0x80000C330701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_PATTERN_POS_1_P0_3                    , RULL(0x80000C330801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_PATTERN_POS_1_P0_4                      , RULL(0x800010330701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_PATTERN_POS_1_P0_4                    , RULL(0x800010330701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_PATTERN_POS_1_P0_4                    , RULL(0x800010330801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_PATTERN_POS_1_P1_0                    , RULL(0x800000330701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_PATTERN_POS_1_P1_0                    , RULL(0x800000330801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_PATTERN_POS_1_P1_1                    , RULL(0x800004330701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_PATTERN_POS_1_P1_1                    , RULL(0x800004330801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_PATTERN_POS_1_P1_2                    , RULL(0x800008330701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_PATTERN_POS_1_P1_2                    , RULL(0x800008330801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_PATTERN_POS_1_P1_3                    , RULL(0x80000C330701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_PATTERN_POS_1_P1_3                    , RULL(0x80000C330801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_PATTERN_POS_1_P1_4                    , RULL(0x800010330701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_PATTERN_POS_1_P1_4                    , RULL(0x800010330801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_PATTERN_POS_1_P2_0                    , RULL(0x800000330701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_PATTERN_POS_1_P2_0                    , RULL(0x800000330801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_PATTERN_POS_1_P2_1                    , RULL(0x800004330701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_PATTERN_POS_1_P2_1                    , RULL(0x800004330801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_PATTERN_POS_1_P2_2                    , RULL(0x800008330701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_PATTERN_POS_1_P2_2                    , RULL(0x800008330801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_PATTERN_POS_1_P2_3                    , RULL(0x80000C330701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_PATTERN_POS_1_P2_3                    , RULL(0x80000C330801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_PATTERN_POS_1_P2_4                    , RULL(0x800010330701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_PATTERN_POS_1_P2_4                    , RULL(0x800010330801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_PATTERN_POS_1_P3_0                    , RULL(0x8000003307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_PATTERN_POS_1_P3_0                    , RULL(0x8000003308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_PATTERN_POS_1_P3_1                    , RULL(0x8000043307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_PATTERN_POS_1_P3_1                    , RULL(0x8000043308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_PATTERN_POS_1_P3_2                    , RULL(0x8000083307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_PATTERN_POS_1_P3_2                    , RULL(0x8000083308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_PATTERN_POS_1_P3_3                    , RULL(0x80000C3307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_PATTERN_POS_1_P3_3                    , RULL(0x80000C3308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_PATTERN_POS_1_P3_4                    , RULL(0x8000103307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_PATTERN_POS_1_P3_4                    , RULL(0x8000103308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_PATTERN_POS_2_P0_0                      , RULL(0x800000340701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_PATTERN_POS_2_P0_0                    , RULL(0x800000340701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_PATTERN_POS_2_P0_0                    , RULL(0x800000340801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_PATTERN_POS_2_P0_1                      , RULL(0x800004340701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_PATTERN_POS_2_P0_1                    , RULL(0x800004340701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_PATTERN_POS_2_P0_1                    , RULL(0x800004340801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_PATTERN_POS_2_P0_2                      , RULL(0x800008340701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_PATTERN_POS_2_P0_2                    , RULL(0x800008340701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_PATTERN_POS_2_P0_2                    , RULL(0x800008340801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_PATTERN_POS_2_P0_3                      , RULL(0x80000C340701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_PATTERN_POS_2_P0_3                    , RULL(0x80000C340701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_PATTERN_POS_2_P0_3                    , RULL(0x80000C340801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_PATTERN_POS_2_P0_4                      , RULL(0x800010340701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_PATTERN_POS_2_P0_4                    , RULL(0x800010340701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_PATTERN_POS_2_P0_4                    , RULL(0x800010340801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_PATTERN_POS_2_P1_0                    , RULL(0x800000340701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_PATTERN_POS_2_P1_0                    , RULL(0x800000340801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_PATTERN_POS_2_P1_1                    , RULL(0x800004340701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_PATTERN_POS_2_P1_1                    , RULL(0x800004340801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_PATTERN_POS_2_P1_2                    , RULL(0x800008340701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_PATTERN_POS_2_P1_2                    , RULL(0x800008340801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_PATTERN_POS_2_P1_3                    , RULL(0x80000C340701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_PATTERN_POS_2_P1_3                    , RULL(0x80000C340801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_PATTERN_POS_2_P1_4                    , RULL(0x800010340701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_PATTERN_POS_2_P1_4                    , RULL(0x800010340801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_PATTERN_POS_2_P2_0                    , RULL(0x800000340701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_PATTERN_POS_2_P2_0                    , RULL(0x800000340801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_PATTERN_POS_2_P2_1                    , RULL(0x800004340701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_PATTERN_POS_2_P2_1                    , RULL(0x800004340801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_PATTERN_POS_2_P2_2                    , RULL(0x800008340701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_PATTERN_POS_2_P2_2                    , RULL(0x800008340801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_PATTERN_POS_2_P2_3                    , RULL(0x80000C340701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_PATTERN_POS_2_P2_3                    , RULL(0x80000C340801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_PATTERN_POS_2_P2_4                    , RULL(0x800010340701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_PATTERN_POS_2_P2_4                    , RULL(0x800010340801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_PATTERN_POS_2_P3_0                    , RULL(0x8000003407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_PATTERN_POS_2_P3_0                    , RULL(0x8000003408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_PATTERN_POS_2_P3_1                    , RULL(0x8000043407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_PATTERN_POS_2_P3_1                    , RULL(0x8000043408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_PATTERN_POS_2_P3_2                    , RULL(0x8000083407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_PATTERN_POS_2_P3_2                    , RULL(0x8000083408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_PATTERN_POS_2_P3_3                    , RULL(0x80000C3407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_PATTERN_POS_2_P3_3                    , RULL(0x80000C3408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_PATTERN_POS_2_P3_4                    , RULL(0x8000103407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_PATTERN_POS_2_P3_4                    , RULL(0x8000103408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG0_P0_0                     , RULL(0x8000001E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG0_P0_0                   , RULL(0x8000001E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG0_P0_0                   , RULL(0x8000001E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG0_P0_1                     , RULL(0x8000041E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG0_P0_1                   , RULL(0x8000041E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG0_P0_1                   , RULL(0x8000041E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG0_P0_2                     , RULL(0x8000081E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG0_P0_2                   , RULL(0x8000081E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG0_P0_2                   , RULL(0x8000081E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG0_P0_3                     , RULL(0x80000C1E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG0_P0_3                   , RULL(0x80000C1E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG0_P0_3                   , RULL(0x80000C1E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG0_P0_4                     , RULL(0x8000101E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG0_P0_4                   , RULL(0x8000101E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG0_P0_4                   , RULL(0x8000101E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG0_P1_0                   , RULL(0x8000001E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG0_P1_0                   , RULL(0x8000001E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG0_P1_1                   , RULL(0x8000041E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG0_P1_1                   , RULL(0x8000041E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG0_P1_2                   , RULL(0x8000081E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG0_P1_2                   , RULL(0x8000081E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG0_P1_3                   , RULL(0x80000C1E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG0_P1_3                   , RULL(0x80000C1E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG0_P1_4                   , RULL(0x8000101E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG0_P1_4                   , RULL(0x8000101E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG0_P2_0                   , RULL(0x8000001E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG0_P2_0                   , RULL(0x8000001E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG0_P2_1                   , RULL(0x8000041E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG0_P2_1                   , RULL(0x8000041E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG0_P2_2                   , RULL(0x8000081E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG0_P2_2                   , RULL(0x8000081E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG0_P2_3                   , RULL(0x80000C1E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG0_P2_3                   , RULL(0x80000C1E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG0_P2_4                   , RULL(0x8000101E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG0_P2_4                   , RULL(0x8000101E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG0_P3_0                   , RULL(0x8000001E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG0_P3_0                   , RULL(0x8000001E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG0_P3_1                   , RULL(0x8000041E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG0_P3_1                   , RULL(0x8000041E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG0_P3_2                   , RULL(0x8000081E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG0_P3_2                   , RULL(0x8000081E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG0_P3_3                   , RULL(0x80000C1E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG0_P3_3                   , RULL(0x80000C1E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG0_P3_4                   , RULL(0x8000101E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG0_P3_4                   , RULL(0x8000101E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG1_P0_0                     , RULL(0x800000350701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG1_P0_0                   , RULL(0x800000350701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG1_P0_0                   , RULL(0x800000350801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG1_P0_1                     , RULL(0x800004350701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG1_P0_1                   , RULL(0x800004350701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG1_P0_1                   , RULL(0x800004350801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG1_P0_2                     , RULL(0x800008350701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG1_P0_2                   , RULL(0x800008350701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG1_P0_2                   , RULL(0x800008350801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG1_P0_3                     , RULL(0x80000C350701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG1_P0_3                   , RULL(0x80000C350701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG1_P0_3                   , RULL(0x80000C350801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG1_P0_4                     , RULL(0x800010350701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG1_P0_4                   , RULL(0x800010350701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG1_P0_4                   , RULL(0x800010350801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG1_P1_0                   , RULL(0x800000350701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG1_P1_0                   , RULL(0x800000350801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG1_P1_1                   , RULL(0x800004350701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG1_P1_1                   , RULL(0x800004350801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG1_P1_2                   , RULL(0x800008350701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG1_P1_2                   , RULL(0x800008350801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG1_P1_3                   , RULL(0x80000C350701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG1_P1_3                   , RULL(0x80000C350801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG1_P1_4                   , RULL(0x800010350701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG1_P1_4                   , RULL(0x800010350801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG1_P2_0                   , RULL(0x800000350701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG1_P2_0                   , RULL(0x800000350801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG1_P2_1                   , RULL(0x800004350701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG1_P2_1                   , RULL(0x800004350801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG1_P2_2                   , RULL(0x800008350701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG1_P2_2                   , RULL(0x800008350801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG1_P2_3                   , RULL(0x80000C350701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG1_P2_3                   , RULL(0x80000C350801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG1_P2_4                   , RULL(0x800010350701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG1_P2_4                   , RULL(0x800010350801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG1_P3_0                   , RULL(0x8000003507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG1_P3_0                   , RULL(0x8000003508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG1_P3_1                   , RULL(0x8000043507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG1_P3_1                   , RULL(0x8000043508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG1_P3_2                   , RULL(0x8000083507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG1_P3_2                   , RULL(0x8000083508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG1_P3_3                   , RULL(0x80000C3507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG1_P3_3                   , RULL(0x80000C3508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG1_P3_4                   , RULL(0x8000103507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG1_P3_4                   , RULL(0x8000103508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG2_P0_0                     , RULL(0x800000360701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG2_P0_0                   , RULL(0x800000360701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG2_P0_0                   , RULL(0x800000360801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG2_P0_1                     , RULL(0x800004360701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG2_P0_1                   , RULL(0x800004360701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG2_P0_1                   , RULL(0x800004360801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG2_P0_2                     , RULL(0x800008360701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG2_P0_2                   , RULL(0x800008360701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG2_P0_2                   , RULL(0x800008360801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG2_P0_3                     , RULL(0x80000C360701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG2_P0_3                   , RULL(0x80000C360701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG2_P0_3                   , RULL(0x80000C360801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG2_P0_4                     , RULL(0x800010360701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM      );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG2_P0_4                   , RULL(0x800010360701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM      );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG2_P0_4                   , RULL(0x800010360801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG2_P1_0                   , RULL(0x800000360701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG2_P1_0                   , RULL(0x800000360801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG2_P1_1                   , RULL(0x800004360701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG2_P1_1                   , RULL(0x800004360801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG2_P1_2                   , RULL(0x800008360701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG2_P1_2                   , RULL(0x800008360801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG2_P1_3                   , RULL(0x80000C360701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG2_P1_3                   , RULL(0x80000C360801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG2_P1_4                   , RULL(0x800010360701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM      );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG2_P1_4                   , RULL(0x800010360801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG2_P2_0                   , RULL(0x800000360701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG2_P2_0                   , RULL(0x800000360801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG2_P2_1                   , RULL(0x800004360701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG2_P2_1                   , RULL(0x800004360801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG2_P2_2                   , RULL(0x800008360701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG2_P2_2                   , RULL(0x800008360801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG2_P2_3                   , RULL(0x80000C360701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG2_P2_3                   , RULL(0x80000C360801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG2_P2_4                   , RULL(0x800010360701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM      );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG2_P2_4                   , RULL(0x800010360801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG2_P3_0                   , RULL(0x8000003607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG2_P3_0                   , RULL(0x8000003608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG2_P3_1                   , RULL(0x8000043607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG2_P3_1                   , RULL(0x8000043608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG2_P3_2                   , RULL(0x8000083607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG2_P3_2                   , RULL(0x8000083608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG2_P3_3                   , RULL(0x80000C3607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG2_P3_3                   , RULL(0x80000C3608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG2_P3_4                   , RULL(0x8000103607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM      );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG2_P3_4                   , RULL(0x8000103608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM      );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG3_P0_0                     , RULL(0x8000006D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG3_P0_0                   , RULL(0x8000006D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG3_P0_0                   , RULL(0x8000006D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG3_P0_1                     , RULL(0x8000046D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG3_P0_1                   , RULL(0x8000046D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG3_P0_1                   , RULL(0x8000046D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG3_P0_2                     , RULL(0x8000086D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG3_P0_2                   , RULL(0x8000086D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG3_P0_2                   , RULL(0x8000086D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG3_P0_3                     , RULL(0x80000C6D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG3_P0_3                   , RULL(0x80000C6D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG3_P0_3                   , RULL(0x80000C6D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG3_P0_4                     , RULL(0x8000106D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG3_P0_4                   , RULL(0x8000106D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG3_P0_4                   , RULL(0x8000106D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG3_P1_0                   , RULL(0x8000006D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG3_P1_0                   , RULL(0x8000006D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG3_P1_1                   , RULL(0x8000046D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG3_P1_1                   , RULL(0x8000046D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG3_P1_2                   , RULL(0x8000086D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG3_P1_2                   , RULL(0x8000086D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG3_P1_3                   , RULL(0x80000C6D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG3_P1_3                   , RULL(0x80000C6D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG3_P1_4                   , RULL(0x8000106D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG3_P1_4                   , RULL(0x8000106D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG3_P2_0                   , RULL(0x8000006D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG3_P2_0                   , RULL(0x8000006D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG3_P2_1                   , RULL(0x8000046D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG3_P2_1                   , RULL(0x8000046D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG3_P2_2                   , RULL(0x8000086D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG3_P2_2                   , RULL(0x8000086D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG3_P2_3                   , RULL(0x80000C6D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG3_P2_3                   , RULL(0x80000C6D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG3_P2_4                   , RULL(0x8000106D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG3_P2_4                   , RULL(0x8000106D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG3_P3_0                   , RULL(0x8000006D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG3_P3_0                   , RULL(0x8000006D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG3_P3_1                   , RULL(0x8000046D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG3_P3_1                   , RULL(0x8000046D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG3_P3_2                   , RULL(0x8000086D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG3_P3_2                   , RULL(0x8000086D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG3_P3_3                   , RULL(0x80000C6D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG3_P3_3                   , RULL(0x80000C6D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG3_P3_4                   , RULL(0x8000106D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG3_P3_4                   , RULL(0x8000106D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG4_P0_0                     , RULL(0x8000006E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG4_P0_0                   , RULL(0x8000006E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG4_P0_0                   , RULL(0x8000006E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG4_P0_1                     , RULL(0x8000046E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG4_P0_1                   , RULL(0x8000046E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG4_P0_1                   , RULL(0x8000046E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG4_P0_2                     , RULL(0x8000086E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG4_P0_2                   , RULL(0x8000086E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG4_P0_2                   , RULL(0x8000086E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG4_P0_3                     , RULL(0x80000C6E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG4_P0_3                   , RULL(0x80000C6E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG4_P0_3                   , RULL(0x80000C6E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG4_P0_4                     , RULL(0x8000106E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG4_P0_4                   , RULL(0x8000106E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG4_P0_4                   , RULL(0x8000106E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG4_P1_0                   , RULL(0x8000006E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG4_P1_0                   , RULL(0x8000006E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG4_P1_1                   , RULL(0x8000046E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG4_P1_1                   , RULL(0x8000046E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG4_P1_2                   , RULL(0x8000086E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG4_P1_2                   , RULL(0x8000086E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG4_P1_3                   , RULL(0x80000C6E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG4_P1_3                   , RULL(0x80000C6E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG4_P1_4                   , RULL(0x8000106E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG4_P1_4                   , RULL(0x8000106E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG4_P2_0                   , RULL(0x8000006E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG4_P2_0                   , RULL(0x8000006E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG4_P2_1                   , RULL(0x8000046E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG4_P2_1                   , RULL(0x8000046E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG4_P2_2                   , RULL(0x8000086E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG4_P2_2                   , RULL(0x8000086E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG4_P2_3                   , RULL(0x80000C6E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG4_P2_3                   , RULL(0x80000C6E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG4_P2_4                   , RULL(0x8000106E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG4_P2_4                   , RULL(0x8000106E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG4_P3_0                   , RULL(0x8000006E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG4_P3_0                   , RULL(0x8000006E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG4_P3_1                   , RULL(0x8000046E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG4_P3_1                   , RULL(0x8000046E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG4_P3_2                   , RULL(0x8000086E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG4_P3_2                   , RULL(0x8000086E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG4_P3_3                   , RULL(0x80000C6E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG4_P3_3                   , RULL(0x80000C6E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG4_P3_4                   , RULL(0x8000106E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG4_P3_4                   , RULL(0x8000106E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG5_P0_0                     , RULL(0x800000120701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG5_P0_0                   , RULL(0x800000120701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG5_P0_0                   , RULL(0x800000120801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG5_P0_1                     , RULL(0x800004120701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG5_P0_1                   , RULL(0x800004120701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG5_P0_1                   , RULL(0x800004120801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG5_P0_2                     , RULL(0x800008120701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG5_P0_2                   , RULL(0x800008120701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG5_P0_2                   , RULL(0x800008120801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG5_P0_3                     , RULL(0x80000C120701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG5_P0_3                   , RULL(0x80000C120701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG5_P0_3                   , RULL(0x80000C120801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_DIA_CONFIG5_P0_4                     , RULL(0x800010120701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_DIA_CONFIG5_P0_4                   , RULL(0x800010120701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_DIA_CONFIG5_P0_4                   , RULL(0x800010120801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG5_P1_0                   , RULL(0x800000120701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG5_P1_0                   , RULL(0x800000120801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG5_P1_1                   , RULL(0x800004120701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG5_P1_1                   , RULL(0x800004120801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG5_P1_2                   , RULL(0x800008120701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG5_P1_2                   , RULL(0x800008120801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG5_P1_3                   , RULL(0x80000C120701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG5_P1_3                   , RULL(0x80000C120801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_DIA_CONFIG5_P1_4                   , RULL(0x800010120701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_DIA_CONFIG5_P1_4                   , RULL(0x800010120801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG5_P2_0                   , RULL(0x800000120701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG5_P2_0                   , RULL(0x800000120801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG5_P2_1                   , RULL(0x800004120701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG5_P2_1                   , RULL(0x800004120801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG5_P2_2                   , RULL(0x800008120701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG5_P2_2                   , RULL(0x800008120801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG5_P2_3                   , RULL(0x80000C120701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG5_P2_3                   , RULL(0x80000C120801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_DIA_CONFIG5_P2_4                   , RULL(0x800010120701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_DIA_CONFIG5_P2_4                   , RULL(0x800010120801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG5_P3_0                   , RULL(0x8000001207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG5_P3_0                   , RULL(0x8000001208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG5_P3_1                   , RULL(0x8000041207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG5_P3_1                   , RULL(0x8000041208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG5_P3_2                   , RULL(0x8000081207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG5_P3_2                   , RULL(0x8000081208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG5_P3_3                   , RULL(0x80000C1207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG5_P3_3                   , RULL(0x80000C1208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_DIA_CONFIG5_P3_4                   , RULL(0x8000101207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_DIA_CONFIG5_P3_4                   , RULL(0x8000101208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_ERROR_MASK0_P0_0                     , RULL(0x800000150701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_ERROR_MASK0_P0_0                   , RULL(0x800000150701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_ERROR_MASK0_P0_0                   , RULL(0x800000150801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_ERROR_MASK0_P0_1                     , RULL(0x800004150701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_ERROR_MASK0_P0_1                   , RULL(0x800004150701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_ERROR_MASK0_P0_1                   , RULL(0x800004150801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_ERROR_MASK0_P0_2                     , RULL(0x800008150701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_ERROR_MASK0_P0_2                   , RULL(0x800008150701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_ERROR_MASK0_P0_2                   , RULL(0x800008150801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_ERROR_MASK0_P0_3                     , RULL(0x80000C150701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_ERROR_MASK0_P0_3                   , RULL(0x80000C150701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_ERROR_MASK0_P0_3                   , RULL(0x80000C150801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_ERROR_MASK0_P0_4                     , RULL(0x800010150701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_ERROR_MASK0_P0_4                   , RULL(0x800010150701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_ERROR_MASK0_P0_4                   , RULL(0x800010150801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_ERROR_MASK0_P1_0                   , RULL(0x800000150701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_ERROR_MASK0_P1_0                   , RULL(0x800000150801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_ERROR_MASK0_P1_1                   , RULL(0x800004150701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_ERROR_MASK0_P1_1                   , RULL(0x800004150801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_ERROR_MASK0_P1_2                   , RULL(0x800008150701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_ERROR_MASK0_P1_2                   , RULL(0x800008150801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_ERROR_MASK0_P1_3                   , RULL(0x80000C150701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_ERROR_MASK0_P1_3                   , RULL(0x80000C150801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_ERROR_MASK0_P1_4                   , RULL(0x800010150701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_ERROR_MASK0_P1_4                   , RULL(0x800010150801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_ERROR_MASK0_P2_0                   , RULL(0x800000150701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_ERROR_MASK0_P2_0                   , RULL(0x800000150801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_ERROR_MASK0_P2_1                   , RULL(0x800004150701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_ERROR_MASK0_P2_1                   , RULL(0x800004150801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_ERROR_MASK0_P2_2                   , RULL(0x800008150701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_ERROR_MASK0_P2_2                   , RULL(0x800008150801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_ERROR_MASK0_P2_3                   , RULL(0x80000C150701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_ERROR_MASK0_P2_3                   , RULL(0x80000C150801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_ERROR_MASK0_P2_4                   , RULL(0x800010150701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_ERROR_MASK0_P2_4                   , RULL(0x800010150801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_ERROR_MASK0_P3_0                   , RULL(0x8000001507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_ERROR_MASK0_P3_0                   , RULL(0x8000001508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_ERROR_MASK0_P3_1                   , RULL(0x8000041507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_ERROR_MASK0_P3_1                   , RULL(0x8000041508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_ERROR_MASK0_P3_2                   , RULL(0x8000081507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_ERROR_MASK0_P3_2                   , RULL(0x8000081508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_ERROR_MASK0_P3_3                   , RULL(0x80000C1507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_ERROR_MASK0_P3_3                   , RULL(0x80000C1508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_ERROR_MASK0_P3_4                   , RULL(0x8000101507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_ERROR_MASK0_P3_4                   , RULL(0x8000101508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS0_P0_0                     , RULL(0x8000000E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS0_P0_0                   , RULL(0x8000000E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS0_P0_0                   , RULL(0x8000000E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS0_P0_1                     , RULL(0x8000040E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS0_P0_1                   , RULL(0x8000040E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS0_P0_1                   , RULL(0x8000040E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS0_P0_2                     , RULL(0x8000080E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS0_P0_2                   , RULL(0x8000080E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS0_P0_2                   , RULL(0x8000080E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS0_P0_3                     , RULL(0x80000C0E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS0_P0_3                   , RULL(0x80000C0E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS0_P0_3                   , RULL(0x80000C0E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS0_P0_4                     , RULL(0x8000100E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS0_P0_4                   , RULL(0x8000100E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS0_P0_4                   , RULL(0x8000100E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS0_P1_0                   , RULL(0x8000000E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS0_P1_0                   , RULL(0x8000000E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS0_P1_1                   , RULL(0x8000040E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS0_P1_1                   , RULL(0x8000040E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS0_P1_2                   , RULL(0x8000080E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS0_P1_2                   , RULL(0x8000080E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS0_P1_3                   , RULL(0x80000C0E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS0_P1_3                   , RULL(0x80000C0E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS0_P1_4                   , RULL(0x8000100E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS0_P1_4                   , RULL(0x8000100E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS0_P2_0                   , RULL(0x8000000E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS0_P2_0                   , RULL(0x8000000E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS0_P2_1                   , RULL(0x8000040E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS0_P2_1                   , RULL(0x8000040E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS0_P2_2                   , RULL(0x8000080E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS0_P2_2                   , RULL(0x8000080E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS0_P2_3                   , RULL(0x80000C0E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS0_P2_3                   , RULL(0x80000C0E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS0_P2_4                   , RULL(0x8000100E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS0_P2_4                   , RULL(0x8000100E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS0_P3_0                   , RULL(0x8000000E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS0_P3_0                   , RULL(0x8000000E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS0_P3_1                   , RULL(0x8000040E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS0_P3_1                   , RULL(0x8000040E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS0_P3_2                   , RULL(0x8000080E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS0_P3_2                   , RULL(0x8000080E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS0_P3_3                   , RULL(0x80000C0E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS0_P3_3                   , RULL(0x80000C0E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS0_P3_4                   , RULL(0x8000100E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS0_P3_4                   , RULL(0x8000100E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS1_P0_0                     , RULL(0x8000000F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS1_P0_0                   , RULL(0x8000000F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS1_P0_0                   , RULL(0x8000000F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS1_P0_1                     , RULL(0x8000040F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS1_P0_1                   , RULL(0x8000040F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS1_P0_1                   , RULL(0x8000040F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS1_P0_2                     , RULL(0x8000080F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS1_P0_2                   , RULL(0x8000080F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS1_P0_2                   , RULL(0x8000080F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS1_P0_3                     , RULL(0x80000C0F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS1_P0_3                   , RULL(0x80000C0F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS1_P0_3                   , RULL(0x80000C0F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS1_P0_4                     , RULL(0x8000100F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS1_P0_4                   , RULL(0x8000100F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS1_P0_4                   , RULL(0x8000100F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS1_P1_0                   , RULL(0x8000000F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS1_P1_0                   , RULL(0x8000000F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS1_P1_1                   , RULL(0x8000040F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS1_P1_1                   , RULL(0x8000040F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS1_P1_2                   , RULL(0x8000080F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS1_P1_2                   , RULL(0x8000080F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS1_P1_3                   , RULL(0x80000C0F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS1_P1_3                   , RULL(0x80000C0F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS1_P1_4                   , RULL(0x8000100F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS1_P1_4                   , RULL(0x8000100F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS1_P2_0                   , RULL(0x8000000F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS1_P2_0                   , RULL(0x8000000F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS1_P2_1                   , RULL(0x8000040F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS1_P2_1                   , RULL(0x8000040F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS1_P2_2                   , RULL(0x8000080F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS1_P2_2                   , RULL(0x8000080F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS1_P2_3                   , RULL(0x80000C0F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS1_P2_3                   , RULL(0x80000C0F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS1_P2_4                   , RULL(0x8000100F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS1_P2_4                   , RULL(0x8000100F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS1_P3_0                   , RULL(0x8000000F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS1_P3_0                   , RULL(0x8000000F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS1_P3_1                   , RULL(0x8000040F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS1_P3_1                   , RULL(0x8000040F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS1_P3_2                   , RULL(0x8000080F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS1_P3_2                   , RULL(0x8000080F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS1_P3_3                   , RULL(0x80000C0F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS1_P3_3                   , RULL(0x80000C0F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS1_P3_4                   , RULL(0x8000100F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS1_P3_4                   , RULL(0x8000100F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS2_P0_0                     , RULL(0x800000100701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS2_P0_0                   , RULL(0x800000100701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS2_P0_0                   , RULL(0x800000100801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS2_P0_1                     , RULL(0x800004100701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS2_P0_1                   , RULL(0x800004100701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS2_P0_1                   , RULL(0x800004100801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS2_P0_2                     , RULL(0x800008100701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS2_P0_2                   , RULL(0x800008100701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS2_P0_2                   , RULL(0x800008100801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS2_P0_3                     , RULL(0x80000C100701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS2_P0_3                   , RULL(0x80000C100701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS2_P0_3                   , RULL(0x80000C100801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS2_P0_4                     , RULL(0x800010100701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS2_P0_4                   , RULL(0x800010100701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS2_P0_4                   , RULL(0x800010100801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS2_P1_0                   , RULL(0x800000100701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS2_P1_0                   , RULL(0x800000100801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS2_P1_1                   , RULL(0x800004100701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS2_P1_1                   , RULL(0x800004100801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS2_P1_2                   , RULL(0x800008100701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS2_P1_2                   , RULL(0x800008100801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS2_P1_3                   , RULL(0x80000C100701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS2_P1_3                   , RULL(0x80000C100801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS2_P1_4                   , RULL(0x800010100701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS2_P1_4                   , RULL(0x800010100801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS2_P2_0                   , RULL(0x800000100701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS2_P2_0                   , RULL(0x800000100801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS2_P2_1                   , RULL(0x800004100701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS2_P2_1                   , RULL(0x800004100801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS2_P2_2                   , RULL(0x800008100701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS2_P2_2                   , RULL(0x800008100801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS2_P2_3                   , RULL(0x80000C100701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS2_P2_3                   , RULL(0x80000C100801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS2_P2_4                   , RULL(0x800010100701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS2_P2_4                   , RULL(0x800010100801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS2_P3_0                   , RULL(0x8000001007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS2_P3_0                   , RULL(0x8000001008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS2_P3_1                   , RULL(0x8000041007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS2_P3_1                   , RULL(0x8000041008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS2_P3_2                   , RULL(0x8000081007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS2_P3_2                   , RULL(0x8000081008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS2_P3_3                   , RULL(0x80000C1007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS2_P3_3                   , RULL(0x80000C1008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS2_P3_4                   , RULL(0x8000101007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS2_P3_4                   , RULL(0x8000101008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS3_P0_0                     , RULL(0x800000110701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS3_P0_0                   , RULL(0x800000110701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS3_P0_0                   , RULL(0x800000110801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS3_P0_1                     , RULL(0x800004110701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS3_P0_1                   , RULL(0x800004110701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS3_P0_1                   , RULL(0x800004110801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS3_P0_2                     , RULL(0x800008110701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS3_P0_2                   , RULL(0x800008110701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS3_P0_2                   , RULL(0x800008110801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS3_P0_3                     , RULL(0x80000C110701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS3_P0_3                   , RULL(0x80000C110701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS3_P0_3                   , RULL(0x80000C110801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_LVL_STATUS3_P0_4                     , RULL(0x800010110701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_LVL_STATUS3_P0_4                   , RULL(0x800010110701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_LVL_STATUS3_P0_4                   , RULL(0x800010110801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS3_P1_0                   , RULL(0x800000110701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS3_P1_0                   , RULL(0x800000110801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS3_P1_1                   , RULL(0x800004110701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS3_P1_1                   , RULL(0x800004110801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS3_P1_2                   , RULL(0x800008110701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS3_P1_2                   , RULL(0x800008110801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS3_P1_3                   , RULL(0x80000C110701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS3_P1_3                   , RULL(0x80000C110801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_LVL_STATUS3_P1_4                   , RULL(0x800010110701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_LVL_STATUS3_P1_4                   , RULL(0x800010110801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS3_P2_0                   , RULL(0x800000110701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS3_P2_0                   , RULL(0x800000110801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS3_P2_1                   , RULL(0x800004110701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS3_P2_1                   , RULL(0x800004110801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS3_P2_2                   , RULL(0x800008110701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS3_P2_2                   , RULL(0x800008110801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS3_P2_3                   , RULL(0x80000C110701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS3_P2_3                   , RULL(0x80000C110801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_LVL_STATUS3_P2_4                   , RULL(0x800010110701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_LVL_STATUS3_P2_4                   , RULL(0x800010110801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS3_P3_0                   , RULL(0x8000001107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS3_P3_0                   , RULL(0x8000001108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS3_P3_1                   , RULL(0x8000041107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS3_P3_1                   , RULL(0x8000041108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS3_P3_2                   , RULL(0x8000081107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS3_P3_2                   , RULL(0x8000081108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS3_P3_3                   , RULL(0x80000C1107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS3_P3_3                   , RULL(0x80000C1108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_LVL_STATUS3_P3_4                   , RULL(0x8000101107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_LVL_STATUS3_P3_4                   , RULL(0x8000101108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_STATUS0_P0_0                         , RULL(0x800000140701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_STATUS0_P0_0                       , RULL(0x800000140701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_STATUS0_P0_0                       , RULL(0x800000140801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_STATUS0_P0_1                         , RULL(0x800004140701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_STATUS0_P0_1                       , RULL(0x800004140701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_STATUS0_P0_1                       , RULL(0x800004140801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_STATUS0_P0_2                         , RULL(0x800008140701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_STATUS0_P0_2                       , RULL(0x800008140701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_STATUS0_P0_2                       , RULL(0x800008140801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_STATUS0_P0_3                         , RULL(0x80000C140701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_STATUS0_P0_3                       , RULL(0x80000C140701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_STATUS0_P0_3                       , RULL(0x80000C140801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_STATUS0_P0_4                         , RULL(0x800010140701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_STATUS0_P0_4                       , RULL(0x800010140701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_STATUS0_P0_4                       , RULL(0x800010140801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_STATUS0_P1_0                       , RULL(0x800000140701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_STATUS0_P1_0                       , RULL(0x800000140801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_STATUS0_P1_1                       , RULL(0x800004140701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_STATUS0_P1_1                       , RULL(0x800004140801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_STATUS0_P1_2                       , RULL(0x800008140701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_STATUS0_P1_2                       , RULL(0x800008140801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_STATUS0_P1_3                       , RULL(0x80000C140701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_STATUS0_P1_3                       , RULL(0x80000C140801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_STATUS0_P1_4                       , RULL(0x800010140701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_STATUS0_P1_4                       , RULL(0x800010140801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_STATUS0_P2_0                       , RULL(0x800000140701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_STATUS0_P2_0                       , RULL(0x800000140801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_STATUS0_P2_1                       , RULL(0x800004140701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_STATUS0_P2_1                       , RULL(0x800004140801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_STATUS0_P2_2                       , RULL(0x800008140701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_STATUS0_P2_2                       , RULL(0x800008140801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_STATUS0_P2_3                       , RULL(0x80000C140701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_STATUS0_P2_3                       , RULL(0x80000C140801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_STATUS0_P2_4                       , RULL(0x800010140701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_STATUS0_P2_4                       , RULL(0x800010140801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_STATUS0_P3_0                       , RULL(0x8000001407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_STATUS0_P3_0                       , RULL(0x8000001408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_STATUS0_P3_1                       , RULL(0x8000041407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_STATUS0_P3_1                       , RULL(0x8000041408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_STATUS0_P3_2                       , RULL(0x8000081407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_STATUS0_P3_2                       , RULL(0x8000081408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_STATUS0_P3_3                       , RULL(0x80000C1407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_STATUS0_P3_3                       , RULL(0x80000C1408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_STATUS0_P3_4                       , RULL(0x8000101407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_STATUS0_P3_4                       , RULL(0x8000101408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P0_0                  , RULL(0x800000160701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P0_0                , RULL(0x800000160701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P0_0                , RULL(0x800000160801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P0_1                  , RULL(0x800004160701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P0_1                , RULL(0x800004160701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P0_1                , RULL(0x800004160801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P0_2                  , RULL(0x800008160701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P0_2                , RULL(0x800008160701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P0_2                , RULL(0x800008160801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P0_3                  , RULL(0x80000C160701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P0_3                , RULL(0x80000C160701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P0_3                , RULL(0x80000C160801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P0_4                  , RULL(0x800010160701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P0_4                , RULL(0x800010160701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P0_4                , RULL(0x800010160801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P1_0                , RULL(0x800000160701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P1_0                , RULL(0x800000160801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P1_1                , RULL(0x800004160701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P1_1                , RULL(0x800004160801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P1_2                , RULL(0x800008160701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P1_2                , RULL(0x800008160801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P1_3                , RULL(0x80000C160701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P1_3                , RULL(0x80000C160801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P1_4                , RULL(0x800010160701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P1_4                , RULL(0x800010160801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P2_0                , RULL(0x800000160701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P2_0                , RULL(0x800000160801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P2_1                , RULL(0x800004160701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P2_1                , RULL(0x800004160801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P2_2                , RULL(0x800008160701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P2_2                , RULL(0x800008160801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P2_3                , RULL(0x80000C160701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P2_3                , RULL(0x80000C160801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P2_4                , RULL(0x800010160701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P2_4                , RULL(0x800010160801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P3_0                , RULL(0x8000001607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P3_0                , RULL(0x8000001608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P3_1                , RULL(0x8000041607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P3_1                , RULL(0x8000041608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P3_2                , RULL(0x8000081607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P3_2                , RULL(0x8000081608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P3_3                , RULL(0x80000C1607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P3_3                , RULL(0x80000C1608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P3_4                , RULL(0x8000101607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_BYTE0_DAC_P3_4                , RULL(0x8000101608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P0_0                  , RULL(0x8000001F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P0_0                , RULL(0x8000001F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P0_0                , RULL(0x8000001F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P0_1                  , RULL(0x8000041F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P0_1                , RULL(0x8000041F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P0_1                , RULL(0x8000041F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P0_2                  , RULL(0x8000081F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P0_2                , RULL(0x8000081F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P0_2                , RULL(0x8000081F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P0_3                  , RULL(0x80000C1F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P0_3                , RULL(0x80000C1F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P0_3                , RULL(0x80000C1F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P0_4                  , RULL(0x8000101F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P0_4                , RULL(0x8000101F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P0_4                , RULL(0x8000101F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P1_0                , RULL(0x8000001F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P1_0                , RULL(0x8000001F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P1_1                , RULL(0x8000041F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P1_1                , RULL(0x8000041F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P1_2                , RULL(0x8000081F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P1_2                , RULL(0x8000081F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P1_3                , RULL(0x80000C1F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P1_3                , RULL(0x80000C1F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P1_4                , RULL(0x8000101F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P1_4                , RULL(0x8000101F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P2_0                , RULL(0x8000001F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P2_0                , RULL(0x8000001F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P2_1                , RULL(0x8000041F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P2_1                , RULL(0x8000041F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P2_2                , RULL(0x8000081F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P2_2                , RULL(0x8000081F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P2_3                , RULL(0x80000C1F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P2_3                , RULL(0x80000C1F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P2_4                , RULL(0x8000101F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P2_4                , RULL(0x8000101F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P3_0                , RULL(0x8000001F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P3_0                , RULL(0x8000001F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P3_1                , RULL(0x8000041F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P3_1                , RULL(0x8000041F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P3_2                , RULL(0x8000081F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P3_2                , RULL(0x8000081F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P3_3                , RULL(0x80000C1F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P3_3                , RULL(0x80000C1F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P3_4                , RULL(0x8000101F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_BYTE1_DAC_P3_4                , RULL(0x8000101F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_VREF_CAL_EN_P0_0                     , RULL(0x800000760701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_CAL_EN_P0_0                   , RULL(0x800000760701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_CAL_EN_P0_0                   , RULL(0x800000760801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_VREF_CAL_EN_P0_1                     , RULL(0x800004760701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_CAL_EN_P0_1                   , RULL(0x800004760701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_CAL_EN_P0_1                   , RULL(0x800004760801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_VREF_CAL_EN_P0_2                     , RULL(0x800008760701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_CAL_EN_P0_2                   , RULL(0x800008760701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_CAL_EN_P0_2                   , RULL(0x800008760801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_VREF_CAL_EN_P0_3                     , RULL(0x80000C760701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_CAL_EN_P0_3                   , RULL(0x80000C760701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_CAL_EN_P0_3                   , RULL(0x80000C760801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_VREF_CAL_EN_P0_4                     , RULL(0x800010760701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_CAL_EN_P0_4                   , RULL(0x800010760701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_CAL_EN_P0_4                   , RULL(0x800010760801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_CAL_EN_P1_0                   , RULL(0x800000760701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_CAL_EN_P1_0                   , RULL(0x800000760801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_CAL_EN_P1_1                   , RULL(0x800004760701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_CAL_EN_P1_1                   , RULL(0x800004760801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_CAL_EN_P1_2                   , RULL(0x800008760701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_CAL_EN_P1_2                   , RULL(0x800008760801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_CAL_EN_P1_3                   , RULL(0x80000C760701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_CAL_EN_P1_3                   , RULL(0x80000C760801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_CAL_EN_P1_4                   , RULL(0x800010760701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_CAL_EN_P1_4                   , RULL(0x800010760801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_CAL_EN_P2_0                   , RULL(0x800000760701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_CAL_EN_P2_0                   , RULL(0x800000760801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_CAL_EN_P2_1                   , RULL(0x800004760701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_CAL_EN_P2_1                   , RULL(0x800004760801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_CAL_EN_P2_2                   , RULL(0x800008760701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_CAL_EN_P2_2                   , RULL(0x800008760801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_CAL_EN_P2_3                   , RULL(0x80000C760701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_CAL_EN_P2_3                   , RULL(0x80000C760801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_CAL_EN_P2_4                   , RULL(0x800010760701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_CAL_EN_P2_4                   , RULL(0x800010760801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_CAL_EN_P3_0                   , RULL(0x8000007607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_CAL_EN_P3_0                   , RULL(0x8000007608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_CAL_EN_P3_1                   , RULL(0x8000047607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_CAL_EN_P3_1                   , RULL(0x8000047608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_CAL_EN_P3_2                   , RULL(0x8000087607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_CAL_EN_P3_2                   , RULL(0x8000087608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_CAL_EN_P3_3                   , RULL(0x80000C7607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_CAL_EN_P3_3                   , RULL(0x80000C7608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_CAL_EN_P3_4                   , RULL(0x8000107607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_CAL_EN_P3_4                   , RULL(0x8000107608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RD_VREF_CAL_ERROR_P0_0                  , RULL(0x8000007A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_CAL_ERROR_P0_0                , RULL(0x8000007A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_CAL_ERROR_P0_0                , RULL(0x8000007A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_VREF_CAL_ERROR_P0_1                  , RULL(0x8000047A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_CAL_ERROR_P0_1                , RULL(0x8000047A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_CAL_ERROR_P0_1                , RULL(0x8000047A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_VREF_CAL_ERROR_P0_2                  , RULL(0x8000087A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_CAL_ERROR_P0_2                , RULL(0x8000087A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_CAL_ERROR_P0_2                , RULL(0x8000087A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_VREF_CAL_ERROR_P0_3                  , RULL(0x80000C7A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_CAL_ERROR_P0_3                , RULL(0x80000C7A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_CAL_ERROR_P0_3                , RULL(0x80000C7A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_VREF_CAL_ERROR_P0_4                  , RULL(0x8000107A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_CAL_ERROR_P0_4                , RULL(0x8000107A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_CAL_ERROR_P0_4                , RULL(0x8000107A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_CAL_ERROR_P1_0                , RULL(0x8000007A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_CAL_ERROR_P1_0                , RULL(0x8000007A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_CAL_ERROR_P1_1                , RULL(0x8000047A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_CAL_ERROR_P1_1                , RULL(0x8000047A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_CAL_ERROR_P1_2                , RULL(0x8000087A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_CAL_ERROR_P1_2                , RULL(0x8000087A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_CAL_ERROR_P1_3                , RULL(0x80000C7A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_CAL_ERROR_P1_3                , RULL(0x80000C7A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_CAL_ERROR_P1_4                , RULL(0x8000107A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_CAL_ERROR_P1_4                , RULL(0x8000107A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_CAL_ERROR_P2_0                , RULL(0x8000007A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_CAL_ERROR_P2_0                , RULL(0x8000007A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_CAL_ERROR_P2_1                , RULL(0x8000047A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_CAL_ERROR_P2_1                , RULL(0x8000047A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_CAL_ERROR_P2_2                , RULL(0x8000087A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_CAL_ERROR_P2_2                , RULL(0x8000087A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_CAL_ERROR_P2_3                , RULL(0x80000C7A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_CAL_ERROR_P2_3                , RULL(0x80000C7A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_CAL_ERROR_P2_4                , RULL(0x8000107A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_CAL_ERROR_P2_4                , RULL(0x8000107A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_CAL_ERROR_P3_0                , RULL(0x8000007A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_CAL_ERROR_P3_0                , RULL(0x8000007A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_CAL_ERROR_P3_1                , RULL(0x8000047A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_CAL_ERROR_P3_1                , RULL(0x8000047A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_CAL_ERROR_P3_2                , RULL(0x8000087A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_CAL_ERROR_P3_2                , RULL(0x8000087A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_CAL_ERROR_P3_3                , RULL(0x80000C7A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_CAL_ERROR_P3_3                , RULL(0x80000C7A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_CAL_ERROR_P3_4                , RULL(0x8000107A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_CAL_ERROR_P3_4                , RULL(0x8000107A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P0_0               , RULL(0x800000F60701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P0_0             , RULL(0x800000F60701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P0_0             , RULL(0x800000F60801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P0_1               , RULL(0x800004F60701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P0_1             , RULL(0x800004F60701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P0_1             , RULL(0x800004F60801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P0_2               , RULL(0x800008F60701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P0_2             , RULL(0x800008F60701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P0_2             , RULL(0x800008F60801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P0_3               , RULL(0x80000CF60701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P0_3             , RULL(0x80000CF60701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P0_3             , RULL(0x80000CF60801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P0_4               , RULL(0x800010F60701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P0_4             , RULL(0x800010F60701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P0_4             , RULL(0x800010F60801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P1_0             , RULL(0x800000F60701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P1_0             , RULL(0x800000F60801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P1_1             , RULL(0x800004F60701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P1_1             , RULL(0x800004F60801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P1_2             , RULL(0x800008F60701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P1_2             , RULL(0x800008F60801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P1_3             , RULL(0x80000CF60701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P1_3             , RULL(0x80000CF60801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P1_4             , RULL(0x800010F60701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P1_4             , RULL(0x800010F60801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P2_0             , RULL(0x800000F60701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P2_0             , RULL(0x800000F60801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P2_1             , RULL(0x800004F60701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P2_1             , RULL(0x800004F60801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P2_2             , RULL(0x800008F60701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P2_2             , RULL(0x800008F60801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P2_3             , RULL(0x80000CF60701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P2_3             , RULL(0x80000CF60801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P2_4             , RULL(0x800010F60701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P2_4             , RULL(0x800010F60801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P3_0             , RULL(0x800000F607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P3_0             , RULL(0x800000F608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P3_1             , RULL(0x800004F607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P3_1             , RULL(0x800004F608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P3_2             , RULL(0x800008F607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P3_2             , RULL(0x800008F608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P3_3             , RULL(0x80000CF607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P3_3             , RULL(0x80000CF608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P3_4             , RULL(0x800010F607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_RD_VREF_DAC_COMP_OUT_P3_4             , RULL(0x800010F608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P0_0              , RULL(0x800000040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P0_0            , RULL(0x800000040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P0_0            , RULL(0x800000040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P0_1              , RULL(0x800004040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P0_1            , RULL(0x800004040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P0_1            , RULL(0x800004040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P0_2              , RULL(0x800008040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P0_2            , RULL(0x800008040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P0_2            , RULL(0x800008040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P0_3              , RULL(0x80000C040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P0_3            , RULL(0x80000C040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P0_3            , RULL(0x80000C040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P0_4              , RULL(0x800010040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P0_4            , RULL(0x800010040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P0_4            , RULL(0x800010040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P1_0            , RULL(0x800000040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P1_0            , RULL(0x800000040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P1_1            , RULL(0x800004040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P1_1            , RULL(0x800004040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P1_2            , RULL(0x800008040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P1_2            , RULL(0x800008040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P1_3            , RULL(0x80000C040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P1_3            , RULL(0x80000C040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P1_4            , RULL(0x800010040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P1_4            , RULL(0x800010040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P2_0            , RULL(0x800000040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P2_0            , RULL(0x800000040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P2_1            , RULL(0x800004040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P2_1            , RULL(0x800004040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P2_2            , RULL(0x800008040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P2_2            , RULL(0x800008040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P2_3            , RULL(0x80000C040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P2_3            , RULL(0x80000C040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P2_4            , RULL(0x800010040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P2_4            , RULL(0x800010040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P3_0            , RULL(0x8000000407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P3_0            , RULL(0x8000000408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P3_1            , RULL(0x8000040407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P3_1            , RULL(0x8000040408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P3_2            , RULL(0x8000080407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P3_2            , RULL(0x8000080408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P3_3            , RULL(0x80000C0407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P3_3            , RULL(0x80000C0408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P3_4            , RULL(0x8000100407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR0_P3_4            , RULL(0x8000100408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P0_0              , RULL(0x800001040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P0_0            , RULL(0x800001040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P0_0            , RULL(0x800001040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P0_1              , RULL(0x800005040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P0_1            , RULL(0x800005040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P0_1            , RULL(0x800005040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P0_2              , RULL(0x800009040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P0_2            , RULL(0x800009040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P0_2            , RULL(0x800009040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P0_3              , RULL(0x80000D040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P0_3            , RULL(0x80000D040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P0_3            , RULL(0x80000D040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P0_4              , RULL(0x800011040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P0_4            , RULL(0x800011040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P0_4            , RULL(0x800011040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P1_0            , RULL(0x800001040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P1_0            , RULL(0x800001040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P1_1            , RULL(0x800005040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P1_1            , RULL(0x800005040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P1_2            , RULL(0x800009040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P1_2            , RULL(0x800009040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P1_3            , RULL(0x80000D040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P1_3            , RULL(0x80000D040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P1_4            , RULL(0x800011040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P1_4            , RULL(0x800011040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P2_0            , RULL(0x800001040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P2_0            , RULL(0x800001040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P2_1            , RULL(0x800005040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P2_1            , RULL(0x800005040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P2_2            , RULL(0x800009040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P2_2            , RULL(0x800009040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P2_3            , RULL(0x80000D040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P2_3            , RULL(0x80000D040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P2_4            , RULL(0x800011040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P2_4            , RULL(0x800011040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P3_0            , RULL(0x8000010407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P3_0            , RULL(0x8000010408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P3_1            , RULL(0x8000050407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P3_1            , RULL(0x8000050408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P3_2            , RULL(0x8000090407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P3_2            , RULL(0x8000090408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P3_3            , RULL(0x80000D0407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P3_3            , RULL(0x80000D0408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P3_4            , RULL(0x8000110407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR1_P3_4            , RULL(0x8000110408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P0_0              , RULL(0x800002040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P0_0            , RULL(0x800002040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P0_0            , RULL(0x800002040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P0_1              , RULL(0x800006040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P0_1            , RULL(0x800006040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P0_1            , RULL(0x800006040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P0_2              , RULL(0x80000A040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P0_2            , RULL(0x80000A040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P0_2            , RULL(0x80000A040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P0_3              , RULL(0x80000E040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P0_3            , RULL(0x80000E040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P0_3            , RULL(0x80000E040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P0_4              , RULL(0x800012040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P0_4            , RULL(0x800012040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P0_4            , RULL(0x800012040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P1_0            , RULL(0x800002040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P1_0            , RULL(0x800002040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P1_1            , RULL(0x800006040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P1_1            , RULL(0x800006040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P1_2            , RULL(0x80000A040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P1_2            , RULL(0x80000A040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P1_3            , RULL(0x80000E040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P1_3            , RULL(0x80000E040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P1_4            , RULL(0x800012040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P1_4            , RULL(0x800012040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P2_0            , RULL(0x800002040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P2_0            , RULL(0x800002040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P2_1            , RULL(0x800006040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P2_1            , RULL(0x800006040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P2_2            , RULL(0x80000A040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P2_2            , RULL(0x80000A040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P2_3            , RULL(0x80000E040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P2_3            , RULL(0x80000E040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P2_4            , RULL(0x800012040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P2_4            , RULL(0x800012040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P3_0            , RULL(0x8000020407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P3_0            , RULL(0x8000020408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P3_1            , RULL(0x8000060407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P3_1            , RULL(0x8000060408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P3_2            , RULL(0x80000A0407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P3_2            , RULL(0x80000A0408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P3_3            , RULL(0x80000E0407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P3_3            , RULL(0x80000E0408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P3_4            , RULL(0x8000120407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR2_P3_4            , RULL(0x8000120408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P0_0              , RULL(0x800003040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P0_0            , RULL(0x800003040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P0_0            , RULL(0x800003040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P0_1              , RULL(0x800007040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P0_1            , RULL(0x800007040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P0_1            , RULL(0x800007040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P0_2              , RULL(0x80000B040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P0_2            , RULL(0x80000B040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P0_2            , RULL(0x80000B040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P0_3              , RULL(0x80000F040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P0_3            , RULL(0x80000F040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P0_3            , RULL(0x80000F040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P0_4              , RULL(0x800013040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P0_4            , RULL(0x800013040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P0_4            , RULL(0x800013040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P1_0            , RULL(0x800003040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P1_0            , RULL(0x800003040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P1_1            , RULL(0x800007040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P1_1            , RULL(0x800007040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P1_2            , RULL(0x80000B040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P1_2            , RULL(0x80000B040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P1_3            , RULL(0x80000F040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P1_3            , RULL(0x80000F040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P1_4            , RULL(0x800013040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P1_4            , RULL(0x800013040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P2_0            , RULL(0x800003040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P2_0            , RULL(0x800003040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P2_1            , RULL(0x800007040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P2_1            , RULL(0x800007040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P2_2            , RULL(0x80000B040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P2_2            , RULL(0x80000B040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P2_3            , RULL(0x80000F040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P2_3            , RULL(0x80000F040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P2_4            , RULL(0x800013040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P2_4            , RULL(0x800013040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P3_0            , RULL(0x8000030407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P3_0            , RULL(0x8000030408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P3_1            , RULL(0x8000070407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P3_1            , RULL(0x8000070408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P3_2            , RULL(0x80000B0407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P3_2            , RULL(0x80000B0408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P3_3            , RULL(0x80000F0407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P3_3            , RULL(0x80000F0408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P3_4            , RULL(0x8000130407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_CLOCK_RANK_PAIR3_P3_4            , RULL(0x8000130408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P0_0             , RULL(0x800000500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P0_0           , RULL(0x800000500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P0_0           , RULL(0x800000500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P0_1             , RULL(0x800004500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P0_1           , RULL(0x800004500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P0_1           , RULL(0x800004500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P0_2             , RULL(0x800008500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P0_2           , RULL(0x800008500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P0_2           , RULL(0x800008500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P0_3             , RULL(0x80000C500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P0_3           , RULL(0x80000C500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P0_3           , RULL(0x80000C500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P0_4             , RULL(0x800010500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P0_4           , RULL(0x800010500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P0_4           , RULL(0x800010500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P1_0           , RULL(0x800000500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P1_0           , RULL(0x800000500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P1_1           , RULL(0x800004500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P1_1           , RULL(0x800004500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P1_2           , RULL(0x800008500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P1_2           , RULL(0x800008500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P1_3           , RULL(0x80000C500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P1_3           , RULL(0x80000C500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P1_4           , RULL(0x800010500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P1_4           , RULL(0x800010500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P2_0           , RULL(0x800000500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P2_0           , RULL(0x800000500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P2_1           , RULL(0x800004500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P2_1           , RULL(0x800004500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P2_2           , RULL(0x800008500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P2_2           , RULL(0x800008500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P2_3           , RULL(0x80000C500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P2_3           , RULL(0x80000C500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P2_4           , RULL(0x800010500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P2_4           , RULL(0x800010500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P3_0           , RULL(0x8000005007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P3_0           , RULL(0x8000005008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P3_1           , RULL(0x8000045007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P3_1           , RULL(0x8000045008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P3_2           , RULL(0x8000085007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P3_2           , RULL(0x8000085008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P3_3           , RULL(0x80000C5007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P3_3           , RULL(0x80000C5008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P3_4           , RULL(0x8000105007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR0_P3_4           , RULL(0x8000105008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P0_0             , RULL(0x800001500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P0_0           , RULL(0x800001500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P0_0           , RULL(0x800001500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P0_1             , RULL(0x800005500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P0_1           , RULL(0x800005500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P0_1           , RULL(0x800005500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P0_2             , RULL(0x800009500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P0_2           , RULL(0x800009500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P0_2           , RULL(0x800009500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P0_3             , RULL(0x80000D500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P0_3           , RULL(0x80000D500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P0_3           , RULL(0x80000D500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P0_4             , RULL(0x800011500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P0_4           , RULL(0x800011500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P0_4           , RULL(0x800011500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P1_0           , RULL(0x800001500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P1_0           , RULL(0x800001500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P1_1           , RULL(0x800005500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P1_1           , RULL(0x800005500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P1_2           , RULL(0x800009500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P1_2           , RULL(0x800009500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P1_3           , RULL(0x80000D500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P1_3           , RULL(0x80000D500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P1_4           , RULL(0x800011500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P1_4           , RULL(0x800011500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P2_0           , RULL(0x800001500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P2_0           , RULL(0x800001500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P2_1           , RULL(0x800005500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P2_1           , RULL(0x800005500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P2_2           , RULL(0x800009500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P2_2           , RULL(0x800009500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P2_3           , RULL(0x80000D500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P2_3           , RULL(0x80000D500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P2_4           , RULL(0x800011500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P2_4           , RULL(0x800011500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P3_0           , RULL(0x8000015007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P3_0           , RULL(0x8000015008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P3_1           , RULL(0x8000055007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P3_1           , RULL(0x8000055008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P3_2           , RULL(0x8000095007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P3_2           , RULL(0x8000095008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P3_3           , RULL(0x80000D5007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P3_3           , RULL(0x80000D5008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P3_4           , RULL(0x8000115007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR1_P3_4           , RULL(0x8000115008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P0_0             , RULL(0x800002500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P0_0           , RULL(0x800002500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P0_0           , RULL(0x800002500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P0_1             , RULL(0x800006500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P0_1           , RULL(0x800006500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P0_1           , RULL(0x800006500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P0_2             , RULL(0x80000A500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P0_2           , RULL(0x80000A500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P0_2           , RULL(0x80000A500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P0_3             , RULL(0x80000E500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P0_3           , RULL(0x80000E500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P0_3           , RULL(0x80000E500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P0_4             , RULL(0x800012500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P0_4           , RULL(0x800012500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P0_4           , RULL(0x800012500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P1_0           , RULL(0x800002500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P1_0           , RULL(0x800002500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P1_1           , RULL(0x800006500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P1_1           , RULL(0x800006500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P1_2           , RULL(0x80000A500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P1_2           , RULL(0x80000A500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P1_3           , RULL(0x80000E500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P1_3           , RULL(0x80000E500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P1_4           , RULL(0x800012500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P1_4           , RULL(0x800012500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P2_0           , RULL(0x800002500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P2_0           , RULL(0x800002500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P2_1           , RULL(0x800006500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P2_1           , RULL(0x800006500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P2_2           , RULL(0x80000A500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P2_2           , RULL(0x80000A500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P2_3           , RULL(0x80000E500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P2_3           , RULL(0x80000E500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P2_4           , RULL(0x800012500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P2_4           , RULL(0x800012500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P3_0           , RULL(0x8000025007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P3_0           , RULL(0x8000025008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P3_1           , RULL(0x8000065007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P3_1           , RULL(0x8000065008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P3_2           , RULL(0x80000A5007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P3_2           , RULL(0x80000A5008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P3_3           , RULL(0x80000E5007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P3_3           , RULL(0x80000E5008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P3_4           , RULL(0x8000125007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR2_P3_4           , RULL(0x8000125008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P0_0             , RULL(0x800003500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P0_0           , RULL(0x800003500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P0_0           , RULL(0x800003500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P0_1             , RULL(0x800007500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P0_1           , RULL(0x800007500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P0_1           , RULL(0x800007500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P0_2             , RULL(0x80000B500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P0_2           , RULL(0x80000B500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P0_2           , RULL(0x80000B500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P0_3             , RULL(0x80000F500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P0_3           , RULL(0x80000F500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P0_3           , RULL(0x80000F500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P0_4             , RULL(0x800013500701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P0_4           , RULL(0x800013500701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P0_4           , RULL(0x800013500801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P1_0           , RULL(0x800003500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P1_0           , RULL(0x800003500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P1_1           , RULL(0x800007500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P1_1           , RULL(0x800007500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P1_2           , RULL(0x80000B500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P1_2           , RULL(0x80000B500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P1_3           , RULL(0x80000F500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P1_3           , RULL(0x80000F500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P1_4           , RULL(0x800013500701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P1_4           , RULL(0x800013500801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P2_0           , RULL(0x800003500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P2_0           , RULL(0x800003500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P2_1           , RULL(0x800007500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P2_1           , RULL(0x800007500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P2_2           , RULL(0x80000B500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P2_2           , RULL(0x80000B500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P2_3           , RULL(0x80000F500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P2_3           , RULL(0x80000F500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P2_4           , RULL(0x800013500701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P2_4           , RULL(0x800013500801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P3_0           , RULL(0x8000035007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P3_0           , RULL(0x8000035008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P3_1           , RULL(0x8000075007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P3_1           , RULL(0x8000075008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P3_2           , RULL(0x80000B5007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P3_2           , RULL(0x80000B5008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P3_3           , RULL(0x80000F5007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P3_3           , RULL(0x80000F5008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P3_4           , RULL(0x8000135007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY0_RANK_PAIR3_P3_4           , RULL(0x8000135008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P0_0             , RULL(0x800000510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P0_0           , RULL(0x800000510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P0_0           , RULL(0x800000510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P0_1             , RULL(0x800004510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P0_1           , RULL(0x800004510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P0_1           , RULL(0x800004510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P0_2             , RULL(0x800008510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P0_2           , RULL(0x800008510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P0_2           , RULL(0x800008510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P0_3             , RULL(0x80000C510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P0_3           , RULL(0x80000C510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P0_3           , RULL(0x80000C510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P0_4             , RULL(0x800010510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P0_4           , RULL(0x800010510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P0_4           , RULL(0x800010510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P1_0           , RULL(0x800000510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P1_0           , RULL(0x800000510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P1_1           , RULL(0x800004510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P1_1           , RULL(0x800004510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P1_2           , RULL(0x800008510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P1_2           , RULL(0x800008510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P1_3           , RULL(0x80000C510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P1_3           , RULL(0x80000C510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P1_4           , RULL(0x800010510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P1_4           , RULL(0x800010510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P2_0           , RULL(0x800000510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P2_0           , RULL(0x800000510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P2_1           , RULL(0x800004510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P2_1           , RULL(0x800004510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P2_2           , RULL(0x800008510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P2_2           , RULL(0x800008510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P2_3           , RULL(0x80000C510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P2_3           , RULL(0x80000C510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P2_4           , RULL(0x800010510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P2_4           , RULL(0x800010510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P3_0           , RULL(0x8000005107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P3_0           , RULL(0x8000005108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P3_1           , RULL(0x8000045107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P3_1           , RULL(0x8000045108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P3_2           , RULL(0x8000085107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P3_2           , RULL(0x8000085108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P3_3           , RULL(0x80000C5107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P3_3           , RULL(0x80000C5108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P3_4           , RULL(0x8000105107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR0_P3_4           , RULL(0x8000105108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P0_0             , RULL(0x800001510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P0_0           , RULL(0x800001510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P0_0           , RULL(0x800001510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P0_1             , RULL(0x800005510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P0_1           , RULL(0x800005510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P0_1           , RULL(0x800005510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P0_2             , RULL(0x800009510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P0_2           , RULL(0x800009510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P0_2           , RULL(0x800009510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P0_3             , RULL(0x80000D510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P0_3           , RULL(0x80000D510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P0_3           , RULL(0x80000D510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P0_4             , RULL(0x800011510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P0_4           , RULL(0x800011510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P0_4           , RULL(0x800011510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P1_0           , RULL(0x800001510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P1_0           , RULL(0x800001510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P1_1           , RULL(0x800005510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P1_1           , RULL(0x800005510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P1_2           , RULL(0x800009510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P1_2           , RULL(0x800009510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P1_3           , RULL(0x80000D510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P1_3           , RULL(0x80000D510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P1_4           , RULL(0x800011510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P1_4           , RULL(0x800011510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P2_0           , RULL(0x800001510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P2_0           , RULL(0x800001510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P2_1           , RULL(0x800005510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P2_1           , RULL(0x800005510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P2_2           , RULL(0x800009510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P2_2           , RULL(0x800009510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P2_3           , RULL(0x80000D510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P2_3           , RULL(0x80000D510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P2_4           , RULL(0x800011510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P2_4           , RULL(0x800011510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P3_0           , RULL(0x8000015107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P3_0           , RULL(0x8000015108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P3_1           , RULL(0x8000055107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P3_1           , RULL(0x8000055108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P3_2           , RULL(0x8000095107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P3_2           , RULL(0x8000095108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P3_3           , RULL(0x80000D5107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P3_3           , RULL(0x80000D5108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P3_4           , RULL(0x8000115107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR1_P3_4           , RULL(0x8000115108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P0_0             , RULL(0x800002510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P0_0           , RULL(0x800002510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P0_0           , RULL(0x800002510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P0_1             , RULL(0x800006510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P0_1           , RULL(0x800006510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P0_1           , RULL(0x800006510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P0_2             , RULL(0x80000A510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P0_2           , RULL(0x80000A510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P0_2           , RULL(0x80000A510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P0_3             , RULL(0x80000E510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P0_3           , RULL(0x80000E510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P0_3           , RULL(0x80000E510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P0_4             , RULL(0x800012510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P0_4           , RULL(0x800012510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P0_4           , RULL(0x800012510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P1_0           , RULL(0x800002510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P1_0           , RULL(0x800002510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P1_1           , RULL(0x800006510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P1_1           , RULL(0x800006510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P1_2           , RULL(0x80000A510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P1_2           , RULL(0x80000A510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P1_3           , RULL(0x80000E510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P1_3           , RULL(0x80000E510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P1_4           , RULL(0x800012510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P1_4           , RULL(0x800012510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P2_0           , RULL(0x800002510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P2_0           , RULL(0x800002510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P2_1           , RULL(0x800006510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P2_1           , RULL(0x800006510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P2_2           , RULL(0x80000A510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P2_2           , RULL(0x80000A510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P2_3           , RULL(0x80000E510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P2_3           , RULL(0x80000E510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P2_4           , RULL(0x800012510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P2_4           , RULL(0x800012510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P3_0           , RULL(0x8000025107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P3_0           , RULL(0x8000025108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P3_1           , RULL(0x8000065107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P3_1           , RULL(0x8000065108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P3_2           , RULL(0x80000A5107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P3_2           , RULL(0x80000A5108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P3_3           , RULL(0x80000E5107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P3_3           , RULL(0x80000E5108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P3_4           , RULL(0x8000125107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR2_P3_4           , RULL(0x8000125108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P0_0             , RULL(0x800003510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P0_0           , RULL(0x800003510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P0_0           , RULL(0x800003510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P0_1             , RULL(0x800007510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P0_1           , RULL(0x800007510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P0_1           , RULL(0x800007510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P0_2             , RULL(0x80000B510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P0_2           , RULL(0x80000B510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P0_2           , RULL(0x80000B510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P0_3             , RULL(0x80000F510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P0_3           , RULL(0x80000F510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P0_3           , RULL(0x80000F510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P0_4             , RULL(0x800013510701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P0_4           , RULL(0x800013510701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P0_4           , RULL(0x800013510801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P1_0           , RULL(0x800003510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P1_0           , RULL(0x800003510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P1_1           , RULL(0x800007510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P1_1           , RULL(0x800007510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P1_2           , RULL(0x80000B510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P1_2           , RULL(0x80000B510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P1_3           , RULL(0x80000F510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P1_3           , RULL(0x80000F510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P1_4           , RULL(0x800013510701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P1_4           , RULL(0x800013510801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P2_0           , RULL(0x800003510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P2_0           , RULL(0x800003510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P2_1           , RULL(0x800007510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P2_1           , RULL(0x800007510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P2_2           , RULL(0x80000B510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P2_2           , RULL(0x80000B510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P2_3           , RULL(0x80000F510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P2_3           , RULL(0x80000F510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P2_4           , RULL(0x800013510701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P2_4           , RULL(0x800013510801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P3_0           , RULL(0x8000035107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P3_0           , RULL(0x8000035108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P3_1           , RULL(0x8000075107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P3_1           , RULL(0x8000075108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P3_2           , RULL(0x80000B5107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P3_2           , RULL(0x80000B5108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P3_3           , RULL(0x80000F5107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P3_3           , RULL(0x80000F5108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P3_4           , RULL(0x8000135107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY1_RANK_PAIR3_P3_4           , RULL(0x8000135108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P0_0             , RULL(0x800000520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P0_0           , RULL(0x800000520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P0_0           , RULL(0x800000520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P0_1             , RULL(0x800004520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P0_1           , RULL(0x800004520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P0_1           , RULL(0x800004520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P0_2             , RULL(0x800008520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P0_2           , RULL(0x800008520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P0_2           , RULL(0x800008520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P0_3             , RULL(0x80000C520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P0_3           , RULL(0x80000C520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P0_3           , RULL(0x80000C520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P0_4             , RULL(0x800010520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P0_4           , RULL(0x800010520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P0_4           , RULL(0x800010520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P1_0           , RULL(0x800000520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P1_0           , RULL(0x800000520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P1_1           , RULL(0x800004520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P1_1           , RULL(0x800004520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P1_2           , RULL(0x800008520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P1_2           , RULL(0x800008520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P1_3           , RULL(0x80000C520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P1_3           , RULL(0x80000C520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P1_4           , RULL(0x800010520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P1_4           , RULL(0x800010520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P2_0           , RULL(0x800000520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P2_0           , RULL(0x800000520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P2_1           , RULL(0x800004520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P2_1           , RULL(0x800004520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P2_2           , RULL(0x800008520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P2_2           , RULL(0x800008520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P2_3           , RULL(0x80000C520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P2_3           , RULL(0x80000C520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P2_4           , RULL(0x800010520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P2_4           , RULL(0x800010520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P3_0           , RULL(0x8000005207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P3_0           , RULL(0x8000005208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P3_1           , RULL(0x8000045207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P3_1           , RULL(0x8000045208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P3_2           , RULL(0x8000085207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P3_2           , RULL(0x8000085208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P3_3           , RULL(0x80000C5207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P3_3           , RULL(0x80000C5208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P3_4           , RULL(0x8000105207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR0_P3_4           , RULL(0x8000105208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P0_0             , RULL(0x800001520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P0_0           , RULL(0x800001520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P0_0           , RULL(0x800001520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P0_1             , RULL(0x800005520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P0_1           , RULL(0x800005520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P0_1           , RULL(0x800005520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P0_2             , RULL(0x800009520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P0_2           , RULL(0x800009520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P0_2           , RULL(0x800009520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P0_3             , RULL(0x80000D520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P0_3           , RULL(0x80000D520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P0_3           , RULL(0x80000D520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P0_4             , RULL(0x800011520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P0_4           , RULL(0x800011520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P0_4           , RULL(0x800011520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P1_0           , RULL(0x800001520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P1_0           , RULL(0x800001520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P1_1           , RULL(0x800005520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P1_1           , RULL(0x800005520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P1_2           , RULL(0x800009520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P1_2           , RULL(0x800009520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P1_3           , RULL(0x80000D520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P1_3           , RULL(0x80000D520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P1_4           , RULL(0x800011520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P1_4           , RULL(0x800011520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P2_0           , RULL(0x800001520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P2_0           , RULL(0x800001520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P2_1           , RULL(0x800005520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P2_1           , RULL(0x800005520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P2_2           , RULL(0x800009520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P2_2           , RULL(0x800009520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P2_3           , RULL(0x80000D520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P2_3           , RULL(0x80000D520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P2_4           , RULL(0x800011520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P2_4           , RULL(0x800011520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P3_0           , RULL(0x8000015207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P3_0           , RULL(0x8000015208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P3_1           , RULL(0x8000055207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P3_1           , RULL(0x8000055208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P3_2           , RULL(0x8000095207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P3_2           , RULL(0x8000095208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P3_3           , RULL(0x80000D5207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P3_3           , RULL(0x80000D5208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P3_4           , RULL(0x8000115207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR1_P3_4           , RULL(0x8000115208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P0_0             , RULL(0x800002520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P0_0           , RULL(0x800002520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P0_0           , RULL(0x800002520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P0_1             , RULL(0x800006520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P0_1           , RULL(0x800006520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P0_1           , RULL(0x800006520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P0_2             , RULL(0x80000A520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P0_2           , RULL(0x80000A520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P0_2           , RULL(0x80000A520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P0_3             , RULL(0x80000E520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P0_3           , RULL(0x80000E520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P0_3           , RULL(0x80000E520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P0_4             , RULL(0x800012520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P0_4           , RULL(0x800012520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P0_4           , RULL(0x800012520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P1_0           , RULL(0x800002520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P1_0           , RULL(0x800002520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P1_1           , RULL(0x800006520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P1_1           , RULL(0x800006520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P1_2           , RULL(0x80000A520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P1_2           , RULL(0x80000A520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P1_3           , RULL(0x80000E520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P1_3           , RULL(0x80000E520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P1_4           , RULL(0x800012520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P1_4           , RULL(0x800012520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P2_0           , RULL(0x800002520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P2_0           , RULL(0x800002520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P2_1           , RULL(0x800006520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P2_1           , RULL(0x800006520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P2_2           , RULL(0x80000A520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P2_2           , RULL(0x80000A520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P2_3           , RULL(0x80000E520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P2_3           , RULL(0x80000E520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P2_4           , RULL(0x800012520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P2_4           , RULL(0x800012520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P3_0           , RULL(0x8000025207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P3_0           , RULL(0x8000025208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P3_1           , RULL(0x8000065207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P3_1           , RULL(0x8000065208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P3_2           , RULL(0x80000A5207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P3_2           , RULL(0x80000A5208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P3_3           , RULL(0x80000E5207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P3_3           , RULL(0x80000E5208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P3_4           , RULL(0x8000125207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR2_P3_4           , RULL(0x8000125208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P0_0             , RULL(0x800003520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P0_0           , RULL(0x800003520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P0_0           , RULL(0x800003520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P0_1             , RULL(0x800007520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P0_1           , RULL(0x800007520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P0_1           , RULL(0x800007520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P0_2             , RULL(0x80000B520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P0_2           , RULL(0x80000B520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P0_2           , RULL(0x80000B520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P0_3             , RULL(0x80000F520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P0_3           , RULL(0x80000F520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P0_3           , RULL(0x80000F520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P0_4             , RULL(0x800013520701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P0_4           , RULL(0x800013520701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P0_4           , RULL(0x800013520801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P1_0           , RULL(0x800003520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P1_0           , RULL(0x800003520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P1_1           , RULL(0x800007520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P1_1           , RULL(0x800007520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P1_2           , RULL(0x80000B520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P1_2           , RULL(0x80000B520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P1_3           , RULL(0x80000F520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P1_3           , RULL(0x80000F520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P1_4           , RULL(0x800013520701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P1_4           , RULL(0x800013520801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P2_0           , RULL(0x800003520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P2_0           , RULL(0x800003520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P2_1           , RULL(0x800007520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P2_1           , RULL(0x800007520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P2_2           , RULL(0x80000B520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P2_2           , RULL(0x80000B520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P2_3           , RULL(0x80000F520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P2_3           , RULL(0x80000F520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P2_4           , RULL(0x800013520701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P2_4           , RULL(0x800013520801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P3_0           , RULL(0x8000035207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P3_0           , RULL(0x8000035208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P3_1           , RULL(0x8000075207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P3_1           , RULL(0x8000075208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P3_2           , RULL(0x80000B5207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P3_2           , RULL(0x80000B5208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P3_3           , RULL(0x80000F5207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P3_3           , RULL(0x80000F5208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P3_4           , RULL(0x8000135207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY2_RANK_PAIR3_P3_4           , RULL(0x8000135208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P0_0             , RULL(0x800000530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P0_0           , RULL(0x800000530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P0_0           , RULL(0x800000530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P0_1             , RULL(0x800004530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P0_1           , RULL(0x800004530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P0_1           , RULL(0x800004530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P0_2             , RULL(0x800008530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P0_2           , RULL(0x800008530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P0_2           , RULL(0x800008530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P0_3             , RULL(0x80000C530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P0_3           , RULL(0x80000C530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P0_3           , RULL(0x80000C530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P0_4             , RULL(0x800010530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P0_4           , RULL(0x800010530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P0_4           , RULL(0x800010530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P1_0           , RULL(0x800000530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P1_0           , RULL(0x800000530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P1_1           , RULL(0x800004530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P1_1           , RULL(0x800004530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P1_2           , RULL(0x800008530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P1_2           , RULL(0x800008530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P1_3           , RULL(0x80000C530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P1_3           , RULL(0x80000C530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P1_4           , RULL(0x800010530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P1_4           , RULL(0x800010530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P2_0           , RULL(0x800000530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P2_0           , RULL(0x800000530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P2_1           , RULL(0x800004530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P2_1           , RULL(0x800004530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P2_2           , RULL(0x800008530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P2_2           , RULL(0x800008530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P2_3           , RULL(0x80000C530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P2_3           , RULL(0x80000C530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P2_4           , RULL(0x800010530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P2_4           , RULL(0x800010530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P3_0           , RULL(0x8000005307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P3_0           , RULL(0x8000005308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P3_1           , RULL(0x8000045307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P3_1           , RULL(0x8000045308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P3_2           , RULL(0x8000085307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P3_2           , RULL(0x8000085308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P3_3           , RULL(0x80000C5307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P3_3           , RULL(0x80000C5308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P3_4           , RULL(0x8000105307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR0_P3_4           , RULL(0x8000105308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P0_0             , RULL(0x800001530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P0_0           , RULL(0x800001530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P0_0           , RULL(0x800001530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P0_1             , RULL(0x800005530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P0_1           , RULL(0x800005530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P0_1           , RULL(0x800005530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P0_2             , RULL(0x800009530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P0_2           , RULL(0x800009530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P0_2           , RULL(0x800009530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P0_3             , RULL(0x80000D530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P0_3           , RULL(0x80000D530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P0_3           , RULL(0x80000D530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P0_4             , RULL(0x800011530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P0_4           , RULL(0x800011530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P0_4           , RULL(0x800011530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P1_0           , RULL(0x800001530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P1_0           , RULL(0x800001530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P1_1           , RULL(0x800005530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P1_1           , RULL(0x800005530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P1_2           , RULL(0x800009530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P1_2           , RULL(0x800009530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P1_3           , RULL(0x80000D530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P1_3           , RULL(0x80000D530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P1_4           , RULL(0x800011530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P1_4           , RULL(0x800011530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P2_0           , RULL(0x800001530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P2_0           , RULL(0x800001530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P2_1           , RULL(0x800005530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P2_1           , RULL(0x800005530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P2_2           , RULL(0x800009530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P2_2           , RULL(0x800009530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P2_3           , RULL(0x80000D530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P2_3           , RULL(0x80000D530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P2_4           , RULL(0x800011530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P2_4           , RULL(0x800011530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P3_0           , RULL(0x8000015307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P3_0           , RULL(0x8000015308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P3_1           , RULL(0x8000055307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P3_1           , RULL(0x8000055308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P3_2           , RULL(0x8000095307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P3_2           , RULL(0x8000095308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P3_3           , RULL(0x80000D5307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P3_3           , RULL(0x80000D5308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P3_4           , RULL(0x8000115307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR1_P3_4           , RULL(0x8000115308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P0_0             , RULL(0x800002530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P0_0           , RULL(0x800002530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P0_0           , RULL(0x800002530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P0_1             , RULL(0x800006530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P0_1           , RULL(0x800006530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P0_1           , RULL(0x800006530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P0_2             , RULL(0x80000A530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P0_2           , RULL(0x80000A530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P0_2           , RULL(0x80000A530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P0_3             , RULL(0x80000E530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P0_3           , RULL(0x80000E530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P0_3           , RULL(0x80000E530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P0_4             , RULL(0x800012530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P0_4           , RULL(0x800012530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P0_4           , RULL(0x800012530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P1_0           , RULL(0x800002530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P1_0           , RULL(0x800002530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P1_1           , RULL(0x800006530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P1_1           , RULL(0x800006530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P1_2           , RULL(0x80000A530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P1_2           , RULL(0x80000A530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P1_3           , RULL(0x80000E530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P1_3           , RULL(0x80000E530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P1_4           , RULL(0x800012530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P1_4           , RULL(0x800012530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P2_0           , RULL(0x800002530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P2_0           , RULL(0x800002530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P2_1           , RULL(0x800006530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P2_1           , RULL(0x800006530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P2_2           , RULL(0x80000A530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P2_2           , RULL(0x80000A530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P2_3           , RULL(0x80000E530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P2_3           , RULL(0x80000E530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P2_4           , RULL(0x800012530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P2_4           , RULL(0x800012530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P3_0           , RULL(0x8000025307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P3_0           , RULL(0x8000025308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P3_1           , RULL(0x8000065307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P3_1           , RULL(0x8000065308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P3_2           , RULL(0x80000A5307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P3_2           , RULL(0x80000A5308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P3_3           , RULL(0x80000E5307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P3_3           , RULL(0x80000E5308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P3_4           , RULL(0x8000125307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR2_P3_4           , RULL(0x8000125308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P0_0             , RULL(0x800003530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P0_0           , RULL(0x800003530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P0_0           , RULL(0x800003530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P0_1             , RULL(0x800007530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P0_1           , RULL(0x800007530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P0_1           , RULL(0x800007530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P0_2             , RULL(0x80000B530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P0_2           , RULL(0x80000B530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P0_2           , RULL(0x80000B530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P0_3             , RULL(0x80000F530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P0_3           , RULL(0x80000F530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P0_3           , RULL(0x80000F530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P0_4             , RULL(0x800013530701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P0_4           , RULL(0x800013530701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P0_4           , RULL(0x800013530801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P1_0           , RULL(0x800003530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P1_0           , RULL(0x800003530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P1_1           , RULL(0x800007530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P1_1           , RULL(0x800007530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P1_2           , RULL(0x80000B530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P1_2           , RULL(0x80000B530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P1_3           , RULL(0x80000F530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P1_3           , RULL(0x80000F530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P1_4           , RULL(0x800013530701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P1_4           , RULL(0x800013530801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P2_0           , RULL(0x800003530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P2_0           , RULL(0x800003530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P2_1           , RULL(0x800007530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P2_1           , RULL(0x800007530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P2_2           , RULL(0x80000B530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P2_2           , RULL(0x80000B530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P2_3           , RULL(0x80000F530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P2_3           , RULL(0x80000F530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P2_4           , RULL(0x800013530701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P2_4           , RULL(0x800013530801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P3_0           , RULL(0x8000035307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P3_0           , RULL(0x8000035308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P3_1           , RULL(0x8000075307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P3_1           , RULL(0x8000075308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P3_2           , RULL(0x80000B5307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P3_2           , RULL(0x80000B5308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P3_3           , RULL(0x80000F5307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P3_3           , RULL(0x80000F5308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P3_4           , RULL(0x8000135307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY3_RANK_PAIR3_P3_4           , RULL(0x8000135308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P0_0             , RULL(0x800000540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P0_0           , RULL(0x800000540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P0_0           , RULL(0x800000540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P0_1             , RULL(0x800004540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P0_1           , RULL(0x800004540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P0_1           , RULL(0x800004540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P0_2             , RULL(0x800008540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P0_2           , RULL(0x800008540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P0_2           , RULL(0x800008540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P0_3             , RULL(0x80000C540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P0_3           , RULL(0x80000C540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P0_3           , RULL(0x80000C540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P0_4             , RULL(0x800010540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P0_4           , RULL(0x800010540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P0_4           , RULL(0x800010540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P1_0           , RULL(0x800000540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P1_0           , RULL(0x800000540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P1_1           , RULL(0x800004540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P1_1           , RULL(0x800004540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P1_2           , RULL(0x800008540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P1_2           , RULL(0x800008540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P1_3           , RULL(0x80000C540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P1_3           , RULL(0x80000C540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P1_4           , RULL(0x800010540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P1_4           , RULL(0x800010540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P2_0           , RULL(0x800000540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P2_0           , RULL(0x800000540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P2_1           , RULL(0x800004540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P2_1           , RULL(0x800004540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P2_2           , RULL(0x800008540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P2_2           , RULL(0x800008540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P2_3           , RULL(0x80000C540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P2_3           , RULL(0x80000C540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P2_4           , RULL(0x800010540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P2_4           , RULL(0x800010540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P3_0           , RULL(0x8000005407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P3_0           , RULL(0x8000005408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P3_1           , RULL(0x8000045407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P3_1           , RULL(0x8000045408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P3_2           , RULL(0x8000085407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P3_2           , RULL(0x8000085408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P3_3           , RULL(0x80000C5407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P3_3           , RULL(0x80000C5408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P3_4           , RULL(0x8000105407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR0_P3_4           , RULL(0x8000105408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P0_0             , RULL(0x800001540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P0_0           , RULL(0x800001540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P0_0           , RULL(0x800001540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P0_1             , RULL(0x800005540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P0_1           , RULL(0x800005540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P0_1           , RULL(0x800005540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P0_2             , RULL(0x800009540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P0_2           , RULL(0x800009540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P0_2           , RULL(0x800009540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P0_3             , RULL(0x80000D540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P0_3           , RULL(0x80000D540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P0_3           , RULL(0x80000D540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P0_4             , RULL(0x800011540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P0_4           , RULL(0x800011540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P0_4           , RULL(0x800011540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P1_0           , RULL(0x800001540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P1_0           , RULL(0x800001540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P1_1           , RULL(0x800005540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P1_1           , RULL(0x800005540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P1_2           , RULL(0x800009540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P1_2           , RULL(0x800009540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P1_3           , RULL(0x80000D540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P1_3           , RULL(0x80000D540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P1_4           , RULL(0x800011540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P1_4           , RULL(0x800011540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P2_0           , RULL(0x800001540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P2_0           , RULL(0x800001540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P2_1           , RULL(0x800005540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P2_1           , RULL(0x800005540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P2_2           , RULL(0x800009540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P2_2           , RULL(0x800009540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P2_3           , RULL(0x80000D540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P2_3           , RULL(0x80000D540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P2_4           , RULL(0x800011540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P2_4           , RULL(0x800011540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P3_0           , RULL(0x8000015407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P3_0           , RULL(0x8000015408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P3_1           , RULL(0x8000055407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P3_1           , RULL(0x8000055408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P3_2           , RULL(0x8000095407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P3_2           , RULL(0x8000095408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P3_3           , RULL(0x80000D5407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P3_3           , RULL(0x80000D5408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P3_4           , RULL(0x8000115407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR1_P3_4           , RULL(0x8000115408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P0_0             , RULL(0x800002540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P0_0           , RULL(0x800002540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P0_0           , RULL(0x800002540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P0_1             , RULL(0x800006540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P0_1           , RULL(0x800006540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P0_1           , RULL(0x800006540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P0_2             , RULL(0x80000A540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P0_2           , RULL(0x80000A540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P0_2           , RULL(0x80000A540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P0_3             , RULL(0x80000E540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P0_3           , RULL(0x80000E540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P0_3           , RULL(0x80000E540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P0_4             , RULL(0x800012540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P0_4           , RULL(0x800012540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P0_4           , RULL(0x800012540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P1_0           , RULL(0x800002540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P1_0           , RULL(0x800002540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P1_1           , RULL(0x800006540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P1_1           , RULL(0x800006540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P1_2           , RULL(0x80000A540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P1_2           , RULL(0x80000A540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P1_3           , RULL(0x80000E540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P1_3           , RULL(0x80000E540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P1_4           , RULL(0x800012540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P1_4           , RULL(0x800012540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P2_0           , RULL(0x800002540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P2_0           , RULL(0x800002540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P2_1           , RULL(0x800006540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P2_1           , RULL(0x800006540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P2_2           , RULL(0x80000A540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P2_2           , RULL(0x80000A540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P2_3           , RULL(0x80000E540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P2_3           , RULL(0x80000E540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P2_4           , RULL(0x800012540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P2_4           , RULL(0x800012540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P3_0           , RULL(0x8000025407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P3_0           , RULL(0x8000025408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P3_1           , RULL(0x8000065407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P3_1           , RULL(0x8000065408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P3_2           , RULL(0x80000A5407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P3_2           , RULL(0x80000A5408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P3_3           , RULL(0x80000E5407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P3_3           , RULL(0x80000E5408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P3_4           , RULL(0x8000125407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR2_P3_4           , RULL(0x8000125408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P0_0             , RULL(0x800003540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P0_0           , RULL(0x800003540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P0_0           , RULL(0x800003540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P0_1             , RULL(0x800007540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P0_1           , RULL(0x800007540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P0_1           , RULL(0x800007540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P0_2             , RULL(0x80000B540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P0_2           , RULL(0x80000B540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P0_2           , RULL(0x80000B540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P0_3             , RULL(0x80000F540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P0_3           , RULL(0x80000F540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P0_3           , RULL(0x80000F540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P0_4             , RULL(0x800013540701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P0_4           , RULL(0x800013540701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P0_4           , RULL(0x800013540801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P1_0           , RULL(0x800003540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P1_0           , RULL(0x800003540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P1_1           , RULL(0x800007540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P1_1           , RULL(0x800007540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P1_2           , RULL(0x80000B540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P1_2           , RULL(0x80000B540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P1_3           , RULL(0x80000F540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P1_3           , RULL(0x80000F540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P1_4           , RULL(0x800013540701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P1_4           , RULL(0x800013540801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P2_0           , RULL(0x800003540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P2_0           , RULL(0x800003540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P2_1           , RULL(0x800007540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P2_1           , RULL(0x800007540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P2_2           , RULL(0x80000B540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P2_2           , RULL(0x80000B540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P2_3           , RULL(0x80000F540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P2_3           , RULL(0x80000F540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P2_4           , RULL(0x800013540701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P2_4           , RULL(0x800013540801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P3_0           , RULL(0x8000035407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P3_0           , RULL(0x8000035408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P3_1           , RULL(0x8000075407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P3_1           , RULL(0x8000075408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P3_2           , RULL(0x80000B5407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P3_2           , RULL(0x80000B5408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P3_3           , RULL(0x80000F5407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P3_3           , RULL(0x80000F5408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P3_4           , RULL(0x8000135407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY4_RANK_PAIR3_P3_4           , RULL(0x8000135408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P0_0             , RULL(0x800000550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P0_0           , RULL(0x800000550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P0_0           , RULL(0x800000550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P0_1             , RULL(0x800004550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P0_1           , RULL(0x800004550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P0_1           , RULL(0x800004550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P0_2             , RULL(0x800008550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P0_2           , RULL(0x800008550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P0_2           , RULL(0x800008550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P0_3             , RULL(0x80000C550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P0_3           , RULL(0x80000C550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P0_3           , RULL(0x80000C550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P0_4             , RULL(0x800010550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P0_4           , RULL(0x800010550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P0_4           , RULL(0x800010550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P1_0           , RULL(0x800000550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P1_0           , RULL(0x800000550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P1_1           , RULL(0x800004550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P1_1           , RULL(0x800004550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P1_2           , RULL(0x800008550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P1_2           , RULL(0x800008550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P1_3           , RULL(0x80000C550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P1_3           , RULL(0x80000C550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P1_4           , RULL(0x800010550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P1_4           , RULL(0x800010550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P2_0           , RULL(0x800000550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P2_0           , RULL(0x800000550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P2_1           , RULL(0x800004550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P2_1           , RULL(0x800004550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P2_2           , RULL(0x800008550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P2_2           , RULL(0x800008550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P2_3           , RULL(0x80000C550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P2_3           , RULL(0x80000C550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P2_4           , RULL(0x800010550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P2_4           , RULL(0x800010550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P3_0           , RULL(0x8000005507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P3_0           , RULL(0x8000005508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P3_1           , RULL(0x8000045507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P3_1           , RULL(0x8000045508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P3_2           , RULL(0x8000085507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P3_2           , RULL(0x8000085508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P3_3           , RULL(0x80000C5507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P3_3           , RULL(0x80000C5508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P3_4           , RULL(0x8000105507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR0_P3_4           , RULL(0x8000105508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P0_0             , RULL(0x800001550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P0_0           , RULL(0x800001550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P0_0           , RULL(0x800001550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P0_1             , RULL(0x800005550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P0_1           , RULL(0x800005550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P0_1           , RULL(0x800005550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P0_2             , RULL(0x800009550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P0_2           , RULL(0x800009550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P0_2           , RULL(0x800009550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P0_3             , RULL(0x80000D550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P0_3           , RULL(0x80000D550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P0_3           , RULL(0x80000D550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P0_4             , RULL(0x800011550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P0_4           , RULL(0x800011550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P0_4           , RULL(0x800011550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P1_0           , RULL(0x800001550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P1_0           , RULL(0x800001550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P1_1           , RULL(0x800005550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P1_1           , RULL(0x800005550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P1_2           , RULL(0x800009550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P1_2           , RULL(0x800009550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P1_3           , RULL(0x80000D550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P1_3           , RULL(0x80000D550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P1_4           , RULL(0x800011550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P1_4           , RULL(0x800011550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P2_0           , RULL(0x800001550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P2_0           , RULL(0x800001550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P2_1           , RULL(0x800005550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P2_1           , RULL(0x800005550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P2_2           , RULL(0x800009550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P2_2           , RULL(0x800009550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P2_3           , RULL(0x80000D550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P2_3           , RULL(0x80000D550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P2_4           , RULL(0x800011550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P2_4           , RULL(0x800011550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P3_0           , RULL(0x8000015507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P3_0           , RULL(0x8000015508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P3_1           , RULL(0x8000055507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P3_1           , RULL(0x8000055508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P3_2           , RULL(0x8000095507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P3_2           , RULL(0x8000095508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P3_3           , RULL(0x80000D5507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P3_3           , RULL(0x80000D5508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P3_4           , RULL(0x8000115507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR1_P3_4           , RULL(0x8000115508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P0_0             , RULL(0x800002550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P0_0           , RULL(0x800002550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P0_0           , RULL(0x800002550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P0_1             , RULL(0x800006550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P0_1           , RULL(0x800006550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P0_1           , RULL(0x800006550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P0_2             , RULL(0x80000A550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P0_2           , RULL(0x80000A550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P0_2           , RULL(0x80000A550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P0_3             , RULL(0x80000E550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P0_3           , RULL(0x80000E550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P0_3           , RULL(0x80000E550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P0_4             , RULL(0x800012550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P0_4           , RULL(0x800012550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P0_4           , RULL(0x800012550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P1_0           , RULL(0x800002550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P1_0           , RULL(0x800002550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P1_1           , RULL(0x800006550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P1_1           , RULL(0x800006550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P1_2           , RULL(0x80000A550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P1_2           , RULL(0x80000A550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P1_3           , RULL(0x80000E550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P1_3           , RULL(0x80000E550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P1_4           , RULL(0x800012550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P1_4           , RULL(0x800012550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P2_0           , RULL(0x800002550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P2_0           , RULL(0x800002550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P2_1           , RULL(0x800006550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P2_1           , RULL(0x800006550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P2_2           , RULL(0x80000A550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P2_2           , RULL(0x80000A550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P2_3           , RULL(0x80000E550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P2_3           , RULL(0x80000E550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P2_4           , RULL(0x800012550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P2_4           , RULL(0x800012550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P3_0           , RULL(0x8000025507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P3_0           , RULL(0x8000025508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P3_1           , RULL(0x8000065507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P3_1           , RULL(0x8000065508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P3_2           , RULL(0x80000A5507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P3_2           , RULL(0x80000A5508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P3_3           , RULL(0x80000E5507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P3_3           , RULL(0x80000E5508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P3_4           , RULL(0x8000125507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR2_P3_4           , RULL(0x8000125508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P0_0             , RULL(0x800003550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P0_0           , RULL(0x800003550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P0_0           , RULL(0x800003550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P0_1             , RULL(0x800007550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P0_1           , RULL(0x800007550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P0_1           , RULL(0x800007550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P0_2             , RULL(0x80000B550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P0_2           , RULL(0x80000B550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P0_2           , RULL(0x80000B550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P0_3             , RULL(0x80000F550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P0_3           , RULL(0x80000F550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P0_3           , RULL(0x80000F550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P0_4             , RULL(0x800013550701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P0_4           , RULL(0x800013550701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P0_4           , RULL(0x800013550801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P1_0           , RULL(0x800003550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P1_0           , RULL(0x800003550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P1_1           , RULL(0x800007550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P1_1           , RULL(0x800007550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P1_2           , RULL(0x80000B550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P1_2           , RULL(0x80000B550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P1_3           , RULL(0x80000F550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P1_3           , RULL(0x80000F550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P1_4           , RULL(0x800013550701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P1_4           , RULL(0x800013550801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P2_0           , RULL(0x800003550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P2_0           , RULL(0x800003550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P2_1           , RULL(0x800007550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P2_1           , RULL(0x800007550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P2_2           , RULL(0x80000B550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P2_2           , RULL(0x80000B550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P2_3           , RULL(0x80000F550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P2_3           , RULL(0x80000F550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P2_4           , RULL(0x800013550701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P2_4           , RULL(0x800013550801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P3_0           , RULL(0x8000035507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P3_0           , RULL(0x8000035508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P3_1           , RULL(0x8000075507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P3_1           , RULL(0x8000075508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P3_2           , RULL(0x80000B5507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P3_2           , RULL(0x80000B5508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P3_3           , RULL(0x80000F5507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P3_3           , RULL(0x80000F5508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P3_4           , RULL(0x8000135507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY5_RANK_PAIR3_P3_4           , RULL(0x8000135508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P0_0             , RULL(0x800000560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P0_0           , RULL(0x800000560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P0_0           , RULL(0x800000560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P0_1             , RULL(0x800004560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P0_1           , RULL(0x800004560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P0_1           , RULL(0x800004560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P0_2             , RULL(0x800008560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P0_2           , RULL(0x800008560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P0_2           , RULL(0x800008560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P0_3             , RULL(0x80000C560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P0_3           , RULL(0x80000C560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P0_3           , RULL(0x80000C560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P0_4             , RULL(0x800010560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P0_4           , RULL(0x800010560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P0_4           , RULL(0x800010560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P1_0           , RULL(0x800000560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P1_0           , RULL(0x800000560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P1_1           , RULL(0x800004560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P1_1           , RULL(0x800004560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P1_2           , RULL(0x800008560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P1_2           , RULL(0x800008560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P1_3           , RULL(0x80000C560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P1_3           , RULL(0x80000C560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P1_4           , RULL(0x800010560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P1_4           , RULL(0x800010560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P2_0           , RULL(0x800000560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P2_0           , RULL(0x800000560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P2_1           , RULL(0x800004560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P2_1           , RULL(0x800004560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P2_2           , RULL(0x800008560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P2_2           , RULL(0x800008560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P2_3           , RULL(0x80000C560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P2_3           , RULL(0x80000C560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P2_4           , RULL(0x800010560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P2_4           , RULL(0x800010560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P3_0           , RULL(0x8000005607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P3_0           , RULL(0x8000005608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P3_1           , RULL(0x8000045607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P3_1           , RULL(0x8000045608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P3_2           , RULL(0x8000085607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P3_2           , RULL(0x8000085608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P3_3           , RULL(0x80000C5607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P3_3           , RULL(0x80000C5608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P3_4           , RULL(0x8000105607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR0_P3_4           , RULL(0x8000105608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P0_0             , RULL(0x800001560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P0_0           , RULL(0x800001560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P0_0           , RULL(0x800001560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P0_1             , RULL(0x800005560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P0_1           , RULL(0x800005560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P0_1           , RULL(0x800005560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P0_2             , RULL(0x800009560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P0_2           , RULL(0x800009560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P0_2           , RULL(0x800009560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P0_3             , RULL(0x80000D560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P0_3           , RULL(0x80000D560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P0_3           , RULL(0x80000D560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P0_4             , RULL(0x800011560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P0_4           , RULL(0x800011560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P0_4           , RULL(0x800011560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P1_0           , RULL(0x800001560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P1_0           , RULL(0x800001560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P1_1           , RULL(0x800005560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P1_1           , RULL(0x800005560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P1_2           , RULL(0x800009560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P1_2           , RULL(0x800009560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P1_3           , RULL(0x80000D560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P1_3           , RULL(0x80000D560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P1_4           , RULL(0x800011560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P1_4           , RULL(0x800011560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P2_0           , RULL(0x800001560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P2_0           , RULL(0x800001560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P2_1           , RULL(0x800005560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P2_1           , RULL(0x800005560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P2_2           , RULL(0x800009560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P2_2           , RULL(0x800009560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P2_3           , RULL(0x80000D560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P2_3           , RULL(0x80000D560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P2_4           , RULL(0x800011560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P2_4           , RULL(0x800011560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P3_0           , RULL(0x8000015607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P3_0           , RULL(0x8000015608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P3_1           , RULL(0x8000055607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P3_1           , RULL(0x8000055608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P3_2           , RULL(0x8000095607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P3_2           , RULL(0x8000095608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P3_3           , RULL(0x80000D5607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P3_3           , RULL(0x80000D5608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P3_4           , RULL(0x8000115607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR1_P3_4           , RULL(0x8000115608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P0_0             , RULL(0x800002560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P0_0           , RULL(0x800002560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P0_0           , RULL(0x800002560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P0_1             , RULL(0x800006560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P0_1           , RULL(0x800006560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P0_1           , RULL(0x800006560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P0_2             , RULL(0x80000A560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P0_2           , RULL(0x80000A560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P0_2           , RULL(0x80000A560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P0_3             , RULL(0x80000E560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P0_3           , RULL(0x80000E560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P0_3           , RULL(0x80000E560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P0_4             , RULL(0x800012560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P0_4           , RULL(0x800012560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P0_4           , RULL(0x800012560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P1_0           , RULL(0x800002560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P1_0           , RULL(0x800002560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P1_1           , RULL(0x800006560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P1_1           , RULL(0x800006560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P1_2           , RULL(0x80000A560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P1_2           , RULL(0x80000A560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P1_3           , RULL(0x80000E560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P1_3           , RULL(0x80000E560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P1_4           , RULL(0x800012560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P1_4           , RULL(0x800012560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P2_0           , RULL(0x800002560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P2_0           , RULL(0x800002560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P2_1           , RULL(0x800006560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P2_1           , RULL(0x800006560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P2_2           , RULL(0x80000A560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P2_2           , RULL(0x80000A560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P2_3           , RULL(0x80000E560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P2_3           , RULL(0x80000E560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P2_4           , RULL(0x800012560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P2_4           , RULL(0x800012560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P3_0           , RULL(0x8000025607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P3_0           , RULL(0x8000025608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P3_1           , RULL(0x8000065607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P3_1           , RULL(0x8000065608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P3_2           , RULL(0x80000A5607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P3_2           , RULL(0x80000A5608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P3_3           , RULL(0x80000E5607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P3_3           , RULL(0x80000E5608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P3_4           , RULL(0x8000125607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR2_P3_4           , RULL(0x8000125608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P0_0             , RULL(0x800003560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P0_0           , RULL(0x800003560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P0_0           , RULL(0x800003560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P0_1             , RULL(0x800007560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P0_1           , RULL(0x800007560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P0_1           , RULL(0x800007560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P0_2             , RULL(0x80000B560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P0_2           , RULL(0x80000B560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P0_2           , RULL(0x80000B560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P0_3             , RULL(0x80000F560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P0_3           , RULL(0x80000F560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P0_3           , RULL(0x80000F560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P0_4             , RULL(0x800013560701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P0_4           , RULL(0x800013560701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P0_4           , RULL(0x800013560801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P1_0           , RULL(0x800003560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P1_0           , RULL(0x800003560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P1_1           , RULL(0x800007560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P1_1           , RULL(0x800007560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P1_2           , RULL(0x80000B560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P1_2           , RULL(0x80000B560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P1_3           , RULL(0x80000F560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P1_3           , RULL(0x80000F560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P1_4           , RULL(0x800013560701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P1_4           , RULL(0x800013560801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P2_0           , RULL(0x800003560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P2_0           , RULL(0x800003560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P2_1           , RULL(0x800007560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P2_1           , RULL(0x800007560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P2_2           , RULL(0x80000B560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P2_2           , RULL(0x80000B560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P2_3           , RULL(0x80000F560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P2_3           , RULL(0x80000F560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P2_4           , RULL(0x800013560701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P2_4           , RULL(0x800013560801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P3_0           , RULL(0x8000035607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P3_0           , RULL(0x8000035608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P3_1           , RULL(0x8000075607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P3_1           , RULL(0x8000075608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P3_2           , RULL(0x80000B5607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P3_2           , RULL(0x80000B5608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P3_3           , RULL(0x80000F5607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P3_3           , RULL(0x80000F5608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P3_4           , RULL(0x8000135607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY6_RANK_PAIR3_P3_4           , RULL(0x8000135608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P0_0             , RULL(0x800000570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P0_0           , RULL(0x800000570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P0_0           , RULL(0x800000570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P0_1             , RULL(0x800004570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P0_1           , RULL(0x800004570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P0_1           , RULL(0x800004570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P0_2             , RULL(0x800008570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P0_2           , RULL(0x800008570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P0_2           , RULL(0x800008570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P0_3             , RULL(0x80000C570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P0_3           , RULL(0x80000C570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P0_3           , RULL(0x80000C570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P0_4             , RULL(0x800010570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P0_4           , RULL(0x800010570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P0_4           , RULL(0x800010570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P1_0           , RULL(0x800000570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P1_0           , RULL(0x800000570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P1_1           , RULL(0x800004570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P1_1           , RULL(0x800004570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P1_2           , RULL(0x800008570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P1_2           , RULL(0x800008570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P1_3           , RULL(0x80000C570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P1_3           , RULL(0x80000C570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P1_4           , RULL(0x800010570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P1_4           , RULL(0x800010570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P2_0           , RULL(0x800000570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P2_0           , RULL(0x800000570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P2_1           , RULL(0x800004570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P2_1           , RULL(0x800004570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P2_2           , RULL(0x800008570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P2_2           , RULL(0x800008570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P2_3           , RULL(0x80000C570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P2_3           , RULL(0x80000C570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P2_4           , RULL(0x800010570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P2_4           , RULL(0x800010570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P3_0           , RULL(0x8000005707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P3_0           , RULL(0x8000005708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P3_1           , RULL(0x8000045707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P3_1           , RULL(0x8000045708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P3_2           , RULL(0x8000085707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P3_2           , RULL(0x8000085708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P3_3           , RULL(0x80000C5707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P3_3           , RULL(0x80000C5708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P3_4           , RULL(0x8000105707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR0_P3_4           , RULL(0x8000105708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P0_0             , RULL(0x800001570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P0_0           , RULL(0x800001570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P0_0           , RULL(0x800001570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P0_1             , RULL(0x800005570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P0_1           , RULL(0x800005570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P0_1           , RULL(0x800005570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P0_2             , RULL(0x800009570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P0_2           , RULL(0x800009570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P0_2           , RULL(0x800009570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P0_3             , RULL(0x80000D570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P0_3           , RULL(0x80000D570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P0_3           , RULL(0x80000D570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P0_4             , RULL(0x800011570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P0_4           , RULL(0x800011570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P0_4           , RULL(0x800011570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P1_0           , RULL(0x800001570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P1_0           , RULL(0x800001570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P1_1           , RULL(0x800005570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P1_1           , RULL(0x800005570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P1_2           , RULL(0x800009570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P1_2           , RULL(0x800009570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P1_3           , RULL(0x80000D570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P1_3           , RULL(0x80000D570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P1_4           , RULL(0x800011570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P1_4           , RULL(0x800011570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P2_0           , RULL(0x800001570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P2_0           , RULL(0x800001570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P2_1           , RULL(0x800005570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P2_1           , RULL(0x800005570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P2_2           , RULL(0x800009570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P2_2           , RULL(0x800009570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P2_3           , RULL(0x80000D570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P2_3           , RULL(0x80000D570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P2_4           , RULL(0x800011570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P2_4           , RULL(0x800011570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P3_0           , RULL(0x8000015707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P3_0           , RULL(0x8000015708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P3_1           , RULL(0x8000055707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P3_1           , RULL(0x8000055708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P3_2           , RULL(0x8000095707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P3_2           , RULL(0x8000095708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P3_3           , RULL(0x80000D5707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P3_3           , RULL(0x80000D5708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P3_4           , RULL(0x8000115707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR1_P3_4           , RULL(0x8000115708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P0_0             , RULL(0x800002570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P0_0           , RULL(0x800002570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P0_0           , RULL(0x800002570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P0_1             , RULL(0x800006570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P0_1           , RULL(0x800006570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P0_1           , RULL(0x800006570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P0_2             , RULL(0x80000A570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P0_2           , RULL(0x80000A570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P0_2           , RULL(0x80000A570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P0_3             , RULL(0x80000E570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P0_3           , RULL(0x80000E570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P0_3           , RULL(0x80000E570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P0_4             , RULL(0x800012570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P0_4           , RULL(0x800012570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P0_4           , RULL(0x800012570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P1_0           , RULL(0x800002570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P1_0           , RULL(0x800002570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P1_1           , RULL(0x800006570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P1_1           , RULL(0x800006570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P1_2           , RULL(0x80000A570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P1_2           , RULL(0x80000A570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P1_3           , RULL(0x80000E570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P1_3           , RULL(0x80000E570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P1_4           , RULL(0x800012570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P1_4           , RULL(0x800012570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P2_0           , RULL(0x800002570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P2_0           , RULL(0x800002570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P2_1           , RULL(0x800006570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P2_1           , RULL(0x800006570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P2_2           , RULL(0x80000A570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P2_2           , RULL(0x80000A570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P2_3           , RULL(0x80000E570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P2_3           , RULL(0x80000E570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P2_4           , RULL(0x800012570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P2_4           , RULL(0x800012570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P3_0           , RULL(0x8000025707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P3_0           , RULL(0x8000025708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P3_1           , RULL(0x8000065707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P3_1           , RULL(0x8000065708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P3_2           , RULL(0x80000A5707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P3_2           , RULL(0x80000A5708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P3_3           , RULL(0x80000E5707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P3_3           , RULL(0x80000E5708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P3_4           , RULL(0x8000125707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR2_P3_4           , RULL(0x8000125708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P0_0             , RULL(0x800003570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P0_0           , RULL(0x800003570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P0_0           , RULL(0x800003570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P0_1             , RULL(0x800007570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P0_1           , RULL(0x800007570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P0_1           , RULL(0x800007570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P0_2             , RULL(0x80000B570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P0_2           , RULL(0x80000B570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P0_2           , RULL(0x80000B570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P0_3             , RULL(0x80000F570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P0_3           , RULL(0x80000F570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P0_3           , RULL(0x80000F570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P0_4             , RULL(0x800013570701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P0_4           , RULL(0x800013570701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P0_4           , RULL(0x800013570801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P1_0           , RULL(0x800003570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P1_0           , RULL(0x800003570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P1_1           , RULL(0x800007570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P1_1           , RULL(0x800007570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P1_2           , RULL(0x80000B570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P1_2           , RULL(0x80000B570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P1_3           , RULL(0x80000F570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P1_3           , RULL(0x80000F570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P1_4           , RULL(0x800013570701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P1_4           , RULL(0x800013570801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P2_0           , RULL(0x800003570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P2_0           , RULL(0x800003570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P2_1           , RULL(0x800007570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P2_1           , RULL(0x800007570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P2_2           , RULL(0x80000B570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P2_2           , RULL(0x80000B570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P2_3           , RULL(0x80000F570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P2_3           , RULL(0x80000F570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P2_4           , RULL(0x800013570701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P2_4           , RULL(0x800013570801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P3_0           , RULL(0x8000035707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P3_0           , RULL(0x8000035708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P3_1           , RULL(0x8000075707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P3_1           , RULL(0x8000075708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P3_2           , RULL(0x80000B5707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P3_2           , RULL(0x80000B5708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P3_3           , RULL(0x80000F5707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P3_3           , RULL(0x80000F5708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P3_4           , RULL(0x8000135707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY7_RANK_PAIR3_P3_4           , RULL(0x8000135708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P0_0      , RULL(0x8000000C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P0_0    , RULL(0x8000000C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P0_0    , RULL(0x8000000C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P0_1      , RULL(0x8000040C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P0_1    , RULL(0x8000040C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P0_1    , RULL(0x8000040C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P0_2      , RULL(0x8000080C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P0_2    , RULL(0x8000080C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P0_2    , RULL(0x8000080C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P0_3      , RULL(0x80000C0C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P0_3    , RULL(0x80000C0C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P0_3    , RULL(0x80000C0C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P0_4      , RULL(0x8000100C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P0_4    , RULL(0x8000100C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P0_4    , RULL(0x8000100C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P1_0    , RULL(0x8000000C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P1_0    , RULL(0x8000000C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P1_1    , RULL(0x8000040C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P1_1    , RULL(0x8000040C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P1_2    , RULL(0x8000080C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P1_2    , RULL(0x8000080C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P1_3    , RULL(0x80000C0C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P1_3    , RULL(0x80000C0C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P1_4    , RULL(0x8000100C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P1_4    , RULL(0x8000100C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P2_0    , RULL(0x8000000C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P2_0    , RULL(0x8000000C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P2_1    , RULL(0x8000040C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P2_1    , RULL(0x8000040C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P2_2    , RULL(0x8000080C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P2_2    , RULL(0x8000080C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P2_3    , RULL(0x80000C0C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P2_3    , RULL(0x80000C0C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P2_4    , RULL(0x8000100C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P2_4    , RULL(0x8000100C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P3_0    , RULL(0x8000000C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P3_0    , RULL(0x8000000C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P3_1    , RULL(0x8000040C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P3_1    , RULL(0x8000040C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P3_2    , RULL(0x8000080C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P3_2    , RULL(0x8000080C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P3_3    , RULL(0x80000C0C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P3_3    , RULL(0x80000C0C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P3_4    , RULL(0x8000100C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR0_P3_4    , RULL(0x8000100C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P0_0      , RULL(0x8000010C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P0_0    , RULL(0x8000010C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P0_0    , RULL(0x8000010C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P0_1      , RULL(0x8000050C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P0_1    , RULL(0x8000050C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P0_1    , RULL(0x8000050C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P0_2      , RULL(0x8000090C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P0_2    , RULL(0x8000090C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P0_2    , RULL(0x8000090C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P0_3      , RULL(0x80000D0C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P0_3    , RULL(0x80000D0C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P0_3    , RULL(0x80000D0C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P0_4      , RULL(0x8000110C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P0_4    , RULL(0x8000110C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P0_4    , RULL(0x8000110C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P1_0    , RULL(0x8000010C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P1_0    , RULL(0x8000010C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P1_1    , RULL(0x8000050C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P1_1    , RULL(0x8000050C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P1_2    , RULL(0x8000090C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P1_2    , RULL(0x8000090C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P1_3    , RULL(0x80000D0C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P1_3    , RULL(0x80000D0C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P1_4    , RULL(0x8000110C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P1_4    , RULL(0x8000110C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P2_0    , RULL(0x8000010C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P2_0    , RULL(0x8000010C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P2_1    , RULL(0x8000050C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P2_1    , RULL(0x8000050C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P2_2    , RULL(0x8000090C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P2_2    , RULL(0x8000090C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P2_3    , RULL(0x80000D0C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P2_3    , RULL(0x80000D0C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P2_4    , RULL(0x8000110C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P2_4    , RULL(0x8000110C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P3_0    , RULL(0x8000010C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P3_0    , RULL(0x8000010C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P3_1    , RULL(0x8000050C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P3_1    , RULL(0x8000050C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P3_2    , RULL(0x8000090C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P3_2    , RULL(0x8000090C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P3_3    , RULL(0x80000D0C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P3_3    , RULL(0x80000D0C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P3_4    , RULL(0x8000110C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR1_P3_4    , RULL(0x8000110C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P0_0      , RULL(0x8000020C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P0_0    , RULL(0x8000020C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P0_0    , RULL(0x8000020C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P0_1      , RULL(0x8000060C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P0_1    , RULL(0x8000060C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P0_1    , RULL(0x8000060C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P0_2      , RULL(0x80000A0C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P0_2    , RULL(0x80000A0C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P0_2    , RULL(0x80000A0C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P0_3      , RULL(0x80000E0C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P0_3    , RULL(0x80000E0C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P0_3    , RULL(0x80000E0C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P0_4      , RULL(0x8000120C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P0_4    , RULL(0x8000120C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P0_4    , RULL(0x8000120C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P1_0    , RULL(0x8000020C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P1_0    , RULL(0x8000020C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P1_1    , RULL(0x8000060C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P1_1    , RULL(0x8000060C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P1_2    , RULL(0x80000A0C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P1_2    , RULL(0x80000A0C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P1_3    , RULL(0x80000E0C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P1_3    , RULL(0x80000E0C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P1_4    , RULL(0x8000120C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P1_4    , RULL(0x8000120C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P2_0    , RULL(0x8000020C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P2_0    , RULL(0x8000020C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P2_1    , RULL(0x8000060C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P2_1    , RULL(0x8000060C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P2_2    , RULL(0x80000A0C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P2_2    , RULL(0x80000A0C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P2_3    , RULL(0x80000E0C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P2_3    , RULL(0x80000E0C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P2_4    , RULL(0x8000120C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P2_4    , RULL(0x8000120C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P3_0    , RULL(0x8000020C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P3_0    , RULL(0x8000020C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P3_1    , RULL(0x8000060C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P3_1    , RULL(0x8000060C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P3_2    , RULL(0x80000A0C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P3_2    , RULL(0x80000A0C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P3_3    , RULL(0x80000E0C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P3_3    , RULL(0x80000E0C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P3_4    , RULL(0x8000120C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR2_P3_4    , RULL(0x8000120C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P0_0      , RULL(0x8000030C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P0_0    , RULL(0x8000030C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P0_0    , RULL(0x8000030C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P0_1      , RULL(0x8000070C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P0_1    , RULL(0x8000070C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P0_1    , RULL(0x8000070C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P0_2      , RULL(0x80000B0C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P0_2    , RULL(0x80000B0C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P0_2    , RULL(0x80000B0C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P0_3      , RULL(0x80000F0C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P0_3    , RULL(0x80000F0C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P0_3    , RULL(0x80000F0C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P0_4      , RULL(0x8000130C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P0_4    , RULL(0x8000130C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P0_4    , RULL(0x8000130C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P1_0    , RULL(0x8000030C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P1_0    , RULL(0x8000030C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P1_1    , RULL(0x8000070C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P1_1    , RULL(0x8000070C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P1_2    , RULL(0x80000B0C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P1_2    , RULL(0x80000B0C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P1_3    , RULL(0x80000F0C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P1_3    , RULL(0x80000F0C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P1_4    , RULL(0x8000130C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P1_4    , RULL(0x8000130C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P2_0    , RULL(0x8000030C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P2_0    , RULL(0x8000030C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P2_1    , RULL(0x8000070C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P2_1    , RULL(0x8000070C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P2_2    , RULL(0x80000B0C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P2_2    , RULL(0x80000B0C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P2_3    , RULL(0x80000F0C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P2_3    , RULL(0x80000F0C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P2_4    , RULL(0x8000130C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P2_4    , RULL(0x8000130C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P3_0    , RULL(0x8000030C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P3_0    , RULL(0x8000030C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P3_1    , RULL(0x8000070C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P3_1    , RULL(0x8000070C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P3_2    , RULL(0x80000B0C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P3_2    , RULL(0x80000B0C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P3_3    , RULL(0x80000F0C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P3_3    , RULL(0x80000F0C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P3_4    , RULL(0x8000130C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET0_RANK_PAIR3_P3_4    , RULL(0x8000130C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P0_0      , RULL(0x8000000D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P0_0    , RULL(0x8000000D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P0_0    , RULL(0x8000000D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P0_1      , RULL(0x8000040D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P0_1    , RULL(0x8000040D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P0_1    , RULL(0x8000040D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P0_2      , RULL(0x8000080D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P0_2    , RULL(0x8000080D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P0_2    , RULL(0x8000080D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P0_3      , RULL(0x80000C0D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P0_3    , RULL(0x80000C0D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P0_3    , RULL(0x80000C0D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P0_4      , RULL(0x8000100D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P0_4    , RULL(0x8000100D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P0_4    , RULL(0x8000100D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P1_0    , RULL(0x8000000D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P1_0    , RULL(0x8000000D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P1_1    , RULL(0x8000040D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P1_1    , RULL(0x8000040D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P1_2    , RULL(0x8000080D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P1_2    , RULL(0x8000080D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P1_3    , RULL(0x80000C0D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P1_3    , RULL(0x80000C0D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P1_4    , RULL(0x8000100D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P1_4    , RULL(0x8000100D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P2_0    , RULL(0x8000000D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P2_0    , RULL(0x8000000D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P2_1    , RULL(0x8000040D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P2_1    , RULL(0x8000040D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P2_2    , RULL(0x8000080D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P2_2    , RULL(0x8000080D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P2_3    , RULL(0x80000C0D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P2_3    , RULL(0x80000C0D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P2_4    , RULL(0x8000100D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P2_4    , RULL(0x8000100D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P3_0    , RULL(0x8000000D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P3_0    , RULL(0x8000000D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P3_1    , RULL(0x8000040D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P3_1    , RULL(0x8000040D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P3_2    , RULL(0x8000080D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P3_2    , RULL(0x8000080D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P3_3    , RULL(0x80000C0D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P3_3    , RULL(0x80000C0D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P3_4    , RULL(0x8000100D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR0_P3_4    , RULL(0x8000100D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P0_0      , RULL(0x8000010D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P0_0    , RULL(0x8000010D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P0_0    , RULL(0x8000010D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P0_1      , RULL(0x8000050D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P0_1    , RULL(0x8000050D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P0_1    , RULL(0x8000050D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P0_2      , RULL(0x8000090D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P0_2    , RULL(0x8000090D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P0_2    , RULL(0x8000090D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P0_3      , RULL(0x80000D0D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P0_3    , RULL(0x80000D0D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P0_3    , RULL(0x80000D0D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P0_4      , RULL(0x8000110D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P0_4    , RULL(0x8000110D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P0_4    , RULL(0x8000110D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P1_0    , RULL(0x8000010D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P1_0    , RULL(0x8000010D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P1_1    , RULL(0x8000050D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P1_1    , RULL(0x8000050D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P1_2    , RULL(0x8000090D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P1_2    , RULL(0x8000090D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P1_3    , RULL(0x80000D0D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P1_3    , RULL(0x80000D0D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P1_4    , RULL(0x8000110D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P1_4    , RULL(0x8000110D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P2_0    , RULL(0x8000010D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P2_0    , RULL(0x8000010D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P2_1    , RULL(0x8000050D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P2_1    , RULL(0x8000050D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P2_2    , RULL(0x8000090D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P2_2    , RULL(0x8000090D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P2_3    , RULL(0x80000D0D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P2_3    , RULL(0x80000D0D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P2_4    , RULL(0x8000110D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P2_4    , RULL(0x8000110D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P3_0    , RULL(0x8000010D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P3_0    , RULL(0x8000010D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P3_1    , RULL(0x8000050D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P3_1    , RULL(0x8000050D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P3_2    , RULL(0x8000090D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P3_2    , RULL(0x8000090D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P3_3    , RULL(0x80000D0D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P3_3    , RULL(0x80000D0D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P3_4    , RULL(0x8000110D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR1_P3_4    , RULL(0x8000110D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P0_0      , RULL(0x8000020D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P0_0    , RULL(0x8000020D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P0_0    , RULL(0x8000020D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P0_1      , RULL(0x8000060D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P0_1    , RULL(0x8000060D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P0_1    , RULL(0x8000060D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P0_2      , RULL(0x80000A0D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P0_2    , RULL(0x80000A0D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P0_2    , RULL(0x80000A0D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P0_3      , RULL(0x80000E0D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P0_3    , RULL(0x80000E0D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P0_3    , RULL(0x80000E0D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P0_4      , RULL(0x8000120D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P0_4    , RULL(0x8000120D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P0_4    , RULL(0x8000120D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P1_0    , RULL(0x8000020D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P1_0    , RULL(0x8000020D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P1_1    , RULL(0x8000060D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P1_1    , RULL(0x8000060D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P1_2    , RULL(0x80000A0D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P1_2    , RULL(0x80000A0D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P1_3    , RULL(0x80000E0D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P1_3    , RULL(0x80000E0D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P1_4    , RULL(0x8000120D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P1_4    , RULL(0x8000120D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P2_0    , RULL(0x8000020D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P2_0    , RULL(0x8000020D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P2_1    , RULL(0x8000060D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P2_1    , RULL(0x8000060D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P2_2    , RULL(0x80000A0D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P2_2    , RULL(0x80000A0D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P2_3    , RULL(0x80000E0D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P2_3    , RULL(0x80000E0D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P2_4    , RULL(0x8000120D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P2_4    , RULL(0x8000120D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P3_0    , RULL(0x8000020D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P3_0    , RULL(0x8000020D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P3_1    , RULL(0x8000060D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P3_1    , RULL(0x8000060D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P3_2    , RULL(0x80000A0D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P3_2    , RULL(0x80000A0D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P3_3    , RULL(0x80000E0D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P3_3    , RULL(0x80000E0D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P3_4    , RULL(0x8000120D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR2_P3_4    , RULL(0x8000120D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P0_0      , RULL(0x8000030D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P0_0    , RULL(0x8000030D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P0_0    , RULL(0x8000030D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P0_1      , RULL(0x8000070D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P0_1    , RULL(0x8000070D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P0_1    , RULL(0x8000070D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P0_2      , RULL(0x80000B0D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P0_2    , RULL(0x80000B0D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P0_2    , RULL(0x80000B0D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P0_3      , RULL(0x80000F0D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P0_3    , RULL(0x80000F0D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P0_3    , RULL(0x80000F0D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P0_4      , RULL(0x8000130D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P0_4    , RULL(0x8000130D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P0_4    , RULL(0x8000130D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P1_0    , RULL(0x8000030D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P1_0    , RULL(0x8000030D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P1_1    , RULL(0x8000070D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P1_1    , RULL(0x8000070D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P1_2    , RULL(0x80000B0D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P1_2    , RULL(0x80000B0D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P1_3    , RULL(0x80000F0D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P1_3    , RULL(0x80000F0D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P1_4    , RULL(0x8000130D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P1_4    , RULL(0x8000130D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P2_0    , RULL(0x8000030D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P2_0    , RULL(0x8000030D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P2_1    , RULL(0x8000070D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P2_1    , RULL(0x8000070D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P2_2    , RULL(0x80000B0D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P2_2    , RULL(0x80000B0D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P2_3    , RULL(0x80000F0D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P2_3    , RULL(0x80000F0D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P2_4    , RULL(0x8000130D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P2_4    , RULL(0x8000130D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P3_0    , RULL(0x8000030D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P3_0    , RULL(0x8000030D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P3_1    , RULL(0x8000070D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P3_1    , RULL(0x8000070D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P3_2    , RULL(0x80000B0D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P3_2    , RULL(0x80000B0D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P3_3    , RULL(0x80000F0D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P3_3    , RULL(0x80000F0D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P3_4    , RULL(0x8000130D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DELAY_OFFSET1_RANK_PAIR3_P3_4    , RULL(0x8000130D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P0_0          , RULL(0x800000720701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P0_0        , RULL(0x800000720701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P0_0        , RULL(0x800000720801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P0_1          , RULL(0x800004720701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P0_1        , RULL(0x800004720701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P0_1        , RULL(0x800004720801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P0_2          , RULL(0x800008720701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P0_2        , RULL(0x800008720701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P0_2        , RULL(0x800008720801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P0_3          , RULL(0x80000C720701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P0_3        , RULL(0x80000C720701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P0_3        , RULL(0x80000C720801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P0_4          , RULL(0x800010720701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P0_4        , RULL(0x800010720701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P0_4        , RULL(0x800010720801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P1_0        , RULL(0x800000720701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P1_0        , RULL(0x800000720801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P1_1        , RULL(0x800004720701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P1_1        , RULL(0x800004720801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P1_2        , RULL(0x800008720701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P1_2        , RULL(0x800008720801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P1_3        , RULL(0x80000C720701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P1_3        , RULL(0x80000C720801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P1_4        , RULL(0x800010720701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P1_4        , RULL(0x800010720801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P2_0        , RULL(0x800000720701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P2_0        , RULL(0x800000720801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P2_1        , RULL(0x800004720701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P2_1        , RULL(0x800004720801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P2_2        , RULL(0x800008720701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P2_2        , RULL(0x800008720801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P2_3        , RULL(0x80000C720701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P2_3        , RULL(0x80000C720801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P2_4        , RULL(0x800010720701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P2_4        , RULL(0x800010720801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P3_0        , RULL(0x8000007207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P3_0        , RULL(0x8000007208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P3_1        , RULL(0x8000047207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P3_1        , RULL(0x8000047208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P3_2        , RULL(0x8000087207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P3_2        , RULL(0x8000087208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P3_3        , RULL(0x80000C7207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P3_3        , RULL(0x80000C7208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P3_4        , RULL(0x8000107207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_DQS_TIMING_REFERENCE_P3_4        , RULL(0x8000107208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P0_0          , RULL(0x800000600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P0_0        , RULL(0x800000600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P0_0        , RULL(0x800000600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P0_1          , RULL(0x800004600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P0_1        , RULL(0x800004600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P0_1        , RULL(0x800004600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P0_2          , RULL(0x800008600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P0_2        , RULL(0x800008600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P0_2        , RULL(0x800008600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P0_3          , RULL(0x80000C600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P0_3        , RULL(0x80000C600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P0_3        , RULL(0x80000C600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P0_4          , RULL(0x800010600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P0_4        , RULL(0x800010600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P0_4        , RULL(0x800010600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P1_0        , RULL(0x800000600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P1_0        , RULL(0x800000600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P1_1        , RULL(0x800004600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P1_1        , RULL(0x800004600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P1_2        , RULL(0x800008600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P1_2        , RULL(0x800008600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P1_3        , RULL(0x80000C600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P1_3        , RULL(0x80000C600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P1_4        , RULL(0x800010600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P1_4        , RULL(0x800010600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P2_0        , RULL(0x800000600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P2_0        , RULL(0x800000600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P2_1        , RULL(0x800004600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P2_1        , RULL(0x800004600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P2_2        , RULL(0x800008600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P2_2        , RULL(0x800008600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P2_3        , RULL(0x80000C600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P2_3        , RULL(0x80000C600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P2_4        , RULL(0x800010600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P2_4        , RULL(0x800010600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P3_0        , RULL(0x8000006007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P3_0        , RULL(0x8000006008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P3_1        , RULL(0x8000046007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P3_1        , RULL(0x8000046008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P3_2        , RULL(0x8000086007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P3_2        , RULL(0x8000086008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P3_3        , RULL(0x80000C6007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P3_3        , RULL(0x80000C6008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P3_4        , RULL(0x8000106007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR0_P3_4        , RULL(0x8000106008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P0_0          , RULL(0x800001600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P0_0        , RULL(0x800001600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P0_0        , RULL(0x800001600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P0_1          , RULL(0x800005600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P0_1        , RULL(0x800005600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P0_1        , RULL(0x800005600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P0_2          , RULL(0x800009600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P0_2        , RULL(0x800009600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P0_2        , RULL(0x800009600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P0_3          , RULL(0x80000D600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P0_3        , RULL(0x80000D600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P0_3        , RULL(0x80000D600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P0_4          , RULL(0x800011600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P0_4        , RULL(0x800011600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P0_4        , RULL(0x800011600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P1_0        , RULL(0x800001600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P1_0        , RULL(0x800001600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P1_1        , RULL(0x800005600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P1_1        , RULL(0x800005600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P1_2        , RULL(0x800009600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P1_2        , RULL(0x800009600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P1_3        , RULL(0x80000D600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P1_3        , RULL(0x80000D600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P1_4        , RULL(0x800011600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P1_4        , RULL(0x800011600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P2_0        , RULL(0x800001600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P2_0        , RULL(0x800001600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P2_1        , RULL(0x800005600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P2_1        , RULL(0x800005600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P2_2        , RULL(0x800009600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P2_2        , RULL(0x800009600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P2_3        , RULL(0x80000D600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P2_3        , RULL(0x80000D600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P2_4        , RULL(0x800011600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P2_4        , RULL(0x800011600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P3_0        , RULL(0x8000016007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P3_0        , RULL(0x8000016008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P3_1        , RULL(0x8000056007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P3_1        , RULL(0x8000056008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P3_2        , RULL(0x8000096007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P3_2        , RULL(0x8000096008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P3_3        , RULL(0x80000D6007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P3_3        , RULL(0x80000D6008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P3_4        , RULL(0x8000116007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR1_P3_4        , RULL(0x8000116008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P0_0          , RULL(0x800002600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P0_0        , RULL(0x800002600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P0_0        , RULL(0x800002600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P0_1          , RULL(0x800006600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P0_1        , RULL(0x800006600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P0_1        , RULL(0x800006600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P0_2          , RULL(0x80000A600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P0_2        , RULL(0x80000A600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P0_2        , RULL(0x80000A600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P0_3          , RULL(0x80000E600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P0_3        , RULL(0x80000E600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P0_3        , RULL(0x80000E600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P0_4          , RULL(0x800012600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P0_4        , RULL(0x800012600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P0_4        , RULL(0x800012600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P1_0        , RULL(0x800002600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P1_0        , RULL(0x800002600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P1_1        , RULL(0x800006600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P1_1        , RULL(0x800006600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P1_2        , RULL(0x80000A600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P1_2        , RULL(0x80000A600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P1_3        , RULL(0x80000E600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P1_3        , RULL(0x80000E600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P1_4        , RULL(0x800012600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P1_4        , RULL(0x800012600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P2_0        , RULL(0x800002600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P2_0        , RULL(0x800002600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P2_1        , RULL(0x800006600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P2_1        , RULL(0x800006600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P2_2        , RULL(0x80000A600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P2_2        , RULL(0x80000A600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P2_3        , RULL(0x80000E600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P2_3        , RULL(0x80000E600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P2_4        , RULL(0x800012600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P2_4        , RULL(0x800012600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P3_0        , RULL(0x8000026007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P3_0        , RULL(0x8000026008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P3_1        , RULL(0x8000066007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P3_1        , RULL(0x8000066008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P3_2        , RULL(0x80000A6007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P3_2        , RULL(0x80000A6008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P3_3        , RULL(0x80000E6007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P3_3        , RULL(0x80000E6008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P3_4        , RULL(0x8000126007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR2_P3_4        , RULL(0x8000126008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P0_0          , RULL(0x800003600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P0_0        , RULL(0x800003600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P0_0        , RULL(0x800003600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P0_1          , RULL(0x800007600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P0_1        , RULL(0x800007600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P0_1        , RULL(0x800007600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P0_2          , RULL(0x80000B600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P0_2        , RULL(0x80000B600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P0_2        , RULL(0x80000B600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P0_3          , RULL(0x80000F600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P0_3        , RULL(0x80000F600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P0_3        , RULL(0x80000F600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P0_4          , RULL(0x800013600701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P0_4        , RULL(0x800013600701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P0_4        , RULL(0x800013600801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P1_0        , RULL(0x800003600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P1_0        , RULL(0x800003600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P1_1        , RULL(0x800007600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P1_1        , RULL(0x800007600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P1_2        , RULL(0x80000B600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P1_2        , RULL(0x80000B600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P1_3        , RULL(0x80000F600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P1_3        , RULL(0x80000F600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P1_4        , RULL(0x800013600701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P1_4        , RULL(0x800013600801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P2_0        , RULL(0x800003600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P2_0        , RULL(0x800003600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P2_1        , RULL(0x800007600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P2_1        , RULL(0x800007600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P2_2        , RULL(0x80000B600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P2_2        , RULL(0x80000B600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P2_3        , RULL(0x80000F600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P2_3        , RULL(0x80000F600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P2_4        , RULL(0x800013600701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P2_4        , RULL(0x800013600801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P3_0        , RULL(0x8000036007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P3_0        , RULL(0x8000036008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P3_1        , RULL(0x8000076007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P3_1        , RULL(0x8000076008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P3_2        , RULL(0x80000B6007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P3_2        , RULL(0x80000B6008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P3_3        , RULL(0x80000F6007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P3_3        , RULL(0x80000F6008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P3_4        , RULL(0x8000136007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE0_RANK_PAIR3_P3_4        , RULL(0x8000136008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P0_0         , RULL(0x8000006A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P0_0       , RULL(0x8000006A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P0_0       , RULL(0x8000006A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P0_1         , RULL(0x8000046A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P0_1       , RULL(0x8000046A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P0_1       , RULL(0x8000046A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P0_2         , RULL(0x8000086A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P0_2       , RULL(0x8000086A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P0_2       , RULL(0x8000086A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P0_3         , RULL(0x80000C6A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P0_3       , RULL(0x80000C6A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P0_3       , RULL(0x80000C6A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P0_4         , RULL(0x8000106A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P0_4       , RULL(0x8000106A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P0_4       , RULL(0x8000106A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P1_0       , RULL(0x8000006A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P1_0       , RULL(0x8000006A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P1_1       , RULL(0x8000046A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P1_1       , RULL(0x8000046A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P1_2       , RULL(0x8000086A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P1_2       , RULL(0x8000086A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P1_3       , RULL(0x80000C6A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P1_3       , RULL(0x80000C6A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P1_4       , RULL(0x8000106A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P1_4       , RULL(0x8000106A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P2_0       , RULL(0x8000006A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P2_0       , RULL(0x8000006A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P2_1       , RULL(0x8000046A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P2_1       , RULL(0x8000046A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P2_2       , RULL(0x8000086A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P2_2       , RULL(0x8000086A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P2_3       , RULL(0x80000C6A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P2_3       , RULL(0x80000C6A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P2_4       , RULL(0x8000106A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P2_4       , RULL(0x8000106A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P3_0       , RULL(0x8000006A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P3_0       , RULL(0x8000006A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P3_1       , RULL(0x8000046A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P3_1       , RULL(0x8000046A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P3_2       , RULL(0x8000086A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P3_2       , RULL(0x8000086A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P3_3       , RULL(0x80000C6A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P3_3       , RULL(0x80000C6A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P3_4       , RULL(0x8000106A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR0_P3_4       , RULL(0x8000106A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P0_0         , RULL(0x8000016A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P0_0       , RULL(0x8000016A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P0_0       , RULL(0x8000016A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P0_1         , RULL(0x8000056A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P0_1       , RULL(0x8000056A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P0_1       , RULL(0x8000056A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P0_2         , RULL(0x8000096A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P0_2       , RULL(0x8000096A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P0_2       , RULL(0x8000096A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P0_3         , RULL(0x80000D6A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P0_3       , RULL(0x80000D6A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P0_3       , RULL(0x80000D6A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P0_4         , RULL(0x8000116A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P0_4       , RULL(0x8000116A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P0_4       , RULL(0x8000116A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P1_0       , RULL(0x8000016A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P1_0       , RULL(0x8000016A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P1_1       , RULL(0x8000056A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P1_1       , RULL(0x8000056A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P1_2       , RULL(0x8000096A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P1_2       , RULL(0x8000096A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P1_3       , RULL(0x80000D6A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P1_3       , RULL(0x80000D6A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P1_4       , RULL(0x8000116A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P1_4       , RULL(0x8000116A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P2_0       , RULL(0x8000016A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P2_0       , RULL(0x8000016A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P2_1       , RULL(0x8000056A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P2_1       , RULL(0x8000056A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P2_2       , RULL(0x8000096A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P2_2       , RULL(0x8000096A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P2_3       , RULL(0x80000D6A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P2_3       , RULL(0x80000D6A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P2_4       , RULL(0x8000116A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P2_4       , RULL(0x8000116A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P3_0       , RULL(0x8000016A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P3_0       , RULL(0x8000016A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P3_1       , RULL(0x8000056A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P3_1       , RULL(0x8000056A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P3_2       , RULL(0x8000096A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P3_2       , RULL(0x8000096A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P3_3       , RULL(0x80000D6A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P3_3       , RULL(0x80000D6A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P3_4       , RULL(0x8000116A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR1_P3_4       , RULL(0x8000116A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P0_0         , RULL(0x8000026A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P0_0       , RULL(0x8000026A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P0_0       , RULL(0x8000026A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P0_1         , RULL(0x8000066A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P0_1       , RULL(0x8000066A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P0_1       , RULL(0x8000066A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P0_2         , RULL(0x80000A6A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P0_2       , RULL(0x80000A6A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P0_2       , RULL(0x80000A6A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P0_3         , RULL(0x80000E6A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P0_3       , RULL(0x80000E6A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P0_3       , RULL(0x80000E6A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P0_4         , RULL(0x8000126A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P0_4       , RULL(0x8000126A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P0_4       , RULL(0x8000126A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P1_0       , RULL(0x8000026A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P1_0       , RULL(0x8000026A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P1_1       , RULL(0x8000066A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P1_1       , RULL(0x8000066A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P1_2       , RULL(0x80000A6A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P1_2       , RULL(0x80000A6A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P1_3       , RULL(0x80000E6A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P1_3       , RULL(0x80000E6A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P1_4       , RULL(0x8000126A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P1_4       , RULL(0x8000126A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P2_0       , RULL(0x8000026A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P2_0       , RULL(0x8000026A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P2_1       , RULL(0x8000066A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P2_1       , RULL(0x8000066A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P2_2       , RULL(0x80000A6A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P2_2       , RULL(0x80000A6A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P2_3       , RULL(0x80000E6A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P2_3       , RULL(0x80000E6A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P2_4       , RULL(0x8000126A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P2_4       , RULL(0x8000126A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P3_0       , RULL(0x8000026A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P3_0       , RULL(0x8000026A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P3_1       , RULL(0x8000066A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P3_1       , RULL(0x8000066A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P3_2       , RULL(0x80000A6A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P3_2       , RULL(0x80000A6A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P3_3       , RULL(0x80000E6A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P3_3       , RULL(0x80000E6A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P3_4       , RULL(0x8000126A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR2_P3_4       , RULL(0x8000126A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P0_0         , RULL(0x8000036A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P0_0       , RULL(0x8000036A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P0_0       , RULL(0x8000036A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P0_1         , RULL(0x8000076A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P0_1       , RULL(0x8000076A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P0_1       , RULL(0x8000076A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P0_2         , RULL(0x80000B6A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P0_2       , RULL(0x80000B6A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P0_2       , RULL(0x80000B6A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P0_3         , RULL(0x80000F6A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P0_3       , RULL(0x80000F6A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P0_3       , RULL(0x80000F6A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P0_4         , RULL(0x8000136A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P0_4       , RULL(0x8000136A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P0_4       , RULL(0x8000136A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P1_0       , RULL(0x8000036A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P1_0       , RULL(0x8000036A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P1_1       , RULL(0x8000076A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P1_1       , RULL(0x8000076A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P1_2       , RULL(0x80000B6A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P1_2       , RULL(0x80000B6A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P1_3       , RULL(0x80000F6A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P1_3       , RULL(0x80000F6A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P1_4       , RULL(0x8000136A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P1_4       , RULL(0x8000136A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P2_0       , RULL(0x8000036A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P2_0       , RULL(0x8000036A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P2_1       , RULL(0x8000076A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P2_1       , RULL(0x8000076A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P2_2       , RULL(0x80000B6A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P2_2       , RULL(0x80000B6A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P2_3       , RULL(0x80000F6A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P2_3       , RULL(0x80000F6A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P2_4       , RULL(0x8000136A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P2_4       , RULL(0x8000136A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P3_0       , RULL(0x8000036A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P3_0       , RULL(0x8000036A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P3_1       , RULL(0x8000076A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P3_1       , RULL(0x8000076A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P3_2       , RULL(0x80000B6A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P3_2       , RULL(0x80000B6A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P3_3       , RULL(0x80000F6A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P3_3       , RULL(0x80000F6A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P3_4       , RULL(0x8000136A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE10_RANK_PAIR3_P3_4       , RULL(0x8000136A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P0_0         , RULL(0x8000006B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P0_0       , RULL(0x8000006B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P0_0       , RULL(0x8000006B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P0_1         , RULL(0x8000046B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P0_1       , RULL(0x8000046B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P0_1       , RULL(0x8000046B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P0_2         , RULL(0x8000086B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P0_2       , RULL(0x8000086B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P0_2       , RULL(0x8000086B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P0_3         , RULL(0x80000C6B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P0_3       , RULL(0x80000C6B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P0_3       , RULL(0x80000C6B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P0_4         , RULL(0x8000106B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P0_4       , RULL(0x8000106B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P0_4       , RULL(0x8000106B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P1_0       , RULL(0x8000006B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P1_0       , RULL(0x8000006B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P1_1       , RULL(0x8000046B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P1_1       , RULL(0x8000046B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P1_2       , RULL(0x8000086B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P1_2       , RULL(0x8000086B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P1_3       , RULL(0x80000C6B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P1_3       , RULL(0x80000C6B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P1_4       , RULL(0x8000106B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P1_4       , RULL(0x8000106B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P2_0       , RULL(0x8000006B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P2_0       , RULL(0x8000006B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P2_1       , RULL(0x8000046B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P2_1       , RULL(0x8000046B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P2_2       , RULL(0x8000086B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P2_2       , RULL(0x8000086B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P2_3       , RULL(0x80000C6B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P2_3       , RULL(0x80000C6B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P2_4       , RULL(0x8000106B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P2_4       , RULL(0x8000106B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P3_0       , RULL(0x8000006B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P3_0       , RULL(0x8000006B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P3_1       , RULL(0x8000046B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P3_1       , RULL(0x8000046B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P3_2       , RULL(0x8000086B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P3_2       , RULL(0x8000086B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P3_3       , RULL(0x80000C6B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P3_3       , RULL(0x80000C6B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P3_4       , RULL(0x8000106B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR0_P3_4       , RULL(0x8000106B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P0_0         , RULL(0x8000016B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P0_0       , RULL(0x8000016B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P0_0       , RULL(0x8000016B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P0_1         , RULL(0x8000056B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P0_1       , RULL(0x8000056B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P0_1       , RULL(0x8000056B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P0_2         , RULL(0x8000096B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P0_2       , RULL(0x8000096B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P0_2       , RULL(0x8000096B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P0_3         , RULL(0x80000D6B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P0_3       , RULL(0x80000D6B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P0_3       , RULL(0x80000D6B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P0_4         , RULL(0x8000116B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P0_4       , RULL(0x8000116B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P0_4       , RULL(0x8000116B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P1_0       , RULL(0x8000016B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P1_0       , RULL(0x8000016B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P1_1       , RULL(0x8000056B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P1_1       , RULL(0x8000056B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P1_2       , RULL(0x8000096B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P1_2       , RULL(0x8000096B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P1_3       , RULL(0x80000D6B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P1_3       , RULL(0x80000D6B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P1_4       , RULL(0x8000116B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P1_4       , RULL(0x8000116B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P2_0       , RULL(0x8000016B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P2_0       , RULL(0x8000016B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P2_1       , RULL(0x8000056B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P2_1       , RULL(0x8000056B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P2_2       , RULL(0x8000096B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P2_2       , RULL(0x8000096B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P2_3       , RULL(0x80000D6B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P2_3       , RULL(0x80000D6B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P2_4       , RULL(0x8000116B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P2_4       , RULL(0x8000116B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P3_0       , RULL(0x8000016B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P3_0       , RULL(0x8000016B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P3_1       , RULL(0x8000056B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P3_1       , RULL(0x8000056B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P3_2       , RULL(0x8000096B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P3_2       , RULL(0x8000096B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P3_3       , RULL(0x80000D6B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P3_3       , RULL(0x80000D6B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P3_4       , RULL(0x8000116B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR1_P3_4       , RULL(0x8000116B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P0_0         , RULL(0x8000026B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P0_0       , RULL(0x8000026B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P0_0       , RULL(0x8000026B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P0_1         , RULL(0x8000066B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P0_1       , RULL(0x8000066B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P0_1       , RULL(0x8000066B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P0_2         , RULL(0x80000A6B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P0_2       , RULL(0x80000A6B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P0_2       , RULL(0x80000A6B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P0_3         , RULL(0x80000E6B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P0_3       , RULL(0x80000E6B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P0_3       , RULL(0x80000E6B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P0_4         , RULL(0x8000126B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P0_4       , RULL(0x8000126B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P0_4       , RULL(0x8000126B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P1_0       , RULL(0x8000026B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P1_0       , RULL(0x8000026B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P1_1       , RULL(0x8000066B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P1_1       , RULL(0x8000066B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P1_2       , RULL(0x80000A6B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P1_2       , RULL(0x80000A6B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P1_3       , RULL(0x80000E6B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P1_3       , RULL(0x80000E6B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P1_4       , RULL(0x8000126B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P1_4       , RULL(0x8000126B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P2_0       , RULL(0x8000026B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P2_0       , RULL(0x8000026B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P2_1       , RULL(0x8000066B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P2_1       , RULL(0x8000066B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P2_2       , RULL(0x80000A6B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P2_2       , RULL(0x80000A6B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P2_3       , RULL(0x80000E6B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P2_3       , RULL(0x80000E6B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P2_4       , RULL(0x8000126B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P2_4       , RULL(0x8000126B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P3_0       , RULL(0x8000026B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P3_0       , RULL(0x8000026B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P3_1       , RULL(0x8000066B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P3_1       , RULL(0x8000066B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P3_2       , RULL(0x80000A6B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P3_2       , RULL(0x80000A6B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P3_3       , RULL(0x80000E6B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P3_3       , RULL(0x80000E6B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P3_4       , RULL(0x8000126B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR2_P3_4       , RULL(0x8000126B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P0_0         , RULL(0x8000036B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P0_0       , RULL(0x8000036B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P0_0       , RULL(0x8000036B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P0_1         , RULL(0x8000076B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P0_1       , RULL(0x8000076B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P0_1       , RULL(0x8000076B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P0_2         , RULL(0x80000B6B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P0_2       , RULL(0x80000B6B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P0_2       , RULL(0x80000B6B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P0_3         , RULL(0x80000F6B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P0_3       , RULL(0x80000F6B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P0_3       , RULL(0x80000F6B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P0_4         , RULL(0x8000136B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P0_4       , RULL(0x8000136B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P0_4       , RULL(0x8000136B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P1_0       , RULL(0x8000036B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P1_0       , RULL(0x8000036B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P1_1       , RULL(0x8000076B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P1_1       , RULL(0x8000076B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P1_2       , RULL(0x80000B6B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P1_2       , RULL(0x80000B6B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P1_3       , RULL(0x80000F6B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P1_3       , RULL(0x80000F6B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P1_4       , RULL(0x8000136B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P1_4       , RULL(0x8000136B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P2_0       , RULL(0x8000036B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P2_0       , RULL(0x8000036B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P2_1       , RULL(0x8000076B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P2_1       , RULL(0x8000076B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P2_2       , RULL(0x80000B6B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P2_2       , RULL(0x80000B6B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P2_3       , RULL(0x80000F6B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P2_3       , RULL(0x80000F6B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P2_4       , RULL(0x8000136B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P2_4       , RULL(0x8000136B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P3_0       , RULL(0x8000036B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P3_0       , RULL(0x8000036B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P3_1       , RULL(0x8000076B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P3_1       , RULL(0x8000076B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P3_2       , RULL(0x80000B6B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P3_2       , RULL(0x80000B6B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P3_3       , RULL(0x80000F6B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P3_3       , RULL(0x80000F6B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P3_4       , RULL(0x8000136B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE11_RANK_PAIR3_P3_4       , RULL(0x8000136B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P0_0          , RULL(0x800000610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P0_0        , RULL(0x800000610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P0_0        , RULL(0x800000610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P0_1          , RULL(0x800004610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P0_1        , RULL(0x800004610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P0_1        , RULL(0x800004610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P0_2          , RULL(0x800008610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P0_2        , RULL(0x800008610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P0_2        , RULL(0x800008610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P0_3          , RULL(0x80000C610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P0_3        , RULL(0x80000C610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P0_3        , RULL(0x80000C610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P0_4          , RULL(0x800010610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P0_4        , RULL(0x800010610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P0_4        , RULL(0x800010610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P1_0        , RULL(0x800000610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P1_0        , RULL(0x800000610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P1_1        , RULL(0x800004610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P1_1        , RULL(0x800004610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P1_2        , RULL(0x800008610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P1_2        , RULL(0x800008610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P1_3        , RULL(0x80000C610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P1_3        , RULL(0x80000C610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P1_4        , RULL(0x800010610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P1_4        , RULL(0x800010610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P2_0        , RULL(0x800000610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P2_0        , RULL(0x800000610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P2_1        , RULL(0x800004610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P2_1        , RULL(0x800004610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P2_2        , RULL(0x800008610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P2_2        , RULL(0x800008610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P2_3        , RULL(0x80000C610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P2_3        , RULL(0x80000C610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P2_4        , RULL(0x800010610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P2_4        , RULL(0x800010610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P3_0        , RULL(0x8000006107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P3_0        , RULL(0x8000006108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P3_1        , RULL(0x8000046107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P3_1        , RULL(0x8000046108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P3_2        , RULL(0x8000086107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P3_2        , RULL(0x8000086108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P3_3        , RULL(0x80000C6107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P3_3        , RULL(0x80000C6108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P3_4        , RULL(0x8000106107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR0_P3_4        , RULL(0x8000106108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P0_0          , RULL(0x800001610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P0_0        , RULL(0x800001610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P0_0        , RULL(0x800001610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P0_1          , RULL(0x800005610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P0_1        , RULL(0x800005610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P0_1        , RULL(0x800005610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P0_2          , RULL(0x800009610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P0_2        , RULL(0x800009610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P0_2        , RULL(0x800009610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P0_3          , RULL(0x80000D610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P0_3        , RULL(0x80000D610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P0_3        , RULL(0x80000D610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P0_4          , RULL(0x800011610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P0_4        , RULL(0x800011610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P0_4        , RULL(0x800011610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P1_0        , RULL(0x800001610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P1_0        , RULL(0x800001610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P1_1        , RULL(0x800005610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P1_1        , RULL(0x800005610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P1_2        , RULL(0x800009610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P1_2        , RULL(0x800009610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P1_3        , RULL(0x80000D610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P1_3        , RULL(0x80000D610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P1_4        , RULL(0x800011610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P1_4        , RULL(0x800011610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P2_0        , RULL(0x800001610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P2_0        , RULL(0x800001610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P2_1        , RULL(0x800005610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P2_1        , RULL(0x800005610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P2_2        , RULL(0x800009610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P2_2        , RULL(0x800009610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P2_3        , RULL(0x80000D610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P2_3        , RULL(0x80000D610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P2_4        , RULL(0x800011610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P2_4        , RULL(0x800011610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P3_0        , RULL(0x8000016107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P3_0        , RULL(0x8000016108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P3_1        , RULL(0x8000056107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P3_1        , RULL(0x8000056108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P3_2        , RULL(0x8000096107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P3_2        , RULL(0x8000096108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P3_3        , RULL(0x80000D6107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P3_3        , RULL(0x80000D6108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P3_4        , RULL(0x8000116107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR1_P3_4        , RULL(0x8000116108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P0_0          , RULL(0x800002610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P0_0        , RULL(0x800002610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P0_0        , RULL(0x800002610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P0_1          , RULL(0x800006610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P0_1        , RULL(0x800006610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P0_1        , RULL(0x800006610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P0_2          , RULL(0x80000A610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P0_2        , RULL(0x80000A610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P0_2        , RULL(0x80000A610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P0_3          , RULL(0x80000E610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P0_3        , RULL(0x80000E610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P0_3        , RULL(0x80000E610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P0_4          , RULL(0x800012610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P0_4        , RULL(0x800012610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P0_4        , RULL(0x800012610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P1_0        , RULL(0x800002610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P1_0        , RULL(0x800002610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P1_1        , RULL(0x800006610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P1_1        , RULL(0x800006610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P1_2        , RULL(0x80000A610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P1_2        , RULL(0x80000A610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P1_3        , RULL(0x80000E610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P1_3        , RULL(0x80000E610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P1_4        , RULL(0x800012610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P1_4        , RULL(0x800012610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P2_0        , RULL(0x800002610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P2_0        , RULL(0x800002610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P2_1        , RULL(0x800006610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P2_1        , RULL(0x800006610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P2_2        , RULL(0x80000A610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P2_2        , RULL(0x80000A610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P2_3        , RULL(0x80000E610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P2_3        , RULL(0x80000E610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P2_4        , RULL(0x800012610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P2_4        , RULL(0x800012610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P3_0        , RULL(0x8000026107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P3_0        , RULL(0x8000026108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P3_1        , RULL(0x8000066107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P3_1        , RULL(0x8000066108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P3_2        , RULL(0x80000A6107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P3_2        , RULL(0x80000A6108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P3_3        , RULL(0x80000E6107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P3_3        , RULL(0x80000E6108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P3_4        , RULL(0x8000126107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR2_P3_4        , RULL(0x8000126108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P0_0          , RULL(0x800003610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P0_0        , RULL(0x800003610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P0_0        , RULL(0x800003610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P0_1          , RULL(0x800007610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P0_1        , RULL(0x800007610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P0_1        , RULL(0x800007610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P0_2          , RULL(0x80000B610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P0_2        , RULL(0x80000B610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P0_2        , RULL(0x80000B610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P0_3          , RULL(0x80000F610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P0_3        , RULL(0x80000F610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P0_3        , RULL(0x80000F610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P0_4          , RULL(0x800013610701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P0_4        , RULL(0x800013610701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P0_4        , RULL(0x800013610801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P1_0        , RULL(0x800003610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P1_0        , RULL(0x800003610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P1_1        , RULL(0x800007610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P1_1        , RULL(0x800007610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P1_2        , RULL(0x80000B610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P1_2        , RULL(0x80000B610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P1_3        , RULL(0x80000F610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P1_3        , RULL(0x80000F610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P1_4        , RULL(0x800013610701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P1_4        , RULL(0x800013610801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P2_0        , RULL(0x800003610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P2_0        , RULL(0x800003610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P2_1        , RULL(0x800007610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P2_1        , RULL(0x800007610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P2_2        , RULL(0x80000B610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P2_2        , RULL(0x80000B610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P2_3        , RULL(0x80000F610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P2_3        , RULL(0x80000F610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P2_4        , RULL(0x800013610701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P2_4        , RULL(0x800013610801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P3_0        , RULL(0x8000036107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P3_0        , RULL(0x8000036108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P3_1        , RULL(0x8000076107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P3_1        , RULL(0x8000076108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P3_2        , RULL(0x80000B6107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P3_2        , RULL(0x80000B6108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P3_3        , RULL(0x80000F6107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P3_3        , RULL(0x80000F6108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P3_4        , RULL(0x8000136107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE1_RANK_PAIR3_P3_4        , RULL(0x8000136108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P0_0          , RULL(0x800000620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P0_0        , RULL(0x800000620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P0_0        , RULL(0x800000620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P0_1          , RULL(0x800004620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P0_1        , RULL(0x800004620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P0_1        , RULL(0x800004620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P0_2          , RULL(0x800008620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P0_2        , RULL(0x800008620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P0_2        , RULL(0x800008620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P0_3          , RULL(0x80000C620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P0_3        , RULL(0x80000C620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P0_3        , RULL(0x80000C620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P0_4          , RULL(0x800010620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P0_4        , RULL(0x800010620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P0_4        , RULL(0x800010620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P1_0        , RULL(0x800000620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P1_0        , RULL(0x800000620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P1_1        , RULL(0x800004620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P1_1        , RULL(0x800004620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P1_2        , RULL(0x800008620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P1_2        , RULL(0x800008620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P1_3        , RULL(0x80000C620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P1_3        , RULL(0x80000C620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P1_4        , RULL(0x800010620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P1_4        , RULL(0x800010620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P2_0        , RULL(0x800000620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P2_0        , RULL(0x800000620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P2_1        , RULL(0x800004620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P2_1        , RULL(0x800004620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P2_2        , RULL(0x800008620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P2_2        , RULL(0x800008620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P2_3        , RULL(0x80000C620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P2_3        , RULL(0x80000C620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P2_4        , RULL(0x800010620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P2_4        , RULL(0x800010620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P3_0        , RULL(0x8000006207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P3_0        , RULL(0x8000006208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P3_1        , RULL(0x8000046207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P3_1        , RULL(0x8000046208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P3_2        , RULL(0x8000086207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P3_2        , RULL(0x8000086208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P3_3        , RULL(0x80000C6207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P3_3        , RULL(0x80000C6208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P3_4        , RULL(0x8000106207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR0_P3_4        , RULL(0x8000106208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P0_0          , RULL(0x800001620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P0_0        , RULL(0x800001620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P0_0        , RULL(0x800001620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P0_1          , RULL(0x800005620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P0_1        , RULL(0x800005620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P0_1        , RULL(0x800005620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P0_2          , RULL(0x800009620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P0_2        , RULL(0x800009620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P0_2        , RULL(0x800009620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P0_3          , RULL(0x80000D620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P0_3        , RULL(0x80000D620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P0_3        , RULL(0x80000D620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P0_4          , RULL(0x800011620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P0_4        , RULL(0x800011620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P0_4        , RULL(0x800011620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P1_0        , RULL(0x800001620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P1_0        , RULL(0x800001620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P1_1        , RULL(0x800005620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P1_1        , RULL(0x800005620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P1_2        , RULL(0x800009620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P1_2        , RULL(0x800009620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P1_3        , RULL(0x80000D620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P1_3        , RULL(0x80000D620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P1_4        , RULL(0x800011620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P1_4        , RULL(0x800011620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P2_0        , RULL(0x800001620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P2_0        , RULL(0x800001620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P2_1        , RULL(0x800005620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P2_1        , RULL(0x800005620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P2_2        , RULL(0x800009620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P2_2        , RULL(0x800009620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P2_3        , RULL(0x80000D620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P2_3        , RULL(0x80000D620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P2_4        , RULL(0x800011620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P2_4        , RULL(0x800011620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P3_0        , RULL(0x8000016207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P3_0        , RULL(0x8000016208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P3_1        , RULL(0x8000056207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P3_1        , RULL(0x8000056208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P3_2        , RULL(0x8000096207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P3_2        , RULL(0x8000096208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P3_3        , RULL(0x80000D6207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P3_3        , RULL(0x80000D6208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P3_4        , RULL(0x8000116207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR1_P3_4        , RULL(0x8000116208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P0_0          , RULL(0x800002620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P0_0        , RULL(0x800002620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P0_0        , RULL(0x800002620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P0_1          , RULL(0x800006620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P0_1        , RULL(0x800006620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P0_1        , RULL(0x800006620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P0_2          , RULL(0x80000A620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P0_2        , RULL(0x80000A620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P0_2        , RULL(0x80000A620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P0_3          , RULL(0x80000E620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P0_3        , RULL(0x80000E620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P0_3        , RULL(0x80000E620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P0_4          , RULL(0x800012620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P0_4        , RULL(0x800012620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P0_4        , RULL(0x800012620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P1_0        , RULL(0x800002620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P1_0        , RULL(0x800002620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P1_1        , RULL(0x800006620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P1_1        , RULL(0x800006620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P1_2        , RULL(0x80000A620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P1_2        , RULL(0x80000A620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P1_3        , RULL(0x80000E620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P1_3        , RULL(0x80000E620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P1_4        , RULL(0x800012620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P1_4        , RULL(0x800012620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P2_0        , RULL(0x800002620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P2_0        , RULL(0x800002620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P2_1        , RULL(0x800006620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P2_1        , RULL(0x800006620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P2_2        , RULL(0x80000A620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P2_2        , RULL(0x80000A620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P2_3        , RULL(0x80000E620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P2_3        , RULL(0x80000E620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P2_4        , RULL(0x800012620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P2_4        , RULL(0x800012620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P3_0        , RULL(0x8000026207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P3_0        , RULL(0x8000026208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P3_1        , RULL(0x8000066207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P3_1        , RULL(0x8000066208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P3_2        , RULL(0x80000A6207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P3_2        , RULL(0x80000A6208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P3_3        , RULL(0x80000E6207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P3_3        , RULL(0x80000E6208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P3_4        , RULL(0x8000126207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR2_P3_4        , RULL(0x8000126208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P0_0          , RULL(0x800003620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P0_0        , RULL(0x800003620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P0_0        , RULL(0x800003620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P0_1          , RULL(0x800007620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P0_1        , RULL(0x800007620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P0_1        , RULL(0x800007620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P0_2          , RULL(0x80000B620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P0_2        , RULL(0x80000B620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P0_2        , RULL(0x80000B620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P0_3          , RULL(0x80000F620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P0_3        , RULL(0x80000F620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P0_3        , RULL(0x80000F620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P0_4          , RULL(0x800013620701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P0_4        , RULL(0x800013620701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P0_4        , RULL(0x800013620801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P1_0        , RULL(0x800003620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P1_0        , RULL(0x800003620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P1_1        , RULL(0x800007620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P1_1        , RULL(0x800007620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P1_2        , RULL(0x80000B620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P1_2        , RULL(0x80000B620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P1_3        , RULL(0x80000F620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P1_3        , RULL(0x80000F620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P1_4        , RULL(0x800013620701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P1_4        , RULL(0x800013620801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P2_0        , RULL(0x800003620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P2_0        , RULL(0x800003620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P2_1        , RULL(0x800007620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P2_1        , RULL(0x800007620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P2_2        , RULL(0x80000B620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P2_2        , RULL(0x80000B620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P2_3        , RULL(0x80000F620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P2_3        , RULL(0x80000F620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P2_4        , RULL(0x800013620701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P2_4        , RULL(0x800013620801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P3_0        , RULL(0x8000036207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P3_0        , RULL(0x8000036208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P3_1        , RULL(0x8000076207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P3_1        , RULL(0x8000076208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P3_2        , RULL(0x80000B6207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P3_2        , RULL(0x80000B6208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P3_3        , RULL(0x80000F6207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P3_3        , RULL(0x80000F6208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P3_4        , RULL(0x8000136207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE2_RANK_PAIR3_P3_4        , RULL(0x8000136208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P0_0          , RULL(0x800000630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P0_0        , RULL(0x800000630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P0_0        , RULL(0x800000630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P0_1          , RULL(0x800004630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P0_1        , RULL(0x800004630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P0_1        , RULL(0x800004630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P0_2          , RULL(0x800008630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P0_2        , RULL(0x800008630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P0_2        , RULL(0x800008630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P0_3          , RULL(0x80000C630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P0_3        , RULL(0x80000C630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P0_3        , RULL(0x80000C630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P0_4          , RULL(0x800010630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P0_4        , RULL(0x800010630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P0_4        , RULL(0x800010630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P1_0        , RULL(0x800000630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P1_0        , RULL(0x800000630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P1_1        , RULL(0x800004630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P1_1        , RULL(0x800004630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P1_2        , RULL(0x800008630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P1_2        , RULL(0x800008630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P1_3        , RULL(0x80000C630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P1_3        , RULL(0x80000C630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P1_4        , RULL(0x800010630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P1_4        , RULL(0x800010630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P2_0        , RULL(0x800000630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P2_0        , RULL(0x800000630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P2_1        , RULL(0x800004630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P2_1        , RULL(0x800004630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P2_2        , RULL(0x800008630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P2_2        , RULL(0x800008630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P2_3        , RULL(0x80000C630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P2_3        , RULL(0x80000C630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P2_4        , RULL(0x800010630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P2_4        , RULL(0x800010630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P3_0        , RULL(0x8000006307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P3_0        , RULL(0x8000006308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P3_1        , RULL(0x8000046307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P3_1        , RULL(0x8000046308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P3_2        , RULL(0x8000086307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P3_2        , RULL(0x8000086308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P3_3        , RULL(0x80000C6307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P3_3        , RULL(0x80000C6308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P3_4        , RULL(0x8000106307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR0_P3_4        , RULL(0x8000106308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P0_0          , RULL(0x800001630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P0_0        , RULL(0x800001630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P0_0        , RULL(0x800001630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P0_1          , RULL(0x800005630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P0_1        , RULL(0x800005630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P0_1        , RULL(0x800005630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P0_2          , RULL(0x800009630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P0_2        , RULL(0x800009630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P0_2        , RULL(0x800009630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P0_3          , RULL(0x80000D630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P0_3        , RULL(0x80000D630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P0_3        , RULL(0x80000D630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P0_4          , RULL(0x800011630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P0_4        , RULL(0x800011630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P0_4        , RULL(0x800011630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P1_0        , RULL(0x800001630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P1_0        , RULL(0x800001630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P1_1        , RULL(0x800005630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P1_1        , RULL(0x800005630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P1_2        , RULL(0x800009630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P1_2        , RULL(0x800009630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P1_3        , RULL(0x80000D630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P1_3        , RULL(0x80000D630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P1_4        , RULL(0x800011630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P1_4        , RULL(0x800011630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P2_0        , RULL(0x800001630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P2_0        , RULL(0x800001630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P2_1        , RULL(0x800005630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P2_1        , RULL(0x800005630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P2_2        , RULL(0x800009630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P2_2        , RULL(0x800009630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P2_3        , RULL(0x80000D630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P2_3        , RULL(0x80000D630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P2_4        , RULL(0x800011630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P2_4        , RULL(0x800011630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P3_0        , RULL(0x8000016307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P3_0        , RULL(0x8000016308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P3_1        , RULL(0x8000056307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P3_1        , RULL(0x8000056308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P3_2        , RULL(0x8000096307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P3_2        , RULL(0x8000096308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P3_3        , RULL(0x80000D6307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P3_3        , RULL(0x80000D6308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P3_4        , RULL(0x8000116307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR1_P3_4        , RULL(0x8000116308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P0_0          , RULL(0x800002630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P0_0        , RULL(0x800002630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P0_0        , RULL(0x800002630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P0_1          , RULL(0x800006630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P0_1        , RULL(0x800006630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P0_1        , RULL(0x800006630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P0_2          , RULL(0x80000A630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P0_2        , RULL(0x80000A630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P0_2        , RULL(0x80000A630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P0_3          , RULL(0x80000E630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P0_3        , RULL(0x80000E630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P0_3        , RULL(0x80000E630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P0_4          , RULL(0x800012630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P0_4        , RULL(0x800012630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P0_4        , RULL(0x800012630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P1_0        , RULL(0x800002630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P1_0        , RULL(0x800002630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P1_1        , RULL(0x800006630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P1_1        , RULL(0x800006630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P1_2        , RULL(0x80000A630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P1_2        , RULL(0x80000A630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P1_3        , RULL(0x80000E630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P1_3        , RULL(0x80000E630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P1_4        , RULL(0x800012630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P1_4        , RULL(0x800012630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P2_0        , RULL(0x800002630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P2_0        , RULL(0x800002630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P2_1        , RULL(0x800006630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P2_1        , RULL(0x800006630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P2_2        , RULL(0x80000A630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P2_2        , RULL(0x80000A630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P2_3        , RULL(0x80000E630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P2_3        , RULL(0x80000E630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P2_4        , RULL(0x800012630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P2_4        , RULL(0x800012630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P3_0        , RULL(0x8000026307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P3_0        , RULL(0x8000026308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P3_1        , RULL(0x8000066307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P3_1        , RULL(0x8000066308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P3_2        , RULL(0x80000A6307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P3_2        , RULL(0x80000A6308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P3_3        , RULL(0x80000E6307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P3_3        , RULL(0x80000E6308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P3_4        , RULL(0x8000126307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR2_P3_4        , RULL(0x8000126308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P0_0          , RULL(0x800003630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P0_0        , RULL(0x800003630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P0_0        , RULL(0x800003630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P0_1          , RULL(0x800007630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P0_1        , RULL(0x800007630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P0_1        , RULL(0x800007630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P0_2          , RULL(0x80000B630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P0_2        , RULL(0x80000B630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P0_2        , RULL(0x80000B630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P0_3          , RULL(0x80000F630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P0_3        , RULL(0x80000F630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P0_3        , RULL(0x80000F630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P0_4          , RULL(0x800013630701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P0_4        , RULL(0x800013630701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P0_4        , RULL(0x800013630801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P1_0        , RULL(0x800003630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P1_0        , RULL(0x800003630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P1_1        , RULL(0x800007630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P1_1        , RULL(0x800007630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P1_2        , RULL(0x80000B630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P1_2        , RULL(0x80000B630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P1_3        , RULL(0x80000F630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P1_3        , RULL(0x80000F630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P1_4        , RULL(0x800013630701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P1_4        , RULL(0x800013630801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P2_0        , RULL(0x800003630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P2_0        , RULL(0x800003630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P2_1        , RULL(0x800007630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P2_1        , RULL(0x800007630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P2_2        , RULL(0x80000B630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P2_2        , RULL(0x80000B630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P2_3        , RULL(0x80000F630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P2_3        , RULL(0x80000F630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P2_4        , RULL(0x800013630701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P2_4        , RULL(0x800013630801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P3_0        , RULL(0x8000036307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P3_0        , RULL(0x8000036308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P3_1        , RULL(0x8000076307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P3_1        , RULL(0x8000076308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P3_2        , RULL(0x80000B6307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P3_2        , RULL(0x80000B6308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P3_3        , RULL(0x80000F6307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P3_3        , RULL(0x80000F6308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P3_4        , RULL(0x8000136307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE3_RANK_PAIR3_P3_4        , RULL(0x8000136308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P0_0          , RULL(0x800000640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P0_0        , RULL(0x800000640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P0_0        , RULL(0x800000640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P0_1          , RULL(0x800004640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P0_1        , RULL(0x800004640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P0_1        , RULL(0x800004640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P0_2          , RULL(0x800008640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P0_2        , RULL(0x800008640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P0_2        , RULL(0x800008640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P0_3          , RULL(0x80000C640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P0_3        , RULL(0x80000C640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P0_3        , RULL(0x80000C640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P0_4          , RULL(0x800010640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P0_4        , RULL(0x800010640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P0_4        , RULL(0x800010640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P1_0        , RULL(0x800000640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P1_0        , RULL(0x800000640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P1_1        , RULL(0x800004640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P1_1        , RULL(0x800004640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P1_2        , RULL(0x800008640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P1_2        , RULL(0x800008640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P1_3        , RULL(0x80000C640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P1_3        , RULL(0x80000C640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P1_4        , RULL(0x800010640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P1_4        , RULL(0x800010640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P2_0        , RULL(0x800000640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P2_0        , RULL(0x800000640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P2_1        , RULL(0x800004640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P2_1        , RULL(0x800004640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P2_2        , RULL(0x800008640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P2_2        , RULL(0x800008640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P2_3        , RULL(0x80000C640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P2_3        , RULL(0x80000C640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P2_4        , RULL(0x800010640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P2_4        , RULL(0x800010640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P3_0        , RULL(0x8000006407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P3_0        , RULL(0x8000006408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P3_1        , RULL(0x8000046407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P3_1        , RULL(0x8000046408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P3_2        , RULL(0x8000086407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P3_2        , RULL(0x8000086408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P3_3        , RULL(0x80000C6407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P3_3        , RULL(0x80000C6408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P3_4        , RULL(0x8000106407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR0_P3_4        , RULL(0x8000106408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P0_0          , RULL(0x800001640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P0_0        , RULL(0x800001640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P0_0        , RULL(0x800001640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P0_1          , RULL(0x800005640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P0_1        , RULL(0x800005640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P0_1        , RULL(0x800005640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P0_2          , RULL(0x800009640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P0_2        , RULL(0x800009640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P0_2        , RULL(0x800009640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P0_3          , RULL(0x80000D640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P0_3        , RULL(0x80000D640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P0_3        , RULL(0x80000D640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P0_4          , RULL(0x800011640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P0_4        , RULL(0x800011640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P0_4        , RULL(0x800011640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P1_0        , RULL(0x800001640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P1_0        , RULL(0x800001640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P1_1        , RULL(0x800005640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P1_1        , RULL(0x800005640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P1_2        , RULL(0x800009640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P1_2        , RULL(0x800009640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P1_3        , RULL(0x80000D640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P1_3        , RULL(0x80000D640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P1_4        , RULL(0x800011640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P1_4        , RULL(0x800011640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P2_0        , RULL(0x800001640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P2_0        , RULL(0x800001640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P2_1        , RULL(0x800005640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P2_1        , RULL(0x800005640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P2_2        , RULL(0x800009640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P2_2        , RULL(0x800009640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P2_3        , RULL(0x80000D640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P2_3        , RULL(0x80000D640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P2_4        , RULL(0x800011640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P2_4        , RULL(0x800011640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P3_0        , RULL(0x8000016407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P3_0        , RULL(0x8000016408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P3_1        , RULL(0x8000056407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P3_1        , RULL(0x8000056408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P3_2        , RULL(0x8000096407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P3_2        , RULL(0x8000096408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P3_3        , RULL(0x80000D6407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P3_3        , RULL(0x80000D6408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P3_4        , RULL(0x8000116407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR1_P3_4        , RULL(0x8000116408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P0_0          , RULL(0x800002640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P0_0        , RULL(0x800002640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P0_0        , RULL(0x800002640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P0_1          , RULL(0x800006640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P0_1        , RULL(0x800006640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P0_1        , RULL(0x800006640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P0_2          , RULL(0x80000A640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P0_2        , RULL(0x80000A640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P0_2        , RULL(0x80000A640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P0_3          , RULL(0x80000E640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P0_3        , RULL(0x80000E640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P0_3        , RULL(0x80000E640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P0_4          , RULL(0x800012640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P0_4        , RULL(0x800012640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P0_4        , RULL(0x800012640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P1_0        , RULL(0x800002640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P1_0        , RULL(0x800002640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P1_1        , RULL(0x800006640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P1_1        , RULL(0x800006640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P1_2        , RULL(0x80000A640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P1_2        , RULL(0x80000A640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P1_3        , RULL(0x80000E640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P1_3        , RULL(0x80000E640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P1_4        , RULL(0x800012640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P1_4        , RULL(0x800012640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P2_0        , RULL(0x800002640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P2_0        , RULL(0x800002640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P2_1        , RULL(0x800006640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P2_1        , RULL(0x800006640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P2_2        , RULL(0x80000A640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P2_2        , RULL(0x80000A640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P2_3        , RULL(0x80000E640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P2_3        , RULL(0x80000E640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P2_4        , RULL(0x800012640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P2_4        , RULL(0x800012640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P3_0        , RULL(0x8000026407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P3_0        , RULL(0x8000026408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P3_1        , RULL(0x8000066407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P3_1        , RULL(0x8000066408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P3_2        , RULL(0x80000A6407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P3_2        , RULL(0x80000A6408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P3_3        , RULL(0x80000E6407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P3_3        , RULL(0x80000E6408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P3_4        , RULL(0x8000126407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR2_P3_4        , RULL(0x8000126408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P0_0          , RULL(0x800003640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P0_0        , RULL(0x800003640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P0_0        , RULL(0x800003640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P0_1          , RULL(0x800007640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P0_1        , RULL(0x800007640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P0_1        , RULL(0x800007640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P0_2          , RULL(0x80000B640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P0_2        , RULL(0x80000B640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P0_2        , RULL(0x80000B640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P0_3          , RULL(0x80000F640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P0_3        , RULL(0x80000F640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P0_3        , RULL(0x80000F640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P0_4          , RULL(0x800013640701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P0_4        , RULL(0x800013640701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P0_4        , RULL(0x800013640801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P1_0        , RULL(0x800003640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P1_0        , RULL(0x800003640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P1_1        , RULL(0x800007640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P1_1        , RULL(0x800007640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P1_2        , RULL(0x80000B640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P1_2        , RULL(0x80000B640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P1_3        , RULL(0x80000F640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P1_3        , RULL(0x80000F640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P1_4        , RULL(0x800013640701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P1_4        , RULL(0x800013640801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P2_0        , RULL(0x800003640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P2_0        , RULL(0x800003640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P2_1        , RULL(0x800007640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P2_1        , RULL(0x800007640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P2_2        , RULL(0x80000B640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P2_2        , RULL(0x80000B640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P2_3        , RULL(0x80000F640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P2_3        , RULL(0x80000F640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P2_4        , RULL(0x800013640701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P2_4        , RULL(0x800013640801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P3_0        , RULL(0x8000036407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P3_0        , RULL(0x8000036408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P3_1        , RULL(0x8000076407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P3_1        , RULL(0x8000076408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P3_2        , RULL(0x80000B6407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P3_2        , RULL(0x80000B6408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P3_3        , RULL(0x80000F6407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P3_3        , RULL(0x80000F6408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P3_4        , RULL(0x8000136407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE4_RANK_PAIR3_P3_4        , RULL(0x8000136408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P0_0          , RULL(0x800000650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P0_0        , RULL(0x800000650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P0_0        , RULL(0x800000650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P0_1          , RULL(0x800004650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P0_1        , RULL(0x800004650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P0_1        , RULL(0x800004650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P0_2          , RULL(0x800008650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P0_2        , RULL(0x800008650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P0_2        , RULL(0x800008650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P0_3          , RULL(0x80000C650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P0_3        , RULL(0x80000C650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P0_3        , RULL(0x80000C650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P0_4          , RULL(0x800010650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P0_4        , RULL(0x800010650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P0_4        , RULL(0x800010650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P1_0        , RULL(0x800000650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P1_0        , RULL(0x800000650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P1_1        , RULL(0x800004650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P1_1        , RULL(0x800004650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P1_2        , RULL(0x800008650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P1_2        , RULL(0x800008650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P1_3        , RULL(0x80000C650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P1_3        , RULL(0x80000C650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P1_4        , RULL(0x800010650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P1_4        , RULL(0x800010650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P2_0        , RULL(0x800000650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P2_0        , RULL(0x800000650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P2_1        , RULL(0x800004650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P2_1        , RULL(0x800004650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P2_2        , RULL(0x800008650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P2_2        , RULL(0x800008650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P2_3        , RULL(0x80000C650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P2_3        , RULL(0x80000C650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P2_4        , RULL(0x800010650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P2_4        , RULL(0x800010650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P3_0        , RULL(0x8000006507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P3_0        , RULL(0x8000006508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P3_1        , RULL(0x8000046507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P3_1        , RULL(0x8000046508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P3_2        , RULL(0x8000086507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P3_2        , RULL(0x8000086508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P3_3        , RULL(0x80000C6507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P3_3        , RULL(0x80000C6508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P3_4        , RULL(0x8000106507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR0_P3_4        , RULL(0x8000106508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P0_0          , RULL(0x800001650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P0_0        , RULL(0x800001650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P0_0        , RULL(0x800001650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P0_1          , RULL(0x800005650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P0_1        , RULL(0x800005650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P0_1        , RULL(0x800005650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P0_2          , RULL(0x800009650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P0_2        , RULL(0x800009650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P0_2        , RULL(0x800009650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P0_3          , RULL(0x80000D650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P0_3        , RULL(0x80000D650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P0_3        , RULL(0x80000D650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P0_4          , RULL(0x800011650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P0_4        , RULL(0x800011650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P0_4        , RULL(0x800011650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P1_0        , RULL(0x800001650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P1_0        , RULL(0x800001650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P1_1        , RULL(0x800005650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P1_1        , RULL(0x800005650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P1_2        , RULL(0x800009650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P1_2        , RULL(0x800009650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P1_3        , RULL(0x80000D650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P1_3        , RULL(0x80000D650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P1_4        , RULL(0x800011650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P1_4        , RULL(0x800011650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P2_0        , RULL(0x800001650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P2_0        , RULL(0x800001650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P2_1        , RULL(0x800005650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P2_1        , RULL(0x800005650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P2_2        , RULL(0x800009650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P2_2        , RULL(0x800009650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P2_3        , RULL(0x80000D650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P2_3        , RULL(0x80000D650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P2_4        , RULL(0x800011650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P2_4        , RULL(0x800011650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P3_0        , RULL(0x8000016507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P3_0        , RULL(0x8000016508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P3_1        , RULL(0x8000056507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P3_1        , RULL(0x8000056508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P3_2        , RULL(0x8000096507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P3_2        , RULL(0x8000096508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P3_3        , RULL(0x80000D6507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P3_3        , RULL(0x80000D6508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P3_4        , RULL(0x8000116507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR1_P3_4        , RULL(0x8000116508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P0_0          , RULL(0x800002650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P0_0        , RULL(0x800002650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P0_0        , RULL(0x800002650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P0_1          , RULL(0x800006650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P0_1        , RULL(0x800006650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P0_1        , RULL(0x800006650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P0_2          , RULL(0x80000A650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P0_2        , RULL(0x80000A650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P0_2        , RULL(0x80000A650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P0_3          , RULL(0x80000E650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P0_3        , RULL(0x80000E650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P0_3        , RULL(0x80000E650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P0_4          , RULL(0x800012650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P0_4        , RULL(0x800012650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P0_4        , RULL(0x800012650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P1_0        , RULL(0x800002650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P1_0        , RULL(0x800002650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P1_1        , RULL(0x800006650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P1_1        , RULL(0x800006650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P1_2        , RULL(0x80000A650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P1_2        , RULL(0x80000A650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P1_3        , RULL(0x80000E650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P1_3        , RULL(0x80000E650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P1_4        , RULL(0x800012650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P1_4        , RULL(0x800012650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P2_0        , RULL(0x800002650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P2_0        , RULL(0x800002650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P2_1        , RULL(0x800006650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P2_1        , RULL(0x800006650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P2_2        , RULL(0x80000A650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P2_2        , RULL(0x80000A650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P2_3        , RULL(0x80000E650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P2_3        , RULL(0x80000E650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P2_4        , RULL(0x800012650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P2_4        , RULL(0x800012650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P3_0        , RULL(0x8000026507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P3_0        , RULL(0x8000026508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P3_1        , RULL(0x8000066507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P3_1        , RULL(0x8000066508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P3_2        , RULL(0x80000A6507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P3_2        , RULL(0x80000A6508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P3_3        , RULL(0x80000E6507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P3_3        , RULL(0x80000E6508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P3_4        , RULL(0x8000126507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR2_P3_4        , RULL(0x8000126508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P0_0          , RULL(0x800003650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P0_0        , RULL(0x800003650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P0_0        , RULL(0x800003650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P0_1          , RULL(0x800007650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P0_1        , RULL(0x800007650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P0_1        , RULL(0x800007650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P0_2          , RULL(0x80000B650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P0_2        , RULL(0x80000B650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P0_2        , RULL(0x80000B650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P0_3          , RULL(0x80000F650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P0_3        , RULL(0x80000F650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P0_3        , RULL(0x80000F650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P0_4          , RULL(0x800013650701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P0_4        , RULL(0x800013650701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P0_4        , RULL(0x800013650801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P1_0        , RULL(0x800003650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P1_0        , RULL(0x800003650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P1_1        , RULL(0x800007650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P1_1        , RULL(0x800007650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P1_2        , RULL(0x80000B650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P1_2        , RULL(0x80000B650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P1_3        , RULL(0x80000F650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P1_3        , RULL(0x80000F650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P1_4        , RULL(0x800013650701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P1_4        , RULL(0x800013650801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P2_0        , RULL(0x800003650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P2_0        , RULL(0x800003650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P2_1        , RULL(0x800007650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P2_1        , RULL(0x800007650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P2_2        , RULL(0x80000B650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P2_2        , RULL(0x80000B650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P2_3        , RULL(0x80000F650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P2_3        , RULL(0x80000F650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P2_4        , RULL(0x800013650701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P2_4        , RULL(0x800013650801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P3_0        , RULL(0x8000036507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P3_0        , RULL(0x8000036508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P3_1        , RULL(0x8000076507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P3_1        , RULL(0x8000076508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P3_2        , RULL(0x80000B6507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P3_2        , RULL(0x80000B6508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P3_3        , RULL(0x80000F6507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P3_3        , RULL(0x80000F6508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P3_4        , RULL(0x8000136507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE5_RANK_PAIR3_P3_4        , RULL(0x8000136508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P0_0          , RULL(0x800000660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P0_0        , RULL(0x800000660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P0_0        , RULL(0x800000660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P0_1          , RULL(0x800004660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P0_1        , RULL(0x800004660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P0_1        , RULL(0x800004660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P0_2          , RULL(0x800008660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P0_2        , RULL(0x800008660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P0_2        , RULL(0x800008660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P0_3          , RULL(0x80000C660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P0_3        , RULL(0x80000C660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P0_3        , RULL(0x80000C660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P0_4          , RULL(0x800010660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P0_4        , RULL(0x800010660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P0_4        , RULL(0x800010660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P1_0        , RULL(0x800000660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P1_0        , RULL(0x800000660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P1_1        , RULL(0x800004660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P1_1        , RULL(0x800004660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P1_2        , RULL(0x800008660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P1_2        , RULL(0x800008660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P1_3        , RULL(0x80000C660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P1_3        , RULL(0x80000C660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P1_4        , RULL(0x800010660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P1_4        , RULL(0x800010660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P2_0        , RULL(0x800000660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P2_0        , RULL(0x800000660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P2_1        , RULL(0x800004660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P2_1        , RULL(0x800004660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P2_2        , RULL(0x800008660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P2_2        , RULL(0x800008660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P2_3        , RULL(0x80000C660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P2_3        , RULL(0x80000C660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P2_4        , RULL(0x800010660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P2_4        , RULL(0x800010660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P3_0        , RULL(0x8000006607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P3_0        , RULL(0x8000006608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P3_1        , RULL(0x8000046607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P3_1        , RULL(0x8000046608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P3_2        , RULL(0x8000086607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P3_2        , RULL(0x8000086608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P3_3        , RULL(0x80000C6607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P3_3        , RULL(0x80000C6608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P3_4        , RULL(0x8000106607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR0_P3_4        , RULL(0x8000106608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P0_0          , RULL(0x800001660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P0_0        , RULL(0x800001660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P0_0        , RULL(0x800001660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P0_1          , RULL(0x800005660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P0_1        , RULL(0x800005660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P0_1        , RULL(0x800005660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P0_2          , RULL(0x800009660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P0_2        , RULL(0x800009660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P0_2        , RULL(0x800009660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P0_3          , RULL(0x80000D660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P0_3        , RULL(0x80000D660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P0_3        , RULL(0x80000D660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P0_4          , RULL(0x800011660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P0_4        , RULL(0x800011660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P0_4        , RULL(0x800011660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P1_0        , RULL(0x800001660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P1_0        , RULL(0x800001660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P1_1        , RULL(0x800005660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P1_1        , RULL(0x800005660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P1_2        , RULL(0x800009660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P1_2        , RULL(0x800009660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P1_3        , RULL(0x80000D660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P1_3        , RULL(0x80000D660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P1_4        , RULL(0x800011660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P1_4        , RULL(0x800011660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P2_0        , RULL(0x800001660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P2_0        , RULL(0x800001660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P2_1        , RULL(0x800005660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P2_1        , RULL(0x800005660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P2_2        , RULL(0x800009660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P2_2        , RULL(0x800009660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P2_3        , RULL(0x80000D660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P2_3        , RULL(0x80000D660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P2_4        , RULL(0x800011660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P2_4        , RULL(0x800011660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P3_0        , RULL(0x8000016607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P3_0        , RULL(0x8000016608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P3_1        , RULL(0x8000056607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P3_1        , RULL(0x8000056608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P3_2        , RULL(0x8000096607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P3_2        , RULL(0x8000096608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P3_3        , RULL(0x80000D6607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P3_3        , RULL(0x80000D6608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P3_4        , RULL(0x8000116607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR1_P3_4        , RULL(0x8000116608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P0_0          , RULL(0x800002660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P0_0        , RULL(0x800002660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P0_0        , RULL(0x800002660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P0_1          , RULL(0x800006660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P0_1        , RULL(0x800006660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P0_1        , RULL(0x800006660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P0_2          , RULL(0x80000A660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P0_2        , RULL(0x80000A660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P0_2        , RULL(0x80000A660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P0_3          , RULL(0x80000E660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P0_3        , RULL(0x80000E660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P0_3        , RULL(0x80000E660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P0_4          , RULL(0x800012660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P0_4        , RULL(0x800012660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P0_4        , RULL(0x800012660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P1_0        , RULL(0x800002660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P1_0        , RULL(0x800002660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P1_1        , RULL(0x800006660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P1_1        , RULL(0x800006660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P1_2        , RULL(0x80000A660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P1_2        , RULL(0x80000A660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P1_3        , RULL(0x80000E660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P1_3        , RULL(0x80000E660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P1_4        , RULL(0x800012660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P1_4        , RULL(0x800012660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P2_0        , RULL(0x800002660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P2_0        , RULL(0x800002660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P2_1        , RULL(0x800006660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P2_1        , RULL(0x800006660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P2_2        , RULL(0x80000A660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P2_2        , RULL(0x80000A660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P2_3        , RULL(0x80000E660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P2_3        , RULL(0x80000E660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P2_4        , RULL(0x800012660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P2_4        , RULL(0x800012660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P3_0        , RULL(0x8000026607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P3_0        , RULL(0x8000026608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P3_1        , RULL(0x8000066607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P3_1        , RULL(0x8000066608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P3_2        , RULL(0x80000A6607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P3_2        , RULL(0x80000A6608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P3_3        , RULL(0x80000E6607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P3_3        , RULL(0x80000E6608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P3_4        , RULL(0x8000126607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR2_P3_4        , RULL(0x8000126608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P0_0          , RULL(0x800003660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P0_0        , RULL(0x800003660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P0_0        , RULL(0x800003660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P0_1          , RULL(0x800007660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P0_1        , RULL(0x800007660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P0_1        , RULL(0x800007660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P0_2          , RULL(0x80000B660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P0_2        , RULL(0x80000B660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P0_2        , RULL(0x80000B660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P0_3          , RULL(0x80000F660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P0_3        , RULL(0x80000F660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P0_3        , RULL(0x80000F660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P0_4          , RULL(0x800013660701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P0_4        , RULL(0x800013660701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P0_4        , RULL(0x800013660801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P1_0        , RULL(0x800003660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P1_0        , RULL(0x800003660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P1_1        , RULL(0x800007660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P1_1        , RULL(0x800007660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P1_2        , RULL(0x80000B660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P1_2        , RULL(0x80000B660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P1_3        , RULL(0x80000F660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P1_3        , RULL(0x80000F660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P1_4        , RULL(0x800013660701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P1_4        , RULL(0x800013660801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P2_0        , RULL(0x800003660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P2_0        , RULL(0x800003660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P2_1        , RULL(0x800007660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P2_1        , RULL(0x800007660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P2_2        , RULL(0x80000B660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P2_2        , RULL(0x80000B660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P2_3        , RULL(0x80000F660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P2_3        , RULL(0x80000F660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P2_4        , RULL(0x800013660701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P2_4        , RULL(0x800013660801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P3_0        , RULL(0x8000036607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P3_0        , RULL(0x8000036608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P3_1        , RULL(0x8000076607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P3_1        , RULL(0x8000076608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P3_2        , RULL(0x80000B6607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P3_2        , RULL(0x80000B6608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P3_3        , RULL(0x80000F6607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P3_3        , RULL(0x80000F6608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P3_4        , RULL(0x8000136607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE6_RANK_PAIR3_P3_4        , RULL(0x8000136608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P0_0          , RULL(0x800000670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P0_0        , RULL(0x800000670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P0_0        , RULL(0x800000670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P0_1          , RULL(0x800004670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P0_1        , RULL(0x800004670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P0_1        , RULL(0x800004670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P0_2          , RULL(0x800008670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P0_2        , RULL(0x800008670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P0_2        , RULL(0x800008670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P0_3          , RULL(0x80000C670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P0_3        , RULL(0x80000C670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P0_3        , RULL(0x80000C670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P0_4          , RULL(0x800010670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P0_4        , RULL(0x800010670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P0_4        , RULL(0x800010670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P1_0        , RULL(0x800000670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P1_0        , RULL(0x800000670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P1_1        , RULL(0x800004670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P1_1        , RULL(0x800004670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P1_2        , RULL(0x800008670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P1_2        , RULL(0x800008670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P1_3        , RULL(0x80000C670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P1_3        , RULL(0x80000C670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P1_4        , RULL(0x800010670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P1_4        , RULL(0x800010670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P2_0        , RULL(0x800000670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P2_0        , RULL(0x800000670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P2_1        , RULL(0x800004670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P2_1        , RULL(0x800004670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P2_2        , RULL(0x800008670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P2_2        , RULL(0x800008670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P2_3        , RULL(0x80000C670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P2_3        , RULL(0x80000C670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P2_4        , RULL(0x800010670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P2_4        , RULL(0x800010670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P3_0        , RULL(0x8000006707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P3_0        , RULL(0x8000006708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P3_1        , RULL(0x8000046707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P3_1        , RULL(0x8000046708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P3_2        , RULL(0x8000086707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P3_2        , RULL(0x8000086708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P3_3        , RULL(0x80000C6707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P3_3        , RULL(0x80000C6708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P3_4        , RULL(0x8000106707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR0_P3_4        , RULL(0x8000106708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P0_0          , RULL(0x800001670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P0_0        , RULL(0x800001670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P0_0        , RULL(0x800001670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P0_1          , RULL(0x800005670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P0_1        , RULL(0x800005670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P0_1        , RULL(0x800005670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P0_2          , RULL(0x800009670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P0_2        , RULL(0x800009670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P0_2        , RULL(0x800009670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P0_3          , RULL(0x80000D670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P0_3        , RULL(0x80000D670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P0_3        , RULL(0x80000D670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P0_4          , RULL(0x800011670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P0_4        , RULL(0x800011670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P0_4        , RULL(0x800011670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P1_0        , RULL(0x800001670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P1_0        , RULL(0x800001670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P1_1        , RULL(0x800005670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P1_1        , RULL(0x800005670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P1_2        , RULL(0x800009670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P1_2        , RULL(0x800009670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P1_3        , RULL(0x80000D670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P1_3        , RULL(0x80000D670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P1_4        , RULL(0x800011670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P1_4        , RULL(0x800011670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P2_0        , RULL(0x800001670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P2_0        , RULL(0x800001670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P2_1        , RULL(0x800005670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P2_1        , RULL(0x800005670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P2_2        , RULL(0x800009670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P2_2        , RULL(0x800009670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P2_3        , RULL(0x80000D670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P2_3        , RULL(0x80000D670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P2_4        , RULL(0x800011670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P2_4        , RULL(0x800011670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P3_0        , RULL(0x8000016707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P3_0        , RULL(0x8000016708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P3_1        , RULL(0x8000056707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P3_1        , RULL(0x8000056708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P3_2        , RULL(0x8000096707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P3_2        , RULL(0x8000096708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P3_3        , RULL(0x80000D6707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P3_3        , RULL(0x80000D6708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P3_4        , RULL(0x8000116707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR1_P3_4        , RULL(0x8000116708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P0_0          , RULL(0x800002670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P0_0        , RULL(0x800002670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P0_0        , RULL(0x800002670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P0_1          , RULL(0x800006670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P0_1        , RULL(0x800006670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P0_1        , RULL(0x800006670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P0_2          , RULL(0x80000A670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P0_2        , RULL(0x80000A670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P0_2        , RULL(0x80000A670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P0_3          , RULL(0x80000E670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P0_3        , RULL(0x80000E670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P0_3        , RULL(0x80000E670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P0_4          , RULL(0x800012670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P0_4        , RULL(0x800012670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P0_4        , RULL(0x800012670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P1_0        , RULL(0x800002670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P1_0        , RULL(0x800002670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P1_1        , RULL(0x800006670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P1_1        , RULL(0x800006670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P1_2        , RULL(0x80000A670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P1_2        , RULL(0x80000A670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P1_3        , RULL(0x80000E670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P1_3        , RULL(0x80000E670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P1_4        , RULL(0x800012670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P1_4        , RULL(0x800012670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P2_0        , RULL(0x800002670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P2_0        , RULL(0x800002670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P2_1        , RULL(0x800006670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P2_1        , RULL(0x800006670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P2_2        , RULL(0x80000A670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P2_2        , RULL(0x80000A670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P2_3        , RULL(0x80000E670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P2_3        , RULL(0x80000E670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P2_4        , RULL(0x800012670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P2_4        , RULL(0x800012670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P3_0        , RULL(0x8000026707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P3_0        , RULL(0x8000026708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P3_1        , RULL(0x8000066707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P3_1        , RULL(0x8000066708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P3_2        , RULL(0x80000A6707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P3_2        , RULL(0x80000A6708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P3_3        , RULL(0x80000E6707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P3_3        , RULL(0x80000E6708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P3_4        , RULL(0x8000126707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR2_P3_4        , RULL(0x8000126708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P0_0          , RULL(0x800003670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P0_0        , RULL(0x800003670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P0_0        , RULL(0x800003670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P0_1          , RULL(0x800007670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P0_1        , RULL(0x800007670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P0_1        , RULL(0x800007670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P0_2          , RULL(0x80000B670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P0_2        , RULL(0x80000B670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P0_2        , RULL(0x80000B670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P0_3          , RULL(0x80000F670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P0_3        , RULL(0x80000F670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P0_3        , RULL(0x80000F670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P0_4          , RULL(0x800013670701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P0_4        , RULL(0x800013670701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P0_4        , RULL(0x800013670801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P1_0        , RULL(0x800003670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P1_0        , RULL(0x800003670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P1_1        , RULL(0x800007670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P1_1        , RULL(0x800007670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P1_2        , RULL(0x80000B670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P1_2        , RULL(0x80000B670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P1_3        , RULL(0x80000F670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P1_3        , RULL(0x80000F670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P1_4        , RULL(0x800013670701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P1_4        , RULL(0x800013670801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P2_0        , RULL(0x800003670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P2_0        , RULL(0x800003670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P2_1        , RULL(0x800007670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P2_1        , RULL(0x800007670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P2_2        , RULL(0x80000B670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P2_2        , RULL(0x80000B670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P2_3        , RULL(0x80000F670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P2_3        , RULL(0x80000F670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P2_4        , RULL(0x800013670701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P2_4        , RULL(0x800013670801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P3_0        , RULL(0x8000036707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P3_0        , RULL(0x8000036708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P3_1        , RULL(0x8000076707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P3_1        , RULL(0x8000076708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P3_2        , RULL(0x80000B6707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P3_2        , RULL(0x80000B6708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P3_3        , RULL(0x80000F6707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P3_3        , RULL(0x80000F6708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P3_4        , RULL(0x8000136707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE7_RANK_PAIR3_P3_4        , RULL(0x8000136708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P0_0          , RULL(0x800000680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P0_0        , RULL(0x800000680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P0_0        , RULL(0x800000680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P0_1          , RULL(0x800004680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P0_1        , RULL(0x800004680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P0_1        , RULL(0x800004680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P0_2          , RULL(0x800008680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P0_2        , RULL(0x800008680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P0_2        , RULL(0x800008680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P0_3          , RULL(0x80000C680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P0_3        , RULL(0x80000C680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P0_3        , RULL(0x80000C680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P0_4          , RULL(0x800010680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P0_4        , RULL(0x800010680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P0_4        , RULL(0x800010680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P1_0        , RULL(0x800000680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P1_0        , RULL(0x800000680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P1_1        , RULL(0x800004680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P1_1        , RULL(0x800004680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P1_2        , RULL(0x800008680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P1_2        , RULL(0x800008680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P1_3        , RULL(0x80000C680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P1_3        , RULL(0x80000C680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P1_4        , RULL(0x800010680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P1_4        , RULL(0x800010680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P2_0        , RULL(0x800000680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P2_0        , RULL(0x800000680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P2_1        , RULL(0x800004680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P2_1        , RULL(0x800004680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P2_2        , RULL(0x800008680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P2_2        , RULL(0x800008680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P2_3        , RULL(0x80000C680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P2_3        , RULL(0x80000C680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P2_4        , RULL(0x800010680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P2_4        , RULL(0x800010680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P3_0        , RULL(0x8000006807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P3_0        , RULL(0x8000006808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P3_1        , RULL(0x8000046807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P3_1        , RULL(0x8000046808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P3_2        , RULL(0x8000086807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P3_2        , RULL(0x8000086808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P3_3        , RULL(0x80000C6807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P3_3        , RULL(0x80000C6808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P3_4        , RULL(0x8000106807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR0_P3_4        , RULL(0x8000106808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P0_0          , RULL(0x800001680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P0_0        , RULL(0x800001680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P0_0        , RULL(0x800001680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P0_1          , RULL(0x800005680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P0_1        , RULL(0x800005680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P0_1        , RULL(0x800005680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P0_2          , RULL(0x800009680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P0_2        , RULL(0x800009680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P0_2        , RULL(0x800009680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P0_3          , RULL(0x80000D680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P0_3        , RULL(0x80000D680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P0_3        , RULL(0x80000D680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P0_4          , RULL(0x800011680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P0_4        , RULL(0x800011680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P0_4        , RULL(0x800011680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P1_0        , RULL(0x800001680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P1_0        , RULL(0x800001680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P1_1        , RULL(0x800005680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P1_1        , RULL(0x800005680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P1_2        , RULL(0x800009680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P1_2        , RULL(0x800009680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P1_3        , RULL(0x80000D680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P1_3        , RULL(0x80000D680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P1_4        , RULL(0x800011680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P1_4        , RULL(0x800011680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P2_0        , RULL(0x800001680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P2_0        , RULL(0x800001680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P2_1        , RULL(0x800005680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P2_1        , RULL(0x800005680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P2_2        , RULL(0x800009680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P2_2        , RULL(0x800009680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P2_3        , RULL(0x80000D680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P2_3        , RULL(0x80000D680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P2_4        , RULL(0x800011680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P2_4        , RULL(0x800011680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P3_0        , RULL(0x8000016807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P3_0        , RULL(0x8000016808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P3_1        , RULL(0x8000056807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P3_1        , RULL(0x8000056808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P3_2        , RULL(0x8000096807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P3_2        , RULL(0x8000096808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P3_3        , RULL(0x80000D6807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P3_3        , RULL(0x80000D6808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P3_4        , RULL(0x8000116807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR1_P3_4        , RULL(0x8000116808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P0_0          , RULL(0x800002680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P0_0        , RULL(0x800002680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P0_0        , RULL(0x800002680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P0_1          , RULL(0x800006680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P0_1        , RULL(0x800006680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P0_1        , RULL(0x800006680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P0_2          , RULL(0x80000A680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P0_2        , RULL(0x80000A680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P0_2        , RULL(0x80000A680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P0_3          , RULL(0x80000E680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P0_3        , RULL(0x80000E680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P0_3        , RULL(0x80000E680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P0_4          , RULL(0x800012680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P0_4        , RULL(0x800012680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P0_4        , RULL(0x800012680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P1_0        , RULL(0x800002680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P1_0        , RULL(0x800002680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P1_1        , RULL(0x800006680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P1_1        , RULL(0x800006680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P1_2        , RULL(0x80000A680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P1_2        , RULL(0x80000A680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P1_3        , RULL(0x80000E680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P1_3        , RULL(0x80000E680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P1_4        , RULL(0x800012680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P1_4        , RULL(0x800012680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P2_0        , RULL(0x800002680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P2_0        , RULL(0x800002680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P2_1        , RULL(0x800006680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P2_1        , RULL(0x800006680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P2_2        , RULL(0x80000A680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P2_2        , RULL(0x80000A680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P2_3        , RULL(0x80000E680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P2_3        , RULL(0x80000E680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P2_4        , RULL(0x800012680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P2_4        , RULL(0x800012680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P3_0        , RULL(0x8000026807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P3_0        , RULL(0x8000026808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P3_1        , RULL(0x8000066807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P3_1        , RULL(0x8000066808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P3_2        , RULL(0x80000A6807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P3_2        , RULL(0x80000A6808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P3_3        , RULL(0x80000E6807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P3_3        , RULL(0x80000E6808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P3_4        , RULL(0x8000126807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR2_P3_4        , RULL(0x8000126808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P0_0          , RULL(0x800003680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P0_0        , RULL(0x800003680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P0_0        , RULL(0x800003680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P0_1          , RULL(0x800007680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P0_1        , RULL(0x800007680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P0_1        , RULL(0x800007680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P0_2          , RULL(0x80000B680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P0_2        , RULL(0x80000B680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P0_2        , RULL(0x80000B680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P0_3          , RULL(0x80000F680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P0_3        , RULL(0x80000F680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P0_3        , RULL(0x80000F680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P0_4          , RULL(0x800013680701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P0_4        , RULL(0x800013680701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P0_4        , RULL(0x800013680801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P1_0        , RULL(0x800003680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P1_0        , RULL(0x800003680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P1_1        , RULL(0x800007680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P1_1        , RULL(0x800007680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P1_2        , RULL(0x80000B680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P1_2        , RULL(0x80000B680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P1_3        , RULL(0x80000F680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P1_3        , RULL(0x80000F680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P1_4        , RULL(0x800013680701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P1_4        , RULL(0x800013680801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P2_0        , RULL(0x800003680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P2_0        , RULL(0x800003680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P2_1        , RULL(0x800007680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P2_1        , RULL(0x800007680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P2_2        , RULL(0x80000B680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P2_2        , RULL(0x80000B680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P2_3        , RULL(0x80000F680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P2_3        , RULL(0x80000F680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P2_4        , RULL(0x800013680701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P2_4        , RULL(0x800013680801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P3_0        , RULL(0x8000036807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P3_0        , RULL(0x8000036808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P3_1        , RULL(0x8000076807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P3_1        , RULL(0x8000076808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P3_2        , RULL(0x80000B6807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P3_2        , RULL(0x80000B6808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P3_3        , RULL(0x80000F6807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P3_3        , RULL(0x80000F6808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P3_4        , RULL(0x8000136807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE8_RANK_PAIR3_P3_4        , RULL(0x8000136808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P0_0          , RULL(0x800000690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P0_0        , RULL(0x800000690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P0_0        , RULL(0x800000690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P0_1          , RULL(0x800004690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P0_1        , RULL(0x800004690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P0_1        , RULL(0x800004690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P0_2          , RULL(0x800008690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P0_2        , RULL(0x800008690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P0_2        , RULL(0x800008690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P0_3          , RULL(0x80000C690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P0_3        , RULL(0x80000C690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P0_3        , RULL(0x80000C690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P0_4          , RULL(0x800010690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P0_4        , RULL(0x800010690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P0_4        , RULL(0x800010690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P1_0        , RULL(0x800000690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P1_0        , RULL(0x800000690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P1_1        , RULL(0x800004690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P1_1        , RULL(0x800004690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P1_2        , RULL(0x800008690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P1_2        , RULL(0x800008690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P1_3        , RULL(0x80000C690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P1_3        , RULL(0x80000C690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P1_4        , RULL(0x800010690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P1_4        , RULL(0x800010690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P2_0        , RULL(0x800000690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P2_0        , RULL(0x800000690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P2_1        , RULL(0x800004690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P2_1        , RULL(0x800004690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P2_2        , RULL(0x800008690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P2_2        , RULL(0x800008690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P2_3        , RULL(0x80000C690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P2_3        , RULL(0x80000C690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P2_4        , RULL(0x800010690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P2_4        , RULL(0x800010690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P3_0        , RULL(0x8000006907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P3_0        , RULL(0x8000006908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P3_1        , RULL(0x8000046907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P3_1        , RULL(0x8000046908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P3_2        , RULL(0x8000086907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P3_2        , RULL(0x8000086908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P3_3        , RULL(0x80000C6907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P3_3        , RULL(0x80000C6908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P3_4        , RULL(0x8000106907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR0_P3_4        , RULL(0x8000106908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P0_0          , RULL(0x800001690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P0_0        , RULL(0x800001690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P0_0        , RULL(0x800001690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P0_1          , RULL(0x800005690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P0_1        , RULL(0x800005690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P0_1        , RULL(0x800005690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P0_2          , RULL(0x800009690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P0_2        , RULL(0x800009690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P0_2        , RULL(0x800009690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P0_3          , RULL(0x80000D690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P0_3        , RULL(0x80000D690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P0_3        , RULL(0x80000D690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P0_4          , RULL(0x800011690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P0_4        , RULL(0x800011690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P0_4        , RULL(0x800011690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P1_0        , RULL(0x800001690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P1_0        , RULL(0x800001690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P1_1        , RULL(0x800005690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P1_1        , RULL(0x800005690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P1_2        , RULL(0x800009690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P1_2        , RULL(0x800009690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P1_3        , RULL(0x80000D690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P1_3        , RULL(0x80000D690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P1_4        , RULL(0x800011690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P1_4        , RULL(0x800011690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P2_0        , RULL(0x800001690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P2_0        , RULL(0x800001690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P2_1        , RULL(0x800005690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P2_1        , RULL(0x800005690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P2_2        , RULL(0x800009690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P2_2        , RULL(0x800009690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P2_3        , RULL(0x80000D690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P2_3        , RULL(0x80000D690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P2_4        , RULL(0x800011690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P2_4        , RULL(0x800011690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P3_0        , RULL(0x8000016907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P3_0        , RULL(0x8000016908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P3_1        , RULL(0x8000056907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P3_1        , RULL(0x8000056908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P3_2        , RULL(0x8000096907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P3_2        , RULL(0x8000096908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P3_3        , RULL(0x80000D6907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P3_3        , RULL(0x80000D6908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P3_4        , RULL(0x8000116907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR1_P3_4        , RULL(0x8000116908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P0_0          , RULL(0x800002690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P0_0        , RULL(0x800002690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P0_0        , RULL(0x800002690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P0_1          , RULL(0x800006690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P0_1        , RULL(0x800006690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P0_1        , RULL(0x800006690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P0_2          , RULL(0x80000A690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P0_2        , RULL(0x80000A690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P0_2        , RULL(0x80000A690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P0_3          , RULL(0x80000E690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P0_3        , RULL(0x80000E690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P0_3        , RULL(0x80000E690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P0_4          , RULL(0x800012690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P0_4        , RULL(0x800012690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P0_4        , RULL(0x800012690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P1_0        , RULL(0x800002690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P1_0        , RULL(0x800002690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P1_1        , RULL(0x800006690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P1_1        , RULL(0x800006690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P1_2        , RULL(0x80000A690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P1_2        , RULL(0x80000A690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P1_3        , RULL(0x80000E690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P1_3        , RULL(0x80000E690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P1_4        , RULL(0x800012690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P1_4        , RULL(0x800012690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P2_0        , RULL(0x800002690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P2_0        , RULL(0x800002690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P2_1        , RULL(0x800006690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P2_1        , RULL(0x800006690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P2_2        , RULL(0x80000A690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P2_2        , RULL(0x80000A690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P2_3        , RULL(0x80000E690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P2_3        , RULL(0x80000E690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P2_4        , RULL(0x800012690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P2_4        , RULL(0x800012690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P3_0        , RULL(0x8000026907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P3_0        , RULL(0x8000026908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P3_1        , RULL(0x8000066907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P3_1        , RULL(0x8000066908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P3_2        , RULL(0x80000A6907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P3_2        , RULL(0x80000A6908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P3_3        , RULL(0x80000E6907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P3_3        , RULL(0x80000E6908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P3_4        , RULL(0x8000126907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR2_P3_4        , RULL(0x8000126908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P0_0          , RULL(0x800003690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P0_0        , RULL(0x800003690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P0_0        , RULL(0x800003690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P0_1          , RULL(0x800007690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P0_1        , RULL(0x800007690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P0_1        , RULL(0x800007690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P0_2          , RULL(0x80000B690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P0_2        , RULL(0x80000B690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P0_2        , RULL(0x80000B690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P0_3          , RULL(0x80000F690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P0_3        , RULL(0x80000F690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P0_3        , RULL(0x80000F690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P0_4          , RULL(0x800013690701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P0_4        , RULL(0x800013690701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P0_4        , RULL(0x800013690801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P1_0        , RULL(0x800003690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P1_0        , RULL(0x800003690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P1_1        , RULL(0x800007690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P1_1        , RULL(0x800007690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P1_2        , RULL(0x80000B690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P1_2        , RULL(0x80000B690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P1_3        , RULL(0x80000F690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P1_3        , RULL(0x80000F690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P1_4        , RULL(0x800013690701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P1_4        , RULL(0x800013690801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P2_0        , RULL(0x800003690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P2_0        , RULL(0x800003690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P2_1        , RULL(0x800007690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P2_1        , RULL(0x800007690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P2_2        , RULL(0x80000B690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P2_2        , RULL(0x80000B690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P2_3        , RULL(0x80000F690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P2_3        , RULL(0x80000F690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P2_4        , RULL(0x800013690701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P2_4        , RULL(0x800013690801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P3_0        , RULL(0x8000036907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P3_0        , RULL(0x8000036908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P3_1        , RULL(0x8000076907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P3_1        , RULL(0x8000076908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P3_2        , RULL(0x80000B6907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P3_2        , RULL(0x80000B6908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P3_3        , RULL(0x80000F6907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P3_3        , RULL(0x80000F6908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P3_4        , RULL(0x8000136907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_EYE_SIZE9_RANK_PAIR3_P3_4        , RULL(0x8000136908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_TIMING_REFERENCE0_P0_0             , RULL(0x800000700701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_TIMING_REFERENCE0_P0_0           , RULL(0x800000700701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_TIMING_REFERENCE0_P0_0           , RULL(0x800000700801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_TIMING_REFERENCE0_P0_1             , RULL(0x800004700701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_TIMING_REFERENCE0_P0_1           , RULL(0x800004700701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_TIMING_REFERENCE0_P0_1           , RULL(0x800004700801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_TIMING_REFERENCE0_P0_2             , RULL(0x800008700701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_TIMING_REFERENCE0_P0_2           , RULL(0x800008700701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_TIMING_REFERENCE0_P0_2           , RULL(0x800008700801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_TIMING_REFERENCE0_P0_3             , RULL(0x80000C700701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_TIMING_REFERENCE0_P0_3           , RULL(0x80000C700701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_TIMING_REFERENCE0_P0_3           , RULL(0x80000C700801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_TIMING_REFERENCE0_P0_4             , RULL(0x800010700701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_TIMING_REFERENCE0_P0_4           , RULL(0x800010700701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_TIMING_REFERENCE0_P0_4           , RULL(0x800010700801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_TIMING_REFERENCE0_P1_0           , RULL(0x800000700701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_TIMING_REFERENCE0_P1_0           , RULL(0x800000700801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_TIMING_REFERENCE0_P1_1           , RULL(0x800004700701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_TIMING_REFERENCE0_P1_1           , RULL(0x800004700801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_TIMING_REFERENCE0_P1_2           , RULL(0x800008700701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_TIMING_REFERENCE0_P1_2           , RULL(0x800008700801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_TIMING_REFERENCE0_P1_3           , RULL(0x80000C700701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_TIMING_REFERENCE0_P1_3           , RULL(0x80000C700801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_TIMING_REFERENCE0_P1_4           , RULL(0x800010700701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_TIMING_REFERENCE0_P1_4           , RULL(0x800010700801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_TIMING_REFERENCE0_P2_0           , RULL(0x800000700701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_TIMING_REFERENCE0_P2_0           , RULL(0x800000700801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_TIMING_REFERENCE0_P2_1           , RULL(0x800004700701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_TIMING_REFERENCE0_P2_1           , RULL(0x800004700801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_TIMING_REFERENCE0_P2_2           , RULL(0x800008700701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_TIMING_REFERENCE0_P2_2           , RULL(0x800008700801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_TIMING_REFERENCE0_P2_3           , RULL(0x80000C700701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_TIMING_REFERENCE0_P2_3           , RULL(0x80000C700801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_TIMING_REFERENCE0_P2_4           , RULL(0x800010700701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_TIMING_REFERENCE0_P2_4           , RULL(0x800010700801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_TIMING_REFERENCE0_P3_0           , RULL(0x8000007007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_TIMING_REFERENCE0_P3_0           , RULL(0x8000007008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_TIMING_REFERENCE0_P3_1           , RULL(0x8000047007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_TIMING_REFERENCE0_P3_1           , RULL(0x8000047008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_TIMING_REFERENCE0_P3_2           , RULL(0x8000087007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_TIMING_REFERENCE0_P3_2           , RULL(0x8000087008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_TIMING_REFERENCE0_P3_3           , RULL(0x80000C7007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_TIMING_REFERENCE0_P3_3           , RULL(0x80000C7008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_TIMING_REFERENCE0_P3_4           , RULL(0x8000107007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_TIMING_REFERENCE0_P3_4           , RULL(0x8000107008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_TIMING_REFERENCE1_P0_0             , RULL(0x800000710701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_TIMING_REFERENCE1_P0_0           , RULL(0x800000710701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_TIMING_REFERENCE1_P0_0           , RULL(0x800000710801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_TIMING_REFERENCE1_P0_1             , RULL(0x800004710701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_TIMING_REFERENCE1_P0_1           , RULL(0x800004710701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_TIMING_REFERENCE1_P0_1           , RULL(0x800004710801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_TIMING_REFERENCE1_P0_2             , RULL(0x800008710701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_TIMING_REFERENCE1_P0_2           , RULL(0x800008710701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_TIMING_REFERENCE1_P0_2           , RULL(0x800008710801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_TIMING_REFERENCE1_P0_3             , RULL(0x80000C710701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_TIMING_REFERENCE1_P0_3           , RULL(0x80000C710701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_TIMING_REFERENCE1_P0_3           , RULL(0x80000C710801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_READ_TIMING_REFERENCE1_P0_4             , RULL(0x800010710701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_READ_TIMING_REFERENCE1_P0_4           , RULL(0x800010710701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_READ_TIMING_REFERENCE1_P0_4           , RULL(0x800010710801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_TIMING_REFERENCE1_P1_0           , RULL(0x800000710701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_TIMING_REFERENCE1_P1_0           , RULL(0x800000710801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_TIMING_REFERENCE1_P1_1           , RULL(0x800004710701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_TIMING_REFERENCE1_P1_1           , RULL(0x800004710801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_TIMING_REFERENCE1_P1_2           , RULL(0x800008710701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_TIMING_REFERENCE1_P1_2           , RULL(0x800008710801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_TIMING_REFERENCE1_P1_3           , RULL(0x80000C710701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_TIMING_REFERENCE1_P1_3           , RULL(0x80000C710801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_READ_TIMING_REFERENCE1_P1_4           , RULL(0x800010710701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_READ_TIMING_REFERENCE1_P1_4           , RULL(0x800010710801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_TIMING_REFERENCE1_P2_0           , RULL(0x800000710701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_TIMING_REFERENCE1_P2_0           , RULL(0x800000710801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_TIMING_REFERENCE1_P2_1           , RULL(0x800004710701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_TIMING_REFERENCE1_P2_1           , RULL(0x800004710801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_TIMING_REFERENCE1_P2_2           , RULL(0x800008710701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_TIMING_REFERENCE1_P2_2           , RULL(0x800008710801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_TIMING_REFERENCE1_P2_3           , RULL(0x80000C710701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_TIMING_REFERENCE1_P2_3           , RULL(0x80000C710801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_READ_TIMING_REFERENCE1_P2_4           , RULL(0x800010710701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_READ_TIMING_REFERENCE1_P2_4           , RULL(0x800010710801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_TIMING_REFERENCE1_P3_0           , RULL(0x8000007107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_TIMING_REFERENCE1_P3_0           , RULL(0x8000007108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_TIMING_REFERENCE1_P3_1           , RULL(0x8000047107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_TIMING_REFERENCE1_P3_1           , RULL(0x8000047108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_TIMING_REFERENCE1_P3_2           , RULL(0x8000087107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_TIMING_REFERENCE1_P3_2           , RULL(0x8000087108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_TIMING_REFERENCE1_P3_3           , RULL(0x80000C7107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_TIMING_REFERENCE1_P3_3           , RULL(0x80000C7108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_READ_TIMING_REFERENCE1_P3_4           , RULL(0x8000107107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_READ_TIMING_REFERENCE1_P3_4           , RULL(0x8000107108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RX_PEAK_AMP_P0_0                        , RULL(0x800000060701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RX_PEAK_AMP_P0_0                      , RULL(0x800000060701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RX_PEAK_AMP_P0_0                      , RULL(0x800000060801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RX_PEAK_AMP_P0_1                        , RULL(0x800004060701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RX_PEAK_AMP_P0_1                      , RULL(0x800004060701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RX_PEAK_AMP_P0_1                      , RULL(0x800004060801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RX_PEAK_AMP_P0_2                        , RULL(0x800008060701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RX_PEAK_AMP_P0_2                      , RULL(0x800008060701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RX_PEAK_AMP_P0_2                      , RULL(0x800008060801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RX_PEAK_AMP_P0_3                        , RULL(0x80000C060701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RX_PEAK_AMP_P0_3                      , RULL(0x80000C060701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RX_PEAK_AMP_P0_3                      , RULL(0x80000C060801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_RX_PEAK_AMP_P0_4                        , RULL(0x800010060701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_RX_PEAK_AMP_P0_4                      , RULL(0x800010060701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_RX_PEAK_AMP_P0_4                      , RULL(0x800010060801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RX_PEAK_AMP_P1_0                      , RULL(0x800000060701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RX_PEAK_AMP_P1_0                      , RULL(0x800000060801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RX_PEAK_AMP_P1_1                      , RULL(0x800004060701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RX_PEAK_AMP_P1_1                      , RULL(0x800004060801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RX_PEAK_AMP_P1_2                      , RULL(0x800008060701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RX_PEAK_AMP_P1_2                      , RULL(0x800008060801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RX_PEAK_AMP_P1_3                      , RULL(0x80000C060701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RX_PEAK_AMP_P1_3                      , RULL(0x80000C060801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_RX_PEAK_AMP_P1_4                      , RULL(0x800010060701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_RX_PEAK_AMP_P1_4                      , RULL(0x800010060801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RX_PEAK_AMP_P2_0                      , RULL(0x800000060701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RX_PEAK_AMP_P2_0                      , RULL(0x800000060801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RX_PEAK_AMP_P2_1                      , RULL(0x800004060701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RX_PEAK_AMP_P2_1                      , RULL(0x800004060801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RX_PEAK_AMP_P2_2                      , RULL(0x800008060701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RX_PEAK_AMP_P2_2                      , RULL(0x800008060801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RX_PEAK_AMP_P2_3                      , RULL(0x80000C060701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RX_PEAK_AMP_P2_3                      , RULL(0x80000C060801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_RX_PEAK_AMP_P2_4                      , RULL(0x800010060701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_RX_PEAK_AMP_P2_4                      , RULL(0x800010060801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RX_PEAK_AMP_P3_0                      , RULL(0x8000000607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RX_PEAK_AMP_P3_0                      , RULL(0x8000000608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RX_PEAK_AMP_P3_1                      , RULL(0x8000040607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RX_PEAK_AMP_P3_1                      , RULL(0x8000040608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RX_PEAK_AMP_P3_2                      , RULL(0x8000080607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RX_PEAK_AMP_P3_2                      , RULL(0x8000080608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RX_PEAK_AMP_P3_3                      , RULL(0x80000C0607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RX_PEAK_AMP_P3_3                      , RULL(0x80000C0608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_RX_PEAK_AMP_P3_4                      , RULL(0x8000100607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_RX_PEAK_AMP_P3_4                      , RULL(0x8000100608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_SYSCLK_PR0_P0_0                         , RULL(0x800000070701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_SYSCLK_PR0_P0_0                       , RULL(0x800000070701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_SYSCLK_PR0_P0_0                       , RULL(0x800000070801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_SYSCLK_PR0_P0_1                         , RULL(0x800004070701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_SYSCLK_PR0_P0_1                       , RULL(0x800004070701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_SYSCLK_PR0_P0_1                       , RULL(0x800004070801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_SYSCLK_PR0_P0_2                         , RULL(0x800008070701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_SYSCLK_PR0_P0_2                       , RULL(0x800008070701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_SYSCLK_PR0_P0_2                       , RULL(0x800008070801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_SYSCLK_PR0_P0_3                         , RULL(0x80000C070701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_SYSCLK_PR0_P0_3                       , RULL(0x80000C070701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_SYSCLK_PR0_P0_3                       , RULL(0x80000C070801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_SYSCLK_PR0_P0_4                         , RULL(0x800010070701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_SYSCLK_PR0_P0_4                       , RULL(0x800010070701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_SYSCLK_PR0_P0_4                       , RULL(0x800010070801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_SYSCLK_PR0_P1_0                       , RULL(0x800000070701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_SYSCLK_PR0_P1_0                       , RULL(0x800000070801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_SYSCLK_PR0_P1_1                       , RULL(0x800004070701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_SYSCLK_PR0_P1_1                       , RULL(0x800004070801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_SYSCLK_PR0_P1_2                       , RULL(0x800008070701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_SYSCLK_PR0_P1_2                       , RULL(0x800008070801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_SYSCLK_PR0_P1_3                       , RULL(0x80000C070701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_SYSCLK_PR0_P1_3                       , RULL(0x80000C070801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_SYSCLK_PR0_P1_4                       , RULL(0x800010070701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_SYSCLK_PR0_P1_4                       , RULL(0x800010070801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_SYSCLK_PR0_P2_0                       , RULL(0x800000070701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_SYSCLK_PR0_P2_0                       , RULL(0x800000070801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_SYSCLK_PR0_P2_1                       , RULL(0x800004070701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_SYSCLK_PR0_P2_1                       , RULL(0x800004070801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_SYSCLK_PR0_P2_2                       , RULL(0x800008070701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_SYSCLK_PR0_P2_2                       , RULL(0x800008070801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_SYSCLK_PR0_P2_3                       , RULL(0x80000C070701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_SYSCLK_PR0_P2_3                       , RULL(0x80000C070801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_SYSCLK_PR0_P2_4                       , RULL(0x800010070701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_SYSCLK_PR0_P2_4                       , RULL(0x800010070801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_SYSCLK_PR0_P3_0                       , RULL(0x8000000707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_SYSCLK_PR0_P3_0                       , RULL(0x8000000708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_SYSCLK_PR0_P3_1                       , RULL(0x8000040707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_SYSCLK_PR0_P3_1                       , RULL(0x8000040708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_SYSCLK_PR0_P3_2                       , RULL(0x8000080707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_SYSCLK_PR0_P3_2                       , RULL(0x8000080708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_SYSCLK_PR0_P3_3                       , RULL(0x80000C0707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_SYSCLK_PR0_P3_3                       , RULL(0x80000C0708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_SYSCLK_PR0_P3_4                       , RULL(0x8000100707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_SYSCLK_PR0_P3_4                       , RULL(0x8000100708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_SYSCLK_PR1_P0_0                         , RULL(0x8000007F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_SYSCLK_PR1_P0_0                       , RULL(0x8000007F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_SYSCLK_PR1_P0_0                       , RULL(0x8000007F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_SYSCLK_PR1_P0_1                         , RULL(0x8000047F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_SYSCLK_PR1_P0_1                       , RULL(0x8000047F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_SYSCLK_PR1_P0_1                       , RULL(0x8000047F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_SYSCLK_PR1_P0_2                         , RULL(0x8000087F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_SYSCLK_PR1_P0_2                       , RULL(0x8000087F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_SYSCLK_PR1_P0_2                       , RULL(0x8000087F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_SYSCLK_PR1_P0_3                         , RULL(0x80000C7F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_SYSCLK_PR1_P0_3                       , RULL(0x80000C7F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_SYSCLK_PR1_P0_3                       , RULL(0x80000C7F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_SYSCLK_PR1_P0_4                         , RULL(0x8000107F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_SYSCLK_PR1_P0_4                       , RULL(0x8000107F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_SYSCLK_PR1_P0_4                       , RULL(0x8000107F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_SYSCLK_PR1_P1_0                       , RULL(0x8000007F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_SYSCLK_PR1_P1_0                       , RULL(0x8000007F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_SYSCLK_PR1_P1_1                       , RULL(0x8000047F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_SYSCLK_PR1_P1_1                       , RULL(0x8000047F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_SYSCLK_PR1_P1_2                       , RULL(0x8000087F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_SYSCLK_PR1_P1_2                       , RULL(0x8000087F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_SYSCLK_PR1_P1_3                       , RULL(0x80000C7F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_SYSCLK_PR1_P1_3                       , RULL(0x80000C7F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_SYSCLK_PR1_P1_4                       , RULL(0x8000107F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_SYSCLK_PR1_P1_4                       , RULL(0x8000107F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_SYSCLK_PR1_P2_0                       , RULL(0x8000007F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_SYSCLK_PR1_P2_0                       , RULL(0x8000007F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_SYSCLK_PR1_P2_1                       , RULL(0x8000047F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_SYSCLK_PR1_P2_1                       , RULL(0x8000047F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_SYSCLK_PR1_P2_2                       , RULL(0x8000087F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_SYSCLK_PR1_P2_2                       , RULL(0x8000087F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_SYSCLK_PR1_P2_3                       , RULL(0x80000C7F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_SYSCLK_PR1_P2_3                       , RULL(0x80000C7F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_SYSCLK_PR1_P2_4                       , RULL(0x8000107F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_SYSCLK_PR1_P2_4                       , RULL(0x8000107F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_SYSCLK_PR1_P3_0                       , RULL(0x8000007F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_SYSCLK_PR1_P3_0                       , RULL(0x8000007F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_SYSCLK_PR1_P3_1                       , RULL(0x8000047F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_SYSCLK_PR1_P3_1                       , RULL(0x8000047F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_SYSCLK_PR1_P3_2                       , RULL(0x8000087F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_SYSCLK_PR1_P3_2                       , RULL(0x8000087F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_SYSCLK_PR1_P3_3                       , RULL(0x80000C7F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_SYSCLK_PR1_P3_3                       , RULL(0x80000C7F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_SYSCLK_PR1_P3_4                       , RULL(0x8000107F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_SYSCLK_PR1_P3_4                       , RULL(0x8000107F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_SYSCLK_PR_VALUE_P0_0                    , RULL(0x800000730701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_SYSCLK_PR_VALUE_P0_0                  , RULL(0x800000730701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_SYSCLK_PR_VALUE_P0_0                  , RULL(0x800000730801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_SYSCLK_PR_VALUE_P0_1                    , RULL(0x800004730701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_SYSCLK_PR_VALUE_P0_1                  , RULL(0x800004730701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_SYSCLK_PR_VALUE_P0_1                  , RULL(0x800004730801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_SYSCLK_PR_VALUE_P0_2                    , RULL(0x800008730701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_SYSCLK_PR_VALUE_P0_2                  , RULL(0x800008730701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_SYSCLK_PR_VALUE_P0_2                  , RULL(0x800008730801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_SYSCLK_PR_VALUE_P0_3                    , RULL(0x80000C730701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_SYSCLK_PR_VALUE_P0_3                  , RULL(0x80000C730701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_SYSCLK_PR_VALUE_P0_3                  , RULL(0x80000C730801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_SYSCLK_PR_VALUE_P0_4                    , RULL(0x800010730701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_SYSCLK_PR_VALUE_P0_4                  , RULL(0x800010730701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_SYSCLK_PR_VALUE_P0_4                  , RULL(0x800010730801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_SYSCLK_PR_VALUE_P1_0                  , RULL(0x800000730701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_SYSCLK_PR_VALUE_P1_0                  , RULL(0x800000730801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_SYSCLK_PR_VALUE_P1_1                  , RULL(0x800004730701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_SYSCLK_PR_VALUE_P1_1                  , RULL(0x800004730801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_SYSCLK_PR_VALUE_P1_2                  , RULL(0x800008730701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_SYSCLK_PR_VALUE_P1_2                  , RULL(0x800008730801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_SYSCLK_PR_VALUE_P1_3                  , RULL(0x80000C730701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_SYSCLK_PR_VALUE_P1_3                  , RULL(0x80000C730801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_SYSCLK_PR_VALUE_P1_4                  , RULL(0x800010730701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_SYSCLK_PR_VALUE_P1_4                  , RULL(0x800010730801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_SYSCLK_PR_VALUE_P2_0                  , RULL(0x800000730701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_SYSCLK_PR_VALUE_P2_0                  , RULL(0x800000730801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_SYSCLK_PR_VALUE_P2_1                  , RULL(0x800004730701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_SYSCLK_PR_VALUE_P2_1                  , RULL(0x800004730801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_SYSCLK_PR_VALUE_P2_2                  , RULL(0x800008730701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_SYSCLK_PR_VALUE_P2_2                  , RULL(0x800008730801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_SYSCLK_PR_VALUE_P2_3                  , RULL(0x80000C730701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_SYSCLK_PR_VALUE_P2_3                  , RULL(0x80000C730801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_SYSCLK_PR_VALUE_P2_4                  , RULL(0x800010730701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_SYSCLK_PR_VALUE_P2_4                  , RULL(0x800010730801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_SYSCLK_PR_VALUE_P3_0                  , RULL(0x8000007307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_SYSCLK_PR_VALUE_P3_0                  , RULL(0x8000007308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_SYSCLK_PR_VALUE_P3_1                  , RULL(0x8000047307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_SYSCLK_PR_VALUE_P3_1                  , RULL(0x8000047308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_SYSCLK_PR_VALUE_P3_2                  , RULL(0x8000087307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_SYSCLK_PR_VALUE_P3_2                  , RULL(0x8000087308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_SYSCLK_PR_VALUE_P3_3                  , RULL(0x80000C7307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_SYSCLK_PR_VALUE_P3_3                  , RULL(0x80000C7308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_SYSCLK_PR_VALUE_P3_4                  , RULL(0x8000107307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_SYSCLK_PR_VALUE_P3_4                  , RULL(0x8000107308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP0_P0_0                       , RULL(0x800000050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP0_P0_0                     , RULL(0x800000050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP0_P0_0                     , RULL(0x800000050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP0_P0_1                       , RULL(0x800004050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP0_P0_1                     , RULL(0x800004050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP0_P0_1                     , RULL(0x800004050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP0_P0_2                       , RULL(0x800008050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP0_P0_2                     , RULL(0x800008050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP0_P0_2                     , RULL(0x800008050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP0_P0_3                       , RULL(0x80000C050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP0_P0_3                     , RULL(0x80000C050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP0_P0_3                     , RULL(0x80000C050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP0_P0_4                       , RULL(0x800010050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP0_P0_4                     , RULL(0x800010050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP0_P0_4                     , RULL(0x800010050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP0_P1_0                     , RULL(0x800000050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP0_P1_0                     , RULL(0x800000050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP0_P1_1                     , RULL(0x800004050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP0_P1_1                     , RULL(0x800004050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP0_P1_2                     , RULL(0x800008050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP0_P1_2                     , RULL(0x800008050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP0_P1_3                     , RULL(0x80000C050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP0_P1_3                     , RULL(0x80000C050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP0_P1_4                     , RULL(0x800010050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP0_P1_4                     , RULL(0x800010050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP0_P2_0                     , RULL(0x800000050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP0_P2_0                     , RULL(0x800000050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP0_P2_1                     , RULL(0x800004050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP0_P2_1                     , RULL(0x800004050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP0_P2_2                     , RULL(0x800008050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP0_P2_2                     , RULL(0x800008050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP0_P2_3                     , RULL(0x80000C050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP0_P2_3                     , RULL(0x80000C050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP0_P2_4                     , RULL(0x800010050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP0_P2_4                     , RULL(0x800010050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP0_P3_0                     , RULL(0x8000000507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP0_P3_0                     , RULL(0x8000000508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP0_P3_1                     , RULL(0x8000040507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP0_P3_1                     , RULL(0x8000040508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP0_P3_2                     , RULL(0x8000080507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP0_P3_2                     , RULL(0x8000080508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP0_P3_3                     , RULL(0x80000C0507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP0_P3_3                     , RULL(0x80000C0508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP0_P3_4                     , RULL(0x8000100507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP0_P3_4                     , RULL(0x8000100508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP1_P0_0                       , RULL(0x800001050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP1_P0_0                     , RULL(0x800001050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP1_P0_0                     , RULL(0x800001050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP1_P0_1                       , RULL(0x800005050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP1_P0_1                     , RULL(0x800005050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP1_P0_1                     , RULL(0x800005050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP1_P0_2                       , RULL(0x800009050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP1_P0_2                     , RULL(0x800009050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP1_P0_2                     , RULL(0x800009050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP1_P0_3                       , RULL(0x80000D050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP1_P0_3                     , RULL(0x80000D050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP1_P0_3                     , RULL(0x80000D050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP1_P0_4                       , RULL(0x800011050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP1_P0_4                     , RULL(0x800011050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP1_P0_4                     , RULL(0x800011050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP1_P1_0                     , RULL(0x800001050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP1_P1_0                     , RULL(0x800001050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP1_P1_1                     , RULL(0x800005050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP1_P1_1                     , RULL(0x800005050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP1_P1_2                     , RULL(0x800009050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP1_P1_2                     , RULL(0x800009050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP1_P1_3                     , RULL(0x80000D050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP1_P1_3                     , RULL(0x80000D050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP1_P1_4                     , RULL(0x800011050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP1_P1_4                     , RULL(0x800011050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP1_P2_0                     , RULL(0x800001050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP1_P2_0                     , RULL(0x800001050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP1_P2_1                     , RULL(0x800005050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP1_P2_1                     , RULL(0x800005050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP1_P2_2                     , RULL(0x800009050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP1_P2_2                     , RULL(0x800009050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP1_P2_3                     , RULL(0x80000D050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP1_P2_3                     , RULL(0x80000D050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP1_P2_4                     , RULL(0x800011050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP1_P2_4                     , RULL(0x800011050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP1_P3_0                     , RULL(0x8000010507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP1_P3_0                     , RULL(0x8000010508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP1_P3_1                     , RULL(0x8000050507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP1_P3_1                     , RULL(0x8000050508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP1_P3_2                     , RULL(0x8000090507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP1_P3_2                     , RULL(0x8000090508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP1_P3_3                     , RULL(0x80000D0507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP1_P3_3                     , RULL(0x80000D0508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP1_P3_4                     , RULL(0x8000110507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP1_P3_4                     , RULL(0x8000110508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP2_P0_0                       , RULL(0x800002050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP2_P0_0                     , RULL(0x800002050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP2_P0_0                     , RULL(0x800002050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP2_P0_1                       , RULL(0x800006050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP2_P0_1                     , RULL(0x800006050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP2_P0_1                     , RULL(0x800006050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP2_P0_2                       , RULL(0x80000A050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP2_P0_2                     , RULL(0x80000A050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP2_P0_2                     , RULL(0x80000A050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP2_P0_3                       , RULL(0x80000E050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP2_P0_3                     , RULL(0x80000E050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP2_P0_3                     , RULL(0x80000E050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP2_P0_4                       , RULL(0x800012050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP2_P0_4                     , RULL(0x800012050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP2_P0_4                     , RULL(0x800012050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP2_P1_0                     , RULL(0x800002050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP2_P1_0                     , RULL(0x800002050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP2_P1_1                     , RULL(0x800006050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP2_P1_1                     , RULL(0x800006050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP2_P1_2                     , RULL(0x80000A050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP2_P1_2                     , RULL(0x80000A050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP2_P1_3                     , RULL(0x80000E050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP2_P1_3                     , RULL(0x80000E050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP2_P1_4                     , RULL(0x800012050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP2_P1_4                     , RULL(0x800012050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP2_P2_0                     , RULL(0x800002050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP2_P2_0                     , RULL(0x800002050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP2_P2_1                     , RULL(0x800006050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP2_P2_1                     , RULL(0x800006050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP2_P2_2                     , RULL(0x80000A050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP2_P2_2                     , RULL(0x80000A050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP2_P2_3                     , RULL(0x80000E050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP2_P2_3                     , RULL(0x80000E050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP2_P2_4                     , RULL(0x800012050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP2_P2_4                     , RULL(0x800012050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP2_P3_0                     , RULL(0x8000020507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP2_P3_0                     , RULL(0x8000020508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP2_P3_1                     , RULL(0x8000060507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP2_P3_1                     , RULL(0x8000060508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP2_P3_2                     , RULL(0x80000A0507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP2_P3_2                     , RULL(0x80000A0508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP2_P3_3                     , RULL(0x80000E0507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP2_P3_3                     , RULL(0x80000E0508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP2_P3_4                     , RULL(0x8000120507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP2_P3_4                     , RULL(0x8000120508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP3_P0_0                       , RULL(0x800003050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP3_P0_0                     , RULL(0x800003050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP3_P0_0                     , RULL(0x800003050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP3_P0_1                       , RULL(0x800007050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP3_P0_1                     , RULL(0x800007050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP3_P0_1                     , RULL(0x800007050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP3_P0_2                       , RULL(0x80000B050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP3_P0_2                     , RULL(0x80000B050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP3_P0_2                     , RULL(0x80000B050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP3_P0_3                       , RULL(0x80000F050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP3_P0_3                     , RULL(0x80000F050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP3_P0_3                     , RULL(0x80000F050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_EN_RP3_P0_4                       , RULL(0x800013050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_EN_RP3_P0_4                     , RULL(0x800013050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_EN_RP3_P0_4                     , RULL(0x800013050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP3_P1_0                     , RULL(0x800003050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP3_P1_0                     , RULL(0x800003050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP3_P1_1                     , RULL(0x800007050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP3_P1_1                     , RULL(0x800007050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP3_P1_2                     , RULL(0x80000B050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP3_P1_2                     , RULL(0x80000B050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP3_P1_3                     , RULL(0x80000F050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP3_P1_3                     , RULL(0x80000F050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_EN_RP3_P1_4                     , RULL(0x800013050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_EN_RP3_P1_4                     , RULL(0x800013050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP3_P2_0                     , RULL(0x800003050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP3_P2_0                     , RULL(0x800003050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP3_P2_1                     , RULL(0x800007050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP3_P2_1                     , RULL(0x800007050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP3_P2_2                     , RULL(0x80000B050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP3_P2_2                     , RULL(0x80000B050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP3_P2_3                     , RULL(0x80000F050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP3_P2_3                     , RULL(0x80000F050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_EN_RP3_P2_4                     , RULL(0x800013050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_EN_RP3_P2_4                     , RULL(0x800013050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP3_P3_0                     , RULL(0x8000030507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP3_P3_0                     , RULL(0x8000030508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP3_P3_1                     , RULL(0x8000070507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP3_P3_1                     , RULL(0x8000070508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP3_P3_2                     , RULL(0x80000B0507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP3_P3_2                     , RULL(0x80000B0508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP3_P3_3                     , RULL(0x80000F0507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP3_P3_3                     , RULL(0x80000F0508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_EN_RP3_P3_4                     , RULL(0x8000130507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_EN_RP3_P3_4                     , RULL(0x8000130508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_PR_P0_0                           , RULL(0x800000740701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_PR_P0_0                         , RULL(0x800000740701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_PR_P0_0                         , RULL(0x800000740801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_PR_P0_1                           , RULL(0x800004740701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_PR_P0_1                         , RULL(0x800004740701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_PR_P0_1                         , RULL(0x800004740801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_PR_P0_2                           , RULL(0x800008740701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_PR_P0_2                         , RULL(0x800008740701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_PR_P0_2                         , RULL(0x800008740801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_PR_P0_3                           , RULL(0x80000C740701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_PR_P0_3                         , RULL(0x80000C740701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_PR_P0_3                         , RULL(0x80000C740801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WRCLK_PR_P0_4                           , RULL(0x800010740701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WRCLK_PR_P0_4                         , RULL(0x800010740701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WRCLK_PR_P0_4                         , RULL(0x800010740801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_PR_P1_0                         , RULL(0x800000740701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_PR_P1_0                         , RULL(0x800000740801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_PR_P1_1                         , RULL(0x800004740701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_PR_P1_1                         , RULL(0x800004740801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_PR_P1_2                         , RULL(0x800008740701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_PR_P1_2                         , RULL(0x800008740801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_PR_P1_3                         , RULL(0x80000C740701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_PR_P1_3                         , RULL(0x80000C740801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WRCLK_PR_P1_4                         , RULL(0x800010740701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WRCLK_PR_P1_4                         , RULL(0x800010740801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_PR_P2_0                         , RULL(0x800000740701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_PR_P2_0                         , RULL(0x800000740801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_PR_P2_1                         , RULL(0x800004740701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_PR_P2_1                         , RULL(0x800004740801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_PR_P2_2                         , RULL(0x800008740701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_PR_P2_2                         , RULL(0x800008740801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_PR_P2_3                         , RULL(0x80000C740701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_PR_P2_3                         , RULL(0x80000C740801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WRCLK_PR_P2_4                         , RULL(0x800010740701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WRCLK_PR_P2_4                         , RULL(0x800010740801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_PR_P3_0                         , RULL(0x8000007407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_PR_P3_0                         , RULL(0x8000007408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_PR_P3_1                         , RULL(0x8000047407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_PR_P3_1                         , RULL(0x8000047408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_PR_P3_2                         , RULL(0x8000087407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_PR_P3_2                         , RULL(0x8000087408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_PR_P3_3                         , RULL(0x80000C7407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_PR_P3_3                         , RULL(0x80000C7408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WRCLK_PR_P3_4                         , RULL(0x8000107407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WRCLK_PR_P3_4                         , RULL(0x8000107408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_CNTR_STATUS0_P0_0                    , RULL(0x800000180701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_CNTR_STATUS0_P0_0                  , RULL(0x800000180701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_CNTR_STATUS0_P0_0                  , RULL(0x800000180801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_CNTR_STATUS0_P0_1                    , RULL(0x800004180701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_CNTR_STATUS0_P0_1                  , RULL(0x800004180701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_CNTR_STATUS0_P0_1                  , RULL(0x800004180801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_CNTR_STATUS0_P0_2                    , RULL(0x800008180701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_CNTR_STATUS0_P0_2                  , RULL(0x800008180701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_CNTR_STATUS0_P0_2                  , RULL(0x800008180801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_CNTR_STATUS0_P0_3                    , RULL(0x80000C180701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_CNTR_STATUS0_P0_3                  , RULL(0x80000C180701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_CNTR_STATUS0_P0_3                  , RULL(0x80000C180801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_CNTR_STATUS0_P0_4                    , RULL(0x800010180701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_CNTR_STATUS0_P0_4                  , RULL(0x800010180701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_CNTR_STATUS0_P0_4                  , RULL(0x800010180801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_CNTR_STATUS0_P1_0                  , RULL(0x800000180701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_CNTR_STATUS0_P1_0                  , RULL(0x800000180801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_CNTR_STATUS0_P1_1                  , RULL(0x800004180701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_CNTR_STATUS0_P1_1                  , RULL(0x800004180801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_CNTR_STATUS0_P1_2                  , RULL(0x800008180701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_CNTR_STATUS0_P1_2                  , RULL(0x800008180801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_CNTR_STATUS0_P1_3                  , RULL(0x80000C180701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_CNTR_STATUS0_P1_3                  , RULL(0x80000C180801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_CNTR_STATUS0_P1_4                  , RULL(0x800010180701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_CNTR_STATUS0_P1_4                  , RULL(0x800010180801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_CNTR_STATUS0_P2_0                  , RULL(0x800000180701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_CNTR_STATUS0_P2_0                  , RULL(0x800000180801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_CNTR_STATUS0_P2_1                  , RULL(0x800004180701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_CNTR_STATUS0_P2_1                  , RULL(0x800004180801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_CNTR_STATUS0_P2_2                  , RULL(0x800008180701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_CNTR_STATUS0_P2_2                  , RULL(0x800008180801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_CNTR_STATUS0_P2_3                  , RULL(0x80000C180701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_CNTR_STATUS0_P2_3                  , RULL(0x80000C180801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_CNTR_STATUS0_P2_4                  , RULL(0x800010180701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_CNTR_STATUS0_P2_4                  , RULL(0x800010180801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_CNTR_STATUS0_P3_0                  , RULL(0x8000001807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_CNTR_STATUS0_P3_0                  , RULL(0x8000001808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_CNTR_STATUS0_P3_1                  , RULL(0x8000041807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_CNTR_STATUS0_P3_1                  , RULL(0x8000041808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_CNTR_STATUS0_P3_2                  , RULL(0x8000081807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_CNTR_STATUS0_P3_2                  , RULL(0x8000081808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_CNTR_STATUS0_P3_3                  , RULL(0x80000C1807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_CNTR_STATUS0_P3_3                  , RULL(0x80000C1808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_CNTR_STATUS0_P3_4                  , RULL(0x8000101807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_CNTR_STATUS0_P3_4                  , RULL(0x8000101808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_CNTR_STATUS1_P0_0                    , RULL(0x800000190701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_CNTR_STATUS1_P0_0                  , RULL(0x800000190701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_CNTR_STATUS1_P0_0                  , RULL(0x800000190801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_CNTR_STATUS1_P0_1                    , RULL(0x800004190701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_CNTR_STATUS1_P0_1                  , RULL(0x800004190701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_CNTR_STATUS1_P0_1                  , RULL(0x800004190801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_CNTR_STATUS1_P0_2                    , RULL(0x800008190701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_CNTR_STATUS1_P0_2                  , RULL(0x800008190701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_CNTR_STATUS1_P0_2                  , RULL(0x800008190801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_CNTR_STATUS1_P0_3                    , RULL(0x80000C190701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_CNTR_STATUS1_P0_3                  , RULL(0x80000C190701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_CNTR_STATUS1_P0_3                  , RULL(0x80000C190801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_CNTR_STATUS1_P0_4                    , RULL(0x800010190701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_CNTR_STATUS1_P0_4                  , RULL(0x800010190701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_CNTR_STATUS1_P0_4                  , RULL(0x800010190801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_CNTR_STATUS1_P1_0                  , RULL(0x800000190701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_CNTR_STATUS1_P1_0                  , RULL(0x800000190801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_CNTR_STATUS1_P1_1                  , RULL(0x800004190701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_CNTR_STATUS1_P1_1                  , RULL(0x800004190801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_CNTR_STATUS1_P1_2                  , RULL(0x800008190701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_CNTR_STATUS1_P1_2                  , RULL(0x800008190801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_CNTR_STATUS1_P1_3                  , RULL(0x80000C190701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_CNTR_STATUS1_P1_3                  , RULL(0x80000C190801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_CNTR_STATUS1_P1_4                  , RULL(0x800010190701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_CNTR_STATUS1_P1_4                  , RULL(0x800010190801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_CNTR_STATUS1_P2_0                  , RULL(0x800000190701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_CNTR_STATUS1_P2_0                  , RULL(0x800000190801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_CNTR_STATUS1_P2_1                  , RULL(0x800004190701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_CNTR_STATUS1_P2_1                  , RULL(0x800004190801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_CNTR_STATUS1_P2_2                  , RULL(0x800008190701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_CNTR_STATUS1_P2_2                  , RULL(0x800008190801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_CNTR_STATUS1_P2_3                  , RULL(0x80000C190701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_CNTR_STATUS1_P2_3                  , RULL(0x80000C190801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_CNTR_STATUS1_P2_4                  , RULL(0x800010190701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_CNTR_STATUS1_P2_4                  , RULL(0x800010190801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_CNTR_STATUS1_P3_0                  , RULL(0x8000001907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_CNTR_STATUS1_P3_0                  , RULL(0x8000001908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_CNTR_STATUS1_P3_1                  , RULL(0x8000041907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_CNTR_STATUS1_P3_1                  , RULL(0x8000041908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_CNTR_STATUS1_P3_2                  , RULL(0x8000081907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_CNTR_STATUS1_P3_2                  , RULL(0x8000081908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_CNTR_STATUS1_P3_3                  , RULL(0x80000C1907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_CNTR_STATUS1_P3_3                  , RULL(0x80000C1908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_CNTR_STATUS1_P3_4                  , RULL(0x8000101907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_CNTR_STATUS1_P3_4                  , RULL(0x8000101908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_CNTR_STATUS2_P0_0                    , RULL(0x8000001A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_CNTR_STATUS2_P0_0                  , RULL(0x8000001A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_CNTR_STATUS2_P0_0                  , RULL(0x8000001A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_CNTR_STATUS2_P0_1                    , RULL(0x8000041A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_CNTR_STATUS2_P0_1                  , RULL(0x8000041A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_CNTR_STATUS2_P0_1                  , RULL(0x8000041A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_CNTR_STATUS2_P0_2                    , RULL(0x8000081A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_CNTR_STATUS2_P0_2                  , RULL(0x8000081A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_CNTR_STATUS2_P0_2                  , RULL(0x8000081A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_CNTR_STATUS2_P0_3                    , RULL(0x80000C1A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_CNTR_STATUS2_P0_3                  , RULL(0x80000C1A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_CNTR_STATUS2_P0_3                  , RULL(0x80000C1A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_CNTR_STATUS2_P0_4                    , RULL(0x8000101A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_CNTR_STATUS2_P0_4                  , RULL(0x8000101A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_CNTR_STATUS2_P0_4                  , RULL(0x8000101A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_CNTR_STATUS2_P1_0                  , RULL(0x8000001A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_CNTR_STATUS2_P1_0                  , RULL(0x8000001A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_CNTR_STATUS2_P1_1                  , RULL(0x8000041A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_CNTR_STATUS2_P1_1                  , RULL(0x8000041A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_CNTR_STATUS2_P1_2                  , RULL(0x8000081A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_CNTR_STATUS2_P1_2                  , RULL(0x8000081A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_CNTR_STATUS2_P1_3                  , RULL(0x80000C1A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_CNTR_STATUS2_P1_3                  , RULL(0x80000C1A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_CNTR_STATUS2_P1_4                  , RULL(0x8000101A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_CNTR_STATUS2_P1_4                  , RULL(0x8000101A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_CNTR_STATUS2_P2_0                  , RULL(0x8000001A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_CNTR_STATUS2_P2_0                  , RULL(0x8000001A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_CNTR_STATUS2_P2_1                  , RULL(0x8000041A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_CNTR_STATUS2_P2_1                  , RULL(0x8000041A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_CNTR_STATUS2_P2_2                  , RULL(0x8000081A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_CNTR_STATUS2_P2_2                  , RULL(0x8000081A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_CNTR_STATUS2_P2_3                  , RULL(0x80000C1A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_CNTR_STATUS2_P2_3                  , RULL(0x80000C1A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_CNTR_STATUS2_P2_4                  , RULL(0x8000101A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_CNTR_STATUS2_P2_4                  , RULL(0x8000101A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_CNTR_STATUS2_P3_0                  , RULL(0x8000001A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_CNTR_STATUS2_P3_0                  , RULL(0x8000001A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_CNTR_STATUS2_P3_1                  , RULL(0x8000041A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_CNTR_STATUS2_P3_1                  , RULL(0x8000041A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_CNTR_STATUS2_P3_2                  , RULL(0x8000081A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_CNTR_STATUS2_P3_2                  , RULL(0x8000081A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_CNTR_STATUS2_P3_3                  , RULL(0x80000C1A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_CNTR_STATUS2_P3_3                  , RULL(0x80000C1A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_CNTR_STATUS2_P3_4                  , RULL(0x8000101A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_CNTR_STATUS2_P3_4                  , RULL(0x8000101A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_ERROR0_P0_0                          , RULL(0x8000001B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_0_DDRPHY_DP16_WR_ERROR0_P0_0                        , RULL(0x8000001B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_4_DDRPHY_DP16_WR_ERROR0_P0_0                        , RULL(0x8000001B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_DDRPHY_DP16_WR_ERROR0_P0_1                          , RULL(0x8000041B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_0_DDRPHY_DP16_WR_ERROR0_P0_1                        , RULL(0x8000041B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_4_DDRPHY_DP16_WR_ERROR0_P0_1                        , RULL(0x8000041B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_DDRPHY_DP16_WR_ERROR0_P0_2                          , RULL(0x8000081B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_0_DDRPHY_DP16_WR_ERROR0_P0_2                        , RULL(0x8000081B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_4_DDRPHY_DP16_WR_ERROR0_P0_2                        , RULL(0x8000081B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_DDRPHY_DP16_WR_ERROR0_P0_3                          , RULL(0x80000C1B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_0_DDRPHY_DP16_WR_ERROR0_P0_3                        , RULL(0x80000C1B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_4_DDRPHY_DP16_WR_ERROR0_P0_3                        , RULL(0x80000C1B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_DDRPHY_DP16_WR_ERROR0_P0_4                          , RULL(0x8000101B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_0_DDRPHY_DP16_WR_ERROR0_P0_4                        , RULL(0x8000101B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_4_DDRPHY_DP16_WR_ERROR0_P0_4                        , RULL(0x8000101B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_1_DDRPHY_DP16_WR_ERROR0_P1_0                        , RULL(0x8000001B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_5_DDRPHY_DP16_WR_ERROR0_P1_0                        , RULL(0x8000001B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_1_DDRPHY_DP16_WR_ERROR0_P1_1                        , RULL(0x8000041B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_5_DDRPHY_DP16_WR_ERROR0_P1_1                        , RULL(0x8000041B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_1_DDRPHY_DP16_WR_ERROR0_P1_2                        , RULL(0x8000081B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_5_DDRPHY_DP16_WR_ERROR0_P1_2                        , RULL(0x8000081B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_1_DDRPHY_DP16_WR_ERROR0_P1_3                        , RULL(0x80000C1B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_5_DDRPHY_DP16_WR_ERROR0_P1_3                        , RULL(0x80000C1B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_1_DDRPHY_DP16_WR_ERROR0_P1_4                        , RULL(0x8000101B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_5_DDRPHY_DP16_WR_ERROR0_P1_4                        , RULL(0x8000101B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_2_DDRPHY_DP16_WR_ERROR0_P2_0                        , RULL(0x8000001B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_6_DDRPHY_DP16_WR_ERROR0_P2_0                        , RULL(0x8000001B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_2_DDRPHY_DP16_WR_ERROR0_P2_1                        , RULL(0x8000041B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_6_DDRPHY_DP16_WR_ERROR0_P2_1                        , RULL(0x8000041B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_2_DDRPHY_DP16_WR_ERROR0_P2_2                        , RULL(0x8000081B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_6_DDRPHY_DP16_WR_ERROR0_P2_2                        , RULL(0x8000081B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_2_DDRPHY_DP16_WR_ERROR0_P2_3                        , RULL(0x80000C1B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_6_DDRPHY_DP16_WR_ERROR0_P2_3                        , RULL(0x80000C1B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_2_DDRPHY_DP16_WR_ERROR0_P2_4                        , RULL(0x8000101B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_6_DDRPHY_DP16_WR_ERROR0_P2_4                        , RULL(0x8000101B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_3_DDRPHY_DP16_WR_ERROR0_P3_0                        , RULL(0x8000001B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_7_DDRPHY_DP16_WR_ERROR0_P3_0                        , RULL(0x8000001B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_3_DDRPHY_DP16_WR_ERROR0_P3_1                        , RULL(0x8000041B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_7_DDRPHY_DP16_WR_ERROR0_P3_1                        , RULL(0x8000041B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_3_DDRPHY_DP16_WR_ERROR0_P3_2                        , RULL(0x8000081B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_7_DDRPHY_DP16_WR_ERROR0_P3_2                        , RULL(0x8000081B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_3_DDRPHY_DP16_WR_ERROR0_P3_3                        , RULL(0x80000C1B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_7_DDRPHY_DP16_WR_ERROR0_P3_3                        , RULL(0x80000C1B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_3_DDRPHY_DP16_WR_ERROR0_P3_4                        , RULL(0x8000101B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RCLRPART );
REG64( MCA_7_DDRPHY_DP16_WR_ERROR0_P3_4                        , RULL(0x8000101B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RCLRPART );

REG64( MCA_DDRPHY_DP16_WR_ERROR_MASK0_P0_0                     , RULL(0x8000001C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_ERROR_MASK0_P0_0                   , RULL(0x8000001C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_ERROR_MASK0_P0_0                   , RULL(0x8000001C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_ERROR_MASK0_P0_1                     , RULL(0x8000041C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_ERROR_MASK0_P0_1                   , RULL(0x8000041C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_ERROR_MASK0_P0_1                   , RULL(0x8000041C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_ERROR_MASK0_P0_2                     , RULL(0x8000081C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_ERROR_MASK0_P0_2                   , RULL(0x8000081C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_ERROR_MASK0_P0_2                   , RULL(0x8000081C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_ERROR_MASK0_P0_3                     , RULL(0x80000C1C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_ERROR_MASK0_P0_3                   , RULL(0x80000C1C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_ERROR_MASK0_P0_3                   , RULL(0x80000C1C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_ERROR_MASK0_P0_4                     , RULL(0x8000101C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_ERROR_MASK0_P0_4                   , RULL(0x8000101C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_ERROR_MASK0_P0_4                   , RULL(0x8000101C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_ERROR_MASK0_P1_0                   , RULL(0x8000001C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_ERROR_MASK0_P1_0                   , RULL(0x8000001C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_ERROR_MASK0_P1_1                   , RULL(0x8000041C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_ERROR_MASK0_P1_1                   , RULL(0x8000041C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_ERROR_MASK0_P1_2                   , RULL(0x8000081C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_ERROR_MASK0_P1_2                   , RULL(0x8000081C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_ERROR_MASK0_P1_3                   , RULL(0x80000C1C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_ERROR_MASK0_P1_3                   , RULL(0x80000C1C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_ERROR_MASK0_P1_4                   , RULL(0x8000101C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_ERROR_MASK0_P1_4                   , RULL(0x8000101C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_ERROR_MASK0_P2_0                   , RULL(0x8000001C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_ERROR_MASK0_P2_0                   , RULL(0x8000001C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_ERROR_MASK0_P2_1                   , RULL(0x8000041C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_ERROR_MASK0_P2_1                   , RULL(0x8000041C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_ERROR_MASK0_P2_2                   , RULL(0x8000081C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_ERROR_MASK0_P2_2                   , RULL(0x8000081C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_ERROR_MASK0_P2_3                   , RULL(0x80000C1C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_ERROR_MASK0_P2_3                   , RULL(0x80000C1C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_ERROR_MASK0_P2_4                   , RULL(0x8000101C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_ERROR_MASK0_P2_4                   , RULL(0x8000101C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_ERROR_MASK0_P3_0                   , RULL(0x8000001C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_ERROR_MASK0_P3_0                   , RULL(0x8000001C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_ERROR_MASK0_P3_1                   , RULL(0x8000041C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_ERROR_MASK0_P3_1                   , RULL(0x8000041C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_ERROR_MASK0_P3_2                   , RULL(0x8000081C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_ERROR_MASK0_P3_2                   , RULL(0x8000081C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_ERROR_MASK0_P3_3                   , RULL(0x80000C1C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_ERROR_MASK0_P3_3                   , RULL(0x80000C1C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_ERROR_MASK0_P3_4                   , RULL(0x8000101C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_ERROR_MASK0_P3_4                   , RULL(0x8000101C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_LVL_STATUS0_P0_0                     , RULL(0x800000170701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_LVL_STATUS0_P0_0                   , RULL(0x800000170701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_LVL_STATUS0_P0_0                   , RULL(0x800000170801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_LVL_STATUS0_P0_1                     , RULL(0x800004170701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_LVL_STATUS0_P0_1                   , RULL(0x800004170701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_LVL_STATUS0_P0_1                   , RULL(0x800004170801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_LVL_STATUS0_P0_2                     , RULL(0x800008170701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_LVL_STATUS0_P0_2                   , RULL(0x800008170701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_LVL_STATUS0_P0_2                   , RULL(0x800008170801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_LVL_STATUS0_P0_3                     , RULL(0x80000C170701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_LVL_STATUS0_P0_3                   , RULL(0x80000C170701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_LVL_STATUS0_P0_3                   , RULL(0x80000C170801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_LVL_STATUS0_P0_4                     , RULL(0x800010170701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_LVL_STATUS0_P0_4                   , RULL(0x800010170701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_LVL_STATUS0_P0_4                   , RULL(0x800010170801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_LVL_STATUS0_P1_0                   , RULL(0x800000170701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_LVL_STATUS0_P1_0                   , RULL(0x800000170801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_LVL_STATUS0_P1_1                   , RULL(0x800004170701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_LVL_STATUS0_P1_1                   , RULL(0x800004170801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_LVL_STATUS0_P1_2                   , RULL(0x800008170701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_LVL_STATUS0_P1_2                   , RULL(0x800008170801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_LVL_STATUS0_P1_3                   , RULL(0x80000C170701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_LVL_STATUS0_P1_3                   , RULL(0x80000C170801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_LVL_STATUS0_P1_4                   , RULL(0x800010170701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_LVL_STATUS0_P1_4                   , RULL(0x800010170801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_LVL_STATUS0_P2_0                   , RULL(0x800000170701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_LVL_STATUS0_P2_0                   , RULL(0x800000170801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_LVL_STATUS0_P2_1                   , RULL(0x800004170701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_LVL_STATUS0_P2_1                   , RULL(0x800004170801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_LVL_STATUS0_P2_2                   , RULL(0x800008170701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_LVL_STATUS0_P2_2                   , RULL(0x800008170801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_LVL_STATUS0_P2_3                   , RULL(0x80000C170701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_LVL_STATUS0_P2_3                   , RULL(0x80000C170801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_LVL_STATUS0_P2_4                   , RULL(0x800010170701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_LVL_STATUS0_P2_4                   , RULL(0x800010170801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_LVL_STATUS0_P3_0                   , RULL(0x8000001707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_LVL_STATUS0_P3_0                   , RULL(0x8000001708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_LVL_STATUS0_P3_1                   , RULL(0x8000041707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_LVL_STATUS0_P3_1                   , RULL(0x8000041708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_LVL_STATUS0_P3_2                   , RULL(0x8000081707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_LVL_STATUS0_P3_2                   , RULL(0x8000081708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_LVL_STATUS0_P3_3                   , RULL(0x80000C1707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_LVL_STATUS0_P3_3                   , RULL(0x80000C1708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_LVL_STATUS0_P3_4                   , RULL(0x8000101707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_LVL_STATUS0_P3_4                   , RULL(0x8000101708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_CONFIG0_P0_0                    , RULL(0x8000006C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_CONFIG0_P0_0                  , RULL(0x8000006C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_CONFIG0_P0_0                  , RULL(0x8000006C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_CONFIG0_P0_1                    , RULL(0x8000046C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_CONFIG0_P0_1                  , RULL(0x8000046C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_CONFIG0_P0_1                  , RULL(0x8000046C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_CONFIG0_P0_2                    , RULL(0x8000086C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_CONFIG0_P0_2                  , RULL(0x8000086C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_CONFIG0_P0_2                  , RULL(0x8000086C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_CONFIG0_P0_3                    , RULL(0x80000C6C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_CONFIG0_P0_3                  , RULL(0x80000C6C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_CONFIG0_P0_3                  , RULL(0x80000C6C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_CONFIG0_P0_4                    , RULL(0x8000106C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_CONFIG0_P0_4                  , RULL(0x8000106C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_CONFIG0_P0_4                  , RULL(0x8000106C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_CONFIG0_P1_0                  , RULL(0x8000006C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_CONFIG0_P1_0                  , RULL(0x8000006C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_CONFIG0_P1_1                  , RULL(0x8000046C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_CONFIG0_P1_1                  , RULL(0x8000046C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_CONFIG0_P1_2                  , RULL(0x8000086C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_CONFIG0_P1_2                  , RULL(0x8000086C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_CONFIG0_P1_3                  , RULL(0x80000C6C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_CONFIG0_P1_3                  , RULL(0x80000C6C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_CONFIG0_P1_4                  , RULL(0x8000106C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_CONFIG0_P1_4                  , RULL(0x8000106C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_CONFIG0_P2_0                  , RULL(0x8000006C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_CONFIG0_P2_0                  , RULL(0x8000006C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_CONFIG0_P2_1                  , RULL(0x8000046C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_CONFIG0_P2_1                  , RULL(0x8000046C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_CONFIG0_P2_2                  , RULL(0x8000086C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_CONFIG0_P2_2                  , RULL(0x8000086C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_CONFIG0_P2_3                  , RULL(0x80000C6C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_CONFIG0_P2_3                  , RULL(0x80000C6C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_CONFIG0_P2_4                  , RULL(0x8000106C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_CONFIG0_P2_4                  , RULL(0x8000106C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_CONFIG0_P3_0                  , RULL(0x8000006C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_CONFIG0_P3_0                  , RULL(0x8000006C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_CONFIG0_P3_1                  , RULL(0x8000046C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_CONFIG0_P3_1                  , RULL(0x8000046C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_CONFIG0_P3_2                  , RULL(0x8000086C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_CONFIG0_P3_2                  , RULL(0x8000086C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_CONFIG0_P3_3                  , RULL(0x80000C6C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_CONFIG0_P3_3                  , RULL(0x80000C6C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_CONFIG0_P3_4                  , RULL(0x8000106C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_CONFIG0_P3_4                  , RULL(0x8000106C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_CONFIG1_P0_0                    , RULL(0x800000EC0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_CONFIG1_P0_0                  , RULL(0x800000EC0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_CONFIG1_P0_0                  , RULL(0x800000EC0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_CONFIG1_P0_1                    , RULL(0x800004EC0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_CONFIG1_P0_1                  , RULL(0x800004EC0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_CONFIG1_P0_1                  , RULL(0x800004EC0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_CONFIG1_P0_2                    , RULL(0x800008EC0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_CONFIG1_P0_2                  , RULL(0x800008EC0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_CONFIG1_P0_2                  , RULL(0x800008EC0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_CONFIG1_P0_3                    , RULL(0x80000CEC0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_CONFIG1_P0_3                  , RULL(0x80000CEC0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_CONFIG1_P0_3                  , RULL(0x80000CEC0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_CONFIG1_P0_4                    , RULL(0x800010EC0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_CONFIG1_P0_4                  , RULL(0x800010EC0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_CONFIG1_P0_4                  , RULL(0x800010EC0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_CONFIG1_P1_0                  , RULL(0x800000EC0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_CONFIG1_P1_0                  , RULL(0x800000EC0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_CONFIG1_P1_1                  , RULL(0x800004EC0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_CONFIG1_P1_1                  , RULL(0x800004EC0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_CONFIG1_P1_2                  , RULL(0x800008EC0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_CONFIG1_P1_2                  , RULL(0x800008EC0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_CONFIG1_P1_3                  , RULL(0x80000CEC0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_CONFIG1_P1_3                  , RULL(0x80000CEC0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_CONFIG1_P1_4                  , RULL(0x800010EC0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_CONFIG1_P1_4                  , RULL(0x800010EC0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_CONFIG1_P2_0                  , RULL(0x800000EC0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_CONFIG1_P2_0                  , RULL(0x800000EC0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_CONFIG1_P2_1                  , RULL(0x800004EC0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_CONFIG1_P2_1                  , RULL(0x800004EC0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_CONFIG1_P2_2                  , RULL(0x800008EC0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_CONFIG1_P2_2                  , RULL(0x800008EC0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_CONFIG1_P2_3                  , RULL(0x80000CEC0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_CONFIG1_P2_3                  , RULL(0x80000CEC0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_CONFIG1_P2_4                  , RULL(0x800010EC0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_CONFIG1_P2_4                  , RULL(0x800010EC0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_CONFIG1_P3_0                  , RULL(0x800000EC07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_CONFIG1_P3_0                  , RULL(0x800000EC08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_CONFIG1_P3_1                  , RULL(0x800004EC07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_CONFIG1_P3_1                  , RULL(0x800004EC08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_CONFIG1_P3_2                  , RULL(0x800008EC07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_CONFIG1_P3_2                  , RULL(0x800008EC08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_CONFIG1_P3_3                  , RULL(0x80000CEC07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_CONFIG1_P3_3                  , RULL(0x80000CEC08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_CONFIG1_P3_4                  , RULL(0x800010EC07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_CONFIG1_P3_4                  , RULL(0x800010EC08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR0_P0_0                     , RULL(0x800000AE0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR0_P0_0                   , RULL(0x800000AE0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR0_P0_0                   , RULL(0x800000AE0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR0_P0_1                     , RULL(0x800004AE0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR0_P0_1                   , RULL(0x800004AE0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR0_P0_1                   , RULL(0x800004AE0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR0_P0_2                     , RULL(0x800008AE0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR0_P0_2                   , RULL(0x800008AE0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR0_P0_2                   , RULL(0x800008AE0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR0_P0_3                     , RULL(0x80000CAE0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR0_P0_3                   , RULL(0x80000CAE0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR0_P0_3                   , RULL(0x80000CAE0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR0_P0_4                     , RULL(0x800010AE0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR0_P0_4                   , RULL(0x800010AE0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR0_P0_4                   , RULL(0x800010AE0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR0_P1_0                   , RULL(0x800000AE0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR0_P1_0                   , RULL(0x800000AE0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR0_P1_1                   , RULL(0x800004AE0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR0_P1_1                   , RULL(0x800004AE0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR0_P1_2                   , RULL(0x800008AE0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR0_P1_2                   , RULL(0x800008AE0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR0_P1_3                   , RULL(0x80000CAE0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR0_P1_3                   , RULL(0x80000CAE0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR0_P1_4                   , RULL(0x800010AE0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR0_P1_4                   , RULL(0x800010AE0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR0_P2_0                   , RULL(0x800000AE0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR0_P2_0                   , RULL(0x800000AE0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR0_P2_1                   , RULL(0x800004AE0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR0_P2_1                   , RULL(0x800004AE0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR0_P2_2                   , RULL(0x800008AE0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR0_P2_2                   , RULL(0x800008AE0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR0_P2_3                   , RULL(0x80000CAE0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR0_P2_3                   , RULL(0x80000CAE0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR0_P2_4                   , RULL(0x800010AE0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR0_P2_4                   , RULL(0x800010AE0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR0_P3_0                   , RULL(0x800000AE07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR0_P3_0                   , RULL(0x800000AE08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR0_P3_1                   , RULL(0x800004AE07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR0_P3_1                   , RULL(0x800004AE08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR0_P3_2                   , RULL(0x800008AE07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR0_P3_2                   , RULL(0x800008AE08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR0_P3_3                   , RULL(0x80000CAE07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR0_P3_3                   , RULL(0x80000CAE08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR0_P3_4                   , RULL(0x800010AE07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR0_P3_4                   , RULL(0x800010AE08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR1_P0_0                     , RULL(0x800000AF0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR1_P0_0                   , RULL(0x800000AF0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR1_P0_0                   , RULL(0x800000AF0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR1_P0_1                     , RULL(0x800004AF0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR1_P0_1                   , RULL(0x800004AF0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR1_P0_1                   , RULL(0x800004AF0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR1_P0_2                     , RULL(0x800008AF0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR1_P0_2                   , RULL(0x800008AF0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR1_P0_2                   , RULL(0x800008AF0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR1_P0_3                     , RULL(0x80000CAF0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR1_P0_3                   , RULL(0x80000CAF0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR1_P0_3                   , RULL(0x80000CAF0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR1_P0_4                     , RULL(0x800010AF0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR1_P0_4                   , RULL(0x800010AF0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR1_P0_4                   , RULL(0x800010AF0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR1_P1_0                   , RULL(0x800000AF0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR1_P1_0                   , RULL(0x800000AF0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR1_P1_1                   , RULL(0x800004AF0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR1_P1_1                   , RULL(0x800004AF0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR1_P1_2                   , RULL(0x800008AF0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR1_P1_2                   , RULL(0x800008AF0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR1_P1_3                   , RULL(0x80000CAF0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR1_P1_3                   , RULL(0x80000CAF0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR1_P1_4                   , RULL(0x800010AF0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR1_P1_4                   , RULL(0x800010AF0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR1_P2_0                   , RULL(0x800000AF0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR1_P2_0                   , RULL(0x800000AF0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR1_P2_1                   , RULL(0x800004AF0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR1_P2_1                   , RULL(0x800004AF0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR1_P2_2                   , RULL(0x800008AF0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR1_P2_2                   , RULL(0x800008AF0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR1_P2_3                   , RULL(0x80000CAF0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR1_P2_3                   , RULL(0x80000CAF0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR1_P2_4                   , RULL(0x800010AF0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR1_P2_4                   , RULL(0x800010AF0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR1_P3_0                   , RULL(0x800000AF07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR1_P3_0                   , RULL(0x800000AF08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR1_P3_1                   , RULL(0x800004AF07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR1_P3_1                   , RULL(0x800004AF08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR1_P3_2                   , RULL(0x800008AF07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR1_P3_2                   , RULL(0x800008AF08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR1_P3_3                   , RULL(0x80000CAF07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR1_P3_3                   , RULL(0x80000CAF08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR1_P3_4                   , RULL(0x800010AF07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR1_P3_4                   , RULL(0x800010AF08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P0_0                , RULL(0x800000FB0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P0_0              , RULL(0x800000FB0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P0_0              , RULL(0x800000FB0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P0_1                , RULL(0x800004FB0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P0_1              , RULL(0x800004FB0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P0_1              , RULL(0x800004FB0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P0_2                , RULL(0x800008FB0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P0_2              , RULL(0x800008FB0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P0_2              , RULL(0x800008FB0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P0_3                , RULL(0x80000CFB0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P0_3              , RULL(0x80000CFB0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P0_3              , RULL(0x80000CFB0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P0_4                , RULL(0x800010FB0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P0_4              , RULL(0x800010FB0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P0_4              , RULL(0x800010FB0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P1_0              , RULL(0x800000FB0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P1_0              , RULL(0x800000FB0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P1_1              , RULL(0x800004FB0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P1_1              , RULL(0x800004FB0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P1_2              , RULL(0x800008FB0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P1_2              , RULL(0x800008FB0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P1_3              , RULL(0x80000CFB0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P1_3              , RULL(0x80000CFB0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P1_4              , RULL(0x800010FB0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P1_4              , RULL(0x800010FB0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P2_0              , RULL(0x800000FB0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P2_0              , RULL(0x800000FB0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P2_1              , RULL(0x800004FB0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P2_1              , RULL(0x800004FB0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P2_2              , RULL(0x800008FB0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P2_2              , RULL(0x800008FB0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P2_3              , RULL(0x80000CFB0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P2_3              , RULL(0x80000CFB0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P2_4              , RULL(0x800010FB0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P2_4              , RULL(0x800010FB0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P3_0              , RULL(0x800000FB07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P3_0              , RULL(0x800000FB08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P3_1              , RULL(0x800004FB07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P3_1              , RULL(0x800004FB08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P3_2              , RULL(0x800008FB07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P3_2              , RULL(0x800008FB08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P3_3              , RULL(0x80000CFB07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P3_3              , RULL(0x80000CFB08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P3_4              , RULL(0x800010FB07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR_MASK0_P3_4              , RULL(0x800010FB08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P0_0                , RULL(0x800000FA0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P0_0              , RULL(0x800000FA0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P0_0              , RULL(0x800000FA0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P0_1                , RULL(0x800004FA0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P0_1              , RULL(0x800004FA0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P0_1              , RULL(0x800004FA0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P0_2                , RULL(0x800008FA0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P0_2              , RULL(0x800008FA0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P0_2              , RULL(0x800008FA0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P0_3                , RULL(0x80000CFA0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P0_3              , RULL(0x80000CFA0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P0_3              , RULL(0x80000CFA0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P0_4                , RULL(0x800010FA0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P0_4              , RULL(0x800010FA0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P0_4              , RULL(0x800010FA0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P1_0              , RULL(0x800000FA0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P1_0              , RULL(0x800000FA0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P1_1              , RULL(0x800004FA0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P1_1              , RULL(0x800004FA0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P1_2              , RULL(0x800008FA0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P1_2              , RULL(0x800008FA0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P1_3              , RULL(0x80000CFA0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P1_3              , RULL(0x80000CFA0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P1_4              , RULL(0x800010FA0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P1_4              , RULL(0x800010FA0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P2_0              , RULL(0x800000FA0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P2_0              , RULL(0x800000FA0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P2_1              , RULL(0x800004FA0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P2_1              , RULL(0x800004FA0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P2_2              , RULL(0x800008FA0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P2_2              , RULL(0x800008FA0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P2_3              , RULL(0x80000CFA0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P2_3              , RULL(0x80000CFA0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P2_4              , RULL(0x800010FA0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P2_4              , RULL(0x800010FA0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P3_0              , RULL(0x800000FA07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P3_0              , RULL(0x800000FA08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P3_1              , RULL(0x800004FA07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P3_1              , RULL(0x800004FA08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P3_2              , RULL(0x800008FA07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P3_2              , RULL(0x800008FA08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P3_3              , RULL(0x80000CFA07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P3_3              , RULL(0x80000CFA08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P3_4              , RULL(0x800010FA07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_ERROR_MASK1_P3_4              , RULL(0x800010FA08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_STATUS0_P0_0                    , RULL(0x8000002E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_STATUS0_P0_0                  , RULL(0x8000002E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_STATUS0_P0_0                  , RULL(0x8000002E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_STATUS0_P0_1                    , RULL(0x8000042E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_STATUS0_P0_1                  , RULL(0x8000042E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_STATUS0_P0_1                  , RULL(0x8000042E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_STATUS0_P0_2                    , RULL(0x8000082E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_STATUS0_P0_2                  , RULL(0x8000082E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_STATUS0_P0_2                  , RULL(0x8000082E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_STATUS0_P0_3                    , RULL(0x80000C2E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_STATUS0_P0_3                  , RULL(0x80000C2E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_STATUS0_P0_3                  , RULL(0x80000C2E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_STATUS0_P0_4                    , RULL(0x8000102E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_STATUS0_P0_4                  , RULL(0x8000102E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_STATUS0_P0_4                  , RULL(0x8000102E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_STATUS0_P1_0                  , RULL(0x8000002E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_STATUS0_P1_0                  , RULL(0x8000002E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_STATUS0_P1_1                  , RULL(0x8000042E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_STATUS0_P1_1                  , RULL(0x8000042E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_STATUS0_P1_2                  , RULL(0x8000082E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_STATUS0_P1_2                  , RULL(0x8000082E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_STATUS0_P1_3                  , RULL(0x80000C2E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_STATUS0_P1_3                  , RULL(0x80000C2E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_STATUS0_P1_4                  , RULL(0x8000102E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_STATUS0_P1_4                  , RULL(0x8000102E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_STATUS0_P2_0                  , RULL(0x8000002E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_STATUS0_P2_0                  , RULL(0x8000002E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_STATUS0_P2_1                  , RULL(0x8000042E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_STATUS0_P2_1                  , RULL(0x8000042E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_STATUS0_P2_2                  , RULL(0x8000082E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_STATUS0_P2_2                  , RULL(0x8000082E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_STATUS0_P2_3                  , RULL(0x80000C2E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_STATUS0_P2_3                  , RULL(0x80000C2E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_STATUS0_P2_4                  , RULL(0x8000102E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_STATUS0_P2_4                  , RULL(0x8000102E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_STATUS0_P3_0                  , RULL(0x8000002E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_STATUS0_P3_0                  , RULL(0x8000002E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_STATUS0_P3_1                  , RULL(0x8000042E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_STATUS0_P3_1                  , RULL(0x8000042E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_STATUS0_P3_2                  , RULL(0x8000082E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_STATUS0_P3_2                  , RULL(0x8000082E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_STATUS0_P3_3                  , RULL(0x80000C2E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_STATUS0_P3_3                  , RULL(0x80000C2E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_STATUS0_P3_4                  , RULL(0x8000102E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_STATUS0_P3_4                  , RULL(0x8000102E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_STATUS1_P0_0                    , RULL(0x8000002F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_STATUS1_P0_0                  , RULL(0x8000002F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_STATUS1_P0_0                  , RULL(0x8000002F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_STATUS1_P0_1                    , RULL(0x8000042F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_STATUS1_P0_1                  , RULL(0x8000042F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_STATUS1_P0_1                  , RULL(0x8000042F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_STATUS1_P0_2                    , RULL(0x8000082F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_STATUS1_P0_2                  , RULL(0x8000082F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_STATUS1_P0_2                  , RULL(0x8000082F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_STATUS1_P0_3                    , RULL(0x80000C2F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_STATUS1_P0_3                  , RULL(0x80000C2F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_STATUS1_P0_3                  , RULL(0x80000C2F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_STATUS1_P0_4                    , RULL(0x8000102F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_STATUS1_P0_4                  , RULL(0x8000102F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_STATUS1_P0_4                  , RULL(0x8000102F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_STATUS1_P1_0                  , RULL(0x8000002F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_STATUS1_P1_0                  , RULL(0x8000002F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_STATUS1_P1_1                  , RULL(0x8000042F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_STATUS1_P1_1                  , RULL(0x8000042F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_STATUS1_P1_2                  , RULL(0x8000082F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_STATUS1_P1_2                  , RULL(0x8000082F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_STATUS1_P1_3                  , RULL(0x80000C2F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_STATUS1_P1_3                  , RULL(0x80000C2F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_STATUS1_P1_4                  , RULL(0x8000102F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_STATUS1_P1_4                  , RULL(0x8000102F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_STATUS1_P2_0                  , RULL(0x8000002F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_STATUS1_P2_0                  , RULL(0x8000002F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_STATUS1_P2_1                  , RULL(0x8000042F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_STATUS1_P2_1                  , RULL(0x8000042F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_STATUS1_P2_2                  , RULL(0x8000082F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_STATUS1_P2_2                  , RULL(0x8000082F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_STATUS1_P2_3                  , RULL(0x80000C2F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_STATUS1_P2_3                  , RULL(0x80000C2F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_STATUS1_P2_4                  , RULL(0x8000102F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_STATUS1_P2_4                  , RULL(0x8000102F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_STATUS1_P3_0                  , RULL(0x8000002F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_STATUS1_P3_0                  , RULL(0x8000002F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_STATUS1_P3_1                  , RULL(0x8000042F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_STATUS1_P3_1                  , RULL(0x8000042F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_STATUS1_P3_2                  , RULL(0x8000082F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_STATUS1_P3_2                  , RULL(0x8000082F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_STATUS1_P3_3                  , RULL(0x80000C2F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_STATUS1_P3_3                  , RULL(0x80000C2F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_STATUS1_P3_4                  , RULL(0x8000102F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_STATUS1_P3_4                  , RULL(0x8000102F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P0_0          , RULL(0x8000005E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P0_0        , RULL(0x8000005E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P0_0        , RULL(0x8000005E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P0_1          , RULL(0x8000045E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P0_1        , RULL(0x8000045E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P0_1        , RULL(0x8000045E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P0_2          , RULL(0x8000085E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P0_2        , RULL(0x8000085E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P0_2        , RULL(0x8000085E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P0_3          , RULL(0x80000C5E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P0_3        , RULL(0x80000C5E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P0_3        , RULL(0x80000C5E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P0_4          , RULL(0x8000105E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P0_4        , RULL(0x8000105E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P0_4        , RULL(0x8000105E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P1_0        , RULL(0x8000005E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P1_0        , RULL(0x8000005E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P1_1        , RULL(0x8000045E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P1_1        , RULL(0x8000045E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P1_2        , RULL(0x8000085E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P1_2        , RULL(0x8000085E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P1_3        , RULL(0x80000C5E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P1_3        , RULL(0x80000C5E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P1_4        , RULL(0x8000105E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P1_4        , RULL(0x8000105E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P2_0        , RULL(0x8000005E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P2_0        , RULL(0x8000005E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P2_1        , RULL(0x8000045E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P2_1        , RULL(0x8000045E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P2_2        , RULL(0x8000085E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P2_2        , RULL(0x8000085E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P2_3        , RULL(0x80000C5E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P2_3        , RULL(0x80000C5E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P2_4        , RULL(0x8000105E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P2_4        , RULL(0x8000105E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P3_0        , RULL(0x8000005E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P3_0        , RULL(0x8000005E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P3_1        , RULL(0x8000045E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P3_1        , RULL(0x8000045E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P3_2        , RULL(0x8000085E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P3_2        , RULL(0x8000085E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P3_3        , RULL(0x80000C5E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P3_3        , RULL(0x80000C5E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P3_4        , RULL(0x8000105E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR0_P3_4        , RULL(0x8000105E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P0_0          , RULL(0x8000015E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P0_0        , RULL(0x8000015E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P0_0        , RULL(0x8000015E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P0_1          , RULL(0x8000055E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P0_1        , RULL(0x8000055E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P0_1        , RULL(0x8000055E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P0_2          , RULL(0x8000095E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P0_2        , RULL(0x8000095E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P0_2        , RULL(0x8000095E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P0_3          , RULL(0x80000D5E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P0_3        , RULL(0x80000D5E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P0_3        , RULL(0x80000D5E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P0_4          , RULL(0x8000115E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P0_4        , RULL(0x8000115E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P0_4        , RULL(0x8000115E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P1_0        , RULL(0x8000015E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P1_0        , RULL(0x8000015E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P1_1        , RULL(0x8000055E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P1_1        , RULL(0x8000055E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P1_2        , RULL(0x8000095E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P1_2        , RULL(0x8000095E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P1_3        , RULL(0x80000D5E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P1_3        , RULL(0x80000D5E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P1_4        , RULL(0x8000115E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P1_4        , RULL(0x8000115E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P2_0        , RULL(0x8000015E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P2_0        , RULL(0x8000015E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P2_1        , RULL(0x8000055E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P2_1        , RULL(0x8000055E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P2_2        , RULL(0x8000095E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P2_2        , RULL(0x8000095E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P2_3        , RULL(0x80000D5E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P2_3        , RULL(0x80000D5E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P2_4        , RULL(0x8000115E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P2_4        , RULL(0x8000115E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P3_0        , RULL(0x8000015E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P3_0        , RULL(0x8000015E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P3_1        , RULL(0x8000055E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P3_1        , RULL(0x8000055E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P3_2        , RULL(0x8000095E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P3_2        , RULL(0x8000095E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P3_3        , RULL(0x80000D5E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P3_3        , RULL(0x80000D5E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P3_4        , RULL(0x8000115E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR1_P3_4        , RULL(0x8000115E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P0_0          , RULL(0x8000025E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P0_0        , RULL(0x8000025E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P0_0        , RULL(0x8000025E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P0_1          , RULL(0x8000065E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P0_1        , RULL(0x8000065E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P0_1        , RULL(0x8000065E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P0_2          , RULL(0x80000A5E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P0_2        , RULL(0x80000A5E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P0_2        , RULL(0x80000A5E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P0_3          , RULL(0x80000E5E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P0_3        , RULL(0x80000E5E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P0_3        , RULL(0x80000E5E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P0_4          , RULL(0x8000125E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P0_4        , RULL(0x8000125E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P0_4        , RULL(0x8000125E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P1_0        , RULL(0x8000025E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P1_0        , RULL(0x8000025E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P1_1        , RULL(0x8000065E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P1_1        , RULL(0x8000065E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P1_2        , RULL(0x80000A5E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P1_2        , RULL(0x80000A5E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P1_3        , RULL(0x80000E5E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P1_3        , RULL(0x80000E5E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P1_4        , RULL(0x8000125E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P1_4        , RULL(0x8000125E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P2_0        , RULL(0x8000025E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P2_0        , RULL(0x8000025E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P2_1        , RULL(0x8000065E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P2_1        , RULL(0x8000065E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P2_2        , RULL(0x80000A5E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P2_2        , RULL(0x80000A5E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P2_3        , RULL(0x80000E5E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P2_3        , RULL(0x80000E5E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P2_4        , RULL(0x8000125E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P2_4        , RULL(0x8000125E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P3_0        , RULL(0x8000025E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P3_0        , RULL(0x8000025E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P3_1        , RULL(0x8000065E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P3_1        , RULL(0x8000065E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P3_2        , RULL(0x80000A5E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P3_2        , RULL(0x80000A5E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P3_3        , RULL(0x80000E5E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P3_3        , RULL(0x80000E5E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P3_4        , RULL(0x8000125E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR2_P3_4        , RULL(0x8000125E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P0_0          , RULL(0x8000035E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P0_0        , RULL(0x8000035E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P0_0        , RULL(0x8000035E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P0_1          , RULL(0x8000075E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P0_1        , RULL(0x8000075E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P0_1        , RULL(0x8000075E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P0_2          , RULL(0x80000B5E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P0_2        , RULL(0x80000B5E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P0_2        , RULL(0x80000B5E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P0_3          , RULL(0x80000F5E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P0_3        , RULL(0x80000F5E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P0_3        , RULL(0x80000F5E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P0_4          , RULL(0x8000135E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P0_4        , RULL(0x8000135E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P0_4        , RULL(0x8000135E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P1_0        , RULL(0x8000035E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P1_0        , RULL(0x8000035E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P1_1        , RULL(0x8000075E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P1_1        , RULL(0x8000075E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P1_2        , RULL(0x80000B5E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P1_2        , RULL(0x80000B5E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P1_3        , RULL(0x80000F5E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P1_3        , RULL(0x80000F5E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P1_4        , RULL(0x8000135E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P1_4        , RULL(0x8000135E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P2_0        , RULL(0x8000035E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P2_0        , RULL(0x8000035E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P2_1        , RULL(0x8000075E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P2_1        , RULL(0x8000075E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P2_2        , RULL(0x80000B5E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P2_2        , RULL(0x80000B5E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P2_3        , RULL(0x80000F5E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P2_3        , RULL(0x80000F5E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P2_4        , RULL(0x8000135E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P2_4        , RULL(0x8000135E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P3_0        , RULL(0x8000035E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P3_0        , RULL(0x8000035E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P3_1        , RULL(0x8000075E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P3_1        , RULL(0x8000075E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P3_2        , RULL(0x80000B5E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P3_2        , RULL(0x80000B5E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P3_3        , RULL(0x80000F5E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P3_3        , RULL(0x80000F5E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P3_4        , RULL(0x8000135E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE0_RANK_PAIR3_P3_4        , RULL(0x8000135E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P0_0          , RULL(0x8000005F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P0_0        , RULL(0x8000005F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P0_0        , RULL(0x8000005F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P0_1          , RULL(0x8000045F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P0_1        , RULL(0x8000045F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P0_1        , RULL(0x8000045F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P0_2          , RULL(0x8000085F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P0_2        , RULL(0x8000085F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P0_2        , RULL(0x8000085F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P0_3          , RULL(0x80000C5F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P0_3        , RULL(0x80000C5F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P0_3        , RULL(0x80000C5F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P0_4          , RULL(0x8000105F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P0_4        , RULL(0x8000105F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P0_4        , RULL(0x8000105F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P1_0        , RULL(0x8000005F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P1_0        , RULL(0x8000005F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P1_1        , RULL(0x8000045F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P1_1        , RULL(0x8000045F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P1_2        , RULL(0x8000085F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P1_2        , RULL(0x8000085F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P1_3        , RULL(0x80000C5F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P1_3        , RULL(0x80000C5F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P1_4        , RULL(0x8000105F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P1_4        , RULL(0x8000105F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P2_0        , RULL(0x8000005F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P2_0        , RULL(0x8000005F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P2_1        , RULL(0x8000045F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P2_1        , RULL(0x8000045F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P2_2        , RULL(0x8000085F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P2_2        , RULL(0x8000085F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P2_3        , RULL(0x80000C5F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P2_3        , RULL(0x80000C5F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P2_4        , RULL(0x8000105F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P2_4        , RULL(0x8000105F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P3_0        , RULL(0x8000005F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P3_0        , RULL(0x8000005F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P3_1        , RULL(0x8000045F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P3_1        , RULL(0x8000045F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P3_2        , RULL(0x8000085F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P3_2        , RULL(0x8000085F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P3_3        , RULL(0x80000C5F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P3_3        , RULL(0x80000C5F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P3_4        , RULL(0x8000105F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR0_P3_4        , RULL(0x8000105F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P0_0          , RULL(0x8000015F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P0_0        , RULL(0x8000015F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P0_0        , RULL(0x8000015F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P0_1          , RULL(0x8000055F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P0_1        , RULL(0x8000055F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P0_1        , RULL(0x8000055F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P0_2          , RULL(0x8000095F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P0_2        , RULL(0x8000095F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P0_2        , RULL(0x8000095F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P0_3          , RULL(0x80000D5F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P0_3        , RULL(0x80000D5F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P0_3        , RULL(0x80000D5F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P0_4          , RULL(0x8000115F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P0_4        , RULL(0x8000115F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P0_4        , RULL(0x8000115F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P1_0        , RULL(0x8000015F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P1_0        , RULL(0x8000015F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P1_1        , RULL(0x8000055F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P1_1        , RULL(0x8000055F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P1_2        , RULL(0x8000095F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P1_2        , RULL(0x8000095F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P1_3        , RULL(0x80000D5F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P1_3        , RULL(0x80000D5F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P1_4        , RULL(0x8000115F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P1_4        , RULL(0x8000115F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P2_0        , RULL(0x8000015F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P2_0        , RULL(0x8000015F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P2_1        , RULL(0x8000055F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P2_1        , RULL(0x8000055F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P2_2        , RULL(0x8000095F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P2_2        , RULL(0x8000095F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P2_3        , RULL(0x80000D5F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P2_3        , RULL(0x80000D5F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P2_4        , RULL(0x8000115F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P2_4        , RULL(0x8000115F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P3_0        , RULL(0x8000015F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P3_0        , RULL(0x8000015F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P3_1        , RULL(0x8000055F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P3_1        , RULL(0x8000055F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P3_2        , RULL(0x8000095F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P3_2        , RULL(0x8000095F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P3_3        , RULL(0x80000D5F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P3_3        , RULL(0x80000D5F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P3_4        , RULL(0x8000115F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR1_P3_4        , RULL(0x8000115F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P0_0          , RULL(0x8000025F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P0_0        , RULL(0x8000025F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P0_0        , RULL(0x8000025F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P0_1          , RULL(0x8000065F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P0_1        , RULL(0x8000065F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P0_1        , RULL(0x8000065F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P0_2          , RULL(0x80000A5F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P0_2        , RULL(0x80000A5F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P0_2        , RULL(0x80000A5F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P0_3          , RULL(0x80000E5F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P0_3        , RULL(0x80000E5F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P0_3        , RULL(0x80000E5F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P0_4          , RULL(0x8000125F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P0_4        , RULL(0x8000125F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P0_4        , RULL(0x8000125F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P1_0        , RULL(0x8000025F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P1_0        , RULL(0x8000025F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P1_1        , RULL(0x8000065F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P1_1        , RULL(0x8000065F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P1_2        , RULL(0x80000A5F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P1_2        , RULL(0x80000A5F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P1_3        , RULL(0x80000E5F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P1_3        , RULL(0x80000E5F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P1_4        , RULL(0x8000125F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P1_4        , RULL(0x8000125F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P2_0        , RULL(0x8000025F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P2_0        , RULL(0x8000025F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P2_1        , RULL(0x8000065F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P2_1        , RULL(0x8000065F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P2_2        , RULL(0x80000A5F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P2_2        , RULL(0x80000A5F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P2_3        , RULL(0x80000E5F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P2_3        , RULL(0x80000E5F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P2_4        , RULL(0x8000125F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P2_4        , RULL(0x8000125F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P3_0        , RULL(0x8000025F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P3_0        , RULL(0x8000025F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P3_1        , RULL(0x8000065F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P3_1        , RULL(0x8000065F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P3_2        , RULL(0x80000A5F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P3_2        , RULL(0x80000A5F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P3_3        , RULL(0x80000E5F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P3_3        , RULL(0x80000E5F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P3_4        , RULL(0x8000125F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR2_P3_4        , RULL(0x8000125F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P0_0          , RULL(0x8000035F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P0_0        , RULL(0x8000035F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P0_0        , RULL(0x8000035F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P0_1          , RULL(0x8000075F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P0_1        , RULL(0x8000075F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P0_1        , RULL(0x8000075F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P0_2          , RULL(0x80000B5F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P0_2        , RULL(0x80000B5F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P0_2        , RULL(0x80000B5F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P0_3          , RULL(0x80000F5F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P0_3        , RULL(0x80000F5F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P0_3        , RULL(0x80000F5F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P0_4          , RULL(0x8000135F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P0_4        , RULL(0x8000135F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P0_4        , RULL(0x8000135F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P1_0        , RULL(0x8000035F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P1_0        , RULL(0x8000035F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P1_1        , RULL(0x8000075F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P1_1        , RULL(0x8000075F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P1_2        , RULL(0x80000B5F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P1_2        , RULL(0x80000B5F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P1_3        , RULL(0x80000F5F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P1_3        , RULL(0x80000F5F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P1_4        , RULL(0x8000135F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P1_4        , RULL(0x8000135F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P2_0        , RULL(0x8000035F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P2_0        , RULL(0x8000035F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P2_1        , RULL(0x8000075F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P2_1        , RULL(0x8000075F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P2_2        , RULL(0x80000B5F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P2_2        , RULL(0x80000B5F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P2_3        , RULL(0x80000F5F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P2_3        , RULL(0x80000F5F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P2_4        , RULL(0x8000135F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P2_4        , RULL(0x8000135F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P3_0        , RULL(0x8000035F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P3_0        , RULL(0x8000035F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P3_1        , RULL(0x8000075F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P3_1        , RULL(0x8000075F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P3_2        , RULL(0x80000B5F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P3_2        , RULL(0x80000B5F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P3_3        , RULL(0x80000F5F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P3_3        , RULL(0x80000F5F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P3_4        , RULL(0x8000135F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_DP16_WR_VREF_VALUE1_RANK_PAIR3_P3_4        , RULL(0x8000135F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_BASE_CNTR0_P0                             , RULL(0x8000C0040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_PC_BASE_CNTR0_P0                           , RULL(0x8000C0040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_PC_BASE_CNTR0_P0                           , RULL(0x8000C0040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_PC_BASE_CNTR0_P1                           , RULL(0x8000C0040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_PC_BASE_CNTR0_P1                           , RULL(0x8000C0040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_PC_BASE_CNTR0_P2                           , RULL(0x8000C0040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_PC_BASE_CNTR0_P2                           , RULL(0x8000C0040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_PC_BASE_CNTR0_P3                           , RULL(0x8000C00407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_PC_BASE_CNTR0_P3                           , RULL(0x8000C00408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_PC_BASE_CNTR1_P0                             , RULL(0x8000C0060701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_PC_BASE_CNTR1_P0                           , RULL(0x8000C0060701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_PC_BASE_CNTR1_P0                           , RULL(0x8000C0060801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_PC_BASE_CNTR1_P1                           , RULL(0x8000C0060701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_PC_BASE_CNTR1_P1                           , RULL(0x8000C0060801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_PC_BASE_CNTR1_P2                           , RULL(0x8000C0060701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_PC_BASE_CNTR1_P2                           , RULL(0x8000C0060801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_PC_BASE_CNTR1_P3                           , RULL(0x8000C00607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_PC_BASE_CNTR1_P3                           , RULL(0x8000C00608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_PC_CAL_TIMER_P0                              , RULL(0x8000C0070701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_PC_CAL_TIMER_P0                            , RULL(0x8000C0070701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_PC_CAL_TIMER_P0                            , RULL(0x8000C0070801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_PC_CAL_TIMER_P1                            , RULL(0x8000C0070701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_PC_CAL_TIMER_P1                            , RULL(0x8000C0070801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_PC_CAL_TIMER_P2                            , RULL(0x8000C0070701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_PC_CAL_TIMER_P2                            , RULL(0x8000C0070801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_PC_CAL_TIMER_P3                            , RULL(0x8000C00707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_PC_CAL_TIMER_P3                            , RULL(0x8000C00708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_PC_CAL_TIMER_RELOAD_VALUE_P0                 , RULL(0x8000C0080701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_CAL_TIMER_RELOAD_VALUE_P0               , RULL(0x8000C0080701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_CAL_TIMER_RELOAD_VALUE_P0               , RULL(0x8000C0080801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_CAL_TIMER_RELOAD_VALUE_P1               , RULL(0x8000C0080701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_CAL_TIMER_RELOAD_VALUE_P1               , RULL(0x8000C0080801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_CAL_TIMER_RELOAD_VALUE_P2               , RULL(0x8000C0080701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_CAL_TIMER_RELOAD_VALUE_P2               , RULL(0x8000C0080801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_CAL_TIMER_RELOAD_VALUE_P3               , RULL(0x8000C00807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_CAL_TIMER_RELOAD_VALUE_P3               , RULL(0x8000C00808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_CONFIG0_P0                                , RULL(0x8000C00C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_CONFIG0_P0                              , RULL(0x8000C00C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_CONFIG0_P0                              , RULL(0x8000C00C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_CONFIG0_P1                              , RULL(0x8000C00C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_CONFIG0_P1                              , RULL(0x8000C00C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_CONFIG0_P2                              , RULL(0x8000C00C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_CONFIG0_P2                              , RULL(0x8000C00C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_CONFIG0_P3                              , RULL(0x8000C00C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_CONFIG0_P3                              , RULL(0x8000C00C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_CONFIG1_P0                                , RULL(0x8000C00D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_CONFIG1_P0                              , RULL(0x8000C00D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_CONFIG1_P0                              , RULL(0x8000C00D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_CONFIG1_P1                              , RULL(0x8000C00D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_CONFIG1_P1                              , RULL(0x8000C00D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_CONFIG1_P2                              , RULL(0x8000C00D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_CONFIG1_P2                              , RULL(0x8000C00D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_CONFIG1_P3                              , RULL(0x8000C00D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_CONFIG1_P3                              , RULL(0x8000C00D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_CSID_CFG_P0                               , RULL(0x8000C0330701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_CSID_CFG_P0                             , RULL(0x8000C0330701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_CSID_CFG_P0                             , RULL(0x8000C0330801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_CSID_CFG_P1                             , RULL(0x8000C0330701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_CSID_CFG_P1                             , RULL(0x8000C0330801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_CSID_CFG_P2                             , RULL(0x8000C0330701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_CSID_CFG_P2                             , RULL(0x8000C0330801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_CSID_CFG_P3                             , RULL(0x8000C03307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_CSID_CFG_P3                             , RULL(0x8000C03308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_DLL_ZCAL_CAL_STATUS_P0                    , RULL(0x8000C0000701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_PC_DLL_ZCAL_CAL_STATUS_P0                  , RULL(0x8000C0000701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_PC_DLL_ZCAL_CAL_STATUS_P0                  , RULL(0x8000C0000801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_PC_DLL_ZCAL_CAL_STATUS_P1                  , RULL(0x8000C0000701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_PC_DLL_ZCAL_CAL_STATUS_P1                  , RULL(0x8000C0000801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_PC_DLL_ZCAL_CAL_STATUS_P2                  , RULL(0x8000C0000701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_PC_DLL_ZCAL_CAL_STATUS_P2                  , RULL(0x8000C0000801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_PC_DLL_ZCAL_CAL_STATUS_P3                  , RULL(0x8000C00007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_PC_DLL_ZCAL_CAL_STATUS_P3                  , RULL(0x8000C00008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_PC_ERROR_MASK0_P0                            , RULL(0x8000C0130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_ERROR_MASK0_P0                          , RULL(0x8000C0130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_ERROR_MASK0_P0                          , RULL(0x8000C0130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_ERROR_MASK0_P1                          , RULL(0x8000C0130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_ERROR_MASK0_P1                          , RULL(0x8000C0130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_ERROR_MASK0_P2                          , RULL(0x8000C0130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_ERROR_MASK0_P2                          , RULL(0x8000C0130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_ERROR_MASK0_P3                          , RULL(0x8000C01307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_ERROR_MASK0_P3                          , RULL(0x8000C01308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_ERROR_STATUS0_P0                          , RULL(0x8000C0120701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_PC_ERROR_STATUS0_P0                        , RULL(0x8000C0120701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_PC_ERROR_STATUS0_P0                        , RULL(0x8000C0120801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_PC_ERROR_STATUS0_P1                        , RULL(0x8000C0120701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_PC_ERROR_STATUS0_P1                        , RULL(0x8000C0120801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_PC_ERROR_STATUS0_P2                        , RULL(0x8000C0120701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_PC_ERROR_STATUS0_P2                        , RULL(0x8000C0120801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_PC_ERROR_STATUS0_P3                        , RULL(0x8000C01207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_PC_ERROR_STATUS0_P3                        , RULL(0x8000C01208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_PC_INIT_CAL_CONFIG0_P0                       , RULL(0x8000C0160701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_INIT_CAL_CONFIG0_P0                     , RULL(0x8000C0160701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_INIT_CAL_CONFIG0_P0                     , RULL(0x8000C0160801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_INIT_CAL_CONFIG0_P1                     , RULL(0x8000C0160701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_INIT_CAL_CONFIG0_P1                     , RULL(0x8000C0160801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_INIT_CAL_CONFIG0_P2                     , RULL(0x8000C0160701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_INIT_CAL_CONFIG0_P2                     , RULL(0x8000C0160801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_INIT_CAL_CONFIG0_P3                     , RULL(0x8000C01607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_INIT_CAL_CONFIG0_P3                     , RULL(0x8000C01608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_INIT_CAL_CONFIG1_P0                       , RULL(0x8000C0170701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_INIT_CAL_CONFIG1_P0                     , RULL(0x8000C0170701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_INIT_CAL_CONFIG1_P0                     , RULL(0x8000C0170801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_INIT_CAL_CONFIG1_P1                     , RULL(0x8000C0170701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_INIT_CAL_CONFIG1_P1                     , RULL(0x8000C0170801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_INIT_CAL_CONFIG1_P2                     , RULL(0x8000C0170701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_INIT_CAL_CONFIG1_P2                     , RULL(0x8000C0170801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_INIT_CAL_CONFIG1_P3                     , RULL(0x8000C01707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_INIT_CAL_CONFIG1_P3                     , RULL(0x8000C01708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_INIT_CAL_ERROR_P0                         , RULL(0x8000C0180701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_PC_INIT_CAL_ERROR_P0                       , RULL(0x8000C0180701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_PC_INIT_CAL_ERROR_P0                       , RULL(0x8000C0180801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_PC_INIT_CAL_ERROR_P1                       , RULL(0x8000C0180701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_PC_INIT_CAL_ERROR_P1                       , RULL(0x8000C0180801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_PC_INIT_CAL_ERROR_P2                       , RULL(0x8000C0180701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_PC_INIT_CAL_ERROR_P2                       , RULL(0x8000C0180801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_PC_INIT_CAL_ERROR_P3                       , RULL(0x8000C01807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_PC_INIT_CAL_ERROR_P3                       , RULL(0x8000C01808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_PC_INIT_CAL_MASK_P0                          , RULL(0x8000C01A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_INIT_CAL_MASK_P0                        , RULL(0x8000C01A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_INIT_CAL_MASK_P0                        , RULL(0x8000C01A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_INIT_CAL_MASK_P1                        , RULL(0x8000C01A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_INIT_CAL_MASK_P1                        , RULL(0x8000C01A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_INIT_CAL_MASK_P2                        , RULL(0x8000C01A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_INIT_CAL_MASK_P2                        , RULL(0x8000C01A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_INIT_CAL_MASK_P3                        , RULL(0x8000C01A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_INIT_CAL_MASK_P3                        , RULL(0x8000C01A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_INIT_CAL_STATUS_P0                        , RULL(0x8000C0190701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_PC_INIT_CAL_STATUS_P0                      , RULL(0x8000C0190701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_PC_INIT_CAL_STATUS_P0                      , RULL(0x8000C0190801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_PC_INIT_CAL_STATUS_P1                      , RULL(0x8000C0190701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_PC_INIT_CAL_STATUS_P1                      , RULL(0x8000C0190801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_PC_INIT_CAL_STATUS_P2                      , RULL(0x8000C0190701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_PC_INIT_CAL_STATUS_P2                      , RULL(0x8000C0190801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_PC_INIT_CAL_STATUS_P3                      , RULL(0x8000C01907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_PC_INIT_CAL_STATUS_P3                      , RULL(0x8000C01908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_PC_IO_PVT_FET_CONTROL_P0                     , RULL(0x8000C0140701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_IO_PVT_FET_CONTROL_P0                   , RULL(0x8000C0140701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_IO_PVT_FET_CONTROL_P0                   , RULL(0x8000C0140801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_IO_PVT_FET_CONTROL_P1                   , RULL(0x8000C0140701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_IO_PVT_FET_CONTROL_P1                   , RULL(0x8000C0140801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_IO_PVT_FET_CONTROL_P2                   , RULL(0x8000C0140701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_IO_PVT_FET_CONTROL_P2                   , RULL(0x8000C0140801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_IO_PVT_FET_CONTROL_P3                   , RULL(0x8000C01407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_IO_PVT_FET_CONTROL_P3                   , RULL(0x8000C01408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_IO_PVT_FET_STATUS_P0                      , RULL(0x8000C01B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_PC_IO_PVT_FET_STATUS_P0                    , RULL(0x8000C01B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_PC_IO_PVT_FET_STATUS_P0                    , RULL(0x8000C01B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_PC_IO_PVT_FET_STATUS_P1                    , RULL(0x8000C01B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_PC_IO_PVT_FET_STATUS_P1                    , RULL(0x8000C01B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_PC_IO_PVT_FET_STATUS_P2                    , RULL(0x8000C01B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_PC_IO_PVT_FET_STATUS_P2                    , RULL(0x8000C01B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_PC_IO_PVT_FET_STATUS_P3                    , RULL(0x8000C01B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_PC_IO_PVT_FET_STATUS_P3                    , RULL(0x8000C01B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_PC_MR0_PRI_RP0_P0                            , RULL(0x8000C01C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR0_PRI_RP0_P0                          , RULL(0x8000C01C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR0_PRI_RP0_P0                          , RULL(0x8000C01C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR0_PRI_RP0_P1                          , RULL(0x8000C01C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR0_PRI_RP0_P1                          , RULL(0x8000C01C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR0_PRI_RP0_P2                          , RULL(0x8000C01C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR0_PRI_RP0_P2                          , RULL(0x8000C01C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR0_PRI_RP0_P3                          , RULL(0x8000C01C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR0_PRI_RP0_P3                          , RULL(0x8000C01C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR0_PRI_RP1_P0                            , RULL(0x8000C11C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR0_PRI_RP1_P0                          , RULL(0x8000C11C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR0_PRI_RP1_P0                          , RULL(0x8000C11C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR0_PRI_RP1_P1                          , RULL(0x8000C11C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR0_PRI_RP1_P1                          , RULL(0x8000C11C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR0_PRI_RP1_P2                          , RULL(0x8000C11C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR0_PRI_RP1_P2                          , RULL(0x8000C11C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR0_PRI_RP1_P3                          , RULL(0x8000C11C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR0_PRI_RP1_P3                          , RULL(0x8000C11C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR0_PRI_RP2_P0                            , RULL(0x8000C21C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR0_PRI_RP2_P0                          , RULL(0x8000C21C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR0_PRI_RP2_P0                          , RULL(0x8000C21C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR0_PRI_RP2_P1                          , RULL(0x8000C21C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR0_PRI_RP2_P1                          , RULL(0x8000C21C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR0_PRI_RP2_P2                          , RULL(0x8000C21C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR0_PRI_RP2_P2                          , RULL(0x8000C21C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR0_PRI_RP2_P3                          , RULL(0x8000C21C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR0_PRI_RP2_P3                          , RULL(0x8000C21C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR0_PRI_RP3_P0                            , RULL(0x8000C31C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR0_PRI_RP3_P0                          , RULL(0x8000C31C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR0_PRI_RP3_P0                          , RULL(0x8000C31C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR0_PRI_RP3_P1                          , RULL(0x8000C31C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR0_PRI_RP3_P1                          , RULL(0x8000C31C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR0_PRI_RP3_P2                          , RULL(0x8000C31C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR0_PRI_RP3_P2                          , RULL(0x8000C31C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR0_PRI_RP3_P3                          , RULL(0x8000C31C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR0_PRI_RP3_P3                          , RULL(0x8000C31C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR0_SEC_RP0_P0                            , RULL(0x8000C0200701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR0_SEC_RP0_P0                          , RULL(0x8000C0200701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR0_SEC_RP0_P0                          , RULL(0x8000C0200801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR0_SEC_RP0_P1                          , RULL(0x8000C0200701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR0_SEC_RP0_P1                          , RULL(0x8000C0200801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR0_SEC_RP0_P2                          , RULL(0x8000C0200701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR0_SEC_RP0_P2                          , RULL(0x8000C0200801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR0_SEC_RP0_P3                          , RULL(0x8000C02007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR0_SEC_RP0_P3                          , RULL(0x8000C02008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR0_SEC_RP1_P0                            , RULL(0x8000C1200701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR0_SEC_RP1_P0                          , RULL(0x8000C1200701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR0_SEC_RP1_P0                          , RULL(0x8000C1200801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR0_SEC_RP1_P1                          , RULL(0x8000C1200701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR0_SEC_RP1_P1                          , RULL(0x8000C1200801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR0_SEC_RP1_P2                          , RULL(0x8000C1200701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR0_SEC_RP1_P2                          , RULL(0x8000C1200801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR0_SEC_RP1_P3                          , RULL(0x8000C12007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR0_SEC_RP1_P3                          , RULL(0x8000C12008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR0_SEC_RP2_P0                            , RULL(0x8000C2200701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR0_SEC_RP2_P0                          , RULL(0x8000C2200701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR0_SEC_RP2_P0                          , RULL(0x8000C2200801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR0_SEC_RP2_P1                          , RULL(0x8000C2200701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR0_SEC_RP2_P1                          , RULL(0x8000C2200801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR0_SEC_RP2_P2                          , RULL(0x8000C2200701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR0_SEC_RP2_P2                          , RULL(0x8000C2200801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR0_SEC_RP2_P3                          , RULL(0x8000C22007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR0_SEC_RP2_P3                          , RULL(0x8000C22008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR0_SEC_RP3_P0                            , RULL(0x8000C3200701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR0_SEC_RP3_P0                          , RULL(0x8000C3200701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR0_SEC_RP3_P0                          , RULL(0x8000C3200801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR0_SEC_RP3_P1                          , RULL(0x8000C3200701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR0_SEC_RP3_P1                          , RULL(0x8000C3200801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR0_SEC_RP3_P2                          , RULL(0x8000C3200701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR0_SEC_RP3_P2                          , RULL(0x8000C3200801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR0_SEC_RP3_P3                          , RULL(0x8000C32007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR0_SEC_RP3_P3                          , RULL(0x8000C32008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR1_PRI_RP0_P0                            , RULL(0x8000C01D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR1_PRI_RP0_P0                          , RULL(0x8000C01D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR1_PRI_RP0_P0                          , RULL(0x8000C01D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR1_PRI_RP0_P1                          , RULL(0x8000C01D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR1_PRI_RP0_P1                          , RULL(0x8000C01D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR1_PRI_RP0_P2                          , RULL(0x8000C01D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR1_PRI_RP0_P2                          , RULL(0x8000C01D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR1_PRI_RP0_P3                          , RULL(0x8000C01D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR1_PRI_RP0_P3                          , RULL(0x8000C01D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR1_PRI_RP1_P0                            , RULL(0x8000C11D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR1_PRI_RP1_P0                          , RULL(0x8000C11D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR1_PRI_RP1_P0                          , RULL(0x8000C11D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR1_PRI_RP1_P1                          , RULL(0x8000C11D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR1_PRI_RP1_P1                          , RULL(0x8000C11D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR1_PRI_RP1_P2                          , RULL(0x8000C11D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR1_PRI_RP1_P2                          , RULL(0x8000C11D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR1_PRI_RP1_P3                          , RULL(0x8000C11D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR1_PRI_RP1_P3                          , RULL(0x8000C11D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR1_PRI_RP2_P0                            , RULL(0x8000C21D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR1_PRI_RP2_P0                          , RULL(0x8000C21D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR1_PRI_RP2_P0                          , RULL(0x8000C21D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR1_PRI_RP2_P1                          , RULL(0x8000C21D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR1_PRI_RP2_P1                          , RULL(0x8000C21D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR1_PRI_RP2_P2                          , RULL(0x8000C21D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR1_PRI_RP2_P2                          , RULL(0x8000C21D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR1_PRI_RP2_P3                          , RULL(0x8000C21D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR1_PRI_RP2_P3                          , RULL(0x8000C21D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR1_PRI_RP3_P0                            , RULL(0x8000C31D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR1_PRI_RP3_P0                          , RULL(0x8000C31D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR1_PRI_RP3_P0                          , RULL(0x8000C31D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR1_PRI_RP3_P1                          , RULL(0x8000C31D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR1_PRI_RP3_P1                          , RULL(0x8000C31D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR1_PRI_RP3_P2                          , RULL(0x8000C31D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR1_PRI_RP3_P2                          , RULL(0x8000C31D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR1_PRI_RP3_P3                          , RULL(0x8000C31D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR1_PRI_RP3_P3                          , RULL(0x8000C31D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR1_SEC_RP0_P0                            , RULL(0x8000C0210701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR1_SEC_RP0_P0                          , RULL(0x8000C0210701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR1_SEC_RP0_P0                          , RULL(0x8000C0210801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR1_SEC_RP0_P1                          , RULL(0x8000C0210701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR1_SEC_RP0_P1                          , RULL(0x8000C0210801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR1_SEC_RP0_P2                          , RULL(0x8000C0210701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR1_SEC_RP0_P2                          , RULL(0x8000C0210801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR1_SEC_RP0_P3                          , RULL(0x8000C02107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR1_SEC_RP0_P3                          , RULL(0x8000C02108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR1_SEC_RP1_P0                            , RULL(0x8000C1210701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR1_SEC_RP1_P0                          , RULL(0x8000C1210701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR1_SEC_RP1_P0                          , RULL(0x8000C1210801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR1_SEC_RP1_P1                          , RULL(0x8000C1210701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR1_SEC_RP1_P1                          , RULL(0x8000C1210801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR1_SEC_RP1_P2                          , RULL(0x8000C1210701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR1_SEC_RP1_P2                          , RULL(0x8000C1210801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR1_SEC_RP1_P3                          , RULL(0x8000C12107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR1_SEC_RP1_P3                          , RULL(0x8000C12108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR1_SEC_RP2_P0                            , RULL(0x8000C2210701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR1_SEC_RP2_P0                          , RULL(0x8000C2210701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR1_SEC_RP2_P0                          , RULL(0x8000C2210801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR1_SEC_RP2_P1                          , RULL(0x8000C2210701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR1_SEC_RP2_P1                          , RULL(0x8000C2210801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR1_SEC_RP2_P2                          , RULL(0x8000C2210701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR1_SEC_RP2_P2                          , RULL(0x8000C2210801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR1_SEC_RP2_P3                          , RULL(0x8000C22107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR1_SEC_RP2_P3                          , RULL(0x8000C22108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR1_SEC_RP3_P0                            , RULL(0x8000C3210701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR1_SEC_RP3_P0                          , RULL(0x8000C3210701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR1_SEC_RP3_P0                          , RULL(0x8000C3210801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR1_SEC_RP3_P1                          , RULL(0x8000C3210701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR1_SEC_RP3_P1                          , RULL(0x8000C3210801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR1_SEC_RP3_P2                          , RULL(0x8000C3210701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR1_SEC_RP3_P2                          , RULL(0x8000C3210801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR1_SEC_RP3_P3                          , RULL(0x8000C32107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR1_SEC_RP3_P3                          , RULL(0x8000C32108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR2_PRI_RP0_P0                            , RULL(0x8000C01E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR2_PRI_RP0_P0                          , RULL(0x8000C01E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR2_PRI_RP0_P0                          , RULL(0x8000C01E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR2_PRI_RP0_P1                          , RULL(0x8000C01E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR2_PRI_RP0_P1                          , RULL(0x8000C01E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR2_PRI_RP0_P2                          , RULL(0x8000C01E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR2_PRI_RP0_P2                          , RULL(0x8000C01E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR2_PRI_RP0_P3                          , RULL(0x8000C01E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR2_PRI_RP0_P3                          , RULL(0x8000C01E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR2_PRI_RP1_P0                            , RULL(0x8000C11E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR2_PRI_RP1_P0                          , RULL(0x8000C11E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR2_PRI_RP1_P0                          , RULL(0x8000C11E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR2_PRI_RP1_P1                          , RULL(0x8000C11E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR2_PRI_RP1_P1                          , RULL(0x8000C11E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR2_PRI_RP1_P2                          , RULL(0x8000C11E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR2_PRI_RP1_P2                          , RULL(0x8000C11E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR2_PRI_RP1_P3                          , RULL(0x8000C11E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR2_PRI_RP1_P3                          , RULL(0x8000C11E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR2_PRI_RP2_P0                            , RULL(0x8000C21E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR2_PRI_RP2_P0                          , RULL(0x8000C21E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR2_PRI_RP2_P0                          , RULL(0x8000C21E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR2_PRI_RP2_P1                          , RULL(0x8000C21E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR2_PRI_RP2_P1                          , RULL(0x8000C21E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR2_PRI_RP2_P2                          , RULL(0x8000C21E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR2_PRI_RP2_P2                          , RULL(0x8000C21E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR2_PRI_RP2_P3                          , RULL(0x8000C21E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR2_PRI_RP2_P3                          , RULL(0x8000C21E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR2_PRI_RP3_P0                            , RULL(0x8000C31E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR2_PRI_RP3_P0                          , RULL(0x8000C31E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR2_PRI_RP3_P0                          , RULL(0x8000C31E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR2_PRI_RP3_P1                          , RULL(0x8000C31E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR2_PRI_RP3_P1                          , RULL(0x8000C31E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR2_PRI_RP3_P2                          , RULL(0x8000C31E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR2_PRI_RP3_P2                          , RULL(0x8000C31E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR2_PRI_RP3_P3                          , RULL(0x8000C31E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR2_PRI_RP3_P3                          , RULL(0x8000C31E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR2_SEC_RP0_P0                            , RULL(0x8000C0220701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR2_SEC_RP0_P0                          , RULL(0x8000C0220701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR2_SEC_RP0_P0                          , RULL(0x8000C0220801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR2_SEC_RP0_P1                          , RULL(0x8000C0220701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR2_SEC_RP0_P1                          , RULL(0x8000C0220801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR2_SEC_RP0_P2                          , RULL(0x8000C0220701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR2_SEC_RP0_P2                          , RULL(0x8000C0220801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR2_SEC_RP0_P3                          , RULL(0x8000C02207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR2_SEC_RP0_P3                          , RULL(0x8000C02208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR2_SEC_RP1_P0                            , RULL(0x8000C1220701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR2_SEC_RP1_P0                          , RULL(0x8000C1220701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR2_SEC_RP1_P0                          , RULL(0x8000C1220801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR2_SEC_RP1_P1                          , RULL(0x8000C1220701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR2_SEC_RP1_P1                          , RULL(0x8000C1220801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR2_SEC_RP1_P2                          , RULL(0x8000C1220701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR2_SEC_RP1_P2                          , RULL(0x8000C1220801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR2_SEC_RP1_P3                          , RULL(0x8000C12207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR2_SEC_RP1_P3                          , RULL(0x8000C12208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR2_SEC_RP2_P0                            , RULL(0x8000C2220701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR2_SEC_RP2_P0                          , RULL(0x8000C2220701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR2_SEC_RP2_P0                          , RULL(0x8000C2220801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR2_SEC_RP2_P1                          , RULL(0x8000C2220701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR2_SEC_RP2_P1                          , RULL(0x8000C2220801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR2_SEC_RP2_P2                          , RULL(0x8000C2220701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR2_SEC_RP2_P2                          , RULL(0x8000C2220801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR2_SEC_RP2_P3                          , RULL(0x8000C22207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR2_SEC_RP2_P3                          , RULL(0x8000C22208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR2_SEC_RP3_P0                            , RULL(0x8000C3220701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR2_SEC_RP3_P0                          , RULL(0x8000C3220701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR2_SEC_RP3_P0                          , RULL(0x8000C3220801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR2_SEC_RP3_P1                          , RULL(0x8000C3220701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR2_SEC_RP3_P1                          , RULL(0x8000C3220801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR2_SEC_RP3_P2                          , RULL(0x8000C3220701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR2_SEC_RP3_P2                          , RULL(0x8000C3220801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR2_SEC_RP3_P3                          , RULL(0x8000C32207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR2_SEC_RP3_P3                          , RULL(0x8000C32208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR3_PRI_RP0_P0                            , RULL(0x8000C01F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR3_PRI_RP0_P0                          , RULL(0x8000C01F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR3_PRI_RP0_P0                          , RULL(0x8000C01F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR3_PRI_RP0_P1                          , RULL(0x8000C01F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR3_PRI_RP0_P1                          , RULL(0x8000C01F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR3_PRI_RP0_P2                          , RULL(0x8000C01F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR3_PRI_RP0_P2                          , RULL(0x8000C01F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR3_PRI_RP0_P3                          , RULL(0x8000C01F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR3_PRI_RP0_P3                          , RULL(0x8000C01F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR3_PRI_RP1_P0                            , RULL(0x8000C11F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR3_PRI_RP1_P0                          , RULL(0x8000C11F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR3_PRI_RP1_P0                          , RULL(0x8000C11F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR3_PRI_RP1_P1                          , RULL(0x8000C11F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR3_PRI_RP1_P1                          , RULL(0x8000C11F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR3_PRI_RP1_P2                          , RULL(0x8000C11F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR3_PRI_RP1_P2                          , RULL(0x8000C11F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR3_PRI_RP1_P3                          , RULL(0x8000C11F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR3_PRI_RP1_P3                          , RULL(0x8000C11F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR3_PRI_RP2_P0                            , RULL(0x8000C21F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR3_PRI_RP2_P0                          , RULL(0x8000C21F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR3_PRI_RP2_P0                          , RULL(0x8000C21F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR3_PRI_RP2_P1                          , RULL(0x8000C21F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR3_PRI_RP2_P1                          , RULL(0x8000C21F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR3_PRI_RP2_P2                          , RULL(0x8000C21F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR3_PRI_RP2_P2                          , RULL(0x8000C21F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR3_PRI_RP2_P3                          , RULL(0x8000C21F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR3_PRI_RP2_P3                          , RULL(0x8000C21F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR3_PRI_RP3_P0                            , RULL(0x8000C31F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR3_PRI_RP3_P0                          , RULL(0x8000C31F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR3_PRI_RP3_P0                          , RULL(0x8000C31F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR3_PRI_RP3_P1                          , RULL(0x8000C31F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR3_PRI_RP3_P1                          , RULL(0x8000C31F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR3_PRI_RP3_P2                          , RULL(0x8000C31F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR3_PRI_RP3_P2                          , RULL(0x8000C31F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR3_PRI_RP3_P3                          , RULL(0x8000C31F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR3_PRI_RP3_P3                          , RULL(0x8000C31F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR3_SEC_RP0_P0                            , RULL(0x8000C0230701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR3_SEC_RP0_P0                          , RULL(0x8000C0230701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR3_SEC_RP0_P0                          , RULL(0x8000C0230801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR3_SEC_RP0_P1                          , RULL(0x8000C0230701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR3_SEC_RP0_P1                          , RULL(0x8000C0230801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR3_SEC_RP0_P2                          , RULL(0x8000C0230701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR3_SEC_RP0_P2                          , RULL(0x8000C0230801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR3_SEC_RP0_P3                          , RULL(0x8000C02307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR3_SEC_RP0_P3                          , RULL(0x8000C02308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR3_SEC_RP1_P0                            , RULL(0x8000C1230701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR3_SEC_RP1_P0                          , RULL(0x8000C1230701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR3_SEC_RP1_P0                          , RULL(0x8000C1230801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR3_SEC_RP1_P1                          , RULL(0x8000C1230701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR3_SEC_RP1_P1                          , RULL(0x8000C1230801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR3_SEC_RP1_P2                          , RULL(0x8000C1230701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR3_SEC_RP1_P2                          , RULL(0x8000C1230801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR3_SEC_RP1_P3                          , RULL(0x8000C12307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR3_SEC_RP1_P3                          , RULL(0x8000C12308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR3_SEC_RP2_P0                            , RULL(0x8000C2230701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR3_SEC_RP2_P0                          , RULL(0x8000C2230701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR3_SEC_RP2_P0                          , RULL(0x8000C2230801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR3_SEC_RP2_P1                          , RULL(0x8000C2230701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR3_SEC_RP2_P1                          , RULL(0x8000C2230801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR3_SEC_RP2_P2                          , RULL(0x8000C2230701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR3_SEC_RP2_P2                          , RULL(0x8000C2230801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR3_SEC_RP2_P3                          , RULL(0x8000C22307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR3_SEC_RP2_P3                          , RULL(0x8000C22308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_MR3_SEC_RP3_P0                            , RULL(0x8000C3230701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_MR3_SEC_RP3_P0                          , RULL(0x8000C3230701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_MR3_SEC_RP3_P0                          , RULL(0x8000C3230801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_MR3_SEC_RP3_P1                          , RULL(0x8000C3230701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_MR3_SEC_RP3_P1                          , RULL(0x8000C3230801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_MR3_SEC_RP3_P2                          , RULL(0x8000C3230701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_MR3_SEC_RP3_P2                          , RULL(0x8000C3230801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_MR3_SEC_RP3_P3                          , RULL(0x8000C32307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_MR3_SEC_RP3_P3                          , RULL(0x8000C32308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_PER_CAL_CONFIG_P0                         , RULL(0x8000C00B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_PER_CAL_CONFIG_P0                       , RULL(0x8000C00B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_PER_CAL_CONFIG_P0                       , RULL(0x8000C00B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_PER_CAL_CONFIG_P1                       , RULL(0x8000C00B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_PER_CAL_CONFIG_P1                       , RULL(0x8000C00B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_PER_CAL_CONFIG_P2                       , RULL(0x8000C00B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_PER_CAL_CONFIG_P2                       , RULL(0x8000C00B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_PER_CAL_CONFIG_P3                       , RULL(0x8000C00B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_PER_CAL_CONFIG_P3                       , RULL(0x8000C00B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_PER_ZCAL_CONFIG_P0                        , RULL(0x8000C00F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_PER_ZCAL_CONFIG_P0                      , RULL(0x8000C00F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_PER_ZCAL_CONFIG_P0                      , RULL(0x8000C00F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_PER_ZCAL_CONFIG_P1                      , RULL(0x8000C00F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_PER_ZCAL_CONFIG_P1                      , RULL(0x8000C00F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_PER_ZCAL_CONFIG_P2                      , RULL(0x8000C00F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_PER_ZCAL_CONFIG_P2                      , RULL(0x8000C00F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_PER_ZCAL_CONFIG_P3                      , RULL(0x8000C00F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_PER_ZCAL_CONFIG_P3                      , RULL(0x8000C00F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_POWERDOWN_1_P0                            , RULL(0x8000C0100701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_POWERDOWN_1_P0                          , RULL(0x8000C0100701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_POWERDOWN_1_P0                          , RULL(0x8000C0100801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_POWERDOWN_1_P1                          , RULL(0x8000C0100701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_POWERDOWN_1_P1                          , RULL(0x8000C0100801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_POWERDOWN_1_P2                          , RULL(0x8000C0100701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_POWERDOWN_1_P2                          , RULL(0x8000C0100801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_POWERDOWN_1_P3                          , RULL(0x8000C01007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_POWERDOWN_1_P3                          , RULL(0x8000C01008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_RANK_GROUP_EXT_P0                         , RULL(0x8000C0350701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_RANK_GROUP_EXT_P0                       , RULL(0x8000C0350701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_RANK_GROUP_EXT_P0                       , RULL(0x8000C0350801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_RANK_GROUP_EXT_P1                       , RULL(0x8000C0350701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_RANK_GROUP_EXT_P1                       , RULL(0x8000C0350801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_RANK_GROUP_EXT_P2                       , RULL(0x8000C0350701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_RANK_GROUP_EXT_P2                       , RULL(0x8000C0350801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_RANK_GROUP_EXT_P3                       , RULL(0x8000C03507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_RANK_GROUP_EXT_P3                       , RULL(0x8000C03508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_RANK_GROUP_P0                             , RULL(0x8000C0110701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_RANK_GROUP_P0                           , RULL(0x8000C0110701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_RANK_GROUP_P0                           , RULL(0x8000C0110801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_RANK_GROUP_P1                           , RULL(0x8000C0110701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_RANK_GROUP_P1                           , RULL(0x8000C0110801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_RANK_GROUP_P2                           , RULL(0x8000C0110701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_RANK_GROUP_P2                           , RULL(0x8000C0110801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_RANK_GROUP_P3                           , RULL(0x8000C01107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_RANK_GROUP_P3                           , RULL(0x8000C01108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_RANK_PAIR0_P0                             , RULL(0x8000C0020701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_RANK_PAIR0_P0                           , RULL(0x8000C0020701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_RANK_PAIR0_P0                           , RULL(0x8000C0020801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_RANK_PAIR0_P1                           , RULL(0x8000C0020701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_RANK_PAIR0_P1                           , RULL(0x8000C0020801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_RANK_PAIR0_P2                           , RULL(0x8000C0020701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_RANK_PAIR0_P2                           , RULL(0x8000C0020801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_RANK_PAIR0_P3                           , RULL(0x8000C00207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_RANK_PAIR0_P3                           , RULL(0x8000C00208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_RANK_PAIR1_P0                             , RULL(0x8000C0030701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_RANK_PAIR1_P0                           , RULL(0x8000C0030701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_RANK_PAIR1_P0                           , RULL(0x8000C0030801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_RANK_PAIR1_P1                           , RULL(0x8000C0030701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_RANK_PAIR1_P1                           , RULL(0x8000C0030801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_RANK_PAIR1_P2                           , RULL(0x8000C0030701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_RANK_PAIR1_P2                           , RULL(0x8000C0030801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_RANK_PAIR1_P3                           , RULL(0x8000C00307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_RANK_PAIR1_P3                           , RULL(0x8000C00308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_RANK_PAIR2_P0                             , RULL(0x8000C0300701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_RANK_PAIR2_P0                           , RULL(0x8000C0300701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_RANK_PAIR2_P0                           , RULL(0x8000C0300801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_RANK_PAIR2_P1                           , RULL(0x8000C0300701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_RANK_PAIR2_P1                           , RULL(0x8000C0300801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_RANK_PAIR2_P2                           , RULL(0x8000C0300701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_RANK_PAIR2_P2                           , RULL(0x8000C0300801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_RANK_PAIR2_P3                           , RULL(0x8000C03007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_RANK_PAIR2_P3                           , RULL(0x8000C03008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_RANK_PAIR3_P0                             , RULL(0x8000C0310701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_RANK_PAIR3_P0                           , RULL(0x8000C0310701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_RANK_PAIR3_P0                           , RULL(0x8000C0310801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_RANK_PAIR3_P1                           , RULL(0x8000C0310701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_RANK_PAIR3_P1                           , RULL(0x8000C0310801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_RANK_PAIR3_P2                           , RULL(0x8000C0310701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_RANK_PAIR3_P2                           , RULL(0x8000C0310801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_RANK_PAIR3_P3                           , RULL(0x8000C03107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_RANK_PAIR3_P3                           , RULL(0x8000C03108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_RELOAD_VALUE0_P0                          , RULL(0x8000C0050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_RELOAD_VALUE0_P0                        , RULL(0x8000C0050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_RELOAD_VALUE0_P0                        , RULL(0x8000C0050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_RELOAD_VALUE0_P1                        , RULL(0x8000C0050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_RELOAD_VALUE0_P1                        , RULL(0x8000C0050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_RELOAD_VALUE0_P2                        , RULL(0x8000C0050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_RELOAD_VALUE0_P2                        , RULL(0x8000C0050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_RELOAD_VALUE0_P3                        , RULL(0x8000C00507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_RELOAD_VALUE0_P3                        , RULL(0x8000C00508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_RESETS_P0                                 , RULL(0x8000C00E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_RESETS_P0                               , RULL(0x8000C00E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_RESETS_P0                               , RULL(0x8000C00E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_RESETS_P1                               , RULL(0x8000C00E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_RESETS_P1                               , RULL(0x8000C00E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_RESETS_P2                               , RULL(0x8000C00E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_RESETS_P2                               , RULL(0x8000C00E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_RESETS_P3                               , RULL(0x8000C00E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_RESETS_P3                               , RULL(0x8000C00E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_VREF_DRV_CONTROL_P0                       , RULL(0x8000C0150701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_VREF_DRV_CONTROL_P0                     , RULL(0x8000C0150701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_VREF_DRV_CONTROL_P0                     , RULL(0x8000C0150801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_VREF_DRV_CONTROL_P1                     , RULL(0x8000C0150701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_VREF_DRV_CONTROL_P1                     , RULL(0x8000C0150801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_VREF_DRV_CONTROL_P2                     , RULL(0x8000C0150701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_VREF_DRV_CONTROL_P2                     , RULL(0x8000C0150801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_VREF_DRV_CONTROL_P3                     , RULL(0x8000C01507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_VREF_DRV_CONTROL_P3                     , RULL(0x8000C01508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_PC_ZCAL_TIMER_P0                             , RULL(0x8000C0090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_PC_ZCAL_TIMER_P0                           , RULL(0x8000C0090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_PC_ZCAL_TIMER_P0                           , RULL(0x8000C0090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_PC_ZCAL_TIMER_P1                           , RULL(0x8000C0090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_PC_ZCAL_TIMER_P1                           , RULL(0x8000C0090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_PC_ZCAL_TIMER_P2                           , RULL(0x8000C0090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_PC_ZCAL_TIMER_P2                           , RULL(0x8000C0090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_PC_ZCAL_TIMER_P3                           , RULL(0x8000C00907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_PC_ZCAL_TIMER_P3                           , RULL(0x8000C00908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_PC_ZCAL_TIMER_RELOAD_VALUE_P0                , RULL(0x8000C00A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_PC_ZCAL_TIMER_RELOAD_VALUE_P0              , RULL(0x8000C00A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_PC_ZCAL_TIMER_RELOAD_VALUE_P0              , RULL(0x8000C00A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_PC_ZCAL_TIMER_RELOAD_VALUE_P1              , RULL(0x8000C00A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_PC_ZCAL_TIMER_RELOAD_VALUE_P1              , RULL(0x8000C00A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_PC_ZCAL_TIMER_RELOAD_VALUE_P2              , RULL(0x8000C00A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_PC_ZCAL_TIMER_RELOAD_VALUE_P2              , RULL(0x8000C00A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_PC_ZCAL_TIMER_RELOAD_VALUE_P3              , RULL(0x8000C00A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_PC_ZCAL_TIMER_RELOAD_VALUE_P3              , RULL(0x8000C00A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_RC_CONFIG0_P0                                , RULL(0x8000C8000701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_RC_CONFIG0_P0                              , RULL(0x8000C8000701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_RC_CONFIG0_P0                              , RULL(0x8000C8000801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_RC_CONFIG0_P1                              , RULL(0x8000C8000701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_RC_CONFIG0_P1                              , RULL(0x8000C8000801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_RC_CONFIG0_P2                              , RULL(0x8000C8000701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_RC_CONFIG0_P2                              , RULL(0x8000C8000801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_RC_CONFIG0_P3                              , RULL(0x8000C80007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_RC_CONFIG0_P3                              , RULL(0x8000C80008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_RC_CONFIG1_P0                                , RULL(0x8000C8010701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_RC_CONFIG1_P0                              , RULL(0x8000C8010701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_RC_CONFIG1_P0                              , RULL(0x8000C8010801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_RC_CONFIG1_P1                              , RULL(0x8000C8010701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_RC_CONFIG1_P1                              , RULL(0x8000C8010801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_RC_CONFIG1_P2                              , RULL(0x8000C8010701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_RC_CONFIG1_P2                              , RULL(0x8000C8010801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_RC_CONFIG1_P3                              , RULL(0x8000C80107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_RC_CONFIG1_P3                              , RULL(0x8000C80108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_RC_CONFIG2_P0                                , RULL(0x8000C8020701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_RC_CONFIG2_P0                              , RULL(0x8000C8020701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_RC_CONFIG2_P0                              , RULL(0x8000C8020801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_RC_CONFIG2_P1                              , RULL(0x8000C8020701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_RC_CONFIG2_P1                              , RULL(0x8000C8020801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_RC_CONFIG2_P2                              , RULL(0x8000C8020701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_RC_CONFIG2_P2                              , RULL(0x8000C8020801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_RC_CONFIG2_P3                              , RULL(0x8000C80207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_RC_CONFIG2_P3                              , RULL(0x8000C80208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_RC_CONFIG3_P0                                , RULL(0x8000C8070701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_RC_CONFIG3_P0                              , RULL(0x8000C8070701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_RC_CONFIG3_P0                              , RULL(0x8000C8070801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_RC_CONFIG3_P1                              , RULL(0x8000C8070701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_RC_CONFIG3_P1                              , RULL(0x8000C8070801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_RC_CONFIG3_P2                              , RULL(0x8000C8070701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_RC_CONFIG3_P2                              , RULL(0x8000C8070801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_RC_CONFIG3_P3                              , RULL(0x8000C80707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_RC_CONFIG3_P3                              , RULL(0x8000C80708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_RC_ERROR_MASK0_P0                            , RULL(0x8000C8060701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_RC_ERROR_MASK0_P0                          , RULL(0x8000C8060701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_RC_ERROR_MASK0_P0                          , RULL(0x8000C8060801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_RC_ERROR_MASK0_P1                          , RULL(0x8000C8060701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_RC_ERROR_MASK0_P1                          , RULL(0x8000C8060801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_RC_ERROR_MASK0_P2                          , RULL(0x8000C8060701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_RC_ERROR_MASK0_P2                          , RULL(0x8000C8060801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_RC_ERROR_MASK0_P3                          , RULL(0x8000C80607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_RC_ERROR_MASK0_P3                          , RULL(0x8000C80608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_RC_ERROR_STATUS0_P0                          , RULL(0x8000C8050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_RC_ERROR_STATUS0_P0                        , RULL(0x8000C8050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_RC_ERROR_STATUS0_P0                        , RULL(0x8000C8050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_RC_ERROR_STATUS0_P1                        , RULL(0x8000C8050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_RC_ERROR_STATUS0_P1                        , RULL(0x8000C8050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_RC_ERROR_STATUS0_P2                        , RULL(0x8000C8050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_RC_ERROR_STATUS0_P2                        , RULL(0x8000C8050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_RC_ERROR_STATUS0_P3                        , RULL(0x8000C80507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_RC_ERROR_STATUS0_P3                        , RULL(0x8000C80508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_RC_RDVREF_CONFIG0_P0                         , RULL(0x8000C8090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_RC_RDVREF_CONFIG0_P0                       , RULL(0x8000C8090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_RC_RDVREF_CONFIG0_P0                       , RULL(0x8000C8090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_RC_RDVREF_CONFIG0_P1                       , RULL(0x8000C8090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_RC_RDVREF_CONFIG0_P1                       , RULL(0x8000C8090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_RC_RDVREF_CONFIG0_P2                       , RULL(0x8000C8090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_RC_RDVREF_CONFIG0_P2                       , RULL(0x8000C8090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_RC_RDVREF_CONFIG0_P3                       , RULL(0x8000C80907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_RC_RDVREF_CONFIG0_P3                       , RULL(0x8000C80908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_RC_RDVREF_CONFIG1_P0                         , RULL(0x8000C80A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_RC_RDVREF_CONFIG1_P0                       , RULL(0x8000C80A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_RC_RDVREF_CONFIG1_P0                       , RULL(0x8000C80A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_RC_RDVREF_CONFIG1_P1                       , RULL(0x8000C80A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_RC_RDVREF_CONFIG1_P1                       , RULL(0x8000C80A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_RC_RDVREF_CONFIG1_P2                       , RULL(0x8000C80A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_RC_RDVREF_CONFIG1_P2                       , RULL(0x8000C80A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_RC_RDVREF_CONFIG1_P3                       , RULL(0x8000C80A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_RC_RDVREF_CONFIG1_P3                       , RULL(0x8000C80A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_CONFIG0_P0                               , RULL(0x8000C4020701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_CONFIG0_P0                             , RULL(0x8000C4020701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_CONFIG0_P0                             , RULL(0x8000C4020801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_CONFIG0_P1                             , RULL(0x8000C4020701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_CONFIG0_P1                             , RULL(0x8000C4020801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_CONFIG0_P2                             , RULL(0x8000C4020701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_CONFIG0_P2                             , RULL(0x8000C4020801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_CONFIG0_P3                             , RULL(0x8000C40207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_CONFIG0_P3                             , RULL(0x8000C40208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_ERROR_MASK0_P0                           , RULL(0x8000C4090701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_ERROR_MASK0_P0                         , RULL(0x8000C4090701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_ERROR_MASK0_P0                         , RULL(0x8000C4090801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_ERROR_MASK0_P1                         , RULL(0x8000C4090701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_ERROR_MASK0_P1                         , RULL(0x8000C4090801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_ERROR_MASK0_P2                         , RULL(0x8000C4090701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_ERROR_MASK0_P2                         , RULL(0x8000C4090801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_ERROR_MASK0_P3                         , RULL(0x8000C40907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_ERROR_MASK0_P3                         , RULL(0x8000C40908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_ERROR_STATUS0_P0                         , RULL(0x8000C4080701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_SEQ_ERROR_STATUS0_P0                       , RULL(0x8000C4080701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_SEQ_ERROR_STATUS0_P0                       , RULL(0x8000C4080801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_SEQ_ERROR_STATUS0_P1                       , RULL(0x8000C4080701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_SEQ_ERROR_STATUS0_P1                       , RULL(0x8000C4080801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_SEQ_ERROR_STATUS0_P2                       , RULL(0x8000C4080701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_SEQ_ERROR_STATUS0_P2                       , RULL(0x8000C4080801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_SEQ_ERROR_STATUS0_P3                       , RULL(0x8000C40807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_SEQ_ERROR_STATUS0_P3                       , RULL(0x8000C40808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_SEQ_LPT_ADDR2_P0                             , RULL(0x8000C4170701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_LPT_ADDR2_P0                           , RULL(0x8000C4170701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_LPT_ADDR2_P0                           , RULL(0x8000C4170801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_LPT_ADDR2_P1                           , RULL(0x8000C4170701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_LPT_ADDR2_P1                           , RULL(0x8000C4170801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_LPT_ADDR2_P2                           , RULL(0x8000C4170701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_LPT_ADDR2_P2                           , RULL(0x8000C4170801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_LPT_ADDR2_P3                           , RULL(0x8000C41707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_LPT_ADDR2_P3                           , RULL(0x8000C41708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_LPT_ADDR3_P0                             , RULL(0x8000C4180701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_LPT_ADDR3_P0                           , RULL(0x8000C4180701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_LPT_ADDR3_P0                           , RULL(0x8000C4180801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_LPT_ADDR3_P1                           , RULL(0x8000C4180701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_LPT_ADDR3_P1                           , RULL(0x8000C4180801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_LPT_ADDR3_P2                           , RULL(0x8000C4180701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_LPT_ADDR3_P2                           , RULL(0x8000C4180801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_LPT_ADDR3_P3                           , RULL(0x8000C41807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_LPT_ADDR3_P3                           , RULL(0x8000C41808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_LPT_ADDR4_P0                             , RULL(0x8000C4190701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_LPT_ADDR4_P0                           , RULL(0x8000C4190701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_LPT_ADDR4_P0                           , RULL(0x8000C4190801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_LPT_ADDR4_P1                           , RULL(0x8000C4190701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_LPT_ADDR4_P1                           , RULL(0x8000C4190801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_LPT_ADDR4_P2                           , RULL(0x8000C4190701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_LPT_ADDR4_P2                           , RULL(0x8000C4190801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_LPT_ADDR4_P3                           , RULL(0x8000C41907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_LPT_ADDR4_P3                           , RULL(0x8000C41908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_MEM_TIMING_PARAM0_P0                     , RULL(0x8000C4120701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_MEM_TIMING_PARAM0_P0                   , RULL(0x8000C4120701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_MEM_TIMING_PARAM0_P0                   , RULL(0x8000C4120801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_MEM_TIMING_PARAM0_P1                   , RULL(0x8000C4120701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_MEM_TIMING_PARAM0_P1                   , RULL(0x8000C4120801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_MEM_TIMING_PARAM0_P2                   , RULL(0x8000C4120701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_MEM_TIMING_PARAM0_P2                   , RULL(0x8000C4120801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_MEM_TIMING_PARAM0_P3                   , RULL(0x8000C41207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_MEM_TIMING_PARAM0_P3                   , RULL(0x8000C41208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_MEM_TIMING_PARAM1_P0                     , RULL(0x8000C4130701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_MEM_TIMING_PARAM1_P0                   , RULL(0x8000C4130701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_MEM_TIMING_PARAM1_P0                   , RULL(0x8000C4130801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_MEM_TIMING_PARAM1_P1                   , RULL(0x8000C4130701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_MEM_TIMING_PARAM1_P1                   , RULL(0x8000C4130801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_MEM_TIMING_PARAM1_P2                   , RULL(0x8000C4130701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_MEM_TIMING_PARAM1_P2                   , RULL(0x8000C4130801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_MEM_TIMING_PARAM1_P3                   , RULL(0x8000C41307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_MEM_TIMING_PARAM1_P3                   , RULL(0x8000C41308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_MEM_TIMING_PARAM2_P0                     , RULL(0x8000C4140701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_MEM_TIMING_PARAM2_P0                   , RULL(0x8000C4140701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_MEM_TIMING_PARAM2_P0                   , RULL(0x8000C4140801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_MEM_TIMING_PARAM2_P1                   , RULL(0x8000C4140701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_MEM_TIMING_PARAM2_P1                   , RULL(0x8000C4140801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_MEM_TIMING_PARAM2_P2                   , RULL(0x8000C4140701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_MEM_TIMING_PARAM2_P2                   , RULL(0x8000C4140801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_MEM_TIMING_PARAM2_P3                   , RULL(0x8000C41407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_MEM_TIMING_PARAM2_P3                   , RULL(0x8000C41408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_ODT_DEFAULT_CONFIG_P0                    , RULL(0x8000C4240701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_ODT_DEFAULT_CONFIG_P0                  , RULL(0x8000C4240701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_ODT_DEFAULT_CONFIG_P0                  , RULL(0x8000C4240801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_ODT_DEFAULT_CONFIG_P1                  , RULL(0x8000C4240701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_ODT_DEFAULT_CONFIG_P1                  , RULL(0x8000C4240801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_ODT_DEFAULT_CONFIG_P2                  , RULL(0x8000C4240701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_ODT_DEFAULT_CONFIG_P2                  , RULL(0x8000C4240801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_ODT_DEFAULT_CONFIG_P3                  , RULL(0x8000C42407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_ODT_DEFAULT_CONFIG_P3                  , RULL(0x8000C42408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_ODT_RD_CONFIG0_P0                        , RULL(0x8000C40E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_ODT_RD_CONFIG0_P0                      , RULL(0x8000C40E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_ODT_RD_CONFIG0_P0                      , RULL(0x8000C40E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_ODT_RD_CONFIG0_P1                      , RULL(0x8000C40E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_ODT_RD_CONFIG0_P1                      , RULL(0x8000C40E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_ODT_RD_CONFIG0_P2                      , RULL(0x8000C40E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_ODT_RD_CONFIG0_P2                      , RULL(0x8000C40E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_ODT_RD_CONFIG0_P3                      , RULL(0x8000C40E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_ODT_RD_CONFIG0_P3                      , RULL(0x8000C40E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_ODT_RD_CONFIG1_P0                        , RULL(0x8000C40F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_ODT_RD_CONFIG1_P0                      , RULL(0x8000C40F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_ODT_RD_CONFIG1_P0                      , RULL(0x8000C40F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_ODT_RD_CONFIG1_P1                      , RULL(0x8000C40F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_ODT_RD_CONFIG1_P1                      , RULL(0x8000C40F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_ODT_RD_CONFIG1_P2                      , RULL(0x8000C40F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_ODT_RD_CONFIG1_P2                      , RULL(0x8000C40F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_ODT_RD_CONFIG1_P3                      , RULL(0x8000C40F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_ODT_RD_CONFIG1_P3                      , RULL(0x8000C40F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_ODT_RD_CONFIG2_P0                        , RULL(0x8000C4100701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_ODT_RD_CONFIG2_P0                      , RULL(0x8000C4100701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_ODT_RD_CONFIG2_P0                      , RULL(0x8000C4100801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_ODT_RD_CONFIG2_P1                      , RULL(0x8000C4100701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_ODT_RD_CONFIG2_P1                      , RULL(0x8000C4100801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_ODT_RD_CONFIG2_P2                      , RULL(0x8000C4100701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_ODT_RD_CONFIG2_P2                      , RULL(0x8000C4100801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_ODT_RD_CONFIG2_P3                      , RULL(0x8000C41007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_ODT_RD_CONFIG2_P3                      , RULL(0x8000C41008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_ODT_RD_CONFIG3_P0                        , RULL(0x8000C4110701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_ODT_RD_CONFIG3_P0                      , RULL(0x8000C4110701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_ODT_RD_CONFIG3_P0                      , RULL(0x8000C4110801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_ODT_RD_CONFIG3_P1                      , RULL(0x8000C4110701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_ODT_RD_CONFIG3_P1                      , RULL(0x8000C4110801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_ODT_RD_CONFIG3_P2                      , RULL(0x8000C4110701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_ODT_RD_CONFIG3_P2                      , RULL(0x8000C4110801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_ODT_RD_CONFIG3_P3                      , RULL(0x8000C41107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_ODT_RD_CONFIG3_P3                      , RULL(0x8000C41108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_ODT_WR_CONFIG0_P0                        , RULL(0x8000C40A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_ODT_WR_CONFIG0_P0                      , RULL(0x8000C40A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_ODT_WR_CONFIG0_P0                      , RULL(0x8000C40A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_ODT_WR_CONFIG0_P1                      , RULL(0x8000C40A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_ODT_WR_CONFIG0_P1                      , RULL(0x8000C40A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_ODT_WR_CONFIG0_P2                      , RULL(0x8000C40A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_ODT_WR_CONFIG0_P2                      , RULL(0x8000C40A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_ODT_WR_CONFIG0_P3                      , RULL(0x8000C40A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_ODT_WR_CONFIG0_P3                      , RULL(0x8000C40A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_ODT_WR_CONFIG1_P0                        , RULL(0x8000C40B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_ODT_WR_CONFIG1_P0                      , RULL(0x8000C40B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_ODT_WR_CONFIG1_P0                      , RULL(0x8000C40B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_ODT_WR_CONFIG1_P1                      , RULL(0x8000C40B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_ODT_WR_CONFIG1_P1                      , RULL(0x8000C40B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_ODT_WR_CONFIG1_P2                      , RULL(0x8000C40B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_ODT_WR_CONFIG1_P2                      , RULL(0x8000C40B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_ODT_WR_CONFIG1_P3                      , RULL(0x8000C40B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_ODT_WR_CONFIG1_P3                      , RULL(0x8000C40B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_ODT_WR_CONFIG2_P0                        , RULL(0x8000C40C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_ODT_WR_CONFIG2_P0                      , RULL(0x8000C40C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_ODT_WR_CONFIG2_P0                      , RULL(0x8000C40C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_ODT_WR_CONFIG2_P1                      , RULL(0x8000C40C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_ODT_WR_CONFIG2_P1                      , RULL(0x8000C40C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_ODT_WR_CONFIG2_P2                      , RULL(0x8000C40C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_ODT_WR_CONFIG2_P2                      , RULL(0x8000C40C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_ODT_WR_CONFIG2_P3                      , RULL(0x8000C40C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_ODT_WR_CONFIG2_P3                      , RULL(0x8000C40C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_ODT_WR_CONFIG3_P0                        , RULL(0x8000C40D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_ODT_WR_CONFIG3_P0                      , RULL(0x8000C40D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_ODT_WR_CONFIG3_P0                      , RULL(0x8000C40D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_ODT_WR_CONFIG3_P1                      , RULL(0x8000C40D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_ODT_WR_CONFIG3_P1                      , RULL(0x8000C40D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_ODT_WR_CONFIG3_P2                      , RULL(0x8000C40D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_ODT_WR_CONFIG3_P2                      , RULL(0x8000C40D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_ODT_WR_CONFIG3_P3                      , RULL(0x8000C40D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_ODT_WR_CONFIG3_P3                      , RULL(0x8000C40D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_RD_WR_DATA0_P0                           , RULL(0x8000C4000701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_RD_WR_DATA0_P0                         , RULL(0x8000C4000701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_RD_WR_DATA0_P0                         , RULL(0x8000C4000801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_RD_WR_DATA0_P1                         , RULL(0x8000C4000701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_RD_WR_DATA0_P1                         , RULL(0x8000C4000801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_RD_WR_DATA0_P2                         , RULL(0x8000C4000701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_RD_WR_DATA0_P2                         , RULL(0x8000C4000801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_RD_WR_DATA0_P3                         , RULL(0x8000C40007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_RD_WR_DATA0_P3                         , RULL(0x8000C40008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_RD_WR_DATA1_P0                           , RULL(0x8000C4010701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_RD_WR_DATA1_P0                         , RULL(0x8000C4010701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_RD_WR_DATA1_P0                         , RULL(0x8000C4010801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_RD_WR_DATA1_P1                         , RULL(0x8000C4010701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_RD_WR_DATA1_P1                         , RULL(0x8000C4010801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_RD_WR_DATA1_P2                         , RULL(0x8000C4010701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_RD_WR_DATA1_P2                         , RULL(0x8000C4010801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_RD_WR_DATA1_P3                         , RULL(0x8000C40107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_RD_WR_DATA1_P3                         , RULL(0x8000C40108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_RESERVED_ADDR0_P0                        , RULL(0x8000C4030701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_RESERVED_ADDR0_P0                      , RULL(0x8000C4030701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_RESERVED_ADDR0_P0                      , RULL(0x8000C4030801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_RESERVED_ADDR0_P1                      , RULL(0x8000C4030701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_RESERVED_ADDR0_P1                      , RULL(0x8000C4030801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_RESERVED_ADDR0_P2                      , RULL(0x8000C4030701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_RESERVED_ADDR0_P2                      , RULL(0x8000C4030801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_RESERVED_ADDR0_P3                      , RULL(0x8000C40307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_RESERVED_ADDR0_P3                      , RULL(0x8000C40308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_RESERVED_ADDR1_P0                        , RULL(0x8000C4040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_RESERVED_ADDR1_P0                      , RULL(0x8000C4040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_RESERVED_ADDR1_P0                      , RULL(0x8000C4040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_RESERVED_ADDR1_P1                      , RULL(0x8000C4040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_RESERVED_ADDR1_P1                      , RULL(0x8000C4040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_RESERVED_ADDR1_P2                      , RULL(0x8000C4040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_RESERVED_ADDR1_P2                      , RULL(0x8000C4040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_RESERVED_ADDR1_P3                      , RULL(0x8000C40407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_RESERVED_ADDR1_P3                      , RULL(0x8000C40408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_RESERVED_ADDR2_P0                        , RULL(0x8000C4050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_RESERVED_ADDR2_P0                      , RULL(0x8000C4050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_RESERVED_ADDR2_P0                      , RULL(0x8000C4050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_RESERVED_ADDR2_P1                      , RULL(0x8000C4050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_RESERVED_ADDR2_P1                      , RULL(0x8000C4050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_RESERVED_ADDR2_P2                      , RULL(0x8000C4050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_RESERVED_ADDR2_P2                      , RULL(0x8000C4050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_RESERVED_ADDR2_P3                      , RULL(0x8000C40507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_RESERVED_ADDR2_P3                      , RULL(0x8000C40508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_RESERVED_ADDR3_P0                        , RULL(0x8000C4060701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_RESERVED_ADDR3_P0                      , RULL(0x8000C4060701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_RESERVED_ADDR3_P0                      , RULL(0x8000C4060801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_RESERVED_ADDR3_P1                      , RULL(0x8000C4060701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_RESERVED_ADDR3_P1                      , RULL(0x8000C4060801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_RESERVED_ADDR3_P2                      , RULL(0x8000C4060701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_RESERVED_ADDR3_P2                      , RULL(0x8000C4060801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_RESERVED_ADDR3_P3                      , RULL(0x8000C40607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_RESERVED_ADDR3_P3                      , RULL(0x8000C40608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_RESERVED_ADDR4_P0                        , RULL(0x8000C4070701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_RESERVED_ADDR4_P0                      , RULL(0x8000C4070701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_RESERVED_ADDR4_P0                      , RULL(0x8000C4070801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_RESERVED_ADDR4_P1                      , RULL(0x8000C4070701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_RESERVED_ADDR4_P1                      , RULL(0x8000C4070801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_RESERVED_ADDR4_P2                      , RULL(0x8000C4070701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_RESERVED_ADDR4_P2                      , RULL(0x8000C4070801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_RESERVED_ADDR4_P3                      , RULL(0x8000C40707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_RESERVED_ADDR4_P3                      , RULL(0x8000C40708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_WR_TERM_SWAP0_P0                         , RULL(0x8000C41D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_WR_TERM_SWAP0_P0                       , RULL(0x8000C41D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_WR_TERM_SWAP0_P0                       , RULL(0x8000C41D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_WR_TERM_SWAP0_P1                       , RULL(0x8000C41D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_WR_TERM_SWAP0_P1                       , RULL(0x8000C41D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_WR_TERM_SWAP0_P2                       , RULL(0x8000C41D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_WR_TERM_SWAP0_P2                       , RULL(0x8000C41D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_WR_TERM_SWAP0_P3                       , RULL(0x8000C41D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_WR_TERM_SWAP0_P3                       , RULL(0x8000C41D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_WR_TERM_SWAP1_P0                         , RULL(0x8000C41E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_WR_TERM_SWAP1_P0                       , RULL(0x8000C41E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_WR_TERM_SWAP1_P0                       , RULL(0x8000C41E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_WR_TERM_SWAP1_P1                       , RULL(0x8000C41E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_WR_TERM_SWAP1_P1                       , RULL(0x8000C41E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_WR_TERM_SWAP1_P2                       , RULL(0x8000C41E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_WR_TERM_SWAP1_P2                       , RULL(0x8000C41E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_WR_TERM_SWAP1_P3                       , RULL(0x8000C41E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_WR_TERM_SWAP1_P3                       , RULL(0x8000C41E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_ZQCAL_ENC_RANK_CTL_REG0_P0               , RULL(0x8000C41A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_ZQCAL_ENC_RANK_CTL_REG0_P0             , RULL(0x8000C41A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_ZQCAL_ENC_RANK_CTL_REG0_P0             , RULL(0x8000C41A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_ZQCAL_ENC_RANK_CTL_REG0_P1             , RULL(0x8000C41A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_ZQCAL_ENC_RANK_CTL_REG0_P1             , RULL(0x8000C41A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_ZQCAL_ENC_RANK_CTL_REG0_P2             , RULL(0x8000C41A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_ZQCAL_ENC_RANK_CTL_REG0_P2             , RULL(0x8000C41A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_ZQCAL_ENC_RANK_CTL_REG0_P3             , RULL(0x8000C41A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_ZQCAL_ENC_RANK_CTL_REG0_P3             , RULL(0x8000C41A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_SEQ_ZQCAL_ENC_RANK_CTL_REG1_P0               , RULL(0x8000C41B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_SEQ_ZQCAL_ENC_RANK_CTL_REG1_P0             , RULL(0x8000C41B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_SEQ_ZQCAL_ENC_RANK_CTL_REG1_P0             , RULL(0x8000C41B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_SEQ_ZQCAL_ENC_RANK_CTL_REG1_P1             , RULL(0x8000C41B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_SEQ_ZQCAL_ENC_RANK_CTL_REG1_P1             , RULL(0x8000C41B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_SEQ_ZQCAL_ENC_RANK_CTL_REG1_P2             , RULL(0x8000C41B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_SEQ_ZQCAL_ENC_RANK_CTL_REG1_P2             , RULL(0x8000C41B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_SEQ_ZQCAL_ENC_RANK_CTL_REG1_P3             , RULL(0x8000C41B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_SEQ_ZQCAL_ENC_RANK_CTL_REG1_P3             , RULL(0x8000C41B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_WC_CONFIG0_P0                                , RULL(0x8000CC000701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_WC_CONFIG0_P0                              , RULL(0x8000CC000701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_WC_CONFIG0_P0                              , RULL(0x8000CC000801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_WC_CONFIG0_P1                              , RULL(0x8000CC000701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_WC_CONFIG0_P1                              , RULL(0x8000CC000801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_WC_CONFIG0_P2                              , RULL(0x8000CC000701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_WC_CONFIG0_P2                              , RULL(0x8000CC000801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_WC_CONFIG0_P3                              , RULL(0x8000CC0007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_WC_CONFIG0_P3                              , RULL(0x8000CC0008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_WC_CONFIG1_P0                                , RULL(0x8000CC010701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_WC_CONFIG1_P0                              , RULL(0x8000CC010701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_WC_CONFIG1_P0                              , RULL(0x8000CC010801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_WC_CONFIG1_P1                              , RULL(0x8000CC010701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_WC_CONFIG1_P1                              , RULL(0x8000CC010801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_WC_CONFIG1_P2                              , RULL(0x8000CC010701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_WC_CONFIG1_P2                              , RULL(0x8000CC010801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_WC_CONFIG1_P3                              , RULL(0x8000CC0107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_WC_CONFIG1_P3                              , RULL(0x8000CC0108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_WC_CONFIG2_P0                                , RULL(0x8000CC020701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_WC_CONFIG2_P0                              , RULL(0x8000CC020701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_WC_CONFIG2_P0                              , RULL(0x8000CC020801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_WC_CONFIG2_P1                              , RULL(0x8000CC020701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_WC_CONFIG2_P1                              , RULL(0x8000CC020801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_WC_CONFIG2_P2                              , RULL(0x8000CC020701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_WC_CONFIG2_P2                              , RULL(0x8000CC020801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_WC_CONFIG2_P3                              , RULL(0x8000CC0207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_WC_CONFIG2_P3                              , RULL(0x8000CC0208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_WC_CONFIG3_P0                                , RULL(0x8000CC050701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_WC_CONFIG3_P0                              , RULL(0x8000CC050701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_WC_CONFIG3_P0                              , RULL(0x8000CC050801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_WC_CONFIG3_P1                              , RULL(0x8000CC050701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_WC_CONFIG3_P1                              , RULL(0x8000CC050801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_WC_CONFIG3_P2                              , RULL(0x8000CC050701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_WC_CONFIG3_P2                              , RULL(0x8000CC050801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_WC_CONFIG3_P3                              , RULL(0x8000CC0507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_WC_CONFIG3_P3                              , RULL(0x8000CC0508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_WC_ERROR_MASK0_P0                            , RULL(0x8000CC040701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_WC_ERROR_MASK0_P0                          , RULL(0x8000CC040701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_WC_ERROR_MASK0_P0                          , RULL(0x8000CC040801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_WC_ERROR_MASK0_P1                          , RULL(0x8000CC040701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_WC_ERROR_MASK0_P1                          , RULL(0x8000CC040801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_WC_ERROR_MASK0_P2                          , RULL(0x8000CC040701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_WC_ERROR_MASK0_P2                          , RULL(0x8000CC040801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_WC_ERROR_MASK0_P3                          , RULL(0x8000CC0407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_WC_ERROR_MASK0_P3                          , RULL(0x8000CC0408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DDRPHY_WC_ERROR_STATUS0_P0                          , RULL(0x8000CC030701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RO   );
REG64( MCA_0_DDRPHY_WC_ERROR_STATUS0_P0                        , RULL(0x8000CC030701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_4_DDRPHY_WC_ERROR_STATUS0_P0                        , RULL(0x8000CC030801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_1_DDRPHY_WC_ERROR_STATUS0_P1                        , RULL(0x8000CC030701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_5_DDRPHY_WC_ERROR_STATUS0_P1                        , RULL(0x8000CC030801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_2_DDRPHY_WC_ERROR_STATUS0_P2                        , RULL(0x8000CC030701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_6_DDRPHY_WC_ERROR_STATUS0_P2                        , RULL(0x8000CC030801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_3_DDRPHY_WC_ERROR_STATUS0_P3                        , RULL(0x8000CC0307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RO   );
REG64( MCA_7_DDRPHY_WC_ERROR_STATUS0_P3                        , RULL(0x8000CC0308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RO   );

REG64( MCA_DDRPHY_WC_RTT_WR_SWAP_ENABLE_P0                     , RULL(0x8000CC060701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DDRPHY_WC_RTT_WR_SWAP_ENABLE_P0                   , RULL(0x8000CC060701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DDRPHY_WC_RTT_WR_SWAP_ENABLE_P0                   , RULL(0x8000CC060801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DDRPHY_WC_RTT_WR_SWAP_ENABLE_P1                   , RULL(0x8000CC060701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DDRPHY_WC_RTT_WR_SWAP_ENABLE_P1                   , RULL(0x8000CC060801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DDRPHY_WC_RTT_WR_SWAP_ENABLE_P2                   , RULL(0x8000CC060701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DDRPHY_WC_RTT_WR_SWAP_ENABLE_P2                   , RULL(0x8000CC060801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DDRPHY_WC_RTT_WR_SWAP_ENABLE_P3                   , RULL(0x8000CC0607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DDRPHY_WC_RTT_WR_SWAP_ENABLE_P3                   , RULL(0x8000CC0608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DEBUG_TRACE_CONTROL                               , RULL(0x070107D0), SH_UNT_MCA_3    , SH_ACS_SCOM      );
REG64( MCA_7_DEBUG_TRACE_CONTROL                               , RULL(0x080107D0), SH_UNT_MCA_7    , SH_ACS_SCOM      );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP0_REG_P0_0                  , RULL(0x800000380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP0_REG_P0_0                , RULL(0x800000380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP0_REG_P0_0                , RULL(0x800000380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP0_REG_P0_1                  , RULL(0x800004380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP0_REG_P0_1                , RULL(0x800004380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP0_REG_P0_1                , RULL(0x800004380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP0_REG_P0_2                  , RULL(0x800008380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP0_REG_P0_2                , RULL(0x800008380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP0_REG_P0_2                , RULL(0x800008380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP0_REG_P0_3                  , RULL(0x80000C380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP0_REG_P0_3                , RULL(0x80000C380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP0_REG_P0_3                , RULL(0x80000C380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP0_REG_P0_4                  , RULL(0x800010380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP0_REG_P0_4                , RULL(0x800010380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP0_REG_P0_4                , RULL(0x800010380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP0_REG_P1_0                , RULL(0x800000380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP0_REG_P1_0                , RULL(0x800000380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP0_REG_P1_1                , RULL(0x800004380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP0_REG_P1_1                , RULL(0x800004380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP0_REG_P1_2                , RULL(0x800008380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP0_REG_P1_2                , RULL(0x800008380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP0_REG_P1_3                , RULL(0x80000C380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP0_REG_P1_3                , RULL(0x80000C380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP0_REG_P1_4                , RULL(0x800010380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP0_REG_P1_4                , RULL(0x800010380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP0_REG_P2_0                , RULL(0x800000380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP0_REG_P2_0                , RULL(0x800000380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP0_REG_P2_1                , RULL(0x800004380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP0_REG_P2_1                , RULL(0x800004380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP0_REG_P2_2                , RULL(0x800008380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP0_REG_P2_2                , RULL(0x800008380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP0_REG_P2_3                , RULL(0x80000C380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP0_REG_P2_3                , RULL(0x80000C380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP0_REG_P2_4                , RULL(0x800010380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP0_REG_P2_4                , RULL(0x800010380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP0_REG_P3_0                , RULL(0x8000003807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP0_REG_P3_0                , RULL(0x8000003808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP0_REG_P3_1                , RULL(0x8000043807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP0_REG_P3_1                , RULL(0x8000043808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP0_REG_P3_2                , RULL(0x8000083807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP0_REG_P3_2                , RULL(0x8000083808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP0_REG_P3_3                , RULL(0x80000C3807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP0_REG_P3_3                , RULL(0x80000C3808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP0_REG_P3_4                , RULL(0x8000103807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP0_REG_P3_4                , RULL(0x8000103808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP1_REG_P0_0                  , RULL(0x800001380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP1_REG_P0_0                , RULL(0x800001380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP1_REG_P0_0                , RULL(0x800001380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP1_REG_P0_1                  , RULL(0x800005380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP1_REG_P0_1                , RULL(0x800005380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP1_REG_P0_1                , RULL(0x800005380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP1_REG_P0_2                  , RULL(0x800009380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP1_REG_P0_2                , RULL(0x800009380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP1_REG_P0_2                , RULL(0x800009380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP1_REG_P0_3                  , RULL(0x80000D380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP1_REG_P0_3                , RULL(0x80000D380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP1_REG_P0_3                , RULL(0x80000D380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP1_REG_P0_4                  , RULL(0x800011380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP1_REG_P0_4                , RULL(0x800011380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP1_REG_P0_4                , RULL(0x800011380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP1_REG_P1_0                , RULL(0x800001380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP1_REG_P1_0                , RULL(0x800001380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP1_REG_P1_1                , RULL(0x800005380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP1_REG_P1_1                , RULL(0x800005380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP1_REG_P1_2                , RULL(0x800009380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP1_REG_P1_2                , RULL(0x800009380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP1_REG_P1_3                , RULL(0x80000D380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP1_REG_P1_3                , RULL(0x80000D380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP1_REG_P1_4                , RULL(0x800011380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP1_REG_P1_4                , RULL(0x800011380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP1_REG_P2_0                , RULL(0x800001380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP1_REG_P2_0                , RULL(0x800001380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP1_REG_P2_1                , RULL(0x800005380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP1_REG_P2_1                , RULL(0x800005380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP1_REG_P2_2                , RULL(0x800009380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP1_REG_P2_2                , RULL(0x800009380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP1_REG_P2_3                , RULL(0x80000D380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP1_REG_P2_3                , RULL(0x80000D380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP1_REG_P2_4                , RULL(0x800011380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP1_REG_P2_4                , RULL(0x800011380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP1_REG_P3_0                , RULL(0x8000013807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP1_REG_P3_0                , RULL(0x8000013808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP1_REG_P3_1                , RULL(0x8000053807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP1_REG_P3_1                , RULL(0x8000053808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP1_REG_P3_2                , RULL(0x8000093807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP1_REG_P3_2                , RULL(0x8000093808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP1_REG_P3_3                , RULL(0x80000D3807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP1_REG_P3_3                , RULL(0x80000D3808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP1_REG_P3_4                , RULL(0x8000113807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP1_REG_P3_4                , RULL(0x8000113808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP2_REG_P0_0                  , RULL(0x800002380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP2_REG_P0_0                , RULL(0x800002380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP2_REG_P0_0                , RULL(0x800002380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP2_REG_P0_1                  , RULL(0x800006380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP2_REG_P0_1                , RULL(0x800006380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP2_REG_P0_1                , RULL(0x800006380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP2_REG_P0_2                  , RULL(0x80000A380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP2_REG_P0_2                , RULL(0x80000A380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP2_REG_P0_2                , RULL(0x80000A380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP2_REG_P0_3                  , RULL(0x80000E380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP2_REG_P0_3                , RULL(0x80000E380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP2_REG_P0_3                , RULL(0x80000E380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP2_REG_P0_4                  , RULL(0x800012380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP2_REG_P0_4                , RULL(0x800012380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP2_REG_P0_4                , RULL(0x800012380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP2_REG_P1_0                , RULL(0x800002380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP2_REG_P1_0                , RULL(0x800002380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP2_REG_P1_1                , RULL(0x800006380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP2_REG_P1_1                , RULL(0x800006380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP2_REG_P1_2                , RULL(0x80000A380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP2_REG_P1_2                , RULL(0x80000A380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP2_REG_P1_3                , RULL(0x80000E380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP2_REG_P1_3                , RULL(0x80000E380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP2_REG_P1_4                , RULL(0x800012380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP2_REG_P1_4                , RULL(0x800012380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP2_REG_P2_0                , RULL(0x800002380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP2_REG_P2_0                , RULL(0x800002380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP2_REG_P2_1                , RULL(0x800006380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP2_REG_P2_1                , RULL(0x800006380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP2_REG_P2_2                , RULL(0x80000A380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP2_REG_P2_2                , RULL(0x80000A380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP2_REG_P2_3                , RULL(0x80000E380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP2_REG_P2_3                , RULL(0x80000E380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP2_REG_P2_4                , RULL(0x800012380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP2_REG_P2_4                , RULL(0x800012380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP2_REG_P3_0                , RULL(0x8000023807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP2_REG_P3_0                , RULL(0x8000023808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP2_REG_P3_1                , RULL(0x8000063807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP2_REG_P3_1                , RULL(0x8000063808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP2_REG_P3_2                , RULL(0x80000A3807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP2_REG_P3_2                , RULL(0x80000A3808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP2_REG_P3_3                , RULL(0x80000E3807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP2_REG_P3_3                , RULL(0x80000E3808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP2_REG_P3_4                , RULL(0x8000123807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP2_REG_P3_4                , RULL(0x8000123808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP3_REG_P0_0                  , RULL(0x800003380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP3_REG_P0_0                , RULL(0x800003380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP3_REG_P0_0                , RULL(0x800003380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP3_REG_P0_1                  , RULL(0x800007380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP3_REG_P0_1                , RULL(0x800007380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP3_REG_P0_1                , RULL(0x800007380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP3_REG_P0_2                  , RULL(0x80000B380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP3_REG_P0_2                , RULL(0x80000B380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP3_REG_P0_2                , RULL(0x80000B380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP3_REG_P0_3                  , RULL(0x80000F380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP3_REG_P0_3                , RULL(0x80000F380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP3_REG_P0_3                , RULL(0x80000F380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_0_RP3_REG_P0_4                  , RULL(0x800013380701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_0_RP3_REG_P0_4                , RULL(0x800013380701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_0_RP3_REG_P0_4                , RULL(0x800013380801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP3_REG_P1_0                , RULL(0x800003380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP3_REG_P1_0                , RULL(0x800003380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP3_REG_P1_1                , RULL(0x800007380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP3_REG_P1_1                , RULL(0x800007380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP3_REG_P1_2                , RULL(0x80000B380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP3_REG_P1_2                , RULL(0x80000B380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP3_REG_P1_3                , RULL(0x80000F380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP3_REG_P1_3                , RULL(0x80000F380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_0_RP3_REG_P1_4                , RULL(0x800013380701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_0_RP3_REG_P1_4                , RULL(0x800013380801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP3_REG_P2_0                , RULL(0x800003380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP3_REG_P2_0                , RULL(0x800003380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP3_REG_P2_1                , RULL(0x800007380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP3_REG_P2_1                , RULL(0x800007380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP3_REG_P2_2                , RULL(0x80000B380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP3_REG_P2_2                , RULL(0x80000B380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP3_REG_P2_3                , RULL(0x80000F380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP3_REG_P2_3                , RULL(0x80000F380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_0_RP3_REG_P2_4                , RULL(0x800013380701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_0_RP3_REG_P2_4                , RULL(0x800013380801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP3_REG_P3_0                , RULL(0x8000033807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP3_REG_P3_0                , RULL(0x8000033808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP3_REG_P3_1                , RULL(0x8000073807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP3_REG_P3_1                , RULL(0x8000073808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP3_REG_P3_2                , RULL(0x80000B3807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP3_REG_P3_2                , RULL(0x80000B3808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP3_REG_P3_3                , RULL(0x80000F3807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP3_REG_P3_3                , RULL(0x80000F3808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_0_RP3_REG_P3_4                , RULL(0x8000133807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_0_RP3_REG_P3_4                , RULL(0x8000133808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP0_REG_P0_0                 , RULL(0x800000420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP0_REG_P0_0               , RULL(0x800000420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP0_REG_P0_0               , RULL(0x800000420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP0_REG_P0_1                 , RULL(0x800004420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP0_REG_P0_1               , RULL(0x800004420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP0_REG_P0_1               , RULL(0x800004420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP0_REG_P0_2                 , RULL(0x800008420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP0_REG_P0_2               , RULL(0x800008420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP0_REG_P0_2               , RULL(0x800008420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP0_REG_P0_3                 , RULL(0x80000C420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP0_REG_P0_3               , RULL(0x80000C420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP0_REG_P0_3               , RULL(0x80000C420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP0_REG_P0_4                 , RULL(0x800010420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP0_REG_P0_4               , RULL(0x800010420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP0_REG_P0_4               , RULL(0x800010420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP0_REG_P1_0               , RULL(0x800000420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP0_REG_P1_0               , RULL(0x800000420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP0_REG_P1_1               , RULL(0x800004420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP0_REG_P1_1               , RULL(0x800004420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP0_REG_P1_2               , RULL(0x800008420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP0_REG_P1_2               , RULL(0x800008420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP0_REG_P1_3               , RULL(0x80000C420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP0_REG_P1_3               , RULL(0x80000C420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP0_REG_P1_4               , RULL(0x800010420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP0_REG_P1_4               , RULL(0x800010420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP0_REG_P2_0               , RULL(0x800000420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP0_REG_P2_0               , RULL(0x800000420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP0_REG_P2_1               , RULL(0x800004420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP0_REG_P2_1               , RULL(0x800004420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP0_REG_P2_2               , RULL(0x800008420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP0_REG_P2_2               , RULL(0x800008420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP0_REG_P2_3               , RULL(0x80000C420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP0_REG_P2_3               , RULL(0x80000C420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP0_REG_P2_4               , RULL(0x800010420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP0_REG_P2_4               , RULL(0x800010420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP0_REG_P3_0               , RULL(0x8000004207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP0_REG_P3_0               , RULL(0x8000004208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP0_REG_P3_1               , RULL(0x8000044207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP0_REG_P3_1               , RULL(0x8000044208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP0_REG_P3_2               , RULL(0x8000084207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP0_REG_P3_2               , RULL(0x8000084208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP0_REG_P3_3               , RULL(0x80000C4207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP0_REG_P3_3               , RULL(0x80000C4208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP0_REG_P3_4               , RULL(0x8000104207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP0_REG_P3_4               , RULL(0x8000104208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP1_REG_P0_0                 , RULL(0x800001420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP1_REG_P0_0               , RULL(0x800001420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP1_REG_P0_0               , RULL(0x800001420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP1_REG_P0_1                 , RULL(0x800005420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP1_REG_P0_1               , RULL(0x800005420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP1_REG_P0_1               , RULL(0x800005420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP1_REG_P0_2                 , RULL(0x800009420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP1_REG_P0_2               , RULL(0x800009420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP1_REG_P0_2               , RULL(0x800009420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP1_REG_P0_3                 , RULL(0x80000D420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP1_REG_P0_3               , RULL(0x80000D420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP1_REG_P0_3               , RULL(0x80000D420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP1_REG_P0_4                 , RULL(0x800011420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP1_REG_P0_4               , RULL(0x800011420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP1_REG_P0_4               , RULL(0x800011420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP1_REG_P1_0               , RULL(0x800001420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP1_REG_P1_0               , RULL(0x800001420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP1_REG_P1_1               , RULL(0x800005420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP1_REG_P1_1               , RULL(0x800005420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP1_REG_P1_2               , RULL(0x800009420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP1_REG_P1_2               , RULL(0x800009420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP1_REG_P1_3               , RULL(0x80000D420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP1_REG_P1_3               , RULL(0x80000D420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP1_REG_P1_4               , RULL(0x800011420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP1_REG_P1_4               , RULL(0x800011420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP1_REG_P2_0               , RULL(0x800001420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP1_REG_P2_0               , RULL(0x800001420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP1_REG_P2_1               , RULL(0x800005420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP1_REG_P2_1               , RULL(0x800005420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP1_REG_P2_2               , RULL(0x800009420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP1_REG_P2_2               , RULL(0x800009420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP1_REG_P2_3               , RULL(0x80000D420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP1_REG_P2_3               , RULL(0x80000D420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP1_REG_P2_4               , RULL(0x800011420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP1_REG_P2_4               , RULL(0x800011420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP1_REG_P3_0               , RULL(0x8000014207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP1_REG_P3_0               , RULL(0x8000014208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP1_REG_P3_1               , RULL(0x8000054207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP1_REG_P3_1               , RULL(0x8000054208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP1_REG_P3_2               , RULL(0x8000094207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP1_REG_P3_2               , RULL(0x8000094208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP1_REG_P3_3               , RULL(0x80000D4207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP1_REG_P3_3               , RULL(0x80000D4208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP1_REG_P3_4               , RULL(0x8000114207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP1_REG_P3_4               , RULL(0x8000114208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP2_REG_P0_0                 , RULL(0x800002420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP2_REG_P0_0               , RULL(0x800002420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP2_REG_P0_0               , RULL(0x800002420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP2_REG_P0_1                 , RULL(0x800006420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP2_REG_P0_1               , RULL(0x800006420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP2_REG_P0_1               , RULL(0x800006420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP2_REG_P0_2                 , RULL(0x80000A420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP2_REG_P0_2               , RULL(0x80000A420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP2_REG_P0_2               , RULL(0x80000A420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP2_REG_P0_3                 , RULL(0x80000E420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP2_REG_P0_3               , RULL(0x80000E420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP2_REG_P0_3               , RULL(0x80000E420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP2_REG_P0_4                 , RULL(0x800012420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP2_REG_P0_4               , RULL(0x800012420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP2_REG_P0_4               , RULL(0x800012420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP2_REG_P1_0               , RULL(0x800002420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP2_REG_P1_0               , RULL(0x800002420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP2_REG_P1_1               , RULL(0x800006420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP2_REG_P1_1               , RULL(0x800006420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP2_REG_P1_2               , RULL(0x80000A420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP2_REG_P1_2               , RULL(0x80000A420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP2_REG_P1_3               , RULL(0x80000E420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP2_REG_P1_3               , RULL(0x80000E420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP2_REG_P1_4               , RULL(0x800012420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP2_REG_P1_4               , RULL(0x800012420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP2_REG_P2_0               , RULL(0x800002420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP2_REG_P2_0               , RULL(0x800002420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP2_REG_P2_1               , RULL(0x800006420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP2_REG_P2_1               , RULL(0x800006420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP2_REG_P2_2               , RULL(0x80000A420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP2_REG_P2_2               , RULL(0x80000A420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP2_REG_P2_3               , RULL(0x80000E420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP2_REG_P2_3               , RULL(0x80000E420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP2_REG_P2_4               , RULL(0x800012420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP2_REG_P2_4               , RULL(0x800012420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP2_REG_P3_0               , RULL(0x8000024207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP2_REG_P3_0               , RULL(0x8000024208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP2_REG_P3_1               , RULL(0x8000064207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP2_REG_P3_1               , RULL(0x8000064208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP2_REG_P3_2               , RULL(0x80000A4207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP2_REG_P3_2               , RULL(0x80000A4208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP2_REG_P3_3               , RULL(0x80000E4207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP2_REG_P3_3               , RULL(0x80000E4208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP2_REG_P3_4               , RULL(0x8000124207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP2_REG_P3_4               , RULL(0x8000124208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP3_REG_P0_0                 , RULL(0x800003420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP3_REG_P0_0               , RULL(0x800003420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP3_REG_P0_0               , RULL(0x800003420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP3_REG_P0_1                 , RULL(0x800007420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP3_REG_P0_1               , RULL(0x800007420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP3_REG_P0_1               , RULL(0x800007420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP3_REG_P0_2                 , RULL(0x80000B420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP3_REG_P0_2               , RULL(0x80000B420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP3_REG_P0_2               , RULL(0x80000B420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP3_REG_P0_3                 , RULL(0x80000F420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP3_REG_P0_3               , RULL(0x80000F420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP3_REG_P0_3               , RULL(0x80000F420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_10_RP3_REG_P0_4                 , RULL(0x800013420701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_10_RP3_REG_P0_4               , RULL(0x800013420701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_10_RP3_REG_P0_4               , RULL(0x800013420801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP3_REG_P1_0               , RULL(0x800003420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP3_REG_P1_0               , RULL(0x800003420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP3_REG_P1_1               , RULL(0x800007420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP3_REG_P1_1               , RULL(0x800007420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP3_REG_P1_2               , RULL(0x80000B420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP3_REG_P1_2               , RULL(0x80000B420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP3_REG_P1_3               , RULL(0x80000F420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP3_REG_P1_3               , RULL(0x80000F420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_10_RP3_REG_P1_4               , RULL(0x800013420701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_10_RP3_REG_P1_4               , RULL(0x800013420801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP3_REG_P2_0               , RULL(0x800003420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP3_REG_P2_0               , RULL(0x800003420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP3_REG_P2_1               , RULL(0x800007420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP3_REG_P2_1               , RULL(0x800007420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP3_REG_P2_2               , RULL(0x80000B420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP3_REG_P2_2               , RULL(0x80000B420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP3_REG_P2_3               , RULL(0x80000F420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP3_REG_P2_3               , RULL(0x80000F420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_10_RP3_REG_P2_4               , RULL(0x800013420701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_10_RP3_REG_P2_4               , RULL(0x800013420801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP3_REG_P3_0               , RULL(0x8000034207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP3_REG_P3_0               , RULL(0x8000034208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP3_REG_P3_1               , RULL(0x8000074207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP3_REG_P3_1               , RULL(0x8000074208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP3_REG_P3_2               , RULL(0x80000B4207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP3_REG_P3_2               , RULL(0x80000B4208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP3_REG_P3_3               , RULL(0x80000F4207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP3_REG_P3_3               , RULL(0x80000F4208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_10_RP3_REG_P3_4               , RULL(0x8000134207011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_10_RP3_REG_P3_4               , RULL(0x8000134208011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP0_REG_P0_0                 , RULL(0x800000430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP0_REG_P0_0               , RULL(0x800000430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP0_REG_P0_0               , RULL(0x800000430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP0_REG_P0_1                 , RULL(0x800004430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP0_REG_P0_1               , RULL(0x800004430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP0_REG_P0_1               , RULL(0x800004430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP0_REG_P0_2                 , RULL(0x800008430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP0_REG_P0_2               , RULL(0x800008430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP0_REG_P0_2               , RULL(0x800008430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP0_REG_P0_3                 , RULL(0x80000C430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP0_REG_P0_3               , RULL(0x80000C430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP0_REG_P0_3               , RULL(0x80000C430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP0_REG_P0_4                 , RULL(0x800010430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP0_REG_P0_4               , RULL(0x800010430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP0_REG_P0_4               , RULL(0x800010430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP0_REG_P1_0               , RULL(0x800000430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP0_REG_P1_0               , RULL(0x800000430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP0_REG_P1_1               , RULL(0x800004430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP0_REG_P1_1               , RULL(0x800004430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP0_REG_P1_2               , RULL(0x800008430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP0_REG_P1_2               , RULL(0x800008430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP0_REG_P1_3               , RULL(0x80000C430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP0_REG_P1_3               , RULL(0x80000C430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP0_REG_P1_4               , RULL(0x800010430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP0_REG_P1_4               , RULL(0x800010430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP0_REG_P2_0               , RULL(0x800000430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP0_REG_P2_0               , RULL(0x800000430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP0_REG_P2_1               , RULL(0x800004430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP0_REG_P2_1               , RULL(0x800004430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP0_REG_P2_2               , RULL(0x800008430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP0_REG_P2_2               , RULL(0x800008430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP0_REG_P2_3               , RULL(0x80000C430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP0_REG_P2_3               , RULL(0x80000C430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP0_REG_P2_4               , RULL(0x800010430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP0_REG_P2_4               , RULL(0x800010430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP0_REG_P3_0               , RULL(0x8000004307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP0_REG_P3_0               , RULL(0x8000004308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP0_REG_P3_1               , RULL(0x8000044307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP0_REG_P3_1               , RULL(0x8000044308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP0_REG_P3_2               , RULL(0x8000084307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP0_REG_P3_2               , RULL(0x8000084308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP0_REG_P3_3               , RULL(0x80000C4307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP0_REG_P3_3               , RULL(0x80000C4308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP0_REG_P3_4               , RULL(0x8000104307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP0_REG_P3_4               , RULL(0x8000104308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP1_REG_P0_0                 , RULL(0x800001430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP1_REG_P0_0               , RULL(0x800001430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP1_REG_P0_0               , RULL(0x800001430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP1_REG_P0_1                 , RULL(0x800005430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP1_REG_P0_1               , RULL(0x800005430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP1_REG_P0_1               , RULL(0x800005430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP1_REG_P0_2                 , RULL(0x800009430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP1_REG_P0_2               , RULL(0x800009430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP1_REG_P0_2               , RULL(0x800009430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP1_REG_P0_3                 , RULL(0x80000D430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP1_REG_P0_3               , RULL(0x80000D430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP1_REG_P0_3               , RULL(0x80000D430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP1_REG_P0_4                 , RULL(0x800011430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP1_REG_P0_4               , RULL(0x800011430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP1_REG_P0_4               , RULL(0x800011430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP1_REG_P1_0               , RULL(0x800001430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP1_REG_P1_0               , RULL(0x800001430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP1_REG_P1_1               , RULL(0x800005430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP1_REG_P1_1               , RULL(0x800005430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP1_REG_P1_2               , RULL(0x800009430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP1_REG_P1_2               , RULL(0x800009430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP1_REG_P1_3               , RULL(0x80000D430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP1_REG_P1_3               , RULL(0x80000D430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP1_REG_P1_4               , RULL(0x800011430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP1_REG_P1_4               , RULL(0x800011430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP1_REG_P2_0               , RULL(0x800001430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP1_REG_P2_0               , RULL(0x800001430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP1_REG_P2_1               , RULL(0x800005430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP1_REG_P2_1               , RULL(0x800005430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP1_REG_P2_2               , RULL(0x800009430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP1_REG_P2_2               , RULL(0x800009430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP1_REG_P2_3               , RULL(0x80000D430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP1_REG_P2_3               , RULL(0x80000D430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP1_REG_P2_4               , RULL(0x800011430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP1_REG_P2_4               , RULL(0x800011430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP1_REG_P3_0               , RULL(0x8000014307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP1_REG_P3_0               , RULL(0x8000014308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP1_REG_P3_1               , RULL(0x8000054307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP1_REG_P3_1               , RULL(0x8000054308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP1_REG_P3_2               , RULL(0x8000094307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP1_REG_P3_2               , RULL(0x8000094308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP1_REG_P3_3               , RULL(0x80000D4307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP1_REG_P3_3               , RULL(0x80000D4308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP1_REG_P3_4               , RULL(0x8000114307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP1_REG_P3_4               , RULL(0x8000114308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP2_REG_P0_0                 , RULL(0x800002430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP2_REG_P0_0               , RULL(0x800002430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP2_REG_P0_0               , RULL(0x800002430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP2_REG_P0_1                 , RULL(0x800006430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP2_REG_P0_1               , RULL(0x800006430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP2_REG_P0_1               , RULL(0x800006430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP2_REG_P0_2                 , RULL(0x80000A430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP2_REG_P0_2               , RULL(0x80000A430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP2_REG_P0_2               , RULL(0x80000A430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP2_REG_P0_3                 , RULL(0x80000E430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP2_REG_P0_3               , RULL(0x80000E430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP2_REG_P0_3               , RULL(0x80000E430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP2_REG_P0_4                 , RULL(0x800012430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP2_REG_P0_4               , RULL(0x800012430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP2_REG_P0_4               , RULL(0x800012430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP2_REG_P1_0               , RULL(0x800002430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP2_REG_P1_0               , RULL(0x800002430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP2_REG_P1_1               , RULL(0x800006430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP2_REG_P1_1               , RULL(0x800006430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP2_REG_P1_2               , RULL(0x80000A430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP2_REG_P1_2               , RULL(0x80000A430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP2_REG_P1_3               , RULL(0x80000E430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP2_REG_P1_3               , RULL(0x80000E430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP2_REG_P1_4               , RULL(0x800012430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP2_REG_P1_4               , RULL(0x800012430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP2_REG_P2_0               , RULL(0x800002430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP2_REG_P2_0               , RULL(0x800002430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP2_REG_P2_1               , RULL(0x800006430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP2_REG_P2_1               , RULL(0x800006430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP2_REG_P2_2               , RULL(0x80000A430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP2_REG_P2_2               , RULL(0x80000A430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP2_REG_P2_3               , RULL(0x80000E430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP2_REG_P2_3               , RULL(0x80000E430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP2_REG_P2_4               , RULL(0x800012430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP2_REG_P2_4               , RULL(0x800012430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP2_REG_P3_0               , RULL(0x8000024307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP2_REG_P3_0               , RULL(0x8000024308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP2_REG_P3_1               , RULL(0x8000064307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP2_REG_P3_1               , RULL(0x8000064308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP2_REG_P3_2               , RULL(0x80000A4307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP2_REG_P3_2               , RULL(0x80000A4308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP2_REG_P3_3               , RULL(0x80000E4307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP2_REG_P3_3               , RULL(0x80000E4308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP2_REG_P3_4               , RULL(0x8000124307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP2_REG_P3_4               , RULL(0x8000124308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP3_REG_P0_0                 , RULL(0x800003430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP3_REG_P0_0               , RULL(0x800003430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP3_REG_P0_0               , RULL(0x800003430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP3_REG_P0_1                 , RULL(0x800007430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP3_REG_P0_1               , RULL(0x800007430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP3_REG_P0_1               , RULL(0x800007430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP3_REG_P0_2                 , RULL(0x80000B430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP3_REG_P0_2               , RULL(0x80000B430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP3_REG_P0_2               , RULL(0x80000B430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP3_REG_P0_3                 , RULL(0x80000F430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP3_REG_P0_3               , RULL(0x80000F430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP3_REG_P0_3               , RULL(0x80000F430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_11_RP3_REG_P0_4                 , RULL(0x800013430701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_11_RP3_REG_P0_4               , RULL(0x800013430701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_11_RP3_REG_P0_4               , RULL(0x800013430801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP3_REG_P1_0               , RULL(0x800003430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP3_REG_P1_0               , RULL(0x800003430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP3_REG_P1_1               , RULL(0x800007430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP3_REG_P1_1               , RULL(0x800007430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP3_REG_P1_2               , RULL(0x80000B430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP3_REG_P1_2               , RULL(0x80000B430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP3_REG_P1_3               , RULL(0x80000F430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP3_REG_P1_3               , RULL(0x80000F430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_11_RP3_REG_P1_4               , RULL(0x800013430701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_11_RP3_REG_P1_4               , RULL(0x800013430801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP3_REG_P2_0               , RULL(0x800003430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP3_REG_P2_0               , RULL(0x800003430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP3_REG_P2_1               , RULL(0x800007430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP3_REG_P2_1               , RULL(0x800007430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP3_REG_P2_2               , RULL(0x80000B430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP3_REG_P2_2               , RULL(0x80000B430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP3_REG_P2_3               , RULL(0x80000F430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP3_REG_P2_3               , RULL(0x80000F430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_11_RP3_REG_P2_4               , RULL(0x800013430701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_11_RP3_REG_P2_4               , RULL(0x800013430801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP3_REG_P3_0               , RULL(0x8000034307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP3_REG_P3_0               , RULL(0x8000034308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP3_REG_P3_1               , RULL(0x8000074307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP3_REG_P3_1               , RULL(0x8000074308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP3_REG_P3_2               , RULL(0x80000B4307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP3_REG_P3_2               , RULL(0x80000B4308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP3_REG_P3_3               , RULL(0x80000F4307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP3_REG_P3_3               , RULL(0x80000F4308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_11_RP3_REG_P3_4               , RULL(0x8000134307011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_11_RP3_REG_P3_4               , RULL(0x8000134308011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP0_REG_P0_0                 , RULL(0x800000440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP0_REG_P0_0               , RULL(0x800000440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP0_REG_P0_0               , RULL(0x800000440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP0_REG_P0_1                 , RULL(0x800004440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP0_REG_P0_1               , RULL(0x800004440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP0_REG_P0_1               , RULL(0x800004440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP0_REG_P0_2                 , RULL(0x800008440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP0_REG_P0_2               , RULL(0x800008440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP0_REG_P0_2               , RULL(0x800008440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP0_REG_P0_3                 , RULL(0x80000C440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP0_REG_P0_3               , RULL(0x80000C440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP0_REG_P0_3               , RULL(0x80000C440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP0_REG_P0_4                 , RULL(0x800010440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP0_REG_P0_4               , RULL(0x800010440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP0_REG_P0_4               , RULL(0x800010440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP0_REG_P1_0               , RULL(0x800000440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP0_REG_P1_0               , RULL(0x800000440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP0_REG_P1_1               , RULL(0x800004440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP0_REG_P1_1               , RULL(0x800004440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP0_REG_P1_2               , RULL(0x800008440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP0_REG_P1_2               , RULL(0x800008440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP0_REG_P1_3               , RULL(0x80000C440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP0_REG_P1_3               , RULL(0x80000C440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP0_REG_P1_4               , RULL(0x800010440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP0_REG_P1_4               , RULL(0x800010440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP0_REG_P2_0               , RULL(0x800000440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP0_REG_P2_0               , RULL(0x800000440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP0_REG_P2_1               , RULL(0x800004440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP0_REG_P2_1               , RULL(0x800004440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP0_REG_P2_2               , RULL(0x800008440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP0_REG_P2_2               , RULL(0x800008440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP0_REG_P2_3               , RULL(0x80000C440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP0_REG_P2_3               , RULL(0x80000C440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP0_REG_P2_4               , RULL(0x800010440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP0_REG_P2_4               , RULL(0x800010440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP0_REG_P3_0               , RULL(0x8000004407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP0_REG_P3_0               , RULL(0x8000004408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP0_REG_P3_1               , RULL(0x8000044407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP0_REG_P3_1               , RULL(0x8000044408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP0_REG_P3_2               , RULL(0x8000084407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP0_REG_P3_2               , RULL(0x8000084408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP0_REG_P3_3               , RULL(0x80000C4407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP0_REG_P3_3               , RULL(0x80000C4408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP0_REG_P3_4               , RULL(0x8000104407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP0_REG_P3_4               , RULL(0x8000104408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP1_REG_P0_0                 , RULL(0x800001440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP1_REG_P0_0               , RULL(0x800001440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP1_REG_P0_0               , RULL(0x800001440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP1_REG_P0_1                 , RULL(0x800005440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP1_REG_P0_1               , RULL(0x800005440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP1_REG_P0_1               , RULL(0x800005440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP1_REG_P0_2                 , RULL(0x800009440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP1_REG_P0_2               , RULL(0x800009440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP1_REG_P0_2               , RULL(0x800009440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP1_REG_P0_3                 , RULL(0x80000D440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP1_REG_P0_3               , RULL(0x80000D440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP1_REG_P0_3               , RULL(0x80000D440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP1_REG_P0_4                 , RULL(0x800011440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP1_REG_P0_4               , RULL(0x800011440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP1_REG_P0_4               , RULL(0x800011440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP1_REG_P1_0               , RULL(0x800001440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP1_REG_P1_0               , RULL(0x800001440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP1_REG_P1_1               , RULL(0x800005440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP1_REG_P1_1               , RULL(0x800005440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP1_REG_P1_2               , RULL(0x800009440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP1_REG_P1_2               , RULL(0x800009440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP1_REG_P1_3               , RULL(0x80000D440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP1_REG_P1_3               , RULL(0x80000D440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP1_REG_P1_4               , RULL(0x800011440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP1_REG_P1_4               , RULL(0x800011440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP1_REG_P2_0               , RULL(0x800001440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP1_REG_P2_0               , RULL(0x800001440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP1_REG_P2_1               , RULL(0x800005440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP1_REG_P2_1               , RULL(0x800005440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP1_REG_P2_2               , RULL(0x800009440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP1_REG_P2_2               , RULL(0x800009440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP1_REG_P2_3               , RULL(0x80000D440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP1_REG_P2_3               , RULL(0x80000D440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP1_REG_P2_4               , RULL(0x800011440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP1_REG_P2_4               , RULL(0x800011440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP1_REG_P3_0               , RULL(0x8000014407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP1_REG_P3_0               , RULL(0x8000014408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP1_REG_P3_1               , RULL(0x8000054407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP1_REG_P3_1               , RULL(0x8000054408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP1_REG_P3_2               , RULL(0x8000094407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP1_REG_P3_2               , RULL(0x8000094408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP1_REG_P3_3               , RULL(0x80000D4407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP1_REG_P3_3               , RULL(0x80000D4408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP1_REG_P3_4               , RULL(0x8000114407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP1_REG_P3_4               , RULL(0x8000114408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP2_REG_P0_0                 , RULL(0x800002440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP2_REG_P0_0               , RULL(0x800002440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP2_REG_P0_0               , RULL(0x800002440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP2_REG_P0_1                 , RULL(0x800006440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP2_REG_P0_1               , RULL(0x800006440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP2_REG_P0_1               , RULL(0x800006440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP2_REG_P0_2                 , RULL(0x80000A440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP2_REG_P0_2               , RULL(0x80000A440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP2_REG_P0_2               , RULL(0x80000A440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP2_REG_P0_3                 , RULL(0x80000E440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP2_REG_P0_3               , RULL(0x80000E440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP2_REG_P0_3               , RULL(0x80000E440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP2_REG_P0_4                 , RULL(0x800012440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP2_REG_P0_4               , RULL(0x800012440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP2_REG_P0_4               , RULL(0x800012440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP2_REG_P1_0               , RULL(0x800002440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP2_REG_P1_0               , RULL(0x800002440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP2_REG_P1_1               , RULL(0x800006440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP2_REG_P1_1               , RULL(0x800006440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP2_REG_P1_2               , RULL(0x80000A440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP2_REG_P1_2               , RULL(0x80000A440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP2_REG_P1_3               , RULL(0x80000E440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP2_REG_P1_3               , RULL(0x80000E440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP2_REG_P1_4               , RULL(0x800012440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP2_REG_P1_4               , RULL(0x800012440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP2_REG_P2_0               , RULL(0x800002440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP2_REG_P2_0               , RULL(0x800002440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP2_REG_P2_1               , RULL(0x800006440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP2_REG_P2_1               , RULL(0x800006440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP2_REG_P2_2               , RULL(0x80000A440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP2_REG_P2_2               , RULL(0x80000A440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP2_REG_P2_3               , RULL(0x80000E440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP2_REG_P2_3               , RULL(0x80000E440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP2_REG_P2_4               , RULL(0x800012440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP2_REG_P2_4               , RULL(0x800012440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP2_REG_P3_0               , RULL(0x8000024407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP2_REG_P3_0               , RULL(0x8000024408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP2_REG_P3_1               , RULL(0x8000064407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP2_REG_P3_1               , RULL(0x8000064408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP2_REG_P3_2               , RULL(0x80000A4407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP2_REG_P3_2               , RULL(0x80000A4408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP2_REG_P3_3               , RULL(0x80000E4407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP2_REG_P3_3               , RULL(0x80000E4408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP2_REG_P3_4               , RULL(0x8000124407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP2_REG_P3_4               , RULL(0x8000124408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP3_REG_P0_0                 , RULL(0x800003440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP3_REG_P0_0               , RULL(0x800003440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP3_REG_P0_0               , RULL(0x800003440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP3_REG_P0_1                 , RULL(0x800007440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP3_REG_P0_1               , RULL(0x800007440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP3_REG_P0_1               , RULL(0x800007440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP3_REG_P0_2                 , RULL(0x80000B440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP3_REG_P0_2               , RULL(0x80000B440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP3_REG_P0_2               , RULL(0x80000B440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP3_REG_P0_3                 , RULL(0x80000F440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP3_REG_P0_3               , RULL(0x80000F440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP3_REG_P0_3               , RULL(0x80000F440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_12_RP3_REG_P0_4                 , RULL(0x800013440701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_12_RP3_REG_P0_4               , RULL(0x800013440701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_12_RP3_REG_P0_4               , RULL(0x800013440801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP3_REG_P1_0               , RULL(0x800003440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP3_REG_P1_0               , RULL(0x800003440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP3_REG_P1_1               , RULL(0x800007440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP3_REG_P1_1               , RULL(0x800007440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP3_REG_P1_2               , RULL(0x80000B440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP3_REG_P1_2               , RULL(0x80000B440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP3_REG_P1_3               , RULL(0x80000F440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP3_REG_P1_3               , RULL(0x80000F440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_12_RP3_REG_P1_4               , RULL(0x800013440701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_12_RP3_REG_P1_4               , RULL(0x800013440801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP3_REG_P2_0               , RULL(0x800003440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP3_REG_P2_0               , RULL(0x800003440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP3_REG_P2_1               , RULL(0x800007440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP3_REG_P2_1               , RULL(0x800007440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP3_REG_P2_2               , RULL(0x80000B440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP3_REG_P2_2               , RULL(0x80000B440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP3_REG_P2_3               , RULL(0x80000F440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP3_REG_P2_3               , RULL(0x80000F440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_12_RP3_REG_P2_4               , RULL(0x800013440701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_12_RP3_REG_P2_4               , RULL(0x800013440801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP3_REG_P3_0               , RULL(0x8000034407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP3_REG_P3_0               , RULL(0x8000034408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP3_REG_P3_1               , RULL(0x8000074407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP3_REG_P3_1               , RULL(0x8000074408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP3_REG_P3_2               , RULL(0x80000B4407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP3_REG_P3_2               , RULL(0x80000B4408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP3_REG_P3_3               , RULL(0x80000F4407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP3_REG_P3_3               , RULL(0x80000F4408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_12_RP3_REG_P3_4               , RULL(0x8000134407011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_12_RP3_REG_P3_4               , RULL(0x8000134408011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP0_REG_P0_0                 , RULL(0x800000450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP0_REG_P0_0               , RULL(0x800000450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP0_REG_P0_0               , RULL(0x800000450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP0_REG_P0_1                 , RULL(0x800004450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP0_REG_P0_1               , RULL(0x800004450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP0_REG_P0_1               , RULL(0x800004450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP0_REG_P0_2                 , RULL(0x800008450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP0_REG_P0_2               , RULL(0x800008450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP0_REG_P0_2               , RULL(0x800008450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP0_REG_P0_3                 , RULL(0x80000C450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP0_REG_P0_3               , RULL(0x80000C450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP0_REG_P0_3               , RULL(0x80000C450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP0_REG_P0_4                 , RULL(0x800010450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP0_REG_P0_4               , RULL(0x800010450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP0_REG_P0_4               , RULL(0x800010450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP0_REG_P1_0               , RULL(0x800000450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP0_REG_P1_0               , RULL(0x800000450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP0_REG_P1_1               , RULL(0x800004450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP0_REG_P1_1               , RULL(0x800004450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP0_REG_P1_2               , RULL(0x800008450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP0_REG_P1_2               , RULL(0x800008450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP0_REG_P1_3               , RULL(0x80000C450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP0_REG_P1_3               , RULL(0x80000C450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP0_REG_P1_4               , RULL(0x800010450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP0_REG_P1_4               , RULL(0x800010450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP0_REG_P2_0               , RULL(0x800000450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP0_REG_P2_0               , RULL(0x800000450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP0_REG_P2_1               , RULL(0x800004450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP0_REG_P2_1               , RULL(0x800004450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP0_REG_P2_2               , RULL(0x800008450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP0_REG_P2_2               , RULL(0x800008450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP0_REG_P2_3               , RULL(0x80000C450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP0_REG_P2_3               , RULL(0x80000C450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP0_REG_P2_4               , RULL(0x800010450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP0_REG_P2_4               , RULL(0x800010450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP0_REG_P3_0               , RULL(0x8000004507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP0_REG_P3_0               , RULL(0x8000004508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP0_REG_P3_1               , RULL(0x8000044507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP0_REG_P3_1               , RULL(0x8000044508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP0_REG_P3_2               , RULL(0x8000084507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP0_REG_P3_2               , RULL(0x8000084508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP0_REG_P3_3               , RULL(0x80000C4507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP0_REG_P3_3               , RULL(0x80000C4508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP0_REG_P3_4               , RULL(0x8000104507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP0_REG_P3_4               , RULL(0x8000104508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP1_REG_P0_0                 , RULL(0x800001450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP1_REG_P0_0               , RULL(0x800001450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP1_REG_P0_0               , RULL(0x800001450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP1_REG_P0_1                 , RULL(0x800005450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP1_REG_P0_1               , RULL(0x800005450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP1_REG_P0_1               , RULL(0x800005450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP1_REG_P0_2                 , RULL(0x800009450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP1_REG_P0_2               , RULL(0x800009450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP1_REG_P0_2               , RULL(0x800009450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP1_REG_P0_3                 , RULL(0x80000D450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP1_REG_P0_3               , RULL(0x80000D450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP1_REG_P0_3               , RULL(0x80000D450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP1_REG_P0_4                 , RULL(0x800011450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP1_REG_P0_4               , RULL(0x800011450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP1_REG_P0_4               , RULL(0x800011450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP1_REG_P1_0               , RULL(0x800001450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP1_REG_P1_0               , RULL(0x800001450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP1_REG_P1_1               , RULL(0x800005450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP1_REG_P1_1               , RULL(0x800005450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP1_REG_P1_2               , RULL(0x800009450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP1_REG_P1_2               , RULL(0x800009450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP1_REG_P1_3               , RULL(0x80000D450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP1_REG_P1_3               , RULL(0x80000D450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP1_REG_P1_4               , RULL(0x800011450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP1_REG_P1_4               , RULL(0x800011450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP1_REG_P2_0               , RULL(0x800001450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP1_REG_P2_0               , RULL(0x800001450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP1_REG_P2_1               , RULL(0x800005450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP1_REG_P2_1               , RULL(0x800005450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP1_REG_P2_2               , RULL(0x800009450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP1_REG_P2_2               , RULL(0x800009450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP1_REG_P2_3               , RULL(0x80000D450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP1_REG_P2_3               , RULL(0x80000D450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP1_REG_P2_4               , RULL(0x800011450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP1_REG_P2_4               , RULL(0x800011450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP1_REG_P3_0               , RULL(0x8000014507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP1_REG_P3_0               , RULL(0x8000014508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP1_REG_P3_1               , RULL(0x8000054507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP1_REG_P3_1               , RULL(0x8000054508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP1_REG_P3_2               , RULL(0x8000094507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP1_REG_P3_2               , RULL(0x8000094508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP1_REG_P3_3               , RULL(0x80000D4507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP1_REG_P3_3               , RULL(0x80000D4508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP1_REG_P3_4               , RULL(0x8000114507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP1_REG_P3_4               , RULL(0x8000114508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP2_REG_P0_0                 , RULL(0x800002450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP2_REG_P0_0               , RULL(0x800002450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP2_REG_P0_0               , RULL(0x800002450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP2_REG_P0_1                 , RULL(0x800006450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP2_REG_P0_1               , RULL(0x800006450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP2_REG_P0_1               , RULL(0x800006450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP2_REG_P0_2                 , RULL(0x80000A450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP2_REG_P0_2               , RULL(0x80000A450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP2_REG_P0_2               , RULL(0x80000A450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP2_REG_P0_3                 , RULL(0x80000E450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP2_REG_P0_3               , RULL(0x80000E450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP2_REG_P0_3               , RULL(0x80000E450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP2_REG_P0_4                 , RULL(0x800012450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP2_REG_P0_4               , RULL(0x800012450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP2_REG_P0_4               , RULL(0x800012450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP2_REG_P1_0               , RULL(0x800002450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP2_REG_P1_0               , RULL(0x800002450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP2_REG_P1_1               , RULL(0x800006450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP2_REG_P1_1               , RULL(0x800006450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP2_REG_P1_2               , RULL(0x80000A450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP2_REG_P1_2               , RULL(0x80000A450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP2_REG_P1_3               , RULL(0x80000E450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP2_REG_P1_3               , RULL(0x80000E450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP2_REG_P1_4               , RULL(0x800012450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP2_REG_P1_4               , RULL(0x800012450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP2_REG_P2_0               , RULL(0x800002450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP2_REG_P2_0               , RULL(0x800002450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP2_REG_P2_1               , RULL(0x800006450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP2_REG_P2_1               , RULL(0x800006450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP2_REG_P2_2               , RULL(0x80000A450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP2_REG_P2_2               , RULL(0x80000A450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP2_REG_P2_3               , RULL(0x80000E450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP2_REG_P2_3               , RULL(0x80000E450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP2_REG_P2_4               , RULL(0x800012450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP2_REG_P2_4               , RULL(0x800012450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP2_REG_P3_0               , RULL(0x8000024507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP2_REG_P3_0               , RULL(0x8000024508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP2_REG_P3_1               , RULL(0x8000064507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP2_REG_P3_1               , RULL(0x8000064508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP2_REG_P3_2               , RULL(0x80000A4507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP2_REG_P3_2               , RULL(0x80000A4508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP2_REG_P3_3               , RULL(0x80000E4507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP2_REG_P3_3               , RULL(0x80000E4508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP2_REG_P3_4               , RULL(0x8000124507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP2_REG_P3_4               , RULL(0x8000124508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP3_REG_P0_0                 , RULL(0x800003450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP3_REG_P0_0               , RULL(0x800003450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP3_REG_P0_0               , RULL(0x800003450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP3_REG_P0_1                 , RULL(0x800007450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP3_REG_P0_1               , RULL(0x800007450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP3_REG_P0_1               , RULL(0x800007450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP3_REG_P0_2                 , RULL(0x80000B450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP3_REG_P0_2               , RULL(0x80000B450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP3_REG_P0_2               , RULL(0x80000B450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP3_REG_P0_3                 , RULL(0x80000F450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP3_REG_P0_3               , RULL(0x80000F450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP3_REG_P0_3               , RULL(0x80000F450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_13_RP3_REG_P0_4                 , RULL(0x800013450701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_13_RP3_REG_P0_4               , RULL(0x800013450701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_13_RP3_REG_P0_4               , RULL(0x800013450801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP3_REG_P1_0               , RULL(0x800003450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP3_REG_P1_0               , RULL(0x800003450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP3_REG_P1_1               , RULL(0x800007450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP3_REG_P1_1               , RULL(0x800007450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP3_REG_P1_2               , RULL(0x80000B450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP3_REG_P1_2               , RULL(0x80000B450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP3_REG_P1_3               , RULL(0x80000F450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP3_REG_P1_3               , RULL(0x80000F450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_13_RP3_REG_P1_4               , RULL(0x800013450701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_13_RP3_REG_P1_4               , RULL(0x800013450801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP3_REG_P2_0               , RULL(0x800003450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP3_REG_P2_0               , RULL(0x800003450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP3_REG_P2_1               , RULL(0x800007450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP3_REG_P2_1               , RULL(0x800007450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP3_REG_P2_2               , RULL(0x80000B450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP3_REG_P2_2               , RULL(0x80000B450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP3_REG_P2_3               , RULL(0x80000F450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP3_REG_P2_3               , RULL(0x80000F450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_13_RP3_REG_P2_4               , RULL(0x800013450701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_13_RP3_REG_P2_4               , RULL(0x800013450801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP3_REG_P3_0               , RULL(0x8000034507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP3_REG_P3_0               , RULL(0x8000034508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP3_REG_P3_1               , RULL(0x8000074507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP3_REG_P3_1               , RULL(0x8000074508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP3_REG_P3_2               , RULL(0x80000B4507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP3_REG_P3_2               , RULL(0x80000B4508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP3_REG_P3_3               , RULL(0x80000F4507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP3_REG_P3_3               , RULL(0x80000F4508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_13_RP3_REG_P3_4               , RULL(0x8000134507011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_13_RP3_REG_P3_4               , RULL(0x8000134508011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP0_REG_P0_0                 , RULL(0x800000460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP0_REG_P0_0               , RULL(0x800000460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP0_REG_P0_0               , RULL(0x800000460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP0_REG_P0_1                 , RULL(0x800004460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP0_REG_P0_1               , RULL(0x800004460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP0_REG_P0_1               , RULL(0x800004460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP0_REG_P0_2                 , RULL(0x800008460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP0_REG_P0_2               , RULL(0x800008460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP0_REG_P0_2               , RULL(0x800008460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP0_REG_P0_3                 , RULL(0x80000C460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP0_REG_P0_3               , RULL(0x80000C460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP0_REG_P0_3               , RULL(0x80000C460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP0_REG_P0_4                 , RULL(0x800010460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP0_REG_P0_4               , RULL(0x800010460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP0_REG_P0_4               , RULL(0x800010460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP0_REG_P1_0               , RULL(0x800000460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP0_REG_P1_0               , RULL(0x800000460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP0_REG_P1_1               , RULL(0x800004460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP0_REG_P1_1               , RULL(0x800004460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP0_REG_P1_2               , RULL(0x800008460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP0_REG_P1_2               , RULL(0x800008460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP0_REG_P1_3               , RULL(0x80000C460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP0_REG_P1_3               , RULL(0x80000C460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP0_REG_P1_4               , RULL(0x800010460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP0_REG_P1_4               , RULL(0x800010460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP0_REG_P2_0               , RULL(0x800000460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP0_REG_P2_0               , RULL(0x800000460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP0_REG_P2_1               , RULL(0x800004460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP0_REG_P2_1               , RULL(0x800004460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP0_REG_P2_2               , RULL(0x800008460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP0_REG_P2_2               , RULL(0x800008460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP0_REG_P2_3               , RULL(0x80000C460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP0_REG_P2_3               , RULL(0x80000C460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP0_REG_P2_4               , RULL(0x800010460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP0_REG_P2_4               , RULL(0x800010460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP0_REG_P3_0               , RULL(0x8000004607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP0_REG_P3_0               , RULL(0x8000004608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP0_REG_P3_1               , RULL(0x8000044607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP0_REG_P3_1               , RULL(0x8000044608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP0_REG_P3_2               , RULL(0x8000084607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP0_REG_P3_2               , RULL(0x8000084608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP0_REG_P3_3               , RULL(0x80000C4607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP0_REG_P3_3               , RULL(0x80000C4608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP0_REG_P3_4               , RULL(0x8000104607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP0_REG_P3_4               , RULL(0x8000104608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP1_REG_P0_0                 , RULL(0x800001460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP1_REG_P0_0               , RULL(0x800001460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP1_REG_P0_0               , RULL(0x800001460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP1_REG_P0_1                 , RULL(0x800005460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP1_REG_P0_1               , RULL(0x800005460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP1_REG_P0_1               , RULL(0x800005460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP1_REG_P0_2                 , RULL(0x800009460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP1_REG_P0_2               , RULL(0x800009460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP1_REG_P0_2               , RULL(0x800009460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP1_REG_P0_3                 , RULL(0x80000D460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP1_REG_P0_3               , RULL(0x80000D460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP1_REG_P0_3               , RULL(0x80000D460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP1_REG_P0_4                 , RULL(0x800011460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP1_REG_P0_4               , RULL(0x800011460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP1_REG_P0_4               , RULL(0x800011460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP1_REG_P1_0               , RULL(0x800001460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP1_REG_P1_0               , RULL(0x800001460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP1_REG_P1_1               , RULL(0x800005460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP1_REG_P1_1               , RULL(0x800005460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP1_REG_P1_2               , RULL(0x800009460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP1_REG_P1_2               , RULL(0x800009460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP1_REG_P1_3               , RULL(0x80000D460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP1_REG_P1_3               , RULL(0x80000D460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP1_REG_P1_4               , RULL(0x800011460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP1_REG_P1_4               , RULL(0x800011460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP1_REG_P2_0               , RULL(0x800001460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP1_REG_P2_0               , RULL(0x800001460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP1_REG_P2_1               , RULL(0x800005460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP1_REG_P2_1               , RULL(0x800005460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP1_REG_P2_2               , RULL(0x800009460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP1_REG_P2_2               , RULL(0x800009460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP1_REG_P2_3               , RULL(0x80000D460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP1_REG_P2_3               , RULL(0x80000D460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP1_REG_P2_4               , RULL(0x800011460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP1_REG_P2_4               , RULL(0x800011460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP1_REG_P3_0               , RULL(0x8000014607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP1_REG_P3_0               , RULL(0x8000014608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP1_REG_P3_1               , RULL(0x8000054607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP1_REG_P3_1               , RULL(0x8000054608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP1_REG_P3_2               , RULL(0x8000094607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP1_REG_P3_2               , RULL(0x8000094608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP1_REG_P3_3               , RULL(0x80000D4607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP1_REG_P3_3               , RULL(0x80000D4608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP1_REG_P3_4               , RULL(0x8000114607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP1_REG_P3_4               , RULL(0x8000114608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP2_REG_P0_0                 , RULL(0x800002460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP2_REG_P0_0               , RULL(0x800002460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP2_REG_P0_0               , RULL(0x800002460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP2_REG_P0_1                 , RULL(0x800006460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP2_REG_P0_1               , RULL(0x800006460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP2_REG_P0_1               , RULL(0x800006460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP2_REG_P0_2                 , RULL(0x80000A460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP2_REG_P0_2               , RULL(0x80000A460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP2_REG_P0_2               , RULL(0x80000A460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP2_REG_P0_3                 , RULL(0x80000E460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP2_REG_P0_3               , RULL(0x80000E460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP2_REG_P0_3               , RULL(0x80000E460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP2_REG_P0_4                 , RULL(0x800012460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP2_REG_P0_4               , RULL(0x800012460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP2_REG_P0_4               , RULL(0x800012460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP2_REG_P1_0               , RULL(0x800002460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP2_REG_P1_0               , RULL(0x800002460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP2_REG_P1_1               , RULL(0x800006460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP2_REG_P1_1               , RULL(0x800006460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP2_REG_P1_2               , RULL(0x80000A460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP2_REG_P1_2               , RULL(0x80000A460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP2_REG_P1_3               , RULL(0x80000E460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP2_REG_P1_3               , RULL(0x80000E460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP2_REG_P1_4               , RULL(0x800012460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP2_REG_P1_4               , RULL(0x800012460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP2_REG_P2_0               , RULL(0x800002460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP2_REG_P2_0               , RULL(0x800002460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP2_REG_P2_1               , RULL(0x800006460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP2_REG_P2_1               , RULL(0x800006460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP2_REG_P2_2               , RULL(0x80000A460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP2_REG_P2_2               , RULL(0x80000A460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP2_REG_P2_3               , RULL(0x80000E460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP2_REG_P2_3               , RULL(0x80000E460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP2_REG_P2_4               , RULL(0x800012460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP2_REG_P2_4               , RULL(0x800012460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP2_REG_P3_0               , RULL(0x8000024607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP2_REG_P3_0               , RULL(0x8000024608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP2_REG_P3_1               , RULL(0x8000064607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP2_REG_P3_1               , RULL(0x8000064608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP2_REG_P3_2               , RULL(0x80000A4607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP2_REG_P3_2               , RULL(0x80000A4608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP2_REG_P3_3               , RULL(0x80000E4607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP2_REG_P3_3               , RULL(0x80000E4608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP2_REG_P3_4               , RULL(0x8000124607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP2_REG_P3_4               , RULL(0x8000124608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP3_REG_P0_0                 , RULL(0x800003460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP3_REG_P0_0               , RULL(0x800003460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP3_REG_P0_0               , RULL(0x800003460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP3_REG_P0_1                 , RULL(0x800007460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP3_REG_P0_1               , RULL(0x800007460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP3_REG_P0_1               , RULL(0x800007460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP3_REG_P0_2                 , RULL(0x80000B460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP3_REG_P0_2               , RULL(0x80000B460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP3_REG_P0_2               , RULL(0x80000B460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP3_REG_P0_3                 , RULL(0x80000F460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP3_REG_P0_3               , RULL(0x80000F460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP3_REG_P0_3               , RULL(0x80000F460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_14_RP3_REG_P0_4                 , RULL(0x800013460701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_14_RP3_REG_P0_4               , RULL(0x800013460701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_14_RP3_REG_P0_4               , RULL(0x800013460801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP3_REG_P1_0               , RULL(0x800003460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP3_REG_P1_0               , RULL(0x800003460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP3_REG_P1_1               , RULL(0x800007460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP3_REG_P1_1               , RULL(0x800007460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP3_REG_P1_2               , RULL(0x80000B460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP3_REG_P1_2               , RULL(0x80000B460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP3_REG_P1_3               , RULL(0x80000F460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP3_REG_P1_3               , RULL(0x80000F460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_14_RP3_REG_P1_4               , RULL(0x800013460701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_14_RP3_REG_P1_4               , RULL(0x800013460801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP3_REG_P2_0               , RULL(0x800003460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP3_REG_P2_0               , RULL(0x800003460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP3_REG_P2_1               , RULL(0x800007460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP3_REG_P2_1               , RULL(0x800007460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP3_REG_P2_2               , RULL(0x80000B460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP3_REG_P2_2               , RULL(0x80000B460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP3_REG_P2_3               , RULL(0x80000F460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP3_REG_P2_3               , RULL(0x80000F460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_14_RP3_REG_P2_4               , RULL(0x800013460701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_14_RP3_REG_P2_4               , RULL(0x800013460801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP3_REG_P3_0               , RULL(0x8000034607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP3_REG_P3_0               , RULL(0x8000034608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP3_REG_P3_1               , RULL(0x8000074607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP3_REG_P3_1               , RULL(0x8000074608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP3_REG_P3_2               , RULL(0x80000B4607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP3_REG_P3_2               , RULL(0x80000B4608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP3_REG_P3_3               , RULL(0x80000F4607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP3_REG_P3_3               , RULL(0x80000F4608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_14_RP3_REG_P3_4               , RULL(0x8000134607011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_14_RP3_REG_P3_4               , RULL(0x8000134608011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP0_REG_P0_0                 , RULL(0x800000470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP0_REG_P0_0               , RULL(0x800000470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP0_REG_P0_0               , RULL(0x800000470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP0_REG_P0_1                 , RULL(0x800004470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP0_REG_P0_1               , RULL(0x800004470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP0_REG_P0_1               , RULL(0x800004470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP0_REG_P0_2                 , RULL(0x800008470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP0_REG_P0_2               , RULL(0x800008470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP0_REG_P0_2               , RULL(0x800008470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP0_REG_P0_3                 , RULL(0x80000C470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP0_REG_P0_3               , RULL(0x80000C470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP0_REG_P0_3               , RULL(0x80000C470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP0_REG_P0_4                 , RULL(0x800010470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP0_REG_P0_4               , RULL(0x800010470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP0_REG_P0_4               , RULL(0x800010470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP0_REG_P1_0               , RULL(0x800000470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP0_REG_P1_0               , RULL(0x800000470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP0_REG_P1_1               , RULL(0x800004470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP0_REG_P1_1               , RULL(0x800004470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP0_REG_P1_2               , RULL(0x800008470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP0_REG_P1_2               , RULL(0x800008470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP0_REG_P1_3               , RULL(0x80000C470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP0_REG_P1_3               , RULL(0x80000C470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP0_REG_P1_4               , RULL(0x800010470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP0_REG_P1_4               , RULL(0x800010470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP0_REG_P2_0               , RULL(0x800000470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP0_REG_P2_0               , RULL(0x800000470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP0_REG_P2_1               , RULL(0x800004470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP0_REG_P2_1               , RULL(0x800004470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP0_REG_P2_2               , RULL(0x800008470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP0_REG_P2_2               , RULL(0x800008470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP0_REG_P2_3               , RULL(0x80000C470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP0_REG_P2_3               , RULL(0x80000C470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP0_REG_P2_4               , RULL(0x800010470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP0_REG_P2_4               , RULL(0x800010470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP0_REG_P3_0               , RULL(0x8000004707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP0_REG_P3_0               , RULL(0x8000004708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP0_REG_P3_1               , RULL(0x8000044707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP0_REG_P3_1               , RULL(0x8000044708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP0_REG_P3_2               , RULL(0x8000084707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP0_REG_P3_2               , RULL(0x8000084708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP0_REG_P3_3               , RULL(0x80000C4707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP0_REG_P3_3               , RULL(0x80000C4708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP0_REG_P3_4               , RULL(0x8000104707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP0_REG_P3_4               , RULL(0x8000104708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP1_REG_P0_0                 , RULL(0x800001470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP1_REG_P0_0               , RULL(0x800001470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP1_REG_P0_0               , RULL(0x800001470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP1_REG_P0_1                 , RULL(0x800005470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP1_REG_P0_1               , RULL(0x800005470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP1_REG_P0_1               , RULL(0x800005470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP1_REG_P0_2                 , RULL(0x800009470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP1_REG_P0_2               , RULL(0x800009470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP1_REG_P0_2               , RULL(0x800009470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP1_REG_P0_3                 , RULL(0x80000D470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP1_REG_P0_3               , RULL(0x80000D470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP1_REG_P0_3               , RULL(0x80000D470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP1_REG_P0_4                 , RULL(0x800011470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP1_REG_P0_4               , RULL(0x800011470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP1_REG_P0_4               , RULL(0x800011470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP1_REG_P1_0               , RULL(0x800001470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP1_REG_P1_0               , RULL(0x800001470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP1_REG_P1_1               , RULL(0x800005470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP1_REG_P1_1               , RULL(0x800005470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP1_REG_P1_2               , RULL(0x800009470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP1_REG_P1_2               , RULL(0x800009470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP1_REG_P1_3               , RULL(0x80000D470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP1_REG_P1_3               , RULL(0x80000D470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP1_REG_P1_4               , RULL(0x800011470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP1_REG_P1_4               , RULL(0x800011470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP1_REG_P2_0               , RULL(0x800001470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP1_REG_P2_0               , RULL(0x800001470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP1_REG_P2_1               , RULL(0x800005470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP1_REG_P2_1               , RULL(0x800005470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP1_REG_P2_2               , RULL(0x800009470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP1_REG_P2_2               , RULL(0x800009470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP1_REG_P2_3               , RULL(0x80000D470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP1_REG_P2_3               , RULL(0x80000D470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP1_REG_P2_4               , RULL(0x800011470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP1_REG_P2_4               , RULL(0x800011470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP1_REG_P3_0               , RULL(0x8000014707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP1_REG_P3_0               , RULL(0x8000014708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP1_REG_P3_1               , RULL(0x8000054707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP1_REG_P3_1               , RULL(0x8000054708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP1_REG_P3_2               , RULL(0x8000094707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP1_REG_P3_2               , RULL(0x8000094708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP1_REG_P3_3               , RULL(0x80000D4707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP1_REG_P3_3               , RULL(0x80000D4708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP1_REG_P3_4               , RULL(0x8000114707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP1_REG_P3_4               , RULL(0x8000114708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP2_REG_P0_0                 , RULL(0x800002470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP2_REG_P0_0               , RULL(0x800002470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP2_REG_P0_0               , RULL(0x800002470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP2_REG_P0_1                 , RULL(0x800006470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP2_REG_P0_1               , RULL(0x800006470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP2_REG_P0_1               , RULL(0x800006470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP2_REG_P0_2                 , RULL(0x80000A470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP2_REG_P0_2               , RULL(0x80000A470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP2_REG_P0_2               , RULL(0x80000A470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP2_REG_P0_3                 , RULL(0x80000E470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP2_REG_P0_3               , RULL(0x80000E470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP2_REG_P0_3               , RULL(0x80000E470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP2_REG_P0_4                 , RULL(0x800012470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP2_REG_P0_4               , RULL(0x800012470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP2_REG_P0_4               , RULL(0x800012470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP2_REG_P1_0               , RULL(0x800002470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP2_REG_P1_0               , RULL(0x800002470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP2_REG_P1_1               , RULL(0x800006470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP2_REG_P1_1               , RULL(0x800006470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP2_REG_P1_2               , RULL(0x80000A470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP2_REG_P1_2               , RULL(0x80000A470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP2_REG_P1_3               , RULL(0x80000E470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP2_REG_P1_3               , RULL(0x80000E470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP2_REG_P1_4               , RULL(0x800012470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP2_REG_P1_4               , RULL(0x800012470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP2_REG_P2_0               , RULL(0x800002470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP2_REG_P2_0               , RULL(0x800002470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP2_REG_P2_1               , RULL(0x800006470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP2_REG_P2_1               , RULL(0x800006470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP2_REG_P2_2               , RULL(0x80000A470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP2_REG_P2_2               , RULL(0x80000A470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP2_REG_P2_3               , RULL(0x80000E470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP2_REG_P2_3               , RULL(0x80000E470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP2_REG_P2_4               , RULL(0x800012470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP2_REG_P2_4               , RULL(0x800012470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP2_REG_P3_0               , RULL(0x8000024707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP2_REG_P3_0               , RULL(0x8000024708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP2_REG_P3_1               , RULL(0x8000064707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP2_REG_P3_1               , RULL(0x8000064708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP2_REG_P3_2               , RULL(0x80000A4707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP2_REG_P3_2               , RULL(0x80000A4708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP2_REG_P3_3               , RULL(0x80000E4707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP2_REG_P3_3               , RULL(0x80000E4708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP2_REG_P3_4               , RULL(0x8000124707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP2_REG_P3_4               , RULL(0x8000124708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP3_REG_P0_0                 , RULL(0x800003470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP3_REG_P0_0               , RULL(0x800003470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP3_REG_P0_0               , RULL(0x800003470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP3_REG_P0_1                 , RULL(0x800007470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP3_REG_P0_1               , RULL(0x800007470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP3_REG_P0_1               , RULL(0x800007470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP3_REG_P0_2                 , RULL(0x80000B470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP3_REG_P0_2               , RULL(0x80000B470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP3_REG_P0_2               , RULL(0x80000B470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP3_REG_P0_3                 , RULL(0x80000F470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP3_REG_P0_3               , RULL(0x80000F470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP3_REG_P0_3               , RULL(0x80000F470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_15_RP3_REG_P0_4                 , RULL(0x800013470701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_15_RP3_REG_P0_4               , RULL(0x800013470701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_15_RP3_REG_P0_4               , RULL(0x800013470801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP3_REG_P1_0               , RULL(0x800003470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP3_REG_P1_0               , RULL(0x800003470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP3_REG_P1_1               , RULL(0x800007470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP3_REG_P1_1               , RULL(0x800007470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP3_REG_P1_2               , RULL(0x80000B470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP3_REG_P1_2               , RULL(0x80000B470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP3_REG_P1_3               , RULL(0x80000F470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP3_REG_P1_3               , RULL(0x80000F470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_15_RP3_REG_P1_4               , RULL(0x800013470701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_15_RP3_REG_P1_4               , RULL(0x800013470801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP3_REG_P2_0               , RULL(0x800003470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP3_REG_P2_0               , RULL(0x800003470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP3_REG_P2_1               , RULL(0x800007470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP3_REG_P2_1               , RULL(0x800007470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP3_REG_P2_2               , RULL(0x80000B470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP3_REG_P2_2               , RULL(0x80000B470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP3_REG_P2_3               , RULL(0x80000F470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP3_REG_P2_3               , RULL(0x80000F470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_15_RP3_REG_P2_4               , RULL(0x800013470701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_15_RP3_REG_P2_4               , RULL(0x800013470801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP3_REG_P3_0               , RULL(0x8000034707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP3_REG_P3_0               , RULL(0x8000034708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP3_REG_P3_1               , RULL(0x8000074707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP3_REG_P3_1               , RULL(0x8000074708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP3_REG_P3_2               , RULL(0x80000B4707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP3_REG_P3_2               , RULL(0x80000B4708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP3_REG_P3_3               , RULL(0x80000F4707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP3_REG_P3_3               , RULL(0x80000F4708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_15_RP3_REG_P3_4               , RULL(0x8000134707011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_15_RP3_REG_P3_4               , RULL(0x8000134708011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP0_REG_P0_0                 , RULL(0x800000480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP0_REG_P0_0               , RULL(0x800000480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP0_REG_P0_0               , RULL(0x800000480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP0_REG_P0_1                 , RULL(0x800004480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP0_REG_P0_1               , RULL(0x800004480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP0_REG_P0_1               , RULL(0x800004480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP0_REG_P0_2                 , RULL(0x800008480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP0_REG_P0_2               , RULL(0x800008480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP0_REG_P0_2               , RULL(0x800008480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP0_REG_P0_3                 , RULL(0x80000C480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP0_REG_P0_3               , RULL(0x80000C480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP0_REG_P0_3               , RULL(0x80000C480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP0_REG_P0_4                 , RULL(0x800010480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP0_REG_P0_4               , RULL(0x800010480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP0_REG_P0_4               , RULL(0x800010480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP0_REG_P1_0               , RULL(0x800000480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP0_REG_P1_0               , RULL(0x800000480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP0_REG_P1_1               , RULL(0x800004480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP0_REG_P1_1               , RULL(0x800004480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP0_REG_P1_2               , RULL(0x800008480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP0_REG_P1_2               , RULL(0x800008480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP0_REG_P1_3               , RULL(0x80000C480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP0_REG_P1_3               , RULL(0x80000C480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP0_REG_P1_4               , RULL(0x800010480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP0_REG_P1_4               , RULL(0x800010480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP0_REG_P2_0               , RULL(0x800000480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP0_REG_P2_0               , RULL(0x800000480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP0_REG_P2_1               , RULL(0x800004480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP0_REG_P2_1               , RULL(0x800004480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP0_REG_P2_2               , RULL(0x800008480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP0_REG_P2_2               , RULL(0x800008480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP0_REG_P2_3               , RULL(0x80000C480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP0_REG_P2_3               , RULL(0x80000C480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP0_REG_P2_4               , RULL(0x800010480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP0_REG_P2_4               , RULL(0x800010480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP0_REG_P3_0               , RULL(0x8000004807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP0_REG_P3_0               , RULL(0x8000004808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP0_REG_P3_1               , RULL(0x8000044807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP0_REG_P3_1               , RULL(0x8000044808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP0_REG_P3_2               , RULL(0x8000084807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP0_REG_P3_2               , RULL(0x8000084808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP0_REG_P3_3               , RULL(0x80000C4807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP0_REG_P3_3               , RULL(0x80000C4808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP0_REG_P3_4               , RULL(0x8000104807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP0_REG_P3_4               , RULL(0x8000104808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP1_REG_P0_0                 , RULL(0x800001480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP1_REG_P0_0               , RULL(0x800001480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP1_REG_P0_0               , RULL(0x800001480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP1_REG_P0_1                 , RULL(0x800005480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP1_REG_P0_1               , RULL(0x800005480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP1_REG_P0_1               , RULL(0x800005480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP1_REG_P0_2                 , RULL(0x800009480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP1_REG_P0_2               , RULL(0x800009480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP1_REG_P0_2               , RULL(0x800009480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP1_REG_P0_3                 , RULL(0x80000D480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP1_REG_P0_3               , RULL(0x80000D480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP1_REG_P0_3               , RULL(0x80000D480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP1_REG_P0_4                 , RULL(0x800011480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP1_REG_P0_4               , RULL(0x800011480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP1_REG_P0_4               , RULL(0x800011480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP1_REG_P1_0               , RULL(0x800001480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP1_REG_P1_0               , RULL(0x800001480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP1_REG_P1_1               , RULL(0x800005480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP1_REG_P1_1               , RULL(0x800005480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP1_REG_P1_2               , RULL(0x800009480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP1_REG_P1_2               , RULL(0x800009480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP1_REG_P1_3               , RULL(0x80000D480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP1_REG_P1_3               , RULL(0x80000D480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP1_REG_P1_4               , RULL(0x800011480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP1_REG_P1_4               , RULL(0x800011480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP1_REG_P2_0               , RULL(0x800001480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP1_REG_P2_0               , RULL(0x800001480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP1_REG_P2_1               , RULL(0x800005480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP1_REG_P2_1               , RULL(0x800005480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP1_REG_P2_2               , RULL(0x800009480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP1_REG_P2_2               , RULL(0x800009480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP1_REG_P2_3               , RULL(0x80000D480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP1_REG_P2_3               , RULL(0x80000D480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP1_REG_P2_4               , RULL(0x800011480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP1_REG_P2_4               , RULL(0x800011480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP1_REG_P3_0               , RULL(0x8000014807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP1_REG_P3_0               , RULL(0x8000014808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP1_REG_P3_1               , RULL(0x8000054807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP1_REG_P3_1               , RULL(0x8000054808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP1_REG_P3_2               , RULL(0x8000094807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP1_REG_P3_2               , RULL(0x8000094808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP1_REG_P3_3               , RULL(0x80000D4807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP1_REG_P3_3               , RULL(0x80000D4808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP1_REG_P3_4               , RULL(0x8000114807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP1_REG_P3_4               , RULL(0x8000114808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP2_REG_P0_0                 , RULL(0x800002480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP2_REG_P0_0               , RULL(0x800002480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP2_REG_P0_0               , RULL(0x800002480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP2_REG_P0_1                 , RULL(0x800006480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP2_REG_P0_1               , RULL(0x800006480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP2_REG_P0_1               , RULL(0x800006480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP2_REG_P0_2                 , RULL(0x80000A480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP2_REG_P0_2               , RULL(0x80000A480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP2_REG_P0_2               , RULL(0x80000A480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP2_REG_P0_3                 , RULL(0x80000E480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP2_REG_P0_3               , RULL(0x80000E480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP2_REG_P0_3               , RULL(0x80000E480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP2_REG_P0_4                 , RULL(0x800012480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP2_REG_P0_4               , RULL(0x800012480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP2_REG_P0_4               , RULL(0x800012480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP2_REG_P1_0               , RULL(0x800002480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP2_REG_P1_0               , RULL(0x800002480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP2_REG_P1_1               , RULL(0x800006480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP2_REG_P1_1               , RULL(0x800006480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP2_REG_P1_2               , RULL(0x80000A480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP2_REG_P1_2               , RULL(0x80000A480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP2_REG_P1_3               , RULL(0x80000E480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP2_REG_P1_3               , RULL(0x80000E480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP2_REG_P1_4               , RULL(0x800012480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP2_REG_P1_4               , RULL(0x800012480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP2_REG_P2_0               , RULL(0x800002480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP2_REG_P2_0               , RULL(0x800002480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP2_REG_P2_1               , RULL(0x800006480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP2_REG_P2_1               , RULL(0x800006480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP2_REG_P2_2               , RULL(0x80000A480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP2_REG_P2_2               , RULL(0x80000A480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP2_REG_P2_3               , RULL(0x80000E480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP2_REG_P2_3               , RULL(0x80000E480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP2_REG_P2_4               , RULL(0x800012480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP2_REG_P2_4               , RULL(0x800012480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP2_REG_P3_0               , RULL(0x8000024807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP2_REG_P3_0               , RULL(0x8000024808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP2_REG_P3_1               , RULL(0x8000064807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP2_REG_P3_1               , RULL(0x8000064808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP2_REG_P3_2               , RULL(0x80000A4807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP2_REG_P3_2               , RULL(0x80000A4808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP2_REG_P3_3               , RULL(0x80000E4807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP2_REG_P3_3               , RULL(0x80000E4808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP2_REG_P3_4               , RULL(0x8000124807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP2_REG_P3_4               , RULL(0x8000124808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP3_REG_P0_0                 , RULL(0x800003480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP3_REG_P0_0               , RULL(0x800003480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP3_REG_P0_0               , RULL(0x800003480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP3_REG_P0_1                 , RULL(0x800007480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP3_REG_P0_1               , RULL(0x800007480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP3_REG_P0_1               , RULL(0x800007480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP3_REG_P0_2                 , RULL(0x80000B480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP3_REG_P0_2               , RULL(0x80000B480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP3_REG_P0_2               , RULL(0x80000B480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP3_REG_P0_3                 , RULL(0x80000F480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP3_REG_P0_3               , RULL(0x80000F480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP3_REG_P0_3               , RULL(0x80000F480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_16_RP3_REG_P0_4                 , RULL(0x800013480701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_16_RP3_REG_P0_4               , RULL(0x800013480701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_16_RP3_REG_P0_4               , RULL(0x800013480801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP3_REG_P1_0               , RULL(0x800003480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP3_REG_P1_0               , RULL(0x800003480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP3_REG_P1_1               , RULL(0x800007480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP3_REG_P1_1               , RULL(0x800007480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP3_REG_P1_2               , RULL(0x80000B480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP3_REG_P1_2               , RULL(0x80000B480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP3_REG_P1_3               , RULL(0x80000F480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP3_REG_P1_3               , RULL(0x80000F480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_16_RP3_REG_P1_4               , RULL(0x800013480701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_16_RP3_REG_P1_4               , RULL(0x800013480801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP3_REG_P2_0               , RULL(0x800003480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP3_REG_P2_0               , RULL(0x800003480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP3_REG_P2_1               , RULL(0x800007480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP3_REG_P2_1               , RULL(0x800007480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP3_REG_P2_2               , RULL(0x80000B480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP3_REG_P2_2               , RULL(0x80000B480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP3_REG_P2_3               , RULL(0x80000F480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP3_REG_P2_3               , RULL(0x80000F480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_16_RP3_REG_P2_4               , RULL(0x800013480701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_16_RP3_REG_P2_4               , RULL(0x800013480801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP3_REG_P3_0               , RULL(0x8000034807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP3_REG_P3_0               , RULL(0x8000034808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP3_REG_P3_1               , RULL(0x8000074807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP3_REG_P3_1               , RULL(0x8000074808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP3_REG_P3_2               , RULL(0x80000B4807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP3_REG_P3_2               , RULL(0x80000B4808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP3_REG_P3_3               , RULL(0x80000F4807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP3_REG_P3_3               , RULL(0x80000F4808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_16_RP3_REG_P3_4               , RULL(0x8000134807011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_16_RP3_REG_P3_4               , RULL(0x8000134808011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP0_REG_P0_0                 , RULL(0x8000004A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP0_REG_P0_0               , RULL(0x8000004A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP0_REG_P0_0               , RULL(0x8000004A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP0_REG_P0_1                 , RULL(0x8000044A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP0_REG_P0_1               , RULL(0x8000044A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP0_REG_P0_1               , RULL(0x8000044A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP0_REG_P0_2                 , RULL(0x8000084A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP0_REG_P0_2               , RULL(0x8000084A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP0_REG_P0_2               , RULL(0x8000084A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP0_REG_P0_3                 , RULL(0x80000C4A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP0_REG_P0_3               , RULL(0x80000C4A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP0_REG_P0_3               , RULL(0x80000C4A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP0_REG_P0_4                 , RULL(0x8000104A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP0_REG_P0_4               , RULL(0x8000104A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP0_REG_P0_4               , RULL(0x8000104A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP0_REG_P1_0               , RULL(0x8000004A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP0_REG_P1_0               , RULL(0x8000004A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP0_REG_P1_1               , RULL(0x8000044A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP0_REG_P1_1               , RULL(0x8000044A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP0_REG_P1_2               , RULL(0x8000084A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP0_REG_P1_2               , RULL(0x8000084A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP0_REG_P1_3               , RULL(0x80000C4A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP0_REG_P1_3               , RULL(0x80000C4A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP0_REG_P1_4               , RULL(0x8000104A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP0_REG_P1_4               , RULL(0x8000104A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP0_REG_P2_0               , RULL(0x8000004A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP0_REG_P2_0               , RULL(0x8000004A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP0_REG_P2_1               , RULL(0x8000044A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP0_REG_P2_1               , RULL(0x8000044A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP0_REG_P2_2               , RULL(0x8000084A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP0_REG_P2_2               , RULL(0x8000084A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP0_REG_P2_3               , RULL(0x80000C4A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP0_REG_P2_3               , RULL(0x80000C4A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP0_REG_P2_4               , RULL(0x8000104A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP0_REG_P2_4               , RULL(0x8000104A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP0_REG_P3_0               , RULL(0x8000004A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP0_REG_P3_0               , RULL(0x8000004A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP0_REG_P3_1               , RULL(0x8000044A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP0_REG_P3_1               , RULL(0x8000044A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP0_REG_P3_2               , RULL(0x8000084A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP0_REG_P3_2               , RULL(0x8000084A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP0_REG_P3_3               , RULL(0x80000C4A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP0_REG_P3_3               , RULL(0x80000C4A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP0_REG_P3_4               , RULL(0x8000104A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP0_REG_P3_4               , RULL(0x8000104A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP1_REG_P0_0                 , RULL(0x8000014A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP1_REG_P0_0               , RULL(0x8000014A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP1_REG_P0_0               , RULL(0x8000014A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP1_REG_P0_1                 , RULL(0x8000054A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP1_REG_P0_1               , RULL(0x8000054A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP1_REG_P0_1               , RULL(0x8000054A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP1_REG_P0_2                 , RULL(0x8000094A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP1_REG_P0_2               , RULL(0x8000094A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP1_REG_P0_2               , RULL(0x8000094A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP1_REG_P0_3                 , RULL(0x80000D4A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP1_REG_P0_3               , RULL(0x80000D4A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP1_REG_P0_3               , RULL(0x80000D4A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP1_REG_P0_4                 , RULL(0x8000114A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP1_REG_P0_4               , RULL(0x8000114A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP1_REG_P0_4               , RULL(0x8000114A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP1_REG_P1_0               , RULL(0x8000014A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP1_REG_P1_0               , RULL(0x8000014A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP1_REG_P1_1               , RULL(0x8000054A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP1_REG_P1_1               , RULL(0x8000054A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP1_REG_P1_2               , RULL(0x8000094A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP1_REG_P1_2               , RULL(0x8000094A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP1_REG_P1_3               , RULL(0x80000D4A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP1_REG_P1_3               , RULL(0x80000D4A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP1_REG_P1_4               , RULL(0x8000114A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP1_REG_P1_4               , RULL(0x8000114A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP1_REG_P2_0               , RULL(0x8000014A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP1_REG_P2_0               , RULL(0x8000014A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP1_REG_P2_1               , RULL(0x8000054A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP1_REG_P2_1               , RULL(0x8000054A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP1_REG_P2_2               , RULL(0x8000094A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP1_REG_P2_2               , RULL(0x8000094A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP1_REG_P2_3               , RULL(0x80000D4A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP1_REG_P2_3               , RULL(0x80000D4A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP1_REG_P2_4               , RULL(0x8000114A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP1_REG_P2_4               , RULL(0x8000114A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP1_REG_P3_0               , RULL(0x8000014A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP1_REG_P3_0               , RULL(0x8000014A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP1_REG_P3_1               , RULL(0x8000054A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP1_REG_P3_1               , RULL(0x8000054A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP1_REG_P3_2               , RULL(0x8000094A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP1_REG_P3_2               , RULL(0x8000094A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP1_REG_P3_3               , RULL(0x80000D4A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP1_REG_P3_3               , RULL(0x80000D4A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP1_REG_P3_4               , RULL(0x8000114A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP1_REG_P3_4               , RULL(0x8000114A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP2_REG_P0_0                 , RULL(0x8000024A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP2_REG_P0_0               , RULL(0x8000024A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP2_REG_P0_0               , RULL(0x8000024A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP2_REG_P0_1                 , RULL(0x8000064A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP2_REG_P0_1               , RULL(0x8000064A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP2_REG_P0_1               , RULL(0x8000064A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP2_REG_P0_2                 , RULL(0x80000A4A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP2_REG_P0_2               , RULL(0x80000A4A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP2_REG_P0_2               , RULL(0x80000A4A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP2_REG_P0_3                 , RULL(0x80000E4A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP2_REG_P0_3               , RULL(0x80000E4A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP2_REG_P0_3               , RULL(0x80000E4A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP2_REG_P0_4                 , RULL(0x8000124A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP2_REG_P0_4               , RULL(0x8000124A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP2_REG_P0_4               , RULL(0x8000124A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP2_REG_P1_0               , RULL(0x8000024A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP2_REG_P1_0               , RULL(0x8000024A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP2_REG_P1_1               , RULL(0x8000064A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP2_REG_P1_1               , RULL(0x8000064A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP2_REG_P1_2               , RULL(0x80000A4A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP2_REG_P1_2               , RULL(0x80000A4A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP2_REG_P1_3               , RULL(0x80000E4A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP2_REG_P1_3               , RULL(0x80000E4A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP2_REG_P1_4               , RULL(0x8000124A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP2_REG_P1_4               , RULL(0x8000124A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP2_REG_P2_0               , RULL(0x8000024A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP2_REG_P2_0               , RULL(0x8000024A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP2_REG_P2_1               , RULL(0x8000064A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP2_REG_P2_1               , RULL(0x8000064A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP2_REG_P2_2               , RULL(0x80000A4A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP2_REG_P2_2               , RULL(0x80000A4A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP2_REG_P2_3               , RULL(0x80000E4A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP2_REG_P2_3               , RULL(0x80000E4A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP2_REG_P2_4               , RULL(0x8000124A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP2_REG_P2_4               , RULL(0x8000124A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP2_REG_P3_0               , RULL(0x8000024A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP2_REG_P3_0               , RULL(0x8000024A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP2_REG_P3_1               , RULL(0x8000064A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP2_REG_P3_1               , RULL(0x8000064A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP2_REG_P3_2               , RULL(0x80000A4A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP2_REG_P3_2               , RULL(0x80000A4A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP2_REG_P3_3               , RULL(0x80000E4A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP2_REG_P3_3               , RULL(0x80000E4A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP2_REG_P3_4               , RULL(0x8000124A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP2_REG_P3_4               , RULL(0x8000124A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP3_REG_P0_0                 , RULL(0x8000034A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP3_REG_P0_0               , RULL(0x8000034A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP3_REG_P0_0               , RULL(0x8000034A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP3_REG_P0_1                 , RULL(0x8000074A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP3_REG_P0_1               , RULL(0x8000074A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP3_REG_P0_1               , RULL(0x8000074A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP3_REG_P0_2                 , RULL(0x80000B4A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP3_REG_P0_2               , RULL(0x80000B4A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP3_REG_P0_2               , RULL(0x80000B4A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP3_REG_P0_3                 , RULL(0x80000F4A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP3_REG_P0_3               , RULL(0x80000F4A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP3_REG_P0_3               , RULL(0x80000F4A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_18_RP3_REG_P0_4                 , RULL(0x8000134A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_18_RP3_REG_P0_4               , RULL(0x8000134A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_18_RP3_REG_P0_4               , RULL(0x8000134A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP3_REG_P1_0               , RULL(0x8000034A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP3_REG_P1_0               , RULL(0x8000034A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP3_REG_P1_1               , RULL(0x8000074A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP3_REG_P1_1               , RULL(0x8000074A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP3_REG_P1_2               , RULL(0x80000B4A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP3_REG_P1_2               , RULL(0x80000B4A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP3_REG_P1_3               , RULL(0x80000F4A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP3_REG_P1_3               , RULL(0x80000F4A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_18_RP3_REG_P1_4               , RULL(0x8000134A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_18_RP3_REG_P1_4               , RULL(0x8000134A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP3_REG_P2_0               , RULL(0x8000034A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP3_REG_P2_0               , RULL(0x8000034A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP3_REG_P2_1               , RULL(0x8000074A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP3_REG_P2_1               , RULL(0x8000074A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP3_REG_P2_2               , RULL(0x80000B4A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP3_REG_P2_2               , RULL(0x80000B4A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP3_REG_P2_3               , RULL(0x80000F4A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP3_REG_P2_3               , RULL(0x80000F4A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_18_RP3_REG_P2_4               , RULL(0x8000134A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_18_RP3_REG_P2_4               , RULL(0x8000134A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP3_REG_P3_0               , RULL(0x8000034A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP3_REG_P3_0               , RULL(0x8000034A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP3_REG_P3_1               , RULL(0x8000074A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP3_REG_P3_1               , RULL(0x8000074A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP3_REG_P3_2               , RULL(0x80000B4A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP3_REG_P3_2               , RULL(0x80000B4A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP3_REG_P3_3               , RULL(0x80000F4A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP3_REG_P3_3               , RULL(0x80000F4A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_18_RP3_REG_P3_4               , RULL(0x8000134A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_18_RP3_REG_P3_4               , RULL(0x8000134A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP0_REG_P0_0                  , RULL(0x800000390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP0_REG_P0_0                , RULL(0x800000390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP0_REG_P0_0                , RULL(0x800000390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP0_REG_P0_1                  , RULL(0x800004390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP0_REG_P0_1                , RULL(0x800004390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP0_REG_P0_1                , RULL(0x800004390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP0_REG_P0_2                  , RULL(0x800008390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP0_REG_P0_2                , RULL(0x800008390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP0_REG_P0_2                , RULL(0x800008390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP0_REG_P0_3                  , RULL(0x80000C390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP0_REG_P0_3                , RULL(0x80000C390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP0_REG_P0_3                , RULL(0x80000C390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP0_REG_P0_4                  , RULL(0x800010390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP0_REG_P0_4                , RULL(0x800010390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP0_REG_P0_4                , RULL(0x800010390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP0_REG_P1_0                , RULL(0x800000390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP0_REG_P1_0                , RULL(0x800000390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP0_REG_P1_1                , RULL(0x800004390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP0_REG_P1_1                , RULL(0x800004390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP0_REG_P1_2                , RULL(0x800008390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP0_REG_P1_2                , RULL(0x800008390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP0_REG_P1_3                , RULL(0x80000C390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP0_REG_P1_3                , RULL(0x80000C390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP0_REG_P1_4                , RULL(0x800010390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP0_REG_P1_4                , RULL(0x800010390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP0_REG_P2_0                , RULL(0x800000390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP0_REG_P2_0                , RULL(0x800000390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP0_REG_P2_1                , RULL(0x800004390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP0_REG_P2_1                , RULL(0x800004390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP0_REG_P2_2                , RULL(0x800008390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP0_REG_P2_2                , RULL(0x800008390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP0_REG_P2_3                , RULL(0x80000C390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP0_REG_P2_3                , RULL(0x80000C390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP0_REG_P2_4                , RULL(0x800010390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP0_REG_P2_4                , RULL(0x800010390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP0_REG_P3_0                , RULL(0x8000003907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP0_REG_P3_0                , RULL(0x8000003908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP0_REG_P3_1                , RULL(0x8000043907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP0_REG_P3_1                , RULL(0x8000043908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP0_REG_P3_2                , RULL(0x8000083907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP0_REG_P3_2                , RULL(0x8000083908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP0_REG_P3_3                , RULL(0x80000C3907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP0_REG_P3_3                , RULL(0x80000C3908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP0_REG_P3_4                , RULL(0x8000103907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP0_REG_P3_4                , RULL(0x8000103908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP1_REG_P0_0                  , RULL(0x800001390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP1_REG_P0_0                , RULL(0x800001390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP1_REG_P0_0                , RULL(0x800001390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP1_REG_P0_1                  , RULL(0x800005390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP1_REG_P0_1                , RULL(0x800005390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP1_REG_P0_1                , RULL(0x800005390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP1_REG_P0_2                  , RULL(0x800009390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP1_REG_P0_2                , RULL(0x800009390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP1_REG_P0_2                , RULL(0x800009390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP1_REG_P0_3                  , RULL(0x80000D390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP1_REG_P0_3                , RULL(0x80000D390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP1_REG_P0_3                , RULL(0x80000D390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP1_REG_P0_4                  , RULL(0x800011390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP1_REG_P0_4                , RULL(0x800011390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP1_REG_P0_4                , RULL(0x800011390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP1_REG_P1_0                , RULL(0x800001390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP1_REG_P1_0                , RULL(0x800001390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP1_REG_P1_1                , RULL(0x800005390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP1_REG_P1_1                , RULL(0x800005390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP1_REG_P1_2                , RULL(0x800009390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP1_REG_P1_2                , RULL(0x800009390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP1_REG_P1_3                , RULL(0x80000D390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP1_REG_P1_3                , RULL(0x80000D390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP1_REG_P1_4                , RULL(0x800011390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP1_REG_P1_4                , RULL(0x800011390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP1_REG_P2_0                , RULL(0x800001390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP1_REG_P2_0                , RULL(0x800001390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP1_REG_P2_1                , RULL(0x800005390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP1_REG_P2_1                , RULL(0x800005390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP1_REG_P2_2                , RULL(0x800009390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP1_REG_P2_2                , RULL(0x800009390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP1_REG_P2_3                , RULL(0x80000D390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP1_REG_P2_3                , RULL(0x80000D390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP1_REG_P2_4                , RULL(0x800011390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP1_REG_P2_4                , RULL(0x800011390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP1_REG_P3_0                , RULL(0x8000013907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP1_REG_P3_0                , RULL(0x8000013908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP1_REG_P3_1                , RULL(0x8000053907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP1_REG_P3_1                , RULL(0x8000053908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP1_REG_P3_2                , RULL(0x8000093907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP1_REG_P3_2                , RULL(0x8000093908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP1_REG_P3_3                , RULL(0x80000D3907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP1_REG_P3_3                , RULL(0x80000D3908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP1_REG_P3_4                , RULL(0x8000113907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP1_REG_P3_4                , RULL(0x8000113908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP2_REG_P0_0                  , RULL(0x800002390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP2_REG_P0_0                , RULL(0x800002390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP2_REG_P0_0                , RULL(0x800002390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP2_REG_P0_1                  , RULL(0x800006390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP2_REG_P0_1                , RULL(0x800006390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP2_REG_P0_1                , RULL(0x800006390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP2_REG_P0_2                  , RULL(0x80000A390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP2_REG_P0_2                , RULL(0x80000A390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP2_REG_P0_2                , RULL(0x80000A390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP2_REG_P0_3                  , RULL(0x80000E390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP2_REG_P0_3                , RULL(0x80000E390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP2_REG_P0_3                , RULL(0x80000E390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP2_REG_P0_4                  , RULL(0x800012390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP2_REG_P0_4                , RULL(0x800012390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP2_REG_P0_4                , RULL(0x800012390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP2_REG_P1_0                , RULL(0x800002390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP2_REG_P1_0                , RULL(0x800002390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP2_REG_P1_1                , RULL(0x800006390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP2_REG_P1_1                , RULL(0x800006390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP2_REG_P1_2                , RULL(0x80000A390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP2_REG_P1_2                , RULL(0x80000A390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP2_REG_P1_3                , RULL(0x80000E390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP2_REG_P1_3                , RULL(0x80000E390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP2_REG_P1_4                , RULL(0x800012390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP2_REG_P1_4                , RULL(0x800012390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP2_REG_P2_0                , RULL(0x800002390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP2_REG_P2_0                , RULL(0x800002390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP2_REG_P2_1                , RULL(0x800006390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP2_REG_P2_1                , RULL(0x800006390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP2_REG_P2_2                , RULL(0x80000A390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP2_REG_P2_2                , RULL(0x80000A390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP2_REG_P2_3                , RULL(0x80000E390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP2_REG_P2_3                , RULL(0x80000E390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP2_REG_P2_4                , RULL(0x800012390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP2_REG_P2_4                , RULL(0x800012390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP2_REG_P3_0                , RULL(0x8000023907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP2_REG_P3_0                , RULL(0x8000023908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP2_REG_P3_1                , RULL(0x8000063907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP2_REG_P3_1                , RULL(0x8000063908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP2_REG_P3_2                , RULL(0x80000A3907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP2_REG_P3_2                , RULL(0x80000A3908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP2_REG_P3_3                , RULL(0x80000E3907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP2_REG_P3_3                , RULL(0x80000E3908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP2_REG_P3_4                , RULL(0x8000123907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP2_REG_P3_4                , RULL(0x8000123908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP3_REG_P0_0                  , RULL(0x800003390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP3_REG_P0_0                , RULL(0x800003390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP3_REG_P0_0                , RULL(0x800003390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP3_REG_P0_1                  , RULL(0x800007390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP3_REG_P0_1                , RULL(0x800007390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP3_REG_P0_1                , RULL(0x800007390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP3_REG_P0_2                  , RULL(0x80000B390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP3_REG_P0_2                , RULL(0x80000B390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP3_REG_P0_2                , RULL(0x80000B390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP3_REG_P0_3                  , RULL(0x80000F390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP3_REG_P0_3                , RULL(0x80000F390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP3_REG_P0_3                , RULL(0x80000F390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_1_RP3_REG_P0_4                  , RULL(0x800013390701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_1_RP3_REG_P0_4                , RULL(0x800013390701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_1_RP3_REG_P0_4                , RULL(0x800013390801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP3_REG_P1_0                , RULL(0x800003390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP3_REG_P1_0                , RULL(0x800003390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP3_REG_P1_1                , RULL(0x800007390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP3_REG_P1_1                , RULL(0x800007390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP3_REG_P1_2                , RULL(0x80000B390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP3_REG_P1_2                , RULL(0x80000B390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP3_REG_P1_3                , RULL(0x80000F390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP3_REG_P1_3                , RULL(0x80000F390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_1_RP3_REG_P1_4                , RULL(0x800013390701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_1_RP3_REG_P1_4                , RULL(0x800013390801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP3_REG_P2_0                , RULL(0x800003390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP3_REG_P2_0                , RULL(0x800003390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP3_REG_P2_1                , RULL(0x800007390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP3_REG_P2_1                , RULL(0x800007390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP3_REG_P2_2                , RULL(0x80000B390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP3_REG_P2_2                , RULL(0x80000B390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP3_REG_P2_3                , RULL(0x80000F390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP3_REG_P2_3                , RULL(0x80000F390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_1_RP3_REG_P2_4                , RULL(0x800013390701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_1_RP3_REG_P2_4                , RULL(0x800013390801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP3_REG_P3_0                , RULL(0x8000033907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP3_REG_P3_0                , RULL(0x8000033908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP3_REG_P3_1                , RULL(0x8000073907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP3_REG_P3_1                , RULL(0x8000073908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP3_REG_P3_2                , RULL(0x80000B3907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP3_REG_P3_2                , RULL(0x80000B3908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP3_REG_P3_3                , RULL(0x80000F3907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP3_REG_P3_3                , RULL(0x80000F3908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_1_RP3_REG_P3_4                , RULL(0x8000133907011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_1_RP3_REG_P3_4                , RULL(0x8000133908011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP0_REG_P0_0                 , RULL(0x8000004C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP0_REG_P0_0               , RULL(0x8000004C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP0_REG_P0_0               , RULL(0x8000004C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP0_REG_P0_1                 , RULL(0x8000044C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP0_REG_P0_1               , RULL(0x8000044C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP0_REG_P0_1               , RULL(0x8000044C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP0_REG_P0_2                 , RULL(0x8000084C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP0_REG_P0_2               , RULL(0x8000084C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP0_REG_P0_2               , RULL(0x8000084C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP0_REG_P0_3                 , RULL(0x80000C4C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP0_REG_P0_3               , RULL(0x80000C4C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP0_REG_P0_3               , RULL(0x80000C4C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP0_REG_P0_4                 , RULL(0x8000104C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP0_REG_P0_4               , RULL(0x8000104C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP0_REG_P0_4               , RULL(0x8000104C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP0_REG_P1_0               , RULL(0x8000004C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP0_REG_P1_0               , RULL(0x8000004C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP0_REG_P1_1               , RULL(0x8000044C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP0_REG_P1_1               , RULL(0x8000044C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP0_REG_P1_2               , RULL(0x8000084C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP0_REG_P1_2               , RULL(0x8000084C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP0_REG_P1_3               , RULL(0x80000C4C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP0_REG_P1_3               , RULL(0x80000C4C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP0_REG_P1_4               , RULL(0x8000104C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP0_REG_P1_4               , RULL(0x8000104C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP0_REG_P2_0               , RULL(0x8000004C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP0_REG_P2_0               , RULL(0x8000004C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP0_REG_P2_1               , RULL(0x8000044C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP0_REG_P2_1               , RULL(0x8000044C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP0_REG_P2_2               , RULL(0x8000084C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP0_REG_P2_2               , RULL(0x8000084C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP0_REG_P2_3               , RULL(0x80000C4C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP0_REG_P2_3               , RULL(0x80000C4C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP0_REG_P2_4               , RULL(0x8000104C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP0_REG_P2_4               , RULL(0x8000104C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP0_REG_P3_0               , RULL(0x8000004C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP0_REG_P3_0               , RULL(0x8000004C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP0_REG_P3_1               , RULL(0x8000044C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP0_REG_P3_1               , RULL(0x8000044C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP0_REG_P3_2               , RULL(0x8000084C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP0_REG_P3_2               , RULL(0x8000084C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP0_REG_P3_3               , RULL(0x80000C4C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP0_REG_P3_3               , RULL(0x80000C4C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP0_REG_P3_4               , RULL(0x8000104C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP0_REG_P3_4               , RULL(0x8000104C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP1_REG_P0_0                 , RULL(0x8000014C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP1_REG_P0_0               , RULL(0x8000014C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP1_REG_P0_0               , RULL(0x8000014C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP1_REG_P0_1                 , RULL(0x8000054C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP1_REG_P0_1               , RULL(0x8000054C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP1_REG_P0_1               , RULL(0x8000054C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP1_REG_P0_2                 , RULL(0x8000094C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP1_REG_P0_2               , RULL(0x8000094C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP1_REG_P0_2               , RULL(0x8000094C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP1_REG_P0_3                 , RULL(0x80000D4C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP1_REG_P0_3               , RULL(0x80000D4C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP1_REG_P0_3               , RULL(0x80000D4C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP1_REG_P0_4                 , RULL(0x8000114C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP1_REG_P0_4               , RULL(0x8000114C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP1_REG_P0_4               , RULL(0x8000114C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP1_REG_P1_0               , RULL(0x8000014C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP1_REG_P1_0               , RULL(0x8000014C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP1_REG_P1_1               , RULL(0x8000054C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP1_REG_P1_1               , RULL(0x8000054C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP1_REG_P1_2               , RULL(0x8000094C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP1_REG_P1_2               , RULL(0x8000094C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP1_REG_P1_3               , RULL(0x80000D4C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP1_REG_P1_3               , RULL(0x80000D4C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP1_REG_P1_4               , RULL(0x8000114C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP1_REG_P1_4               , RULL(0x8000114C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP1_REG_P2_0               , RULL(0x8000014C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP1_REG_P2_0               , RULL(0x8000014C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP1_REG_P2_1               , RULL(0x8000054C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP1_REG_P2_1               , RULL(0x8000054C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP1_REG_P2_2               , RULL(0x8000094C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP1_REG_P2_2               , RULL(0x8000094C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP1_REG_P2_3               , RULL(0x80000D4C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP1_REG_P2_3               , RULL(0x80000D4C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP1_REG_P2_4               , RULL(0x8000114C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP1_REG_P2_4               , RULL(0x8000114C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP1_REG_P3_0               , RULL(0x8000014C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP1_REG_P3_0               , RULL(0x8000014C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP1_REG_P3_1               , RULL(0x8000054C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP1_REG_P3_1               , RULL(0x8000054C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP1_REG_P3_2               , RULL(0x8000094C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP1_REG_P3_2               , RULL(0x8000094C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP1_REG_P3_3               , RULL(0x80000D4C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP1_REG_P3_3               , RULL(0x80000D4C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP1_REG_P3_4               , RULL(0x8000114C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP1_REG_P3_4               , RULL(0x8000114C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP2_REG_P0_0                 , RULL(0x8000024C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP2_REG_P0_0               , RULL(0x8000024C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP2_REG_P0_0               , RULL(0x8000024C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP2_REG_P0_1                 , RULL(0x8000064C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP2_REG_P0_1               , RULL(0x8000064C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP2_REG_P0_1               , RULL(0x8000064C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP2_REG_P0_2                 , RULL(0x80000A4C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP2_REG_P0_2               , RULL(0x80000A4C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP2_REG_P0_2               , RULL(0x80000A4C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP2_REG_P0_3                 , RULL(0x80000E4C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP2_REG_P0_3               , RULL(0x80000E4C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP2_REG_P0_3               , RULL(0x80000E4C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP2_REG_P0_4                 , RULL(0x8000124C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP2_REG_P0_4               , RULL(0x8000124C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP2_REG_P0_4               , RULL(0x8000124C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP2_REG_P1_0               , RULL(0x8000024C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP2_REG_P1_0               , RULL(0x8000024C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP2_REG_P1_1               , RULL(0x8000064C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP2_REG_P1_1               , RULL(0x8000064C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP2_REG_P1_2               , RULL(0x80000A4C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP2_REG_P1_2               , RULL(0x80000A4C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP2_REG_P1_3               , RULL(0x80000E4C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP2_REG_P1_3               , RULL(0x80000E4C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP2_REG_P1_4               , RULL(0x8000124C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP2_REG_P1_4               , RULL(0x8000124C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP2_REG_P2_0               , RULL(0x8000024C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP2_REG_P2_0               , RULL(0x8000024C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP2_REG_P2_1               , RULL(0x8000064C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP2_REG_P2_1               , RULL(0x8000064C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP2_REG_P2_2               , RULL(0x80000A4C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP2_REG_P2_2               , RULL(0x80000A4C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP2_REG_P2_3               , RULL(0x80000E4C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP2_REG_P2_3               , RULL(0x80000E4C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP2_REG_P2_4               , RULL(0x8000124C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP2_REG_P2_4               , RULL(0x8000124C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP2_REG_P3_0               , RULL(0x8000024C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP2_REG_P3_0               , RULL(0x8000024C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP2_REG_P3_1               , RULL(0x8000064C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP2_REG_P3_1               , RULL(0x8000064C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP2_REG_P3_2               , RULL(0x80000A4C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP2_REG_P3_2               , RULL(0x80000A4C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP2_REG_P3_3               , RULL(0x80000E4C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP2_REG_P3_3               , RULL(0x80000E4C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP2_REG_P3_4               , RULL(0x8000124C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP2_REG_P3_4               , RULL(0x8000124C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP3_REG_P0_0                 , RULL(0x8000034C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP3_REG_P0_0               , RULL(0x8000034C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP3_REG_P0_0               , RULL(0x8000034C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP3_REG_P0_1                 , RULL(0x8000074C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP3_REG_P0_1               , RULL(0x8000074C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP3_REG_P0_1               , RULL(0x8000074C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP3_REG_P0_2                 , RULL(0x80000B4C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP3_REG_P0_2               , RULL(0x80000B4C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP3_REG_P0_2               , RULL(0x80000B4C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP3_REG_P0_3                 , RULL(0x80000F4C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP3_REG_P0_3               , RULL(0x80000F4C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP3_REG_P0_3               , RULL(0x80000F4C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_20_RP3_REG_P0_4                 , RULL(0x8000134C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_20_RP3_REG_P0_4               , RULL(0x8000134C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_20_RP3_REG_P0_4               , RULL(0x8000134C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP3_REG_P1_0               , RULL(0x8000034C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP3_REG_P1_0               , RULL(0x8000034C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP3_REG_P1_1               , RULL(0x8000074C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP3_REG_P1_1               , RULL(0x8000074C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP3_REG_P1_2               , RULL(0x80000B4C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP3_REG_P1_2               , RULL(0x80000B4C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP3_REG_P1_3               , RULL(0x80000F4C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP3_REG_P1_3               , RULL(0x80000F4C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_20_RP3_REG_P1_4               , RULL(0x8000134C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_20_RP3_REG_P1_4               , RULL(0x8000134C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP3_REG_P2_0               , RULL(0x8000034C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP3_REG_P2_0               , RULL(0x8000034C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP3_REG_P2_1               , RULL(0x8000074C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP3_REG_P2_1               , RULL(0x8000074C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP3_REG_P2_2               , RULL(0x80000B4C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP3_REG_P2_2               , RULL(0x80000B4C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP3_REG_P2_3               , RULL(0x80000F4C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP3_REG_P2_3               , RULL(0x80000F4C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_20_RP3_REG_P2_4               , RULL(0x8000134C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_20_RP3_REG_P2_4               , RULL(0x8000134C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP3_REG_P3_0               , RULL(0x8000034C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP3_REG_P3_0               , RULL(0x8000034C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP3_REG_P3_1               , RULL(0x8000074C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP3_REG_P3_1               , RULL(0x8000074C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP3_REG_P3_2               , RULL(0x80000B4C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP3_REG_P3_2               , RULL(0x80000B4C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP3_REG_P3_3               , RULL(0x80000F4C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP3_REG_P3_3               , RULL(0x80000F4C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_20_RP3_REG_P3_4               , RULL(0x8000134C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_20_RP3_REG_P3_4               , RULL(0x8000134C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP0_REG_P0_0                 , RULL(0x8000004E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP0_REG_P0_0               , RULL(0x8000004E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP0_REG_P0_0               , RULL(0x8000004E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP0_REG_P0_1                 , RULL(0x8000044E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP0_REG_P0_1               , RULL(0x8000044E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP0_REG_P0_1               , RULL(0x8000044E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP0_REG_P0_2                 , RULL(0x8000084E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP0_REG_P0_2               , RULL(0x8000084E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP0_REG_P0_2               , RULL(0x8000084E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP0_REG_P0_3                 , RULL(0x80000C4E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP0_REG_P0_3               , RULL(0x80000C4E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP0_REG_P0_3               , RULL(0x80000C4E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP0_REG_P0_4                 , RULL(0x8000104E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP0_REG_P0_4               , RULL(0x8000104E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP0_REG_P0_4               , RULL(0x8000104E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP0_REG_P1_0               , RULL(0x8000004E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP0_REG_P1_0               , RULL(0x8000004E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP0_REG_P1_1               , RULL(0x8000044E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP0_REG_P1_1               , RULL(0x8000044E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP0_REG_P1_2               , RULL(0x8000084E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP0_REG_P1_2               , RULL(0x8000084E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP0_REG_P1_3               , RULL(0x80000C4E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP0_REG_P1_3               , RULL(0x80000C4E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP0_REG_P1_4               , RULL(0x8000104E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP0_REG_P1_4               , RULL(0x8000104E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP0_REG_P2_0               , RULL(0x8000004E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP0_REG_P2_0               , RULL(0x8000004E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP0_REG_P2_1               , RULL(0x8000044E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP0_REG_P2_1               , RULL(0x8000044E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP0_REG_P2_2               , RULL(0x8000084E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP0_REG_P2_2               , RULL(0x8000084E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP0_REG_P2_3               , RULL(0x80000C4E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP0_REG_P2_3               , RULL(0x80000C4E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP0_REG_P2_4               , RULL(0x8000104E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP0_REG_P2_4               , RULL(0x8000104E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP0_REG_P3_0               , RULL(0x8000004E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP0_REG_P3_0               , RULL(0x8000004E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP0_REG_P3_1               , RULL(0x8000044E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP0_REG_P3_1               , RULL(0x8000044E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP0_REG_P3_2               , RULL(0x8000084E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP0_REG_P3_2               , RULL(0x8000084E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP0_REG_P3_3               , RULL(0x80000C4E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP0_REG_P3_3               , RULL(0x80000C4E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP0_REG_P3_4               , RULL(0x8000104E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP0_REG_P3_4               , RULL(0x8000104E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP1_REG_P0_0                 , RULL(0x8000014E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP1_REG_P0_0               , RULL(0x8000014E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP1_REG_P0_0               , RULL(0x8000014E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP1_REG_P0_1                 , RULL(0x8000054E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP1_REG_P0_1               , RULL(0x8000054E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP1_REG_P0_1               , RULL(0x8000054E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP1_REG_P0_2                 , RULL(0x8000094E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP1_REG_P0_2               , RULL(0x8000094E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP1_REG_P0_2               , RULL(0x8000094E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP1_REG_P0_3                 , RULL(0x80000D4E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP1_REG_P0_3               , RULL(0x80000D4E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP1_REG_P0_3               , RULL(0x80000D4E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP1_REG_P0_4                 , RULL(0x8000114E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP1_REG_P0_4               , RULL(0x8000114E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP1_REG_P0_4               , RULL(0x8000114E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP1_REG_P1_0               , RULL(0x8000014E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP1_REG_P1_0               , RULL(0x8000014E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP1_REG_P1_1               , RULL(0x8000054E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP1_REG_P1_1               , RULL(0x8000054E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP1_REG_P1_2               , RULL(0x8000094E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP1_REG_P1_2               , RULL(0x8000094E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP1_REG_P1_3               , RULL(0x80000D4E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP1_REG_P1_3               , RULL(0x80000D4E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP1_REG_P1_4               , RULL(0x8000114E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP1_REG_P1_4               , RULL(0x8000114E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP1_REG_P2_0               , RULL(0x8000014E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP1_REG_P2_0               , RULL(0x8000014E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP1_REG_P2_1               , RULL(0x8000054E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP1_REG_P2_1               , RULL(0x8000054E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP1_REG_P2_2               , RULL(0x8000094E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP1_REG_P2_2               , RULL(0x8000094E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP1_REG_P2_3               , RULL(0x80000D4E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP1_REG_P2_3               , RULL(0x80000D4E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP1_REG_P2_4               , RULL(0x8000114E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP1_REG_P2_4               , RULL(0x8000114E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP1_REG_P3_0               , RULL(0x8000014E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP1_REG_P3_0               , RULL(0x8000014E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP1_REG_P3_1               , RULL(0x8000054E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP1_REG_P3_1               , RULL(0x8000054E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP1_REG_P3_2               , RULL(0x8000094E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP1_REG_P3_2               , RULL(0x8000094E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP1_REG_P3_3               , RULL(0x80000D4E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP1_REG_P3_3               , RULL(0x80000D4E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP1_REG_P3_4               , RULL(0x8000114E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP1_REG_P3_4               , RULL(0x8000114E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP2_REG_P0_0                 , RULL(0x8000024E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP2_REG_P0_0               , RULL(0x8000024E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP2_REG_P0_0               , RULL(0x8000024E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP2_REG_P0_1                 , RULL(0x8000064E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP2_REG_P0_1               , RULL(0x8000064E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP2_REG_P0_1               , RULL(0x8000064E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP2_REG_P0_2                 , RULL(0x80000A4E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP2_REG_P0_2               , RULL(0x80000A4E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP2_REG_P0_2               , RULL(0x80000A4E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP2_REG_P0_3                 , RULL(0x80000E4E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP2_REG_P0_3               , RULL(0x80000E4E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP2_REG_P0_3               , RULL(0x80000E4E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP2_REG_P0_4                 , RULL(0x8000124E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP2_REG_P0_4               , RULL(0x8000124E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP2_REG_P0_4               , RULL(0x8000124E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP2_REG_P1_0               , RULL(0x8000024E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP2_REG_P1_0               , RULL(0x8000024E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP2_REG_P1_1               , RULL(0x8000064E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP2_REG_P1_1               , RULL(0x8000064E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP2_REG_P1_2               , RULL(0x80000A4E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP2_REG_P1_2               , RULL(0x80000A4E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP2_REG_P1_3               , RULL(0x80000E4E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP2_REG_P1_3               , RULL(0x80000E4E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP2_REG_P1_4               , RULL(0x8000124E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP2_REG_P1_4               , RULL(0x8000124E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP2_REG_P2_0               , RULL(0x8000024E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP2_REG_P2_0               , RULL(0x8000024E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP2_REG_P2_1               , RULL(0x8000064E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP2_REG_P2_1               , RULL(0x8000064E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP2_REG_P2_2               , RULL(0x80000A4E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP2_REG_P2_2               , RULL(0x80000A4E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP2_REG_P2_3               , RULL(0x80000E4E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP2_REG_P2_3               , RULL(0x80000E4E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP2_REG_P2_4               , RULL(0x8000124E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP2_REG_P2_4               , RULL(0x8000124E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP2_REG_P3_0               , RULL(0x8000024E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP2_REG_P3_0               , RULL(0x8000024E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP2_REG_P3_1               , RULL(0x8000064E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP2_REG_P3_1               , RULL(0x8000064E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP2_REG_P3_2               , RULL(0x80000A4E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP2_REG_P3_2               , RULL(0x80000A4E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP2_REG_P3_3               , RULL(0x80000E4E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP2_REG_P3_3               , RULL(0x80000E4E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP2_REG_P3_4               , RULL(0x8000124E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP2_REG_P3_4               , RULL(0x8000124E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP3_REG_P0_0                 , RULL(0x8000034E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP3_REG_P0_0               , RULL(0x8000034E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP3_REG_P0_0               , RULL(0x8000034E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP3_REG_P0_1                 , RULL(0x8000074E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP3_REG_P0_1               , RULL(0x8000074E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP3_REG_P0_1               , RULL(0x8000074E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP3_REG_P0_2                 , RULL(0x80000B4E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP3_REG_P0_2               , RULL(0x80000B4E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP3_REG_P0_2               , RULL(0x80000B4E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP3_REG_P0_3                 , RULL(0x80000F4E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP3_REG_P0_3               , RULL(0x80000F4E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP3_REG_P0_3               , RULL(0x80000F4E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_22_RP3_REG_P0_4                 , RULL(0x8000134E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_22_RP3_REG_P0_4               , RULL(0x8000134E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_22_RP3_REG_P0_4               , RULL(0x8000134E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP3_REG_P1_0               , RULL(0x8000034E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP3_REG_P1_0               , RULL(0x8000034E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP3_REG_P1_1               , RULL(0x8000074E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP3_REG_P1_1               , RULL(0x8000074E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP3_REG_P1_2               , RULL(0x80000B4E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP3_REG_P1_2               , RULL(0x80000B4E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP3_REG_P1_3               , RULL(0x80000F4E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP3_REG_P1_3               , RULL(0x80000F4E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_22_RP3_REG_P1_4               , RULL(0x8000134E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_22_RP3_REG_P1_4               , RULL(0x8000134E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP3_REG_P2_0               , RULL(0x8000034E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP3_REG_P2_0               , RULL(0x8000034E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP3_REG_P2_1               , RULL(0x8000074E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP3_REG_P2_1               , RULL(0x8000074E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP3_REG_P2_2               , RULL(0x80000B4E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP3_REG_P2_2               , RULL(0x80000B4E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP3_REG_P2_3               , RULL(0x80000F4E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP3_REG_P2_3               , RULL(0x80000F4E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_22_RP3_REG_P2_4               , RULL(0x8000134E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_22_RP3_REG_P2_4               , RULL(0x8000134E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP3_REG_P3_0               , RULL(0x8000034E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP3_REG_P3_0               , RULL(0x8000034E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP3_REG_P3_1               , RULL(0x8000074E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP3_REG_P3_1               , RULL(0x8000074E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP3_REG_P3_2               , RULL(0x80000B4E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP3_REG_P3_2               , RULL(0x80000B4E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP3_REG_P3_3               , RULL(0x80000F4E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP3_REG_P3_3               , RULL(0x80000F4E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_22_RP3_REG_P3_4               , RULL(0x8000134E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_22_RP3_REG_P3_4               , RULL(0x8000134E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP0_REG_P0_0                  , RULL(0x8000003A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP0_REG_P0_0                , RULL(0x8000003A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP0_REG_P0_0                , RULL(0x8000003A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP0_REG_P0_1                  , RULL(0x8000043A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP0_REG_P0_1                , RULL(0x8000043A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP0_REG_P0_1                , RULL(0x8000043A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP0_REG_P0_2                  , RULL(0x8000083A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP0_REG_P0_2                , RULL(0x8000083A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP0_REG_P0_2                , RULL(0x8000083A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP0_REG_P0_3                  , RULL(0x80000C3A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP0_REG_P0_3                , RULL(0x80000C3A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP0_REG_P0_3                , RULL(0x80000C3A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP0_REG_P0_4                  , RULL(0x8000103A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP0_REG_P0_4                , RULL(0x8000103A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP0_REG_P0_4                , RULL(0x8000103A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP0_REG_P1_0                , RULL(0x8000003A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP0_REG_P1_0                , RULL(0x8000003A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP0_REG_P1_1                , RULL(0x8000043A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP0_REG_P1_1                , RULL(0x8000043A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP0_REG_P1_2                , RULL(0x8000083A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP0_REG_P1_2                , RULL(0x8000083A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP0_REG_P1_3                , RULL(0x80000C3A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP0_REG_P1_3                , RULL(0x80000C3A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP0_REG_P1_4                , RULL(0x8000103A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP0_REG_P1_4                , RULL(0x8000103A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP0_REG_P2_0                , RULL(0x8000003A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP0_REG_P2_0                , RULL(0x8000003A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP0_REG_P2_1                , RULL(0x8000043A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP0_REG_P2_1                , RULL(0x8000043A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP0_REG_P2_2                , RULL(0x8000083A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP0_REG_P2_2                , RULL(0x8000083A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP0_REG_P2_3                , RULL(0x80000C3A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP0_REG_P2_3                , RULL(0x80000C3A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP0_REG_P2_4                , RULL(0x8000103A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP0_REG_P2_4                , RULL(0x8000103A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP0_REG_P3_0                , RULL(0x8000003A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP0_REG_P3_0                , RULL(0x8000003A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP0_REG_P3_1                , RULL(0x8000043A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP0_REG_P3_1                , RULL(0x8000043A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP0_REG_P3_2                , RULL(0x8000083A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP0_REG_P3_2                , RULL(0x8000083A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP0_REG_P3_3                , RULL(0x80000C3A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP0_REG_P3_3                , RULL(0x80000C3A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP0_REG_P3_4                , RULL(0x8000103A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP0_REG_P3_4                , RULL(0x8000103A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP1_REG_P0_0                  , RULL(0x8000013A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP1_REG_P0_0                , RULL(0x8000013A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP1_REG_P0_0                , RULL(0x8000013A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP1_REG_P0_1                  , RULL(0x8000053A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP1_REG_P0_1                , RULL(0x8000053A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP1_REG_P0_1                , RULL(0x8000053A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP1_REG_P0_2                  , RULL(0x8000093A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP1_REG_P0_2                , RULL(0x8000093A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP1_REG_P0_2                , RULL(0x8000093A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP1_REG_P0_3                  , RULL(0x80000D3A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP1_REG_P0_3                , RULL(0x80000D3A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP1_REG_P0_3                , RULL(0x80000D3A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP1_REG_P0_4                  , RULL(0x8000113A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP1_REG_P0_4                , RULL(0x8000113A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP1_REG_P0_4                , RULL(0x8000113A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP1_REG_P1_0                , RULL(0x8000013A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP1_REG_P1_0                , RULL(0x8000013A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP1_REG_P1_1                , RULL(0x8000053A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP1_REG_P1_1                , RULL(0x8000053A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP1_REG_P1_2                , RULL(0x8000093A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP1_REG_P1_2                , RULL(0x8000093A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP1_REG_P1_3                , RULL(0x80000D3A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP1_REG_P1_3                , RULL(0x80000D3A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP1_REG_P1_4                , RULL(0x8000113A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP1_REG_P1_4                , RULL(0x8000113A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP1_REG_P2_0                , RULL(0x8000013A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP1_REG_P2_0                , RULL(0x8000013A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP1_REG_P2_1                , RULL(0x8000053A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP1_REG_P2_1                , RULL(0x8000053A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP1_REG_P2_2                , RULL(0x8000093A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP1_REG_P2_2                , RULL(0x8000093A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP1_REG_P2_3                , RULL(0x80000D3A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP1_REG_P2_3                , RULL(0x80000D3A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP1_REG_P2_4                , RULL(0x8000113A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP1_REG_P2_4                , RULL(0x8000113A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP1_REG_P3_0                , RULL(0x8000013A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP1_REG_P3_0                , RULL(0x8000013A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP1_REG_P3_1                , RULL(0x8000053A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP1_REG_P3_1                , RULL(0x8000053A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP1_REG_P3_2                , RULL(0x8000093A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP1_REG_P3_2                , RULL(0x8000093A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP1_REG_P3_3                , RULL(0x80000D3A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP1_REG_P3_3                , RULL(0x80000D3A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP1_REG_P3_4                , RULL(0x8000113A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP1_REG_P3_4                , RULL(0x8000113A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP2_REG_P0_0                  , RULL(0x8000023A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP2_REG_P0_0                , RULL(0x8000023A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP2_REG_P0_0                , RULL(0x8000023A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP2_REG_P0_1                  , RULL(0x8000063A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP2_REG_P0_1                , RULL(0x8000063A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP2_REG_P0_1                , RULL(0x8000063A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP2_REG_P0_2                  , RULL(0x80000A3A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP2_REG_P0_2                , RULL(0x80000A3A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP2_REG_P0_2                , RULL(0x80000A3A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP2_REG_P0_3                  , RULL(0x80000E3A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP2_REG_P0_3                , RULL(0x80000E3A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP2_REG_P0_3                , RULL(0x80000E3A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP2_REG_P0_4                  , RULL(0x8000123A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP2_REG_P0_4                , RULL(0x8000123A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP2_REG_P0_4                , RULL(0x8000123A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP2_REG_P1_0                , RULL(0x8000023A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP2_REG_P1_0                , RULL(0x8000023A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP2_REG_P1_1                , RULL(0x8000063A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP2_REG_P1_1                , RULL(0x8000063A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP2_REG_P1_2                , RULL(0x80000A3A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP2_REG_P1_2                , RULL(0x80000A3A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP2_REG_P1_3                , RULL(0x80000E3A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP2_REG_P1_3                , RULL(0x80000E3A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP2_REG_P1_4                , RULL(0x8000123A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP2_REG_P1_4                , RULL(0x8000123A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP2_REG_P2_0                , RULL(0x8000023A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP2_REG_P2_0                , RULL(0x8000023A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP2_REG_P2_1                , RULL(0x8000063A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP2_REG_P2_1                , RULL(0x8000063A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP2_REG_P2_2                , RULL(0x80000A3A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP2_REG_P2_2                , RULL(0x80000A3A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP2_REG_P2_3                , RULL(0x80000E3A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP2_REG_P2_3                , RULL(0x80000E3A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP2_REG_P2_4                , RULL(0x8000123A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP2_REG_P2_4                , RULL(0x8000123A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP2_REG_P3_0                , RULL(0x8000023A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP2_REG_P3_0                , RULL(0x8000023A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP2_REG_P3_1                , RULL(0x8000063A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP2_REG_P3_1                , RULL(0x8000063A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP2_REG_P3_2                , RULL(0x80000A3A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP2_REG_P3_2                , RULL(0x80000A3A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP2_REG_P3_3                , RULL(0x80000E3A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP2_REG_P3_3                , RULL(0x80000E3A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP2_REG_P3_4                , RULL(0x8000123A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP2_REG_P3_4                , RULL(0x8000123A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP3_REG_P0_0                  , RULL(0x8000033A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP3_REG_P0_0                , RULL(0x8000033A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP3_REG_P0_0                , RULL(0x8000033A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP3_REG_P0_1                  , RULL(0x8000073A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP3_REG_P0_1                , RULL(0x8000073A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP3_REG_P0_1                , RULL(0x8000073A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP3_REG_P0_2                  , RULL(0x80000B3A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP3_REG_P0_2                , RULL(0x80000B3A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP3_REG_P0_2                , RULL(0x80000B3A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP3_REG_P0_3                  , RULL(0x80000F3A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP3_REG_P0_3                , RULL(0x80000F3A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP3_REG_P0_3                , RULL(0x80000F3A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_2_RP3_REG_P0_4                  , RULL(0x8000133A0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_2_RP3_REG_P0_4                , RULL(0x8000133A0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_2_RP3_REG_P0_4                , RULL(0x8000133A0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP3_REG_P1_0                , RULL(0x8000033A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP3_REG_P1_0                , RULL(0x8000033A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP3_REG_P1_1                , RULL(0x8000073A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP3_REG_P1_1                , RULL(0x8000073A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP3_REG_P1_2                , RULL(0x80000B3A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP3_REG_P1_2                , RULL(0x80000B3A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP3_REG_P1_3                , RULL(0x80000F3A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP3_REG_P1_3                , RULL(0x80000F3A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_2_RP3_REG_P1_4                , RULL(0x8000133A0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_2_RP3_REG_P1_4                , RULL(0x8000133A0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP3_REG_P2_0                , RULL(0x8000033A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP3_REG_P2_0                , RULL(0x8000033A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP3_REG_P2_1                , RULL(0x8000073A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP3_REG_P2_1                , RULL(0x8000073A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP3_REG_P2_2                , RULL(0x80000B3A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP3_REG_P2_2                , RULL(0x80000B3A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP3_REG_P2_3                , RULL(0x80000F3A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP3_REG_P2_3                , RULL(0x80000F3A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_2_RP3_REG_P2_4                , RULL(0x8000133A0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_2_RP3_REG_P2_4                , RULL(0x8000133A0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP3_REG_P3_0                , RULL(0x8000033A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP3_REG_P3_0                , RULL(0x8000033A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP3_REG_P3_1                , RULL(0x8000073A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP3_REG_P3_1                , RULL(0x8000073A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP3_REG_P3_2                , RULL(0x80000B3A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP3_REG_P3_2                , RULL(0x80000B3A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP3_REG_P3_3                , RULL(0x80000F3A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP3_REG_P3_3                , RULL(0x80000F3A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_2_RP3_REG_P3_4                , RULL(0x8000133A07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_2_RP3_REG_P3_4                , RULL(0x8000133A08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP0_REG_P0_0                  , RULL(0x8000003B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP0_REG_P0_0                , RULL(0x8000003B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP0_REG_P0_0                , RULL(0x8000003B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP0_REG_P0_1                  , RULL(0x8000043B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP0_REG_P0_1                , RULL(0x8000043B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP0_REG_P0_1                , RULL(0x8000043B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP0_REG_P0_2                  , RULL(0x8000083B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP0_REG_P0_2                , RULL(0x8000083B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP0_REG_P0_2                , RULL(0x8000083B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP0_REG_P0_3                  , RULL(0x80000C3B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP0_REG_P0_3                , RULL(0x80000C3B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP0_REG_P0_3                , RULL(0x80000C3B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP0_REG_P0_4                  , RULL(0x8000103B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP0_REG_P0_4                , RULL(0x8000103B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP0_REG_P0_4                , RULL(0x8000103B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP0_REG_P1_0                , RULL(0x8000003B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP0_REG_P1_0                , RULL(0x8000003B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP0_REG_P1_1                , RULL(0x8000043B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP0_REG_P1_1                , RULL(0x8000043B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP0_REG_P1_2                , RULL(0x8000083B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP0_REG_P1_2                , RULL(0x8000083B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP0_REG_P1_3                , RULL(0x80000C3B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP0_REG_P1_3                , RULL(0x80000C3B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP0_REG_P1_4                , RULL(0x8000103B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP0_REG_P1_4                , RULL(0x8000103B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP0_REG_P2_0                , RULL(0x8000003B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP0_REG_P2_0                , RULL(0x8000003B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP0_REG_P2_1                , RULL(0x8000043B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP0_REG_P2_1                , RULL(0x8000043B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP0_REG_P2_2                , RULL(0x8000083B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP0_REG_P2_2                , RULL(0x8000083B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP0_REG_P2_3                , RULL(0x80000C3B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP0_REG_P2_3                , RULL(0x80000C3B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP0_REG_P2_4                , RULL(0x8000103B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP0_REG_P2_4                , RULL(0x8000103B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP0_REG_P3_0                , RULL(0x8000003B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP0_REG_P3_0                , RULL(0x8000003B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP0_REG_P3_1                , RULL(0x8000043B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP0_REG_P3_1                , RULL(0x8000043B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP0_REG_P3_2                , RULL(0x8000083B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP0_REG_P3_2                , RULL(0x8000083B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP0_REG_P3_3                , RULL(0x80000C3B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP0_REG_P3_3                , RULL(0x80000C3B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP0_REG_P3_4                , RULL(0x8000103B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP0_REG_P3_4                , RULL(0x8000103B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP1_REG_P0_0                  , RULL(0x8000013B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP1_REG_P0_0                , RULL(0x8000013B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP1_REG_P0_0                , RULL(0x8000013B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP1_REG_P0_1                  , RULL(0x8000053B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP1_REG_P0_1                , RULL(0x8000053B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP1_REG_P0_1                , RULL(0x8000053B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP1_REG_P0_2                  , RULL(0x8000093B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP1_REG_P0_2                , RULL(0x8000093B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP1_REG_P0_2                , RULL(0x8000093B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP1_REG_P0_3                  , RULL(0x80000D3B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP1_REG_P0_3                , RULL(0x80000D3B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP1_REG_P0_3                , RULL(0x80000D3B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP1_REG_P0_4                  , RULL(0x8000113B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP1_REG_P0_4                , RULL(0x8000113B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP1_REG_P0_4                , RULL(0x8000113B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP1_REG_P1_0                , RULL(0x8000013B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP1_REG_P1_0                , RULL(0x8000013B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP1_REG_P1_1                , RULL(0x8000053B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP1_REG_P1_1                , RULL(0x8000053B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP1_REG_P1_2                , RULL(0x8000093B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP1_REG_P1_2                , RULL(0x8000093B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP1_REG_P1_3                , RULL(0x80000D3B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP1_REG_P1_3                , RULL(0x80000D3B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP1_REG_P1_4                , RULL(0x8000113B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP1_REG_P1_4                , RULL(0x8000113B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP1_REG_P2_0                , RULL(0x8000013B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP1_REG_P2_0                , RULL(0x8000013B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP1_REG_P2_1                , RULL(0x8000053B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP1_REG_P2_1                , RULL(0x8000053B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP1_REG_P2_2                , RULL(0x8000093B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP1_REG_P2_2                , RULL(0x8000093B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP1_REG_P2_3                , RULL(0x80000D3B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP1_REG_P2_3                , RULL(0x80000D3B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP1_REG_P2_4                , RULL(0x8000113B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP1_REG_P2_4                , RULL(0x8000113B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP1_REG_P3_0                , RULL(0x8000013B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP1_REG_P3_0                , RULL(0x8000013B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP1_REG_P3_1                , RULL(0x8000053B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP1_REG_P3_1                , RULL(0x8000053B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP1_REG_P3_2                , RULL(0x8000093B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP1_REG_P3_2                , RULL(0x8000093B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP1_REG_P3_3                , RULL(0x80000D3B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP1_REG_P3_3                , RULL(0x80000D3B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP1_REG_P3_4                , RULL(0x8000113B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP1_REG_P3_4                , RULL(0x8000113B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP2_REG_P0_0                  , RULL(0x8000023B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP2_REG_P0_0                , RULL(0x8000023B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP2_REG_P0_0                , RULL(0x8000023B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP2_REG_P0_1                  , RULL(0x8000063B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP2_REG_P0_1                , RULL(0x8000063B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP2_REG_P0_1                , RULL(0x8000063B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP2_REG_P0_2                  , RULL(0x80000A3B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP2_REG_P0_2                , RULL(0x80000A3B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP2_REG_P0_2                , RULL(0x80000A3B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP2_REG_P0_3                  , RULL(0x80000E3B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP2_REG_P0_3                , RULL(0x80000E3B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP2_REG_P0_3                , RULL(0x80000E3B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP2_REG_P0_4                  , RULL(0x8000123B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP2_REG_P0_4                , RULL(0x8000123B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP2_REG_P0_4                , RULL(0x8000123B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP2_REG_P1_0                , RULL(0x8000023B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP2_REG_P1_0                , RULL(0x8000023B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP2_REG_P1_1                , RULL(0x8000063B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP2_REG_P1_1                , RULL(0x8000063B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP2_REG_P1_2                , RULL(0x80000A3B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP2_REG_P1_2                , RULL(0x80000A3B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP2_REG_P1_3                , RULL(0x80000E3B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP2_REG_P1_3                , RULL(0x80000E3B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP2_REG_P1_4                , RULL(0x8000123B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP2_REG_P1_4                , RULL(0x8000123B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP2_REG_P2_0                , RULL(0x8000023B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP2_REG_P2_0                , RULL(0x8000023B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP2_REG_P2_1                , RULL(0x8000063B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP2_REG_P2_1                , RULL(0x8000063B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP2_REG_P2_2                , RULL(0x80000A3B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP2_REG_P2_2                , RULL(0x80000A3B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP2_REG_P2_3                , RULL(0x80000E3B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP2_REG_P2_3                , RULL(0x80000E3B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP2_REG_P2_4                , RULL(0x8000123B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP2_REG_P2_4                , RULL(0x8000123B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP2_REG_P3_0                , RULL(0x8000023B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP2_REG_P3_0                , RULL(0x8000023B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP2_REG_P3_1                , RULL(0x8000063B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP2_REG_P3_1                , RULL(0x8000063B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP2_REG_P3_2                , RULL(0x80000A3B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP2_REG_P3_2                , RULL(0x80000A3B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP2_REG_P3_3                , RULL(0x80000E3B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP2_REG_P3_3                , RULL(0x80000E3B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP2_REG_P3_4                , RULL(0x8000123B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP2_REG_P3_4                , RULL(0x8000123B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP3_REG_P0_0                  , RULL(0x8000033B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP3_REG_P0_0                , RULL(0x8000033B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP3_REG_P0_0                , RULL(0x8000033B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP3_REG_P0_1                  , RULL(0x8000073B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP3_REG_P0_1                , RULL(0x8000073B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP3_REG_P0_1                , RULL(0x8000073B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP3_REG_P0_2                  , RULL(0x80000B3B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP3_REG_P0_2                , RULL(0x80000B3B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP3_REG_P0_2                , RULL(0x80000B3B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP3_REG_P0_3                  , RULL(0x80000F3B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP3_REG_P0_3                , RULL(0x80000F3B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP3_REG_P0_3                , RULL(0x80000F3B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_3_RP3_REG_P0_4                  , RULL(0x8000133B0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_3_RP3_REG_P0_4                , RULL(0x8000133B0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_3_RP3_REG_P0_4                , RULL(0x8000133B0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP3_REG_P1_0                , RULL(0x8000033B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP3_REG_P1_0                , RULL(0x8000033B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP3_REG_P1_1                , RULL(0x8000073B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP3_REG_P1_1                , RULL(0x8000073B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP3_REG_P1_2                , RULL(0x80000B3B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP3_REG_P1_2                , RULL(0x80000B3B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP3_REG_P1_3                , RULL(0x80000F3B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP3_REG_P1_3                , RULL(0x80000F3B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_3_RP3_REG_P1_4                , RULL(0x8000133B0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_3_RP3_REG_P1_4                , RULL(0x8000133B0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP3_REG_P2_0                , RULL(0x8000033B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP3_REG_P2_0                , RULL(0x8000033B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP3_REG_P2_1                , RULL(0x8000073B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP3_REG_P2_1                , RULL(0x8000073B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP3_REG_P2_2                , RULL(0x80000B3B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP3_REG_P2_2                , RULL(0x80000B3B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP3_REG_P2_3                , RULL(0x80000F3B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP3_REG_P2_3                , RULL(0x80000F3B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_3_RP3_REG_P2_4                , RULL(0x8000133B0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_3_RP3_REG_P2_4                , RULL(0x8000133B0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP3_REG_P3_0                , RULL(0x8000033B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP3_REG_P3_0                , RULL(0x8000033B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP3_REG_P3_1                , RULL(0x8000073B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP3_REG_P3_1                , RULL(0x8000073B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP3_REG_P3_2                , RULL(0x80000B3B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP3_REG_P3_2                , RULL(0x80000B3B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP3_REG_P3_3                , RULL(0x80000F3B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP3_REG_P3_3                , RULL(0x80000F3B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_3_RP3_REG_P3_4                , RULL(0x8000133B07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_3_RP3_REG_P3_4                , RULL(0x8000133B08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP0_REG_P0_0                  , RULL(0x8000003C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP0_REG_P0_0                , RULL(0x8000003C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP0_REG_P0_0                , RULL(0x8000003C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP0_REG_P0_1                  , RULL(0x8000043C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP0_REG_P0_1                , RULL(0x8000043C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP0_REG_P0_1                , RULL(0x8000043C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP0_REG_P0_2                  , RULL(0x8000083C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP0_REG_P0_2                , RULL(0x8000083C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP0_REG_P0_2                , RULL(0x8000083C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP0_REG_P0_3                  , RULL(0x80000C3C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP0_REG_P0_3                , RULL(0x80000C3C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP0_REG_P0_3                , RULL(0x80000C3C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP0_REG_P0_4                  , RULL(0x8000103C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP0_REG_P0_4                , RULL(0x8000103C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP0_REG_P0_4                , RULL(0x8000103C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP0_REG_P1_0                , RULL(0x8000003C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP0_REG_P1_0                , RULL(0x8000003C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP0_REG_P1_1                , RULL(0x8000043C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP0_REG_P1_1                , RULL(0x8000043C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP0_REG_P1_2                , RULL(0x8000083C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP0_REG_P1_2                , RULL(0x8000083C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP0_REG_P1_3                , RULL(0x80000C3C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP0_REG_P1_3                , RULL(0x80000C3C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP0_REG_P1_4                , RULL(0x8000103C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP0_REG_P1_4                , RULL(0x8000103C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP0_REG_P2_0                , RULL(0x8000003C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP0_REG_P2_0                , RULL(0x8000003C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP0_REG_P2_1                , RULL(0x8000043C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP0_REG_P2_1                , RULL(0x8000043C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP0_REG_P2_2                , RULL(0x8000083C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP0_REG_P2_2                , RULL(0x8000083C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP0_REG_P2_3                , RULL(0x80000C3C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP0_REG_P2_3                , RULL(0x80000C3C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP0_REG_P2_4                , RULL(0x8000103C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP0_REG_P2_4                , RULL(0x8000103C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP0_REG_P3_0                , RULL(0x8000003C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP0_REG_P3_0                , RULL(0x8000003C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP0_REG_P3_1                , RULL(0x8000043C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP0_REG_P3_1                , RULL(0x8000043C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP0_REG_P3_2                , RULL(0x8000083C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP0_REG_P3_2                , RULL(0x8000083C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP0_REG_P3_3                , RULL(0x80000C3C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP0_REG_P3_3                , RULL(0x80000C3C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP0_REG_P3_4                , RULL(0x8000103C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP0_REG_P3_4                , RULL(0x8000103C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP1_REG_P0_0                  , RULL(0x8000013C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP1_REG_P0_0                , RULL(0x8000013C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP1_REG_P0_0                , RULL(0x8000013C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP1_REG_P0_1                  , RULL(0x8000053C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP1_REG_P0_1                , RULL(0x8000053C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP1_REG_P0_1                , RULL(0x8000053C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP1_REG_P0_2                  , RULL(0x8000093C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP1_REG_P0_2                , RULL(0x8000093C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP1_REG_P0_2                , RULL(0x8000093C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP1_REG_P0_3                  , RULL(0x80000D3C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP1_REG_P0_3                , RULL(0x80000D3C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP1_REG_P0_3                , RULL(0x80000D3C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP1_REG_P0_4                  , RULL(0x8000113C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP1_REG_P0_4                , RULL(0x8000113C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP1_REG_P0_4                , RULL(0x8000113C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP1_REG_P1_0                , RULL(0x8000013C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP1_REG_P1_0                , RULL(0x8000013C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP1_REG_P1_1                , RULL(0x8000053C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP1_REG_P1_1                , RULL(0x8000053C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP1_REG_P1_2                , RULL(0x8000093C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP1_REG_P1_2                , RULL(0x8000093C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP1_REG_P1_3                , RULL(0x80000D3C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP1_REG_P1_3                , RULL(0x80000D3C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP1_REG_P1_4                , RULL(0x8000113C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP1_REG_P1_4                , RULL(0x8000113C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP1_REG_P2_0                , RULL(0x8000013C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP1_REG_P2_0                , RULL(0x8000013C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP1_REG_P2_1                , RULL(0x8000053C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP1_REG_P2_1                , RULL(0x8000053C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP1_REG_P2_2                , RULL(0x8000093C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP1_REG_P2_2                , RULL(0x8000093C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP1_REG_P2_3                , RULL(0x80000D3C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP1_REG_P2_3                , RULL(0x80000D3C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP1_REG_P2_4                , RULL(0x8000113C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP1_REG_P2_4                , RULL(0x8000113C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP1_REG_P3_0                , RULL(0x8000013C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP1_REG_P3_0                , RULL(0x8000013C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP1_REG_P3_1                , RULL(0x8000053C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP1_REG_P3_1                , RULL(0x8000053C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP1_REG_P3_2                , RULL(0x8000093C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP1_REG_P3_2                , RULL(0x8000093C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP1_REG_P3_3                , RULL(0x80000D3C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP1_REG_P3_3                , RULL(0x80000D3C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP1_REG_P3_4                , RULL(0x8000113C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP1_REG_P3_4                , RULL(0x8000113C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP2_REG_P0_0                  , RULL(0x8000023C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP2_REG_P0_0                , RULL(0x8000023C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP2_REG_P0_0                , RULL(0x8000023C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP2_REG_P0_1                  , RULL(0x8000063C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP2_REG_P0_1                , RULL(0x8000063C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP2_REG_P0_1                , RULL(0x8000063C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP2_REG_P0_2                  , RULL(0x80000A3C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP2_REG_P0_2                , RULL(0x80000A3C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP2_REG_P0_2                , RULL(0x80000A3C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP2_REG_P0_3                  , RULL(0x80000E3C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP2_REG_P0_3                , RULL(0x80000E3C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP2_REG_P0_3                , RULL(0x80000E3C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP2_REG_P0_4                  , RULL(0x8000123C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP2_REG_P0_4                , RULL(0x8000123C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP2_REG_P0_4                , RULL(0x8000123C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP2_REG_P1_0                , RULL(0x8000023C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP2_REG_P1_0                , RULL(0x8000023C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP2_REG_P1_1                , RULL(0x8000063C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP2_REG_P1_1                , RULL(0x8000063C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP2_REG_P1_2                , RULL(0x80000A3C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP2_REG_P1_2                , RULL(0x80000A3C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP2_REG_P1_3                , RULL(0x80000E3C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP2_REG_P1_3                , RULL(0x80000E3C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP2_REG_P1_4                , RULL(0x8000123C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP2_REG_P1_4                , RULL(0x8000123C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP2_REG_P2_0                , RULL(0x8000023C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP2_REG_P2_0                , RULL(0x8000023C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP2_REG_P2_1                , RULL(0x8000063C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP2_REG_P2_1                , RULL(0x8000063C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP2_REG_P2_2                , RULL(0x80000A3C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP2_REG_P2_2                , RULL(0x80000A3C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP2_REG_P2_3                , RULL(0x80000E3C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP2_REG_P2_3                , RULL(0x80000E3C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP2_REG_P2_4                , RULL(0x8000123C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP2_REG_P2_4                , RULL(0x8000123C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP2_REG_P3_0                , RULL(0x8000023C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP2_REG_P3_0                , RULL(0x8000023C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP2_REG_P3_1                , RULL(0x8000063C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP2_REG_P3_1                , RULL(0x8000063C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP2_REG_P3_2                , RULL(0x80000A3C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP2_REG_P3_2                , RULL(0x80000A3C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP2_REG_P3_3                , RULL(0x80000E3C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP2_REG_P3_3                , RULL(0x80000E3C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP2_REG_P3_4                , RULL(0x8000123C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP2_REG_P3_4                , RULL(0x8000123C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP3_REG_P0_0                  , RULL(0x8000033C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP3_REG_P0_0                , RULL(0x8000033C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP3_REG_P0_0                , RULL(0x8000033C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP3_REG_P0_1                  , RULL(0x8000073C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP3_REG_P0_1                , RULL(0x8000073C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP3_REG_P0_1                , RULL(0x8000073C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP3_REG_P0_2                  , RULL(0x80000B3C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP3_REG_P0_2                , RULL(0x80000B3C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP3_REG_P0_2                , RULL(0x80000B3C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP3_REG_P0_3                  , RULL(0x80000F3C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP3_REG_P0_3                , RULL(0x80000F3C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP3_REG_P0_3                , RULL(0x80000F3C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_4_RP3_REG_P0_4                  , RULL(0x8000133C0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_4_RP3_REG_P0_4                , RULL(0x8000133C0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_4_RP3_REG_P0_4                , RULL(0x8000133C0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP3_REG_P1_0                , RULL(0x8000033C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP3_REG_P1_0                , RULL(0x8000033C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP3_REG_P1_1                , RULL(0x8000073C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP3_REG_P1_1                , RULL(0x8000073C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP3_REG_P1_2                , RULL(0x80000B3C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP3_REG_P1_2                , RULL(0x80000B3C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP3_REG_P1_3                , RULL(0x80000F3C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP3_REG_P1_3                , RULL(0x80000F3C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_4_RP3_REG_P1_4                , RULL(0x8000133C0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_4_RP3_REG_P1_4                , RULL(0x8000133C0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP3_REG_P2_0                , RULL(0x8000033C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP3_REG_P2_0                , RULL(0x8000033C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP3_REG_P2_1                , RULL(0x8000073C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP3_REG_P2_1                , RULL(0x8000073C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP3_REG_P2_2                , RULL(0x80000B3C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP3_REG_P2_2                , RULL(0x80000B3C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP3_REG_P2_3                , RULL(0x80000F3C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP3_REG_P2_3                , RULL(0x80000F3C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_4_RP3_REG_P2_4                , RULL(0x8000133C0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_4_RP3_REG_P2_4                , RULL(0x8000133C0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP3_REG_P3_0                , RULL(0x8000033C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP3_REG_P3_0                , RULL(0x8000033C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP3_REG_P3_1                , RULL(0x8000073C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP3_REG_P3_1                , RULL(0x8000073C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP3_REG_P3_2                , RULL(0x80000B3C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP3_REG_P3_2                , RULL(0x80000B3C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP3_REG_P3_3                , RULL(0x80000F3C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP3_REG_P3_3                , RULL(0x80000F3C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_4_RP3_REG_P3_4                , RULL(0x8000133C07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_4_RP3_REG_P3_4                , RULL(0x8000133C08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP0_REG_P0_0                  , RULL(0x8000003D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP0_REG_P0_0                , RULL(0x8000003D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP0_REG_P0_0                , RULL(0x8000003D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP0_REG_P0_1                  , RULL(0x8000043D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP0_REG_P0_1                , RULL(0x8000043D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP0_REG_P0_1                , RULL(0x8000043D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP0_REG_P0_2                  , RULL(0x8000083D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP0_REG_P0_2                , RULL(0x8000083D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP0_REG_P0_2                , RULL(0x8000083D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP0_REG_P0_3                  , RULL(0x80000C3D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP0_REG_P0_3                , RULL(0x80000C3D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP0_REG_P0_3                , RULL(0x80000C3D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP0_REG_P0_4                  , RULL(0x8000103D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP0_REG_P0_4                , RULL(0x8000103D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP0_REG_P0_4                , RULL(0x8000103D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP0_REG_P1_0                , RULL(0x8000003D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP0_REG_P1_0                , RULL(0x8000003D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP0_REG_P1_1                , RULL(0x8000043D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP0_REG_P1_1                , RULL(0x8000043D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP0_REG_P1_2                , RULL(0x8000083D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP0_REG_P1_2                , RULL(0x8000083D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP0_REG_P1_3                , RULL(0x80000C3D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP0_REG_P1_3                , RULL(0x80000C3D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP0_REG_P1_4                , RULL(0x8000103D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP0_REG_P1_4                , RULL(0x8000103D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP0_REG_P2_0                , RULL(0x8000003D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP0_REG_P2_0                , RULL(0x8000003D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP0_REG_P2_1                , RULL(0x8000043D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP0_REG_P2_1                , RULL(0x8000043D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP0_REG_P2_2                , RULL(0x8000083D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP0_REG_P2_2                , RULL(0x8000083D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP0_REG_P2_3                , RULL(0x80000C3D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP0_REG_P2_3                , RULL(0x80000C3D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP0_REG_P2_4                , RULL(0x8000103D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP0_REG_P2_4                , RULL(0x8000103D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP0_REG_P3_0                , RULL(0x8000003D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP0_REG_P3_0                , RULL(0x8000003D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP0_REG_P3_1                , RULL(0x8000043D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP0_REG_P3_1                , RULL(0x8000043D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP0_REG_P3_2                , RULL(0x8000083D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP0_REG_P3_2                , RULL(0x8000083D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP0_REG_P3_3                , RULL(0x80000C3D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP0_REG_P3_3                , RULL(0x80000C3D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP0_REG_P3_4                , RULL(0x8000103D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP0_REG_P3_4                , RULL(0x8000103D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP1_REG_P0_0                  , RULL(0x8000013D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP1_REG_P0_0                , RULL(0x8000013D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP1_REG_P0_0                , RULL(0x8000013D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP1_REG_P0_1                  , RULL(0x8000053D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP1_REG_P0_1                , RULL(0x8000053D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP1_REG_P0_1                , RULL(0x8000053D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP1_REG_P0_2                  , RULL(0x8000093D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP1_REG_P0_2                , RULL(0x8000093D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP1_REG_P0_2                , RULL(0x8000093D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP1_REG_P0_3                  , RULL(0x80000D3D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP1_REG_P0_3                , RULL(0x80000D3D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP1_REG_P0_3                , RULL(0x80000D3D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP1_REG_P0_4                  , RULL(0x8000113D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP1_REG_P0_4                , RULL(0x8000113D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP1_REG_P0_4                , RULL(0x8000113D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP1_REG_P1_0                , RULL(0x8000013D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP1_REG_P1_0                , RULL(0x8000013D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP1_REG_P1_1                , RULL(0x8000053D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP1_REG_P1_1                , RULL(0x8000053D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP1_REG_P1_2                , RULL(0x8000093D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP1_REG_P1_2                , RULL(0x8000093D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP1_REG_P1_3                , RULL(0x80000D3D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP1_REG_P1_3                , RULL(0x80000D3D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP1_REG_P1_4                , RULL(0x8000113D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP1_REG_P1_4                , RULL(0x8000113D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP1_REG_P2_0                , RULL(0x8000013D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP1_REG_P2_0                , RULL(0x8000013D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP1_REG_P2_1                , RULL(0x8000053D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP1_REG_P2_1                , RULL(0x8000053D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP1_REG_P2_2                , RULL(0x8000093D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP1_REG_P2_2                , RULL(0x8000093D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP1_REG_P2_3                , RULL(0x80000D3D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP1_REG_P2_3                , RULL(0x80000D3D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP1_REG_P2_4                , RULL(0x8000113D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP1_REG_P2_4                , RULL(0x8000113D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP1_REG_P3_0                , RULL(0x8000013D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP1_REG_P3_0                , RULL(0x8000013D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP1_REG_P3_1                , RULL(0x8000053D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP1_REG_P3_1                , RULL(0x8000053D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP1_REG_P3_2                , RULL(0x8000093D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP1_REG_P3_2                , RULL(0x8000093D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP1_REG_P3_3                , RULL(0x80000D3D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP1_REG_P3_3                , RULL(0x80000D3D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP1_REG_P3_4                , RULL(0x8000113D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP1_REG_P3_4                , RULL(0x8000113D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP2_REG_P0_0                  , RULL(0x8000023D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP2_REG_P0_0                , RULL(0x8000023D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP2_REG_P0_0                , RULL(0x8000023D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP2_REG_P0_1                  , RULL(0x8000063D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP2_REG_P0_1                , RULL(0x8000063D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP2_REG_P0_1                , RULL(0x8000063D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP2_REG_P0_2                  , RULL(0x80000A3D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP2_REG_P0_2                , RULL(0x80000A3D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP2_REG_P0_2                , RULL(0x80000A3D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP2_REG_P0_3                  , RULL(0x80000E3D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP2_REG_P0_3                , RULL(0x80000E3D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP2_REG_P0_3                , RULL(0x80000E3D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP2_REG_P0_4                  , RULL(0x8000123D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP2_REG_P0_4                , RULL(0x8000123D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP2_REG_P0_4                , RULL(0x8000123D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP2_REG_P1_0                , RULL(0x8000023D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP2_REG_P1_0                , RULL(0x8000023D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP2_REG_P1_1                , RULL(0x8000063D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP2_REG_P1_1                , RULL(0x8000063D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP2_REG_P1_2                , RULL(0x80000A3D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP2_REG_P1_2                , RULL(0x80000A3D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP2_REG_P1_3                , RULL(0x80000E3D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP2_REG_P1_3                , RULL(0x80000E3D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP2_REG_P1_4                , RULL(0x8000123D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP2_REG_P1_4                , RULL(0x8000123D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP2_REG_P2_0                , RULL(0x8000023D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP2_REG_P2_0                , RULL(0x8000023D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP2_REG_P2_1                , RULL(0x8000063D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP2_REG_P2_1                , RULL(0x8000063D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP2_REG_P2_2                , RULL(0x80000A3D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP2_REG_P2_2                , RULL(0x80000A3D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP2_REG_P2_3                , RULL(0x80000E3D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP2_REG_P2_3                , RULL(0x80000E3D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP2_REG_P2_4                , RULL(0x8000123D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP2_REG_P2_4                , RULL(0x8000123D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP2_REG_P3_0                , RULL(0x8000023D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP2_REG_P3_0                , RULL(0x8000023D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP2_REG_P3_1                , RULL(0x8000063D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP2_REG_P3_1                , RULL(0x8000063D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP2_REG_P3_2                , RULL(0x80000A3D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP2_REG_P3_2                , RULL(0x80000A3D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP2_REG_P3_3                , RULL(0x80000E3D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP2_REG_P3_3                , RULL(0x80000E3D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP2_REG_P3_4                , RULL(0x8000123D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP2_REG_P3_4                , RULL(0x8000123D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP3_REG_P0_0                  , RULL(0x8000033D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP3_REG_P0_0                , RULL(0x8000033D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP3_REG_P0_0                , RULL(0x8000033D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP3_REG_P0_1                  , RULL(0x8000073D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP3_REG_P0_1                , RULL(0x8000073D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP3_REG_P0_1                , RULL(0x8000073D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP3_REG_P0_2                  , RULL(0x80000B3D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP3_REG_P0_2                , RULL(0x80000B3D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP3_REG_P0_2                , RULL(0x80000B3D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP3_REG_P0_3                  , RULL(0x80000F3D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP3_REG_P0_3                , RULL(0x80000F3D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP3_REG_P0_3                , RULL(0x80000F3D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_5_RP3_REG_P0_4                  , RULL(0x8000133D0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_5_RP3_REG_P0_4                , RULL(0x8000133D0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_5_RP3_REG_P0_4                , RULL(0x8000133D0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP3_REG_P1_0                , RULL(0x8000033D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP3_REG_P1_0                , RULL(0x8000033D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP3_REG_P1_1                , RULL(0x8000073D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP3_REG_P1_1                , RULL(0x8000073D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP3_REG_P1_2                , RULL(0x80000B3D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP3_REG_P1_2                , RULL(0x80000B3D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP3_REG_P1_3                , RULL(0x80000F3D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP3_REG_P1_3                , RULL(0x80000F3D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_5_RP3_REG_P1_4                , RULL(0x8000133D0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_5_RP3_REG_P1_4                , RULL(0x8000133D0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP3_REG_P2_0                , RULL(0x8000033D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP3_REG_P2_0                , RULL(0x8000033D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP3_REG_P2_1                , RULL(0x8000073D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP3_REG_P2_1                , RULL(0x8000073D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP3_REG_P2_2                , RULL(0x80000B3D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP3_REG_P2_2                , RULL(0x80000B3D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP3_REG_P2_3                , RULL(0x80000F3D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP3_REG_P2_3                , RULL(0x80000F3D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_5_RP3_REG_P2_4                , RULL(0x8000133D0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_5_RP3_REG_P2_4                , RULL(0x8000133D0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP3_REG_P3_0                , RULL(0x8000033D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP3_REG_P3_0                , RULL(0x8000033D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP3_REG_P3_1                , RULL(0x8000073D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP3_REG_P3_1                , RULL(0x8000073D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP3_REG_P3_2                , RULL(0x80000B3D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP3_REG_P3_2                , RULL(0x80000B3D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP3_REG_P3_3                , RULL(0x80000F3D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP3_REG_P3_3                , RULL(0x80000F3D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_5_RP3_REG_P3_4                , RULL(0x8000133D07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_5_RP3_REG_P3_4                , RULL(0x8000133D08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP0_REG_P0_0                  , RULL(0x8000003E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP0_REG_P0_0                , RULL(0x8000003E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP0_REG_P0_0                , RULL(0x8000003E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP0_REG_P0_1                  , RULL(0x8000043E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP0_REG_P0_1                , RULL(0x8000043E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP0_REG_P0_1                , RULL(0x8000043E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP0_REG_P0_2                  , RULL(0x8000083E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP0_REG_P0_2                , RULL(0x8000083E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP0_REG_P0_2                , RULL(0x8000083E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP0_REG_P0_3                  , RULL(0x80000C3E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP0_REG_P0_3                , RULL(0x80000C3E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP0_REG_P0_3                , RULL(0x80000C3E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP0_REG_P0_4                  , RULL(0x8000103E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP0_REG_P0_4                , RULL(0x8000103E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP0_REG_P0_4                , RULL(0x8000103E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP0_REG_P1_0                , RULL(0x8000003E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP0_REG_P1_0                , RULL(0x8000003E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP0_REG_P1_1                , RULL(0x8000043E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP0_REG_P1_1                , RULL(0x8000043E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP0_REG_P1_2                , RULL(0x8000083E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP0_REG_P1_2                , RULL(0x8000083E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP0_REG_P1_3                , RULL(0x80000C3E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP0_REG_P1_3                , RULL(0x80000C3E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP0_REG_P1_4                , RULL(0x8000103E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP0_REG_P1_4                , RULL(0x8000103E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP0_REG_P2_0                , RULL(0x8000003E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP0_REG_P2_0                , RULL(0x8000003E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP0_REG_P2_1                , RULL(0x8000043E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP0_REG_P2_1                , RULL(0x8000043E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP0_REG_P2_2                , RULL(0x8000083E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP0_REG_P2_2                , RULL(0x8000083E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP0_REG_P2_3                , RULL(0x80000C3E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP0_REG_P2_3                , RULL(0x80000C3E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP0_REG_P2_4                , RULL(0x8000103E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP0_REG_P2_4                , RULL(0x8000103E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP0_REG_P3_0                , RULL(0x8000003E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP0_REG_P3_0                , RULL(0x8000003E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP0_REG_P3_1                , RULL(0x8000043E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP0_REG_P3_1                , RULL(0x8000043E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP0_REG_P3_2                , RULL(0x8000083E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP0_REG_P3_2                , RULL(0x8000083E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP0_REG_P3_3                , RULL(0x80000C3E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP0_REG_P3_3                , RULL(0x80000C3E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP0_REG_P3_4                , RULL(0x8000103E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP0_REG_P3_4                , RULL(0x8000103E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP1_REG_P0_0                  , RULL(0x8000013E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP1_REG_P0_0                , RULL(0x8000013E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP1_REG_P0_0                , RULL(0x8000013E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP1_REG_P0_1                  , RULL(0x8000053E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP1_REG_P0_1                , RULL(0x8000053E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP1_REG_P0_1                , RULL(0x8000053E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP1_REG_P0_2                  , RULL(0x8000093E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP1_REG_P0_2                , RULL(0x8000093E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP1_REG_P0_2                , RULL(0x8000093E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP1_REG_P0_3                  , RULL(0x80000D3E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP1_REG_P0_3                , RULL(0x80000D3E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP1_REG_P0_3                , RULL(0x80000D3E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP1_REG_P0_4                  , RULL(0x8000113E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP1_REG_P0_4                , RULL(0x8000113E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP1_REG_P0_4                , RULL(0x8000113E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP1_REG_P1_0                , RULL(0x8000013E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP1_REG_P1_0                , RULL(0x8000013E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP1_REG_P1_1                , RULL(0x8000053E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP1_REG_P1_1                , RULL(0x8000053E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP1_REG_P1_2                , RULL(0x8000093E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP1_REG_P1_2                , RULL(0x8000093E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP1_REG_P1_3                , RULL(0x80000D3E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP1_REG_P1_3                , RULL(0x80000D3E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP1_REG_P1_4                , RULL(0x8000113E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP1_REG_P1_4                , RULL(0x8000113E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP1_REG_P2_0                , RULL(0x8000013E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP1_REG_P2_0                , RULL(0x8000013E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP1_REG_P2_1                , RULL(0x8000053E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP1_REG_P2_1                , RULL(0x8000053E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP1_REG_P2_2                , RULL(0x8000093E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP1_REG_P2_2                , RULL(0x8000093E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP1_REG_P2_3                , RULL(0x80000D3E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP1_REG_P2_3                , RULL(0x80000D3E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP1_REG_P2_4                , RULL(0x8000113E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP1_REG_P2_4                , RULL(0x8000113E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP1_REG_P3_0                , RULL(0x8000013E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP1_REG_P3_0                , RULL(0x8000013E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP1_REG_P3_1                , RULL(0x8000053E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP1_REG_P3_1                , RULL(0x8000053E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP1_REG_P3_2                , RULL(0x8000093E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP1_REG_P3_2                , RULL(0x8000093E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP1_REG_P3_3                , RULL(0x80000D3E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP1_REG_P3_3                , RULL(0x80000D3E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP1_REG_P3_4                , RULL(0x8000113E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP1_REG_P3_4                , RULL(0x8000113E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP2_REG_P0_0                  , RULL(0x8000023E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP2_REG_P0_0                , RULL(0x8000023E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP2_REG_P0_0                , RULL(0x8000023E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP2_REG_P0_1                  , RULL(0x8000063E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP2_REG_P0_1                , RULL(0x8000063E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP2_REG_P0_1                , RULL(0x8000063E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP2_REG_P0_2                  , RULL(0x80000A3E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP2_REG_P0_2                , RULL(0x80000A3E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP2_REG_P0_2                , RULL(0x80000A3E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP2_REG_P0_3                  , RULL(0x80000E3E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP2_REG_P0_3                , RULL(0x80000E3E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP2_REG_P0_3                , RULL(0x80000E3E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP2_REG_P0_4                  , RULL(0x8000123E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP2_REG_P0_4                , RULL(0x8000123E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP2_REG_P0_4                , RULL(0x8000123E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP2_REG_P1_0                , RULL(0x8000023E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP2_REG_P1_0                , RULL(0x8000023E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP2_REG_P1_1                , RULL(0x8000063E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP2_REG_P1_1                , RULL(0x8000063E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP2_REG_P1_2                , RULL(0x80000A3E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP2_REG_P1_2                , RULL(0x80000A3E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP2_REG_P1_3                , RULL(0x80000E3E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP2_REG_P1_3                , RULL(0x80000E3E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP2_REG_P1_4                , RULL(0x8000123E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP2_REG_P1_4                , RULL(0x8000123E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP2_REG_P2_0                , RULL(0x8000023E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP2_REG_P2_0                , RULL(0x8000023E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP2_REG_P2_1                , RULL(0x8000063E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP2_REG_P2_1                , RULL(0x8000063E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP2_REG_P2_2                , RULL(0x80000A3E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP2_REG_P2_2                , RULL(0x80000A3E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP2_REG_P2_3                , RULL(0x80000E3E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP2_REG_P2_3                , RULL(0x80000E3E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP2_REG_P2_4                , RULL(0x8000123E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP2_REG_P2_4                , RULL(0x8000123E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP2_REG_P3_0                , RULL(0x8000023E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP2_REG_P3_0                , RULL(0x8000023E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP2_REG_P3_1                , RULL(0x8000063E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP2_REG_P3_1                , RULL(0x8000063E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP2_REG_P3_2                , RULL(0x80000A3E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP2_REG_P3_2                , RULL(0x80000A3E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP2_REG_P3_3                , RULL(0x80000E3E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP2_REG_P3_3                , RULL(0x80000E3E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP2_REG_P3_4                , RULL(0x8000123E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP2_REG_P3_4                , RULL(0x8000123E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP3_REG_P0_0                  , RULL(0x8000033E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP3_REG_P0_0                , RULL(0x8000033E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP3_REG_P0_0                , RULL(0x8000033E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP3_REG_P0_1                  , RULL(0x8000073E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP3_REG_P0_1                , RULL(0x8000073E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP3_REG_P0_1                , RULL(0x8000073E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP3_REG_P0_2                  , RULL(0x80000B3E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP3_REG_P0_2                , RULL(0x80000B3E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP3_REG_P0_2                , RULL(0x80000B3E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP3_REG_P0_3                  , RULL(0x80000F3E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP3_REG_P0_3                , RULL(0x80000F3E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP3_REG_P0_3                , RULL(0x80000F3E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_6_RP3_REG_P0_4                  , RULL(0x8000133E0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_6_RP3_REG_P0_4                , RULL(0x8000133E0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_6_RP3_REG_P0_4                , RULL(0x8000133E0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP3_REG_P1_0                , RULL(0x8000033E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP3_REG_P1_0                , RULL(0x8000033E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP3_REG_P1_1                , RULL(0x8000073E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP3_REG_P1_1                , RULL(0x8000073E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP3_REG_P1_2                , RULL(0x80000B3E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP3_REG_P1_2                , RULL(0x80000B3E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP3_REG_P1_3                , RULL(0x80000F3E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP3_REG_P1_3                , RULL(0x80000F3E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_6_RP3_REG_P1_4                , RULL(0x8000133E0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_6_RP3_REG_P1_4                , RULL(0x8000133E0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP3_REG_P2_0                , RULL(0x8000033E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP3_REG_P2_0                , RULL(0x8000033E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP3_REG_P2_1                , RULL(0x8000073E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP3_REG_P2_1                , RULL(0x8000073E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP3_REG_P2_2                , RULL(0x80000B3E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP3_REG_P2_2                , RULL(0x80000B3E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP3_REG_P2_3                , RULL(0x80000F3E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP3_REG_P2_3                , RULL(0x80000F3E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_6_RP3_REG_P2_4                , RULL(0x8000133E0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_6_RP3_REG_P2_4                , RULL(0x8000133E0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP3_REG_P3_0                , RULL(0x8000033E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP3_REG_P3_0                , RULL(0x8000033E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP3_REG_P3_1                , RULL(0x8000073E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP3_REG_P3_1                , RULL(0x8000073E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP3_REG_P3_2                , RULL(0x80000B3E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP3_REG_P3_2                , RULL(0x80000B3E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP3_REG_P3_3                , RULL(0x80000F3E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP3_REG_P3_3                , RULL(0x80000F3E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_6_RP3_REG_P3_4                , RULL(0x8000133E07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_6_RP3_REG_P3_4                , RULL(0x8000133E08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP0_REG_P0_0                  , RULL(0x8000003F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP0_REG_P0_0                , RULL(0x8000003F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP0_REG_P0_0                , RULL(0x8000003F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP0_REG_P0_1                  , RULL(0x8000043F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP0_REG_P0_1                , RULL(0x8000043F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP0_REG_P0_1                , RULL(0x8000043F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP0_REG_P0_2                  , RULL(0x8000083F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP0_REG_P0_2                , RULL(0x8000083F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP0_REG_P0_2                , RULL(0x8000083F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP0_REG_P0_3                  , RULL(0x80000C3F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP0_REG_P0_3                , RULL(0x80000C3F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP0_REG_P0_3                , RULL(0x80000C3F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP0_REG_P0_4                  , RULL(0x8000103F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP0_REG_P0_4                , RULL(0x8000103F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP0_REG_P0_4                , RULL(0x8000103F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP0_REG_P1_0                , RULL(0x8000003F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP0_REG_P1_0                , RULL(0x8000003F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP0_REG_P1_1                , RULL(0x8000043F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP0_REG_P1_1                , RULL(0x8000043F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP0_REG_P1_2                , RULL(0x8000083F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP0_REG_P1_2                , RULL(0x8000083F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP0_REG_P1_3                , RULL(0x80000C3F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP0_REG_P1_3                , RULL(0x80000C3F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP0_REG_P1_4                , RULL(0x8000103F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP0_REG_P1_4                , RULL(0x8000103F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP0_REG_P2_0                , RULL(0x8000003F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP0_REG_P2_0                , RULL(0x8000003F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP0_REG_P2_1                , RULL(0x8000043F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP0_REG_P2_1                , RULL(0x8000043F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP0_REG_P2_2                , RULL(0x8000083F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP0_REG_P2_2                , RULL(0x8000083F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP0_REG_P2_3                , RULL(0x80000C3F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP0_REG_P2_3                , RULL(0x80000C3F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP0_REG_P2_4                , RULL(0x8000103F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP0_REG_P2_4                , RULL(0x8000103F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP0_REG_P3_0                , RULL(0x8000003F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP0_REG_P3_0                , RULL(0x8000003F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP0_REG_P3_1                , RULL(0x8000043F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP0_REG_P3_1                , RULL(0x8000043F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP0_REG_P3_2                , RULL(0x8000083F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP0_REG_P3_2                , RULL(0x8000083F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP0_REG_P3_3                , RULL(0x80000C3F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP0_REG_P3_3                , RULL(0x80000C3F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP0_REG_P3_4                , RULL(0x8000103F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP0_REG_P3_4                , RULL(0x8000103F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP1_REG_P0_0                  , RULL(0x8000013F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP1_REG_P0_0                , RULL(0x8000013F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP1_REG_P0_0                , RULL(0x8000013F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP1_REG_P0_1                  , RULL(0x8000053F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP1_REG_P0_1                , RULL(0x8000053F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP1_REG_P0_1                , RULL(0x8000053F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP1_REG_P0_2                  , RULL(0x8000093F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP1_REG_P0_2                , RULL(0x8000093F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP1_REG_P0_2                , RULL(0x8000093F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP1_REG_P0_3                  , RULL(0x80000D3F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP1_REG_P0_3                , RULL(0x80000D3F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP1_REG_P0_3                , RULL(0x80000D3F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP1_REG_P0_4                  , RULL(0x8000113F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP1_REG_P0_4                , RULL(0x8000113F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP1_REG_P0_4                , RULL(0x8000113F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP1_REG_P1_0                , RULL(0x8000013F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP1_REG_P1_0                , RULL(0x8000013F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP1_REG_P1_1                , RULL(0x8000053F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP1_REG_P1_1                , RULL(0x8000053F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP1_REG_P1_2                , RULL(0x8000093F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP1_REG_P1_2                , RULL(0x8000093F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP1_REG_P1_3                , RULL(0x80000D3F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP1_REG_P1_3                , RULL(0x80000D3F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP1_REG_P1_4                , RULL(0x8000113F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP1_REG_P1_4                , RULL(0x8000113F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP1_REG_P2_0                , RULL(0x8000013F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP1_REG_P2_0                , RULL(0x8000013F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP1_REG_P2_1                , RULL(0x8000053F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP1_REG_P2_1                , RULL(0x8000053F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP1_REG_P2_2                , RULL(0x8000093F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP1_REG_P2_2                , RULL(0x8000093F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP1_REG_P2_3                , RULL(0x80000D3F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP1_REG_P2_3                , RULL(0x80000D3F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP1_REG_P2_4                , RULL(0x8000113F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP1_REG_P2_4                , RULL(0x8000113F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP1_REG_P3_0                , RULL(0x8000013F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP1_REG_P3_0                , RULL(0x8000013F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP1_REG_P3_1                , RULL(0x8000053F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP1_REG_P3_1                , RULL(0x8000053F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP1_REG_P3_2                , RULL(0x8000093F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP1_REG_P3_2                , RULL(0x8000093F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP1_REG_P3_3                , RULL(0x80000D3F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP1_REG_P3_3                , RULL(0x80000D3F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP1_REG_P3_4                , RULL(0x8000113F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP1_REG_P3_4                , RULL(0x8000113F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP2_REG_P0_0                  , RULL(0x8000023F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP2_REG_P0_0                , RULL(0x8000023F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP2_REG_P0_0                , RULL(0x8000023F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP2_REG_P0_1                  , RULL(0x8000063F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP2_REG_P0_1                , RULL(0x8000063F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP2_REG_P0_1                , RULL(0x8000063F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP2_REG_P0_2                  , RULL(0x80000A3F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP2_REG_P0_2                , RULL(0x80000A3F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP2_REG_P0_2                , RULL(0x80000A3F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP2_REG_P0_3                  , RULL(0x80000E3F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP2_REG_P0_3                , RULL(0x80000E3F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP2_REG_P0_3                , RULL(0x80000E3F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP2_REG_P0_4                  , RULL(0x8000123F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP2_REG_P0_4                , RULL(0x8000123F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP2_REG_P0_4                , RULL(0x8000123F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP2_REG_P1_0                , RULL(0x8000023F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP2_REG_P1_0                , RULL(0x8000023F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP2_REG_P1_1                , RULL(0x8000063F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP2_REG_P1_1                , RULL(0x8000063F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP2_REG_P1_2                , RULL(0x80000A3F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP2_REG_P1_2                , RULL(0x80000A3F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP2_REG_P1_3                , RULL(0x80000E3F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP2_REG_P1_3                , RULL(0x80000E3F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP2_REG_P1_4                , RULL(0x8000123F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP2_REG_P1_4                , RULL(0x8000123F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP2_REG_P2_0                , RULL(0x8000023F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP2_REG_P2_0                , RULL(0x8000023F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP2_REG_P2_1                , RULL(0x8000063F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP2_REG_P2_1                , RULL(0x8000063F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP2_REG_P2_2                , RULL(0x80000A3F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP2_REG_P2_2                , RULL(0x80000A3F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP2_REG_P2_3                , RULL(0x80000E3F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP2_REG_P2_3                , RULL(0x80000E3F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP2_REG_P2_4                , RULL(0x8000123F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP2_REG_P2_4                , RULL(0x8000123F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP2_REG_P3_0                , RULL(0x8000023F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP2_REG_P3_0                , RULL(0x8000023F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP2_REG_P3_1                , RULL(0x8000063F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP2_REG_P3_1                , RULL(0x8000063F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP2_REG_P3_2                , RULL(0x80000A3F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP2_REG_P3_2                , RULL(0x80000A3F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP2_REG_P3_3                , RULL(0x80000E3F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP2_REG_P3_3                , RULL(0x80000E3F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP2_REG_P3_4                , RULL(0x8000123F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP2_REG_P3_4                , RULL(0x8000123F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP3_REG_P0_0                  , RULL(0x8000033F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP3_REG_P0_0                , RULL(0x8000033F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP3_REG_P0_0                , RULL(0x8000033F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP3_REG_P0_1                  , RULL(0x8000073F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP3_REG_P0_1                , RULL(0x8000073F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP3_REG_P0_1                , RULL(0x8000073F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP3_REG_P0_2                  , RULL(0x80000B3F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP3_REG_P0_2                , RULL(0x80000B3F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP3_REG_P0_2                , RULL(0x80000B3F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP3_REG_P0_3                  , RULL(0x80000F3F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP3_REG_P0_3                , RULL(0x80000F3F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP3_REG_P0_3                , RULL(0x80000F3F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_7_RP3_REG_P0_4                  , RULL(0x8000133F0701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_7_RP3_REG_P0_4                , RULL(0x8000133F0701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_7_RP3_REG_P0_4                , RULL(0x8000133F0801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP3_REG_P1_0                , RULL(0x8000033F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP3_REG_P1_0                , RULL(0x8000033F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP3_REG_P1_1                , RULL(0x8000073F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP3_REG_P1_1                , RULL(0x8000073F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP3_REG_P1_2                , RULL(0x80000B3F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP3_REG_P1_2                , RULL(0x80000B3F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP3_REG_P1_3                , RULL(0x80000F3F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP3_REG_P1_3                , RULL(0x80000F3F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_7_RP3_REG_P1_4                , RULL(0x8000133F0701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_7_RP3_REG_P1_4                , RULL(0x8000133F0801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP3_REG_P2_0                , RULL(0x8000033F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP3_REG_P2_0                , RULL(0x8000033F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP3_REG_P2_1                , RULL(0x8000073F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP3_REG_P2_1                , RULL(0x8000073F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP3_REG_P2_2                , RULL(0x80000B3F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP3_REG_P2_2                , RULL(0x80000B3F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP3_REG_P2_3                , RULL(0x80000F3F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP3_REG_P2_3                , RULL(0x80000F3F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_7_RP3_REG_P2_4                , RULL(0x8000133F0701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_7_RP3_REG_P2_4                , RULL(0x8000133F0801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP3_REG_P3_0                , RULL(0x8000033F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP3_REG_P3_0                , RULL(0x8000033F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP3_REG_P3_1                , RULL(0x8000073F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP3_REG_P3_1                , RULL(0x8000073F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP3_REG_P3_2                , RULL(0x80000B3F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP3_REG_P3_2                , RULL(0x80000B3F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP3_REG_P3_3                , RULL(0x80000F3F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP3_REG_P3_3                , RULL(0x80000F3F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_7_RP3_REG_P3_4                , RULL(0x8000133F07011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_7_RP3_REG_P3_4                , RULL(0x8000133F08011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP0_REG_P0_0                  , RULL(0x800000400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP0_REG_P0_0                , RULL(0x800000400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP0_REG_P0_0                , RULL(0x800000400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP0_REG_P0_1                  , RULL(0x800004400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP0_REG_P0_1                , RULL(0x800004400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP0_REG_P0_1                , RULL(0x800004400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP0_REG_P0_2                  , RULL(0x800008400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP0_REG_P0_2                , RULL(0x800008400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP0_REG_P0_2                , RULL(0x800008400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP0_REG_P0_3                  , RULL(0x80000C400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP0_REG_P0_3                , RULL(0x80000C400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP0_REG_P0_3                , RULL(0x80000C400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP0_REG_P0_4                  , RULL(0x800010400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP0_REG_P0_4                , RULL(0x800010400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP0_REG_P0_4                , RULL(0x800010400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP0_REG_P1_0                , RULL(0x800000400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP0_REG_P1_0                , RULL(0x800000400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP0_REG_P1_1                , RULL(0x800004400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP0_REG_P1_1                , RULL(0x800004400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP0_REG_P1_2                , RULL(0x800008400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP0_REG_P1_2                , RULL(0x800008400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP0_REG_P1_3                , RULL(0x80000C400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP0_REG_P1_3                , RULL(0x80000C400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP0_REG_P1_4                , RULL(0x800010400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP0_REG_P1_4                , RULL(0x800010400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP0_REG_P2_0                , RULL(0x800000400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP0_REG_P2_0                , RULL(0x800000400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP0_REG_P2_1                , RULL(0x800004400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP0_REG_P2_1                , RULL(0x800004400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP0_REG_P2_2                , RULL(0x800008400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP0_REG_P2_2                , RULL(0x800008400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP0_REG_P2_3                , RULL(0x80000C400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP0_REG_P2_3                , RULL(0x80000C400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP0_REG_P2_4                , RULL(0x800010400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP0_REG_P2_4                , RULL(0x800010400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP0_REG_P3_0                , RULL(0x8000004007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP0_REG_P3_0                , RULL(0x8000004008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP0_REG_P3_1                , RULL(0x8000044007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP0_REG_P3_1                , RULL(0x8000044008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP0_REG_P3_2                , RULL(0x8000084007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP0_REG_P3_2                , RULL(0x8000084008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP0_REG_P3_3                , RULL(0x80000C4007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP0_REG_P3_3                , RULL(0x80000C4008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP0_REG_P3_4                , RULL(0x8000104007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP0_REG_P3_4                , RULL(0x8000104008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP1_REG_P0_0                  , RULL(0x800001400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP1_REG_P0_0                , RULL(0x800001400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP1_REG_P0_0                , RULL(0x800001400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP1_REG_P0_1                  , RULL(0x800005400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP1_REG_P0_1                , RULL(0x800005400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP1_REG_P0_1                , RULL(0x800005400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP1_REG_P0_2                  , RULL(0x800009400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP1_REG_P0_2                , RULL(0x800009400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP1_REG_P0_2                , RULL(0x800009400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP1_REG_P0_3                  , RULL(0x80000D400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP1_REG_P0_3                , RULL(0x80000D400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP1_REG_P0_3                , RULL(0x80000D400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP1_REG_P0_4                  , RULL(0x800011400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP1_REG_P0_4                , RULL(0x800011400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP1_REG_P0_4                , RULL(0x800011400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP1_REG_P1_0                , RULL(0x800001400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP1_REG_P1_0                , RULL(0x800001400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP1_REG_P1_1                , RULL(0x800005400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP1_REG_P1_1                , RULL(0x800005400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP1_REG_P1_2                , RULL(0x800009400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP1_REG_P1_2                , RULL(0x800009400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP1_REG_P1_3                , RULL(0x80000D400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP1_REG_P1_3                , RULL(0x80000D400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP1_REG_P1_4                , RULL(0x800011400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP1_REG_P1_4                , RULL(0x800011400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP1_REG_P2_0                , RULL(0x800001400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP1_REG_P2_0                , RULL(0x800001400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP1_REG_P2_1                , RULL(0x800005400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP1_REG_P2_1                , RULL(0x800005400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP1_REG_P2_2                , RULL(0x800009400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP1_REG_P2_2                , RULL(0x800009400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP1_REG_P2_3                , RULL(0x80000D400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP1_REG_P2_3                , RULL(0x80000D400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP1_REG_P2_4                , RULL(0x800011400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP1_REG_P2_4                , RULL(0x800011400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP1_REG_P3_0                , RULL(0x8000014007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP1_REG_P3_0                , RULL(0x8000014008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP1_REG_P3_1                , RULL(0x8000054007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP1_REG_P3_1                , RULL(0x8000054008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP1_REG_P3_2                , RULL(0x8000094007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP1_REG_P3_2                , RULL(0x8000094008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP1_REG_P3_3                , RULL(0x80000D4007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP1_REG_P3_3                , RULL(0x80000D4008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP1_REG_P3_4                , RULL(0x8000114007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP1_REG_P3_4                , RULL(0x8000114008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP2_REG_P0_0                  , RULL(0x800002400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP2_REG_P0_0                , RULL(0x800002400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP2_REG_P0_0                , RULL(0x800002400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP2_REG_P0_1                  , RULL(0x800006400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP2_REG_P0_1                , RULL(0x800006400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP2_REG_P0_1                , RULL(0x800006400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP2_REG_P0_2                  , RULL(0x80000A400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP2_REG_P0_2                , RULL(0x80000A400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP2_REG_P0_2                , RULL(0x80000A400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP2_REG_P0_3                  , RULL(0x80000E400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP2_REG_P0_3                , RULL(0x80000E400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP2_REG_P0_3                , RULL(0x80000E400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP2_REG_P0_4                  , RULL(0x800012400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP2_REG_P0_4                , RULL(0x800012400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP2_REG_P0_4                , RULL(0x800012400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP2_REG_P1_0                , RULL(0x800002400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP2_REG_P1_0                , RULL(0x800002400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP2_REG_P1_1                , RULL(0x800006400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP2_REG_P1_1                , RULL(0x800006400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP2_REG_P1_2                , RULL(0x80000A400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP2_REG_P1_2                , RULL(0x80000A400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP2_REG_P1_3                , RULL(0x80000E400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP2_REG_P1_3                , RULL(0x80000E400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP2_REG_P1_4                , RULL(0x800012400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP2_REG_P1_4                , RULL(0x800012400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP2_REG_P2_0                , RULL(0x800002400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP2_REG_P2_0                , RULL(0x800002400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP2_REG_P2_1                , RULL(0x800006400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP2_REG_P2_1                , RULL(0x800006400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP2_REG_P2_2                , RULL(0x80000A400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP2_REG_P2_2                , RULL(0x80000A400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP2_REG_P2_3                , RULL(0x80000E400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP2_REG_P2_3                , RULL(0x80000E400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP2_REG_P2_4                , RULL(0x800012400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP2_REG_P2_4                , RULL(0x800012400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP2_REG_P3_0                , RULL(0x8000024007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP2_REG_P3_0                , RULL(0x8000024008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP2_REG_P3_1                , RULL(0x8000064007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP2_REG_P3_1                , RULL(0x8000064008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP2_REG_P3_2                , RULL(0x80000A4007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP2_REG_P3_2                , RULL(0x80000A4008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP2_REG_P3_3                , RULL(0x80000E4007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP2_REG_P3_3                , RULL(0x80000E4008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP2_REG_P3_4                , RULL(0x8000124007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP2_REG_P3_4                , RULL(0x8000124008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP3_REG_P0_0                  , RULL(0x800003400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP3_REG_P0_0                , RULL(0x800003400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP3_REG_P0_0                , RULL(0x800003400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP3_REG_P0_1                  , RULL(0x800007400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP3_REG_P0_1                , RULL(0x800007400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP3_REG_P0_1                , RULL(0x800007400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP3_REG_P0_2                  , RULL(0x80000B400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP3_REG_P0_2                , RULL(0x80000B400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP3_REG_P0_2                , RULL(0x80000B400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP3_REG_P0_3                  , RULL(0x80000F400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP3_REG_P0_3                , RULL(0x80000F400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP3_REG_P0_3                , RULL(0x80000F400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_8_RP3_REG_P0_4                  , RULL(0x800013400701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_8_RP3_REG_P0_4                , RULL(0x800013400701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_8_RP3_REG_P0_4                , RULL(0x800013400801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP3_REG_P1_0                , RULL(0x800003400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP3_REG_P1_0                , RULL(0x800003400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP3_REG_P1_1                , RULL(0x800007400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP3_REG_P1_1                , RULL(0x800007400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP3_REG_P1_2                , RULL(0x80000B400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP3_REG_P1_2                , RULL(0x80000B400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP3_REG_P1_3                , RULL(0x80000F400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP3_REG_P1_3                , RULL(0x80000F400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_8_RP3_REG_P1_4                , RULL(0x800013400701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_8_RP3_REG_P1_4                , RULL(0x800013400801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP3_REG_P2_0                , RULL(0x800003400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP3_REG_P2_0                , RULL(0x800003400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP3_REG_P2_1                , RULL(0x800007400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP3_REG_P2_1                , RULL(0x800007400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP3_REG_P2_2                , RULL(0x80000B400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP3_REG_P2_2                , RULL(0x80000B400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP3_REG_P2_3                , RULL(0x80000F400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP3_REG_P2_3                , RULL(0x80000F400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_8_RP3_REG_P2_4                , RULL(0x800013400701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_8_RP3_REG_P2_4                , RULL(0x800013400801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP3_REG_P3_0                , RULL(0x8000034007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP3_REG_P3_0                , RULL(0x8000034008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP3_REG_P3_1                , RULL(0x8000074007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP3_REG_P3_1                , RULL(0x8000074008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP3_REG_P3_2                , RULL(0x80000B4007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP3_REG_P3_2                , RULL(0x80000B4008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP3_REG_P3_3                , RULL(0x80000F4007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP3_REG_P3_3                , RULL(0x80000F4008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_8_RP3_REG_P3_4                , RULL(0x8000134007011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_8_RP3_REG_P3_4                , RULL(0x8000134008011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP0_REG_P0_0                  , RULL(0x800000410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP0_REG_P0_0                , RULL(0x800000410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP0_REG_P0_0                , RULL(0x800000410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP0_REG_P0_1                  , RULL(0x800004410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP0_REG_P0_1                , RULL(0x800004410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP0_REG_P0_1                , RULL(0x800004410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP0_REG_P0_2                  , RULL(0x800008410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP0_REG_P0_2                , RULL(0x800008410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP0_REG_P0_2                , RULL(0x800008410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP0_REG_P0_3                  , RULL(0x80000C410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP0_REG_P0_3                , RULL(0x80000C410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP0_REG_P0_3                , RULL(0x80000C410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP0_REG_P0_4                  , RULL(0x800010410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP0_REG_P0_4                , RULL(0x800010410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP0_REG_P0_4                , RULL(0x800010410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP0_REG_P1_0                , RULL(0x800000410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP0_REG_P1_0                , RULL(0x800000410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP0_REG_P1_1                , RULL(0x800004410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP0_REG_P1_1                , RULL(0x800004410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP0_REG_P1_2                , RULL(0x800008410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP0_REG_P1_2                , RULL(0x800008410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP0_REG_P1_3                , RULL(0x80000C410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP0_REG_P1_3                , RULL(0x80000C410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP0_REG_P1_4                , RULL(0x800010410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP0_REG_P1_4                , RULL(0x800010410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP0_REG_P2_0                , RULL(0x800000410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP0_REG_P2_0                , RULL(0x800000410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP0_REG_P2_1                , RULL(0x800004410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP0_REG_P2_1                , RULL(0x800004410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP0_REG_P2_2                , RULL(0x800008410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP0_REG_P2_2                , RULL(0x800008410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP0_REG_P2_3                , RULL(0x80000C410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP0_REG_P2_3                , RULL(0x80000C410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP0_REG_P2_4                , RULL(0x800010410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP0_REG_P2_4                , RULL(0x800010410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP0_REG_P3_0                , RULL(0x8000004107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP0_REG_P3_0                , RULL(0x8000004108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP0_REG_P3_1                , RULL(0x8000044107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP0_REG_P3_1                , RULL(0x8000044108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP0_REG_P3_2                , RULL(0x8000084107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP0_REG_P3_2                , RULL(0x8000084108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP0_REG_P3_3                , RULL(0x80000C4107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP0_REG_P3_3                , RULL(0x80000C4108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP0_REG_P3_4                , RULL(0x8000104107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP0_REG_P3_4                , RULL(0x8000104108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP1_REG_P0_0                  , RULL(0x800001410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP1_REG_P0_0                , RULL(0x800001410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP1_REG_P0_0                , RULL(0x800001410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP1_REG_P0_1                  , RULL(0x800005410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP1_REG_P0_1                , RULL(0x800005410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP1_REG_P0_1                , RULL(0x800005410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP1_REG_P0_2                  , RULL(0x800009410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP1_REG_P0_2                , RULL(0x800009410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP1_REG_P0_2                , RULL(0x800009410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP1_REG_P0_3                  , RULL(0x80000D410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP1_REG_P0_3                , RULL(0x80000D410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP1_REG_P0_3                , RULL(0x80000D410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP1_REG_P0_4                  , RULL(0x800011410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP1_REG_P0_4                , RULL(0x800011410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP1_REG_P0_4                , RULL(0x800011410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP1_REG_P1_0                , RULL(0x800001410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP1_REG_P1_0                , RULL(0x800001410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP1_REG_P1_1                , RULL(0x800005410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP1_REG_P1_1                , RULL(0x800005410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP1_REG_P1_2                , RULL(0x800009410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP1_REG_P1_2                , RULL(0x800009410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP1_REG_P1_3                , RULL(0x80000D410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP1_REG_P1_3                , RULL(0x80000D410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP1_REG_P1_4                , RULL(0x800011410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP1_REG_P1_4                , RULL(0x800011410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP1_REG_P2_0                , RULL(0x800001410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP1_REG_P2_0                , RULL(0x800001410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP1_REG_P2_1                , RULL(0x800005410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP1_REG_P2_1                , RULL(0x800005410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP1_REG_P2_2                , RULL(0x800009410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP1_REG_P2_2                , RULL(0x800009410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP1_REG_P2_3                , RULL(0x80000D410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP1_REG_P2_3                , RULL(0x80000D410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP1_REG_P2_4                , RULL(0x800011410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP1_REG_P2_4                , RULL(0x800011410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP1_REG_P3_0                , RULL(0x8000014107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP1_REG_P3_0                , RULL(0x8000014108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP1_REG_P3_1                , RULL(0x8000054107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP1_REG_P3_1                , RULL(0x8000054108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP1_REG_P3_2                , RULL(0x8000094107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP1_REG_P3_2                , RULL(0x8000094108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP1_REG_P3_3                , RULL(0x80000D4107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP1_REG_P3_3                , RULL(0x80000D4108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP1_REG_P3_4                , RULL(0x8000114107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP1_REG_P3_4                , RULL(0x8000114108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP2_REG_P0_0                  , RULL(0x800002410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP2_REG_P0_0                , RULL(0x800002410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP2_REG_P0_0                , RULL(0x800002410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP2_REG_P0_1                  , RULL(0x800006410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP2_REG_P0_1                , RULL(0x800006410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP2_REG_P0_1                , RULL(0x800006410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP2_REG_P0_2                  , RULL(0x80000A410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP2_REG_P0_2                , RULL(0x80000A410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP2_REG_P0_2                , RULL(0x80000A410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP2_REG_P0_3                  , RULL(0x80000E410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP2_REG_P0_3                , RULL(0x80000E410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP2_REG_P0_3                , RULL(0x80000E410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP2_REG_P0_4                  , RULL(0x800012410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP2_REG_P0_4                , RULL(0x800012410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP2_REG_P0_4                , RULL(0x800012410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP2_REG_P1_0                , RULL(0x800002410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP2_REG_P1_0                , RULL(0x800002410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP2_REG_P1_1                , RULL(0x800006410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP2_REG_P1_1                , RULL(0x800006410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP2_REG_P1_2                , RULL(0x80000A410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP2_REG_P1_2                , RULL(0x80000A410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP2_REG_P1_3                , RULL(0x80000E410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP2_REG_P1_3                , RULL(0x80000E410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP2_REG_P1_4                , RULL(0x800012410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP2_REG_P1_4                , RULL(0x800012410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP2_REG_P2_0                , RULL(0x800002410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP2_REG_P2_0                , RULL(0x800002410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP2_REG_P2_1                , RULL(0x800006410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP2_REG_P2_1                , RULL(0x800006410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP2_REG_P2_2                , RULL(0x80000A410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP2_REG_P2_2                , RULL(0x80000A410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP2_REG_P2_3                , RULL(0x80000E410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP2_REG_P2_3                , RULL(0x80000E410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP2_REG_P2_4                , RULL(0x800012410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP2_REG_P2_4                , RULL(0x800012410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP2_REG_P3_0                , RULL(0x8000024107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP2_REG_P3_0                , RULL(0x8000024108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP2_REG_P3_1                , RULL(0x8000064107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP2_REG_P3_1                , RULL(0x8000064108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP2_REG_P3_2                , RULL(0x80000A4107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP2_REG_P3_2                , RULL(0x80000A4108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP2_REG_P3_3                , RULL(0x80000E4107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP2_REG_P3_3                , RULL(0x80000E4108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP2_REG_P3_4                , RULL(0x8000124107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP2_REG_P3_4                , RULL(0x8000124108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP3_REG_P0_0                  , RULL(0x800003410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP3_REG_P0_0                , RULL(0x800003410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP3_REG_P0_0                , RULL(0x800003410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP3_REG_P0_1                  , RULL(0x800007410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP3_REG_P0_1                , RULL(0x800007410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP3_REG_P0_1                , RULL(0x800007410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP3_REG_P0_2                  , RULL(0x80000B410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP3_REG_P0_2                , RULL(0x80000B410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP3_REG_P0_2                , RULL(0x80000B410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP3_REG_P0_3                  , RULL(0x80000F410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP3_REG_P0_3                , RULL(0x80000F410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP3_REG_P0_3                , RULL(0x80000F410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_DP16_WR_DELAY_VALUE_9_RP3_REG_P0_4                  , RULL(0x800013410701103F), SH_UNT_MCA      ,
       SH_ACS_SCOM_RW   );
REG64( MCA_0_DP16_WR_DELAY_VALUE_9_RP3_REG_P0_4                , RULL(0x800013410701103F), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_DP16_WR_DELAY_VALUE_9_RP3_REG_P0_4                , RULL(0x800013410801103F), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP3_REG_P1_0                , RULL(0x800003410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP3_REG_P1_0                , RULL(0x800003410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP3_REG_P1_1                , RULL(0x800007410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP3_REG_P1_1                , RULL(0x800007410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP3_REG_P1_2                , RULL(0x80000B410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP3_REG_P1_2                , RULL(0x80000B410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP3_REG_P1_3                , RULL(0x80000F410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP3_REG_P1_3                , RULL(0x80000F410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_1_DP16_WR_DELAY_VALUE_9_RP3_REG_P1_4                , RULL(0x800013410701143F), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_DP16_WR_DELAY_VALUE_9_RP3_REG_P1_4                , RULL(0x800013410801143F), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP3_REG_P2_0                , RULL(0x800003410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP3_REG_P2_0                , RULL(0x800003410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP3_REG_P2_1                , RULL(0x800007410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP3_REG_P2_1                , RULL(0x800007410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP3_REG_P2_2                , RULL(0x80000B410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP3_REG_P2_2                , RULL(0x80000B410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP3_REG_P2_3                , RULL(0x80000F410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP3_REG_P2_3                , RULL(0x80000F410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_2_DP16_WR_DELAY_VALUE_9_RP3_REG_P2_4                , RULL(0x800013410701183F), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_DP16_WR_DELAY_VALUE_9_RP3_REG_P2_4                , RULL(0x800013410801183F), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP3_REG_P3_0                , RULL(0x8000034107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP3_REG_P3_0                , RULL(0x8000034108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP3_REG_P3_1                , RULL(0x8000074107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP3_REG_P3_1                , RULL(0x8000074108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP3_REG_P3_2                , RULL(0x80000B4107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP3_REG_P3_2                , RULL(0x80000B4108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP3_REG_P3_3                , RULL(0x80000F4107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP3_REG_P3_3                , RULL(0x80000F4108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_3_DP16_WR_DELAY_VALUE_9_RP3_REG_P3_4                , RULL(0x8000134107011C3F), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_DP16_WR_DELAY_VALUE_9_RP3_REG_P3_4                , RULL(0x8000134108011C3F), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_RW   );

REG64( MCA_EICR                                                , RULL(0x07010A0D), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_EICR                                              , RULL(0x07010A0D), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_EICR                                              , RULL(0x07010A4D), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_EICR                                              , RULL(0x07010A8D), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_EICR                                              , RULL(0x07010ACD), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_EICR                                              , RULL(0x08010A0D), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_EICR                                              , RULL(0x08010A4D), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_EICR                                              , RULL(0x08010A8D), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_EICR                                              , RULL(0x08010ACD), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_ELPR                                                , RULL(0x07010A2F), SH_UNT_MCA      , SH_ACS_SCOM_RO   );
REG64( MCA_0_ELPR                                              , RULL(0x07010A2F), SH_UNT_MCA_0    , SH_ACS_SCOM_RO   );
REG64( MCA_1_ELPR                                              , RULL(0x07010A6F), SH_UNT_MCA_1    , SH_ACS_SCOM_RO   );
REG64( MCA_2_ELPR                                              , RULL(0x07010AAF), SH_UNT_MCA_2    , SH_ACS_SCOM_RO   );
REG64( MCA_3_ELPR                                              , RULL(0x07010AEF), SH_UNT_MCA_3    , SH_ACS_SCOM_RO   );
REG64( MCA_4_ELPR                                              , RULL(0x08010A2F), SH_UNT_MCA_4    , SH_ACS_SCOM_RO   );
REG64( MCA_5_ELPR                                              , RULL(0x08010A6F), SH_UNT_MCA_5    , SH_ACS_SCOM_RO   );
REG64( MCA_6_ELPR                                              , RULL(0x08010AAF), SH_UNT_MCA_6    , SH_ACS_SCOM_RO   );
REG64( MCA_7_ELPR                                              , RULL(0x08010AEF), SH_UNT_MCA_7    , SH_ACS_SCOM_RO   );

REG64( MCA_FIR                                                 , RULL(0x07010A00), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_FIR_OR                                              , RULL(0x07010A02), SH_UNT_MCA      , SH_ACS_SCOM2_OR  );
REG64( MCA_0_FIR                                               , RULL(0x07010A00), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_0_FIR_OR                                            , RULL(0x07010A02), SH_UNT_MCA_0    , SH_ACS_SCOM2_OR  );
REG64( MCA_0_WDF_FIR                                           , RULL(0x07010A01), SH_UNT_MCA_0_WDF, SH_ACS_SCOM1_AND );
REG64( MCA_1_FIR                                               , RULL(0x07010A40), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_1_FIR_OR                                            , RULL(0x07010A42), SH_UNT_MCA_1    , SH_ACS_SCOM2_OR  );
REG64( MCA_1_WDF_FIR                                           , RULL(0x07010A41), SH_UNT_MCA_1_WDF, SH_ACS_SCOM1_AND );
REG64( MCA_2_FIR                                               , RULL(0x07010A80), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_2_FIR_OR                                            , RULL(0x07010A82), SH_UNT_MCA_2    , SH_ACS_SCOM2_OR  );
REG64( MCA_2_WDF_FIR                                           , RULL(0x07010A81), SH_UNT_MCA_2_WDF, SH_ACS_SCOM1_AND );
REG64( MCA_3_FIR                                               , RULL(0x07010AC0), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_3_FIR_OR                                            , RULL(0x07010AC2), SH_UNT_MCA_3    , SH_ACS_SCOM2_OR  );
REG64( MCA_3_WDF_FIR                                           , RULL(0x07010AC1), SH_UNT_MCA_3_WDF, SH_ACS_SCOM1_AND );
REG64( MCA_4_WDF_FIR                                           , RULL(0x08010A01), SH_UNT_MCA_4_WDF, SH_ACS_SCOM1_AND );
REG64( MCA_4_FIR                                               , RULL(0x08010A00), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_4_FIR_OR                                            , RULL(0x08010A02), SH_UNT_MCA_4    , SH_ACS_SCOM2_OR  );
REG64( MCA_5_FIR                                               , RULL(0x08010A40), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_5_FIR_OR                                            , RULL(0x08010A42), SH_UNT_MCA_5    , SH_ACS_SCOM2_OR  );
REG64( MCA_5_WDF_FIR                                           , RULL(0x08010A41), SH_UNT_MCA_5_WDF, SH_ACS_SCOM1_AND );
REG64( MCA_6_WDF_FIR                                           , RULL(0x08010A81), SH_UNT_MCA_6_WDF, SH_ACS_SCOM1_AND );
REG64( MCA_6_FIR                                               , RULL(0x08010A80), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_6_FIR_OR                                            , RULL(0x08010A82), SH_UNT_MCA_6    , SH_ACS_SCOM2_OR  );
REG64( MCA_7_WDF_FIR                                           , RULL(0x08010AC1), SH_UNT_MCA_7_WDF, SH_ACS_SCOM1_AND );
REG64( MCA_7_FIR                                               , RULL(0x08010AC0), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );
REG64( MCA_7_FIR_OR                                            , RULL(0x08010AC2), SH_UNT_MCA_7    , SH_ACS_SCOM2_OR  );
REG64( MCA_WDF_FIR                                             , RULL(0x07010A01), SH_UNT_MCA_WDF  , SH_ACS_SCOM1_AND );

REG64( MCA_FWMS0                                               , RULL(0x07010A18), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_FWMS0                                             , RULL(0x07010A18), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_FWMS0                                             , RULL(0x07010A58), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_FWMS0                                             , RULL(0x07010A98), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_FWMS0                                             , RULL(0x07010AD8), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_FWMS0                                             , RULL(0x08010A18), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_FWMS0                                             , RULL(0x08010A58), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_FWMS0                                             , RULL(0x08010A98), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_FWMS0                                             , RULL(0x08010AD8), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_0_WREITE_FWMS1                                      , RULL(0x07010A19), SH_UNT_MCA_0_WREITE,
       SH_ACS_SCOM_RW   );
REG64( MCA_1_WREITE_FWMS1                                      , RULL(0x07010A59), SH_UNT_MCA_1_WREITE,
       SH_ACS_SCOM_RW   );
REG64( MCA_2_WREITE_FWMS1                                      , RULL(0x07010A99), SH_UNT_MCA_2_WREITE,
       SH_ACS_SCOM_RW   );
REG64( MCA_3_WREITE_FWMS1                                      , RULL(0x07010AD9), SH_UNT_MCA_3_WREITE,
       SH_ACS_SCOM_RW   );
REG64( MCA_4_WREITE_FWMS1                                      , RULL(0x08010A19), SH_UNT_MCA_4_WREITE,
       SH_ACS_SCOM_RW   );
REG64( MCA_5_WREITE_FWMS1                                      , RULL(0x08010A59), SH_UNT_MCA_5_WREITE,
       SH_ACS_SCOM_RW   );
REG64( MCA_6_WREITE_FWMS1                                      , RULL(0x08010A99), SH_UNT_MCA_6_WREITE,
       SH_ACS_SCOM_RW   );
REG64( MCA_7_WREITE_FWMS1                                      , RULL(0x08010AD9), SH_UNT_MCA_7_WREITE,
       SH_ACS_SCOM_RW   );
REG64( MCA_WREITE_FWMS1                                        , RULL(0x07010A19), SH_UNT_MCA_WREITE,
       SH_ACS_SCOM_RW   );

REG64( MCA_FWMS2                                               , RULL(0x07010A1A), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_FWMS2                                             , RULL(0x07010A1A), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_FWMS2                                             , RULL(0x07010A5A), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_FWMS2                                             , RULL(0x07010A9A), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_FWMS2                                             , RULL(0x07010ADA), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_FWMS2                                             , RULL(0x08010A1A), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_FWMS2                                             , RULL(0x08010A5A), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_FWMS2                                             , RULL(0x08010A9A), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_FWMS2                                             , RULL(0x08010ADA), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_FWMS3                                               , RULL(0x07010A1B), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_FWMS3                                             , RULL(0x07010A1B), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_FWMS3                                             , RULL(0x07010A5B), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_FWMS3                                             , RULL(0x07010A9B), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_FWMS3                                             , RULL(0x07010ADB), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_FWMS3                                             , RULL(0x08010A1B), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_FWMS3                                             , RULL(0x08010A5B), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_FWMS3                                             , RULL(0x08010A9B), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_FWMS3                                             , RULL(0x08010ADB), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_FWMS4                                               , RULL(0x07010A1C), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_FWMS4                                             , RULL(0x07010A1C), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_FWMS4                                             , RULL(0x07010A5C), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_FWMS4                                             , RULL(0x07010A9C), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_FWMS4                                             , RULL(0x07010ADC), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_FWMS4                                             , RULL(0x08010A1C), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_FWMS4                                             , RULL(0x08010A5C), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_FWMS4                                             , RULL(0x08010A9C), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_FWMS4                                             , RULL(0x08010ADC), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_FWMS5                                               , RULL(0x07010A1D), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_FWMS5                                             , RULL(0x07010A1D), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_FWMS5                                             , RULL(0x07010A5D), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_FWMS5                                             , RULL(0x07010A9D), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_FWMS5                                             , RULL(0x07010ADD), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_FWMS5                                             , RULL(0x08010A1D), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_FWMS5                                             , RULL(0x08010A5D), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_FWMS5                                             , RULL(0x08010A9D), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_FWMS5                                             , RULL(0x08010ADD), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_FWMS6                                               , RULL(0x07010A1E), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_FWMS6                                             , RULL(0x07010A1E), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_FWMS6                                             , RULL(0x07010A5E), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_FWMS6                                             , RULL(0x07010A9E), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_FWMS6                                             , RULL(0x07010ADE), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_FWMS6                                             , RULL(0x08010A1E), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_FWMS6                                             , RULL(0x08010A5E), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_FWMS6                                             , RULL(0x08010A9E), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_FWMS6                                             , RULL(0x08010ADE), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_FWMS7                                               , RULL(0x07010A1F), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_FWMS7                                             , RULL(0x07010A1F), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_FWMS7                                             , RULL(0x07010A5F), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_FWMS7                                             , RULL(0x07010A9F), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_FWMS7                                             , RULL(0x07010ADF), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_FWMS7                                             , RULL(0x08010A1F), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_FWMS7                                             , RULL(0x08010A5F), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_FWMS7                                             , RULL(0x08010A9F), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_FWMS7                                             , RULL(0x08010ADF), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_HCA_ACCUM_REG                                       , RULL(0x07010A32), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_HCA_ACCUM_REG                                     , RULL(0x07010A32), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_HCA_ACCUM_REG                                     , RULL(0x07010A72), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_HCA_ACCUM_REG                                     , RULL(0x07010AB2), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_HCA_ACCUM_REG                                     , RULL(0x07010AF2), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_HCA_ACCUM_REG                                     , RULL(0x08010A32), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_HCA_ACCUM_REG                                     , RULL(0x08010A72), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_HCA_ACCUM_REG                                     , RULL(0x08010AB2), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_HCA_ACCUM_REG                                     , RULL(0x08010AF2), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_HWMS0                                               , RULL(0x07010A10), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_HWMS0                                             , RULL(0x07010A10), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_HWMS0                                             , RULL(0x07010A50), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_HWMS0                                             , RULL(0x07010A90), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_HWMS0                                             , RULL(0x07010AD0), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_HWMS0                                             , RULL(0x08010A10), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_HWMS0                                             , RULL(0x08010A50), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_HWMS0                                             , RULL(0x08010A90), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_HWMS0                                             , RULL(0x08010AD0), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_0_WDF_HWMS1                                         , RULL(0x07010A11), SH_UNT_MCA_0_WDF, SH_ACS_SCOM_RW   );
REG64( MCA_1_WDF_HWMS1                                         , RULL(0x07010A51), SH_UNT_MCA_1_WDF, SH_ACS_SCOM_RW   );
REG64( MCA_2_WDF_HWMS1                                         , RULL(0x07010A91), SH_UNT_MCA_2_WDF, SH_ACS_SCOM_RW   );
REG64( MCA_3_WDF_HWMS1                                         , RULL(0x07010AD1), SH_UNT_MCA_3_WDF, SH_ACS_SCOM_RW   );
REG64( MCA_4_WDF_HWMS1                                         , RULL(0x08010A11), SH_UNT_MCA_4_WDF, SH_ACS_SCOM_RW   );
REG64( MCA_5_WDF_HWMS1                                         , RULL(0x08010A51), SH_UNT_MCA_5_WDF, SH_ACS_SCOM_RW   );
REG64( MCA_6_WDF_HWMS1                                         , RULL(0x08010A91), SH_UNT_MCA_6_WDF, SH_ACS_SCOM_RW   );
REG64( MCA_7_WDF_HWMS1                                         , RULL(0x08010AD1), SH_UNT_MCA_7_WDF, SH_ACS_SCOM_RW   );
REG64( MCA_WDF_HWMS1                                           , RULL(0x07010A11), SH_UNT_MCA_WDF  , SH_ACS_SCOM_RW   );

REG64( MCA_HWMS2                                               , RULL(0x07010A12), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_HWMS2                                             , RULL(0x07010A12), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_HWMS2                                             , RULL(0x07010A52), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_HWMS2                                             , RULL(0x07010A92), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_HWMS2                                             , RULL(0x07010AD2), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_HWMS2                                             , RULL(0x08010A12), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_HWMS2                                             , RULL(0x08010A52), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_HWMS2                                             , RULL(0x08010A92), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_HWMS2                                             , RULL(0x08010AD2), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_HWMS3                                               , RULL(0x07010A13), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_HWMS3                                             , RULL(0x07010A13), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_HWMS3                                             , RULL(0x07010A53), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_HWMS3                                             , RULL(0x07010A93), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_HWMS3                                             , RULL(0x07010AD3), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_HWMS3                                             , RULL(0x08010A13), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_HWMS3                                             , RULL(0x08010A53), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_HWMS3                                             , RULL(0x08010A93), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_HWMS3                                             , RULL(0x08010AD3), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_HWMS4                                               , RULL(0x07010A14), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_HWMS4                                             , RULL(0x07010A14), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_HWMS4                                             , RULL(0x07010A54), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_HWMS4                                             , RULL(0x07010A94), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_HWMS4                                             , RULL(0x07010AD4), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_HWMS4                                             , RULL(0x08010A14), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_HWMS4                                             , RULL(0x08010A54), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_HWMS4                                             , RULL(0x08010A94), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_HWMS4                                             , RULL(0x08010AD4), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_HWMS5                                               , RULL(0x07010A15), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_HWMS5                                             , RULL(0x07010A15), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_HWMS5                                             , RULL(0x07010A55), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_HWMS5                                             , RULL(0x07010A95), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_HWMS5                                             , RULL(0x07010AD5), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_HWMS5                                             , RULL(0x08010A15), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_HWMS5                                             , RULL(0x08010A55), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_HWMS5                                             , RULL(0x08010A95), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_HWMS5                                             , RULL(0x08010AD5), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_HWMS6                                               , RULL(0x07010A16), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_HWMS6                                             , RULL(0x07010A16), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_HWMS6                                             , RULL(0x07010A56), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_HWMS6                                             , RULL(0x07010A96), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_HWMS6                                             , RULL(0x07010AD6), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_HWMS6                                             , RULL(0x08010A16), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_HWMS6                                             , RULL(0x08010A56), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_HWMS6                                             , RULL(0x08010A96), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_HWMS6                                             , RULL(0x08010AD6), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_HWMS7                                               , RULL(0x07010A17), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_HWMS7                                             , RULL(0x07010A17), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_HWMS7                                             , RULL(0x07010A57), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_HWMS7                                             , RULL(0x07010A97), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_HWMS7                                             , RULL(0x07010AD7), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_HWMS7                                             , RULL(0x08010A17), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_HWMS7                                             , RULL(0x08010A57), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_HWMS7                                             , RULL(0x08010A97), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_HWMS7                                             , RULL(0x08010AD7), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MASK                                                , RULL(0x07010A03), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_MASK_AND                                            , RULL(0x07010A04), SH_UNT_MCA      , SH_ACS_SCOM1_AND );
REG64( MCA_MASK_OR                                             , RULL(0x07010A05), SH_UNT_MCA      , SH_ACS_SCOM2_OR  );
REG64( MCA_0_MASK                                              , RULL(0x07010A03), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_0_MASK_AND                                          , RULL(0x07010A04), SH_UNT_MCA_0    , SH_ACS_SCOM1_AND );
REG64( MCA_0_MASK_OR                                           , RULL(0x07010A05), SH_UNT_MCA_0    , SH_ACS_SCOM2_OR  );
REG64( MCA_1_MASK                                              , RULL(0x07010A43), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MASK_AND                                          , RULL(0x07010A44), SH_UNT_MCA_1    , SH_ACS_SCOM1_AND );
REG64( MCA_1_MASK_OR                                           , RULL(0x07010A45), SH_UNT_MCA_1    , SH_ACS_SCOM2_OR  );
REG64( MCA_2_MASK                                              , RULL(0x07010A83), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MASK_AND                                          , RULL(0x07010A84), SH_UNT_MCA_2    , SH_ACS_SCOM1_AND );
REG64( MCA_2_MASK_OR                                           , RULL(0x07010A85), SH_UNT_MCA_2    , SH_ACS_SCOM2_OR  );
REG64( MCA_3_MASK                                              , RULL(0x07010AC3), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MASK_AND                                          , RULL(0x07010AC4), SH_UNT_MCA_3    , SH_ACS_SCOM1_AND );
REG64( MCA_3_MASK_OR                                           , RULL(0x07010AC5), SH_UNT_MCA_3    , SH_ACS_SCOM2_OR  );
REG64( MCA_4_MASK                                              , RULL(0x08010A03), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MASK_AND                                          , RULL(0x08010A04), SH_UNT_MCA_4    , SH_ACS_SCOM1_AND );
REG64( MCA_4_MASK_OR                                           , RULL(0x08010A05), SH_UNT_MCA_4    , SH_ACS_SCOM2_OR  );
REG64( MCA_5_MASK                                              , RULL(0x08010A43), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MASK_AND                                          , RULL(0x08010A44), SH_UNT_MCA_5    , SH_ACS_SCOM1_AND );
REG64( MCA_5_MASK_OR                                           , RULL(0x08010A45), SH_UNT_MCA_5    , SH_ACS_SCOM2_OR  );
REG64( MCA_6_MASK                                              , RULL(0x08010A83), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MASK_AND                                          , RULL(0x08010A84), SH_UNT_MCA_6    , SH_ACS_SCOM1_AND );
REG64( MCA_6_MASK_OR                                           , RULL(0x08010A85), SH_UNT_MCA_6    , SH_ACS_SCOM2_OR  );
REG64( MCA_7_MASK                                              , RULL(0x08010AC3), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MASK_AND                                          , RULL(0x08010AC4), SH_UNT_MCA_7    , SH_ACS_SCOM1_AND );
REG64( MCA_7_MASK_OR                                           , RULL(0x08010AC5), SH_UNT_MCA_7    , SH_ACS_SCOM2_OR  );

REG64( MCA_MBACALFIRQ                                          , RULL(0x07010900), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_MBACALFIRQ_AND                                      , RULL(0x07010901), SH_UNT_MCA      , SH_ACS_SCOM1_AND );
REG64( MCA_MBACALFIRQ_OR                                       , RULL(0x07010902), SH_UNT_MCA      , SH_ACS_SCOM2_OR  );
REG64( MCA_0_MBACALFIRQ                                        , RULL(0x07010900), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBACALFIRQ_AND                                    , RULL(0x07010901), SH_UNT_MCA_0    , SH_ACS_SCOM1_AND );
REG64( MCA_0_MBACALFIRQ_OR                                     , RULL(0x07010902), SH_UNT_MCA_0    , SH_ACS_SCOM2_OR  );
REG64( MCA_1_MBACALFIRQ                                        , RULL(0x07010940), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBACALFIRQ_AND                                    , RULL(0x07010941), SH_UNT_MCA_1    , SH_ACS_SCOM1_AND );
REG64( MCA_1_MBACALFIRQ_OR                                     , RULL(0x07010942), SH_UNT_MCA_1    , SH_ACS_SCOM2_OR  );
REG64( MCA_2_MBACALFIRQ                                        , RULL(0x07010980), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBACALFIRQ_AND                                    , RULL(0x07010981), SH_UNT_MCA_2    , SH_ACS_SCOM1_AND );
REG64( MCA_2_MBACALFIRQ_OR                                     , RULL(0x07010982), SH_UNT_MCA_2    , SH_ACS_SCOM2_OR  );
REG64( MCA_3_MBACALFIRQ                                        , RULL(0x070109C0), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBACALFIRQ_AND                                    , RULL(0x070109C1), SH_UNT_MCA_3    , SH_ACS_SCOM1_AND );
REG64( MCA_3_MBACALFIRQ_OR                                     , RULL(0x070109C2), SH_UNT_MCA_3    , SH_ACS_SCOM2_OR  );
REG64( MCA_4_MBACALFIRQ                                        , RULL(0x08010900), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBACALFIRQ_AND                                    , RULL(0x08010901), SH_UNT_MCA_4    , SH_ACS_SCOM1_AND );
REG64( MCA_4_MBACALFIRQ_OR                                     , RULL(0x08010902), SH_UNT_MCA_4    , SH_ACS_SCOM2_OR  );
REG64( MCA_5_MBACALFIRQ                                        , RULL(0x08010940), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBACALFIRQ_AND                                    , RULL(0x08010941), SH_UNT_MCA_5    , SH_ACS_SCOM1_AND );
REG64( MCA_5_MBACALFIRQ_OR                                     , RULL(0x08010942), SH_UNT_MCA_5    , SH_ACS_SCOM2_OR  );
REG64( MCA_6_MBACALFIRQ                                        , RULL(0x08010980), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBACALFIRQ_AND                                    , RULL(0x08010981), SH_UNT_MCA_6    , SH_ACS_SCOM1_AND );
REG64( MCA_6_MBACALFIRQ_OR                                     , RULL(0x08010982), SH_UNT_MCA_6    , SH_ACS_SCOM2_OR  );
REG64( MCA_7_MBACALFIRQ                                        , RULL(0x080109C0), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBACALFIRQ_AND                                    , RULL(0x080109C1), SH_UNT_MCA_7    , SH_ACS_SCOM1_AND );
REG64( MCA_7_MBACALFIRQ_OR                                     , RULL(0x080109C2), SH_UNT_MCA_7    , SH_ACS_SCOM2_OR  );

REG64( MCA_MBACALFIR_ACTION0                                   , RULL(0x07010906), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBACALFIR_ACTION0                                 , RULL(0x07010906), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBACALFIR_ACTION0                                 , RULL(0x07010946), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBACALFIR_ACTION0                                 , RULL(0x07010986), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBACALFIR_ACTION0                                 , RULL(0x070109C6), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBACALFIR_ACTION0                                 , RULL(0x08010906), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBACALFIR_ACTION0                                 , RULL(0x08010946), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBACALFIR_ACTION0                                 , RULL(0x08010986), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBACALFIR_ACTION0                                 , RULL(0x080109C6), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBACALFIR_ACTION1                                   , RULL(0x07010907), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBACALFIR_ACTION1                                 , RULL(0x07010907), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBACALFIR_ACTION1                                 , RULL(0x07010947), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBACALFIR_ACTION1                                 , RULL(0x07010987), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBACALFIR_ACTION1                                 , RULL(0x070109C7), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBACALFIR_ACTION1                                 , RULL(0x08010907), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBACALFIR_ACTION1                                 , RULL(0x08010947), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBACALFIR_ACTION1                                 , RULL(0x08010987), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBACALFIR_ACTION1                                 , RULL(0x080109C7), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBACALFIR_MASK                                      , RULL(0x07010903), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_MBACALFIR_MASK_AND                                  , RULL(0x07010904), SH_UNT_MCA      , SH_ACS_SCOM1_AND );
REG64( MCA_MBACALFIR_MASK_OR                                   , RULL(0x07010905), SH_UNT_MCA      , SH_ACS_SCOM2_OR  );
REG64( MCA_0_MBACALFIR_MASK                                    , RULL(0x07010903), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBACALFIR_MASK_AND                                , RULL(0x07010904), SH_UNT_MCA_0    , SH_ACS_SCOM1_AND );
REG64( MCA_0_MBACALFIR_MASK_OR                                 , RULL(0x07010905), SH_UNT_MCA_0    , SH_ACS_SCOM2_OR  );
REG64( MCA_1_MBACALFIR_MASK                                    , RULL(0x07010943), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBACALFIR_MASK_AND                                , RULL(0x07010944), SH_UNT_MCA_1    , SH_ACS_SCOM1_AND );
REG64( MCA_1_MBACALFIR_MASK_OR                                 , RULL(0x07010945), SH_UNT_MCA_1    , SH_ACS_SCOM2_OR  );
REG64( MCA_2_MBACALFIR_MASK                                    , RULL(0x07010983), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBACALFIR_MASK_AND                                , RULL(0x07010984), SH_UNT_MCA_2    , SH_ACS_SCOM1_AND );
REG64( MCA_2_MBACALFIR_MASK_OR                                 , RULL(0x07010985), SH_UNT_MCA_2    , SH_ACS_SCOM2_OR  );
REG64( MCA_3_MBACALFIR_MASK                                    , RULL(0x070109C3), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBACALFIR_MASK_AND                                , RULL(0x070109C4), SH_UNT_MCA_3    , SH_ACS_SCOM1_AND );
REG64( MCA_3_MBACALFIR_MASK_OR                                 , RULL(0x070109C5), SH_UNT_MCA_3    , SH_ACS_SCOM2_OR  );
REG64( MCA_4_MBACALFIR_MASK                                    , RULL(0x08010903), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBACALFIR_MASK_AND                                , RULL(0x08010904), SH_UNT_MCA_4    , SH_ACS_SCOM1_AND );
REG64( MCA_4_MBACALFIR_MASK_OR                                 , RULL(0x08010905), SH_UNT_MCA_4    , SH_ACS_SCOM2_OR  );
REG64( MCA_5_MBACALFIR_MASK                                    , RULL(0x08010943), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBACALFIR_MASK_AND                                , RULL(0x08010944), SH_UNT_MCA_5    , SH_ACS_SCOM1_AND );
REG64( MCA_5_MBACALFIR_MASK_OR                                 , RULL(0x08010945), SH_UNT_MCA_5    , SH_ACS_SCOM2_OR  );
REG64( MCA_6_MBACALFIR_MASK                                    , RULL(0x08010983), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBACALFIR_MASK_AND                                , RULL(0x08010984), SH_UNT_MCA_6    , SH_ACS_SCOM1_AND );
REG64( MCA_6_MBACALFIR_MASK_OR                                 , RULL(0x08010985), SH_UNT_MCA_6    , SH_ACS_SCOM2_OR  );
REG64( MCA_7_MBACALFIR_MASK                                    , RULL(0x080109C3), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBACALFIR_MASK_AND                                , RULL(0x080109C4), SH_UNT_MCA_7    , SH_ACS_SCOM1_AND );
REG64( MCA_7_MBACALFIR_MASK_OR                                 , RULL(0x080109C5), SH_UNT_MCA_7    , SH_ACS_SCOM2_OR  );

REG64( MCA_MBAREF0Q                                            , RULL(0x07010932), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBAREF0Q                                          , RULL(0x07010932), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBAREF0Q                                          , RULL(0x07010972), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBAREF0Q                                          , RULL(0x070109B2), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBAREF0Q                                          , RULL(0x070109F2), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBAREF0Q                                          , RULL(0x08010932), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBAREF0Q                                          , RULL(0x08010972), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBAREF0Q                                          , RULL(0x080109B2), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBAREF0Q                                          , RULL(0x080109F2), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBAREFAQ                                            , RULL(0x07010936), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBAREFAQ                                          , RULL(0x07010936), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBAREFAQ                                          , RULL(0x07010976), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBAREFAQ                                          , RULL(0x070109B6), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBAREFAQ                                          , RULL(0x070109F6), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBAREFAQ                                          , RULL(0x08010936), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBAREFAQ                                          , RULL(0x08010976), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBAREFAQ                                          , RULL(0x080109B6), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBAREFAQ                                          , RULL(0x080109F6), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBARPC0Q                                            , RULL(0x07010934), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBARPC0Q                                          , RULL(0x07010934), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBARPC0Q                                          , RULL(0x07010974), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBARPC0Q                                          , RULL(0x070109B4), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBARPC0Q                                          , RULL(0x070109F4), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBARPC0Q                                          , RULL(0x08010934), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBARPC0Q                                          , RULL(0x08010974), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBARPC0Q                                          , RULL(0x080109B4), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBARPC0Q                                          , RULL(0x080109F4), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBASTR0Q                                            , RULL(0x07010935), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBASTR0Q                                          , RULL(0x07010935), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBASTR0Q                                          , RULL(0x07010975), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBASTR0Q                                          , RULL(0x070109B5), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBASTR0Q                                          , RULL(0x070109F5), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBASTR0Q                                          , RULL(0x08010935), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBASTR0Q                                          , RULL(0x08010975), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBASTR0Q                                          , RULL(0x080109B5), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBASTR0Q                                          , RULL(0x080109F5), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBAUER0Q                                         , RULL(0x0701236E), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBAUER0Q                                       , RULL(0x0701236E), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBAUER0Q                                       , RULL(0x0801236E), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBAUER1Q                                         , RULL(0x07012373), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBAUER1Q                                       , RULL(0x07012373), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBAUER1Q                                       , RULL(0x08012373), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBAUER2Q                                         , RULL(0x07012378), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBAUER2Q                                       , RULL(0x07012378), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBAUER2Q                                       , RULL(0x08012378), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBAUER3Q                                         , RULL(0x0701237D), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBAUER3Q                                       , RULL(0x0701237D), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBAUER3Q                                       , RULL(0x0801237D), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_CAL0Q                                           , RULL(0x0701090F), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_CAL0Q                                         , RULL(0x0701090F), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_CAL0Q                                         , RULL(0x0701094F), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_CAL0Q                                         , RULL(0x0701098F), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_CAL0Q                                         , RULL(0x070109CF), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_CAL0Q                                         , RULL(0x0801090F), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_CAL0Q                                         , RULL(0x0801094F), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_CAL0Q                                         , RULL(0x0801098F), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_CAL0Q                                         , RULL(0x080109CF), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_CAL1Q                                           , RULL(0x07010910), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_CAL1Q                                         , RULL(0x07010910), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_CAL1Q                                         , RULL(0x07010950), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_CAL1Q                                         , RULL(0x07010990), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_CAL1Q                                         , RULL(0x070109D0), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_CAL1Q                                         , RULL(0x08010910), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_CAL1Q                                         , RULL(0x08010950), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_CAL1Q                                         , RULL(0x08010990), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_CAL1Q                                         , RULL(0x080109D0), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_CAL2Q                                           , RULL(0x07010911), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_CAL2Q                                         , RULL(0x07010911), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_CAL2Q                                         , RULL(0x07010951), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_CAL2Q                                         , RULL(0x07010991), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_CAL2Q                                         , RULL(0x070109D1), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_CAL2Q                                         , RULL(0x08010911), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_CAL2Q                                         , RULL(0x08010951), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_CAL2Q                                         , RULL(0x08010991), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_CAL2Q                                         , RULL(0x080109D1), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_CAL3Q                                           , RULL(0x07010912), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_CAL3Q                                         , RULL(0x07010912), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_CAL3Q                                         , RULL(0x07010952), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_CAL3Q                                         , RULL(0x07010992), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_CAL3Q                                         , RULL(0x070109D2), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_CAL3Q                                         , RULL(0x08010912), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_CAL3Q                                         , RULL(0x08010952), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_CAL3Q                                         , RULL(0x08010992), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_CAL3Q                                         , RULL(0x080109D2), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_DBG0Q                                           , RULL(0x07010926), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_DBG0Q                                         , RULL(0x07010926), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_DBG0Q                                         , RULL(0x07010966), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_DBG0Q                                         , RULL(0x070109A6), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_DBG0Q                                         , RULL(0x070109E6), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_DBG0Q                                         , RULL(0x08010926), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_DBG0Q                                         , RULL(0x08010966), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_DBG0Q                                         , RULL(0x080109A6), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_DBG0Q                                         , RULL(0x080109E6), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_DBG1Q                                           , RULL(0x07010927), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_DBG1Q                                         , RULL(0x07010927), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_DBG1Q                                         , RULL(0x07010967), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_DBG1Q                                         , RULL(0x070109A7), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_DBG1Q                                         , RULL(0x070109E7), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_DBG1Q                                         , RULL(0x08010927), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_DBG1Q                                         , RULL(0x08010967), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_DBG1Q                                         , RULL(0x080109A7), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_DBG1Q                                         , RULL(0x080109E7), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_DSM0Q                                           , RULL(0x0701090A), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_DSM0Q                                         , RULL(0x0701090A), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_DSM0Q                                         , RULL(0x0701094A), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_DSM0Q                                         , RULL(0x0701098A), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_DSM0Q                                         , RULL(0x070109CA), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_DSM0Q                                         , RULL(0x0801090A), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_DSM0Q                                         , RULL(0x0801094A), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_DSM0Q                                         , RULL(0x0801098A), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_DSM0Q                                         , RULL(0x080109CA), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_ERR_REPORTQ                                     , RULL(0x0701091A), SH_UNT_MCA      , SH_ACS_SCOM_RO   );
REG64( MCA_0_MBA_ERR_REPORTQ                                   , RULL(0x0701091A), SH_UNT_MCA_0    , SH_ACS_SCOM_RO   );
REG64( MCA_1_MBA_ERR_REPORTQ                                   , RULL(0x0701095A), SH_UNT_MCA_1    , SH_ACS_SCOM_RO   );
REG64( MCA_2_MBA_ERR_REPORTQ                                   , RULL(0x0701099A), SH_UNT_MCA_2    , SH_ACS_SCOM_RO   );
REG64( MCA_3_MBA_ERR_REPORTQ                                   , RULL(0x070109DA), SH_UNT_MCA_3    , SH_ACS_SCOM_RO   );
REG64( MCA_4_MBA_ERR_REPORTQ                                   , RULL(0x0801091A), SH_UNT_MCA_4    , SH_ACS_SCOM_RO   );
REG64( MCA_5_MBA_ERR_REPORTQ                                   , RULL(0x0801095A), SH_UNT_MCA_5    , SH_ACS_SCOM_RO   );
REG64( MCA_6_MBA_ERR_REPORTQ                                   , RULL(0x0801099A), SH_UNT_MCA_6    , SH_ACS_SCOM_RO   );
REG64( MCA_7_MBA_ERR_REPORTQ                                   , RULL(0x080109DA), SH_UNT_MCA_7    , SH_ACS_SCOM_RO   );

REG64( MCA_MBA_FARB0Q                                          , RULL(0x07010913), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_FARB0Q                                        , RULL(0x07010913), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_FARB0Q                                        , RULL(0x07010953), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_FARB0Q                                        , RULL(0x07010993), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_FARB0Q                                        , RULL(0x070109D3), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_FARB0Q                                        , RULL(0x08010913), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_FARB0Q                                        , RULL(0x08010953), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_FARB0Q                                        , RULL(0x08010993), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_FARB0Q                                        , RULL(0x080109D3), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_FARB1Q                                          , RULL(0x07010914), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_FARB1Q                                        , RULL(0x07010914), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_FARB1Q                                        , RULL(0x07010954), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_FARB1Q                                        , RULL(0x07010994), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_FARB1Q                                        , RULL(0x070109D4), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_FARB1Q                                        , RULL(0x08010914), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_FARB1Q                                        , RULL(0x08010954), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_FARB1Q                                        , RULL(0x08010994), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_FARB1Q                                        , RULL(0x080109D4), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_FARB2Q                                          , RULL(0x07010915), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_FARB2Q                                        , RULL(0x07010915), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_FARB2Q                                        , RULL(0x07010955), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_FARB2Q                                        , RULL(0x07010995), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_FARB2Q                                        , RULL(0x070109D5), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_FARB2Q                                        , RULL(0x08010915), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_FARB2Q                                        , RULL(0x08010955), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_FARB2Q                                        , RULL(0x08010995), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_FARB2Q                                        , RULL(0x080109D5), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_FARB3Q                                          , RULL(0x07010916), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_FARB3Q                                        , RULL(0x07010916), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_FARB3Q                                        , RULL(0x07010956), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_FARB3Q                                        , RULL(0x07010996), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_FARB3Q                                        , RULL(0x070109D6), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_FARB3Q                                        , RULL(0x08010916), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_FARB3Q                                        , RULL(0x08010956), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_FARB3Q                                        , RULL(0x08010996), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_FARB3Q                                        , RULL(0x080109D6), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_FARB4Q                                          , RULL(0x07010917), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_FARB4Q                                        , RULL(0x07010917), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_FARB4Q                                        , RULL(0x07010957), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_FARB4Q                                        , RULL(0x07010997), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_FARB4Q                                        , RULL(0x070109D7), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_FARB4Q                                        , RULL(0x08010917), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_FARB4Q                                        , RULL(0x08010957), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_FARB4Q                                        , RULL(0x08010997), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_FARB4Q                                        , RULL(0x080109D7), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_FARB5Q                                          , RULL(0x07010918), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_FARB5Q                                        , RULL(0x07010918), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_FARB5Q                                        , RULL(0x07010958), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_FARB5Q                                        , RULL(0x07010998), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_FARB5Q                                        , RULL(0x070109D8), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_FARB5Q                                        , RULL(0x08010918), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_FARB5Q                                        , RULL(0x08010958), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_FARB5Q                                        , RULL(0x08010998), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_FARB5Q                                        , RULL(0x080109D8), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_FARB6Q                                          , RULL(0x07010919), SH_UNT_MCA      , SH_ACS_SCOM_RO   );
REG64( MCA_0_MBA_FARB6Q                                        , RULL(0x07010919), SH_UNT_MCA_0    , SH_ACS_SCOM_RO   );
REG64( MCA_1_MBA_FARB6Q                                        , RULL(0x07010959), SH_UNT_MCA_1    , SH_ACS_SCOM_RO   );
REG64( MCA_2_MBA_FARB6Q                                        , RULL(0x07010999), SH_UNT_MCA_2    , SH_ACS_SCOM_RO   );
REG64( MCA_3_MBA_FARB6Q                                        , RULL(0x070109D9), SH_UNT_MCA_3    , SH_ACS_SCOM_RO   );
REG64( MCA_4_MBA_FARB6Q                                        , RULL(0x08010919), SH_UNT_MCA_4    , SH_ACS_SCOM_RO   );
REG64( MCA_5_MBA_FARB6Q                                        , RULL(0x08010959), SH_UNT_MCA_5    , SH_ACS_SCOM_RO   );
REG64( MCA_6_MBA_FARB6Q                                        , RULL(0x08010999), SH_UNT_MCA_6    , SH_ACS_SCOM_RO   );
REG64( MCA_7_MBA_FARB6Q                                        , RULL(0x080109D9), SH_UNT_MCA_7    , SH_ACS_SCOM_RO   );

REG64( MCA_MBA_FARB7Q                                          , RULL(0x07010925), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_FARB7Q                                        , RULL(0x07010925), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_FARB7Q                                        , RULL(0x07010965), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_FARB7Q                                        , RULL(0x070109A5), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_FARB7Q                                        , RULL(0x070109E5), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_FARB7Q                                        , RULL(0x08010925), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_FARB7Q                                        , RULL(0x08010965), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_FARB7Q                                        , RULL(0x080109A5), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_FARB7Q                                        , RULL(0x080109E5), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_FARB8Q                                          , RULL(0x07010928), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_FARB8Q                                        , RULL(0x07010928), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_FARB8Q                                        , RULL(0x07010968), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_FARB8Q                                        , RULL(0x070109A8), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_FARB8Q                                        , RULL(0x070109E8), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_FARB8Q                                        , RULL(0x08010928), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_FARB8Q                                        , RULL(0x08010968), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_FARB8Q                                        , RULL(0x080109A8), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_FARB8Q                                        , RULL(0x080109E8), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBA_MCBERRPTQ                                    , RULL(0x070123E7), SH_UNT_MCBIST   , SH_ACS_SCOM_RO   );
REG64( MCBIST_0_MBA_MCBERRPTQ                                  , RULL(0x070123E7), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RO   );
REG64( MCBIST_1_MBA_MCBERRPTQ                                  , RULL(0x080123E7), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RO   );

REG64( MCA_MBA_PMU0Q                                           , RULL(0x07010937), SH_UNT_MCA      , SH_ACS_SCOM_RO   );
REG64( MCA_0_MBA_PMU0Q                                         , RULL(0x07010937), SH_UNT_MCA_0    , SH_ACS_SCOM_RO   );
REG64( MCA_1_MBA_PMU0Q                                         , RULL(0x07010977), SH_UNT_MCA_1    , SH_ACS_SCOM_RO   );
REG64( MCA_2_MBA_PMU0Q                                         , RULL(0x070109B7), SH_UNT_MCA_2    , SH_ACS_SCOM_RO   );
REG64( MCA_3_MBA_PMU0Q                                         , RULL(0x070109F7), SH_UNT_MCA_3    , SH_ACS_SCOM_RO   );
REG64( MCA_4_MBA_PMU0Q                                         , RULL(0x08010937), SH_UNT_MCA_4    , SH_ACS_SCOM_RO   );
REG64( MCA_5_MBA_PMU0Q                                         , RULL(0x08010977), SH_UNT_MCA_5    , SH_ACS_SCOM_RO   );
REG64( MCA_6_MBA_PMU0Q                                         , RULL(0x080109B7), SH_UNT_MCA_6    , SH_ACS_SCOM_RO   );
REG64( MCA_7_MBA_PMU0Q                                         , RULL(0x080109F7), SH_UNT_MCA_7    , SH_ACS_SCOM_RO   );

REG64( MCA_MBA_PMU1Q                                           , RULL(0x07010938), SH_UNT_MCA      , SH_ACS_SCOM_RO   );
REG64( MCA_0_MBA_PMU1Q                                         , RULL(0x07010938), SH_UNT_MCA_0    , SH_ACS_SCOM_RO   );
REG64( MCA_1_MBA_PMU1Q                                         , RULL(0x07010978), SH_UNT_MCA_1    , SH_ACS_SCOM_RO   );
REG64( MCA_2_MBA_PMU1Q                                         , RULL(0x070109B8), SH_UNT_MCA_2    , SH_ACS_SCOM_RO   );
REG64( MCA_3_MBA_PMU1Q                                         , RULL(0x070109F8), SH_UNT_MCA_3    , SH_ACS_SCOM_RO   );
REG64( MCA_4_MBA_PMU1Q                                         , RULL(0x08010938), SH_UNT_MCA_4    , SH_ACS_SCOM_RO   );
REG64( MCA_5_MBA_PMU1Q                                         , RULL(0x08010978), SH_UNT_MCA_5    , SH_ACS_SCOM_RO   );
REG64( MCA_6_MBA_PMU1Q                                         , RULL(0x080109B8), SH_UNT_MCA_6    , SH_ACS_SCOM_RO   );
REG64( MCA_7_MBA_PMU1Q                                         , RULL(0x080109F8), SH_UNT_MCA_7    , SH_ACS_SCOM_RO   );

REG64( MCA_MBA_PMU2Q                                           , RULL(0x07010939), SH_UNT_MCA      , SH_ACS_SCOM_RO   );
REG64( MCA_0_MBA_PMU2Q                                         , RULL(0x07010939), SH_UNT_MCA_0    , SH_ACS_SCOM_RO   );
REG64( MCA_1_MBA_PMU2Q                                         , RULL(0x07010979), SH_UNT_MCA_1    , SH_ACS_SCOM_RO   );
REG64( MCA_2_MBA_PMU2Q                                         , RULL(0x070109B9), SH_UNT_MCA_2    , SH_ACS_SCOM_RO   );
REG64( MCA_3_MBA_PMU2Q                                         , RULL(0x070109F9), SH_UNT_MCA_3    , SH_ACS_SCOM_RO   );
REG64( MCA_4_MBA_PMU2Q                                         , RULL(0x08010939), SH_UNT_MCA_4    , SH_ACS_SCOM_RO   );
REG64( MCA_5_MBA_PMU2Q                                         , RULL(0x08010979), SH_UNT_MCA_5    , SH_ACS_SCOM_RO   );
REG64( MCA_6_MBA_PMU2Q                                         , RULL(0x080109B9), SH_UNT_MCA_6    , SH_ACS_SCOM_RO   );
REG64( MCA_7_MBA_PMU2Q                                         , RULL(0x080109F9), SH_UNT_MCA_7    , SH_ACS_SCOM_RO   );

REG64( MCA_MBA_PMU3Q                                           , RULL(0x0701093A), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_PMU3Q                                         , RULL(0x0701093A), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_PMU3Q                                         , RULL(0x0701097A), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_PMU3Q                                         , RULL(0x070109BA), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_PMU3Q                                         , RULL(0x070109FA), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_PMU3Q                                         , RULL(0x0801093A), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_PMU3Q                                         , RULL(0x0801097A), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_PMU3Q                                         , RULL(0x080109BA), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_PMU3Q                                         , RULL(0x080109FA), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_PMU4Q                                           , RULL(0x0701093B), SH_UNT_MCA      , SH_ACS_SCOM_RO   );
REG64( MCA_0_MBA_PMU4Q                                         , RULL(0x0701093B), SH_UNT_MCA_0    , SH_ACS_SCOM_RO   );
REG64( MCA_1_MBA_PMU4Q                                         , RULL(0x0701097B), SH_UNT_MCA_1    , SH_ACS_SCOM_RO   );
REG64( MCA_2_MBA_PMU4Q                                         , RULL(0x070109BB), SH_UNT_MCA_2    , SH_ACS_SCOM_RO   );
REG64( MCA_3_MBA_PMU4Q                                         , RULL(0x070109FB), SH_UNT_MCA_3    , SH_ACS_SCOM_RO   );
REG64( MCA_4_MBA_PMU4Q                                         , RULL(0x0801093B), SH_UNT_MCA_4    , SH_ACS_SCOM_RO   );
REG64( MCA_5_MBA_PMU4Q                                         , RULL(0x0801097B), SH_UNT_MCA_5    , SH_ACS_SCOM_RO   );
REG64( MCA_6_MBA_PMU4Q                                         , RULL(0x080109BB), SH_UNT_MCA_6    , SH_ACS_SCOM_RO   );
REG64( MCA_7_MBA_PMU4Q                                         , RULL(0x080109FB), SH_UNT_MCA_7    , SH_ACS_SCOM_RO   );

REG64( MCA_MBA_PMU5Q                                           , RULL(0x0701093C), SH_UNT_MCA      , SH_ACS_SCOM_RO   );
REG64( MCA_0_MBA_PMU5Q                                         , RULL(0x0701093C), SH_UNT_MCA_0    , SH_ACS_SCOM_RO   );
REG64( MCA_1_MBA_PMU5Q                                         , RULL(0x0701097C), SH_UNT_MCA_1    , SH_ACS_SCOM_RO   );
REG64( MCA_2_MBA_PMU5Q                                         , RULL(0x070109BC), SH_UNT_MCA_2    , SH_ACS_SCOM_RO   );
REG64( MCA_3_MBA_PMU5Q                                         , RULL(0x070109FC), SH_UNT_MCA_3    , SH_ACS_SCOM_RO   );
REG64( MCA_4_MBA_PMU5Q                                         , RULL(0x0801093C), SH_UNT_MCA_4    , SH_ACS_SCOM_RO   );
REG64( MCA_5_MBA_PMU5Q                                         , RULL(0x0801097C), SH_UNT_MCA_5    , SH_ACS_SCOM_RO   );
REG64( MCA_6_MBA_PMU5Q                                         , RULL(0x080109BC), SH_UNT_MCA_6    , SH_ACS_SCOM_RO   );
REG64( MCA_7_MBA_PMU5Q                                         , RULL(0x080109FC), SH_UNT_MCA_7    , SH_ACS_SCOM_RO   );

REG64( MCA_MBA_PMU6Q                                           , RULL(0x0701093D), SH_UNT_MCA      , SH_ACS_SCOM_RO   );
REG64( MCA_0_MBA_PMU6Q                                         , RULL(0x0701093D), SH_UNT_MCA_0    , SH_ACS_SCOM_RO   );
REG64( MCA_1_MBA_PMU6Q                                         , RULL(0x0701097D), SH_UNT_MCA_1    , SH_ACS_SCOM_RO   );
REG64( MCA_2_MBA_PMU6Q                                         , RULL(0x070109BD), SH_UNT_MCA_2    , SH_ACS_SCOM_RO   );
REG64( MCA_3_MBA_PMU6Q                                         , RULL(0x070109FD), SH_UNT_MCA_3    , SH_ACS_SCOM_RO   );
REG64( MCA_4_MBA_PMU6Q                                         , RULL(0x0801093D), SH_UNT_MCA_4    , SH_ACS_SCOM_RO   );
REG64( MCA_5_MBA_PMU6Q                                         , RULL(0x0801097D), SH_UNT_MCA_5    , SH_ACS_SCOM_RO   );
REG64( MCA_6_MBA_PMU6Q                                         , RULL(0x080109BD), SH_UNT_MCA_6    , SH_ACS_SCOM_RO   );
REG64( MCA_7_MBA_PMU6Q                                         , RULL(0x080109FD), SH_UNT_MCA_7    , SH_ACS_SCOM_RO   );

REG64( MCA_MBA_PMU7Q                                           , RULL(0x0701093E), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_PMU7Q                                         , RULL(0x0701093E), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_PMU7Q                                         , RULL(0x0701097E), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_PMU7Q                                         , RULL(0x070109BE), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_PMU7Q                                         , RULL(0x070109FE), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_PMU7Q                                         , RULL(0x0801093E), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_PMU7Q                                         , RULL(0x0801097E), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_PMU7Q                                         , RULL(0x080109BE), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_PMU7Q                                         , RULL(0x080109FE), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_PMU8Q                                           , RULL(0x0701093F), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_PMU8Q                                         , RULL(0x0701093F), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_PMU8Q                                         , RULL(0x0701097F), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_PMU8Q                                         , RULL(0x070109BF), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_PMU8Q                                         , RULL(0x070109FF), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_PMU8Q                                         , RULL(0x0801093F), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_PMU8Q                                         , RULL(0x0801097F), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_PMU8Q                                         , RULL(0x080109BF), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_PMU8Q                                         , RULL(0x080109FF), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_RRQ0Q                                           , RULL(0x0701090E), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_RRQ0Q                                         , RULL(0x0701090E), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_RRQ0Q                                         , RULL(0x0701094E), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_RRQ0Q                                         , RULL(0x0701098E), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_RRQ0Q                                         , RULL(0x070109CE), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_RRQ0Q                                         , RULL(0x0801090E), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_RRQ0Q                                         , RULL(0x0801094E), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_RRQ0Q                                         , RULL(0x0801098E), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_RRQ0Q                                         , RULL(0x080109CE), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_TMR0Q                                           , RULL(0x0701090B), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_TMR0Q                                         , RULL(0x0701090B), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_TMR0Q                                         , RULL(0x0701094B), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_TMR0Q                                         , RULL(0x0701098B), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_TMR0Q                                         , RULL(0x070109CB), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_TMR0Q                                         , RULL(0x0801090B), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_TMR0Q                                         , RULL(0x0801094B), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_TMR0Q                                         , RULL(0x0801098B), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_TMR0Q                                         , RULL(0x080109CB), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_TMR1Q                                           , RULL(0x0701090C), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_TMR1Q                                         , RULL(0x0701090C), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_TMR1Q                                         , RULL(0x0701094C), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_TMR1Q                                         , RULL(0x0701098C), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_TMR1Q                                         , RULL(0x070109CC), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_TMR1Q                                         , RULL(0x0801090C), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_TMR1Q                                         , RULL(0x0801094C), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_TMR1Q                                         , RULL(0x0801098C), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_TMR1Q                                         , RULL(0x080109CC), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_TMR2Q                                           , RULL(0x0701092F), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_TMR2Q                                         , RULL(0x0701092F), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_TMR2Q                                         , RULL(0x0701096F), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_TMR2Q                                         , RULL(0x070109AF), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_TMR2Q                                         , RULL(0x070109EF), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_TMR2Q                                         , RULL(0x0801092F), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_TMR2Q                                         , RULL(0x0801096F), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_TMR2Q                                         , RULL(0x080109AF), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_TMR2Q                                         , RULL(0x080109EF), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_MBA_WRQ0Q                                           , RULL(0x0701090D), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBA_WRQ0Q                                         , RULL(0x0701090D), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBA_WRQ0Q                                         , RULL(0x0701094D), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBA_WRQ0Q                                         , RULL(0x0701098D), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBA_WRQ0Q                                         , RULL(0x070109CD), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBA_WRQ0Q                                         , RULL(0x0801090D), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBA_WRQ0Q                                         , RULL(0x0801094D), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBA_WRQ0Q                                         , RULL(0x0801098D), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBA_WRQ0Q                                         , RULL(0x080109CD), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBECTLQ                                          , RULL(0x07012310), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBECTLQ                                        , RULL(0x07012310), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBECTLQ                                        , RULL(0x08012310), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCA_MBMDI                                               , RULL(0x07010A2D), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MBMDI                                             , RULL(0x07010A2D), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MBMDI                                             , RULL(0x07010A6D), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MBMDI                                             , RULL(0x07010AAD), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MBMDI                                             , RULL(0x07010AED), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MBMDI                                             , RULL(0x08010A2D), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MBMDI                                             , RULL(0x08010A6D), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MBMDI                                             , RULL(0x08010AAD), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MBMDI                                             , RULL(0x08010AED), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBMPER0Q                                         , RULL(0x0701236C), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBMPER0Q                                       , RULL(0x0701236C), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBMPER0Q                                       , RULL(0x0801236C), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBMPER1Q                                         , RULL(0x07012371), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBMPER1Q                                       , RULL(0x07012371), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBMPER1Q                                       , RULL(0x08012371), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBMPER2Q                                         , RULL(0x07012376), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBMPER2Q                                       , RULL(0x07012376), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBMPER2Q                                       , RULL(0x08012376), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBMPER3Q                                         , RULL(0x0701237B), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBMPER3Q                                       , RULL(0x0701237B), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBMPER3Q                                       , RULL(0x0801237B), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBNCER0Q                                         , RULL(0x0701236A), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBNCER0Q                                       , RULL(0x0701236A), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBNCER0Q                                       , RULL(0x0801236A), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBNCER1Q                                         , RULL(0x0701236F), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBNCER1Q                                       , RULL(0x0701236F), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBNCER1Q                                       , RULL(0x0801236F), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBNCER2Q                                         , RULL(0x07012374), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBNCER2Q                                       , RULL(0x07012374), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBNCER2Q                                       , RULL(0x08012374), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBNCER3Q                                         , RULL(0x07012379), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBNCER3Q                                       , RULL(0x07012379), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBNCER3Q                                       , RULL(0x08012379), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBRCER0Q                                         , RULL(0x0701236B), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBRCER0Q                                       , RULL(0x0701236B), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBRCER0Q                                       , RULL(0x0801236B), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBRCER1Q                                         , RULL(0x07012370), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBRCER1Q                                       , RULL(0x07012370), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBRCER1Q                                       , RULL(0x08012370), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBRCER2Q                                         , RULL(0x07012375), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBRCER2Q                                       , RULL(0x07012375), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBRCER2Q                                       , RULL(0x08012375), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBRCER3Q                                         , RULL(0x0701237A), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBRCER3Q                                       , RULL(0x0701237A), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBRCER3Q                                       , RULL(0x0801237A), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBSEC0Q                                          , RULL(0x07012355), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBSEC0Q                                        , RULL(0x07012355), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBSEC0Q                                        , RULL(0x08012355), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBSEC1Q                                          , RULL(0x07012356), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBSEC1Q                                        , RULL(0x07012356), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBSEC1Q                                        , RULL(0x08012356), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBSEVR0Q                                         , RULL(0x0701237E), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBSEVR0Q                                       , RULL(0x0701237E), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBSEVR0Q                                       , RULL(0x0801237E), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBSEVR1Q                                         , RULL(0x0701237F), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBSEVR1Q                                       , RULL(0x0701237F), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBSEVR1Q                                       , RULL(0x0801237F), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBSMODESQ                                        , RULL(0x07012362), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBSMODESQ                                      , RULL(0x07012362), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBSMODESQ                                      , RULL(0x08012362), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBSMSECQ                                         , RULL(0x07012369), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBSMSECQ                                       , RULL(0x07012369), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBSMSECQ                                       , RULL(0x08012369), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBSSYMEC0Q                                       , RULL(0x07012358), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBSSYMEC0Q                                     , RULL(0x07012358), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBSSYMEC0Q                                     , RULL(0x08012358), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBSSYMEC1Q                                       , RULL(0x07012359), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBSSYMEC1Q                                     , RULL(0x07012359), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBSSYMEC1Q                                     , RULL(0x08012359), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBSSYMEC2Q                                       , RULL(0x0701235A), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBSSYMEC2Q                                     , RULL(0x0701235A), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBSSYMEC2Q                                     , RULL(0x0801235A), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBSSYMEC3Q                                       , RULL(0x0701235B), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBSSYMEC3Q                                     , RULL(0x0701235B), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBSSYMEC3Q                                     , RULL(0x0801235B), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBSSYMEC4Q                                       , RULL(0x0701235C), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBSSYMEC4Q                                     , RULL(0x0701235C), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBSSYMEC4Q                                     , RULL(0x0801235C), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBSSYMEC5Q                                       , RULL(0x0701235D), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBSSYMEC5Q                                     , RULL(0x0701235D), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBSSYMEC5Q                                     , RULL(0x0801235D), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBSSYMEC6Q                                       , RULL(0x0701235E), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBSSYMEC6Q                                     , RULL(0x0701235E), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBSSYMEC6Q                                     , RULL(0x0801235E), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBSSYMEC7Q                                       , RULL(0x0701235F), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBSSYMEC7Q                                     , RULL(0x0701235F), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBSSYMEC7Q                                     , RULL(0x0801235F), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBSSYMEC8Q                                       , RULL(0x07012360), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBSSYMEC8Q                                     , RULL(0x07012360), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBSSYMEC8Q                                     , RULL(0x08012360), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBSTRQ                                           , RULL(0x07012357), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBSTRQ                                         , RULL(0x07012357), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBSTRQ                                         , RULL(0x08012357), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBUER0Q                                          , RULL(0x0701236D), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBUER0Q                                        , RULL(0x0701236D), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBUER0Q                                        , RULL(0x0801236D), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBUER1Q                                          , RULL(0x07012372), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBUER1Q                                        , RULL(0x07012372), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBUER1Q                                        , RULL(0x08012372), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBUER2Q                                          , RULL(0x07012377), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBUER2Q                                        , RULL(0x07012377), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBUER2Q                                        , RULL(0x08012377), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MBUER3Q                                          , RULL(0x0701237C), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MBUER3Q                                        , RULL(0x0701237C), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MBUER3Q                                        , RULL(0x0801237C), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCS_0_PORT02_MCAMOC                                     , RULL(0x05010825), SH_UNT_MCS_0_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_0_PORT13_MCAMOC                                     , RULL(0x05010835), SH_UNT_MCS_0_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT02_MCAMOC                                     , RULL(0x050108A5), SH_UNT_MCS_1_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT13_MCAMOC                                     , RULL(0x050108B5), SH_UNT_MCS_1_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT02_MCAMOC                                     , RULL(0x03010825), SH_UNT_MCS_2_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT13_MCAMOC                                     , RULL(0x03010835), SH_UNT_MCS_2_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT02_MCAMOC                                     , RULL(0x030108A5), SH_UNT_MCS_3_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT13_MCAMOC                                     , RULL(0x030108B5), SH_UNT_MCS_3_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT02_MCAMOC                                       , RULL(0x05010825), SH_UNT_MCS_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT13_MCAMOC                                       , RULL(0x05010835), SH_UNT_MCS_PORT13,
       SH_ACS_SCOM3_RW  );

REG64( MCBIST_MCBACQ                                           , RULL(0x070123D5), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBACQ                                         , RULL(0x070123D5), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBACQ                                         , RULL(0x080123D5), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBAGRAQ                                         , RULL(0x070123D6), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBAGRAQ                                       , RULL(0x070123D6), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBAGRAQ                                       , RULL(0x080123D6), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBAMR0A0Q                                       , RULL(0x070123C8), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBAMR0A0Q                                     , RULL(0x070123C8), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBAMR0A0Q                                     , RULL(0x080123C8), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBAMR1A0Q                                       , RULL(0x070123C9), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBAMR1A0Q                                     , RULL(0x070123C9), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBAMR1A0Q                                     , RULL(0x080123C9), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBAMR2A0Q                                       , RULL(0x070123CA), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBAMR2A0Q                                     , RULL(0x070123CA), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBAMR2A0Q                                     , RULL(0x080123CA), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBAMR3A0Q                                       , RULL(0x070123CB), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBAMR3A0Q                                     , RULL(0x070123CB), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBAMR3A0Q                                     , RULL(0x080123CB), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBCFGQ                                          , RULL(0x070123E0), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBCFGQ                                        , RULL(0x070123E0), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBCFGQ                                        , RULL(0x080123E0), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCA_MCBCM                                               , RULL(0x07010A2C), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_MCBCM                                             , RULL(0x07010A2C), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_MCBCM                                             , RULL(0x07010A6C), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_MCBCM                                             , RULL(0x07010AAC), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_MCBCM                                             , RULL(0x07010AEC), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_MCBCM                                             , RULL(0x08010A2C), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_MCBCM                                             , RULL(0x08010A6C), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_MCBCM                                             , RULL(0x08010AAC), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_MCBCM                                             , RULL(0x08010AEC), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBDRCRQ                                         , RULL(0x070123BD), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBDRCRQ                                       , RULL(0x070123BD), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBDRCRQ                                       , RULL(0x080123BD), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBDRSRQ                                         , RULL(0x070123BC), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBDRSRQ                                       , RULL(0x070123BC), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBDRSRQ                                       , RULL(0x080123BC), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBEA0Q                                          , RULL(0x070123CE), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBEA0Q                                        , RULL(0x070123CE), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBEA0Q                                        , RULL(0x080123CE), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBEA1Q                                          , RULL(0x070123CF), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBEA1Q                                        , RULL(0x070123CF), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBEA1Q                                        , RULL(0x080123CF), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBEA2Q                                          , RULL(0x070123D2), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBEA2Q                                        , RULL(0x070123D2), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBEA2Q                                        , RULL(0x080123D2), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBEA3Q                                          , RULL(0x070123D3), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBEA3Q                                        , RULL(0x070123D3), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBEA3Q                                        , RULL(0x080123D3), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBFD0Q                                          , RULL(0x070123BE), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBFD0Q                                        , RULL(0x070123BE), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBFD0Q                                        , RULL(0x080123BE), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBFD1Q                                          , RULL(0x070123BF), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBFD1Q                                        , RULL(0x070123BF), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBFD1Q                                        , RULL(0x080123BF), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBFD2Q                                          , RULL(0x070123C0), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBFD2Q                                        , RULL(0x070123C0), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBFD2Q                                        , RULL(0x080123C0), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBFD3Q                                          , RULL(0x070123C1), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBFD3Q                                        , RULL(0x070123C1), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBFD3Q                                        , RULL(0x080123C1), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBFD4Q                                          , RULL(0x070123C2), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBFD4Q                                        , RULL(0x070123C2), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBFD4Q                                        , RULL(0x080123C2), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBFD5Q                                          , RULL(0x070123C3), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBFD5Q                                        , RULL(0x070123C3), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBFD5Q                                        , RULL(0x080123C3), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBFD6Q                                          , RULL(0x070123C4), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBFD6Q                                        , RULL(0x070123C4), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBFD6Q                                        , RULL(0x080123C4), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBFD7Q                                          , RULL(0x070123C5), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBFD7Q                                        , RULL(0x070123C5), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBFD7Q                                        , RULL(0x080123C5), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBFDQ                                           , RULL(0x070123C6), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBFDQ                                         , RULL(0x070123C6), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBFDQ                                         , RULL(0x080123C6), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBISTFIRACT0                                    , RULL(0x07012306), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBISTFIRACT0                                  , RULL(0x07012306), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBISTFIRACT0                                  , RULL(0x08012306), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBISTFIRACT1                                    , RULL(0x07012307), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBISTFIRACT1                                  , RULL(0x07012307), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBISTFIRACT1                                  , RULL(0x08012307), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBISTFIRMASK                                    , RULL(0x07012303), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_MCBISTFIRMASK_AND                                , RULL(0x07012304), SH_UNT_MCBIST   , SH_ACS_SCOM1_AND );
REG64( MCBIST_MCBISTFIRMASK_OR                                 , RULL(0x07012305), SH_UNT_MCBIST   , SH_ACS_SCOM2_OR  );
REG64( MCBIST_0_MCBISTFIRMASK                                  , RULL(0x07012303), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBISTFIRMASK_AND                              , RULL(0x07012304), SH_UNT_MCBIST_0 , SH_ACS_SCOM1_AND );
REG64( MCBIST_0_MCBISTFIRMASK_OR                               , RULL(0x07012305), SH_UNT_MCBIST_0 , SH_ACS_SCOM2_OR  );
REG64( MCBIST_1_MCBISTFIRMASK                                  , RULL(0x08012303), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBISTFIRMASK_AND                              , RULL(0x08012304), SH_UNT_MCBIST_1 , SH_ACS_SCOM1_AND );
REG64( MCBIST_1_MCBISTFIRMASK_OR                               , RULL(0x08012305), SH_UNT_MCBIST_1 , SH_ACS_SCOM2_OR  );

REG64( MCBIST_MCBISTFIRQ                                       , RULL(0x07012300), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_MCBISTFIRQ_AND                                   , RULL(0x07012301), SH_UNT_MCBIST   , SH_ACS_SCOM1_AND );
REG64( MCBIST_MCBISTFIRQ_OR                                    , RULL(0x07012302), SH_UNT_MCBIST   , SH_ACS_SCOM2_OR  );
REG64( MCBIST_0_MCBISTFIRQ                                     , RULL(0x07012300), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBISTFIRQ_AND                                 , RULL(0x07012301), SH_UNT_MCBIST_0 , SH_ACS_SCOM1_AND );
REG64( MCBIST_0_MCBISTFIRQ_OR                                  , RULL(0x07012302), SH_UNT_MCBIST_0 , SH_ACS_SCOM2_OR  );
REG64( MCBIST_1_MCBISTFIRQ                                     , RULL(0x08012300), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBISTFIRQ_AND                                 , RULL(0x08012301), SH_UNT_MCBIST_1 , SH_ACS_SCOM1_AND );
REG64( MCBIST_1_MCBISTFIRQ_OR                                  , RULL(0x08012302), SH_UNT_MCBIST_1 , SH_ACS_SCOM2_OR  );

REG64( MCBIST_MCBISTFIRWOF                                     , RULL(0x07012308), SH_UNT_MCBIST   ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCBIST_0_MCBISTFIRWOF                                   , RULL(0x07012308), SH_UNT_MCBIST_0 ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCBIST_1_MCBISTFIRWOF                                   , RULL(0x08012308), SH_UNT_MCBIST_1 ,
       SH_ACS_SCOM_WCLRREG );

REG64( MCBIST_MCBLFSRA0Q                                       , RULL(0x070123D4), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBLFSRA0Q                                     , RULL(0x070123D4), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBLFSRA0Q                                     , RULL(0x080123D4), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBMCATQ                                         , RULL(0x070123D7), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBMCATQ                                       , RULL(0x070123D7), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBMCATQ                                       , RULL(0x080123D7), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBMR0Q                                          , RULL(0x070123A8), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBMR0Q                                        , RULL(0x070123A8), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBMR0Q                                        , RULL(0x080123A8), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBMR1Q                                          , RULL(0x070123A9), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBMR1Q                                        , RULL(0x070123A9), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBMR1Q                                        , RULL(0x080123A9), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBMR2Q                                          , RULL(0x070123AA), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBMR2Q                                        , RULL(0x070123AA), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBMR2Q                                        , RULL(0x080123AA), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBMR3Q                                          , RULL(0x070123AB), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBMR3Q                                        , RULL(0x070123AB), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBMR3Q                                        , RULL(0x080123AB), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBMR4Q                                          , RULL(0x070123AC), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBMR4Q                                        , RULL(0x070123AC), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBMR4Q                                        , RULL(0x080123AC), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBMR5Q                                          , RULL(0x070123AD), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBMR5Q                                        , RULL(0x070123AD), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBMR5Q                                        , RULL(0x080123AD), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBMR6Q                                          , RULL(0x070123AE), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBMR6Q                                        , RULL(0x070123AE), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBMR6Q                                        , RULL(0x080123AE), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBMR7Q                                          , RULL(0x070123DF), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBMR7Q                                        , RULL(0x070123DF), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBMR7Q                                        , RULL(0x080123DF), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBPARMQ                                         , RULL(0x070123AF), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBPARMQ                                       , RULL(0x070123AF), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBPARMQ                                       , RULL(0x080123AF), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBRCRQ                                          , RULL(0x070123B1), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBRCRQ                                        , RULL(0x070123B1), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBRCRQ                                        , RULL(0x080123B1), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBRDS0Q                                         , RULL(0x070123B2), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBRDS0Q                                       , RULL(0x070123B2), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBRDS0Q                                       , RULL(0x080123B2), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBRDS1Q                                         , RULL(0x070123B3), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBRDS1Q                                       , RULL(0x070123B3), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBRDS1Q                                       , RULL(0x080123B3), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBSA0Q                                          , RULL(0x070123CC), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBSA0Q                                        , RULL(0x070123CC), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBSA0Q                                        , RULL(0x080123CC), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBSA1Q                                          , RULL(0x070123CD), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBSA1Q                                        , RULL(0x070123CD), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBSA1Q                                        , RULL(0x080123CD), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBSA2Q                                          , RULL(0x070123D0), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBSA2Q                                        , RULL(0x070123D0), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBSA2Q                                        , RULL(0x080123D0), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBSA3Q                                          , RULL(0x070123D1), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCBSA3Q                                        , RULL(0x070123D1), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCBSA3Q                                        , RULL(0x080123D1), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCBSTATQ                                         , RULL(0x07012366), SH_UNT_MCBIST   , SH_ACS_SCOM_RO   );
REG64( MCBIST_0_MCBSTATQ                                       , RULL(0x07012366), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RO   );
REG64( MCBIST_1_MCBSTATQ                                       , RULL(0x08012366), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RO   );

REG64( MCS_0_PORT02_MCBUSYQ                                    , RULL(0x05010827), SH_UNT_MCS_0_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_0_PORT13_MCBUSYQ                                    , RULL(0x05010837), SH_UNT_MCS_0_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT02_MCBUSYQ                                    , RULL(0x050108A7), SH_UNT_MCS_1_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT13_MCBUSYQ                                    , RULL(0x050108B7), SH_UNT_MCS_1_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT02_MCBUSYQ                                    , RULL(0x03010827), SH_UNT_MCS_2_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT13_MCBUSYQ                                    , RULL(0x03010837), SH_UNT_MCS_2_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT02_MCBUSYQ                                    , RULL(0x030108A7), SH_UNT_MCS_3_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT13_MCBUSYQ                                    , RULL(0x030108B7), SH_UNT_MCS_3_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT02_MCBUSYQ                                      , RULL(0x05010827), SH_UNT_MCS_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT13_MCBUSYQ                                      , RULL(0x05010837), SH_UNT_MCS_PORT13,
       SH_ACS_SCOM3_RW  );

REG64( MCBIST_MCB_CNTLQ                                        , RULL(0x070123DB), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCB_CNTLQ                                      , RULL(0x070123DB), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCB_CNTLQ                                      , RULL(0x080123DB), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_MCB_CNTLSTATQ                                    , RULL(0x070123DC), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_MCB_CNTLSTATQ                                  , RULL(0x070123DC), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_MCB_CNTLSTATQ                                  , RULL(0x080123DC), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCS_MCDBG0                                              , RULL(0x05010816), SH_UNT_MCS      , SH_ACS_SCOM_RO   );
REG64( MCS_0_MCDBG0                                            , RULL(0x05010816), SH_UNT_MCS_0    , SH_ACS_SCOM_RO   );
REG64( MCS_1_MCDBG0                                            , RULL(0x05010896), SH_UNT_MCS_1    , SH_ACS_SCOM_RO   );
REG64( MCS_2_MCDBG0                                            , RULL(0x03010816), SH_UNT_MCS_2    , SH_ACS_SCOM_RO   );
REG64( MCS_3_MCDBG0                                            , RULL(0x03010896), SH_UNT_MCS_3    , SH_ACS_SCOM_RO   );

REG64( MCS_MCDBG1                                              , RULL(0x05010817), SH_UNT_MCS      , SH_ACS_SCOM_RO   );
REG64( MCS_0_MCDBG1                                            , RULL(0x05010817), SH_UNT_MCS_0    , SH_ACS_SCOM_RO   );
REG64( MCS_1_MCDBG1                                            , RULL(0x05010897), SH_UNT_MCS_1    , SH_ACS_SCOM_RO   );
REG64( MCS_2_MCDBG1                                            , RULL(0x03010817), SH_UNT_MCS_2    , SH_ACS_SCOM_RO   );
REG64( MCS_3_MCDBG1                                            , RULL(0x03010897), SH_UNT_MCS_3    , SH_ACS_SCOM_RO   );

REG64( MCS_0_PORT02_MCEBUSCL                                   , RULL(0x05010829), SH_UNT_MCS_0_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_0_PORT13_MCEBUSCL                                   , RULL(0x05010839), SH_UNT_MCS_0_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT02_MCEBUSCL                                   , RULL(0x050108A9), SH_UNT_MCS_1_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT13_MCEBUSCL                                   , RULL(0x050108B9), SH_UNT_MCS_1_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT02_MCEBUSCL                                   , RULL(0x03010829), SH_UNT_MCS_2_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT13_MCEBUSCL                                   , RULL(0x03010839), SH_UNT_MCS_2_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT02_MCEBUSCL                                   , RULL(0x030108A9), SH_UNT_MCS_3_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT13_MCEBUSCL                                   , RULL(0x030108B9), SH_UNT_MCS_3_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT02_MCEBUSCL                                     , RULL(0x05010829), SH_UNT_MCS_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT13_MCEBUSCL                                     , RULL(0x05010839), SH_UNT_MCS_PORT13,
       SH_ACS_SCOM3_RW  );

REG64( MCS_0_PORT13_MCEBUSEN0                                  , RULL(0x0501083C), SH_UNT_MCS_0_PORT13,
       SH_ACS_SCOM_RW   );
REG64( MCS_1_PORT13_MCEBUSEN0                                  , RULL(0x050108BC), SH_UNT_MCS_1_PORT13,
       SH_ACS_SCOM_RW   );
REG64( MCS_2_PORT13_MCEBUSEN0                                  , RULL(0x0301083C), SH_UNT_MCS_2_PORT13,
       SH_ACS_SCOM_RW   );
REG64( MCS_3_PORT13_MCEBUSEN0                                  , RULL(0x030108BC), SH_UNT_MCS_3_PORT13,
       SH_ACS_SCOM_RW   );
REG64( MCS_PORT13_MCEBUSEN0                                    , RULL(0x0501083C), SH_UNT_MCS_PORT13,
       SH_ACS_SCOM_RW   );

REG64( MCS_0_PORT13_MCEBUSEN1                                  , RULL(0x0501083D), SH_UNT_MCS_0_PORT13,
       SH_ACS_SCOM_RW   );
REG64( MCS_1_PORT13_MCEBUSEN1                                  , RULL(0x050108BD), SH_UNT_MCS_1_PORT13,
       SH_ACS_SCOM_RW   );
REG64( MCS_2_PORT13_MCEBUSEN1                                  , RULL(0x0301083D), SH_UNT_MCS_2_PORT13,
       SH_ACS_SCOM_RW   );
REG64( MCS_3_PORT13_MCEBUSEN1                                  , RULL(0x030108BD), SH_UNT_MCS_3_PORT13,
       SH_ACS_SCOM_RW   );
REG64( MCS_PORT13_MCEBUSEN1                                    , RULL(0x0501083D), SH_UNT_MCS_PORT13,
       SH_ACS_SCOM_RW   );

REG64( MCS_0_PORT13_MCEBUSEN2                                  , RULL(0x0501083E), SH_UNT_MCS_0_PORT13,
       SH_ACS_SCOM_RW   );
REG64( MCS_1_PORT13_MCEBUSEN2                                  , RULL(0x050108BE), SH_UNT_MCS_1_PORT13,
       SH_ACS_SCOM_RW   );
REG64( MCS_2_PORT13_MCEBUSEN2                                  , RULL(0x0301083E), SH_UNT_MCS_2_PORT13,
       SH_ACS_SCOM_RW   );
REG64( MCS_3_PORT13_MCEBUSEN2                                  , RULL(0x030108BE), SH_UNT_MCS_3_PORT13,
       SH_ACS_SCOM_RW   );
REG64( MCS_PORT13_MCEBUSEN2                                    , RULL(0x0501083E), SH_UNT_MCS_PORT13,
       SH_ACS_SCOM_RW   );

REG64( MCS_0_PORT13_MCEBUSEN3                                  , RULL(0x0501083F), SH_UNT_MCS_0_PORT13,
       SH_ACS_SCOM_RW   );
REG64( MCS_1_PORT13_MCEBUSEN3                                  , RULL(0x050108BF), SH_UNT_MCS_1_PORT13,
       SH_ACS_SCOM_RW   );
REG64( MCS_2_PORT13_MCEBUSEN3                                  , RULL(0x0301083F), SH_UNT_MCS_2_PORT13,
       SH_ACS_SCOM_RW   );
REG64( MCS_3_PORT13_MCEBUSEN3                                  , RULL(0x030108BF), SH_UNT_MCS_3_PORT13,
       SH_ACS_SCOM_RW   );
REG64( MCS_PORT13_MCEBUSEN3                                    , RULL(0x0501083F), SH_UNT_MCS_PORT13,
       SH_ACS_SCOM_RW   );

REG64( MCS_0_PORT02_MCEPSQ                                     , RULL(0x05010826), SH_UNT_MCS_0_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_0_PORT13_MCEPSQ                                     , RULL(0x05010836), SH_UNT_MCS_0_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT02_MCEPSQ                                     , RULL(0x050108A6), SH_UNT_MCS_1_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT13_MCEPSQ                                     , RULL(0x050108B6), SH_UNT_MCS_1_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT02_MCEPSQ                                     , RULL(0x03010826), SH_UNT_MCS_2_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT13_MCEPSQ                                     , RULL(0x03010836), SH_UNT_MCS_2_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT02_MCEPSQ                                     , RULL(0x030108A6), SH_UNT_MCS_3_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT13_MCEPSQ                                     , RULL(0x030108B6), SH_UNT_MCS_3_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT02_MCEPSQ                                       , RULL(0x05010826), SH_UNT_MCS_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT13_MCEPSQ                                       , RULL(0x05010836), SH_UNT_MCS_PORT13,
       SH_ACS_SCOM3_RW  );

REG64( MCS_MCERPT0                                             , RULL(0x0501081E), SH_UNT_MCS      , SH_ACS_SCOM_RO   );
REG64( MCS_0_MCERPT0                                           , RULL(0x0501081E), SH_UNT_MCS_0    , SH_ACS_SCOM_RO   );
REG64( MCS_1_MCERPT0                                           , RULL(0x0501089E), SH_UNT_MCS_1    , SH_ACS_SCOM_RO   );
REG64( MCS_2_MCERPT0                                           , RULL(0x0301081E), SH_UNT_MCS_2    , SH_ACS_SCOM_RO   );
REG64( MCS_3_MCERPT0                                           , RULL(0x0301089E), SH_UNT_MCS_3    , SH_ACS_SCOM_RO   );

REG64( MCS_MCERPT1                                             , RULL(0x0501081F), SH_UNT_MCS      , SH_ACS_SCOM_RO   );
REG64( MCS_0_MCERPT1                                           , RULL(0x0501081F), SH_UNT_MCS_0    , SH_ACS_SCOM_RO   );
REG64( MCS_1_MCERPT1                                           , RULL(0x0501089F), SH_UNT_MCS_1    , SH_ACS_SCOM_RO   );
REG64( MCS_2_MCERPT1                                           , RULL(0x0301081F), SH_UNT_MCS_2    , SH_ACS_SCOM_RO   );
REG64( MCS_3_MCERPT1                                           , RULL(0x0301089F), SH_UNT_MCS_3    , SH_ACS_SCOM_RO   );

REG64( MCS_MCERPT2                                             , RULL(0x0501081A), SH_UNT_MCS      , SH_ACS_SCOM_RO   );
REG64( MCS_0_MCERPT2                                           , RULL(0x0501081A), SH_UNT_MCS_0    , SH_ACS_SCOM_RO   );
REG64( MCS_1_MCERPT2                                           , RULL(0x0501089A), SH_UNT_MCS_1    , SH_ACS_SCOM_RO   );
REG64( MCS_2_MCERPT2                                           , RULL(0x0301081A), SH_UNT_MCS_2    , SH_ACS_SCOM_RO   );
REG64( MCS_3_MCERPT2                                           , RULL(0x0301089A), SH_UNT_MCS_3    , SH_ACS_SCOM_RO   );

REG64( MCS_0_PORT02_MCERRINJ                                   , RULL(0x05010828), SH_UNT_MCS_0_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_0_PORT13_MCERRINJ                                   , RULL(0x05010838), SH_UNT_MCS_0_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT02_MCERRINJ                                   , RULL(0x050108A8), SH_UNT_MCS_1_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT13_MCERRINJ                                   , RULL(0x050108B8), SH_UNT_MCS_1_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT02_MCERRINJ                                   , RULL(0x03010828), SH_UNT_MCS_2_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT13_MCERRINJ                                   , RULL(0x03010838), SH_UNT_MCS_2_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT02_MCERRINJ                                   , RULL(0x030108A8), SH_UNT_MCS_3_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT13_MCERRINJ                                   , RULL(0x030108B8), SH_UNT_MCS_3_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT02_MCERRINJ                                     , RULL(0x05010828), SH_UNT_MCS_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT13_MCERRINJ                                     , RULL(0x05010838), SH_UNT_MCS_PORT13,
       SH_ACS_SCOM3_RW  );

REG64( MCS_MCFGP                                               , RULL(0x0501080A), SH_UNT_MCS      , SH_ACS_SCOM_RW   );
REG64( MCS_0_MCFGP                                             , RULL(0x0501080A), SH_UNT_MCS_0    , SH_ACS_SCOM_RW   );
REG64( MCS_1_MCFGP                                             , RULL(0x0501088A), SH_UNT_MCS_1    , SH_ACS_SCOM_RW   );
REG64( MCS_2_MCFGP                                             , RULL(0x0301080A), SH_UNT_MCS_2    , SH_ACS_SCOM_RW   );
REG64( MCS_3_MCFGP                                             , RULL(0x0301088A), SH_UNT_MCS_3    , SH_ACS_SCOM_RW   );

REG64( MCS_MCFGPA                                              , RULL(0x0501080B), SH_UNT_MCS      , SH_ACS_SCOM_RW   );
REG64( MCS_0_MCFGPA                                            , RULL(0x0501080B), SH_UNT_MCS_0    , SH_ACS_SCOM_RW   );
REG64( MCS_1_MCFGPA                                            , RULL(0x0501088B), SH_UNT_MCS_1    , SH_ACS_SCOM_RW   );
REG64( MCS_2_MCFGPA                                            , RULL(0x0301080B), SH_UNT_MCS_2    , SH_ACS_SCOM_RW   );
REG64( MCS_3_MCFGPA                                            , RULL(0x0301088B), SH_UNT_MCS_3    , SH_ACS_SCOM_RW   );

REG64( MCS_MCFGPM                                              , RULL(0x0501080C), SH_UNT_MCS      , SH_ACS_SCOM_RW   );
REG64( MCS_0_MCFGPM                                            , RULL(0x0501080C), SH_UNT_MCS_0    , SH_ACS_SCOM_RW   );
REG64( MCS_1_MCFGPM                                            , RULL(0x0501088C), SH_UNT_MCS_1    , SH_ACS_SCOM_RW   );
REG64( MCS_2_MCFGPM                                            , RULL(0x0301080C), SH_UNT_MCS_2    , SH_ACS_SCOM_RW   );
REG64( MCS_3_MCFGPM                                            , RULL(0x0301088C), SH_UNT_MCS_3    , SH_ACS_SCOM_RW   );

REG64( MCS_MCFGPMA                                             , RULL(0x0501080D), SH_UNT_MCS      , SH_ACS_SCOM_RW   );
REG64( MCS_0_MCFGPMA                                           , RULL(0x0501080D), SH_UNT_MCS_0    , SH_ACS_SCOM_RW   );
REG64( MCS_1_MCFGPMA                                           , RULL(0x0501088D), SH_UNT_MCS_1    , SH_ACS_SCOM_RW   );
REG64( MCS_2_MCFGPMA                                           , RULL(0x0301080D), SH_UNT_MCS_2    , SH_ACS_SCOM_RW   );
REG64( MCS_3_MCFGPMA                                           , RULL(0x0301088D), SH_UNT_MCS_3    , SH_ACS_SCOM_RW   );

REG64( MCS_MCFIR                                               , RULL(0x05010800), SH_UNT_MCS      , SH_ACS_SCOM_RW   );
REG64( MCS_MCFIR_AND                                           , RULL(0x05010801), SH_UNT_MCS      , SH_ACS_SCOM1_AND );
REG64( MCS_MCFIR_OR                                            , RULL(0x05010802), SH_UNT_MCS      , SH_ACS_SCOM2_OR  );
REG64( MCS_0_MCFIR                                             , RULL(0x05010800), SH_UNT_MCS_0    , SH_ACS_SCOM_RW   );
REG64( MCS_0_MCFIR_AND                                         , RULL(0x05010801), SH_UNT_MCS_0    , SH_ACS_SCOM1_AND );
REG64( MCS_0_MCFIR_OR                                          , RULL(0x05010802), SH_UNT_MCS_0    , SH_ACS_SCOM2_OR  );
REG64( MCS_1_MCFIR                                             , RULL(0x05010880), SH_UNT_MCS_1    , SH_ACS_SCOM_RW   );
REG64( MCS_1_MCFIR_AND                                         , RULL(0x05010881), SH_UNT_MCS_1    , SH_ACS_SCOM1_AND );
REG64( MCS_1_MCFIR_OR                                          , RULL(0x05010882), SH_UNT_MCS_1    , SH_ACS_SCOM2_OR  );
REG64( MCS_2_MCFIR                                             , RULL(0x03010800), SH_UNT_MCS_2    , SH_ACS_SCOM_RW   );
REG64( MCS_2_MCFIR_AND                                         , RULL(0x03010801), SH_UNT_MCS_2    , SH_ACS_SCOM1_AND );
REG64( MCS_2_MCFIR_OR                                          , RULL(0x03010802), SH_UNT_MCS_2    , SH_ACS_SCOM2_OR  );
REG64( MCS_3_MCFIR                                             , RULL(0x03010880), SH_UNT_MCS_3    , SH_ACS_SCOM_RW   );
REG64( MCS_3_MCFIR_AND                                         , RULL(0x03010881), SH_UNT_MCS_3    , SH_ACS_SCOM1_AND );
REG64( MCS_3_MCFIR_OR                                          , RULL(0x03010882), SH_UNT_MCS_3    , SH_ACS_SCOM2_OR  );

REG64( MCS_MCFIRACT0                                           , RULL(0x05010806), SH_UNT_MCS      , SH_ACS_SCOM_RW   );
REG64( MCS_0_MCFIRACT0                                         , RULL(0x05010806), SH_UNT_MCS_0    , SH_ACS_SCOM_RW   );
REG64( MCS_1_MCFIRACT0                                         , RULL(0x05010886), SH_UNT_MCS_1    , SH_ACS_SCOM_RW   );
REG64( MCS_2_MCFIRACT0                                         , RULL(0x03010806), SH_UNT_MCS_2    , SH_ACS_SCOM_RW   );
REG64( MCS_3_MCFIRACT0                                         , RULL(0x03010886), SH_UNT_MCS_3    , SH_ACS_SCOM_RW   );

REG64( MCS_MCFIRACT1                                           , RULL(0x05010807), SH_UNT_MCS      , SH_ACS_SCOM_RW   );
REG64( MCS_0_MCFIRACT1                                         , RULL(0x05010807), SH_UNT_MCS_0    , SH_ACS_SCOM_RW   );
REG64( MCS_1_MCFIRACT1                                         , RULL(0x05010887), SH_UNT_MCS_1    , SH_ACS_SCOM_RW   );
REG64( MCS_2_MCFIRACT1                                         , RULL(0x03010807), SH_UNT_MCS_2    , SH_ACS_SCOM_RW   );
REG64( MCS_3_MCFIRACT1                                         , RULL(0x03010887), SH_UNT_MCS_3    , SH_ACS_SCOM_RW   );

REG64( MCS_MCFIRMASK                                           , RULL(0x05010803), SH_UNT_MCS      , SH_ACS_SCOM_RW   );
REG64( MCS_MCFIRMASK_AND                                       , RULL(0x05010804), SH_UNT_MCS      , SH_ACS_SCOM1_AND );
REG64( MCS_MCFIRMASK_OR                                        , RULL(0x05010805), SH_UNT_MCS      , SH_ACS_SCOM2_OR  );
REG64( MCS_0_MCFIRMASK                                         , RULL(0x05010803), SH_UNT_MCS_0    , SH_ACS_SCOM_RW   );
REG64( MCS_0_MCFIRMASK_AND                                     , RULL(0x05010804), SH_UNT_MCS_0    , SH_ACS_SCOM1_AND );
REG64( MCS_0_MCFIRMASK_OR                                      , RULL(0x05010805), SH_UNT_MCS_0    , SH_ACS_SCOM2_OR  );
REG64( MCS_1_MCFIRMASK                                         , RULL(0x05010883), SH_UNT_MCS_1    , SH_ACS_SCOM_RW   );
REG64( MCS_1_MCFIRMASK_AND                                     , RULL(0x05010884), SH_UNT_MCS_1    , SH_ACS_SCOM1_AND );
REG64( MCS_1_MCFIRMASK_OR                                      , RULL(0x05010885), SH_UNT_MCS_1    , SH_ACS_SCOM2_OR  );
REG64( MCS_2_MCFIRMASK                                         , RULL(0x03010803), SH_UNT_MCS_2    , SH_ACS_SCOM_RW   );
REG64( MCS_2_MCFIRMASK_AND                                     , RULL(0x03010804), SH_UNT_MCS_2    , SH_ACS_SCOM1_AND );
REG64( MCS_2_MCFIRMASK_OR                                      , RULL(0x03010805), SH_UNT_MCS_2    , SH_ACS_SCOM2_OR  );
REG64( MCS_3_MCFIRMASK                                         , RULL(0x03010883), SH_UNT_MCS_3    , SH_ACS_SCOM_RW   );
REG64( MCS_3_MCFIRMASK_AND                                     , RULL(0x03010884), SH_UNT_MCS_3    , SH_ACS_SCOM1_AND );
REG64( MCS_3_MCFIRMASK_OR                                      , RULL(0x03010885), SH_UNT_MCS_3    , SH_ACS_SCOM2_OR  );

REG64( MCS_MCLFSR                                              , RULL(0x05010814), SH_UNT_MCS      , SH_ACS_SCOM_RW   );
REG64( MCS_0_MCLFSR                                            , RULL(0x05010814), SH_UNT_MCS_0    , SH_ACS_SCOM_RW   );
REG64( MCS_1_MCLFSR                                            , RULL(0x05010894), SH_UNT_MCS_1    , SH_ACS_SCOM_RW   );
REG64( MCS_2_MCLFSR                                            , RULL(0x03010814), SH_UNT_MCS_2    , SH_ACS_SCOM_RW   );
REG64( MCS_3_MCLFSR                                            , RULL(0x03010894), SH_UNT_MCS_3    , SH_ACS_SCOM_RW   );

REG64( MCS_MCMODE0                                             , RULL(0x05010811), SH_UNT_MCS      , SH_ACS_SCOM_RW   );
REG64( MCS_0_MCMODE0                                           , RULL(0x05010811), SH_UNT_MCS_0    , SH_ACS_SCOM_RW   );
REG64( MCS_1_MCMODE0                                           , RULL(0x05010891), SH_UNT_MCS_1    , SH_ACS_SCOM_RW   );
REG64( MCS_2_MCMODE0                                           , RULL(0x03010811), SH_UNT_MCS_2    , SH_ACS_SCOM_RW   );
REG64( MCS_3_MCMODE0                                           , RULL(0x03010891), SH_UNT_MCS_3    , SH_ACS_SCOM_RW   );

REG64( MCS_MCMODE1                                             , RULL(0x05010812), SH_UNT_MCS      , SH_ACS_SCOM_RW   );
REG64( MCS_0_MCMODE1                                           , RULL(0x05010812), SH_UNT_MCS_0    , SH_ACS_SCOM_RW   );
REG64( MCS_1_MCMODE1                                           , RULL(0x05010892), SH_UNT_MCS_1    , SH_ACS_SCOM_RW   );
REG64( MCS_2_MCMODE1                                           , RULL(0x03010812), SH_UNT_MCS_2    , SH_ACS_SCOM_RW   );
REG64( MCS_3_MCMODE1                                           , RULL(0x03010892), SH_UNT_MCS_3    , SH_ACS_SCOM_RW   );

REG64( MCS_MCMODE2                                             , RULL(0x05010813), SH_UNT_MCS      , SH_ACS_SCOM_RW   );
REG64( MCS_0_MCMODE2                                           , RULL(0x05010813), SH_UNT_MCS_0    , SH_ACS_SCOM_RW   );
REG64( MCS_1_MCMODE2                                           , RULL(0x05010893), SH_UNT_MCS_1    , SH_ACS_SCOM_RW   );
REG64( MCS_2_MCMODE2                                           , RULL(0x03010813), SH_UNT_MCS_2    , SH_ACS_SCOM_RW   );
REG64( MCS_3_MCMODE2                                           , RULL(0x03010893), SH_UNT_MCS_3    , SH_ACS_SCOM_RW   );

REG64( MCS_0_PORT02_MCP0XLT0                                   , RULL(0x05010820), SH_UNT_MCS_0_PORT02,
       SH_ACS_SCOM3     );
REG64( MCS_0_PORT13_MCP0XLT0                                   , RULL(0x05010830), SH_UNT_MCS_0_PORT13,
       SH_ACS_SCOM3     );
REG64( MCS_1_PORT02_MCP0XLT0                                   , RULL(0x050108A0), SH_UNT_MCS_1_PORT02,
       SH_ACS_SCOM3     );
REG64( MCS_1_PORT13_MCP0XLT0                                   , RULL(0x050108B0), SH_UNT_MCS_1_PORT13,
       SH_ACS_SCOM3     );
REG64( MCS_2_PORT02_MCP0XLT0                                   , RULL(0x03010820), SH_UNT_MCS_2_PORT02,
       SH_ACS_SCOM3     );
REG64( MCS_2_PORT13_MCP0XLT0                                   , RULL(0x03010830), SH_UNT_MCS_2_PORT13,
       SH_ACS_SCOM3     );
REG64( MCS_3_PORT02_MCP0XLT0                                   , RULL(0x030108A0), SH_UNT_MCS_3_PORT02,
       SH_ACS_SCOM3     );
REG64( MCS_3_PORT13_MCP0XLT0                                   , RULL(0x030108B0), SH_UNT_MCS_3_PORT13,
       SH_ACS_SCOM3     );
REG64( MCS_PORT02_MCP0XLT0                                     , RULL(0x05010820), SH_UNT_MCS_PORT02,
       SH_ACS_SCOM3     );
REG64( MCS_PORT13_MCP0XLT0                                     , RULL(0x05010830), SH_UNT_MCS_PORT13,
       SH_ACS_SCOM3     );

REG64( MCS_0_PORT02_MCP0XLT1                                   , RULL(0x05010821), SH_UNT_MCS_0_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_0_PORT13_MCP0XLT1                                   , RULL(0x05010831), SH_UNT_MCS_0_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT02_MCP0XLT1                                   , RULL(0x050108A1), SH_UNT_MCS_1_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT13_MCP0XLT1                                   , RULL(0x050108B1), SH_UNT_MCS_1_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT02_MCP0XLT1                                   , RULL(0x03010821), SH_UNT_MCS_2_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT13_MCP0XLT1                                   , RULL(0x03010831), SH_UNT_MCS_2_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT02_MCP0XLT1                                   , RULL(0x030108A1), SH_UNT_MCS_3_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT13_MCP0XLT1                                   , RULL(0x030108B1), SH_UNT_MCS_3_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT02_MCP0XLT1                                     , RULL(0x05010821), SH_UNT_MCS_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT13_MCP0XLT1                                     , RULL(0x05010831), SH_UNT_MCS_PORT13,
       SH_ACS_SCOM3_RW  );

REG64( MCS_0_PORT02_MCP0XLT2                                   , RULL(0x05010822), SH_UNT_MCS_0_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_0_PORT13_MCP0XLT2                                   , RULL(0x05010832), SH_UNT_MCS_0_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT02_MCP0XLT2                                   , RULL(0x050108A2), SH_UNT_MCS_1_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT13_MCP0XLT2                                   , RULL(0x050108B2), SH_UNT_MCS_1_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT02_MCP0XLT2                                   , RULL(0x03010822), SH_UNT_MCS_2_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT13_MCP0XLT2                                   , RULL(0x03010832), SH_UNT_MCS_2_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT02_MCP0XLT2                                   , RULL(0x030108A2), SH_UNT_MCS_3_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT13_MCP0XLT2                                   , RULL(0x030108B2), SH_UNT_MCS_3_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT02_MCP0XLT2                                     , RULL(0x05010822), SH_UNT_MCS_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT13_MCP0XLT2                                     , RULL(0x05010832), SH_UNT_MCS_PORT13,
       SH_ACS_SCOM3_RW  );

REG64( MCS_0_PORT02_MCPERF0                                    , RULL(0x05010823), SH_UNT_MCS_0_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_0_PORT13_MCPERF0                                    , RULL(0x05010833), SH_UNT_MCS_0_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT02_MCPERF0                                    , RULL(0x050108A3), SH_UNT_MCS_1_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT13_MCPERF0                                    , RULL(0x050108B3), SH_UNT_MCS_1_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT02_MCPERF0                                    , RULL(0x03010823), SH_UNT_MCS_2_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT13_MCPERF0                                    , RULL(0x03010833), SH_UNT_MCS_2_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT02_MCPERF0                                    , RULL(0x030108A3), SH_UNT_MCS_3_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT13_MCPERF0                                    , RULL(0x030108B3), SH_UNT_MCS_3_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT02_MCPERF0                                      , RULL(0x05010823), SH_UNT_MCS_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT13_MCPERF0                                      , RULL(0x05010833), SH_UNT_MCS_PORT13,
       SH_ACS_SCOM3_RW  );

REG64( MCS_MCPERF1                                             , RULL(0x05010810), SH_UNT_MCS      , SH_ACS_SCOM_RW   );
REG64( MCS_0_MCPERF1                                           , RULL(0x05010810), SH_UNT_MCS_0    , SH_ACS_SCOM_RW   );
REG64( MCS_1_MCPERF1                                           , RULL(0x05010890), SH_UNT_MCS_1    , SH_ACS_SCOM_RW   );
REG64( MCS_2_MCPERF1                                           , RULL(0x03010810), SH_UNT_MCS_2    , SH_ACS_SCOM_RW   );
REG64( MCS_3_MCPERF1                                           , RULL(0x03010890), SH_UNT_MCS_3    , SH_ACS_SCOM_RW   );

REG64( MCS_0_PORT02_MCPERF2                                    , RULL(0x05010824), SH_UNT_MCS_0_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_0_PORT13_MCPERF2                                    , RULL(0x05010834), SH_UNT_MCS_0_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT02_MCPERF2                                    , RULL(0x050108A4), SH_UNT_MCS_1_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT13_MCPERF2                                    , RULL(0x050108B4), SH_UNT_MCS_1_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT02_MCPERF2                                    , RULL(0x03010824), SH_UNT_MCS_2_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT13_MCPERF2                                    , RULL(0x03010834), SH_UNT_MCS_2_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT02_MCPERF2                                    , RULL(0x030108A4), SH_UNT_MCS_3_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT13_MCPERF2                                    , RULL(0x030108B4), SH_UNT_MCS_3_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT02_MCPERF2                                      , RULL(0x05010824), SH_UNT_MCS_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT13_MCPERF2                                      , RULL(0x05010834), SH_UNT_MCS_PORT13,
       SH_ACS_SCOM3_RW  );

REG64( MCS_0_PORT02_MCPERF3                                    , RULL(0x0501082B), SH_UNT_MCS_0_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_0_PORT13_MCPERF3                                    , RULL(0x0501083B), SH_UNT_MCS_0_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT02_MCPERF3                                    , RULL(0x050108AB), SH_UNT_MCS_1_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT13_MCPERF3                                    , RULL(0x050108BB), SH_UNT_MCS_1_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT02_MCPERF3                                    , RULL(0x0301082B), SH_UNT_MCS_2_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT13_MCPERF3                                    , RULL(0x0301083B), SH_UNT_MCS_2_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT02_MCPERF3                                    , RULL(0x030108AB), SH_UNT_MCS_3_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT13_MCPERF3                                    , RULL(0x030108BB), SH_UNT_MCS_3_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT02_MCPERF3                                      , RULL(0x0501082B), SH_UNT_MCS_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT13_MCPERF3                                      , RULL(0x0501083B), SH_UNT_MCS_PORT13,
       SH_ACS_SCOM3_RW  );

REG64( MCS_MCSYNC                                              , RULL(0x05010815), SH_UNT_MCS      , SH_ACS_SCOM_RW   );
REG64( MCS_0_MCSYNC                                            , RULL(0x05010815), SH_UNT_MCS_0    , SH_ACS_SCOM_RW   );
REG64( MCS_1_MCSYNC                                            , RULL(0x05010895), SH_UNT_MCS_1    , SH_ACS_SCOM_RW   );
REG64( MCS_2_MCSYNC                                            , RULL(0x03010815), SH_UNT_MCS_2    , SH_ACS_SCOM_RW   );
REG64( MCS_3_MCSYNC                                            , RULL(0x03010895), SH_UNT_MCS_3    , SH_ACS_SCOM_RW   );

REG64( MCS_MCTEST                                              , RULL(0x05010818), SH_UNT_MCS      , SH_ACS_SCOM_RW   );
REG64( MCS_0_MCTEST                                            , RULL(0x05010818), SH_UNT_MCS_0    , SH_ACS_SCOM_RW   );
REG64( MCS_1_MCTEST                                            , RULL(0x05010898), SH_UNT_MCS_1    , SH_ACS_SCOM_RW   );
REG64( MCS_2_MCTEST                                            , RULL(0x03010818), SH_UNT_MCS_2    , SH_ACS_SCOM_RW   );
REG64( MCS_3_MCTEST                                            , RULL(0x03010898), SH_UNT_MCS_3    , SH_ACS_SCOM_RW   );

REG64( MCS_MCTO                                                , RULL(0x0501081B), SH_UNT_MCS      , SH_ACS_SCOM_RW   );
REG64( MCS_0_MCTO                                              , RULL(0x0501081B), SH_UNT_MCS_0    , SH_ACS_SCOM_RW   );
REG64( MCS_1_MCTO                                              , RULL(0x0501089B), SH_UNT_MCS_1    , SH_ACS_SCOM_RW   );
REG64( MCS_2_MCTO                                              , RULL(0x0301081B), SH_UNT_MCS_2    , SH_ACS_SCOM_RW   );
REG64( MCS_3_MCTO                                              , RULL(0x0301089B), SH_UNT_MCS_3    , SH_ACS_SCOM_RW   );

REG64( MCS_0_PORT02_MCWAT                                      , RULL(0x0501082A), SH_UNT_MCS_0_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_0_PORT13_MCWAT                                      , RULL(0x0501083A), SH_UNT_MCS_0_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT02_MCWAT                                      , RULL(0x050108AA), SH_UNT_MCS_1_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_1_PORT13_MCWAT                                      , RULL(0x050108BA), SH_UNT_MCS_1_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT02_MCWAT                                      , RULL(0x0301082A), SH_UNT_MCS_2_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_2_PORT13_MCWAT                                      , RULL(0x0301083A), SH_UNT_MCS_2_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT02_MCWAT                                      , RULL(0x030108AA), SH_UNT_MCS_3_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_3_PORT13_MCWAT                                      , RULL(0x030108BA), SH_UNT_MCS_3_PORT13,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT02_MCWAT                                        , RULL(0x0501082A), SH_UNT_MCS_PORT02,
       SH_ACS_SCOM3_RW  );
REG64( MCS_PORT13_MCWAT                                        , RULL(0x0501083A), SH_UNT_MCS_PORT13,
       SH_ACS_SCOM3_RW  );

REG64( MCS_MCWATCNTL                                           , RULL(0x0501081C), SH_UNT_MCS      , SH_ACS_SCOM_RW   );
REG64( MCS_0_MCWATCNTL                                         , RULL(0x0501081C), SH_UNT_MCS_0    , SH_ACS_SCOM_RW   );
REG64( MCS_1_MCWATCNTL                                         , RULL(0x0501089C), SH_UNT_MCS_1    , SH_ACS_SCOM_RW   );
REG64( MCS_2_MCWATCNTL                                         , RULL(0x0301081C), SH_UNT_MCS_2    , SH_ACS_SCOM_RW   );
REG64( MCS_3_MCWATCNTL                                         , RULL(0x0301089C), SH_UNT_MCS_3    , SH_ACS_SCOM_RW   );

REG64( MCS_MCWATDATA                                           , RULL(0x0501081D), SH_UNT_MCS      , SH_ACS_SCOM_RO   );
REG64( MCS_0_MCWATDATA                                         , RULL(0x0501081D), SH_UNT_MCS_0    , SH_ACS_SCOM_RO   );
REG64( MCS_1_MCWATDATA                                         , RULL(0x0501089D), SH_UNT_MCS_1    , SH_ACS_SCOM_RO   );
REG64( MCS_2_MCWATDATA                                         , RULL(0x0301081D), SH_UNT_MCS_2    , SH_ACS_SCOM_RO   );
REG64( MCS_3_MCWATDATA                                         , RULL(0x0301089D), SH_UNT_MCS_3    , SH_ACS_SCOM_RO   );

REG64( MCA_MSR                                                 , RULL(0x07010A0C), SH_UNT_MCA      , SH_ACS_SCOM_RO   );
REG64( MCA_0_MSR                                               , RULL(0x07010A0C), SH_UNT_MCA_0    , SH_ACS_SCOM_RO   );
REG64( MCA_1_MSR                                               , RULL(0x07010A4C), SH_UNT_MCA_1    , SH_ACS_SCOM_RO   );
REG64( MCA_2_MSR                                               , RULL(0x07010A8C), SH_UNT_MCA_2    , SH_ACS_SCOM_RO   );
REG64( MCA_3_MSR                                               , RULL(0x07010ACC), SH_UNT_MCA_3    , SH_ACS_SCOM_RO   );
REG64( MCA_4_MSR                                               , RULL(0x08010A0C), SH_UNT_MCA_4    , SH_ACS_SCOM_RO   );
REG64( MCA_5_MSR                                               , RULL(0x08010A4C), SH_UNT_MCA_5    , SH_ACS_SCOM_RO   );
REG64( MCA_6_MSR                                               , RULL(0x08010A8C), SH_UNT_MCA_6    , SH_ACS_SCOM_RO   );
REG64( MCA_7_MSR                                               , RULL(0x08010ACC), SH_UNT_MCA_7    , SH_ACS_SCOM_RO   );

REG64( MCA_PHY0_DDRPHY_FIR_ACTION0_REG                         , RULL(0x07011006), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_PHY0_DDRPHY_FIR_ACTION0_REG                       , RULL(0x07011006), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_4_PHY0_DDRPHY_FIR_ACTION0_REG                       , RULL(0x08011006), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );

REG64( MCA_PHY0_DDRPHY_FIR_ACTION1_REG                         , RULL(0x07011007), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_PHY0_DDRPHY_FIR_ACTION1_REG                       , RULL(0x07011007), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_4_PHY0_DDRPHY_FIR_ACTION1_REG                       , RULL(0x08011007), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );

REG64( MCA_PHY0_DDRPHY_FIR_MASK_REG                            , RULL(0x07011003), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_PHY0_DDRPHY_FIR_MASK_REG_AND                        , RULL(0x07011004), SH_UNT_MCA      , SH_ACS_SCOM1_AND );
REG64( MCA_PHY0_DDRPHY_FIR_MASK_REG_OR                         , RULL(0x07011005), SH_UNT_MCA      , SH_ACS_SCOM2_OR  );
REG64( MCA_0_PHY0_DDRPHY_FIR_MASK_REG                          , RULL(0x07011003), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_0_PHY0_DDRPHY_FIR_MASK_REG_AND                      , RULL(0x07011004), SH_UNT_MCA_0    , SH_ACS_SCOM1_AND );
REG64( MCA_0_PHY0_DDRPHY_FIR_MASK_REG_OR                       , RULL(0x07011005), SH_UNT_MCA_0    , SH_ACS_SCOM2_OR  );
REG64( MCA_4_PHY0_DDRPHY_FIR_MASK_REG                          , RULL(0x08011003), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_4_PHY0_DDRPHY_FIR_MASK_REG_AND                      , RULL(0x08011004), SH_UNT_MCA_4    , SH_ACS_SCOM1_AND );
REG64( MCA_4_PHY0_DDRPHY_FIR_MASK_REG_OR                       , RULL(0x08011005), SH_UNT_MCA_4    , SH_ACS_SCOM2_OR  );

REG64( MCA_PHY0_DDRPHY_FIR_REG                                 , RULL(0x07011000), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_PHY0_DDRPHY_FIR_REG_AND                             , RULL(0x07011001), SH_UNT_MCA      , SH_ACS_SCOM1_AND );
REG64( MCA_PHY0_DDRPHY_FIR_REG_OR                              , RULL(0x07011002), SH_UNT_MCA      , SH_ACS_SCOM2_OR  );
REG64( MCA_0_PHY0_DDRPHY_FIR_REG                               , RULL(0x07011000), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_0_PHY0_DDRPHY_FIR_REG_AND                           , RULL(0x07011001), SH_UNT_MCA_0    , SH_ACS_SCOM1_AND );
REG64( MCA_0_PHY0_DDRPHY_FIR_REG_OR                            , RULL(0x07011002), SH_UNT_MCA_0    , SH_ACS_SCOM2_OR  );
REG64( MCA_4_PHY0_DDRPHY_FIR_REG                               , RULL(0x08011000), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_4_PHY0_DDRPHY_FIR_REG_AND                           , RULL(0x08011001), SH_UNT_MCA_4    , SH_ACS_SCOM1_AND );
REG64( MCA_4_PHY0_DDRPHY_FIR_REG_OR                            , RULL(0x08011002), SH_UNT_MCA_4    , SH_ACS_SCOM2_OR  );

REG64( MCA_PHY0_DDRPHY_FIR_WOF_REG                             , RULL(0x07011008), SH_UNT_MCA      ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_0_PHY0_DDRPHY_FIR_WOF_REG                           , RULL(0x07011008), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_4_PHY0_DDRPHY_FIR_WOF_REG                           , RULL(0x08011008), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_WCLRREG );

REG64( MCA_1_PHY1_DDRPHY_FIR_ACTION0_REG                       , RULL(0x07011406), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_5_PHY1_DDRPHY_FIR_ACTION0_REG                       , RULL(0x08011406), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );

REG64( MCA_1_PHY1_DDRPHY_FIR_ACTION1_REG                       , RULL(0x07011407), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_5_PHY1_DDRPHY_FIR_ACTION1_REG                       , RULL(0x08011407), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );

REG64( MCA_1_PHY1_DDRPHY_FIR_MASK_REG                          , RULL(0x07011403), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_1_PHY1_DDRPHY_FIR_MASK_REG_AND                      , RULL(0x07011404), SH_UNT_MCA_1    , SH_ACS_SCOM1_AND );
REG64( MCA_1_PHY1_DDRPHY_FIR_MASK_REG_OR                       , RULL(0x07011405), SH_UNT_MCA_1    , SH_ACS_SCOM2_OR  );
REG64( MCA_5_PHY1_DDRPHY_FIR_MASK_REG                          , RULL(0x08011403), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_5_PHY1_DDRPHY_FIR_MASK_REG_AND                      , RULL(0x08011404), SH_UNT_MCA_5    , SH_ACS_SCOM1_AND );
REG64( MCA_5_PHY1_DDRPHY_FIR_MASK_REG_OR                       , RULL(0x08011405), SH_UNT_MCA_5    , SH_ACS_SCOM2_OR  );

REG64( MCA_1_PHY1_DDRPHY_FIR_REG                               , RULL(0x07011400), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_1_PHY1_DDRPHY_FIR_REG_AND                           , RULL(0x07011401), SH_UNT_MCA_1    , SH_ACS_SCOM1_AND );
REG64( MCA_1_PHY1_DDRPHY_FIR_REG_OR                            , RULL(0x07011402), SH_UNT_MCA_1    , SH_ACS_SCOM2_OR  );
REG64( MCA_5_PHY1_DDRPHY_FIR_REG                               , RULL(0x08011400), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_5_PHY1_DDRPHY_FIR_REG_AND                           , RULL(0x08011401), SH_UNT_MCA_5    , SH_ACS_SCOM1_AND );
REG64( MCA_5_PHY1_DDRPHY_FIR_REG_OR                            , RULL(0x08011402), SH_UNT_MCA_5    , SH_ACS_SCOM2_OR  );

REG64( MCA_1_PHY1_DDRPHY_FIR_WOF_REG                           , RULL(0x07011408), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_5_PHY1_DDRPHY_FIR_WOF_REG                           , RULL(0x08011408), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_WCLRREG );

REG64( MCA_2_PHY2_DDRPHY_FIR_ACTION0_REG                       , RULL(0x07011806), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_6_PHY2_DDRPHY_FIR_ACTION0_REG                       , RULL(0x08011806), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );

REG64( MCA_2_PHY2_DDRPHY_FIR_ACTION1_REG                       , RULL(0x07011807), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_6_PHY2_DDRPHY_FIR_ACTION1_REG                       , RULL(0x08011807), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );

REG64( MCA_2_PHY2_DDRPHY_FIR_MASK_REG                          , RULL(0x07011803), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_2_PHY2_DDRPHY_FIR_MASK_REG_AND                      , RULL(0x07011804), SH_UNT_MCA_2    , SH_ACS_SCOM1_AND );
REG64( MCA_2_PHY2_DDRPHY_FIR_MASK_REG_OR                       , RULL(0x07011805), SH_UNT_MCA_2    , SH_ACS_SCOM2_OR  );
REG64( MCA_6_PHY2_DDRPHY_FIR_MASK_REG                          , RULL(0x08011803), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_6_PHY2_DDRPHY_FIR_MASK_REG_AND                      , RULL(0x08011804), SH_UNT_MCA_6    , SH_ACS_SCOM1_AND );
REG64( MCA_6_PHY2_DDRPHY_FIR_MASK_REG_OR                       , RULL(0x08011805), SH_UNT_MCA_6    , SH_ACS_SCOM2_OR  );

REG64( MCA_2_PHY2_DDRPHY_FIR_REG                               , RULL(0x07011800), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_2_PHY2_DDRPHY_FIR_REG_AND                           , RULL(0x07011801), SH_UNT_MCA_2    , SH_ACS_SCOM1_AND );
REG64( MCA_2_PHY2_DDRPHY_FIR_REG_OR                            , RULL(0x07011802), SH_UNT_MCA_2    , SH_ACS_SCOM2_OR  );
REG64( MCA_6_PHY2_DDRPHY_FIR_REG                               , RULL(0x08011800), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_6_PHY2_DDRPHY_FIR_REG_AND                           , RULL(0x08011801), SH_UNT_MCA_6    , SH_ACS_SCOM1_AND );
REG64( MCA_6_PHY2_DDRPHY_FIR_REG_OR                            , RULL(0x08011802), SH_UNT_MCA_6    , SH_ACS_SCOM2_OR  );

REG64( MCA_2_PHY2_DDRPHY_FIR_WOF_REG                           , RULL(0x07011808), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_6_PHY2_DDRPHY_FIR_WOF_REG                           , RULL(0x08011808), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_WCLRREG );

REG64( MCA_3_PHY3_DDRPHY_FIR_ACTION0_REG                       , RULL(0x07011C06), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_7_PHY3_DDRPHY_FIR_ACTION0_REG                       , RULL(0x08011C06), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_3_PHY3_DDRPHY_FIR_ACTION1_REG                       , RULL(0x07011C07), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_7_PHY3_DDRPHY_FIR_ACTION1_REG                       , RULL(0x08011C07), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_3_PHY3_DDRPHY_FIR_MASK_REG                          , RULL(0x07011C03), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_3_PHY3_DDRPHY_FIR_MASK_REG_AND                      , RULL(0x07011C04), SH_UNT_MCA_3    , SH_ACS_SCOM1_AND );
REG64( MCA_3_PHY3_DDRPHY_FIR_MASK_REG_OR                       , RULL(0x07011C05), SH_UNT_MCA_3    , SH_ACS_SCOM2_OR  );
REG64( MCA_7_PHY3_DDRPHY_FIR_MASK_REG                          , RULL(0x08011C03), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );
REG64( MCA_7_PHY3_DDRPHY_FIR_MASK_REG_AND                      , RULL(0x08011C04), SH_UNT_MCA_7    , SH_ACS_SCOM1_AND );
REG64( MCA_7_PHY3_DDRPHY_FIR_MASK_REG_OR                       , RULL(0x08011C05), SH_UNT_MCA_7    , SH_ACS_SCOM2_OR  );

REG64( MCA_3_PHY3_DDRPHY_FIR_REG                               , RULL(0x07011C00), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_3_PHY3_DDRPHY_FIR_REG_AND                           , RULL(0x07011C01), SH_UNT_MCA_3    , SH_ACS_SCOM1_AND );
REG64( MCA_3_PHY3_DDRPHY_FIR_REG_OR                            , RULL(0x07011C02), SH_UNT_MCA_3    , SH_ACS_SCOM2_OR  );
REG64( MCA_7_PHY3_DDRPHY_FIR_REG                               , RULL(0x08011C00), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );
REG64( MCA_7_PHY3_DDRPHY_FIR_REG_AND                           , RULL(0x08011C01), SH_UNT_MCA_7    , SH_ACS_SCOM1_AND );
REG64( MCA_7_PHY3_DDRPHY_FIR_REG_OR                            , RULL(0x08011C02), SH_UNT_MCA_7    , SH_ACS_SCOM2_OR  );

REG64( MCA_3_PHY3_DDRPHY_FIR_WOF_REG                           , RULL(0x07011C08), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_7_PHY3_DDRPHY_FIR_WOF_REG                           , RULL(0x08011C08), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_WCLRREG );

REG64( MCA_PSCOM_ERROR_MASK                                    , RULL(0x07010002), SH_UNT_MCA      , SH_ACS_SCOM      );
REG64( MCA_0_PSCOM_ERROR_MASK                                  , RULL(0x07010002), SH_UNT_MCA_0    , SH_ACS_SCOM      );
REG64( MCA_4_PSCOM_ERROR_MASK                                  , RULL(0x08010002), SH_UNT_MCA_4    , SH_ACS_SCOM      );

REG64( MCA_PSCOM_MODE_REG                                      , RULL(0x07010000), SH_UNT_MCA      , SH_ACS_SCOM      );
REG64( MCA_0_PSCOM_MODE_REG                                    , RULL(0x07010000), SH_UNT_MCA_0    , SH_ACS_SCOM      );
REG64( MCA_4_PSCOM_MODE_REG                                    , RULL(0x08010000), SH_UNT_MCA_4    , SH_ACS_SCOM      );

REG64( MCA_PSCOM_STATUS_ERROR_REG                              , RULL(0x07010001), SH_UNT_MCA      , SH_ACS_SCOM      );
REG64( MCA_0_PSCOM_STATUS_ERROR_REG                            , RULL(0x07010001), SH_UNT_MCA_0    , SH_ACS_SCOM      );
REG64( MCA_4_PSCOM_STATUS_ERROR_REG                            , RULL(0x08010001), SH_UNT_MCA_4    , SH_ACS_SCOM      );

REG64( MCBIST_RCD_LRDIM_CNTL_WORD0_15Q                         , RULL(0x070123DD), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_RCD_LRDIM_CNTL_WORD0_15Q                       , RULL(0x070123DD), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_RCD_LRDIM_CNTL_WORD0_15Q                       , RULL(0x080123DD), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCA_RECR                                                , RULL(0x07010A0A), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_RECR                                              , RULL(0x07010A0A), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_RECR                                              , RULL(0x07010A4A), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_RECR                                              , RULL(0x07010A8A), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_RECR                                              , RULL(0x07010ACA), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_RECR                                              , RULL(0x08010A0A), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_RECR                                              , RULL(0x08010A4A), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_RECR                                              , RULL(0x08010A8A), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_RECR                                              , RULL(0x08010ACA), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_RING_FENCE_MASK_LATCH_REG                           , RULL(0x07010008), SH_UNT_MCA      , SH_ACS_SCOM      );
REG64( MCA_0_RING_FENCE_MASK_LATCH_REG                         , RULL(0x07010008), SH_UNT_MCA_0    , SH_ACS_SCOM      );
REG64( MCA_4_RING_FENCE_MASK_LATCH_REG                         , RULL(0x08010008), SH_UNT_MCA_4    , SH_ACS_SCOM      );

REG64( MCBIST_RUNTIMECTRQ                                      , RULL(0x070123B0), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_RUNTIMECTRQ                                    , RULL(0x070123B0), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_RUNTIMECTRQ                                    , RULL(0x080123B0), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCA_TCMC01_SLOW_TRA0_TRACE_HI_DATA_REG                  , RULL(0x07010400), SH_UNT_MCA      , SH_ACS_SCOM_RO   );
REG64( MCA_0_TCMC01_SLOW_TRA0_TRACE_HI_DATA_REG                , RULL(0x07010400), SH_UNT_MCA_0    , SH_ACS_SCOM_RO   );

REG64( MCA_TCMC01_SLOW_TRA0_TRACE_LO_DATA_REG                  , RULL(0x07010401), SH_UNT_MCA      , SH_ACS_SCOM_RO   );
REG64( MCA_0_TCMC01_SLOW_TRA0_TRACE_LO_DATA_REG                , RULL(0x07010401), SH_UNT_MCA_0    , SH_ACS_SCOM_RO   );

REG64( MCA_TCMC01_SLOW_TRA0_TRACE_TRCTRL_CONFIG                , RULL(0x07010402), SH_UNT_MCA      , SH_ACS_SCOM      );
REG64( MCA_0_TCMC01_SLOW_TRA0_TRACE_TRCTRL_CONFIG              , RULL(0x07010402), SH_UNT_MCA_0    , SH_ACS_SCOM      );

REG64( MCA_TCMC01_SLOW_TRA0_TRACE_TRDATA_CONFIG_0              , RULL(0x07010403), SH_UNT_MCA      , SH_ACS_SCOM      );
REG64( MCA_0_TCMC01_SLOW_TRA0_TRACE_TRDATA_CONFIG_0            , RULL(0x07010403), SH_UNT_MCA_0    , SH_ACS_SCOM      );

REG64( MCA_TCMC01_SLOW_TRA0_TRACE_TRDATA_CONFIG_1              , RULL(0x07010404), SH_UNT_MCA      , SH_ACS_SCOM      );
REG64( MCA_0_TCMC01_SLOW_TRA0_TRACE_TRDATA_CONFIG_1            , RULL(0x07010404), SH_UNT_MCA_0    , SH_ACS_SCOM      );

REG64( MCA_TCMC01_SLOW_TRA0_TRACE_TRDATA_CONFIG_2              , RULL(0x07010405), SH_UNT_MCA      , SH_ACS_SCOM      );
REG64( MCA_0_TCMC01_SLOW_TRA0_TRACE_TRDATA_CONFIG_2            , RULL(0x07010405), SH_UNT_MCA_0    , SH_ACS_SCOM      );

REG64( MCA_TCMC01_SLOW_TRA0_TRACE_TRDATA_CONFIG_3              , RULL(0x07010406), SH_UNT_MCA      , SH_ACS_SCOM      );
REG64( MCA_0_TCMC01_SLOW_TRA0_TRACE_TRDATA_CONFIG_3            , RULL(0x07010406), SH_UNT_MCA_0    , SH_ACS_SCOM      );

REG64( MCA_TCMC01_SLOW_TRA0_TRACE_TRDATA_CONFIG_4              , RULL(0x07010407), SH_UNT_MCA      , SH_ACS_SCOM      );
REG64( MCA_0_TCMC01_SLOW_TRA0_TRACE_TRDATA_CONFIG_4            , RULL(0x07010407), SH_UNT_MCA_0    , SH_ACS_SCOM      );

REG64( MCA_TCMC01_SLOW_TRA0_TRACE_TRDATA_CONFIG_5              , RULL(0x07010408), SH_UNT_MCA      , SH_ACS_SCOM      );
REG64( MCA_0_TCMC01_SLOW_TRA0_TRACE_TRDATA_CONFIG_5            , RULL(0x07010408), SH_UNT_MCA_0    , SH_ACS_SCOM      );

REG64( MCA_TCMC01_SLOW_TRA0_TRACE_TRDATA_CONFIG_9              , RULL(0x07010409), SH_UNT_MCA      , SH_ACS_SCOM      );
REG64( MCA_0_TCMC01_SLOW_TRA0_TRACE_TRDATA_CONFIG_9            , RULL(0x07010409), SH_UNT_MCA_0    , SH_ACS_SCOM      );

REG64( MCA_4_TCMC23_SLOW_TRA0_TRACE_HI_DATA_REG                , RULL(0x08010400), SH_UNT_MCA_4    , SH_ACS_SCOM_RO   );

REG64( MCA_4_TCMC23_SLOW_TRA0_TRACE_LO_DATA_REG                , RULL(0x08010401), SH_UNT_MCA_4    , SH_ACS_SCOM_RO   );

REG64( MCA_4_TCMC23_SLOW_TRA0_TRACE_TRCTRL_CONFIG              , RULL(0x08010402), SH_UNT_MCA_4    , SH_ACS_SCOM      );

REG64( MCA_4_TCMC23_SLOW_TRA0_TRACE_TRDATA_CONFIG_0            , RULL(0x08010403), SH_UNT_MCA_4    , SH_ACS_SCOM      );

REG64( MCA_4_TCMC23_SLOW_TRA0_TRACE_TRDATA_CONFIG_1            , RULL(0x08010404), SH_UNT_MCA_4    , SH_ACS_SCOM      );

REG64( MCA_4_TCMC23_SLOW_TRA0_TRACE_TRDATA_CONFIG_2            , RULL(0x08010405), SH_UNT_MCA_4    , SH_ACS_SCOM      );

REG64( MCA_4_TCMC23_SLOW_TRA0_TRACE_TRDATA_CONFIG_3            , RULL(0x08010406), SH_UNT_MCA_4    , SH_ACS_SCOM      );

REG64( MCA_4_TCMC23_SLOW_TRA0_TRACE_TRDATA_CONFIG_4            , RULL(0x08010407), SH_UNT_MCA_4    , SH_ACS_SCOM      );

REG64( MCA_4_TCMC23_SLOW_TRA0_TRACE_TRDATA_CONFIG_5            , RULL(0x08010408), SH_UNT_MCA_4    , SH_ACS_SCOM      );

REG64( MCA_4_TCMC23_SLOW_TRA0_TRACE_TRDATA_CONFIG_9            , RULL(0x08010409), SH_UNT_MCA_4    , SH_ACS_SCOM      );

REG64( MCBIST_WATCFG0AQ                                        , RULL(0x07012380), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG0AQ                                      , RULL(0x07012380), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG0AQ                                      , RULL(0x08012380), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG0BQ                                        , RULL(0x07012381), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG0BQ                                      , RULL(0x07012381), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG0BQ                                      , RULL(0x08012381), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG0CQ                                        , RULL(0x07012382), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG0CQ                                      , RULL(0x07012382), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG0CQ                                      , RULL(0x08012382), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG0DQ                                        , RULL(0x07012383), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG0DQ                                      , RULL(0x07012383), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG0DQ                                      , RULL(0x08012383), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG0EQ                                        , RULL(0x07012384), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG0EQ                                      , RULL(0x07012384), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG0EQ                                      , RULL(0x08012384), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG1AQ                                        , RULL(0x07012385), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG1AQ                                      , RULL(0x07012385), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG1AQ                                      , RULL(0x08012385), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG1BQ                                        , RULL(0x07012386), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG1BQ                                      , RULL(0x07012386), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG1BQ                                      , RULL(0x08012386), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG1CQ                                        , RULL(0x07012387), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG1CQ                                      , RULL(0x07012387), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG1CQ                                      , RULL(0x08012387), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG1DQ                                        , RULL(0x07012388), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG1DQ                                      , RULL(0x07012388), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG1DQ                                      , RULL(0x08012388), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG1EQ                                        , RULL(0x07012389), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG1EQ                                      , RULL(0x07012389), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG1EQ                                      , RULL(0x08012389), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG2AQ                                        , RULL(0x0701238A), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG2AQ                                      , RULL(0x0701238A), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG2AQ                                      , RULL(0x0801238A), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG2BQ                                        , RULL(0x0701238B), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG2BQ                                      , RULL(0x0701238B), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG2BQ                                      , RULL(0x0801238B), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG2CQ                                        , RULL(0x0701238C), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG2CQ                                      , RULL(0x0701238C), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG2CQ                                      , RULL(0x0801238C), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG2DQ                                        , RULL(0x0701238D), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG2DQ                                      , RULL(0x0701238D), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG2DQ                                      , RULL(0x0801238D), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG2EQ                                        , RULL(0x0701238E), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG2EQ                                      , RULL(0x0701238E), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG2EQ                                      , RULL(0x0801238E), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG3AQ                                        , RULL(0x0701238F), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG3AQ                                      , RULL(0x0701238F), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG3AQ                                      , RULL(0x0801238F), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG3BQ                                        , RULL(0x07012390), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG3BQ                                      , RULL(0x07012390), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG3BQ                                      , RULL(0x08012390), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG3CQ                                        , RULL(0x07012391), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG3CQ                                      , RULL(0x07012391), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG3CQ                                      , RULL(0x08012391), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG3DQ                                        , RULL(0x07012392), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG3DQ                                      , RULL(0x07012392), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG3DQ                                      , RULL(0x08012392), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCBIST_WATCFG3EQ                                        , RULL(0x07012393), SH_UNT_MCBIST   , SH_ACS_SCOM_RW   );
REG64( MCBIST_0_WATCFG3EQ                                      , RULL(0x07012393), SH_UNT_MCBIST_0 , SH_ACS_SCOM_RW   );
REG64( MCBIST_1_WATCFG3EQ                                      , RULL(0x08012393), SH_UNT_MCBIST_1 , SH_ACS_SCOM_RW   );

REG64( MCA_WBMGR_TAG_INFO                                      , RULL(0x07010A33), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_WBMGR_TAG_INFO                                    , RULL(0x07010A33), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_WBMGR_TAG_INFO                                    , RULL(0x07010A73), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_WBMGR_TAG_INFO                                    , RULL(0x07010AB3), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_WBMGR_TAG_INFO                                    , RULL(0x07010AF3), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_WBMGR_TAG_INFO                                    , RULL(0x08010A33), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_WBMGR_TAG_INFO                                    , RULL(0x08010A73), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_WBMGR_TAG_INFO                                    , RULL(0x08010AB3), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_WBMGR_TAG_INFO                                    , RULL(0x08010AF3), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_WDFCFG                                              , RULL(0x07010A30), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_WDFCFG                                            , RULL(0x07010A30), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_WDFCFG                                            , RULL(0x07010A70), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_WDFCFG                                            , RULL(0x07010AB0), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_WDFCFG                                            , RULL(0x07010AF0), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_WDFCFG                                            , RULL(0x08010A30), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_WDFCFG                                            , RULL(0x08010A70), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_WDFCFG                                            , RULL(0x08010AB0), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_WDFCFG                                            , RULL(0x08010AF0), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_WDFDBG                                              , RULL(0x07010A34), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_WDFDBG                                            , RULL(0x07010A34), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_WDFDBG                                            , RULL(0x07010A74), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_WDFDBG                                            , RULL(0x07010AB4), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_WDFDBG                                            , RULL(0x07010AF4), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_WDFDBG                                            , RULL(0x08010A34), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_WDFDBG                                            , RULL(0x08010A74), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_WDFDBG                                            , RULL(0x08010AB4), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_WDFDBG                                            , RULL(0x08010AF4), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_WECR                                                , RULL(0x07010A28), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_WECR                                              , RULL(0x07010A28), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_WECR                                              , RULL(0x07010A68), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_WECR                                              , RULL(0x07010AA8), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_WECR                                              , RULL(0x07010AE8), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_WECR                                              , RULL(0x08010A28), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_WECR                                              , RULL(0x08010A68), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_WECR                                              , RULL(0x08010AA8), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_WECR                                              , RULL(0x08010AE8), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_WESR                                                , RULL(0x07010A2E), SH_UNT_MCA      , SH_ACS_SCOM_RO   );
REG64( MCA_0_WESR                                              , RULL(0x07010A2E), SH_UNT_MCA_0    , SH_ACS_SCOM_RO   );
REG64( MCA_1_WESR                                              , RULL(0x07010A6E), SH_UNT_MCA_1    , SH_ACS_SCOM_RO   );
REG64( MCA_2_WESR                                              , RULL(0x07010AAE), SH_UNT_MCA_2    , SH_ACS_SCOM_RO   );
REG64( MCA_3_WESR                                              , RULL(0x07010AEE), SH_UNT_MCA_3    , SH_ACS_SCOM_RO   );
REG64( MCA_4_WESR                                              , RULL(0x08010A2E), SH_UNT_MCA_4    , SH_ACS_SCOM_RO   );
REG64( MCA_5_WESR                                              , RULL(0x08010A6E), SH_UNT_MCA_5    , SH_ACS_SCOM_RO   );
REG64( MCA_6_WESR                                              , RULL(0x08010AAE), SH_UNT_MCA_6    , SH_ACS_SCOM_RO   );
REG64( MCA_7_WESR                                              , RULL(0x08010AEE), SH_UNT_MCA_7    , SH_ACS_SCOM_RO   );

REG64( MCA_WOF                                                 , RULL(0x07010A08), SH_UNT_MCA      ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_0_WOF                                               , RULL(0x07010A08), SH_UNT_MCA_0    ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_1_WOF                                               , RULL(0x07010A48), SH_UNT_MCA_1    ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_2_WOF                                               , RULL(0x07010A88), SH_UNT_MCA_2    ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_3_WOF                                               , RULL(0x07010AC8), SH_UNT_MCA_3    ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_4_WOF                                               , RULL(0x08010A08), SH_UNT_MCA_4    ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_5_WOF                                               , RULL(0x08010A48), SH_UNT_MCA_5    ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_6_WOF                                               , RULL(0x08010A88), SH_UNT_MCA_6    ,
       SH_ACS_SCOM_WCLRREG );
REG64( MCA_7_WOF                                               , RULL(0x08010AC8), SH_UNT_MCA_7    ,
       SH_ACS_SCOM_WCLRREG );

REG64( MCA_WRITE_PROTECT_ENABLE_REG                            , RULL(0x07010005), SH_UNT_MCA      , SH_ACS_SCOM      );
REG64( MCA_0_WRITE_PROTECT_ENABLE_REG                          , RULL(0x07010005), SH_UNT_MCA_0    , SH_ACS_SCOM      );
REG64( MCA_4_WRITE_PROTECT_ENABLE_REG                          , RULL(0x08010005), SH_UNT_MCA_4    , SH_ACS_SCOM      );

REG64( MCA_WRITE_PROTECT_RINGS_REG                             , RULL(0x07010006), SH_UNT_MCA      , SH_ACS_SCOM      );
REG64( MCA_0_WRITE_PROTECT_RINGS_REG                           , RULL(0x07010006), SH_UNT_MCA_0    , SH_ACS_SCOM      );
REG64( MCA_4_WRITE_PROTECT_RINGS_REG                           , RULL(0x08010006), SH_UNT_MCA_4    , SH_ACS_SCOM      );

REG64( MCA_WRTCFG                                              , RULL(0x07010A38), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_WRTCFG                                            , RULL(0x07010A38), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_WRTCFG                                            , RULL(0x07010A78), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_WRTCFG                                            , RULL(0x07010AB8), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_WRTCFG                                            , RULL(0x07010AF8), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_WRTCFG                                            , RULL(0x08010A38), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_WRTCFG                                            , RULL(0x08010A78), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_WRTCFG                                            , RULL(0x08010AB8), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_WRTCFG                                            , RULL(0x08010AF8), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_WRTDBGMCA                                           , RULL(0x07010A3A), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_WRTDBGMCA                                         , RULL(0x07010A3A), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_WRTDBGMCA                                         , RULL(0x07010A7A), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_WRTDBGMCA                                         , RULL(0x07010ABA), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_WRTDBGMCA                                         , RULL(0x07010AFA), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_WRTDBGMCA                                         , RULL(0x08010A3A), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_WRTDBGMCA                                         , RULL(0x08010A7A), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_WRTDBGMCA                                         , RULL(0x08010ABA), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_WRTDBGMCA                                         , RULL(0x08010AFA), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_WRTDBGNEST                                          , RULL(0x07010A3B), SH_UNT_MCA      , SH_ACS_SCOM_RW   );
REG64( MCA_0_WRTDBGNEST                                        , RULL(0x07010A3B), SH_UNT_MCA_0    , SH_ACS_SCOM_RW   );
REG64( MCA_1_WRTDBGNEST                                        , RULL(0x07010A7B), SH_UNT_MCA_1    , SH_ACS_SCOM_RW   );
REG64( MCA_2_WRTDBGNEST                                        , RULL(0x07010ABB), SH_UNT_MCA_2    , SH_ACS_SCOM_RW   );
REG64( MCA_3_WRTDBGNEST                                        , RULL(0x07010AFB), SH_UNT_MCA_3    , SH_ACS_SCOM_RW   );
REG64( MCA_4_WRTDBGNEST                                        , RULL(0x08010A3B), SH_UNT_MCA_4    , SH_ACS_SCOM_RW   );
REG64( MCA_5_WRTDBGNEST                                        , RULL(0x08010A7B), SH_UNT_MCA_5    , SH_ACS_SCOM_RW   );
REG64( MCA_6_WRTDBGNEST                                        , RULL(0x08010ABB), SH_UNT_MCA_6    , SH_ACS_SCOM_RW   );
REG64( MCA_7_WRTDBGNEST                                        , RULL(0x08010AFB), SH_UNT_MCA_7    , SH_ACS_SCOM_RW   );

REG64( MCA_0_WREITE_WRT_ECC                                    , RULL(0x07010A39), SH_UNT_MCA_0_WREITE,
       SH_ACS_SCOM_WCLRPART );
REG64( MCA_0_WDF_WRT_ECC                                       , RULL(0x07010A31), SH_UNT_MCA_0_WDF,
       SH_ACS_SCOM_WCLRPART );
REG64( MCA_1_WREITE_WRT_ECC                                    , RULL(0x07010A79), SH_UNT_MCA_1_WREITE,
       SH_ACS_SCOM_WCLRPART );
REG64( MCA_1_WDF_WRT_ECC                                       , RULL(0x07010A71), SH_UNT_MCA_1_WDF,
       SH_ACS_SCOM_WCLRPART );
REG64( MCA_2_WDF_WRT_ECC                                       , RULL(0x07010AB1), SH_UNT_MCA_2_WDF,
       SH_ACS_SCOM_WCLRPART );
REG64( MCA_2_WREITE_WRT_ECC                                    , RULL(0x07010AB9), SH_UNT_MCA_2_WREITE,
       SH_ACS_SCOM_WCLRPART );
REG64( MCA_3_WREITE_WRT_ECC                                    , RULL(0x07010AF9), SH_UNT_MCA_3_WREITE,
       SH_ACS_SCOM_WCLRPART );
REG64( MCA_3_WDF_WRT_ECC                                       , RULL(0x07010AF1), SH_UNT_MCA_3_WDF,
       SH_ACS_SCOM_WCLRPART );
REG64( MCA_4_WREITE_WRT_ECC                                    , RULL(0x08010A39), SH_UNT_MCA_4_WREITE,
       SH_ACS_SCOM_WCLRPART );
REG64( MCA_4_WDF_WRT_ECC                                       , RULL(0x08010A31), SH_UNT_MCA_4_WDF,
       SH_ACS_SCOM_WCLRPART );
REG64( MCA_5_WREITE_WRT_ECC                                    , RULL(0x08010A79), SH_UNT_MCA_5_WREITE,
       SH_ACS_SCOM_WCLRPART );
REG64( MCA_5_WDF_WRT_ECC                                       , RULL(0x08010A71), SH_UNT_MCA_5_WDF,
       SH_ACS_SCOM_WCLRPART );
REG64( MCA_6_WREITE_WRT_ECC                                    , RULL(0x08010AB9), SH_UNT_MCA_6_WREITE,
       SH_ACS_SCOM_WCLRPART );
REG64( MCA_6_WDF_WRT_ECC                                       , RULL(0x08010AB1), SH_UNT_MCA_6_WDF,
       SH_ACS_SCOM_WCLRPART );
REG64( MCA_7_WDF_WRT_ECC                                       , RULL(0x08010AF1), SH_UNT_MCA_7_WDF,
       SH_ACS_SCOM_WCLRPART );
REG64( MCA_7_WREITE_WRT_ECC                                    , RULL(0x08010AF9), SH_UNT_MCA_7_WREITE,
       SH_ACS_SCOM_WCLRPART );
REG64( MCA_WDF_WRT_ECC                                         , RULL(0x07010A31), SH_UNT_MCA_WDF  ,
       SH_ACS_SCOM_WCLRPART );
REG64( MCA_WREITE_WRT_ECC                                      , RULL(0x07010A39), SH_UNT_MCA_WREITE,
       SH_ACS_SCOM_WCLRPART );

REG64( MCA_3_XTRA_TRACE_MODE                                   , RULL(0x070107D1), SH_UNT_MCA_3    , SH_ACS_SCOM      );
REG64( MCA_7_XTRA_TRACE_MODE                                   , RULL(0x080107D1), SH_UNT_MCA_7    , SH_ACS_SCOM      );
#endif

